<?php
/**
 * This script will import a svn dump into Gforge,
 * provided the project already exists
 */

$stdout = fopen('php://stdout','w');
$stderr = fopen('php://stderr', 'w');

if ($argc != 2) {
	fprintf($stderr, "Please specify a source dir, Usage: ${argv[0]} dumps_dir\n");
	exit(1);
}
$source_path = $argv[1];

if (!can_access_file($source_path) || !is_dir($source_path)) {
	fprintf($stderr, "Couldn't access source dir: %s", $source_path);
	exit(1);
}

checkInit();

$svnroot = GFConf::get('scmsvn.localSVNRoot', '/svnroot');
$svn_path = GFConf::get('scmsvn.svnpath', '/usr/bin');

if (!can_access_file($svnroot) || !is_dir($svnroot)) {
	fprintf($stderr, "The configured value for scmsvn.localSVNRoot (%s) doesn't exist or is not readable!\n", $svnroot);
	exit(1);
}

if (!can_access_file($svn_path) || !is_dir($svn_path)) {
	fprintf($stderr, "The configured value for scmsvn.svnpath (%s) doesn't exist or is not readable!\n", $svn_path);
	exit(1);
}

$dumps = scandir($source_path);

//Stop the daemon

system('cd /opt/gforge && php bin/gforge stop &');

foreach($dumps as $dump) {
	if ($dump == '.' || $dump == '..') {
		continue;
	}
	$repo_path = $svnroot.'/'.$dump;
	fprintf($stdout, "Processing %s \n", $dump);
	if (file_exists($repo_path) ) {
		fprintf($stderr, "Directory %s already exists in %, ignoring\n", $dump, $svnroot);
	}
	system($svn_path.'/svnadmin create '.$repo_path);
	if (!can_access_file($repo_path)) {
		fprintf($stderr, "Repository %s doesn't exists, ignoring\n", $dump);
		continue;
	}

	import_svn($repo_path,$source_path.'/'.$dump, $svn_path);
}

//Update repository permissions
system('cd /opt/gforge && php bin/gforge cronjobs create_svn');

fprintf($stdout,"Repositories processed, exiting now..");
//Restart the daemon
$handle = popen('/opt/gforge/bin/gforge start','r');
if($handle) {
	echo (fread($handle, 2096));
	pclose($handle);
}

exit(0);


function import_svn($repo_path, $source_path, $svn_path) {
	$cmd = $svn_path.'/svnadmin load '. $repo_path .' < '.$source_path;
	system($cmd, $ret_val);
	return $ret_val;
}

function can_access_file($full_path) {
	return file_exists($full_path) && is_readable($full_path);
}

function checkInit() {
	require_once('initialize.php');
	require_once('GFConf.php');
}
