<?php
class DatabaseUpdater {
	private $con;
	private $path;
	private $is_install;
	private $install_exceptions = array('20130108.php', '20130327.php', '20130613.php');

	function __construct($is_install) {
		$this->con = Propel::getConnection();
		$this->path = dirname(__FILE__).'/updates/';
		$this->is_install = $is_install;
	}

	public function run() {
		$scripts = $this->getScripts();
		$date = $this->getStartPoint();
		$this->runScripts($scripts, $date);
	}

	private function getStartPoint() {
		$date = array();
		$stmt = $this->con->createStatement();
		$sql = 'SELECT MAX(filedate) As maxdate FROM database_changes ORDER BY maxdate DESC';
		$sql = "SELECT MAX(filedate) As maxdate, 'php' as type FROM database_changes WHERE filename ~ '.php' UNION
			SELECT max(filedate) AS maxdate, 'sql' as type fROM database_changes WHERE filename ~ '.sql'";
		try {
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			if ($rs->getRecordCount() > 0) {

				while ($rs->next()) {
					$date[$rs->getString('type')] = $rs->getInt('maxdate');
				}
			}
		} catch (Exception $e) {
			$stmt->executeQuery('create table database_changes (filename text, filedate text)');
			$stmt->executeQuery('insert into database_changes (filename, filedate) values (\'20060612.sql\',\'20060612\')');
			$this->show('ERROR: ' . $e->getMessage());
			exit();
		}

		if (!isset($date['php'])) {
			$date['php'] = ((int) '20060612');
		}

		if (!isset($date['sql'])) {
			$date['sql'] = ((int) '20060612');
		}
		return $date;
	}

	private function getExecutedScripts() {
		$stmt = $this->con->createStatement();
		$sql = "SELECT * FROM database_changes";
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		$executedScripts = array();
		while($rs->next()) {
			$executedScripts[] = $rs->getString('filename');
		}

		return $executedScripts;
	}

	private function getScripts() {
		$data = array();
		$this->show('Reading directory: ' . $this->path);

		if (is_dir($this->path) && ($dirhnd = opendir($this->path))) {
			while (($file = readdir($dirhnd)) !== false) {
				$pos = strrpos($file, '.');
				if ($pos !== false && $pos > 0) {
					$name = substr($file, 0, $pos);
					if (strlen($name) >= 8) {
						$date = (int) substr($name, 0, 8);
						$type = strtolower(substr($file, $pos + 1));
						if ($date > 20000000 && ($type == 'sql' || $type == 'php')) {
							$data[] = array('date'=>$date, 'filename'=>$file, 'ext'=>$type);
						}
					}
				}
			}
			closedir($dirhnd);
			sort($data);
			reset($data);
		}
		return $data;
	}

	private function runScripts($scripts, $date) {
		$sql = "SELECT MAX(filedate) As maxdate, 'php' as type FROM database_changes WHERE filename ~ '.php'";
		try {
			$stmt = $this->con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			if ($rs->getRecordCount() > 0) {
				$rs->next();
				$max_script_date = $rs->getInt('maxdate');
			}
		} catch (Exception $e) {
			$stmt->executeQuery("SELECT MAX(filedate) As maxdate, 'php' as type FROM database_changes WHERE filename ~ '.php'");
			$this->show('ERROR: ' . $e->getMessage());
			exit();
		}

		$stmt = $this->con->createStatement();
		$sql = "SELECT * FROM database_changes";
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		$executedScripts = $this->getExecutedScripts();


		//First pass, run sql scripts

		foreach ($scripts as $script) {
			if ($script['ext'] == 'sql' && $script['date'] > $date['sql']) {
				//Double check just in case the file has been executed already
				if(!in_array($script['filename'], $executedScripts)) {
					$this->show("Running SQL script: {$script['filename']}");
					$result = $this->runSQLScript($script);
				}

				if ($result) {
					$sql = "INSERT INTO database_changes (filename, filedate) ";
					$sql .="VALUES ('".pg_escape_string($script['filename'])."', '{$script['date']}')";
					try {
						$stmt->executeQuery($sql);
					} catch (Exception $e) {
						$this->show('ERROR: could insert data into database_changes');
						$this->show($e->getMessage());
						exit(1);						}
				} else {
					exit($result);
				}

			}
		}

		//Second pass, run php scripts

		foreach ($scripts as $script) {
			if ($script['ext'] == 'php' && $script['date'] > $date['php']) {
				//Double check just in case the file has been executed already
				if($this->is_install) {
					$this->show("Running PHP script: {$script['filename']}");
					$result = $this->runPHPScript($script);
				} else {
					if (($script['filename'] == '20130108.php' || $script['filename'] == '20130327.php' || $script['filename'] == '20130613.php') && $max_script_date < '20130108') {
						$result = true;
					} else {
						$this->show("Running PHP script: {$script['filename']}");
						$result = $this->runPHPScript($script);
					}
				}

				if ($result) {
					$sql = "INSERT INTO database_changes (filename, filedate) ";
					$sql .="VALUES ('".pg_escape_string($script['filename'])."', '{$script['date']}')";
					try {
						$stmt->executeQuery($sql);
					} catch (Exception $e) {
						$this->show('ERROR: could insert data into database_changes');
						$this->show($e->getMessage());
						exit(1);
					}
				} else {
					exit($result);
				}

			}
		}
		
	}

	private function runPHPScript($script) {
		$return = false;

		if ($this->is_install && in_array($script['filename'], $this->install_exceptions)) {
			return true;
		}

		$filename = $script['filename'];
		$result = array();
		$exec = 'php -f '.$this->path.$filename;

		// run the php script
		$error = '';
		$failed = self::proc_run($exec, $error);

		if($failed) {
			$this->show($this->path.$filename. " FAILED! Error message follows\n");
			echo $error;
		} else {
			$this->show($this->path.$filename." ran correctly\n");
			$return = true;
		}
		/*
		exec($exec, $result, $failure);
		if (count($result)) { // the script produced an output
			$failed = strtolower($result[count($result)-1]);
			if ($this->inString($failed,'failed') || $failure) {
				$this->show($this->path.$filename." FAILED!\n");
				foreach ($result as $line) {
					$this->show($line);
				}
			} else {
				$this->show($this->path.$filename." ran correctly\n");
				$return = true;
			}
		}

		if(!$failure){
			$return = true;
		} */

		return $return;
	}

	private function runSQLScript($script) {
		if ($this->is_install && in_array($script['filename'], $this->install_exceptions)) {
			return true;
		}
		$filename = $script['filename'];
		$sql_file = $this->path.$filename;
		$file = @fopen($sql_file, 'rb');
		if (!$file) {
			return false;
		}
		$content = fread($file, filesize($sql_file));
		fclose($file);

		$content = preg_replace("/--(.*)/", '', $content);

		$parts = explode(';', $content);
		$queries = array();
		$query_temp = '';
		$is_function = false;
		$is_copy_stdin = false;

		for ($i=0;$i<count($parts);$i++) {
			$q = $parts[$i];
			// Check if it's a function
			if (($this->inString($q, 'create function') || $this->inString($q, 'create or replace function') ||
				$this->inString($q, 'replace function')) && !$this->inString($q, 'language plpgsql') &&
				!$this->inString($q, 'language \'plpgsql\'') && !$this->inString($q, 'language \'c\'') &&
				!$this->inString($q, 'language c')) {

				while (!$this->inString($q, 'language plpgsql') && !$this->inString($q, 'language \'plpgsql\'')) {
					$i++;
					$q = $q.';'.$parts[$i];
				}
				$queries[] = trim($q);
			// Check if it is a COPY FROM stdin
			} else if ($this->inString($q, 'copy') && $this->inString($q, 'from stdin')) {
				while (!$this->inString($q, '\.')) {
					$i++;
					$q = $q.';'.$parts[$i];
				}
				$aux = explode('\.', $q, 2);
				$queries[] = ltrim($aux[0]."\\.\n");
				if (trim($aux[1]) != '') {
					$queries[] = trim($aux[1]);
				}
			// Else, we just add it up
			} else {
				if (trim($q) != '') {
					$queries[] = trim($q);
				}
			}
		}

		$i = 0;
		foreach ($queries as $query) {
			// Check if it is a DROP TABLE
			if ($this->inString($query, 'drop table')) {
				$aux = explode(' ', trim($query));
				if (count($aux) == 3 || count($aux) == 4) { // PERFECT!
					if (isset($aux[3]) && strtolower(trim($aux[3])) == 'cascade' ) {
						$this->dropCascadeTableIfExists($aux[2]);
					} else {
						$this->dropTableIfExists($aux[2]);
					}
					continue;
				}
			// Check if it is a DROP SEQUENCE
			} else if ($this->inString($query, 'drop sequence')) {
				$aux = explode(' ', trim($query));
				if (count($aux) == 3) { // PERFECT!
					$this->dropSeqIfExists($aux[2]);
					continue;
				}
			// Check if it is a DROP TRIGGER
			} else if ($this->inString($query, 'drop trigger')) {
				$aux = explode(' ', trim($query));
				if (count($aux) == 5 || count($aux) == 6) { // PERFECT!
					$this->dropTriggerIfExists($aux[2], $aux[4]);
					continue;
				}
			// Check if it is a DROP VIEW
			} else if ($this->inString($query, 'drop view')) {
				$aux = explode(' ', trim($query));
				if (count($aux) == 3 || count($aux) == 4) { // PERFECT!
					$this->dropViewIfExists($aux[2]);
					continue;
				}
			}

			$stmt = $this->con->createStatement();
			try {
				$rs = $stmt->executeQuery($query, ResultSet::FETCHMODE_ASSOC);
			} catch (Exception $e) {
				$this->show('ERROR: ' . $e->getMessage());
				$this->show("\nContinue executing ([Y]es/[N]o)?");
				// Read the input
				$answer = strtolower(trim(fgets(STDIN)));
				if ($answer != 'y' && $answer != 'yes') {
					return false;
				}
			}
		}
		return true;
	}

	private function dropViewIfExists($name) {
		return $this->dropIfExists($name, 'DROP VIEW', 'v');
	}

	private function dropSeqIfExists($name) {
		return $this->dropIfExists($name, 'DROP SEQUENCE', 'S');
	}

	private function dropTableIfExists($name) {
		return $this->dropIfExists($name, 'DROP TABLE', 'r');
	}

	private function dropIfExists($name, $command, $kind) {
		$stmt = $this->con->createStatement();
		$sql = "SELECT * FROM pg_class WHERE relname='$name' AND relkind='$kind'";
		try {
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			if ($rs->getRecordCount() > 0) {
				$sql = "$command $name";
				try {
					$rs = $stmt->executeQuery($sql);
				} catch (Exception $e) {
					$this->show("ERROR: " . $e->getMessage());
				}
			}
		} catch (Exception $e) {
			$this->show("ERROR: Could not check if $kind $name exists.");
			$this->show($e->getMessage());
			return false;
		}
		return true;
	}

	private function dropCascadeTableIfExists($name) {
		return $this->dropCascadeIfExists($name, 'DROP TABLE', 'r');
	}

	private function dropCascadeIfExists($name, $command, $kind) {
		$stmt = $this->con->createStatement();
		$sql = "SELECT * FROM pg_class WHERE relname='$name' AND relkind='$kind'";
		try {
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			if ($rs->getRecordCount() > 0) {
				$sql = "$command $name CASCADE";
				try {
					$rs = $stmt->executeQuery($sql);
				} catch (Exception $e) {
					$this->show("ERROR: " . $e->getMessage());
				}
			}
		} catch (Exception $e) {
			$this->show("ERROR: Could not check if $kind $name exists.");
			$this->show($e->getMessage());
			return false;
		}
		return true;
	}


	private function dropTriggerIfExists($name, $on) {
		$stmt = $this->con->createStatement();
		$sql = "SELECT * FROM pg_trigger WHERE tgname='$name'";
		try {
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			if ($rs->getRecordCount() > 0) {
				$sql = "DROP TRIGGER $name ON $on";
				try {
					$rs = $stmt->executeQuery($sql);
				} catch (Exception $e) {
					$this->show("ERROR: " . $e->getMessage());
				}
			}
		} catch (Exception $e) {
			$this->show("ERROR: Could not check if trigger $name exists.");
			$this->show($e->getMessage());
			return false;
		}
		return true;
	}

	private function inString($haystack, $needle, $case_sensitive = false) {
		if (!$case_sensitive) {
			$result = stripos($haystack, $needle);
		} else {
			$result = strpos($haystack, $needle);
		}

		if ($result !== false) {
			return true;
		} else {
			return false;
		}
	}

	private function show($text, $newLine = true) {
		if ($newLine) {
			$text = $text . "\n";
		}
		fwrite(STDOUT, $text);
	}


	static function proc_run($command, &$error,$descriptor = array()) {

		if (empty($descriptor)) {
			$descriptor = array(
				0 =>array('file', 'php://stdin','r'),
				1 =>array('file', 'php://stdout','w'),
				2 =>array('pipe', 'w')
			);
		}
		$pipes = array();
		$return_value = -1;

		$process = proc_open($command, $descriptor,$pipes);

		if (is_resource($process)) {
			$error = '';
			if (isset($pipes[1])) {
				$error .= stream_get_contents($pipes[1]);
				fclose($pipes[1]);
			}

			if (isset($pipes[2])) {
				$error .= stream_get_contents($pipes[2]);
				fclose($pipes[2]);
			}

			$return_value = proc_close($process);

		}

		return $return_value;
	}
}


// Was this script included by another, or is it being executed directly?
if (array_key_exists('argv', $_SERVER) && count($_SERVER['argv']) > 0) {
	$is_install = false;
	if (in_array('-i', $argv)) {
		$is_install = true;
	}
	require_once('initialize.php');
	$db_updater = new DatabaseUpdater($is_install);
	$db_updater->run();
}
?>
