-- Table: my_report

-- DROP TABLE my_report;

CREATE TABLE my_report (
  my_report_id serial NOT NULL,
  user_id integer NOT NULL,
  is_shared boolean,
  report_name character varying(40),
  open_date_start timestamp without time zone,
  open_date_end timestamp without time zone,
  close_date_start timestamp without time zone,
  close_date_end timestamp without time zone,
  CONSTRAINT my_report__pkey PRIMARY KEY (my_report_id),
  CONSTRAINT my_report_user_id_fkey FOREIGN KEY (user_id)
      REFERENCES "user" (user_id)
);

-- Table: my_report_user

-- DROP TABLE my_report_user;

CREATE TABLE my_report_user (
  my_report_id integer NOT NULL,
  user_id integer NOT NULL,
  CONSTRAINT my_report_user_pkey PRIMARY KEY (my_report_id, user_id),
  CONSTRAINT my_report_user_my_report_id_fkey FOREIGN KEY (my_report_id)
      REFERENCES my_report (my_report_id) ON DELETE CASCADE,
  CONSTRAINT my_report_user_user_id_fkey FOREIGN KEY (user_id)
      REFERENCES "user" (user_id) 
);

-- Table: my_report_project

-- DROP TABLE my_report_project;

CREATE TABLE my_report_project (
  my_report_id integer NOT NULL,
  project_id integer NOT NULL,
  CONSTRAINT my_report_project_pkey PRIMARY KEY (my_report_id, project_id),
  CONSTRAINT my_report_project_my_report_id_fkey FOREIGN KEY (my_report_id)
      REFERENCES my_report (my_report_id) ON DELETE CASCADE,
  CONSTRAINT my_report_project_project_id_fkey FOREIGN KEY (project_id)
      REFERENCES project (project_id) ON DELETE CASCADE
);

-- Table: my_report_tracker

-- DROP TABLE my_report_tracker;

CREATE TABLE my_report_tracker (
  my_report_id integer NOT NULL,
  tracker_id integer NOT NULL,
  CONSTRAINT my_report_tracker_pkey PRIMARY KEY (my_report_id, tracker_id),
  CONSTRAINT my_report_tracker_my_report_id_fkey FOREIGN KEY (my_report_id)
      REFERENCES my_report (my_report_id) ON DELETE CASCADE,
  CONSTRAINT my_report_tracker_tracker_id_fkey FOREIGN KEY (tracker_id)
      REFERENCES tracker (tracker_id) ON DELETE CASCADE
);

-- Table: my_report_field_filter

-- DROP TABLE my_report_field_filter;

CREATE TABLE my_report_field_filter (
  my_report_field_filter_id serial NOT NULL,
  my_report_id integer NOT NULL,
  field_name text NOT NULL,
  type integer NOT NULL,
  CONSTRAINT my_report_field_filter_pkey PRIMARY KEY (my_report_field_filter_id),
  CONSTRAINT my_report_field_filter_my_report_id_fkey FOREIGN KEY (my_report_id)
      REFERENCES my_report (my_report_id) ON DELETE CASCADE
);

-- Table: my_report_field_value

-- DROP TABLE my_report_field_value;

CREATE TABLE my_report_field_value (
  my_report_field_value_id serial NOT NULL,
  my_report_field_filter_id integer NOT NULL,
  field_value text NOT NULL,
  field_range_start text NOT NULL,
  field_range_end text NOT NULL,  
  CONSTRAINT my_report_field_value_pkey PRIMARY KEY (my_report_field_value_id),
  CONSTRAINT my_report_field_value_my_report_field_filter_id_fkey FOREIGN KEY (my_report_field_filter_id)
      REFERENCES my_report_field_filter (my_report_field_filter_id) ON DELETE CASCADE
);

-- Table: my_report_field_display

-- DROP TABLE my_report_field_display;

CREATE TABLE my_report_field_display (
  my_report_field_display_id serial NOT NULL,
  my_report_id integer NOT NULL,
  field_name text NOT NULL,
  type integer NOT NULL,
  CONSTRAINT my_report_field_display_pkey PRIMARY KEY (my_report_field_display_id),
  CONSTRAINT my_report_field_my_report_id_fkey FOREIGN KEY (my_report_id)
      REFERENCES my_report (my_report_id) ON DELETE CASCADE
);
