ALTER TABLE tracker_item ADD COLUMN has_subitems BOOLEAN DEFAULT false;
ALTER TABLE tracker_item ADD COLUMN subitems_count INTEGER DEFAULT 0;

UPDATE tracker_item SET subitems_count=c.real_subitems_count FROM (SELECT t1.tracker_item_id, (SELECT COUNT(*) FROM tracker_item t2 WHERE t2.parent_id=t1.tracker_item_id) AS real_subitems_count FROM tracker_item t1) c WHERE tracker_item.tracker_item_id=c.tracker_item_id;
UPDATE tracker_item SET has_subitems=true WHERE subitems_count > 0;


CREATE OR REPLACE FUNCTION tracker_item_parent_agg() RETURNS TRIGGER AS '
BEGIN
	IF TG_OP=''INSERT'' THEN
		IF NEW.parent_id <> 0 THEN
			UPDATE tracker_item SET has_subitems=true, subitems_count=subitems_count+1 WHERE tracker_item_id=NEW.parent_id;
		END IF;
	ELSIF TG_OP=''DELETE'' THEN
		UPDATE tracker_item SET subitems_count=subitems_count-1, has_subitems=(subitems_count > 1) WHERE tracker_item_id=OLD.parent_id;
		UPDATE tracker_item SET parent_id=OLD.parent_id WHERE parent_id=OLD.tracker_item_id;		-- Move the children to the parent
	ELSIF TG_OP=''UPDATE'' THEN
		IF NEW.parent_id <> OLD.parent_id THEN
			UPDATE tracker_item SET subitems_count=subitems_count-1, has_subitems=(subitems_count > 1) WHERE tracker_item_id=OLD.parent_id;
			UPDATE tracker_item SET has_subitems=true, subitems_count=subitems_count+1 WHERE tracker_item_id=NEW.parent_id;
		END IF;
	END IF;
	
	RETURN NEW;
END;
'
LANGUAGE plpgsql;

CREATE TRIGGER tracker_item_parent_trigg AFTER INSERT OR UPDATE OR DELETE ON tracker_item 
FOR EACH ROW EXECUTE PROCEDURE tracker_item_parent_agg();

CREATE INDEX tracker_item_parent_id ON tracker_item(parent_id);
