--Add new field to the docman folder table
ALTER TABLE docman_folder ADD COLUMN top_parent_folder_id INTEGER;
ALTER TABLE docman_folder ALTER COLUMN top_parent_folder_id SET DEFAULT 0;

CREATE OR REPLACE FUNCTION get_top_folder_id(integer) RETURNS integer AS $$ 
DECLARE
folder_id ALIAS for $1;
folder RECORD;
top_folder_id integer;
BEGIN
	SELECT * INTO folder FROM docman_folder WHERE docman_folder_id=folder_id;
	top_folder_id := folder.parent_folder_id;
	IF top_folder_id <> 0 THEN
		return get_top_folder_id(top_folder_id);
	ELSE
		return folder.docman_folder_id;

	END IF;	
END
$$ LANGUAGE  plpgsql;

--Update the existing records
UPDATE docman_folder SET top_parent_folder_id=get_top_folder_id(docman_folder.docman_folder_id);

--Create the triggers
CREATE OR REPLACE FUNCTION update_top_parent_folder() RETURNS TRIGGER AS
$$
BEGIN
IF TG_OP='INSERT' THEN
		IF NEW.parent_folder_id IS NULL OR NEW.parent_folder_id = 0 THEN
			NEW.top_parent_folder_id = currval('docman_folder_docman_folder_id_seq'::regclass);
		ELSE
			NEW.top_parent_folder_id = get_top_folder_id(NEW.parent_folder_id);
		END IF;
	ELSIF TG_OP='UPDATE' AND NEW.parent_folder_id <> OLD.parent_folder_id  THEN
		IF NEW.parent_folder_id IS NULL OR NEW.parent_folder_id = 0 THEN
			NEW.top_parent_folder_id = NEW.docman_folder_id;
		ELSE
			NEW.top_parent_folder_id = get_top_folder_id(NEW.parent_folder_id);
		END IF;
	END IF;

RETURN NEW;

END
$$ 
LANGUAGE plpgsql;

CREATE TRIGGER update_top_parent_folder_id BEFORE INSERT OR UPDATE ON docman_folder
FOR EACH ROW EXECUTE PROCEDURE update_top_parent_folder();
