DROP VIEW project_activity_vw;
CREATE OR REPLACE VIEW project_activity_vw AS (
	SELECT t.project_id, 'trackeritem' AS section, at.ref_id AS ref_id, 0 AS subref_id, 'create' AS action,
	at.new_value AS description1, '' AS description2, at.change_date AS activity_date,
	u.user_id, u.unix_name, u.firstname, u.lastname, 1 as total
	FROM audit_trail at, tracker t, tracker_item ti, "user" u
	WHERE at.section='trackeritem' AND at.field_name='_created_' AND ti.tracker_item_id=at.ref_id 
	AND t.tracker_id=ti.tracker_id AND u.user_id=at.user_id
	UNION
	SELECT at.ref_id, 'trackeritem' AS section, 0 AS ref_id, 0 AS subref_id, 'delete' AS action,
	at.old_value AS description1, '' AS description2, at.change_date AS activity_date,
	u.user_id, u.unix_name, u.firstname, u.lastname, 1 as total
	FROM audit_trail at, "user" u
	WHERE at.section='project' AND at.field_name='trackeritem' AND at.new_value='_deleted_'
	AND u.user_id=at.user_id
	UNION
	SELECT t.project_id, 'trackeritem' AS section, at.ref_id AS ref_id, 0 AS subref_id, 'statuschange' AS action,
	ti.summary AS description1, at.new_value AS description2, at.change_date AS activity_date,
	u.user_id, u.unix_name, u.firstname, u.lastname, 1 as total
	FROM audit_trail at, tracker t, tracker_item ti, "user" u
	WHERE at.section='trackeritem' AND at.field_name='status' AND ti.tracker_item_id=at.ref_id 
	AND t.tracker_id=ti.tracker_id AND u.user_id=at.user_id
	UNION
	SELECT sc.project_id, 'scm'AS section, sc.scm_commit_id AS ref_id, 0 AS subref_id, 'commit' AS action, 
	sc.message_log AS description1, '' AS description2, sc.commit_date AS activity_date, u.user_id, u.unix_name, u.firstname, u.lastname, 1 as total
	FROM scm_commit sc, "user" u WHERE u.user_id = sc.user_id
	UNION
	SELECT f.ref_id AS project_id, 'forum'::text AS section, f.forum_id AS ref_id, fm.forum_message_id AS subref_id, '' AS action, 
	fm.subject AS description1, '' AS description2, fm.post_date AS activity_date, u.user_id, u.unix_name, u.firstname, u.lastname, 1 as total
	FROM forum f, forum_thread ft, forum_message fm, "user" u
	WHERE u.user_id = fm.created_by AND ft.forum_thread_id = fm.forum_thread_id AND ft.forum_id = f.forum_id AND f.section::text = 'project'::text 
	AND fm.subject::text <> 'Chat message'::text
	AND substring(fm.subject::text from 1 for 11) <> 'Chat event:'
	UNION
	SELECT f.ref_id AS project_id, 'chat'::text AS section, f.forum_id AS ref_id, 0 AS subref_id, '' AS action, 
	'projectchat'::text AS description1, project.project_name AS description2, fm.post_date::timestamp::date AS activity_date, 0 AS user_id, '' AS unix_name, '' AS firstname, '' AS lastname, count(*) as total
	FROM forum f, forum_thread ft, forum_message fm, project
	WHERE ft.forum_thread_id = fm.forum_thread_id AND ft.forum_id = f.forum_id AND f.section::text = 'project'::text 
	AND (fm.subject::text = 'Chat message'::text OR substring(fm.subject::text from 1 for 11) = 'Chat event:')
	AND f.ref_id = project.project_id
	GROUP BY f.ref_id, section, f.forum_id, subref_id, action, description1, description2, activity_date
	UNION
	SELECT tracker.project_id AS project_id, 'chat'::text AS section, tracker_item.tracker_item_id AS ref_id, 0 AS subref_id, '' AS action, 
	'trackeritem'::text AS description1, tracker_item.summary AS description2, fm.post_date::timestamp::date AS activity_date, 0 AS user_id, '' AS unix_name, '' AS firstname, '' AS lastname, count(*) as total
	FROM forum f, forum_thread ft, forum_message fm, tracker_item, tracker
	WHERE ft.forum_thread_id = fm.forum_thread_id AND ft.forum_id = f.forum_id AND f.section::text = 'trackeritem'::text 
	AND (fm.subject::text = 'Chat message'::text OR substring(fm.subject::text from 1 for 11) = 'Chat event:')
	AND tracker_item.tracker_item_id = f.ref_id AND tracker_item.tracker_id = tracker.tracker_id
	GROUP BY project_id, section, tracker_item.tracker_item_id, subref_id, action, description1, description2, activity_date
)
