<?php

require_once('initialize.php');
require_once('GFUrl.php');
#require_once('core/Language.php');
require_once('StringUtils.php');
//Get tracker activity from project_activity_vw

//Get contents from activity_log
$con = Propel::getConnection();
$stmt = $con->createStatement();


//Check if activity_log table already exists
$sql = 'SELECT EXISTS(SELECT * FROM information_schema.tables WHERE table_name=\'activity_log\')';
/** @var ResultSet $rs */
$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

$exists = false;
if ($rs->next() && $rs->getBoolean(1)) {
	$exists = true;
}

if ($exists) {
	//Drop activity_log_id column temporarily
	try {
		$sql = 'ALTER TABLE activity_log DROP COLUMN activity_log_id';
		$stmt->executeQuery($sql);
	} catch (Exception $e) {
		//The activity_log_id may not existis, simply ignore it
	}
}

//Create temporary table
$sql = 'CREATE  TEMPORARY TABLE activity_log_migrate AS
SELECT project_activity_vw.*, tracker.tracker_name AS ed_name, project.unix_name as project_unix_name FROM
project_activity_vw
INNER JOIN tracker_item ON project_activity_vw.ref_id = tracker_item.tracker_item_id
AND section=\'trackeritem\' INNER JOIN tracker ON tracker.tracker_id=tracker_item.tracker_id INNER JOIN project
ON tracker.project_id = project.project_id

UNION

SELECT project_activity_vw.*, forum.forum_name AS ed_name, project.unix_name as project_unix_name FROM
project_activity_vw INNER JOIN forum ON forum.forum_id = project_activity_vw.ref_id AND
project_activity_vw.section=\'forum\'
INNER JOIN project ON forum.ref_id=project.project_id

UNION

SELECT project_activity_vw.*, plugin.plugin_name AS ed_name, project.unix_name AS project_unix_name FROM
project_activity_vw INNER JOIN scm_commit ON scm_commit.scm_commit_id = project_activity_vw.ref_id
INNER JOIN project ON scm_commit.project_id=project.project_id INNER JOIN project_plugin ON project.project_id = project_plugin.project_id INNEr JOIN
plugin ON project_plugin.plugin_id = plugin.plugin_id WHERE plugin_name IN (\'scmcvs\', \'scmsvn\', \'scmgit\') AND section=\'scm\'';

$stmt->executeUpdate($sql);

$sql = 'ALTER TABLE activity_log_migrate ADD COLUMN id SERIAL';
$stmt->executeUpdate($sql);

//Purge records that we are going to migrate
$sql = 'DELETE FROM activity_log WHERE section IN (\'trackeritem\',\'forum\',\'scm\',\'scmcommit\')' ;
$stmt->executeUpdate($sql);


$lastId = 0;

//Perform update in batches of 500 records
$con = Propel::getConnection()->getResource();

while(true) {
	//Encapsulate inside a function so the GC can free the memory everytime we finish processing a subset of records
	$lastId = walkSubset($stmt, $lastId);
	if($lastId == -1) {
		break;
	}

}

//Now, recreate activity_log_id
$sql = 'ALTER TABLE activity_log ADD COLUMN activity_log_id SERIAL';
$stmt->executeQuery($sql);
$sql = 'ALTER TABLE activity_log ADD CONSTRAINT activity_log_pkey PRIMARY KEY (activity_log_id)';
$stmt->executeQuery($sql);
$sql = 'UPDATE activity_log SET section=\'scmcommit\' WHERE section=\'scm\'';
$stmt->executeQuery($sql);

exit(0);

/**
 * @param
 * @param $lastId
 * @param int $limit
 */
function walkSubset($stmt, $lastId, $limit=10000) {
	$con = $stmt->getConnection()->getResource();
	$sql = 'SELECT * FROM activity_log_migrate WHERE id > '. $lastId .' LIMIT '.$limit;

	$rs = $stmt->executeQuery($sql);
	if (!$rs || !$rs->next()) {
		return -1;
	}

	$rs->last();
	$lastId = $rs->getInt('id');
	$events = migrateSet($rs);
	$records = array();
	foreach ($events as $event) {
		$records[] = process_string($event, $con);
	}
	pg_copy_from($con, 'activity_log', $records);

	$rs->close();

	unset($records);
	unset($events);
	unset($rs);

	return $lastId;
}
/**
 * @param string $event
 * @return mixed
 */
function process_string($event, $con) {

	$string = str_replace("\n"," ",implode("\t", $event));
	$string = pg_escape_string($con, $string);

	return $string;

}

/**
 * @param ResultSet $rs
 * @return array
 */
function migrateSet(ResultSet $rs) {
	$rs->beforeFirst();

	$events = array();
	while ($rs->next()) {
		$params = array();
		$params['project_id'] = $rs->getInt('project_id');
		$section = $params['section'] = $rs->getString('section');
		$params['ref_id'] = $rs->getInt('ref_id');
		$params['subref_id'] = $rs->getInt('ref_id');
		$params['action'] = $rs->getString('action');
		$params['unix_name'] = $rs->getString('unix_name');
		$params['user_id'] = $rs->getInt('user_id');
		$params['description1'] = str_replace("\t"," ",$rs->getString('description1'));
		$params['description2'] = str_replace("\t"," ", $rs->getString('description2'));
		$params['activity_date'] = $rs->getDate('activity_date');
		$params['ed_name'] = $rs->getString('ed_name');
		$params['project_unix_name'] = $rs->getString('project_unix_name');

		if ($section == 'trackeritem') {
			$events[] = migrateTrackerItemActivityRow($params);
		} else if ($section == 'forum') {
			$events[] = migrateForumActivityRow($params);
		} else if ($section == 'scm') {
			$events[] = migrateScmActivityRow($params);
		}
	}

	return $events;

}


/**
 * @param array $params
 * @return array|null
 */
function migrateTrackerItemActivityRow($params) {
	if ($params['section'] == 'trackeritem' ||
		$params['section'] == 'tracker') {


		$event = array(
			'activity_date' => formatDate(time($params['activity_date'])),
			'project_id' => $params['project_id'],
			'user_id' => $params['user_id'],
			'section' => $params['section'],
			'ref_id' => $params['ref_id'],
			'description' => getActivityDescriptionForTrackerItem($params),
			'url' => getTrackerItemUrl($params),
			'activity_type' => $params['action'],
			'extended_data' =>json_encode($params)
		);

		return $event;

	}

	return null;

}

/**
 * @param $params
 * @return array|null
 */
function migrateForumActivityRow($params) {
	if ($params['section'] == 'forum') {
		$event = array(
			'activity_date' => formatDate(time($params['activity_date'])),
			'project_id' => $params['project_id'],
			'user_id' => $params['user_id'],
			'section' => $params['section'],
			'ref_id' => $params['ref_id'],
			'description' => 'Forum post:' . $params['description1'],
			'url' => getForumPostUrl($params),
			'activity_type' => $params['action'],
			'extended_data' => json_encode($params)
		);
		return $event;
	}

	return null;
}

/**
 * @param $params
 * @return array|null
 */
function migrateScmActivityRow($params) {
	if ($params['section'] == 'scm') {
		//Trim descriptionxxxxd
		$description = StringUtils::trimString($params['description1'], 256);

		$activity_desc = "Commit: ".$description;
		$params['description1'] = $activity_desc;
		$event = array(
			'activity_date' => formatDate(time($params['activity_date'])),
			'project_id' => $params['project_id'],
			'user_id' => $params['user_id'],
			'section' => $params['section'],
			'ref_id' => $params['ref_id'],
			'description' => $activity_desc,
			'url' => getScmCommitUrl($params),
			'activity_type' => $params['action'],
			'extended_data' => json_encode($params)
		);
		return $event;
	}

	return null;
}

/**
 * @param int $timestamp
 * @return bool|string
 */
function formatDate($timestamp) {
	return date('Ymd H:s O', $timestamp);
}


/**
 * @param $params
 * @return string
 */
function getActivityDescriptionForTrackerItem($params) {
	$action = $params['action'];
	$refid = $params['ref_id'];
	$desc1 = str_replace("\t","\\\t",$params['description1']);
	$desc2 = str_replace("\t","\\\t",$params['description2']);
	$user = $params['unix_name'];

	switch ($action) {
		case 'create':
			return "$user opened #{$refid} in {$params['ed_name']}: $desc1";

		case 'delete':
			return "$user deleted Tracker Item '{$desc1}' from {$params['ed_name']}";

		default: // includes 'statuschange'
			return "$user updated #{$refid} to {$desc2}";
	}
}

/**
 * @param array $params
 * @return string
 */
function getTrackerItemUrl($params) {
	$link = GFUrl::getUrl()->projectURL($params['project_unix_name'], 'TrackerItemEdit', 'tracker','',
	array('tracker_item_id'=>$params['ref_id']));

	return $link;
}

/**
 * @param array $params
 * @return string
 */
function getForumPostUrl($params) {
	$link = GFUrl::getUrl()->projectURL($params['project_unix_name'],'ForumBrowse', 'forum','',
		array('_forum_action'=>'MessageReply', 'message_id'=>$params['subref_id']));

	return $link;
}

/**
 * @param array $params
 * @return string
 */
function getScmCommitUrl($params) {
	$link = GFUrl::getURL()->projectURL($params['project_unix_name'],'ScmCommitDetail', $params['ed_name'], '', array('scm_commit_id' => $params['ref_id']));

	return $link;

}
