-- View: project_activity_vw

DROP VIEW project_activity_vw;

CREATE OR REPLACE VIEW project_activity_vw AS 
	SELECT t.project_id, 'trackeritem'::text AS section, at.ref_id, 0 AS subref_id, 'create'::text AS action, at.new_value AS description1, ''::text AS description2, at.change_date AS activity_date, u.user_id, u.unix_name, u.firstname, u.lastname, 1 AS total
        FROM audit_trail at, tracker t, tracker_item ti, "user" u
        WHERE at.section::text = 'trackeritem'::text AND at.field_name::text = '_created_'::text AND ti.tracker_item_id = at.ref_id AND t.tracker_id = ti.tracker_id AND u.user_id = at.user_id
        
UNION 
	-- Old audit trail records for deleted TIs were recorded against the project - keep showing those, 
	-- even if they don't include the tracker item ID
	 SELECT at.ref_id AS project_id, 'trackeritem'::text AS section, 0 AS ref_id, 0 AS subref_id, 'delete'::text AS action, at.old_value AS description1, ''::text AS description2, at.change_date AS activity_date, u.user_id, u.unix_name, u.firstname, u.lastname, 1 AS total
	FROM audit_trail at, "user" u
	WHERE at.section::text = 'project'::text AND at.field_name::text = 'trackeritem'::text AND at.new_value = '_deleted_'::text AND u.user_id = at.user_id

UNION
	-- Newer audit trail records for deleted TIs are recorded against the tracker itself, 
	-- and include the tracker item id and summary in the old_value field
	SELECT t.project_id, at.field_name AS section, 0 AS ref_id, 0 AS subref_id, 'delete'::text AS action, at.old_value AS description1, ''::text AS description2, at.change_date AS activity_date, u.user_id, u.unix_name, u.firstname, u.lastname, 1 AS total
	FROM audit_trail at, tracker t, tracker_item ti, "user" u
	WHERE at.section::text = 'tracker'::text AND at.field_name::text = 'trackeritem'::text AND at.new_value = '_deleted_' AND t.tracker_id = at.ref_id AND u.user_id = at.user_id

UNION 
	SELECT t.project_id, 'trackeritem'::text AS section, at.ref_id, 0 AS subref_id, 'statuschange'::text AS action, ti.summary AS description1, at.new_value AS description2, at.change_date AS activity_date, u.user_id, u.unix_name, u.firstname, u.lastname, 1 AS total
        FROM audit_trail at, tracker t, tracker_item ti, "user" u
        WHERE at.section::text = 'trackeritem'::text AND at.field_name::text = 'status'::text AND ti.tracker_item_id = at.ref_id AND t.tracker_id = ti.tracker_id AND u.user_id = at.user_id

UNION 
	SELECT sc.project_id, 'scm'::text AS section, sc.scm_commit_id AS ref_id, 0 AS subref_id, 'commit'::text AS action, sc.message_log AS description1, ''::text AS description2, sc.commit_date AS activity_date, u.user_id, u.unix_name, u.firstname, u.lastname, 1 AS total
        FROM scm_commit sc, "user" u
        WHERE u.user_id = sc.user_id

UNION 
	SELECT f.ref_id AS project_id, 'chat'::text AS section, f.forum_id AS ref_id, f.forum_id AS subref_id, ''::text AS action, 'projectchat'::text AS description1, project.project_name AS description2, fm.post_date::timestamp without time zone::date AS activity_date, count(DISTINCT fm.created_by) AS user_id, ''::text AS unix_name, ''::text AS firstname, ''::text AS lastname, count(*) AS total
        FROM forum f, forum_thread ft, forum_message fm, project
        WHERE ft.forum_thread_id = fm.forum_thread_id AND ft.forum_id = f.forum_id AND f.section::text = 'project'::text AND f.ref_id = project.project_id
        GROUP BY f.ref_id, f.section, f.forum_id, ''::text, 'projectchat'::text, project.project_name, fm.post_date::timestamp without time zone::date

UNION 
         SELECT tracker.project_id, 'chat'::text AS section, tracker_item.tracker_item_id AS ref_id, 0 AS subref_id, ''::text AS action, 'trackeritem'::text AS description1, tracker_item.summary AS description2, fm.post_date::timestamp without time zone::date AS activity_date, count(DISTINCT fm.created_by) AS user_id, ''::text AS unix_name, ''::text AS firstname, ''::text AS lastname, count(*) AS total
	FROM forum f, forum_thread ft, forum_message fm, tracker_item, tracker
	WHERE ft.forum_thread_id = fm.forum_thread_id AND ft.forum_id = f.forum_id AND f.section::text = 'trackeritem'::text AND tracker_item.tracker_item_id = f.ref_id AND tracker_item.tracker_id = tracker.tracker_id

GROUP BY 
	tracker.project_id, f.section, tracker_item.tracker_item_id, 0::integer, ''::text, 'trackeritem'::text, tracker_item.summary, fm.post_date::timestamp without time zone::date;

