-- Table: setting

-- DROP TABLE setting;

CREATE OR REPLACE FUNCTION create_setting_table ()
  RETURNS void AS
$_$
BEGIN

IF EXISTS (
    SELECT *
    FROM   pg_catalog.pg_tables 
    WHERE  tablename  = 'setting'
    ) THEN
   RAISE NOTICE 'Table "setting" already exists.';
   ALTER TABLE setting ALTER COLUMN project_id DROP NOT NULL;

ELSE
  CREATE TABLE setting
  (
    setting_id serial NOT NULL,
    "section" character varying(20) NOT NULL,
    ref_id integer NOT NULL,
    setting_name character varying(50) NOT NULL,
    setting_value text,
    set_date timestamp with time zone DEFAULT CURRENT_TIMESTAMP,
    CONSTRAINT setting_pkey PRIMARY KEY (setting_id),
    CONSTRAINT setting_unique_idx UNIQUE ("section", ref_id, setting_name)
  );
  insert into setting (section, ref_id, setting_name, setting_value)
  select 'project', p.project_id, 'project-chat', min(ft.forum_thread_id)
  from forum f
  inner join forum_thread ft
  on f.forum_id = ft.forum_id
  inner join project p
  on f.section = 'project' and f.ref_id = p.project_id
  where (f.forum_name = 'Discussion' or f.forum_name = p.project_name)
  and (ft.thread_name = 'Discussion' or f.thread_count = 1)
  group by p.project_id;
END IF;

END;
$_$ LANGUAGE plpgsql;

SELECT create_setting_table();
