CREATE TABLE user_project_rating (
	user_id INT NOT NULL REFERENCES "user"(user_id) ON DELETE CASCADE,
	project_id INT NOT NULL REFERENCES "project"(project_id) ON DELETE CASCADE,
	rating INT NOT NULL,
	comment TEXT,
	rating_date TIMESTAMP DEFAULT NOW(),
	PRIMARY KEY (user_id, project_id)
);

ALTER TABLE project ADD COLUMN average_rating INT;

INSERT INTO plugin(plugin_name,plugin_desc,plugin_type,plugin_order) VALUES('projectrating','Project rating plugin','site',(SELECT MAX(plugin_order)+1 FROM plugin WHERE plugin_type='site'));

CREATE OR REPLACE FUNCTION project_rating_update() RETURNS TRIGGER AS $$
DECLARE
	update_project_id INT;
BEGIN
	IF TG_OP = 'DELETE' THEN
		update_project_id = OLD.project_id;
	ELSE
		update_project_id = NEW.project_id;
	END IF;
	
	UPDATE project SET average_rating = (SELECT ROUND(AVG(rating)) FROM user_project_rating WHERE project_id=update_project_id) WHERE project_id=update_project_id;
	
	RETURN NULL;
END;
$$
LANGUAGE 'plpgsql';

CREATE TRIGGER trig_project_rating_update AFTER INSERT OR UPDATE OR DELETE ON user_project_rating FOR EACH ROW EXECUTE PROCEDURE project_rating_update();
