CREATE TABLE notification (
  notification_id SERIAL PRIMARY KEY,
  send_date TIMESTAMP default CURRENT_TIMESTAMP, 
  section VARCHAR(50) NOT NULL,
  subsection VARCHAR(50) DEFAULT NULL,
  ref_id INTEGER NOT NULL,
  subject TEXT NOT NULL,
  body TEXT NOT NULL,
  user_id INTEGER,
  FOREIGN KEY (user_id) REFERENCES "user"(user_id) ON DELETE CASCADE
);

CREATE TABLE failed_notification (
  notification_id INTEGER NOT NULL,
  user_id INTEGER NOT NULL,
  send_date TIMESTAMP,
  body TEXT,
  PRIMARY KEY (notification_id, user_id),
  FOREIGN KEY (notification_id) REFERENCES notification(notification_id) ON DELETE CASCADE,
  FOREIGN KEY (user_id) REFERENCES "user"(user_id) ON DELETE CASCADE
);

CREATE TABLE sent_notification (
  notification_id INTEGER NOT NULL,
  user_id INTEGER NOT NULL,
  send_date TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (notification_id, user_id),
  FOREIGN KEY (notification_id) REFERENCES notification(notification_id),
  FOREIGN KEY (user_id) REFERENCES "user"(user_id)
);

CREATE OR REPLACE FUNCTION insert_sent_notification() RETURNS trigger AS $body$
BEGIN
  IF NEW.notification_id IS NOT NULL THEN
    INSERT INTO sent_notification (notification_id, user_id) VALUES (NEW.notification_id, NEW.user_id);
  END IF;
  RETURN NEW;
END;
$body$ LANGUAGE 'plpgsql';

CREATE TRIGGER insert_sent_notification AFTER INSERT ON notification_queue
  FOR EACH ROW EXECUTE PROCEDURE insert_sent_notification();

ALTER TABLE notification_queue ADD COLUMN notification_id INTEGER DEFAULT NULL;
ALTER TABLE notification_queue ADD FOREIGN KEY (notification_id) REFERENCES notification(notification_id);


