CREATE OR REPLACE FUNCTION trove_children(category_id BIGINT, self BOOLEAN)
  RETURNS SETOF trove_category AS $$
DECLARE
  r trove_category%ROWTYPE;
BEGIN
  IF self THEN
    RETURN QUERY SELECT * FROM trove_category WHERE category_id = trove_category_id;
   END IF;
  FOR r IN SELECT * FROM trove_category WHERE parent_trove_category_id = category_id
  LOOP
    RETURN NEXT r;
    RETURN QUERY SELECT * FROM trove_children(r.trove_category_id, FALSE);
  END LOOP;

  RETURN;
END;
$$
LANGUAGE 'plpgsql';



CREATE OR REPLACE FUNCTION trove_children(category_id BIGINT)
  RETURNS SETOF trove_category AS
$$
BEGIN
  RETURN QUERY SELECT * FROM trove_children(category_id, TRUE);
  RETURN;
END;
$$
LANGUAGE 'plpgsql';

CREATE OR REPLACE FUNCTION set_show_in_search(category_id BIGINT, bool_show BOOLEAN)
 RETURNS INTEGER  AS $$
 DECLARE
   count integer;
BEGIN
  UPDATE trove_category SET show_in_search = bool_show WHERE trove_category.trove_category_id IN (SELECT trove_category_id FROM trove_children(category_id));
  GET DIAGNOSTICS count = ROW_COUNT;
  return count;
END;
$$
LANGUAGE 'plpgsql';

CREATE OR REPLACE FUNCTION set_access_level(category_id BIGINT, level INT)
  RETURNS INTEGER AS $$
  DECLARE
    count integer;
BEGIN
    UPDATE trove_category SET access_level = level WHERE trove_category.trove_category_id IN (SELECT trove_category_id FROM trove_children(category_id));
    GET DIAGNOSTICS  count = ROW_COUNT;
    RETURN count;
END;
$$
LANGUAGE 'plpgsql';


CREATE OR REPLACE FUNCTION get_top_parent_trove(category_id BIGINT)
  RETURNS  SETOF trove_category AS $$
DECLARE
  r trove_category%ROWTYPE;
  BEGIN
  SELECT INTO r * FROM trove_category WHERE trove_category_id = category_id;
  IF r.parent_trove_category_id = 0 THEN
     RETURN NEXT r;
  ELSE
     RETURN QUERY SELECT * FROM get_top_parent_trove(r.parent_trove_category_id);
  END IF;
  RETURN;
  END;
$$
LANGUAGE 'plpgsql';

CREATE OR REPLACE FUNCTION update_vectors()
  RETURNS trigger AS
  $BODY$
DECLARE
table_name TEXT;
rec RECORD;
BEGIN
	table_name := TG_ARGV[0];

	IF table_name = 'tracker_item' THEN
		IF TG_OP = 'INSERT' THEN
			INSERT INTO tracker_item_idx (tracker_item_id, tracker_id, vectors) VALUES (NEW.tracker_item_id, NEW.tracker_id, to_tsvector('default', coalesce(NEW.details,'') ||' '|| coalesce(NEW.summary,'')));
		ELSIF TG_OP = 'UPDATE' AND (OLD.details <> NEW.details OR OLD.summary <> NEW.summary) THEN
			UPDATE tracker_item_idx SET tracker_id=NEW.tracker_id, vectors=to_tsvector('default', coalesce(NEW.details,'') ||' '|| coalesce(NEW.summary,'')) WHERE tracker_item_id=NEW.tracker_item_id;
		END IF;

	ELSIF table_name = 'tracker_item_message' THEN
		IF TG_OP = 'INSERT' THEN
			INSERT INTO tracker_item_message_idx (tracker_item_message_id, tracker_item_id, vectors) VALUES (NEW.tracker_item_message_id, NEW.tracker_item_id, to_tsvector('default', coalesce(NEW.body,'')));
		ELSIF TG_OP = 'UPDATE' AND (OLD.body <> NEW.body) THEN
			UPDATE tracker_item_message_idx SET tracker_item_id=NEW.tracker_item_id, vectors=to_tsvector('default', coalesce(NEW.body,'')) WHERE tracker_item_message_id=NEW.tracker_item_message_id;
		END IF;

	ELSIF table_name = 'docman_file' THEN
		IF TG_OP = 'INSERT' THEN
			INSERT INTO docman_file_idx (docman_file_id, vectors) VALUES (NEW.docman_file_id, to_tsvector('default', coalesce(NEW.description,'')));
		ELSIF TG_OP = 'UPDATE' AND OLD.details <> NEW.details THEN
			UPDATE docman_file_idx SET vectors=to_tsvector('default', coalesce(NEW.description,'')) WHERE docman_file_id=NEW.docman_file_id;
		END IF;

	ELSIF table_name = 'docman_folder' THEN
		IF TG_OP = 'INSERT' THEN
			INSERT INTO docman_folder_idx (docman_folder_id, vectors) VALUES (NEW.docman_folder_id, to_tsvector('default', coalesce(NEW.folder_name,'')));
		ELSIF TG_OP = 'UPDATE' THEN
			UPDATE docman_folder_idx SET vectors=to_tsvector('default', coalesce(NEW.folder_name,'')) WHERE docman_folder_id=NEW.docman_folder_id;
		END IF;

	ELSIF table_name = 'docman_file_version' THEN
		IF TG_OP = 'INSERT' THEN
			INSERT INTO docman_file_version_idx (docman_file_version_id, vectors) VALUES (NEW.docman_file_version_id, to_tsvector('default', coalesce(NEW.changes,'')));
		ELSIF TG_OP = 'UPDATE'  AND OLD.changes <> NEW. changes THEN
			UPDATE docman_file_version_idx SET vectors=to_tsvector('default', coalesce(NEW.changes,'')) WHERE docman_file_version_id=NEW.docman_file_version_id;
		END IF;

	ELSIF table_name = 'filesystem' THEN
		IF TG_OP = 'INSERT' THEN
			INSERT INTO filesystem_idx (filesystem_id, vectors) VALUES (NEW.filesystem_id, to_tsvector('default', coalesce(NEW.strings,'')));
			INSERT INTO filesystem_filename_idx (filesystem_id, vectors) VALUES (NEW.filesystem_id, to_tsvector('default', coalesce(NEW.file_name,'')));
		ELSIF TG_OP = 'UPDATE' AND (OLD.strings <> NEW.strings OR OLD.file_name <> NEW.file_name) THEN
			UPDATE filesystem_idx SET vectors=to_tsvector('default', coalesce(NEW.strings,'')) WHERE filesystem_id=NEW.filesystem_id;
			UPDATE filesystem_filename_idx SET vectors=to_tsvector('default', coalesce(NEW.file_name,'')) WHERE filesystem_id=NEW.filesystem_id;
		END IF;

	ELSIF table_name = 'forum_message' THEN
		IF TG_OP = 'INSERT' THEN
			INSERT INTO forum_message_idx (forum_message_id, forum_thread_id, vectors) VALUES (NEW.forum_message_id, NEW.forum_thread_id, to_tsvector('default', coalesce(NEW.subject,'') ||' '|| coalesce(NEW.body,'')));
		ELSIF TG_OP = 'UPDATE' AND (OLD.subject <> NEW.subject OR OLD.body <> NEW.body) THEN
			UPDATE forum_message_idx SET vectors=to_tsvector('default', coalesce(NEW.subject,'') ||' '|| coalesce(NEW.body,'')) WHERE forum_message_id=NEW.forum_message_id;
		END IF;

	ELSIF table_name = 'frs_release' THEN
		IF TG_OP = 'INSERT' THEN
			INSERT INTO frs_release_idx (frs_release_id, vectors) VALUES (NEW.frs_release_id, to_tsvector('default', coalesce(NEW.changes,'') ||' '|| coalesce(NEW.release_notes,'') ||' '|| coalesce(NEW.release_name,'')));
		ELSIF TG_OP = 'UPDATE' AND OLD.changes <> NEW.changes THEN
			UPDATE frs_release_idx SET vectors=to_tsvector('default', coalesce(NEW.changes,'') ||' '|| coalesce(NEW.release_notes,'') ||' '|| coalesce(NEW.release_name,'')) WHERE frs_release_id=NEW.frs_release_id;
		END IF;
	ELSIF table_name = 'snippet' THEN
		IF TG_OP = 'INSERT' THEN
			INSERT INTO snippet_idx (snippet_id, vectors) VALUES (NEW.snippet_id, to_tsvector('default', coalesce(NEW.snippet_name,'') ||' '|| coalesce(NEW.snippet_description,'')));
		ELSIF TG_OP = 'UPDATE' THEN
			UPDATE snippet_idx SET vectors=to_tsvector('default', coalesce(NEW.snippet_name,'') ||' '|| coalesce(NEW.snippet_description,'')) WHERE snippet_id=NEW.snippet_id;
		END IF;

	ELSIF table_name = 'project' THEN
		IF TG_OP = 'INSERT' THEN
			INSERT INTO projects_idx (project_id, vectors) VALUES (NEW.project_id, to_tsvector('default', coalesce(NEW.project_name,'') ||' '|| coalesce(NEW.description,'') ||' '|| coalesce(NEW.unix_name,'')));
		ELSIF TG_OP = 'UPDATE' AND (OLD.description <> NEW.description OR OLD.project_name <> NEW.project_name OR OLD.unix_name <> NEW.unix_name) THEN
			UPDATE projects_idx SET vectors=to_tsvector('default', coalesce(NEW.project_name,'') ||' '|| coalesce(NEW.description,'') ||' '|| coalesce(NEW.unix_name,'')) WHERE project_id=NEW.project_id;
		END IF;

	ELSIF table_name = 'news' THEN
		IF TG_OP = 'INSERT' THEN
			INSERT INTO news_idx (news_id, vectors) VALUES (NEW.news_id, to_tsvector('default', coalesce(NEW.summary,'') ||' '|| coalesce(NEW.details,'')));
		ELSIF TG_OP = 'UPDATE' THEN
			UPDATE news_idx SET vectors=to_tsvector('default', coalesce(NEW.summary,'') ||' '|| coalesce(NEW.details,'')) WHERE news_id=NEW.news_id;
		END IF;

	ELSIF table_name = '"user"' THEN
		IF TG_OP = 'INSERT' THEN
			INSERT INTO users_idx (user_id, vectors) VALUES (NEW.user_id, to_tsvector('default', coalesce(NEW.unix_name,'') ||' '|| coalesce(NEW.firstname,'') ||' '|| coalesce(NEW.lastname,'') ));
		ELSIF TG_OP = 'UPDATE' AND (OLD.unix_name <> NEW.unix_name OR OLD.firstname <> NEW.firstname OR OLD.lastname <> NEW.lastname) THEN
			UPDATE users_idx SET vectors=to_tsvector('default', coalesce(NEW.unix_name,'') ||' '|| coalesce(NEW.firstname,'') ||' '|| coalesce(NEW.lastname,'') ) WHERE user_id=NEW.user_id;
		END IF;
	ELSIF table_name = 'trove_category' THEN
		IF TG_OP = 'INSERT'  THEN
			IF (SELECT show_in_search FROM trove_category WHERE trove_category_id = NEW.trove_category_id) THEN
				INSERT INTO trove_category_idx (trove_category_id, vectors) VALUES (NEW.trove_category_id, to_tsvector('default', coalesce(NEW.alias,'') || ' ' || coalesce(NEW.category_name,'') || ' ' || coalesce(NEW.description,'')));
			END IF;
		ELSIF TG_OP = 'UPDATE' THEN
      IF NEW.show_in_search  THEN
		    IF EXISTS (SELECT 1 FROM trove_category_idx WHERE trove_category_id = NEW.trove_category_id) THEN
          UPDATE trove_category_idx SET vectors = to_tsvector('default', coalesce(NEW.alias, '') || ' ' || coalesce(NEW.category_name,'') || ' ' || coalesce(NEW.description,'')) WHERE trove_category_id = NEW.trove_category_id;
			  ELSE
				  INSERT INTO trove_category_idx (trove_category_id, vectors) VALUES (NEW.trove_category_id, to_tsvector('default', coalesce(NEW.alias,'') || ' ' || coalesce(NEW.category_name,'') || ' ' || coalesce(NEW.description,'')));
        END IF;
		  ELSIF NOT NEW.show_in_search THEN
			    DELETE FROM trove_category_idx WHERE trove_category_id=NEW.trove_category_id;
      END IF;
    END IF;
    -- Not needed since we CASCADE on DELETE, but I'll left it here for documentation purposes
    --ELSIF TG_OP = 'DELETE' THEN --Not sure if really needed, but just in case
      --DELETE FROM trove_category_idx WHERE trove_category_id=NEW.trove_category_id;
    --END IF;

	END IF;

	RETURN NEW;
END;
$BODY$
LANGUAGE plpgsql VOLATILE;
