CREATE OR REPLACE FUNCTION trove_merge(category_id bigint, to_id bigint)
  RETURNS integer AS
$BODY$
DECLARE
    trove_cat RECORD; -- trove category record for inmediate children of source category
    children_count integer; -- count of source category inmediate children
	last_count integer;   -- temp counter
	temp RECORD;		  -- temp record
	template_trove RECORD; -- fixed trove category record used as default project templates container
BEGIN
    children_count := 0;
    -- Move source category inmediate children to destination category
    FOR trove_cat IN SELECT trove_category_id FROM trove_category WHERE parent_trove_category_id = category_id LOOP
        children_count := children_count + 1;
        PERFORM trove_move(trove_cat.trove_category_id, to_id);
    END LOOP;
    -- If any children moved
    IF (children_count > 0) 
    THEN
		-- If about to delete a category in the templates container subtree,
		-- update projects linking to category_id to now link to subtree root
		SELECT trove_category_id INTO template_trove FROM trove_category WHERE alias = 'gfprojtemplates';
		SELECT 1 INTO temp FROM trove_category WHERE trove_category_id = category_id AND trove_category_id IN (
			SELECT trove_category_id FROM trove_children(template_trove.trove_category_id)
		);
		GET DIAGNOSTICS last_count = ROW_COUNT;
		IF last_count = 1
		-- category_id is in the templates container subtree
		THEN
			UPDATE trove_link SET trove_category_id = template_trove.trove_category_id WHERE section='project' AND trove_category_id = category_id;
		END IF;
        -- Delete source category
        DELETE FROM trove_category WHERE trove_category_id = category_id;
    END IF;
    -- Return count of children moved
    RETURN children_count;
END;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;