<?php
/**
 * Adds email alias to trackers with none defined. Skips trackers which:
 * - Already have an email alias
 * - Have a required tracker extra field but no elements
 * - Have a required tracker extra field where no element is defined as default
 * - Have a required tracker extra field which can't have elements defined
 *     (textarea, texbox) but no default content is defined
 */
require_once('initialize.php');
require_once('GFConf.php');
require_once("plugins/tracker/lib/TrackerPeer.php");
require_once("plugins/mailman/lib/MailmanPeer.php");

$con = Propel::getConnection();
$stmt = $con->createStatement();
$sql = <<<SQL
    SELECT tracker.tracker_id
    FROM tracker
    WHERE
        -- skip trackers with email alias defined
        tracker.tracker_id NOT IN
        (
            SELECT ref_id AS tracker_id
            FROM setting
            WHERE section='tracker' AND setting_name='email_alias'
        )
        -- skip trackers that have a required extra field with no default
        -- value defined
        AND tracker.tracker_id NOT IN
        (
            -- trackers with extra fields that are set as required, the extra
            -- field has no element selected as default
            SELECT DISTINCT tracker.tracker_id
            FROM tracker
            NATURAL JOIN tracker_extra_field tef
            WHERE
                tef.field_type IN (1,2,3,5,7,8,12)
                AND tef.is_required
                AND tef.tracker_extra_field_id NOT IN
                (
                    -- tracker extra fields set as required, with an element
                    -- as default
                    SELECT distinct tef.tracker_extra_field_id
                    FROM tracker_extra_field tef
                    LEFT JOIN tracker_extra_field_element tefe
                    ON tef.tracker_extra_field_id=tefe.tracker_extra_field_id
                    WHERE tef.field_type IN (1,2,3,5,7,8,12)
                        AND tef.is_required
                        AND tefe.default_selected
                )

            UNION

            -- trackers with extra fields (of 'text' type) that are set as
            -- required, default value not defined
            SELECT DISTINCT tracker.tracker_id
            FROM tracker
            NATURAL JOIN tracker_extra_field tef
            WHERE
                tef.field_type NOT IN (1,2,3,5,7,8,12)
                AND (tef.default_value IS NULL OR tef.default_value='')
                AND tef.is_required

        )
SQL;

$hostName = GFConf::get('system.hostName');
$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
$counter = 0;
// avoid E_DEPRECATED warnings, Validator uses ancient ereg()
error_reporting(E_ALL & ~E_DEPRECATED);
while ($rs->next()) {
    $tracker_id = $rs->getString('tracker_id');
    $tracker = TrackerPeer::retrieveByPk($tracker_id);
    //Based on TrackerAddAction.php, see lines 66 to 97.
    $email_alias = $tracker->getDefaultEmailAlias();
    if (!empty($email_alias)
        && Validator::validateEmail($email_alias.'@'.$hostName)
        && Validator::validateEmailAlias($email_alias)
        && $tracker->isEmailAliasUnique($email_alias)
        ) {
        $tracker->setEmailAlias($email_alias);
        $counter++;
    }
}
//execute cronjob to updated postfix aliases
$base_str = 'cd /opt/gforge && /usr/bin/php -d include_path=".:/opt/gforge:/opt/gforge/lib:/opt/gforge/plugins:/opt/gforge/lib/external" /opt/gforge/bin/gforge cronjob  /opt/gforge';
`$base_str/cronjobs/mail_aliases.php`;
$plural = (!$counter || $counter>1) ? 's' : '';

exit(0);
