<?php
/**
 * GForge 5.0 theme
 *
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('session/Session.php');
require_once('SystemUtils.php');

class ExcelXML {

	const STYLE_HEADER = 'header';
	const STYLE_STRING = 'string';
	const STYLE_NUMBER = 'number';
	const STYLE_DATE = 'date';

	private $user;
	private $created_time;
	private $saved_time;
	private $saved_styles;
	private $style_types;
	private $xml_styles;
	private $xml;
	private $headers;
	private $contents;
	private $row_is_open = false;
	private $bookname;
	
	function __construct($name) {
		$this->bookname = $name;
		$session = Session::getSession();
		$this->user = $session->getUser();
		$this->initObject();
	}
	
	public function openRow() {
		if ($this->row_is_open) {
			$this->closeRow();
		}
		$this->contents .= "<Row>\r\n";
		$this->row_is_open = true;
	}
	
	public function closeRow() {
		if ($this->row_is_open) {
			$this->contents .= "</Row>\r\n";
			$this->row_is_open = false;
		}
	}

	public function setHeader($data) {
		if (is_array($data)) {
			foreach ($data as $item) {
				$this->setHeader($item);
			}
		} else {
			$this->headers[] = $data;
		}
	}

	private function setCellHeader($data) {
		$style = $this->style_types[ExcelXML::STYLE_HEADER];
		return $this->setCell($data, $style, 'String', true);
	}

	public function setCellString($data) {
		$style = $this->style_types[ExcelXML::STYLE_STRING];
		$this->setCell($data, $style, 'String');
	}

	public function setCellNumber($data) {
		$style = $this->style_types[ExcelXML::STYLE_NUMBER];
		$this->setCell($data, $style, 'Number');
	}

	public function setCellDate($data) {
		$style = $this->style_types[ExcelXML::STYLE_DATE];
		if (strlen($data) == 10) {
			$data .= 'T00:00:00.000';
		}
		$this->setCell($data, $style, 'DateTime');
	}

	public function setCell($data, $style, $type, $ret = false, $trim=true) {
		if (!$this->row_is_open) {
			$this->openRow();
		}
		if($trim) {
			$data = trim(htmlspecialchars($data));
		}
		$result = '';
		if ($data == '') {
			$result .= "<Cell ss:StyleID=\"$style\"/>";
		} else {
			$result .= "<Cell ss:StyleID=\"$style\"><Data ss:Type=\"$type\">$data</Data></Cell>";
		}
		if ($ret) {
			return $result;
		}
		else {
			$this->contents .= $result."\r\n";
		}
	}

	public function setStyle($name, $params, $type = '') {
		if ($type != ExcelXML::STYLE_HEADER && $type != ExcelXML::STYLE_STRING && 
			$type != ExcelXML::STYLE_NUMBER && $type != ExcelXML::STYLE_DATE) {
			$this->xml_styles[$name] =$this->doStyle($name, $params);
		} else {
			$this->style_types[$type] = $name;
			$this->xml_styles[$name] =$this->doStyle($name, $params);
		}
	}
	
	public function getStyle($type) {
		return $this->style_types[$type];
	}
	
	public function getXML() {
		$this->xml .= $this->doXMLHeaders();
		$this->xml .= $this->doXMLSheet();
		return $this->xml;
	}
	
	private function doXMLHeaders() {
		$author = is_object($this->user) ? $this->user->getFullName() : '(anonymous user)';
		$this->saved_time = date('Y-m-d\TH:i:s');

		$arrHeader = array();
		$arrHeader[] = '<?xml version="1.0"?>';
		$arrHeader[] = '<Workbook xmlns="urn:schemas-microsoft-com:office:spreadsheet"';
		$arrHeader[] = ' xmlns:o="urn:schemas-microsoft-com:office:office"';
		$arrHeader[] = ' xmlns:x="urn:schemas-microsoft-com:office:excel"';
		$arrHeader[] = ' xmlns:ss="urn:schemas-microsoft-com:office:spreadsheet">';
		$arrHeader[] = '<DocumentProperties xmlns="urn:schemas-microsoft-com:office:office">';
		$arrHeader[] = '<Author>'.$author.'</Author>';
		$arrHeader[] = '<LastAuthor>'.$author.'</LastAuthor>';
		$arrHeader[] = '<Created>'.$this->created_time.'</Created>';
		$arrHeader[] = '<LastSaved>'.$this->saved_time.'</LastSaved>';
  		$arrHeader[] = '</DocumentProperties>';
		$arrHeader[] = '<Styles>';
		foreach ($this->xml_styles as $style) {
			$arrHeader[] = $style;
		}
		$arrHeader[] = '</Styles>';
		return implode("\r\n", $arrHeader);
	}

	private function doXMLSheetHeader() {
		$arrSheetHeader = array();
 		foreach ($this->headers as $header) {
  			$arrSheetHeader[] = '<Column ss:AutoFitWidth="1"/>';
  		}
  		$arrSheetHeader[] = '<Row>';
  		$this->row_is_open = true;
  		foreach ($this->headers as $header) {
  			$arrSheetHeader[] = $this->setCellHeader($header);
  		}
  		$this->row_is_open = false;
  		$arrSheetHeader[] = '</Row>';
  		return implode("\r\n", $arrSheetHeader);
  	}

	private function doXMLSheet() {
		$arrBook = array();
		$arrBook[] = '<Worksheet ss:Name="'.$this->bookname.'">';
  		$arrBook[] = '<Table x:FullColumns="1" x:FullRows="1">';
  		$arrBook[] = $this->doXMLSheetHeader();
  		$arrBook[] = trim($this->contents);
  		$arrBook[] = '</Table>';
  		$arrBook[] = '<WorksheetOptions xmlns="urn:schemas-microsoft-com:office:excel">';
  		$arrBook[] = '<PageSetup><Header x:Margin="0"/><Footer x:Margin="0"/>';
  		$arrBook[] = '<PageMargins/></PageSetup>';
  		$arrBook[] = '<Selected/><ProtectObjects>False</ProtectObjects>';
  		$arrBook[] = '<ProtectScenarios>False</ProtectScenarios>';
  		$arrBook[] = '</WorksheetOptions></Worksheet>';
  		$arrBook[] = '</Workbook>';
		return implode("\r\n", $arrBook);
	}

	private function doStyle($name, $params) {
		$this->saved_styles[$name] = $params;
		$border = (isset($params['border']) && $params['border']==true) ? true : false;
		$isDate = (isset($params['isDate']) && $params['isDate']==true) ? true : false;
		$fontColor = (isset($params['fontColor'])) ? $params['fontColor'] : '#000000';
		$fontSize = (isset($params['fontSize'])) ? $params['fontSize'] : '10';
		$fontName = (isset($params['fontName'])) ? $params['fontName'] : 'Arial';
		$fontWeight = (isset($params['fontWeight'])) ? $params['fontWeight'] : '0';
		$bgColor = (isset($params['bgColor'])) ? $params['bgColor'] : '#FFFFFF';
		$alignment = (isset($params['alignment'])) ? $params['alignment'] : false;
		$format = array();
		$format[] = "<Style ss:ID=\"$name\">";
		if ($alignment !== false) {
			$format[] = "<Alignment $alignment />";
		}
		if ($border) {
			$format[] = '<Borders>';
			$format[] = '<Border ss:Position="Bottom" ss:LineStyle="Continuous" ss:Weight="1"/>';
			$format[] = '<Border ss:Position="Left" ss:LineStyle="Continuous" ss:Weight="1"/>';
			$format[] = '<Border ss:Position="Right" ss:LineStyle="Continuous" ss:Weight="1"/>';
			$format[] = '<Border ss:Position="Top" ss:LineStyle="Continuous" ss:Weight="1"/>';
			$format[] = '</Borders>';
		}
		$format[] = "<Font ss:FontName=\"$fontName\" ss:Size=\"$fontSize\" ";
		$format[] = "ss:Color=\"$fontColor\" ss:Bold=\"$fontWeight\"/>";
		$format[] = "<Interior ss:Color=\"$bgColor\" ss:Pattern=\"Solid\"/>";
		if ($isDate) {
			$format[] = "<NumberFormat ss:Format=\"Short Date\"/>";
		}
		$format[] = '</Style>';
		return implode("\r\n", $format);
	}

	private function initObject() {
		$this->created_time = date('Y-m-d\TH:i:s');
		$this->xml = '';
		$this->contents = '';
		$this->headers = array();
		$this->style_types = array();
		$this->xml_styles = array();
		$this->saved_styles = array();

		// Init Default Header Style
		$hdrParams = array('border'=>false, 'bgColor'=>'#339966', 'fontName'=>'Verdana',
									'fontColor'=>'#FFFFFF', 'fontWeight'=>'1');
		$this->setStyle('sHeader', $hdrParams, ExcelXML::STYLE_HEADER);

		// Init Default String Style
		$strParams = array('border'=>true, 'bgColor'=>'#CCFFCC');
		$this->setStyle('sString', $strParams, ExcelXML::STYLE_STRING);
		$this->setStyle('sNumber', $strParams, ExcelXML::STYLE_NUMBER);

		// Init Default Date Style
		$dateParams = array('border'=>true, 'bgColor'=>'#CCFFCC', 'isDate'=>true);
		$this->setStyle('sDate', $dateParams, ExcelXML::STYLE_DATE);
	}
}
?>
