<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */


class Permission {

	private $Project;
	private	$rolesettings = array();
	private $Role;
	private $User = null;

	function __construct($Project, $User = null) {
		$this->Project = $Project;

		//RolePeer - get RoleSetting array
		$sess = Session::getSession();

		if (is_null($User)) {
			// Use the session user by default
			$this->User = $sess->getUser();
		} else {
			$this->User = $User;
		}

		if (is_null($this->User)) {
			// $this->User may be null if for example we're running with cronjob privileges
			return;
		}
		$user_id = $this->User->getUserId();

		if ($this->Project->getParentProjectId() > 0) {
			$projectsql=" IN (".$this->Project->getRootPath().")";
		} else {
			$projectsql="='".$this->Project->getProjectId()."'";
		}
		$sql = "SELECT role_setting.section,role_setting.ref_id, role_setting.related_id, max(value) AS value FROM role_setting
			NATURAL JOIN user_project_role NATURAL JOIN user_project
			WHERE section=role_setting.section AND ref_id=role_setting.ref_id
			AND (user_project.user_id='$user_id' OR user_project.user_id IN
				(SELECT usergroup_id FROM usergroup_user WHERE member_user_id='$user_id'))
			AND user_project.project_id $projectsql
			GROUP BY section,ref_id,related_id";
		$rs = Propel::getConnection()->createStatement()->executeQuery($sql);

		while ($rs->next()) {
			$this->rolesettings[$rs->getString('section')][$rs->getInt('ref_id')]= array(
				'value' => $rs->getInt('value'),
				'related_id' => $rs->getInt('related_id')
			);
		}
	}

	function getRole() {
		return $this->Role;
	}

	private function findRoleSetting($section, $ref_id) {
		if (array_key_exists($section, $this->rolesettings)) {
			$s = $this->rolesettings[$section];
			if (is_array($s) && array_key_exists($ref_id, $s)) {
				$r = $s[$ref_id];
				if (is_array($r)) return $r;
			}
		}
		return false;
	}

	private function getValue($section, $ref_id) {
		$r = $this->findRoleSetting($section, $ref_id);
		if (is_array($r) && array_key_exists('value', $r)) {
			return $r['value'];
		}
		return false;
	}

	public function getRelatedId($section, $ref_id) {
		$r = $this->findRoleSetting($section, $ref_id);
		if (is_array($r) && array_key_exists('related_id', $r)) {
			return $r['related_id'];
		}
		return false;
	}

	function check ($section, $ref_id, $level) {
		// If the user is null, it means we're referring to the session user
		if (is_null($this->User)) {
			$sess = Session::getSession();
			if (!$sess->isLoggedIn()) {
				return false;
			}
		}

		if ($this->isAdmin()) {
			return true;
		} else {
			$val = $this->getValue($section, $ref_id);
			if ($val !== false && $val >= $level) {
				return true;
			}
		}
		return false;
	}

	public function isProjectAdmin() {
		return $this->getValue('projectadmin', 0) == Project::PROJECT_ROLE_ADMIN;
	}

	/**
	 * User is administrator for this project?
	 * @return	bool
	 */
	public function isAdmin() {
		if (is_null($this->User)) {
			// If the user is null, it means we're referring to the session user
			// (usually means this is running like a cronjob, so the line below
			// would return true)
			return Session::getSession()->isSiteAdmin();
		}

		if ($this->User->isSiteAdmin() ||
			$this->isProjectAdmin()) {
			return true;
		}
		return false;
	}

    public function higherThan($section, $ref_id, $level) {
        if ($this->isAdmin() ) {
            return true;
        } else {
            $val = $this->getValue($section, $ref_id);
			if ($val > $level) {
				return true;
			}
        }
    }

    public function checkAllHigherThan($section, $ref_ids, $level) {
        if ($this->isAdmin()) {
            return true;
        } else {
            $values = array();

            foreach ($ref_ids as $ref_id) {
                $values[] = $this->getValue($section, $ref_id);
            }

            return min($values) > $level;
        }
    }
}

?>
