<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	<ruben@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

class TemplateXML {
	private $dom;
	private $template;
	private $version;
	
	function __construct(Project $ProjectTemplate, $version) {
		$this->dom = new DOMDocument('1.0', 'UTF-8');
		#Beautify the output
		$this->dom->formatOutput = true;
		$this->template = $ProjectTemplate;
		$this->version = $version;
		$this->convert();
	}
	
	private function convert() {
		$domGForge = $this->dom->createElement('GForge');
		$domGForgeNode = $this->dom->appendChild($domGForge);
		$domTemplate = $this->dom->createElement('ProjectTemplate');
		$domTemplate->setAttribute('version', $this->version);
		$domTemplateNode = $domGForge->appendChild($domTemplate);
		
		$domTemplateUnixName = $this->dom->createElement('UnixName', $this->template->getUnixName());
		$domTemplateNode->appendChild($domTemplateUnixName);
		$domTemplateName = $this->dom->createElement('Name', $this->template->getProjectName());
		$domTemplateNode->appendChild($domTemplateName);
		$domTemplateName = $this->dom->createElement('Description', $this->template->getDescription());
		$domTemplateNode->appendChild($domTemplateName);
		
		$this->doRoles($domTemplateNode);
		$this->doPlugins($domTemplateNode);
	}
	
	private function doRoles($domTemplateNode) {
		//Export Roles associated with the project

		$crit = new Criteria();
		$crit->add(RolePeer::PROJECT_ID, $this->template->getProjectId());
		$crit->addAscendingOrderByColumn(RolePeer::ROLE_NAME);
		$roles = RolePeer::doSelect($crit);

		foreach ($roles as $role) {
			$domRole = $this->dom->createElement('Role');
			$domRole->setAttribute('name', $role->getRoleName());
			$c = new Criteria();
			$c->add(RoleSettingPeer::ROLE_ID,$role->getRoleID());
			$c->add(RoleSettingPeer::SECTION, 'projectadmin');
			$rolesetting = RoleSettingPeer::doSelectOne($c);
			if(is_object($rolesetting)) $domRole->setAttribute('projectadmin', $rolesetting->getValue());
			$domTemplateNode->appendChild($domRole);
		}
		
	}

	private function doPlugins($domTemplateNode) {
		$domPlugins = $this->dom->createElement('Plugins');
		$domTemplateNode->appendChild($domPlugins);
		$this->convertTracker($domPlugins);
		$this->convertMailman($domPlugins);
		$this->convertFrs($domPlugins);
		$this->convertForum($domPlugins);
		$this->convertDocman($domPlugins);
		$this->convertSCM($domPlugins);
	}
	
	private function convertTracker($domPlugins) {
		$TrackerPlugin = $this->template->getPluginByName(TrackerPlugin::SECTION);
		// Check if the template is using the Frs plugin
		if ($TrackerPlugin  != null && is_object($TrackerPlugin )) {
			$domTrackers = $this->dom->createElement('Trackers');
			$this->convertRoleSetting($domTrackers, 'tracker', 0);
			// Get all packages
			$c = new Criteria();
			$c->add(TrackerPeer::PROJECT_ID, $this->template->getProjectId());

			$TemplateTrackers = TrackerPeer::doSelect($c);
			foreach ($TemplateTrackers as $Tracker) {
				$domTracker = $this->dom->createElement('Tracker');
				$domTracker->setAttribute('name', $Tracker->getTrackerName());
				$domTracker->setAttribute('is_public', ($Tracker->getIsPublic() == true) ? 'true' : 'false');
				$domTracker->setAttribute('restrict_browse', ($Tracker->getRestrictBrowse() == true) ? 'true' : 'false');
				$domTracker->setAttribute('email_updates', ($Tracker->getEmailAllUpdates() == true) ? 'true' : 'false');
				$domTracker->setAttribute('due_period', $Tracker->getDuePeriod());
				$domTracker->setAttribute('type', $Tracker->getDatatype());
				$domTrackerDesc = $this->dom->createElement('Description', $Tracker->getDescription());
				$domTracker->appendChild($domTrackerDesc);
				$domTracker->appendChild($this->convertTrackerFields($Tracker));
				$this->convertRoleSetting($domTracker, 'tracker', $Tracker->getTrackerId());
				$domTrackers->appendChild($domTracker);
			}
			$domPlugins->appendChild($domTrackers);
		}
	}
	
	private function convertTrackerFields($Tracker) {
		$domExtraFields = $this->dom->createElement('ExtraFields');
		// Get all the extra fields
		$ExtraFields = $Tracker->getTrackerExtraFields();
		foreach ($ExtraFields as $ExtraField) {
			$domExtraField = $this->dom->createElement('ExtraField');
			$domExtraField->setAttribute('name', $ExtraField->getFieldName());
			$domExtraField->setAttribute('type', $ExtraField->getFieldType());
			$domExtraField->setAttribute('attr1', $ExtraField->getAttribute1());
			$domExtraField->setAttribute('attr2', $ExtraField->getAttribute2());

			$domExtraFieldProp = $this->dom->createElement('Properties');
			$domExtraFieldProp->setAttribute('order', $ExtraField->getFieldOrder());
			$domExtraFieldProp->setAttribute('required', ($ExtraField->getIsRequired() == true) ? 'true' : 'false');
			$domExtraFieldProp->setAttribute('quick_browse', ($ExtraField->getUseInQuickBrowse() == true) ? 'true' : 'false');
			$domExtraFieldProp->setAttribute('on_submit', ($ExtraField->getShowOnSubmit() == true) ? 'true' : 'false');
			//$domExtraFieldProp->setAttribute('auto_assign', ($ExtraField->getUseAutoAssign() == true) ? 'true' : 'false');
			$domExtraFieldProp->setAttribute('workflow', ($ExtraField->getUseWorkFlow() == true) ? 'true' : 'false');
			$domExtraFieldProp->setAttribute('commit_rule', ($ExtraField->getUseCommitRule() == true) ? 'true' : 'false');
			$domExtraField->appendChild($domExtraFieldProp);
			$domExtraField->appendChild($this->convertTrackerFieldValues($ExtraField));
			$domExtraFields->appendChild($domExtraField);
		}
		return $domExtraFields;
	}
	
	private function convertTrackerFieldValues($ExtraField) {
		$domFieldElements = $this->dom->createElement('ExtraFieldElements');
		// Get all the extra fields elements
		$FieldElements = $ExtraField->getTrackerExtraFieldElements();
		foreach ($FieldElements as $FieldElement) {
			$domFieldElement = $this->dom->createElement('ExtraFieldElement');
			$domFieldElement->setAttribute('name', $FieldElement->getElementName());
			$domFieldElement->setAttribute('status', $FieldElement->getStatusId());
			$domFieldElement->setAttribute('order', $FieldElement->getSortOrder());
			$domFieldElement->setAttribute('default', ($FieldElement->getDefaultSelected() == true) ? 'true' : 'false');
			$domFieldElement->setAttribute('can_commit', ($FieldElement->getCanCommit() == true) ? 'true' : 'false');

			$this->convertTrackerFieldValueTransitionRules($FieldElement, $domFieldElement);
			$domFieldElements->appendChild($domFieldElement);
			
		}
		return $domFieldElements;
	}
	
	private function convertTrackerFieldValueTransitionRules($FieldElement, $domNode) {
		//$domTransitionRules = $this->dom->createElement('ExtraFieldElementTransitionRules');
		$TransitionRules = $FieldElement->getTrackerExtraFieldElementTransitionsRelatedByElementId();
		
		foreach ($TransitionRules as $TransitionRule) {
			$domTransitionRule = $this->dom->createElement('ExtraFieldElementTransitionRule');
			
			$domTransitionRule->setAttribute('toElement', TrackerExtraFieldElementPeer::retrieveByPk($TransitionRule->getToElementId())->getElementName());
			$domTransitionRule->setAttribute('notificationMessage', $TransitionRule->getNotificationMessage());
						
			//Export the Fields assigned to the rule
			$fields = $TransitionRule->getTrackerExtraFieldElementTransitionFieldsJoinTrackerExtraField();
			foreach ($fields as $field) {
				$domTransitionRuleFields = $this->dom->createElement('ExtraFieldElementTransitionRuleField');
				$domTransitionRuleFields->setAttribute('name', $field->getTrackerExtraField()->getFieldName());
				$domTransitionRule->appendChild($domTransitionRuleFields);
			}
			
			//Export the roles assigned to the Rule
			$roles = $TransitionRule->getTrackerExtraFieldElementTransitionRolesJoinRole();
			foreach ($roles as $role) {
				$domTransitionRuleRole = $this->dom->createElement('ExtraFieldElementTransitionRuleRole');
				$domTransitionRuleRole->setAttribute('name', $role->getRole()->getRoleName());
				$domTransitionRule->appendChild($domTransitionRuleRole);
			}
			
			//Export the roles to be notified assigned to the Rule
			$notifyroles = $TransitionRule->getTrackerExtraFieldElementTransitionNotifysJoinRole();
			foreach ($notifyroles as $role) {
				$domTransitionRuleNotifyRole = $this->dom->createElement('ExtraFieldElementTransitionRuleNotifyRole');
				$domTransitionRuleNotifyRole->setAttribute('name', $role->getRole()->getRoleName());
				$domTransitionRule->appendChild($domTransitionRuleNotifyRole);
			}
			
			//Export the roles to which the element will be reassigned.
			$reassign_to_roles = $TransitionRule->getTrackerEfeTransitionReassigns();
			foreach ($reassign_to_roles as $role) {
				$domTransitionRuleAssignRole = $this->dom->createElement('ExtraFieldElementTransitionRuleAssignRole');
				$domTransitionRuleAssignRole->setAttribute('name', $role->getRole()->getRoleName());
				$domTransitionRule->appendChild($domTransitionRuleAssignRole);
			}

			$domNode->appendChild($domTransitionRule);
		}
		return $domNode;
	}
	
	private function convertMailman($domPlugins) {
		$MailmanPlugin = $this->template->getPluginByName(MailmanPlugin::SECTION);
		// Check if the template is using the Mailman plugin
		if ($MailmanPlugin  != null && is_object($MailmanPlugin )) {
			$domMailman = $this->dom->createElement('Mailman');
			// Get all lists
			$c = new Criteria();
			$c->add(MailmanPeer::PROJECT_ID, $this->template->getProjectId());

			$TemplateLists = MailmanPeer::doSelect($c);
			foreach ($TemplateLists as $List) {
				$domMailmanList = $this->dom->createElement('List');
				// Extract the project name from the unix name
				$name = $List->getListName();
				preg_match('/^'.$this->template->getUnixName().'-(.*)/i', $name, $matches);
				$domMailmanList->setAttribute('name', $matches[1]);
				$domMailmanList->setAttribute('is_public', ($List->getIsPublic() == true) ? 'true' : 'false');
				$domMailmanListDesc = $this->dom->createElement('Description', $List->getListDescription());
				$domMailmanList->appendChild($domMailmanListDesc);
				$domMailman->appendChild($domMailmanList);
				$this->convertRoleSetting($domMailman, 'mailman', $List->getMailmanId());
			}
			$domPlugins->appendChild($domMailman);
		}
	}
	
	private function convertFrs($domPlugins) {
		$FrsPlugin = $this->template->getPluginByName(FrsPlugin::SECTION);
		
		// Check if the template is using the Frs plugin
		if ($FrsPlugin  != null && is_object($FrsPlugin )) {
			$domFrs = $this->dom->createElement('Frs');
			$this->convertRoleSetting($domFrs, 'frs', 0);
			
			// Get all packages
			$c = new Criteria();
			$c->add(FrsPackagePeer::PROJECT_ID, $this->template->getProjectId());

			$TemplatePackages = FrsPackagePeer::doSelect($c);
			foreach ($TemplatePackages as $Package) {
				$domFrsPackage = $this->dom->createElement('Package');
				$domFrsPackage->setAttribute('name', $Package->getPackageName());
				$domFrsPackage->setAttribute('is_public', ($Package->getIsPublic() == true) ? 'true' : 'false');
				$domFrsPackage->setAttribute('status', $Package->getStatusId());
				$domFrsPackage->setAttribute('req_login', ($Package->getRequireLogin() == true) ? 'true' : 'false');
				$domFrs->appendChild($domFrsPackage);
			}
			$domPlugins->appendChild($domFrs);
		}
	}

	private function convertForum($domPlugins) {
		$ForumPlugin = $this->template->getPluginByName(ForumPlugin::SECTION);
		// Check if the template is using the Forum plugin
		if ($ForumPlugin  != null && is_object($ForumPlugin )) {
			$domForums = $this->dom->createElement('Forums');
			
			$this->convertRoleSetting($domForums, 'forum', 0);
			
			// Get all forums
			$c = new Criteria();
			$c->add(ForumPeer::SECTION,'project');
			$c->add(ForumPeer::REF_ID, $this->template->getProjectId());
			
			$TemplateForums = ForumPeer::doSelect($c);
			foreach ($TemplateForums as $Forum) {
				$domForumsForum = $this->dom->createElement('Forum');
				$domForumsForum->setAttribute('name', $Forum->getForumName());
				$domForumsForum->setAttribute('is_public', ($Forum->getIsPublic() == true) ? 'true' : 'false');
				$domForumsForum->setAttribute('mod_level', $Forum->getModerationLevel());
				$domForumsForumDesc = $this->dom->createElement('Description', $Forum->getDescription());
				$domForumsForum->appendChild($domForumsForumDesc);
				$this->convertRoleSetting($domForumsForum, 'forum', $Forum->getForumId());
				$domForums->appendChild($domForumsForum);
			}
			$domPlugins->appendChild($domForums);
		}
	}

	private function convertDocman($domPlugins) {
		$DocmanPlugin = $this->template->getPluginByName(DocmanPlugin::SECTION);
		// Check if the template is using the Docman plugin
		if ($DocmanPlugin != null && is_object($DocmanPlugin)) {
			$domDocman = $this->dom->createElement('Docman');
			$domDocman = $this->convertDocmanChildren($domDocman, 0);
			$this->convertRoleSetting($domDocman, 'docman', 0);
			$domPlugins->appendChild($domDocman);
		}
	}

	private function convertDocmanChildren($domObj, $parent_folder_id) {
		$project_id = $this->template->getProjectId();

		$c = new Criteria();
		$c->add(DocmanFolderPeer::PROJECT_ID, $project_id);
		$c->add(DocmanFolderPeer::PARENT_FOLDER_ID, $parent_folder_id);
	
		$DocmanFolders = DocmanFolderPeer::doSelect($c);
		foreach ($DocmanFolders as $Folder) {
			$domDocmanFolder = $this->dom->createElement('Folder');
			$domDocmanFolder->setAttribute('name', $Folder->getFolderName());
			$domDocmanFolder->setAttribute('is_public', ($Folder->getIsPublic() == true) ? 'true' : 'false');
			$domObj->appendChild($domDocmanFolder);
			// Recurse to get all the children
			$domDocmanFolder = $this->convertDocmanChildren($domDocmanFolder, $Folder->getDocmanFolderId());
		}
		return $domObj;
	}
	
	private function convertSCM($domPlugins) {
		
		$domSCM = $this->dom->createElement('SCM');
		$this->convertRoleSetting($domSCM, 'scm', 0);
		$domPlugins->appendChild($domSCM);
	}
	
	private function convertRoleSetting($domObj, $section, $ref_id) {
			
		//Export Role Settings
		$crit = new Criteria();
		$crit->add(RolePeer::PROJECT_ID, $this->template->getProjectId());
		$crit->addAscendingOrderByColumn(RolePeer::ROLE_NAME);
		$roles = RolePeer::doSelect($crit);

		foreach($roles as $role) {
			$c = new Criteria();
			$c->add(RoleSettingPeer::ROLE_ID,$role->getRoleID());
			$c->add(RoleSettingPeer::SECTION, $section);
			$c->add(RoleSettingPeer::REF_ID, $ref_id);
			$rolesettings = RoleSettingPeer::doSelect($c);

			foreach ($rolesettings as $rolesetting) {
				$domRoleSetting = $this->dom->createElement('RoleSetting');
				$domRoleSetting->setAttribute('role', $role->getName());
				$domRoleSetting->setAttribute('value', $rolesetting->getValue());
				$domObj->appendChild($domRoleSetting);
			}
		}
	}
	
	
	public function save($output=null) {
		if ($output==null) {
			header('Content-type: text/xml');
			header('Content-disposition: attachment; filename="'.$this->template->getUnixName().'.xml"');
			echo $this->dom->saveXML();
		} else {
			$this->dom->save($output);
		}
	}
}
?>