<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2007-01-28
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'core/om/BaseAssociation.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");
require_once("lib/core/WallPost.php");

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class Association extends BaseAssociation implements GFPermission {
	
	private $invalid_data = array();

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		if ($this->userIsAdmin()) {
			return true;
		}
		/*
		//requires manual customization
		$perm = $this->getProject()->getPermission();
		if ($perm->check(self::SECTION, SELF::CREATE)) {
			return true;
		}
		*/
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		return true;
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		return true;
	}

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see Association::setDeleted()
	 * @see Association::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete Association object: permission denied");
		}
		ActionManager::notify("ASSOCIATION_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create Association object: permission denied");
		} else if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update Association object: permission denied");
		}
		
		// Check for valid data
		$this->invalid_data = array();
		
		if (empty($this->section)) {
			$this->invalid_data['section'] = Language::getSessText('Association.EmptySection');
		}
		if (empty($this->to_section)) {
			$this->invalid_data['to_section'] = Language::getSessText('Association.EmptyToSection');
		}
		if (empty($this->ref_id) || !is_numeric($this->ref_id) || $this->ref_id <= 0) {
			$this->invalid_data['ref_id'] = Language::getSessText('Association.EmptyRefId');
		}
		if (empty($this->to_ref_id) || !is_numeric($this->to_ref_id) || $this->to_ref_id <= 0) {
			$this->invalid_data['to_ref_id'] = Language::getSessText('Association.EmptyToRefId');
		}
	
		// Check for duplicates
		if (empty($this->invalid_data)) {
			$c = new Criteria();
			$c->add(AssociationPeer::SECTION, $this->section);
			$c->add(AssociationPeer::TO_SECTION, $this->to_section);
			$c->add(AssociationPeer::REF_ID, $this->ref_id);
			$c->add(AssociationPeer::TO_REF_ID, $this->to_ref_id);
			
			$cnt = AssociationPeer::doCount($c);
			if ($cnt > 0) {
				throw new InvalidDataException(array(Language::getSessText('Association.Duplicated')));
			}
		} else {
			throw new InvalidDataException($this->invalid_data);
		}
		
		$this->checkPermissions();
		
		ActionManager::notify("ASSOCIATION_SAVE_BEFORE", $this);
		parent::save($con);
		ActionManager::notify("ASSOCIATION_SAVE", $this);
	}
	
	public function checkPermissions() {
		ActionManager::notify('ASSOCIATION_CHECK',$this);
	}

	/**
	 * Dispatches association link rendering for objets that do not belong to a plugin
	 *
	 * @param array $params
	 * @return String $link
	 */
	public function renderNonPluginAssociationLink($params) {
		$link = "";
 
		switch($params[0]) {
			case WallPost::SECTION:
				$link = WallPost::renderAssociationLink($params);
			break;
		}

		return $link;		
	}

}
?>
