<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'core/om/BaseFilesystem.php';
require_once("plugin/ActionManager.php");
require_once('FilesystemDownload.php');
require_once 'GFConf.php';
require_once 'StringUtils.php';

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class Filesystem extends BaseFilesystem {

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null, $notify_save=true) {

		if (!$this->getFilesystemId()) {
			$md5 = '';
		} else {
			if (!file_exists($this->getFilePath())) {
				echo "\n".'File Does Not Exist: '.$this->getFilePath();
				return;
			}
			$md5 = md5_file($this->getFilePath());
			$this->setMd5Hash($md5);
		}

		ActionManager::notify("FILESYSTEM_SAVE_BEFORE", $this);
		parent::save($con);
        if ($notify_save) {
            //Only TrackerPlugin uses this, and we set it to false
            //When we are indexing files from the cronjob, to avoid notyfing users
            //about really old tickets. If you need this on a different plugin, please
            //handle it with care.
            ActionManager::notify("FILESYSTEM_SAVE", $this);
        }

	}

	/**
	 * Returns the contents of the file
	 *
	 * @return string	contents of the file, false on error or empty file
	 *
	 */
	
	public function getFileData($increment=true) {
		if ($increment) {
			$this->incrementDownloadCount();
		}
		return file_get_contents($this->getFilePath());
	}
	
	/**
	 * Dumps the contents of the file to stdout (instead of storing them in a variable)
	 */
	public function dumpFileData($increment=true) {
		if ($increment) {
			$this->incrementDownloadCount();
		}
		$filehandle = fopen($this->getFilePath(), 'rb');
		if ($filehandle === false) {
		return false;
		}
		while (!feof($filehandle)) {
			$chunk = fread($filehandle, 1048576);
			echo $chunk;
			ob_flush();
			flush();
		}

		//readfile($this->getFilePath());
	}

	public function incrementDownloadCount() {
		$fsd = new FilesystemDownload();
		$fsd->setFilesystem($this);
		if (Session::getSession()->isLoggedIn()) {
			$user = Session::getSession()->getUser()->getUserId();
		} else {
			$user = 100;
		}
		$fsd->setUserId($user);
		$fsd->setIpAddress($_SERVER["REMOTE_ADDR"]);
		$fsd->save();
	}
	
	/**
	 * Function used to test whether the file is valid or not (perhaps something happend in filesystem and file got deleted)
	 *
	 */
	public function dataValid() {
		$size = filesize($this->getFilePath());
		if ($size === false) {
			// Bypasses a bug that fails when file size > 2gb
			$size = exec("stat -c%s \"" . escapeshellcmd($this->getFilePath()) ."\"", $out, $ret);
			if ($ret) {
				$size = 0;
			}
		}
		if (file_exists($this->getFilePath()) && $size > 0) {
			return true;
		} 
		return false;
	}

	public function setFileName($string) {
		$old_name = $this->getFileNameSafe();
		parent::setFileName($string);
		$this->setFileNameSafe($old_name);
	}

	public function setFileNameSafe($old_name) {
		if (strlen($this->getFileName()) < 3) {
			throw new Exception('file name must be at least 3 characters');
		}
		parent::setFileNameSafe(StringUtils::removeInvalidChars($this->getFileName(), false));
		
		if (!$this->isNew()) {
			// change the name of the file in the FS, for update only
			rename($this->getDirPath().'/'.$old_name,$this->getDirPath().'/'.$this->getFileNameSafe());
		}
	}

	/**
	 * Deletes the file and directory from the filesystem and the record from the DB
	 *
	 * @param Connection $con
	 */
	public function delete($con = null) {
		unlink($this->getFilePath()); // delete the file
		rmdir($this->getDirPath()); // delete the directory
		ActionManager::notify("FILESYSTEM_DELETE", $this);
		parent::delete();
	}

	public function getFilePath() {
		if (strlen($this->getFileNameSafe()) < 3) {
			throw new Exception('Unix name must be at least 3 characters');
		}
		return $this->getDirPath().'/'.$this->getFileNameSafe();
	}

	private function getDirPath() {
		if (!$this->getFilesystemId()) {
			throw new Exception('Must Save Before Getting Directory');
		}
		if (strlen($this->getSection()) < 3) {
			throw new Exception('Section must be at least 3 characters');
		}
		$filepath=GFConf::get('system.filesPath').'/'.$this->getSection();
		$id = str_pad($this->getFilesystemId(), 3, '0', STR_PAD_LEFT);
		$separated=$id[0].'/'.$id[1].'/'.$id[2];
		return $filepath.'/'.$separated.'/'.$this->getFilesystemId();
	}

	public function getRelativeDirPath() {
		$filepath='/'.$this->getSection();
		$id = str_pad($this->getFilesystemId(), 3, '0', STR_PAD_LEFT);
		$separated=$id[0].'/'.$id[1].'/'.$id[2];
		return $filepath.'/'.$separated.'/'.$this->getFilesystemId();
	}

	private function mkdir($setOwnerApache=false) {

		$apacheUser = GFConf::get('system.apacheUser');

		if (!$this->getFilesystemId()) {
			throw new Exception('Must Save Before Setting File Data');
		}
		if (strlen($this->getSection()) < 3) {
			throw new Exception('Section must be at least 3 characters');
		}
		$filepath=GFConf::get('system.filesPath').'/'.$this->getSection();
		if (!is_dir($filepath)) {
			mkdir ($filepath);
			if ($setOwnerApache) {
				chown($filepath, $apacheUser);
			}

		}
		$id = str_pad($this->getFilesystemId(), 3, '0', STR_PAD_LEFT);
		$path = $filepath.DIRECTORY_SEPARATOR.$id[0];
		if (!is_dir($path)) {
			mkdir ($path);
			if ($setOwnerApache) {
				chown($path, $apacheUser);
			}
		}
		$path = $filepath.DIRECTORY_SEPARATOR.$id[0].DIRECTORY_SEPARATOR.$id[1];
		if (!is_dir($path)) {
			mkdir ($path);
			if ($setOwnerApache) {
				chown($path, $apacheUser);
			}
		}

		$path = $filepath.DIRECTORY_SEPARATOR.$id[0].DIRECTORY_SEPARATOR.$id[1].DIRECTORY_SEPARATOR.$id[2];
		if (!is_dir($path)) {
			mkdir ($path);
			if ($setOwnerApache) {
				chown($path, $apacheUser);
			}
		}

		$path = $filepath.DIRECTORY_SEPARATOR.$id[0].DIRECTORY_SEPARATOR.$id[1].DIRECTORY_SEPARATOR.$id[2].
			DIRECTORY_SEPARATOR.$this->getFilesystemId();
		if (!is_dir($path)) {
			mkdir ($path);
			if ($setOwnerApache) {
				chown($path, $apacheUser);
			}
		}
	}

	public function setFileDataFromString($data) {
		$this->mkdir();
		if (strlen($data)==0) {
			$data = " "; // file_put_contents fails if you give an empty string
		}
		if (! file_put_contents($this->getFilePath(),$data)) {
			throw new Exception('Cannot put content in '.$this->getFilePath());
		}
		$this->setFileSize( filesize($this->getFilePath()) ); // update the file size
		if ($this->getFileType() == 'text' || $this->getFileType() == 'text/plain' || $this->getFileType() == 'text/html') {
			$this->setStrings($data);
		} else {
			$this->setStrings('');
		}
		$this->save();
	}

	/**
	 * Loads the file with the input passed as parameter
	 *
	 * @param string	the path to the file or the contents
	 * @param boolean	Whether to handle the file as an uploaded file (performs extra security checks)
	 *
	 */	
	
	public function setFileDataFromPath($path, $isUploaded=true, $setOwnerApache=false, $sendNotify=true) {
		$this->mkdir($setOwnerApache);

		if ($isUploaded) {
			if (!is_uploaded_file($path)) {
				throw new Exception("PHP says $path is not an uploaded file");
			}
			if (!move_uploaded_file($path, $this->getFilePath()) ) {
				throw new Exception("Cannot move uploaded file $path to ".$this->getFilePath());
			}
		} else {
			if (!copy($path, $this->getFilePath())) {
				throw new Exception("Cannot copy file $path to ".$this->getFilePath());
			}
		}

		try {
			$this->setFileSize(filesize($this->getFilePath())); // update the file size
			if ($this->getFileType() == 'text' || $this->getFileType() == 'text/plain' || $this->getFileType() == 'text/html') {
				$strings = '';
				if ( file_exists($this->getFilePath()) ) {
						$strings = file_get_contents($this->getFilePath());
				}
				$this->setStrings( strip_tags($strings) );				// Note that $this->setStrings() automatically converts the strings to utf8
			} else {
				$this->setStrings('');
			}
		} catch(Exception $e) {
		}
		$this->save(null, $sendNotify);

	}
	
	/**
	 * Sets the file data from an uploaded file
	 * @param	array	Must come from $_FILES
	 */
	public function setFileDataFromUploadedFile($file) {
		$this->setFileSize($file['size']);
		$this->setFileType(SystemUtils::getMimeType($file['name']));
		$this->setFileName($file['name']);
		$this->save();
		$this->setFileDataFromPath($file['tmp_name'], true);
	}

	/**
	 * Sets the file data from a local file.
	 * @param 	string 	Path to the file in the local filesystem.
	 * @param boolean $sendNotify send notification to trackerplugin
	 */
	public function setFileDataFromLocalFile($filePath, $sendNotify = true) {
		if (!is_file($filePath)) {
			throw new Exception("Local file $filePath does not exist");
		}

		$filename = basename($filePath);
		$filesize = filesize($filePath);
		$filetype = SystemUtils::getMimeType($filename);

		$this->setFileSize($filesize);
		$this->setFileType($filetype);
		$this->setFileName($filename);
		$this->save(null, $sendNotify);
		$this->setFileDataFromPath($filePath, false, false, $sendNotify);
	}

	/**
	 * Copies the file from the ftp directory
	 *
	 * @param string	the path to the file or the contents
	 * @param boolean	is_string. if true, then the contents is the parameter. else, it�s a path to a file to load
	 *
	 */	
	
	public function setFileDataFromFTP($path) {
		$this->mkdir();
		if (!file_exists($path)) {
			throw new Exception('File doesn\'t exist!');
		}
		if (!rename($path, $this->getFilePath())) {
			throw new Exception('Cannot copy file '.$this->getFilePath());
		}
		$this->setFileSize( filesize($this->getFilePath()) ); // update the file size
		if (($this->getFileType() == 'text' || $this->getFileType() == 'text/plain' || $this->getFileType() == 'text/html') && (filesize($this->getFilePath()) < 1024000 )) {
			$strings = file_get_contents($this->getFilePath());
			$this->setStrings($strings);				// Note that $this->setStrings() automatically converts the strings to utf8
		} else {
			$this->setStrings('');
		}
		$this->save();
	}
	
	public function setStrings($strings) {
		// The strings are used for indexing. Pre-process the strings so they will be compatible with tsearch2
		// (tsearch2 can't handle > 1MB and words longer than certain length)
		if ($strings == '-1') {
			parent::setStrings($strings);
			return;
		}
		$str_limit = 1000000;		// Should be 1MB -1 but this is safer...
		$word_limit = 100;
		$strings = substr($strings, 0, $str_limit);
		$words = explode(' ', $strings);
		$words_filtered = array();
		for ($i = 0; $i < count($words); $i++) {
			if (empty($words[$i])) {
				continue;
			}

			if (!StringUtils::isUTF8Compliant($words[$i])) {
					$words[$i] = utf8_encode($words[$i]);
			}
	
			if (StringUtils::isUTF8Compliant($words[$i])) {
					$words_filtered[] = substr($words[$i], 0, $word_limit);
			}
		}
		$strings = "";
		for ($i=0; $i < sizeof($words_filtered); $i++) {
			if (StringUtils::isUTF8Compliant($strings . " " . $words_filtered[$i])) {
				if (!is_numeric($words_filtered[$i])) {
					$strings .= " " . $words_filtered[$i];
				}
			}
		}
		$strings = trim($strings);
		//Make sure the string size stays below the limit - truncate)
		$strings = substr($strings, 0, $str_limit-1);
		$strings = trim($strings);

		if (StringUtils::isUTF8Compliant($strings)) {
			parent::setStrings($strings);
		} else {
			parent::setStrings('-1');
		}

	}

	/**
	 * Returns the upload date for this file. Since currently we're not storing the date on the database, check the file's mtime.
     * In the future, we will store this on the DB, but some files won't have their upload time set to we will have to resort to
     * this.
     * @return	string
	 */
	public function getUploadDate() {
		$file = $this->getFilePath();
		if (!is_file($file)) {
			return '';
		}
		$mtime = filemtime($file);
		return date('Y-m-d H:i:s', $mtime);
	}
	
	
	public function canOpenInBrowser() {
		$arr_patterns = array('/text\/plain/i', '/image/i', '/text\/html/i');
		$isMatch = false;
		foreach ($arr_patterns as $str_pattern) {
				if (preg_match($str_pattern, $this->getFileType())) {
						$isMatch = true;
						break;
				}
		}
		return $isMatch;
	}

}
?>
