<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'core/om/BaseMonitor.php';
require_once("plugin/ActionManager.php");

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class Monitor extends BaseMonitor {
	
	const SITE_SECTION = 'site';
	
	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see Monitor::setDeleted()
	 * @see Monitor::isDeleted()
	 */
	function delete($con = null) {
		ActionManager::notify("MONITOR_DELETE", $this);
		parent::delete($con);
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		$lang = Session::getSession()->getLanguage();
		
		// Invoke an action to see if we're allowed to monitor this item
		$params = array('section' => $this->getSection(), 'ref_id' => $this->getRefId());
		$prs = ActionManager::notify('MONITOR_CHECK_PERMISSIONS', $params);
		
		// Allow monitoring only if at least one plugin says monitoring is allowed
		// for this item
		if ($prs->count() == 0) {
			// if there wasn't any reply, it means the monitor isn't handled by anybody.
			// some monitors are not inside plugins and must be handled here
			if (strlen($this->getSection()) == 0 || $this->getSection()=='site') {
				$allow = true;
			} else {
				$allow = false;		
			}
		} else {
			$allow = false;
			foreach ($prs as $pluginresult) {
				$can_monitor = $pluginresult->getResult();
				if ($can_monitor) {
					$allow = true;		// found one that allowed us to monitor the item
					break;
				}
			}
		}
		
		if (!$allow) {
			throw new PermissionDeniedException($lang->getText('Monitor.NotAllowed', array($this->getSection(), $this->getRefId())));
		}

		// if we get to this point, it means we can safely monitor this item
		parent::save($con);
		ActionManager::notify("MONITOR_SAVE", $this);
	}

}
?>
