<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2008-07-16
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'core/om/BaseMyReport.php';
require_once("plugin/ActionManager.php");
require_once 'core/MyReportFieldFilterPeer.php';
require_once 'core/MyReportFieldValuePeer.php';
require_once ('core/MyReportUserPeer.php');

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class MyReport extends BaseMyReport {
	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see MyReport::setDeleted()
	 * @see MyReport::isDeleted()
	 */
	function delete($con = null) {
		ActionManager::notify("MY_REPORT_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		parent::save($con);
		ActionManager::notify("MY_REPORT_SAVE", $this);
	}


	const FIELDTYPE_BASEVALUE = 100;	// Report field types start from this value onward
	const FIELDTYPE_SUMMARY = 101;
	const FIELDTYPE_DETAILS = 102;
	const FIELDTYPE_ASSIGNEES = 103;
	const FIELDTYPE_PRIORITY = 104;
	const FIELDTYPE_OPENDATE = 105;
	const FIELDTYPE_CLOSEDATE = 106;
	const FIELDTYPE_LASTMODIFIEDDATE = 107;
	const FIELDTYPE_TRACKERNAME = 108;
	const FIELDTYPE_PROJECTNAME = 109;

	const REPORTTYPE_TRACKER = 0;		// This report type is for trackers.

	/* Filter Tracker Items according to the spec given in the report */
	function getTrackerItemIds() {
		$my_report_id = $this->getMyReportId();
		
		
		
		// Build a list of field_names and field_values from the report spec.
		$crit = new Criteria();
		$crit->add(MyReportFieldFilterPeer::MY_REPORT_ID, $my_report_id );
		$fields = MyReportFieldFilterPeer::doSelect($crit);
		
		// Count how many values are defined in total	
		$crit = new Criteria();
		//$crit->addJoin(MyReportFieldValuePeer::MY_REPORT_FIELD_FILTER_ID, MyReportFieldFilterPeer::MY_REPORT_FIELD_FILTER_ID);
		$crit->add(MyReportFieldFilterPeer::MY_REPORT_ID, $my_report_id);
		$crit->add(MyReportFieldFilterPeer::TYPE, MyReport::FIELDTYPE_BASEVALUE, Criteria::LESS_THAN);
		$value_count = MyReportFieldFilterPeer::doCount($crit);
		
		$dateRestrictions ='';

		if($this->getOpenDateStart(null) > 50000) {
			$dateRestrictions .= "tracker_item.open_date >= '" . $this->getOpenDateStart('M d Y') . "' AND ";
		}
		
		if($this->getOpenDateEnd(null) > 50000) {
			$dateRestrictions .= "tracker_item.open_date <= '" . $this->getOpenDateEnd('M d Y') . "' AND ";
		}
		
		if($this->getCloseDateStart(null) > 50000) {
			$dateRestrictions .= "tracker_item.close_date >= '" . $this->getCloseDateStart('M d Y') . "' AND ";
		}
		
		if($this->getCloseDateEnd(null) > 50000) {
			$dateRestrictions .= "tracker_item.close_date <= '" . $this->getCloseDateEnd('M d Y') . "' AND ";
		}
		
		$sql = "SELECT DISTINCT (tracker_item_id), 
				COUNT(tracker_item_id) AS cnt 
				FROM tracker_extra_field_data NATURAL JOIN tracker_extra_field NATURAL JOIN tracker_item
				WHERE $dateRestrictions
				tracker_extra_field.tracker_id IN (SELECT tracker_id FROM my_report_tracker WHERE my_report_id = $my_report_id)				
				AND ";
		
		$sqlExtras = '';
		$sql_glue = ' OR ';
		

		// Add the conditions imposed by the filters
		
		
		foreach($fields as $field) {
			// Retrieve the field values
			$crit = new Criteria();
			$crit->add(MyReportFieldValuePeer::MY_REPORT_FIELD_FILTER_ID, $field->getMyReportFieldFilterId());			
			
			
			switch($field->getType())
			{
				case TrackerExtraField::TYPE_MEMBERS:
			
					$sql_aux = "tracker_extra_field.field_type=" . $field->getType() . " AND
					tracker_extra_field.field_name='" . $field->getFieldName() . "' AND
					tracker_extra_field_data.field_data IN 
						(SELECT  \"user\".\"user_id\" FROM \"user\" 
						WHERE 
						\"user\".\"user_id\" IN (SELECT field_value FROM my_report_field_value WHERE my_report_field_filter_id=" . $field->getMyReportFieldFilterId() . ")
						)";
					$sqlExtras .= $sql_aux . $sql_glue;
					break;
				case TrackerExtraField::TYPE_SELECT:
				case TrackerExtraField::TYPE_CHECKBOX:
				case TrackerExtraField::TYPE_RADIO:
				case TrackerExtraField::TYPE_MULTISELECT:
				case TrackerExtraField::TYPE_STATUS:
					$sql_aux = "tracker_extra_field.field_type=" . $field->getType() . " AND
								tracker_extra_field.field_name='" . $field->getFieldName() . "' AND
								tracker_extra_field_data.field_data IN 
									(SELECT element_id FROM tracker_extra_field_element WHERE element_name IN 
										(SELECT field_value FROM my_report_field_value WHERE my_report_field_filter_id=" . $field->getMyReportFieldFilterId() . ")
										AND tracker_extra_field_element.tracker_extra_field_id = tracker_extra_field.tracker_extra_field_id
										)";
					$sqlExtras .= $sql_aux . $sql_glue;
					break;
				
				case TrackerExtraField::TYPE_TEXT:
				case TrackerExtraField::TYPE_TEXTAREA:
					$field_value = MyReportFieldValuePeer::doSelectOne($crit);
					$field_value = $field_value->getFieldValue();					
					$sql_aux = "tracker_extra_field.field_type=" . $field->getType() . " AND
								tracker_extra_field.field_name='" . $field->getFieldName() . "' AND
								tracker_extra_field_data.field_data ILIKE '%$field_value%'";
					$sqlExtras .= $sql_aux . $sql_glue;
					break;
					
				case TrackerExtraField::TYPE_RELEASE:
					$sql_aux = "tracker_extra_field.field_type=" . $field->getType() . " AND
								tracker_extra_field.field_name='" . $field->getFieldName() . "' AND
								tracker_extra_field_data.field_data IN 
									(SELECT frs_release.frs_release_id FROM frs_release NATURAL JOIN frs_package WHERE frs_package.package_name || '::' || frs_release.release_name IN 
										(SELECT field_value FROM my_report_field_value WHERE my_report_field_filter_id=" . $field->getMyReportFieldFilterId() . ")
										AND frs_package.project_id IN (SELECT project_id FROM my_report_project WHERE my_report_id=$my_report_id)
										)";
					$sqlExtras .= $sql_aux . $sql_glue;
					break;
					
				case TrackerExtraField::TYPE_DATE:
					$field_value = MyReportFieldValuePeer::doSelectOne($crit);
					$range_start = strftime('%Y-%m-%d',strtotime($field_value->getFieldRangeStart()));
					$range_end = strftime('%Y-%m-%d',strtotime($field_value->getFieldRangeEnd()));
					$sql_aux = "tracker_extra_field.field_type=" . $field->getType() ." AND
								tracker_extra_field.field_name='" . $field->getFieldName() . "' AND 
								tracker_extra_field_data.field_data != '' AND
								tracker_extra_field_data.field_data::date BETWEEN '$range_start'::date AND '$range_end'::date";
					$sqlExtras .= $sql_aux . $sql_glue;
					break;

		 		case TrackerExtraField::TYPE_SERIAL:
					$field_value = MyReportFieldValuePeer::doSelectOne($crit);
					$range_start = $field_value->getFieldRangeStart();
					$range_end = $field_value->getFieldRangeEnd();
					$sql_aux = "tracker_extra_field.field_type=" . $field->getType() . " AND
								tracker_extra_field.field_name='" . $field->getFieldName() . "' AND 
								tracker_extra_field_data.field_data != '' AND 
								CAST (tracker_extra_field_data.field_data AS bigint) BETWEEN $range_start AND $range_end";

		 		case TrackerExtraField::TYPE_NUMERIC:
					$field_value = MyReportFieldValuePeer::doSelectOne($crit);
					$range_start = $field_value->getFieldRangeStart();
					$range_end = $field_value->getFieldRangeEnd();
					$sql_aux = "tracker_extra_field.field_type=" . $field->getType() . " AND
								tracker_extra_field.field_name='" . $field->getFieldName() . "' AND 
								tracker_extra_field_data.field_data != '' AND 
								CAST (tracker_extra_field_data.field_data AS float) BETWEEN $range_start AND $range_end";
					$sqlExtras .= $sql_aux . $sql_glue;
					break;
				case MyReport::FIELDTYPE_SUMMARY:
					$field_value = MyReportFieldValuePeer::doSelectOne($crit);
					$field_value = $field_value->getFieldValue();
					$sql .= "tracker_item.summary ILIKE '%$field_value%' AND ";
					break;
					
				case MyReport::FIELDTYPE_DETAILS:
					$field_value = MyReportFieldValuePeer::doSelectOne($crit);
					$field_value = $field_value->getFieldValue();
					$sql .= "tracker_item.details ILIKE '%$field_value%' AND ";
					break;
					
				case MyReport::FIELDTYPE_PRIORITY:					
					$sql .= "tracker_item.priority IN 
					(SELECT field_value FROM my_report_field_value WHERE my_report_field_filter_id=" . $field->getMyReportFieldFilterId() . ") AND ";
					break;
					
				case MyReport::FIELDTYPE_ASSIGNEES:
					$sql .= "tracker_item.tracker_item_id IN 
									(SELECT tracker_item_id FROM tracker_item_assignee WHERE assignee IN 
										(SELECT field_value FROM my_report_field_value WHERE my_report_field_filter_id=" . $field->getMyReportFieldFilterId() . ")) AND ";
					break;
					
				default:
					break;
					
			}
			
			
		}
		
		if (strlen($sqlExtras)>strlen($sql_glue)) {
			$sql .= "($sqlExtras";	
			$sql =  substr($sql,0,strlen($sql) - strlen($sql_glue)) . ") ";
		}
		else {
			// Remove the trailing AND
			$sql = substr($sql,0,strlen($sql) - strlen(' AND '));
		}
		
		
		$sql .= " GROUP BY tracker_item_id HAVING (COUNT(tracker_item_id)>=$value_count)";

		$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		$tracker_item_ids = array();
		while ($rs->next()) {
			$tracker_item_ids[] = $rs->getInt('tracker_item_id');
		}
		return $tracker_item_ids;
	}
	
	/* Get the data that's to be displayed according to whats stated in the report */
	
	function getDisplayData($tracker_item_ids=null) {
		if($tracker_item_ids==null) {
			$tracker_item_ids = $this->getTrackerItemIds();
		}
		$my_report_id = $this->getMyReportId();
		$tracker_item_id_list = implode(',', $tracker_item_ids);
		$data = array();
		
		$priorities = Tracker::getPriorities();
		$sort_id_arr = array();
		
		// Get the sort column
		$crit = new Criteria();
		$crit->add(MyReportFieldDisplayPeer::MY_REPORT_ID, $this->getMyReportId());
		$crit->addDescendingOrderByColumn(MyReportFieldDisplayPeer::ORDER_PRIORITY);
		$crit->addAscendingOrderByColumn(MyReportFieldDisplayPeer::COLUMN_ORDER);	// Just in case there no ordering made, use the left most column
		$sortCol = MyReportFieldDisplayPeer::doSelectOne($crit);
		if(is_object($sortCol)) {
			$rowSortColumn = $sortCol->getFieldName();
			$rowSortOrder = $sortCol->getOrderAsc();
		}
		else $rowSortColumn = '';
		
		// Build a list of field_names and field_values from the report spec.
		$crit = new Criteria();
		$crit->add(MyReportFieldDisplayPeer::MY_REPORT_ID, $my_report_id );
		$crit->addAscendingOrderByColumn(MyReportFieldDisplayPeer::COLUMN_ORDER);
		$fields = MyReportFieldDisplayPeer::doSelect($crit);
		
		$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		
		foreach($fields as $field) {
			// Retrieve the field values
			switch($field->getType())
			{
				case TrackerExtraField::TYPE_SELECT:
				case TrackerExtraField::TYPE_CHECKBOX:
				case TrackerExtraField::TYPE_RADIO:
				case TrackerExtraField::TYPE_MULTISELECT:
				case TrackerExtraField::TYPE_STATUS:
					$sql = "SELECT tracker_item_id, element_name FROM
					tracker_extra_field_element
					JOIN tracker_extra_field_data ON tracker_extra_field_element.element_id = tracker_extra_field_data.field_data
					JOIN tracker_extra_field ON tracker_extra_field_element.tracker_extra_field_id = tracker_extra_field.tracker_extra_field_id
					WHERE
					field_name = '" . $field->getFieldName() . "'
					AND field_type = " . $field->getType() . "
					AND tracker_item_id IN ($tracker_item_id_list)";
				if($field->getFieldName()==$rowSortColumn) {
					$sql .= ' ORDER BY element_name ' . ($rowSortOrder ? 'ASC':'DESC');
				}

					break;

				case TrackerExtraField::TYPE_MEMBERS:
					$sql = "SELECT tracker_item_id, \"user\".\"firstname\" || ' ' || \"user\".\"lastname\"
					as members FROM 
					tracker_extra_field_data NATURAL JOIN tracker_extra_field
					INNER JOIN \"user\" ON 
					tracker_extra_field_data.field_data = \"user\".\"user_id\"
					WHERE field_name= '" . $field->getFieldName() . "'
					AND field_type = " . $field->getType() . "
					AND tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY members ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;
				case TrackerExtraField::TYPE_RELEASE:					
					$sql = "SELECT tracker_item_id, frs_package.package_name || '::' || frs_release.release_name as release FROM
					tracker_extra_field_data NATURAL JOIN tracker_extra_field
					JOIN frs_release ON tracker_extra_field_data.field_data = frs_release.frs_release_id
					NATURAL JOIN frs_package
					WHERE field_name = '" . $field->getFieldName() . "'
					AND field_type = " . $field->getType() . "
					AND tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY release ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;
					
				case TrackerExtraField::TYPE_TEXT:
				case TrackerExtraField::TYPE_TEXTAREA:	
					$sql = "SELECT tracker_item_id, field_data FROM
					tracker_extra_field_data NATURAL JOIN tracker_extra_field
					WHERE field_name = '" . $field->getFieldName() . "'
					AND field_type = " . $field->getType() . "
					AND tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY field_data ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;
				case TrackerExtraField::TYPE_DATE:
					$sql = "SELECT tracker_item_id, field_data FROM
					tracker_extra_field_data NATURAL JOIN tracker_extra_field
					WHERE field_name = '" . $field->getFieldName() . "'
					AND field_type = " . $field->getType() . "
					AND tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY CAST(field_data AS date) ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;
				case TrackerExtraField::TYPE_NUMERIC:
					$sql = "SELECT tracker_item_id, field_data FROM
					tracker_extra_field_data NATURAL JOIN tracker_extra_field
					WHERE field_name = '" . $field->getFieldName() . "'
					AND field_type = " . $field->getType() . "
					AND tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY CASE WHEN field_data=\'\' THEN NULL ELSE CAST(field_data AS float) END ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;
				case TrackerExtraField::TYPE_SERIAL:
					$sql = "SELECT tracker_item_id, field_data FROM
					tracker_extra_field_data NATURAL JOIN tracker_extra_field
					WHERE field_name = '" . $field->getFieldName() . "'
					AND field_type = " . $field->getType() . "
					AND tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY CAST(field_data AS bigint)' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;

				case MyReport::FIELDTYPE_SUMMARY:
					$sql = "SELECT tracker_item_id, summary FROM tracker_item WHERE tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY summary ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;
					
				case MyReport::FIELDTYPE_DETAILS:
					$sql = "SELECT tracker_item_id, details FROM tracker_item WHERE tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY details ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;
					
				case MyReport::FIELDTYPE_PRIORITY:
					$sql = "SELECT tracker_item_id, priority FROM tracker_item WHERE tracker_item_id IN ($tracker_item_id_list)";					
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY priority ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;

				case MyReport::FIELDTYPE_ASSIGNEES:
					$sql = "SELECT tracker_item_id, \"user\".firstname || ' ' || \"user\".lastname AS full_name FROM tracker_item_assignee LEFT JOIN \"user\" ON \"user\".user_id=tracker_item_assignee.assignee WHERE tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY full_name ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;
					
				case MyReport::FIELDTYPE_OPENDATE:
					$sql = "SELECT tracker_item_id, open_date FROM tracker_item WHERE tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY open_date ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;
				case MyReport::FIELDTYPE_CLOSEDATE:
					$sql = "SELECT tracker_item_id, close_date FROM tracker_item WHERE tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY close_date ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;
				case MyReport::FIELDTYPE_LASTMODIFIEDDATE:
					$sql = "SELECT tracker_item_id, last_modified_date FROM tracker_item WHERE tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY last_modified_date ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;
				case MyReport::FIELDTYPE_TRACKERNAME:
					$sql = "SELECT tracker_item_id, tracker_name FROM tracker_item NATURAL JOIN tracker WHERE tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY tracker_name ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;
				case MyReport::FIELDTYPE_PROJECTNAME:
					$sql = "SELECT tracker_item_id, project_name FROM tracker_item LEFT JOIN tracker ON tracker.tracker_id=tracker_item.tracker_id LEFT JOIN project ON project.project_id=tracker.project_id WHERE tracker_item_id IN ($tracker_item_id_list)";
					if($field->getFieldName()==$rowSortColumn) {
						$sql .= ' ORDER BY project_name ' . ($rowSortOrder ? 'ASC':'DESC');
					}
					break;
			}

			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			while ($rs->next()) {
				$id = $rs->getInt(1);
				if(!isset($data[$id])) {
					$data[$id] = array();
				}
				
				$field_name = $field->getFieldName();
				switch($field->getType()) {
					case MyReport::FIELDTYPE_PRIORITY:										
						$field_data = $priorities[$rs->getInt(2)];
						break;
					case TrackerExtraField::TYPE_DATE:
						$field_data = date('Y-m-d',  strtotime($rs->getString(2)));
						break;
					default:
						$field_data = $rs->getString(2);
						break;
				}
				
				if(!isset($data[$id][$field_name])) {
					$data[$id][$field_name] = array();
				}
				
				$data[$id][$field_name][] = $field_data;
				
				if($field->getFieldName()==$rowSortColumn) {
					$sort_id_arr[] = $id;
				}
			}
		}

		// Sort Data by ID
		$sorted_data = $unsorted_data = array();
		foreach($sort_id_arr as $id) {
			if(array_key_exists($id, $data)) {
				$sorted_data[$id] = $data[$id];
			}
			else {
				$unsorted_data[$id] = $data[$id];
			}
		}

		$unsorted_data = array();
		foreach($data as $data_id => $data_item) {
			if (!isset($sort_id_arr[$data_id])) {
				$unsorted_data[$data_id] = $data_item;
			}
		}

		foreach($unsorted_data as $id => $udata) {
			$sorted_data[$id] = $udata;
		}
		
		
		return $sorted_data;		
	}
	
	/* Check if the user can view the report */
	
	function userCanRead() {
		$user_id = Session::getSession()->getUser()->getUserId();
		if ($this->getUserId()==$user_id) {
			// User is owner, he can view the report
			return true;
		}
		
		$crit = new Criteria();
		$crit->add(MyReportUserPeer::MY_REPORT_ID, $this->getMyReportId());
		$ct1 = $crit->getNewCriterion(MyReportUserPeer::USER_ID, $user_id);
		$ct2 = $crit->getNewCriterion(MyReportUserPeer::USER_ID, MyReportUserPeer::USER_ID .' IN (SELECT usergroup_id FROM usergroup_user WHERE member_user_id=' . $user_id . ')', Criteria::CUSTOM);
		$ct1->addOr($ct2);
		$crit->addOr($ct1);
		$up = MyReportUserPeer::doSelectOne($crit);
		if(is_object($up)) {
			// Report has been shared with user or a group the user belongs to, he can view the report
			return true;
		}
		return false;		
	}
}
?>
