<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
// The parent class
require_once 'core/om/BasePluginPeer.php';
require_once 'lib/core/ProjectPeer.php';
require_once 'plugins/tracker/lib/TrackerItemPeer.php';
require_once 'plugins/docman/lib/DocmanFilePeer.php';
require_once 'plugins/frs/lib/FrsReleasePeer.php';
require_once 'plugins/wiki/lib/WikiPage.php';
require_once 'plugins/forum/lib/ForumMessage.php';
require_once 'plugins/pastebin/lib/Pastebin.php';

// The object class
include_once 'core/Plugin.php';

/** 
 * The skeleton for this class was autogenerated by Propel on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package core 
 */
class PluginPeer extends BasePluginPeer {
	private static $_pkCache = array();

	/**
	 * Retrieve a single object by pkey or NULL if not found.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key
	 * @param	bool $useCache	Whether to use caching. This will increase performance if the same object
	 *							is accessed several times
	 * @param	Connection $con	The connection to use
	 * @return	Plugin
	 */
	public static function retrieveByPk($pk, $useCache = true, $con = null) {
		if ($useCache && array_key_exists($pk, PluginPeer::$_pkCache)) {
			return PluginPeer::$_pkCache[$pk];
		}
		
		$object = parent::retrieveByPk($pk, $con);
		if ($useCache) {
			PluginPeer::$_pkCache[$pk] = $object;
		}
		return $object;
	}
	
	/**
	 * Removes an object from the cache. This is usually done after saving.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key of the object to remove
	 */
	public static function invalidate($pk) {
		if (array_key_exists($pk, PluginPeer::$_pkCache)) {
			unset(PluginPeer::$_pkCache[$pk]);
		}
	}

	// Association methods here - anything that uses section and ref_id between plugins

	/**
	 * Creates display-ready strings for short and full title of
	 * the GForge object for the specified section_re_id.
	 * @param type $name Description
	 * @return array The title and full_title keys contain the constructed names.
	 */

	/**
	 *
	 * @param string $section Identifies the plugin/entity being referenced. Examples include forumthread, trackeritem, docmanfile, etc.
	 * @param int $ref_id Unique identifier for the item.
	 * @return array The title and full_title keys contain the constructed names. The url key contains a link to the item detail (or equivalent) page
	 * @throws Exception  If the specified item does not exist
	 */
	public static function getRelatedItemTitle($section, $ref_id) {

		$item = null;
		try {

			$item = self::getRelatedItem($section, $ref_id);

			if ($item == null) {
				throw new Exception("The specified $section ($ref_id) does not exist.");
			}

			$url = '';
			$title = '';
			$title_full = '';
			if ($section == Project::SECTION) {
				$url = $item->getProjectUrl();
				$title = $title_full = $item->getProjectName();
			} elseif ($section == TrackerItem::SECTION) {
				$url = $item->getTrackerItemUrl();
				$title = '[#' . $item->getTrackerItemId() . '] ' . $item->getSummary();
				$title_full = $title;
			} elseif ($section == DocmanFile::SECTION) {
				$url = $item->getDocmanFolder()->getFolderUrl();
				$title = $item->getLatestVersion()->getFile()->getFilename();
				$title_full = $title;
			} elseif ($section == FrsRelease::SECTION) {
				$url = $item->getFrsReleaseUrl();
				$title = $item->getReleaseName();
				$title_full = $title;
			} else if ($section == WikiPage::SECTION) {
				$url = $item->getWikiPageUrl();
				$title = 'Wiki: [#' . $ref_id . '] ' . $item->getPagename();
				$title_full = $title;
			} else if ($section == Forum::SECTION) {
				$url = $item->getForumUrl();
				$title = $item->getForumName();
				$title_full = $title;
			} else if ($section == ForumThread::SECTION) {
				$url = $item->getForumThreadUrl();
				$title = $item->getThreadName();
                if ($title != $item->getForum()->getForumName()) {
                    $title = $item->getForum()->getForumName() . '/' . $title;
                }
                if ($title == 'Discussion') {
                    $title = $item->getForum()->getForumName();
                }
				$title_full = $title;
			} else if ($section == ForumMessage::SECTION) {
				$url = $item->getForumMessageUrl();
				$title = 'Message #'.$item->getForumMessageId().' in '.$item->getForumThread()->getForum()->getForumName().'/'.$item->getForumThread()->getThreadName();
				$title_full = $title;
			} else if ($section == Mailman::SECTION) {
				$url = $item->getListinfoURL();
				$title = 'List: [#'.$item->getMailmanId(). '] '.$item->getListName();
				$title_full = $title;
			} else if ($section == Pastebin::SECTION) {
				$url = $item->getPasteUrl();
				$title = 'List: [#'.$item->getPastebinId(). '] '.$item->getTitle();
				$title_full = $title;
			}

			$arr_title = array();
			$arr_title['url'] = $url;
			$arr_title['title'] = $title;
			$arr_title['title_full'] = $title_full;
			return $arr_title;

		} catch (Exception $ex) {
			error_log($ex->getMessage());
			error_log($ex->getTraceAsString());
			return null;
		}
	}

	public static function relatedItemExistsFor($section, $ref_id) {
		$item = self::getRelatedItem($section, $ref_id);
		return ($item != null);
	}

	public static function getRelatedItem($section, $ref_id) {
		switch ($section){
			case Project::SECTION:
				$item = ProjectPeer::retrieveByPK($ref_id);
				break;
			case Tracker::SECTION:
				$item = TrackerPeer::retrieveByPK($ref_id);
				break;
			case TrackerItem::SECTION:
				$item = TrackerItemPeer::retrieveByPK($ref_id);
				break;
			case DocmanFile::SECTION:
				$item = DocmanFilePeer::retrieveByPk($ref_id);
				break;
			case FrsRelease::SECTION:
				$item = FRSReleasePeer::retrieveByPk($ref_id);
				break;
			case WikiPage::SECTION:
				$item = WikiPagePeer::retrieveByPk($ref_id);
				break;
			case ForumMessage::SECTION:
				$item = ForumMessagePeer::retrieveByPk($ref_id);
				break;
			case ScmCommit::SECTION:
				$item = ScmCommitPeer::retrieveByPk($ref_id);
				break;
			case News::SECTION:
				$item = NewsPeer::retrieveByPk($ref_id);
				break;
			case UserBlog::SECTION:
				$item = UserBlogPeer::retrieveByPk($ref_id);
				break;
			case Mailman::SECTION:
				$item = MailmanPeer::retrieveByPk($ref_id);
				break;
			case Pastebin::SECTION:
				$item = PastebinPeer::retrieveByPk($ref_id);
				break;
			default:
				error_log("PluginPeer::getRelatedItem($section, $ref_id) is not valid");
                $item = null;
                break;
		}

		return $item;
	}

	public static function getRelatedProject($section, $ref_id) {

		$item = self::getRelatedItem($section, $ref_id);

		if ($item != null) {
			if ($section == Project::SECTION) {
				return $item;
			} elseif ($section == Tracker::SECTION) {
				return $item->getProjectId();
			} elseif ($section == TrackerItem::SECTION) {
				return $item->getTracker()->getProject();
			} elseif ($section == DocmanFile::SECTION) {
				return $item->getDocmanFolder()->getProject();
			} elseif ($section == FrsRelease::SECTION) {
				return $item->getFrsPackage()->getProject();
			} elseif ($section == WikiPage::SECTION) {
				return $item->getRelatedProject();
			} elseif ($section == ForumMessage::SECTION) {
				return $item->getForumThread()->getForum()->getProject();
			} elseif ($section == Mailman::SECTION) {
				return $item->getProjectId();
			} elseif ($section == Pastebin::SECTION) {
				return $item->getPastebinId();
			} else {
				error_log("PluginPeer::getRelatedProject($section, $ref_id) is not valid");
            }
		}

		return null;
	}
}
?>