<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'core/om/BaseRole.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");
require_once('core/UserProjectRolePeer.php');

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class Role extends BaseRole implements GFPermission {
	/** This variable will be used when assigning a role to a newly-created project. The
	    user that creates the project, while not being a site admin, will be able to create
	    a single Role object */
	private $is_new_project = false;
	private $invalid_data = array();

	/**
	 * isPublic - true/false
	 * @return boolean
	 */
	function isPublic() {
		return false;
	}

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanCreate() {
		
		// special case, new project registration and allowed non-siteadmin to do so
		if ($this->is_new_project && (GFConf::get('system.projectRegRestricted') == false)) {
			return true;
		}
		return $this->userIsAdmin();
	}
	
	/**
	 * This function must be called when a user creates a new project and needs to create
	 * an initial role for the project.
	 */
	public function setIsNewProject() {
		$this->is_new_project = true;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		return $this->getProject()->getPermission()->isAdmin();
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		if ($this->userIsAdmin()) {
			return true;
		}
		return false;
	}

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see Role::setDeleted()
	 * @see Role::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete Role object: permission denied");
		}
		
		if ($this->hasUsers()) {
			throw new PermissionDeniedException("Could not delete Role object: users associated to it");
		}

		ActionManager::notify("ROLE_DELETE", $this);
		parent::delete($con);
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create Role object: permission denied");
		} elseif (!$this->isNew() && !$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update Role object: permission denied");
		}
		
		if (empty($this->role_name)) {
			$this->invalid_data['role_name'] = Language::getSessText('ProjectAdmin.Role.NeedName');
		}
		
		if (count($this->invalid_data) > 0) {
			throw new InvalidDataException($this->invalid_data);
		}
		
		
		parent::save($con);
		ActionManager::notify("ROLE_SAVE", $this);
	}
	
	/**
	 * Returns true if role has users associated to it
	 * @return bool
	 */
	function hasUsers() {
		$c = new Criteria();
		$c->add(UserProjectRolePeer::ROLE_ID, $this->getRoleId());
		$n = UserProjectRolePeer::doCount($c);
		return ($n > 0);
	}

	/**
	 * Determines whether this role is a project admin
	 *
	 * @return boolean	true if it�s admin
	 */
	function isProjectAdmin() {
		return false; // TODO -> code this when role system is defined
	}

	function getId() {
		return $this->getRoleId();
	}

	function getName() {
		return $this->getRoleName();
	}
	
	function setRoleName($name) {
		if (empty($name)) {
			$this->invalid_data['role_name'] = Language::getSessText('ProjectAdmin.Role.NeedName');
			return;
		}
		
		parent::setRoleName($name);
		unset($this->invalid_data['role_name']);
	}

	/**
	 * @return null|TrackerExtraField
	 */
	public function getRelatedMemberField() {
		if ($this->getProject()->usesPlugin('tracker')) {
			$c = new Criteria();
			$c->add(RoleSettingPeer::ROLE_ID, $this->getRoleId());
			$c->add(RoleSettingPeer::SECTION, TrackerPlugin::SECTION);
			/** @var RoleSetting[] $roleSetting */
			$roleSetting = RoleSettingPeer::doSelect($c);
			foreach($roleSetting as $setting) {
				if($setting->getRelatedId()) {
					return TrackerExtraFieldPeer::retrieveByPk($setting->getRelatedId());
				}
			}
		}

		return null;
	}

	/**
	 * @param User $user
	 * @return boolean
	 */

	public function matchesMemberFieldRules($user) {
		$extraField = $this->getRelatedMemberField();

		if ($extraField) {
			if ($user->getIsGroup() && $extraField->getAttribute1() == TrackerExtraField::MEMBER_USERS) {
				return false;
			}

			if (!$user->getIsGroup() && $extraField->getAttribute1() == TrackerExtraField::MEMBER_GROUPS) {
				return false;
			}
		}

		return true;
	}

}
?>
