<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-03-05
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'core/om/BaseTroveCategory.php';
require_once("plugin/ActionManager.php");

/** 
 * The skeleton for this class was autogenerated by Propel  
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class TroveCategory extends BaseTroveCategory implements GFWidget  {

	private $invalid_data = array();

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see TroveCategory::setDeleted()
	 * @see TroveCategory::isDeleted()
	 */
	function delete($con = null) {
		$errors = array();
		
        /*//We shouldn't delete root categories
		if ($this->getParentTroveCategoryId() == 0) {
			$errors[] = Language::getSessText('TroveEdit.CantDeleteRoot');
		}*/
		
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}
		
		// If about to delete a category in the templates container subtree
		if ($this->isTemplateProjectTrove()) {
			$con = Propel::getConnection();
			$stmt = $con->createStatement();
			// update projects linking to this category (or descendants) to now link to subtree root
			$sql = "UPDATE trove_link SET trove_category_id = ".GFConf::get('system.troveGfProjTemplates')." WHERE section='project' "
					. "AND trove_category_id IN (SELECT trove_category_id FROM trove_children({$this->getTroveCategoryId()}))";
			$stmt->executeUpdate($sql);
		}
		
		ActionManager::notify("TROVE_CATEGORY_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		$missing_data = array();
		
		if (is_null($this->alias) || empty($this->alias)) {
			$missing_data['alias'] = Language::getSessText('TroveAdd.MustEnterAlias');
		}
		if (is_null($this->category_name) || empty($this->category_name)) {
			$missing_data['category_name'] = Language::getSessText('TroveAdd.MustEnterCategory');
		}
		
		// Note that the invalid data has precedence over the missing data
		// (because invalid fields aren't set)
		$errors = array_merge($missing_data, $this->invalid_data);
		if (count($errors) > 0) {
			throw new InvalidDataException($errors);
		}
		
		parent::save($con);
		ActionManager::notify("TROVE_CATEGORY_SAVE", $this);
	}

	function setAlias($alias) {
		if (empty($alias)) {			
			$this->invalid_data['alias'] = Language::getSessText('TroveAdd.MustEnterAlias');
			return;
		}
		
		// Force to lowercase
		$alias = strtolower($alias);
		
		if (!ereg('^[a-z0-9_\\-]{3,}$', $alias)) {
			$this->invalid_data['alias'] = Language::getSessText('TroveAdd.InvalidAlias');
			return;
		}
		
		//Check if alias already exist... in that case, create a random alias
		$alias_new = $alias;
		$duplicated = false;
		$i = 1;
		do {
			$c = new Criteria();
			$c->add(TroveCategoryPeer::ALIAS, $alias_new, Criteria::EQUAL);
			if (!$this->isNew()) {
				$c->add(TroveCategoryPeer::TROVE_CATEGORY_ID, $this->getTroveCategoryId(), Criteria::NOT_EQUAL);
			}
			$matching_aliases = TroveCategoryPeer::doSelect($c);
			if (count($matching_aliases) > 0) {
				// duplicated alias
				$duplicated = true;
				$i++;
				$alias_new = $alias.$i;		// new try
			} else {
				$duplicated = false;
			}
		} while ($duplicated);

		parent::setAlias($alias_new);
		unset($this->invalid_data['alias']);
	}
		
	function setCategoryName($category_name) {
		if (empty($category_name)) {
			$this->invalid_data['category_name'] = Language::getSessText('TroveAdd.MustEnterCategory');
			return;
		}
		
		parent::setCategoryName($category_name);
		unset($this->invalid_data['category_name']);
	}
	
	
	public function getName() {
		return $this->getCategoryName();
	}
	
	public function getId() {
		return $this->getTroveCategoryId();
	}

	function setDescription($description) {
		if (empty($description)) {
			$this->invalid_data['category_description'] = Language::getSessText('TroveAdd.MustEnterDescription');
			return;
		}
		
		parent::setDescription($description);
		unset($this->invalid_data['category_description']);
	}

	public function setShowInSearch($v) {
			$v = $v == true ? 'true':'false';
 			$con = Propel::getConnection();
			$stmt = $con->createStatement();
			$sql = "SELECT  set_show_in_search({$this->getTroveCategoryId()}, $v)";
			$stmt->executeQuery($sql);
	}

	public function setAccessLevel($v) {
			$v = Validator::sanitizeInt($v);
			$con = Propel::getConnection();
			$stmt = $con->createStatement();
			$sql = "SELECT set_access_level({$this->getTroveCategoryId()}, $v)";
			$stmt->executeQuery($sql);
	}

	public function getAllChildrens() {
		$con = Propel::getConnection();
		$stmt =$con->createStatement();
		//trove_children(trove_id) will return all the childs
		$sql = "SELECT *  FROM trove_children({$this->getTroveCategoryId()})";
		$rs = $stmt->executeQuery($sql,ResultSet::FETCHMODE_NUM);

		return TroveCategoryPeer::populateObjects($rs);
	}
    
    public function move($dest_trove) {
        // Sanitize destination trove (0 is valid for moving to Root Cat)
        $v = Validator::sanitizeInt($dest_trove);
        if (($dest_trove == 0 || $v > 0) && $v != $this->getTroveCategoryId()) {
            $con = Propel::getConnection();
            $stmt = $con->createStatement();
            $sql = "SELECT trove_move({$this->getTroveCategoryId()}, $v)";
            $stmt->executeQuery($sql);
        }
	}
    
    public function merge($dest_trove) {
        // Sanitize destination trove (0 is valid for merging into Root Cat)
        $v = Validator::sanitizeInt($dest_trove);
        if (($dest_trove == 0 || $v > 0) && $v != $this->getTroveCategoryId()) {
            $con = Propel::getConnection();
            $stmt = $con->createStatement();
            $sql = "SELECT trove_merge({$this->getTroveCategoryId()}, $v)";
            $stmt->executeQuery($sql);
        }
	}
    
    public function isFixed() {
        return in_array($this->getTroveCategoryId(), TroveCategoryPeer::getFixedCategoriesIds());
    }
    
    public function isEnabled() {
        if ($this->isFixed()) {
            $fixed_cats = array_flip(TroveCategoryPeer::getFixedCategoriesIds());
            return GFConf::get($fixed_cats[$this->getTroveCategoryId()].'_enabled', TRUE);
        } else {
			// Not applicable
            return NULL;
        }        
    }
    
    /**
     * Toggled setting that indicates whether this category is enabled or not.
     * Returns TRUE if changed to enabled, FALSE if changed to disabled, NULL if value cannot be toggled.
     * 
     * @return boolean|null
     */
    public function toggleEnabled() {
		if ($this->getTroveCategoryId() != GFConf::get('system.troveGfProjTemplates')) {
			$fixed_cats = array_flip(TroveCategoryPeer::getFixedCategoriesIds());
			$conf_key = $fixed_cats[$this->getTroveCategoryId()].'_enabled';
			if ($this->isEnabled()) {
				GFConf::set($conf_key, FALSE);
				return FALSE;
			} elseif ($this->isFixed()) {
				GFConf::set($conf_key, TRUE);
				return TRUE;
			}
		}
        return NULL;
    }
	
	/**
	 * Whether this category is a template project container
	 * 
	 * @return boolean
	 */
	public function isTemplateProjectTrove() {
		$con = Propel::getConnection();
		$stmt = $con->createStatement();
		$sql = "SELECT 1 FROM trove_category WHERE trove_category_id={$this->getTroveCategoryId()} "
		. "AND trove_category_id IN (SELECT trove_category_id FROM trove_children(".GFConf::get('system.troveGfProjTemplates')."))";
		$rs = $stmt->executeQuery($sql);
		return $rs->getRecordCount() == 1;
	}
	
	/**
	 * Whether this category has no child in the trove tree
	 * 
	 * @return boolean
	 */
	public function isLeaf() {
		return !in_array($this->getTroveCategoryId(), TroveCategoryPeer::getNonLeafTroveCategoriesIdsAsArray());
	}
}
?>