<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-03-05
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
// The parent class
require_once 'core/om/BaseTroveCategoryPeer.php';

// The object class
include_once 'core/TroveCategory.php';

require_once('TroveLinkPeer.php');

/** 
 * The skeleton for this class was autogenerated by Propel  
 *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package core 
 */
class TroveCategoryPeer extends BaseTroveCategoryPeer {
	private static $_pkCache = array();
    
    // Fixed categories ids cache
    private static $_fixed_categories_ids = array();
	
	// Non leaf categories ids cache
	private static $_non_leaf_categories_ids = array();

	/**
	 * Retrieve a single object by pkey or NULL if not found.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key
	 * @param	bool $useCache	Whether to use caching. This will increase performance if the same object
	 *							is accessed several times
	 * @param	Connection $con	The connection to use
	 * @return	TroveCategory
	 */
	public static function retrieveByPk($pk, $useCache = true, $con = null) {
		if ($useCache && array_key_exists($pk, TroveCategoryPeer::$_pkCache)) {
			return TroveCategoryPeer::$_pkCache[$pk];
		}
		
		$object = parent::retrieveByPk($pk, $con);
		if ($useCache) {
			TroveCategoryPeer::$_pkCache[$pk] = $object;
		}
		return $object;
	}
	
	/**
	 * Removes an object from the cache. This is usually done after saving.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key of the object to remove
	 */
	public static function invalidate($pk) {
		if (array_key_exists($pk, TroveCategoryPeer::$_pkCache)) {
			unset(TroveCategoryPeer::$_pkCache[$pk]);
		}
	}
	
	/**
	 * Returns an array with the names of the root categories, as described in the
	 * language file
	 */
	public static function getRootCategoriesNames() {
		$lang = Language::getSessLang();

		$ret = array();
		$ret[GFConf::get('system.troveDevStatus')] = $lang->getText('Trove.DevStatus');
		$ret[GFConf::get('system.troveLicense')] = $lang->getText('Trove.License');
		$ret[GFConf::get('system.troveOS')] = $lang->getText('Trove.OS');
		$ret[GFConf::get('system.troveSpokenLanguage')] = $lang->getText('Trove.SpokenLanguage');
		$ret[GFConf::get('system.troveProgrammingLanguage')] = $lang->getText('Trove.ProgrammingLanguage');
		$ret[GFConf::get('system.troveTopic')] = $lang->getText('Trove.Topic');
		$ret[GFConf::get('system.troveIntendedAudience')] = $lang->getText('Trove.IntendedAudience');
		
		$extra_troves = GFConf::get('system.extraTroveRootCategories');	
		$c = new Criteria();
		$c->add(TroveCategoryPeer::TROVE_CATEGORY_ID, $extra_troves, Criteria::IN);
		$categories = TroveCategoryPeer::doSelect($c);
		
		foreach ($categories as $category) {
			$ret[$category->getTroveCategoryId()] = $category->getCategoryName(); 
		}
		
		return $ret;
	}
	
    /**
     * Returns an array with the ids of every category configured as fixed. 
     * Fixed categories have special business rules that are different from those for the rest of the categories.
     * I.e: you can't delete a fixed category.
     * 
     * @return array
     */
    public static function getFixedCategoriesIds() {
        if (!empty(self::$_fixed_categories_ids)) 
            return self::$_fixed_categories_ids;
        
        $ids = array();
        foreach (self::getFixedCategoriesSettings() as $setting) {
            $ids[$setting->getSettingName()] = unserialize($setting->getSettingValue());
        }
        self::$_fixed_categories_ids = $ids;
        return $ids;
    }

	public static function getFixedCategoriesIdsWithEnableSettings() {
		$ids = self::getFixedCategoriesIds();
		$toggleSettings = SettingPeer::getSettingsFromArrayAsArray('config', 0, array(
			'system.troveDevStatus_enabled',
			'system.troveLicense_enabled',
			'system.troveOS_enabled',
			'system.troveSpokenLanguage_enabled',
			'system.troveProgrammingLanguage_enabled',
			'system.troveTopic_enabled',
			'system.troveIntendedAudience_enabled'
		));
		$fixedTroveWithToggleSettings = array();
		foreach ($ids as $settingName => $id) {
			if (array_key_exists($settingName.'_enabled', $toggleSettings)) {
				$enabled = unserialize($toggleSettings[$settingName.'_enabled']) == 1 ? true : false;
			} else {
				$enabled = TRUE;
			}
			$fixedTroveWithToggleSettings[$id] = $enabled;
		}
		return $fixedTroveWithToggleSettings;

	}
    
    /**
     * Returns an array with a Setting object for each category configured as fixed.
     * Fixed categories have special business rules that are different from those for the rest of the categories.
     * I.e: you can't delete a fixed category.
     *
     * @return Setting[]
     */
    public static function getFixedCategoriesSettings() {
        return SettingPeer::getSettingsFromArray('config', 0, array(
            'system.troveDevStatus',
            'system.troveLicense',
            'system.troveOS',
            'system.troveSpokenLanguage',
            'system.troveProgrammingLanguage',
            'system.troveTopic',
            'system.troveIntendedAudience',
			'system.troveGfProjTemplates'
        ));
    }
    
	/**
	 * Returns and array containing a Trove object for each descendant of the
	 * category identified by $trove_id
	 * 
	 * @param int $trove_id root trove category identifier
	 * @return array
	 */
	public static function getDescendantsAsArray($trove_id) {
		$c = new Criteria();
		$c->add(self::ROOT_TROVE_CATEGORY_ID, $trove_id);
		$c->addAscendingOrderByColumn(self::CATEGORY_NAME);
		return self::doSelect($c);
	}
	
	/**
	 * Returns an array containing a Trove object for each root category
	 * 
	 * @param boolean $include_root_cat whether to include "Root" in the result
	 * @return array
	 */
	public static function getRootCategoriesAsArray($include_root_cat = FALSE) {
		$c = new Criteria();
		$c->add(self::PARENT_TROVE_CATEGORY_ID, 0, Criteria::EQUAL);
		if (!$include_root_cat) {
			$c->add(self::TROVE_CATEGORY_ID, 0, Criteria::NOT_EQUAL);
		}
		$c->addAscendingOrderByColumn(self::CATEGORY_NAME);
		return self::doSelect($c);
	}
	
    /*
     * Returns an array [cat_id => cat_name] with all root trove categories
     */
    public static function getRootCategoriesOptions() {
        $ret = array();
        foreach (self::getRootCategoriesAsArray(TRUE) as $category) {
			$ret[$category->getTroveCategoryId()] = $category->getCategoryName(); 
		}
		return $ret;
    }
    
	/**
	 * Returns an array with the names of the root categories, and the number of associated
	 * items
	 */
	public static function getRootCategoriesNamesWithCount($section) {
		$lang = Language::getSessLang();

		$ret = array();
		$ret[GFConf::get('system.troveDevStatus')]['name'] = $lang->getText('Trove.DevStatus');
		$ret[GFConf::get('system.troveDevStatus')]['category_count'] = 0;
		$ret[GFConf::get('system.troveDevStatus')]['children_count'] = 0;
		$ret[GFConf::get('system.troveLicense')]['name'] = $lang->getText('Trove.License');
		$ret[GFConf::get('system.troveLicense')]['category_count'] = 0;
		$ret[GFConf::get('system.troveLicense')]['children_count'] = 0;
		$ret[GFConf::get('system.troveOS')]['name'] = $lang->getText('Trove.OS');
		$ret[GFConf::get('system.troveOS')]['category_count'] = 0;
		$ret[GFConf::get('system.troveOS')]['children_count'] = 0;
		$ret[GFConf::get('system.troveSpokenLanguage')]['name'] = $lang->getText('Trove.SpokenLanguage');
		$ret[GFConf::get('system.troveSpokenLanguage')]['category_count'] = 0;
		$ret[GFConf::get('system.troveSpokenLanguage')]['children_count'] = 0;
		$ret[GFConf::get('system.troveProgrammingLanguage')]['name'] = $lang->getText('Trove.ProgrammingLanguage');
		$ret[GFConf::get('system.troveProgrammingLanguage')]['category_count'] = 0;
		$ret[GFConf::get('system.troveProgrammingLanguage')]['children_count'] = 0;
		$ret[GFConf::get('system.troveTopic')]['name'] = $lang->getText('Trove.Topic');
		$ret[GFConf::get('system.troveTopic')]['category_count'] = 0;
		$ret[GFConf::get('system.troveTopic')]['children_count'] = 0;
		$ret[GFConf::get('system.troveIntendedAudience')]['name'] = $lang->getText('Trove.IntendedAudience');
		$ret[GFConf::get('system.troveIntendedAudience')]['category_count'] = 0;
		$ret[GFConf::get('system.troveIntendedAudience')]['children_count'] = 0;
		
		$extra_troves = GFConf::get('system.extraTroveRootCategories');	
		$c = new Criteria();
		$c->add(TroveCategoryPeer::TROVE_CATEGORY_ID, $extra_troves, Criteria::IN);
		$categories = TroveCategoryPeer::doSelect($c);
		
		foreach ($categories as $category) {
			$ret[$category->getTroveCategoryId()]['name'] = $category->getCategoryName(); 
			$ret[$category->getTroveCategoryId()]['category_count'] = 0;
			$ret[$category->getTroveCategoryId()]['children_count'] = 0;
		}
		
		$root_ids = implode(',',array_keys($ret));
		$con = Propel::getConnection(self::DATABASE_NAME);
		$stmt = $con->prepareStatement(
					'SELECT tc.trove_category_id,
						(SELECT link_count FROM trove_link_count AS tlc WHERE tlc.trove_category_id=tc.trove_category_id AND tlc.section=?),
						(SELECT COUNT(*) FROM trove_link AS tl WHERE tl.trove_category_id=tc.trove_category_id AND tl.section=?)
					FROM trove_category AS tc WHERE tc.trove_category_id IN ('.$root_ids.')	
					');
		$stmt->setString(1, $section);
		$stmt->setString(2, $section);
		$rs = $stmt->executeQuery(ResultSet::FETCHMODE_NUM);

		while ($rs->next()) {
			$root_id = $rs->getInt(1);
			$children_count = $rs->getInt(2);
			$category_count = $rs->getInt(3);
			
			$ret[$root_id]['children_count'] = (!is_null($children_count)) ? $children_count : 0;
			$ret[$root_id]['category_count'] = $category_count;
		}
		
		return $ret;
	}
	
	/**
	 * Returns information about the parent and children canametegories for a given
	 * category
	 */
	public static function getCategoryNodeWithCount($current_trove_category_id, $section, $join_criteria) {
		$category_data = array();
		
		// This is a complex query that left-joins the trove_link_count table to see how many
		// items are related to a category
		$con = Propel::getConnection(self::DATABASE_NAME);
		$stmt = $con->prepareStatement(
				'SELECT tc.trove_category_id, tc.parent_trove_category_id, tc.category_name,
					(SELECT link_count FROM trove_link_count AS tlc WHERE tlc.trove_category_id=tc.trove_category_id AND tlc.section=?),
					(SELECT COUNT(*) FROM trove_link AS tl, project AS p
						WHERE tl.trove_category_id=tc.trove_category_id AND tl.section=? AND
						tl.ref_id=p.project_id AND p.is_public=true AND p.status=?
					)
				FROM trove_category AS tc 
				'
			);
		$stmt->setString(1, $section);
		$stmt->setString(2, $section);
		$stmt->setInt(3, Project::STATUS_ACTIVE);
		$rs = $stmt->executeQuery(ResultSet::FETCHMODE_NUM);
		
		while ($rs->next()) {
			$category_id = $rs->getInt(1);
			$parent_id = $rs->getInt(2);
			$category_name = $rs->getString(3);
			$total_count = $rs->getInt(4);		// Total count includes all the children
			$category_count = $rs->getInt(5);

			// Note that this will be inacurate (even negative) if the cronjobs haven't run, because
			// the trove_link_count table isn't updated
			$children_count = $total_count-$category_count;
			
			if (is_null($total_count)) $total_count = 0;	// This is caused by the LEFT JOIN when category has no links			
			
			$category_data[$category_id] = array('parent_id' => $parent_id,
					'name' => $category_name, 'category_count' => $category_count,
					'children_count' => $children_count);
		}
		
		if (!array_key_exists($current_trove_category_id, $category_data)) {
			// category doesn't exist
			return null;
		}
		
		// Now, get the parents and children for this category
		$children = array();
		foreach ($category_data as $category_id => $data) {
			if ($data['parent_id'] == $current_trove_category_id) {
				$children[] = array('trove_category_id' => $category_id, 'category_name' => $data['name'],
								'category_count' => $data['category_count'], 
								'children_count' => $data['children_count']);
			}
		}
		
		// Now search for the branch in the trove tree that contains the current category. 
		// We will generate a list of the parents from the root to the current category included.
		$branch = array();
		$parent_pointer = $current_trove_category_id;
		while (true) {
			if ($parent_pointer == 0 || !array_key_exists($parent_pointer, $category_data)) {
				// Got to end of "recursion"
				break;
			}
			$parent = array('trove_category_id' => $parent_pointer, 'category_name' => $category_data[$parent_pointer]['name'],
					'category_count' => $category_data[$parent_pointer]['category_count'],
					'children_count' => $category_data[$parent_pointer]['children_count']);
			array_unshift($branch, $parent);		// 'push' the parent to the beginning of the list
			$parent_pointer = $category_data[$parent_pointer]['parent_id'];
		}
		
		return array('branch' => $branch, 'children' => $children);
	}

	/**
	*	Returns number of children
	*
	*/
	public static function getCategoryChildrenCount($category_id) {
		$c = new Criteria();
		$c->add(TroveCategoryPeer::PARENT_TROVE_CATEGORY_ID, $category_id);
		return TroveCategoryPeer::doCount($c);
	}

	/**
	* Returns the trove categories that have the flag display_in_search = true
	*
	*/
	public static function getTroveCategoriesInSearch() {
		$Language = Session::getSession()->getLanguage();
		$c = new Criteria();
		$c->add(TroveCategoryPeer::TROVE_CATEGORY_ID, 0, Criteria::NOT_EQUAL);
		$c->add(TroveCategoryPeer::PARENT_TROVE_CATEGORY_ID, 0);
		$c->add(TroveCategoryPeer::SHOW_IN_SEARCH, 1);
		$c->addAscendingOrderByColumn(TroveCategoryPeer::CATEGORY_NAME);
		return TroveCategoryPeer::doSelect($c);
	}

	/**
	* Returns an array with the ids => alias of the trove categories that *can not* be accessed 
	* by the specified level.
	*
	*/
	public static function getDisabledCategories($accessLevel=0) {
		$Language = Session::getSession()->getLanguage();
		$c = new Criteria();
		$c->add(TroveCategoryPeer::TROVE_CATEGORY_ID, 0, Criteria::NOT_EQUAL);
		$c->add(TroveCategoryPeer::PARENT_TROVE_CATEGORY_ID, 0);
		$c->add(TroveCategoryPeer::ACCESS_LEVEL, (int)$accessLevel, Criteria::GREATER_THAN);
		$c->addAscendingOrderByColumn(TroveCategoryPeer::CATEGORY_NAME);
		$return = array();
		foreach (TroveCategoryPeer::doSelect($c) as $trove) {
			$return[$trove->getId()] = $trove->getAlias();
		}
		return $return;
	}


	/**
	 * Returns a result set with the following fields from trove_category table:
	 * @internal int trove_category_id
	 * @internal string alias
	 * @internal string category_name
	 * @internal int parent_trove_category_id
	 * and the calculated fields l
	 * @internal int level (depth of the current node in the tree)
	 * @internal int path (path of trove categories up to the current node)
	 * @param int $trove_category_id
	 * @return ResultSet[]
	 */
	public static function getTroveCategoriesRs($trove_category_id=0) {

		$con = Propel::getConnection();

		if (!$trove_category_id) {
			$where_condition = ' WHERE parent_trove_category_id=0 AND trove_category_id <> 0 ';
		} else {
			$where_condition = ' WHERE trove_category_id= ?';
		}

		$sql = "WITH RECURSIVE trove_category_level( trove_category_id, alias, category_name, parent_trove_category_id,
			access_level, show_in_search, level, path) AS
			(
                SELECT trove_category_id, alias, category_name, parent_trove_category_id,access_level, show_in_search,
                0 as level, category_name AS path
                    FROM trove_category $where_condition
                UNION
                SELECT trove_category.trove_category_id, trove_category.alias, trove_category.category_name, trove_category.parent_trove_category_id,
                trove_category.access_level, trove_category.show_in_search, level + 1 AS level, path || '/' || trove_category.category_name AS path
                    FROM trove_category JOIN trove_category_level ON trove_category.parent_trove_category_id=trove_category_level.trove_category_id
			)
			SELECT * FROM trove_category_level ORDER BY category_name";

		$stmt = $con->prepareStatement($sql);
		if ($trove_category_id > 0) {
			$stmt->setInt(1,$trove_category_id);
		}

		$rs = $stmt->executeQuery(ResultSet::FETCHMODE_ASSOC);

		return $rs;

	}
	
	/**
	 * Traverse a tree from $start and returns a two dimensional array
	 * @param int $start starting trove_category_id
	 * @return array
	 */
	public static function getTroveCategoriesAsArray($start=0) {
		$categories = array();
		$rs = self::getTroveCategoriesRs($start);

		while($rs->next()) {
			$this_cat['trove_category_id']= $rs->getInt('trove_category_id');
			$this_cat['parent_trove_category_id'] = $rs->getInt('parent_trove_category_id');
			$this_cat['alias'] = $rs->getInt('alias');
			$this_cat['category_name'] = $rs->getString('category_name');
			$this_cat['path'] = $rs->getString('path');
			$this_cat['level'] = $rs->getInt('level');
			$this_cat['access_level'] = $rs->getInt('access_level');
			$categories[$this_cat['trove_category_id']] = $this_cat;
		}

		return $categories;
	}

	/**
	 * Returns an hierarchical array starting from $start
	 *
	 * @param int $start starting trove_category_id
	 * @return array
	 */
	public static function getTroveTree($start=0) {
		$categories = array();
		$rs = self::getTroveCategoriesRs($start);

		while($rs->next()) {
			$this_cat = array();
			$this_cat['trove_category_id']= $rs->getInt('trove_category_id');
			$this_cat['parent_trove_category_id'] = $rs->getInt('parent_trove_category_id');
			$this_cat['alias'] = $rs->getInt('alias');
			$this_cat['category_name'] = $rs->getString('category_name');
			$this_cat['path'] = $rs->getString('path');
			$this_cat['level'] = $rs->getInt('level');
			$this_cat['access_level'] = $rs->getInt('access_level');
            $this_cat['show_in_search'] = $rs->getBoolean('show_in_search');
			$this_cat['childs'] = array();

			$categories[$this_cat['trove_category_id']] = $this_cat;
		}

		return self::buildTree($categories, $start);

	}
	
	/**
	 * Recursive function used to build an hierarchical array of Trove Categories
	 * @param $categories
	 * @param int $parentId
	 * @return array
	 */
	private static function buildTree($categories, $parentId=0) {
		$tree = array();

		foreach($categories as $category){
			if($category['parent_trove_category_id'] == $parentId) {
				$children = self::buildTree($categories, $category['trove_category_id']);
				if ($children) {
					$category['children'] = $children;
				}
				$tree[] = $category;
			}
		}

		return $tree;
	}
	
	/**
	 * Returns a ResultSet with the id of every category that has at least one child in the trove tree
	 * 
	 * @return ResultSet
	 */
	public static function getNonLeafTroveCategoriesIdsRs() {
		$con = Propel::getConnection();
		$stmt = $con->prepareStatement("SELECT DISTINCT(t1.trove_category_id) "
				. "FROM trove_category t1, trove_category t2 "
				. "WHERE t1.trove_category_id = t2.parent_trove_category_id "
				. "ORDER BY t1.trove_category_id"
		);
		return $stmt->executeQuery(ResultSet::FETCHMODE_ASSOC);
	}

	/**
	 * Returns a ResultSet with the id of every category that has at least one child in the trove tree
	 * 
	 * @return type
	 */
	public static function getNonLeafTroveCategoriesIdsAsArray() {
		if (!empty(self::$_non_leaf_categories_ids)) 
            return self::$_non_leaf_categories_ids;
		
		$categories = array();
		$rs = self::getNonLeafTroveCategoriesIdsRs();
		while($rs->next()) {
			$categories[] = $rs->getInt('trove_category_id');
		}
		self::$_non_leaf_categories_ids = $categories;
		return $categories;
	}
	
	/**
	 * Returns a ResultSet with the project links count for every trove_category whose id is included in $trove_ids
	 * 
	 * @param array $trove_ids ids to be selected
	 * @return ResultSet
	 */
	public static function getProjectLinkCountsRs($trove_ids) {
		$con = Propel::getConnection();
		$stmt = $con->prepareStatement("SELECT trove_category_id, category_name, project_links_count FROM trove_category WHERE trove_category_id in (".implode(', ', $trove_ids).")");
		return $stmt->executeQuery(ResultSet::FETCHMODE_ASSOC);
	}
	
	/**
	 * Returns an array with the project links count for every trove_category whose id is included in $trove_ids
	 * 
	 * @param array $trove_ids ids to be selected
	 * @return array
	 */
	public static function getProjectLinkCountsAsArray($trove_ids) {
		$categories = array();
		$rs = self::getProjectLinkCountsRs($trove_ids);
		while($rs->next()) {
			$categories[$rs->getInt('trove_category_id')] = array(
				'category_name' => $rs->getString('category_name'),
				'project_links_count' => $rs->getInt('project_links_count')
			);
		}
		return $categories;
	}
	
}
?>
