<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// The parent class
require_once 'core/om/BaseTroveLinkPeer.php';

// The object class
include_once 'core/TroveLink.php';

/** 
 * The skeleton for this class was autogenerated by Propel
 *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package core 
 */
class TroveLinkPeer extends BaseTroveLinkPeer {
	public static function getSelectedCategoriesArray($section, $ref_id) {
		$c = new Criteria();
		$c->add(self::SECTION, $section);
		$c->add(self::REF_ID, $ref_id);
		$c->addSelectColumn(self::TROVE_CATEGORY_ID);
		
		$rs = BasePeer::doSelect($c);
		$result = array();
		while ($rs->next()) {
			$result[] = $rs->get(1);
		}
		
		return $result;
	}
	
	/**
	 * Get a list of all the selected categories that belong to a certain parent category
	 */	
	public static function getSelectedRootCategoryValues($section, $ref_id, $root_category_id) {
		$c = new Criteria();
		$c->add(self::SECTION, $section);
		$c->add(self::REF_ID, $ref_id);
		$c->addJoin(self::TROVE_CATEGORY_ID, TroveCategoryPeer::TROVE_CATEGORY_ID);
		$c->add(TroveCategoryPeer::ROOT_TROVE_CATEGORY_ID, $root_category_id);
		$c->addAscendingOrderByColumn(TroveCategoryPeer::CATEGORY_NAME);
		
		$res = array();
		$links = self::doSelectJoinTroveCategory($c);
		foreach ($links as $link) {
			$res[] = $link->getTroveCategory();
		}
		
		return $res;
	}
	
	public static function getSelectedCategoriesTree($section, $ref_id) {
		$c = new Criteria();
		$c->add(self::SECTION, $section);
		$c->add(self::REF_ID, $ref_id);
		$c->addAscendingOrderByColumn(TroveCategoryPeer::CATEGORY_NAME);
		$c->addJoin(self::TROVE_CATEGORY_ID, TroveCategoryPeer::TROVE_CATEGORY_ID);
		
		$categories = TroveCategoryPeer::doSelect($c);
		$res = array();
		$roots_names = TroveCategoryPeer::getRootCategoriesNames();

		asort($roots_names);
		foreach ($roots_names as $root_id => $name) {
			$res[] = array(
						'root_id'	=> $root_id,
						'name'		=> $name,
						'children'	=> array()
					);
		}


		$n = count($res);
		$has_any = false;
		foreach ($categories as $category) {
			$root_id = $category->getRootTroveCategoryId();
			for ($i=0; $i < $n; $i++) {
				if ($res[$i]['root_id'] == $root_id) {
					$has_any = true;
					$res[$i]['children'][] = $category;
				}
			}
		}
		
		if ($has_any) {
			return $res;
		} else {
			return array();
		}
	}
	
	/**
	 * Returns a ResultSet containing all links from Project identified by $project_id
	 * to categories in the subtree whose root is $subtree_root_id
	 * 
	 * @param int $project_id
	 * @param int $subtree_root_id
	 * @returns ResultSet
	 */
	public static function getProjectLinksToTroveSubtreeRs($project_id, $subtree_root_id) {
		$con = Propel::getConnection();
		$sql = "SELECT * FROM trove_link tl WHERE tl.section = '".Project::SECTION."' AND tl.ref_id = ? AND trove_category_id IN ("
				. "SELECT trove_category_id FROM trove_children(?)"
				. ")";
		$stmt = $con->prepareStatement($sql);
		$stmt->setInt(1, $project_id);
		$stmt->setInt(2, $subtree_root_id);
		return $stmt->executeQuery();
	}
	
	/**
	 * Removes all links from Project identified by $project_id
	 * to categories in the subtree whose root is $subtree_root_id
	 * 
	 * @param int $project_id
	 * @param int $subtree_root_id
	 * @returns int
	 */
	public static function removeProjectLinksToTroveSubtree($project_id, $subtree_root_id) {
		$site_admin = Session::getSession()->getUser()->isSiteAdmin();
		$con = Propel::getConnection();
		$sql = "DELETE FROM trove_link tl WHERE tl.section = '".Project::SECTION."' AND tl.ref_id = ? AND trove_category_id IN ("
				. "SELECT trove_category_id FROM trove_children(?)"
				. ($site_admin ? "" : " WHERE access_level = 0")
				. ")";
		$stmt = $con->prepareStatement($sql);
		$stmt->setInt(1, $project_id);
		$stmt->setInt(2, $subtree_root_id);
		return $stmt->executeUpdate();
	}
	
	/**
	 * Returns a ResultSet containing all links from projects to template container troves
	 * 
	 * @return ResultSet
	 */
	public static function getTemplateProjectLinksRs(){
		$con = Propel::getConnection();
		$sql = "SELECT trove_category_id, project_id, project_name FROM trove_link tl, project p "
				. "WHERE tl.ref_id = p.project_id AND tl.section='".Project::SECTION."' "
				. "AND trove_category_id IN (SELECT trove_category_id FROM trove_children(".GFConf::get('system.troveGfProjTemplates')."))";
		return $con->prepareStatement($sql)->executeQuery(ResultSet::FETCHMODE_ASSOC);		
	}
	
	/**
	 * Returns an array containing all links from projects to template container troves
	 * 
	 * @return array trove_id => array( project_id => project_name, ... )
	 */
	public static function getTemplateProjectLinksAsArray() {
		$links = array();
		$rs = self::getTemplateProjectLinksRs();
		while($rs->next()) {
			if (!isset($links[$rs->getInt('trove_category_id')])) {$links[$rs->getInt('trove_category_id')] = array();}
			$links[$rs->getInt('trove_category_id')][$rs->getInt('project_id')] = $rs->getString('project_name');
		}
		return $links;
	}
	
	/**
	 * Returns a ResultSet containing all troves linked to the Project identified by $project_id
	 * 
	 * @param int $project_id
	 * @return ResultSet
	 */
	public static function getProjectLinksRs($project_id) {
		$con = Propel::getConnection();
		$sql = "SELECT tc.trove_category_id, tc.category_name FROM trove_link tl, trove_category tc "
				. "WHERE tl.trove_category_id = tc.trove_category_id AND tl.ref_id = ? AND tl.section='".Project::SECTION."'";
		
		// Exclude descendants of disabled categories
		$disabled_fix_cat_ids = array_keys(
			array_filter(TroveCategoryPeer::getFixedCategoriesIdsWithEnableSettings(), function($enabled) { return !$enabled; })
		);
		if (!empty($disabled_fix_cat_ids)) { 
			$sql .= " AND tc.root_trove_category_id NOT IN (".implode(', ', $disabled_fix_cat_ids).")"; 
		}
		
		$stmt = $con->prepareStatement($sql);
		$stmt->setInt(1, $project_id);
		return $stmt->executeQuery(ResultSet::FETCHMODE_ASSOC);
	}
	
	/**
	 * Returns an array containing all troves linked to the Project identified by $project_id
	 * 
	 * @param int $project_id
	 * @return array trove_category_id => category_name
	 */
	public static function getProjectLinksAsArray($project_id) {
		$links = array();
		$rs = self::getProjectLinksRs($project_id);
		while($rs->next()) { 
			$links[$rs->getInt('trove_category_id')] = $rs->getString('category_name');
		}
		return $links;
	}

}
?>