<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
// The parent class
require_once 'core/om/BaseUserPreferencePeer.php';

// The object class
include_once 'core/UserPreference.php';

/**
 * The skeleton for this class was autogenerated by Propel  *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package core
 */
class UserPreferencePeer extends BaseUserPreferencePeer {

	/**
	 * Get the value of the specified preference for this user, if it is set in the database.
	 * @param int $user_id Unique identifier for the user
	 * @param string $prefName Unique identifier for the preference type
	 * @param string $default_value Value to be returned if the preference value is not found for the specified user.
	 * @return string The value found in the database or the default value.
	 */
	public static function getUserPreferenceValue($user_id, $prefName, $default_value = '') {
		$pref = self::retrieveByPK($user_id, $prefName);
		if ($pref != null &&
				$pref->getPreferenceValue() != null) {
			return $pref->getPreferenceValue();
		}
		return $default_value;
	}


	/**
	 * Get the set of preference records whose name matches the specified criteria value
	 * @param string $prefName The string to be matched to the preference's name field
	 * @return array Zero or more UserPreference records, matching the specified prefix
	 */
	public static function getUserPrefsByPrefix($prefName) {
		$c = new Criteria();
		$c->add(self::PREFERENCE_NAME, $prefName);
		$c->setLimit(51);  // We'll use 50 records, but get 51 just to know there are "more than 50"
		$c->addAscendingOrderByColumn(UserPeer::UNIX_NAME);

		return self::doSelectJoinUser($c);
	}

	/**
	 * Get the set of preference records for the specified user, whose name starts with the specified criteria value
	 * @param int $user_id Unique identifier for the user
	 * @param string $prefix The string prefix to be matched to the preference's name field
	 * @return array Zero or more UserPreference records, matching the specified criteria
	 */
	public static function getUserPrefsByUserAndPrefix($user_id, $prefix) {
		$c = new Criteria();
		$c->add(self::USER_ID, $user_id);
		$c->add(self::PREFERENCE_NAME, $prefix . '%', Criteria::ILIKE);
		$c->addAscendingOrderByColumn(UserPeer::UNIX_NAME);

		return self::doSelectJoinUser($c);
	}

	/**
	 * Deletes, for all users, preference records matched by $prefName,
	 * that were set prior to the specified date/time
	 * @param string $prefName of the preference to be affected for all users
	 * @param string $before Date & time before which records will be deleted.
	 * @return int The set of records actually deleted
	 */
	public static function deleteOldUserPrefs($prefName, $before) {
		$c = new Criteria();
		$c->add(self::PREFERENCE_NAME, $prefName, Criteria::EQUAL);
		$c->add(self::SET_DATE, $before, Criteria::LESS_THAN);
		$user_prefs = self::doSelect($c);
		self::doDelete($c);
		return $user_prefs;
	}

	/**
	 * Updates, for all users, preference records matched by $prefName,
	 * that were last set prior to the specified date/time
	 * @param string $prefName of the preference to be affected for all users
	 * @param string $before Date & time before which records will be deleted.
	 * @param string $newValue Affected records will have this value after the update.
	 * @return int The set of records actually updated
	 */
	public static function updateUserPrefsBefore($prefName, $before, $newValue) {
		$c = new Criteria();
		$c->add(self::PREFERENCE_NAME, $prefName, Criteria::EQUAL);
		$c->add(self::SET_DATE, $before, Criteria::LESS_THAN);
		$c->add(self::PREFERENCE_VALUE, $newValue, Criteria::NOT_EQUAL);
		$user_prefs = self::doSelect($c);

		$upd = new Criteria();
		$upd->add(self::PREFERENCE_VALUE, $newValue);

		BasePeer::doUpdate($c, $upd, Propel::getConnection(self::DATABASE_NAME));

		return $user_prefs;
	}

	/**
	 * Updates set date for preference records whose user id and name matches the specified crtieria.
	 * @param int $userId Unique identifier for the user
	 * @param string $prefName The string to be matched to the preference's name field
	 * @return int The number of records actually updated
	 */
	public static function refreshUserPref($userId, $prefName, $prefValue) {
		$c = new Criteria();
		$c->add(self::USER_ID, $userId);
		$c->add(self::PREFERENCE_NAME, $prefName);
		$c->add(self::PREFERENCE_VALUE, $prefValue);

		$upd = new Criteria();
		$upd->add(self::SET_DATE, time());

		return BasePeer::doUpdate($c, $upd, Propel::getConnection(self::DATABASE_NAME));
	}

	/**
	 * Updates or creates a preference record for the given user id and name
	 * @param int $userId Unique identifier for the user
	 * @param string $prefName The string to be matched to the preference's name field
	 * @return int The number of records actually updated/inserted
	 */
	public static function setUserPref($userId, $prefName, $prefValue) {
		$c = new Criteria();
		$c->add(self::USER_ID, $userId);
		$c->add(self::PREFERENCE_NAME, $prefName);

		$upd = new Criteria();
		$upd->add(self::PREFERENCE_VALUE, $prefValue);
		$upd->add(self::SET_DATE, time());

		if (BasePeer::doUpdate($c, $upd, Propel::getConnection(self::DATABASE_NAME)) == 1) {
			return 1;
		} else {
			$pref = new UserPreference();
			$pref->setPreferenceName($prefName);
			$pref->setUserId($userId);
			$pref->setSetDate(time());
			$pref->setPreferenceValue($prefValue);
			$pref->save();
			return 1;
		}
	}

	/**
	 * Deletes any thread-related user preferences
	 * @param int $threadId Unique identifier for the thread
	 * @return none
	 */
	public static function deleteDiscussionPreferences($threadId) {
		$c = new Criteria();
		$cton1 = $c->getNewCriterion(self::PREFERENCE_NAME, 'chat-room-' . $threadId);
		$cton2 = $c->getNewCriterion(self::PREFERENCE_NAME, 'chat-' . $threadId . '-sound');
		$cton1->addOr($cton2);
		$c->add($cton1);
		self::doDelete($c);

		return;
	}

}