<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'core/om/BaseUserProject.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class UserProject extends BaseUserProject implements GFPermission {
	private $is_new_project = false;

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 */
	public function userCanCreate() {
		// special case, new project registration and allowed non-siteadmin to do so
		if ($this->is_new_project && (GFConf::get('system.projectRegRestricted') == false)) {
			return true;
		}
		
		if ($this->userIsAdmin()) {
			return true;
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	public function userIsAdmin() {
		return $this->getProject()->userIsAdmin();
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 */
	public function userCanDelete() {
		if ($this->userIsAdmin()) {
			return true;
		} else {
			// The user must be able to remove himself from the project
			$session = Session::getSession();
			$session_user = $session->getUser();
			if (!$session_user || ($session_user->getUserId() != $this->getUser()->getUserId())) {
				return false;
			}
			
			return true;
		}
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		if ($this->userIsAdmin()) {
			return true;
		}
		return false;
	}

	/**
	 * This function must be called when a user creates a new project and needs to assign itself
	 * to the project
	 */
	public function setIsNewProject() {
		$this->is_new_project = true;
	}

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see UserProject::setDeleted()
	 * @see UserProject::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete UserProject object: permission denied");
		}
		
		$user = UserPeer::retrieveByPk($this->getUserId());
		$project = ProjectPeer::retrieveByPk($this->getProjectId());
		
		$params = array("user" => $user, "project" => $project);
		ActionManager::notify("USER_PROJECT_DELETE", $params);
		
		parent::delete($con);
		
		// Create the audit trail
		$audit = new AuditTrail();
		$audit->setUser(Session::getSession()->getUser());
		$audit->setSection('user_project');
		$audit->setRefId($this->getProjectId());
		$audit->setFieldName(null);
		$audit->setNewValue(null);
		$audit->setOldValue($this->getUserId());
		$audit->save();
		
		// If the user was actually a user group, notify about the individual users being removed
		if ($user->getIsGroup()) {
			$users = $user->getUsergroupUsers();
			foreach ($users as $group_user) {
				if (!$group_user->isProjectMember($project)) {
					// If the user doesn't belong to the project after removing the user group, it means
					// it was definitely removed from the project. Fire up the notification.
					$params = array("user" => $group_user, "project" => $project);
					ActionManager::notify("USER_PROJECT_DELETE", $params);

				}
			}


		}
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null, $skipPermCheck = false) {
		if ($this->isNew() && !$this->userCanCreate() && !$skipPermCheck) {
			throw new PermissionDeniedException("Could not create UserProject object: permission denied");
		} else if (!$this->isNew() && !$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update UserProject object: permission denied");
		}

		$isNew = $this->isNew();
		ActionManager::notify("USER_PROJECT_SAVE", $this);
		parent::save($con, $skipPermCheck);
		
		if ($isNew && !$skipPermCheck) {
			// Create the audit trail
			$audit = new AuditTrail();
			$audit->setUser(Session::getSession()->getUser());
			$audit->setSection('user_project');
			$audit->setRefId($this->getProjectId());
			$audit->setFieldName(null);
			$audit->setNewValue($this->getUserId());
			$audit->setOldValue(null);
			$audit->save();
		}
	}


}
?>
