<?php
require_once 'core/om/BaseUserProjectRole.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once('core/AuditTrail.php');

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class UserProjectRole extends BaseUserProjectRole implements GFPermission {
	private $is_new_project = false;
	
	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 */
	public function userCanCreate() {
		// special case, new project registration and allowed non-siteadmin to do so
		if ($this->is_new_project && (GFConf::get('system.projectRegRestricted') == false)) {
			return true;
		}

		if ($this->userIsAdmin()) {
			return true;
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		return $this->getUserProject()->userIsAdmin();
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 */
	public function userCanDelete() {
		if ($this->userIsAdmin()) {
			return true;
		} else {
			// The user must be able to remove himself from the project
			$session = Session::getSession();
			$session_user = $session->getUser();
			if (!$session_user || ($session_user->getUserId() != $this->getUserProject()->getUser()->getUserId())) {
				return false;
			}
			
			return true;
		}
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanRead() {
		if ($this->userIsAdmin()) {
			return true;
		}
		return false;
	}
	
	/**
	 * This function must be called when a user creates a new project and needs to assign itself
	 * to the project
	 */
	public function setIsNewProject() {
		$this->is_new_project = true;
	}

	public function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete UserProjectRole object: permission denied");
		}

		parent::delete($con);

		// Create the audit trail
		$audit = new AuditTrail();
		$audit->setUser(Session::getSession()->getUser());
		$audit->setSection('user_project_role');
		$audit->setRefId($this->getUserProject()->getProjectId());
		$audit->setFieldName(null);
		$audit->setNewValue(null);
		// Ugly hack we must do because we can't store the user_project_id, because
		// the user may be removed from the project and we don't want to lose the
		// trail
		$value = $this->getUserProject()->getUserId().','.$this->getRoleId();
		$audit->setOldValue($value);
		$audit->save();
	}
	
	public function save($con = null, $skipPermCheck = false) {
		if ($this->isNew() && !$this->userCanCreate() && !$skipPermCheck) {
			throw new PermissionDeniedException("Could not create UserProjectRole object: permission denied");
		} else if (!$this->isNew() && !$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update UserProjectRole object: permission denied");
		}

		$isNew = $this->isNew();
		parent::save($con);

		if ($isNew && !$skipPermCheck) {
			// Create the audit trail
			$audit = new AuditTrail();
			$audit->setUser(Session::getSession()->getUser());
			$audit->setSection('user_project_role');
			$audit->setRefId($this->getUserProject()->getProjectId());
			$audit->setFieldName(null);
			// Ugly hack we must do because we can't store the user_project_id, because
			// the user may be removed from the project and we don't want to lose the
			// trail
			$value = $this->getUserProject()->getUserId().','.$this->getRoleId();
			$audit->setNewValue($value);
			$audit->setOldValue(null);
			$audit->save();
		}
	}

}
