<?php
require_once 'core/om/BaseUsergroupUser.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package core 
 */
class UsergroupUser extends BaseUsergroupUser implements GFPermission {

	private $project_monitors_to_delete = '';
	
	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool
	 */
	public function userCanCreate() {
		if ($this->userIsAdmin()) {
			return true;
		}

		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		if (Session::getSession()->isSiteAdmin()) {
			return true;
		}
		return false;
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 */
	public function userCanRead() {
		if ($this->userIsAdmin()) {
			return true;
		}
		return false;
	}

	public function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create UsergroupUser object: permission denied");
		} else if (!$this->isNew() && !$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update UsergroupUser object: permission denied");
		}

		// Check that the user is not a group (we can't have nested groups) 	 
		$member = UserPeer::retrieveByPk($this->member_user_id); 	 
		if (is_null($member)) {
			// Do nothing if the user is invalid
			return null;
		} 	 

		$maxmembers = GFConf::varExists('system.licenseMaxMembers') ? GFConf::get('system.licenseMaxMembers') : 0;

		//if the number of users assigned to projects is limited by the license, we must
		//ensure that adding this user to the usergroup doesn't exceed the license allowed members
		if ($maxmembers > 0){

			//verify if this usergroup is already assigned to at least a project		
			$c = new Criteria();
			$c->add(UserProjectPeer::USER_ID, $this->usergroup_id);
			$userproject = UserProjectPeer::doSelectOne($c);
			
			//if getting added to a group, count to see if the license is full    
			$sql = 'SELECT count(*) AS count FROM "user" WHERE user_id IN (
								SELECT user_id FROM user_project NATURAL JOIN "user" WHERE "user".is_group=false
								UNION
								SELECT member_user_id AS user_id FROM usergroup_user WHERE usergroup_id IN (
								SELECT user_id FROM user_project NATURAL JOIN "user" WHERE "user".is_group=true
								)
							)';
			$stmt = Propel::getConnection()->createStatement();
			$rs = $stmt->executeQuery($sql);
			$rs->next();
			$count_licenses_used = $rs->getInt('count');
	
			//Check if the user is already using a license (i.e.: assigned to a project individually or through another usergroup
			$sql .= ' AND user_id = '.$this->member_user_id;
			$stmt = Propel::getConnection()->createStatement();
			$rs = $stmt->executeQuery($sql);
			$rs->next();
			$count_this_user = $rs->getInt('count');	//if > 0, then the user is assigned to some project
	
			//if a, b and c are true, where:
			//	a) the newly added user is not assigned to any project
			//	b) the usergroup is assigned to some project
			//	c) the number of licenses is equal-to-or-greater-than the allowed max members
			//then the newly added user would consume another license, therefore the exception is thrown
			if ($count_this_user == 0 && $count_licenses_used >= $maxmembers && is_object($userproject)) {
				throw new InvalidLicenseException('Maximum licensed members exceeded. Contact GForge Group for help.');
			}
		}

		if ($member->getIsGroup()) { 	 
			throw new InvalidDataException(array('member_user_id' => Language::getSessText('UserGroupUser.MemberIsGroup'))); 	 
		} 	 
		parent::save($con); 	 
	}
	
	public function delete($con = null) {
		//Sets the list of user_ids of the users being removed from the project whose monitors can be deleted
		$this->setProjectMonitorsToDelete();

		ActionManager::notify("USERGROUP_USER_REMOVE", $this);
		parent::delete($con);
	}


	/**
	 * Getter method for the "project_monitors_to_delete" property
	 */
	public function getProjectMonitorsToDelete() {
		return($this->project_monitors_to_delete);
	}


	/**
	 * Sets the property "project_monitors_to_delete", which is a comma separated list of project_ids
	 * for which the monitors for the deleted user can be deleted.
	 */
	public function setProjectMonitorsToDelete() {
		$arr_projects = array();
		$con = Propel::getConnection();		

		//Gets the projects whose monitors for this user can be deleted. They must meet:
		//	a. the user doesn't belong individually to the project
		//	b. the group is assigned to the project
		//	c. the user doesn't belong to any other group assigned to the project
		$sql = 'SELECT project_id FROM project WHERE
						project_id NOT IN 
							(SELECT project_id FROM user_project WHERE
							 user_id = '.$this->getMemberUserId().')
					 AND
					 project_id NOT IN
							(SELECT project_id FROM user_project, "user" WHERE
							 user_project.user_id = "user".user_id
							 AND 
							 "user".is_group = true
							 AND
							 user_project.user_id IN 
							 	(SELECT usergroup_id FROM usergroup_user WHERE 
							 	 member_user_id = '.$this->getMemberUserId().'
							 	 AND
							 	 usergroup_id <> '.$this->getUsergroupId().')
							)
					AND
					project_id IN
							(SELECT project_id FROM user_project, "user" WHERE
							 user_project.user_id = "user".user_id
							 AND 
							 "user".is_group = true
							 AND
							 user_project.user_id IN 
							 	(SELECT usergroup_id FROM usergroup_user WHERE
							 	member_user_id = '.$this->getMemberUserId().'
							 	AND
							 	usergroup_id = '.$this->getUsergroupId().')
							 )';

		$stmt = $con->createStatement();	
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

		//Get the user_ids of the users whose monitors must be deleted
		while ($rs->next()) {
			$arr_projects[] = $rs->getInt('project_id');					
		}

		//build project_id list of the projects whose monitors must be deleted for this user
		$str_projects = implode(",", $arr_projects);
		$str_projects = ($str_projects == "") ? 0 : $str_projects;

		$this->project_monitors_to_delete = $str_projects;
	}

}
?>
