<?php
// The parent class
require_once 'core/om/BaseWallPostPeer.php';

// The object class
include_once 'core/WallPost.php';

// Needed for finding users by unix names in AddUserWallPost
require_once 'core/UserPeer.php';
require_once 'core/ProjectPeer.php';

/** 
 * The skeleton for this class was autogenerated by Propel  *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package core 
 */
class WallPostPeer extends BaseWallPostPeer {

	const POST_FILTER_TYPE_BYUSER = 1;
	const POST_FILTER_TYPE_TOUSER = 2;
	const POST_FILTER_TYPE_BY_OR_TO_USER = 0;

	public static function GetNotificationsSince($user_id, $since_id = 0, $max_results = 0) {
		$crit = self::CreateUserCriteria($user_id, self::POST_FILTER_TYPE_TOUSER, 0, 0, $since_id);
		$start_date = date('c', strtotime('today'));
		$crit->add(self::POST_DATE, $start_date, Criteria::GREATER_EQUAL);
		if ($max_results > 0) {
			$crit->setLimit($max_results);
		}
		//error_log("GetNotificationsSince($user_id, $since_id, $max_results) SQL is {$crit->toString()}");
		return self::doSelectJoinUser($crit);
	}

	public static function GetHistoryNotifications($user_id, $since_id = 0, $max_results = 0) {
		$crit = self::CreateUserCriteria($user_id, self::POST_FILTER_TYPE_TOUSER, 0, 0, $since_id, true);
		if ($max_results > 0) {
			$crit->setLimit($max_results);
		}
		//error_log("GetNotificationsSince($user_id, $since_id, $max_results) SQL is {$crit->toString()}");
		return self::doSelectJoinUser($crit);
	}


    public static function CreateUserCriteria($user_id, $filter = self::POST_FILTER_TYPE_BY_OR_TO_USER, $year = 0, $month = 0, $since_id = 0, $history=false) {
//Retrieves the wall posts
	$crit = new Criteria();

//Refines search with year/month, if selected
	if (!$history) {
		if ($year != 0 && $month != 0) {
		    $start_date = $year . "-" . $month . "-01";
		    $last_day_month = date('t', strtotime($month . "/1/" . $year));
		    $end_date = $year . "-" . $month . "-" . $last_day_month;
	
		    $crit->add(self::POST_DATE, $start_date, Criteria::GREATER_EQUAL);
		    $crit->addAnd(self::POST_DATE, $end_date, Criteria::LESS_EQUAL);
		}
	
		if ($since_id > 0) {
		    $crit->add(self::WALL_POST_ID, $since_id, Criteria::GREATER_THAN);
		}
	} else {
		if ($since_id > 0) {
		    $crit->add(self::WALL_POST_ID, $since_id, Criteria::LESS_THAN);
		}
	}


//Refines search with:
	$crit->add(self::SECTION, User::SECTION);
	switch ($filter) {
	    case self::POST_FILTER_TYPE_BYUSER:
		//Filter posts by current user posts
		$crit->add(self::POSTED_BY, $user_id);
		break;

	    case self::POST_FILTER_TYPE_TOUSER:
		//Filter posts by current user as target of the posts
		$crit->add(self::REF_ID, $user_id);
		break;

	    default:  // includes static::POST_FILTER_TYPE_BY_OR_TO_USER
		$c1 = $crit->getNewCriterion(self::POSTED_BY, $user_id);
		$c2 = $crit->getNewCriterion(WallPostPeer::REF_ID, $user_id);
		$c1->addOr($c2);
		$crit->add($c1);
		break;
	}

	$crit->addDescendingOrderByColumn(WallPostPeer::WALL_POST_ID);
	return $crit;
    }

    public static function GetArchiveList($user_id) {
//Get archives data (month/year)
	$section = User::SECTION;
	$str_sql = "SELECT date_part('month', post_date) as smonth, date_part('year', post_date) as syear
						FROM wall_post
						WHERE section='$section' AND ref_id=$user_id
						GROUP BY syear, smonth
						ORDER BY syear, smonth DESC";

	$con = Propel::getConnection(self::DATABASE_NAME);
	$stmtArchives = $con->createStatement();
	$rs_archives = $stmtArchives->executeQuery($str_sql);

	$arr_archives = array();
	$i = 0;
	foreach ($rs_archives as $row) {
	    $arr_archives[$i]['month'] = $row['smonth'];
	    $arr_archives[$i]['year'] = $row['syear'];
	    $arr_archives[$i]['desc'] = date("F", mktime(0, 0, 0, $row['smonth'])) . " " . $row['syear'];
	    $i++;
	}

	return $arr_archives;
    }

    static function AddUserWallPost($from_user_id, $to_user_id, $message) {

	if ($from_user_id != $to_user_id) {
	    $post = new WallPost();
	    $post->setPostDate(time());
	    $post->setPostedBy($from_user_id);
	    $post->setSection(User::SECTION);
	    $post->setRefId($to_user_id);
	    $post->setComment($message);

	    $post->save();
	    return TRUE;
	}

	return FALSE;
    }
}
