<?php
/**
 * Copyright (c) 2005-2014 GForge, LLC
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
class CSRFHandler {
	private $lifetime; //Minutes for which the token is valid
	private $sid;
	private $user_id;
	private $referer_params;

	public function __construct($user_id, $sid, $referer_params=array()) {
		$this->user_id = $user_id;
		$this->sid  = preg_replace('/[^a-z0-9]+/i','',$sid);
		if (is_array($referer_params)) {
			$this->referer_params = $referer_params;
		} else {
			$this->referer_params = array();
		}
		$this->lifetime =  GFConf::get('system.csrfTokenLifetime', 60);
	}

	public function getCSRFToken($key) {
		//Generate the token
		$token = md5(microtime() . $this->sid . rand());
		//Create a data array to store the token + timestamp
		$arr_token = array();
		$arr_token['timestamp'] = time();
		$arr_token['token'] = $token;
		//Save it in the session
		$_SESSION[$key] = serialize($arr_token);
		//Return the token
		return $token;
	}

	public function checkCSRFToken($key, $token) {
		//Get stored token
		$arr_token = isset($_SESSION[$key]) ? unserialize($_SESSION[$key]) : null;

		if ( !(is_array($arr_token) && isset($arr_token['token'])) ) {
			$this->logCsrfViolation();
			return false;
		}

		//Delete stored token for this form
		unset($_SESSION[$key]);

		if ($token != "" && $arr_token['token'] == $token && ($arr_token['timestamp'] + $this->lifetime*60) >= time()) {
			return true;
		}

		$this->logCsrfViolation();
		return false;
	}

	private function logCsrfViolation() {
		//Creates an object for logging a CSRF violation
		$params = array();
		$params['section'] = (isset($this->referer_params['section'])) ? $this->referer_params['section'] : '';
		$params['user_id'] = $this->user_id;
		$params['ref_id'] =  (isset($this->referer_params['ref_id'])) ? $this->referer_params['ref_id'] : 0;
		$params['activity_type'] = 'csrf';
		$referer_url =  (isset($this->referer_params['ref_id'])) ? $this->referer_params['ref_id'] : $_SERVER['HTTP_REFERER'];
		$params['description'] = Language::getSessText('Security.CSRFViolationLog', array($referer_url));
		ActivityLogPeer::SaveEvent($params);
	}
}
?>