<?php 
/**
 * Copyright (c) 2006 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
try {
	require_once('initialize.php');
	require_once('lib/session/Session.php');
	require_once('lib/core/Filesystem.php');
	require_once('lib/core/Project.php');
	require_once('lib/exceptions/PermissionDeniedException.php');

	$Section = 'htmleditor';

	$fileid = Validator::sanitizeInt($_GET['file']);

	$c = new Criteria();
	$c->add(FilesystemPeer::SECTION, $Section); // TODO -> this section name should be stored somewhere as a constant
	$c->add(FilesystemPeer::FILESYSTEM_ID, $fileid);

	$img = FilesystemPeer::doSelectOne($c);
	if (is_object($img)) {

		// See if the image is linked to a project. In that case, check permissions
		$project_id = $img->getRefId();
		if ($project_id) {
			$session = Session::getSession();

			// user may already be logged in
			$ip = $_SERVER['REMOTE_ADDR'];
			if ($session->checkUserSession()) {
				try {
					$session->setup($ip);
				} catch (Exception $e) {}
			}

			$project = ProjectPeer::retrieveByPk($project_id);
			if ($project && !$project->userCanRead()) {
				throw new PermissionDeniedException();
			}
		}

		header('Content-Type: ' . $img->getFileType());
		$img->dumpFileData();
		exit();
	}
	/*The following stuff is ripped from gf.php */
} catch (PropelException $pe) {
	echo '<strong>Unhandled PropelException</strong> ' . $pe->getMessage();
	echo '<br /><pre>';
	echo $pe->getTraceAsString();
	echo '</pre>';
	die();
} catch (NotLoggedInException $nlie) {
	header("Location: /gf/account/?action=Login&redirect=".urlencode($_SERVER['REQUEST_URI']));
	exit;
} catch (PermissionDeniedException $pde) {
	if (!Session::getSession()->isLoggedIn()) {
		header("Location: /gf/account/?action=Login&redirect=".urlencode($_SERVER['REQUEST_URI']));
		exit;
	} else {
		$params=array();
		$params['title']='Permission Denied';
		require_once('html/SiteTemplate.php');
		$template = new SiteTemplate();
		$template->header($params);
		echo '<h1>Permission Denied</h1>';
		$template->footer();
	}
} catch (InvalidDataException $e) {
		$params=array();
		$params['title'] = 'Invalid data';
		require_once('html/SiteTemplate.php');
		$template = new SiteTemplate();
		$template->header($params);
		echo '<h1>Invalid data</h1>';
		
		$errors = $e->getErrors();
		$feedback = implode('<br /> *', $errors);
		echo '<!--'.$feedback.'-->';
		
		$template->footer();
} catch (InvalidProjectException $e) {
	$params=array();
	$params['title']='Invalid project';
	require_once('html/SiteTemplate.php');
	$template = new SiteTemplate();
	$template->header($params);
	echo '<h1>Invalid project</h1>';
	//echo $e->getMessage();
	$template->footer();
	exit;
} catch (InvalidLicenseException $e) {
	$params=array();
	$params['title']='Exception';
	require_once('html/SiteTemplate.php');
	$template = new SiteTemplate();
	$template->header($params);
	echo $e->getMessage();
	$template->footer();
	exit;
} catch (Exception $e) {
	$params=array();
	$params['title']='Exception';
	require_once('html/SiteTemplate.php');
	$template = new SiteTemplate();
	$template->header($params);
	echo $e->getMessage();
	$template->footer();
	exit;
} 
?>
