function FileUploadWidgetInit(container_id, widget_id, server_url, extra_parameters, enable_drag_drop) {
    if (enable_drag_drop === undefined) {
        enable_drag_drop = true;
    }

    var str_zone = undefined;
    if (enable_drag_drop) {
        str_zone = $(document);
    }

	this.container_id = container_id;
	this.widget_id = widget_id;
	this.server_url = server_url;

	if (!extra_parameters) {
		extra_parameters = [];
	} else if (typeof extra_parameters == 'string') {
		extra_parameters = [extra_parameters];
	}

	this.extra_parameters = extra_parameters;
	
	$(function () {
		'use strict';

		//Create data object for the template
		var tmpl_data = {widget_id : widget_id, server_url : server_url, extra_parameters : extra_parameters};

		//Render the widget template
		var widget_html = new EJS({url: '/upload_widget.ejs'}).render(tmpl_data);

		//Set the rendered widget to the DIV container
		$('#' + container_id).html(widget_html);
	
		// Initialize the jQuery File Upload widget:
        $('#' + widget_id).fileupload({dropZone : str_zone});
	});
}

function FileUploadWidgetSetParameter(parameter_id, parameter_value) {
	$('#' + parameter_id).attr('value', parameter_value);
}

function setupUploadWidget() {
	// Set up the upload widget
	var widget = $('.upload_widget');
	if (widget && widget.length == 0) {
		return;
	}

	if(widget){
		widget.append('<strong>' + window.locale['fileupload']['attachedFilesTitle'] + '</strong><br /><br />\
			<table width="100%" class="tabular" cellpadding="5" cellspacing="1" summary=""><tbody>\
				<tr class="th">\
					<th width="18px"></th>\
					<th>File name</th>\
					<th>File type</th>\
					<th>File size</th>\
					<th>Uploaded on</th>\
					<th>Posted by</th>\
					<th>Delete</th>\
				</tr>\
				</tbody>\
			</table>');
	}

	if (_ENABLE_UPLOADS) {
			widget.append('<br /><input type="button" onClick="javascript:showUpload(); return false;" value="' + window.locale['fileupload']['attachFiles'] + '" />');
		if (BrowserDetect.browser.toLowerCase() != 'explorer') {
			widget.append('<br /><br /><div class="infobox">' + window.locale['fileupload']['uploadHint'] + '</div>');
		} else {
			widget.append('<br /><br /><div class="infobox">' + window.locale['fileupload']['uploadHint2'] + '</div>');
		}
	}

	// The _UPLOAD_UPLOADED_FILES array is created by the UploadHandler::loadJSUploadedFiles method.
	var hasFiles = false;
	if (typeof _UPLOAD_UPLOADED_FILES !== 'undefined') {
		for (idx in _UPLOAD_UPLOADED_FILES) {
			var file = _UPLOAD_UPLOADED_FILES[idx];
			appendUploadedFile(widget, file);
			hasFiles = true;
		}
	}

	if (widget && !hasFiles ) {
		// No files were attached
		widget.find('tr:last').after('<tr class="nofiles"><td colspan="5">' + window.locale['fileupload']['noFiles'] + '</td></tr>');
	}

	// Append the variable that sends the hash of the upload widget to the UploadHandler.
	if (typeof _FILE_UPLOAD_FORM_HASH !== 'undefined') {
		var form = widget.parents('form');
		if (form.find('#_file_upload_form_hash').length == 0) {
			form.append('<input type="hidden" name="_file_upload_form_hash" id="_file_upload_form_hash" value="' + _FILE_UPLOAD_FORM_HASH + '" />');
		}
	}
}

$(document).ready(function() {
	setupUploadWidget();
});

function appendUploadedFile(widget, file) {
	var uploadedBy = (typeof file.uploaded_by !== 'undefined') ? file.uploaded_by : '';
	var deleteValue = (typeof file.file_id !== 'undefined' ? file.file_id : file.file_name);
	var file_name = file.file_name;
	if (typeof file.link !== 'undefined') {
		file_name = '<a href="' + file.link + '" target="_blank">' + file_name + '</a>'
		var link_download = '<a href="' + file.link_download + '"><img src="/themes/gforge5/images/download.png" /></a>';
	} else {
		var link_download = '';
	}
	var html ='<tr>\
				<td>' + link_download + '</td>\
				<td>' + file_name + '</td>\
				<td>' + file.file_type + '</td>\
				<td>' + formatFileSize(file.file_size) + '</td>\
				<td>' + file.date + '</td>\
				<td>' + uploadedBy + '</td>\
				<td><input type="checkbox" name="_upload_delete_files[]" value="' + deleteValue + '" /> Delete</td>\
			</tr>';

	widget.find('tr.nofiles').remove();		// Delete the row that says that no files were found

	widget.find('tr:last').after(html);

	widget.find('tr:even').removeClass('l').addClass('r');
	widget.find('tr:odd').removeClass('r').addClass('l');
	
	//Update files count
	if ( $('ul.tabbernav').length != 0 ) {
		var tabsobj = $('ul.tabbernav li:nth-child(2)');
		var el = $(tabsobj).children('a');
		if ($(el).length > 0) {
			var str_html = $(el).html();
	 		var pattern = /\((.+?)\)/g;
	    matches = str_html.match(pattern);
			if (matches != undefined && matches.length > 0) {
				var s = matches[0];
				s = s.replace('(', '');
				s = s.replace(')', '');
				var item_count = parseInt(s) + 1;
				str_html = str_html.replace(pattern, '(' + item_count + ')');
				$(el).html(str_html);
			}
		}
	}
}

function formatFileSize(bytes) {
	if (typeof bytes !== 'number') {
		return '';
	}
	if (bytes >= 1000000000) {
		return (bytes / 1000000000).toFixed(0) + ' GB';
	}
	if (bytes >= 1000000) {
		return (bytes / 1000000).toFixed(0) + ' MB';
	}
	if (bytes > 1000) {
		return (bytes / 1000).toFixed(0) + ' KB';
	}
	return bytes + ' B';
}

function handleFileUpload(file) {
	var widget = $('.upload_widget');
	appendUploadedFile(widget, file);
}

/**
 * Shows the upload widget popup
 */
function showUpload() {
$("#fileupload_wrapper").dialog({
	resizable: false,
	height:360,
	width: 530,
	title: "File upload",
	modal: false,
	buttons: {
		close: function() {
			$( this ).dialog( "close" );
		}
	}
});
}
