/*! Copyright 2012, Ben Lin (http://dreamerslab.com/)

 * Licensed under the MIT License (LICENSE.txt).
 *
 * Version: 1.0.13
 *
 * Requires: jQuery 1.2.3 ~ 1.8.2
 */
;( function ( $ ){
  $.fn.extend({
    actual : function ( method, options ){
      // check if the jQuery method exist
      if( !this[ method ]){
        throw '$.actual => The jQuery method "' + method + '" you called does not exist';
      }

      var defaults = {
        absolute      : false,
        clone         : false,
        includeMargin : false
      };

      var configs = $.extend( defaults, options );

      var $target = this.eq( 0 );
      var fix, restore;

      if( configs.clone === true ){
        fix = function (){
          var style = 'position: absolute !important; top: -1000 !important; ';

          // this is useful with css3pie
          $target = $target.
            clone().
            attr( 'style', style ).
            appendTo( 'body' );
        };

        restore = function (){
          // remove DOM element after getting the width
          $target.remove();
        };
      }else{
        var tmp   = [];
        var style = '';
        var $hidden;

        fix = function (){
          // get all hidden parents
          $hidden = $target.
            parents().
            andSelf().
            filter( ':hidden' );

          style += 'visibility: hidden !important; display: block !important; ';

          if( configs.absolute === true ) style += 'position: absolute !important; ';

          // save the origin style props
          // set the hidden el css to be got the actual value later
          $hidden.each( function (){
            var $this = $( this );

            // Save original style. If no style was set, attr() returns undefined
            tmp.push( $this.attr( 'style' ));
            $this.attr( 'style', style );
          });
        };

        restore = function (){
          // restore origin style values
          $hidden.each( function ( i ){
            var $this = $( this );
            var _tmp  = tmp[ i ];

            if( _tmp === undefined ){
              $this.removeAttr( 'style' );
            }else{
              $this.attr( 'style', _tmp );
            }
          });
        };
      }

      fix();
      // get the actual value with user specific methed
      // it can be 'width', 'height', 'outerWidth', 'innerWidth'... etc
      // configs.includeMargin only works for 'outerWidth' and 'outerHeight'
      var actual = /(outer)/g.test( method ) ?
        $target[ method ]( configs.includeMargin ) :
        $target[ method ]();

      restore();
      // IMPORTANT, this plugin only return the value of the first element
      return actual;
    }
  });
})( jQuery );



//START Ellipsis JQuery Plugin
(function($) {
	$.fn.ellipsis = function(lines, enableUpdating, moreText, lessText) {
		return $(this).each(function() {

			var el = $(this);

			if (el.css("overflow") == "hidden") {
 				var obj_div = $(el).siblings('.chat-autolink-description-bkp'); 
				var originalText = '';

				if ( $.trim( $(obj_div).html() ) != null && $.trim( $(obj_div).html() ) != "") {
					originalText = $(obj_div).html();
				} else {
					originalText = el.html();
				}

				$(obj_div).html(originalText);
				var availWidth = el.width();
				//var availHeight = el.height();
				var availHeight = 320;

				var MoreLessTag;
				if (moreText) {
					enableUpdating = false;
					var readmore_url = el.attr('readmore-url');
					MoreLessTag = " ...&nbsp;<br /><br /><strong><a class='MoreLessTag' href=''>" + moreText + "</a></strong>";
				}
				else MoreLessTag = "";
 
				var t = $(this.cloneNode(true))
					.hide()
					.css({
						'position': 'absolute',
						'overflow': 'visible',
						'max-width': 'none',
						'max-height': 'none'
					});

					if (lines) t.css("height", "auto").width(availWidth);
					else t.css("width", "auto");
					el.after(t);
					t.append(" <br /><br /><strong><a class='MoreLessTag' href='#' >" + lessText + "</a></strong>");

					var fullHeight = t.actual('height');
					var avail = (lines) ? availHeight : availWidth;
					var test = (lines) ? t.actual('height') : t.width();
					var foundMin = false, foundMax = false;

					if (test > avail) {
						//Binary search style trimming of the temp element to find its optimal size
						var min = 0;
						var max = originalText.length;
						while (min <= max) {
							var trimLocation = (min + max) / 2;
							var text = originalText.substr(0, trimLocation);
							t.html(text + "&hellip;" + MoreLessTag);

							test = (lines) ? t.actual('height') : t.width();
							if (test > avail) {
								if (foundMax)
									foundMin = true;

								max = trimLocation - 1;
								if (min > max) {
									//If we would be ending decrement the min and regenerate the text so we don't end with a
									//slightly larger text than there is space for
									trimLocation = (max + max - 2) / 2;
									text = originalText.substr(0, trimLocation);
									t.html(text + "&hellip;" + MoreLessTag);
									break;
								}
							}
							else if (test < avail) {
								min = trimLocation + 1;
							}
							else {
								if (foundMin && foundMax && ((max - min) / max < .2))
									break;
								foundMax = true;
								min = trimLocation + 1;
							}
						}
						el.html(t.html());
					}
 
				
				t.remove();

				if (moreText) {
					var moreless_obj = $(el).find('.MoreLessTag');
					$(moreless_obj).click( function(event) {
						event.preventDefault();
						var target = $(event.target);
						var parent_obj = $(target).parents('.chat-autolink-description');
						var bkp_obj_div = $(parent_obj).siblings('.chat-autolink-description-bkp');
						parent_obj.html( $(bkp_obj_div).html() );
						parent_obj.animate({ "height": fullHeight+70 }, "normal", null, function() {});

						parent_obj.append("<br /><br /><strong><a class='MoreLessTag' href='#' >" + lessText + "</a></strong>");
						var less_obj = $(parent_obj).find('.MoreLessTag');

						$(less_obj).click( function(event) {
							event.preventDefault();
							var target = $(event.target);
							var parent_obj = $(target).parents('.chat-autolink-description');
							var obj_div = $(parent_obj).siblings('.chat-autolink-description-bkp');

							$(parent_obj).html( $(obj_div).html() );
							$(parent_obj).animate({ "height": availHeight+30 }, "normal", null, function() {
								$(parent_obj).ellipsis(true, false, moreText, lessText);
							});
						});
					});

				} else {
					var replaceTags = new RegExp(/<\/?[^>]+>/gi);
					el.attr("alt", originalText.replace(replaceTags, ''));
					el.attr("title", originalText.replace(replaceTags, ''));
				}

				if (enableUpdating == true) {
					var oldW = el.width();
					var oldH = el.height();
					el.one("resize", function() {
						if (el.width() != oldW || (lines && el.height != oldH)) {
							el.html(originalText);
							el.ellipsis(lines, enableUpdating, moreText, lessText);
						}
					});
				}
			}
 
		});
	};
})(jQuery); 
//END Ellipsis JQuery Plugin


