<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('html/SiteTemplate.php');
require_once('exceptions/NotLoggedInException.php');

class MyTemplate extends SiteTemplate {

	function header($params) {		
		if (!Session::getSession()->isLoggedIn()) {
			throw new NotLoggedInException();
		}
		if (!isset($params['title'])) {
			Throw new exception('Missing title in my template');
		}
		$Lang = Session::getSession()->getLanguage();
		$params['title']=$Lang->getText('Tabs.MyPage').' > '.$params['title'];
		$params['selectedtoptab']='my';
		$this->addBreadCrumb('<a href="/gf/my/">'.$Lang->getText('Tabs.MyPage').'</a>');
		return parent::header($params);
		
	}
	
	/**
	 *      getPluginNavElements - this overrides parent class and adds its own
	 *      plugin listener
	 *      @return PluginResultSet
	 */
	function getPluginNavElements($params) {
		$prs = parent::getPluginNavElements($params);
		$tprs = ActionManager::Notify('GET_MY_NAV_ELEMENTS',$params);
		foreach ($tprs as $pr) {
			$prs->addResult($pr);
		}
		return $prs;
	}

	/**
	 * getTemplateNavElements - this function can be overridden
	 * by subclasses so you can add navigation entries
	 * @return array - complex array of data.
	 */
	function getTemplateNavElements($params) {
		if (isset($params['selectedbottomtab']) && $params['selectedbottomtab']=='myaccount') {
			$pine = $this->getPluginNavElements($params);
			$ret = array();
			foreach ($pine as $result) {
				$ret[]=$result->getResult();
			}
			$ret[] = array(
				'title'=> Language::getSessText('My.MyStuff'),
				'elements'=> array(
				array('link'=>'/gf/account/?action=UserPasswordEdit',
					'description'=> Language::getSessText('My.ChangePass')),
				array('link'=>'/gf/my/?action=UserEmailEdit',
					'description'=> Language::getSessText('My.ChangeEmail')),
				array('link'=>'/gf/my/?action=UserPluginsEdit',
					'description'=> Language::getSessText('My.Plugins')),
				array('link'=>'/gf/my/?action=UserSSHKeysEdit',
					'description'=> Language::getSessText('My.EditSSHKeys')),
				array('link' => GFUrl::getUrl()->baseURL('ProjectAdd', '', 'register'),
					'description'=> Language::getSessText('My.RegProject')),
                array('link' => GFUrl::getUrl()->baseURL('DisableAccount', '', 'account'),
                        'description'=> Language::getSessText('My.DisableAccount'))
					)
				);
			if(GFConf::get('system.usesLdap')  && !GFConf::get('ldap.dbLoginPrefered')){
				unset($ret[0]['elements'][0]);
				unset($ret[0]['elements'][1]);
			}

			return $ret;
		}
	}
	function secondTabs($params) {
		if(array_key_exists('user', $params)){
			if (array_key_exists('selectedbottomtab', $params)) {
				$selectedTab = $params['selectedbottomtab'];
			} else {
				$selectedTab = false;
			}
			
			$tabs = array();
			// "My page" tab
			$tabs[] = array(
					'text'	 =>	Language::getSessText('Tabs.MyToDo'),
					'id'	 =>	'my',
					'action' => 	'/gf/my/'
				);
			// "My Account" tab
			$tabs[] = array(
					'text'	 =>	Language::getSessText('My.MyAccount'),
					'id'	 =>	'myaccount',
					'action' => 	'/gf/account/'
				);
			// "My Reports" tab
			$tabs[] = array(
					'text'	 =>	Language::getSessText('Tabs.MyReports'),
					'id'	 =>	'myreports',
					'action' => 	'/gf/my/reports/'
				);
			// "Message Wall" tab
			$tabs[] = array(
					'text'	 =>	Language::getSessText('Tabs.MyMessageWall'),
					'id'	 =>	'messagewall',
					'action' => 	'/gf/my/messagewall/'
				);
			// "Monitors" tab
			$tabs[] = array(
					'text'	 =>	Language::getSessText('Tabs.Monitors'),
					'id'	 =>	'monitor',
					'action' => 	'/gf/my/monitors/'
				);
			// "Dashboard" tab
			$tabs[] = array(
					'text'	 =>	Language::getSessText('Tabs.Dashboard'),
					'id'	 =>	'dashboard',
					'action' => 	'/gf/my/dashboard/'
				);
			// "My Activity" tab
			$tabs[] = array(
					'text'	 =>	Language::getSessText('Tabs.MyActivity'),
					'id'	 =>	'activity',
					'action' => 	'/gf/my/activity/'
				);
			// "My Gantt View" tab
			$tabs[] = array(
					'text'	 =>	Language::getSessText('Tabs.MyGanttView'),
					'id'	 =>	'myganttview',
					'action' => 	'/gf/my/gantt/'
				);
				
				
			$prs = ActionManager::notify("BUILD_MYPAGE_TABS",$params);
			foreach ($prs as $result) {
				$tabs[] = $result->getResult();
			}
			
			echo $this->renderTabs($tabs, $selectedTab, true, $params);
		}
	}

	function generateRSSLinks() {
		$out = array();
                $url = GFUrl::getUrl();

		$out['myforums']['title'] = GFConf::get('system.name').' - Monitored Forums';
		$out['myforums']['url'] = $url->getFullURL($url->baseURL('Rss', '', '', array('section' => 'myforums')));
		return $out;
	}

}

?>
