<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once("session/Login.interface.php");
require_once("exceptions/LoginException.php");
require_once("exceptions/PasswordExpiredException.php");
require_once("core/User.php");
require_once("core/UserPeer.php");
require_once("core/UserSession.php");

class DatabaseLogin implements Login {
	/**
	 * @return UserSession
	 */
	public function doLogin($username, $password, $checkStatus=true) {
		$username=strtolower($username);
		$c = new Criteria(UserPeer::DATABASE_NAME);
		$c->add(UserPeer::UNIX_NAME, $username);
		$c->add(UserPeer::PASSWORD_MD5, md5($password));
		if ($checkStatus) {
			$c->add(UserPeer::STATUS, User::STATUS_ACTIVE);
		}
		$user = UserPeer::doSelectOne($c);
		
		if (is_null($user)) {
			$c = new Criteria(UserPeer::DATABASE_NAME);
			$c->add(UserPeer::UNIX_NAME, $username);
			if ($checkStatus) {
				$c->add(UserPeer::STATUS, User::STATUS_ACTIVE);
			}
			$user = UserPeer::doSelectOne($c);
			if (is_null($user)) {
				throw new LoginException("Invalid username and/or password");
			} else {
				if ($user->isLocked()) {
					throw new LoginException("Your account is locked, please contact a site admin");
				} else {
					$user->setLoginAttempts($user->getLoginAttempts() + 1);
					throw new LoginException("Invalid username and/or password");
				}
			}
		} else {
			if (GFConf::get('system.user_password_aging') > 0) {
				//If user_password_aging config option is greater than 0, then password engine is activated
				$user_timestamp = strtotime($user->getPasswordChangeDate()) + (GFConf::get('system.user_password_aging') * 86400);
				$curr_timestamp = time();
				if ($user_timestamp < $curr_timestamp) {
					throw new PasswordExpiredException("The password has expired");
				}
			} else if ($user->isLocked()) {
				throw new LoginException("Your account is locked, please contact a site admin");
			}
		}
		$c = new Criteria();
		$c->add(UserSessionPeer::SESSION_HASH, session_id());
		$session = UserSessionPeer::doSelectOne($c);
		if (is_null($session)) {
			$session = new UserSession();
		}
		$session->setUser($user);
		return $session;
	}
	
	public function doLogout() {
	}
}
?>
