<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez ruben@gforgegroup.com
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('GFConf.php');
require_once('plugins/build/lib/XMLLogHelper.php');

class LogFactory {

	function __construct() {
	}

	public static function getLog() {
		$Project = GFUrl::getUrl()->getProject();
		$unixName = $Project->getUnixName();
		$pathToCC = GFConf::get('build.pathToCC');
		$pathToProject = $pathToCC.'/'.GFConf::get('build.projectsDir').'/'.$unixName;
		$pathToLog = $pathToCC.'/'.GFConf::get('build.logsDir').'/'.$unixName;
		$configDir = GFConf::get('build.configDir');

		$log = self::readPropertyFile($pathToProject.'/'.$configDir.'/gforge.properties', 'last_log');
		if ($log == '') {
			$logs = self::getFilesOrder($pathToLog);
			if ($logs !== false && count($logs) > 0) {
				$lastone = count($logs) - 1;
				$log = $pathToLog.'/'.$logs[$lastone];
			}
		}
		return $log;
	}
	
	public static function getPlatformLog($platform_id, $log_id = 0) {
		if ($log_id > 0) {
			$Fs = FilesystemPeer::retrieveByPk($log_id);
		} else {
			$c = new Criteria();
			$c->add(FilesystemPeer::SECTION, CruisecontrolPlatform::SECTION);
			$c->add(FilesystemPeer::REF_ID, $platform_id);
			$c->addDescendingOrderByColumn(FilesystemPeer::FILESYSTEM_ID);
			$Fs = FilesystemPeer::doSelectOne($c);
		}
		if ($Fs && is_object($Fs)) {
			return new XMLLogHelper($Fs->getFileData());
		} else {
			return null;
		}
	}
	
	public static function getAllLogs() {
		$Project = GFUrl::getUrl()->getProject();
		$unixName = $Project->getUnixName();
		$pathToCC = GFConf::get('build.pathToCC');
		$pathToLog = $pathToCC.'/'.GFConf::get('build.logsDir').'/'.$unixName;

		return self::getFilesOrder($pathToLog);
	}
	
	public static function getStatusFromLogs() {
		$Project = GFUrl::getUrl()->getProject();
		$unixName = $Project->getUnixName();
		$pathToCC = GFConf::get('build.pathToCC');
		$pathToLog = $pathToCC.'/'.GFConf::get('build.logsDir').'/'.$unixName;
		$text = '';
		$text = @file_get_contents($pathToLog.'/status.txt');
		return ucfirst(trim($text));
	}
	
	public static function getTicketId() {
		$Project = GFUrl::getUrl()->getProject();
		$unixName = $Project->getUnixName();
		$pathToCC = GFConf::get('build.pathToCC');
		$pathToProject = $pathToCC.'/'.GFConf::get('build.projectsDir').'/'.$unixName;
		$configDir = GFConf::get('build.configDir');
		$itemId = self::readPropertyFile($pathToProject.'/'.$configDir.'/gforge.properties', 'tracker_item_id');
		return $itemId;
	}
	
	private function readPropertyFile($path, $property) {
		$lines = @file($path);
		if (is_array($lines)) {
			$property = strtolower($property) . '=';
			$property_len = strlen($property);
			foreach($lines as $line) {
				$line = trim($line);
				if (strlen($line) > $property_len) {
					$substr = strtolower(substr($line, 0, $property_len));
					if ($substr == $property) {
						$value_len = strlen($line) - $property_len;
						return (substr($line, -$value_len, $value_len));
					}
				}
			}
		}
		return '';
	}
	
	private function getFilesOrder($path) {
		$files = false;
		$dir = @opendir($path);
		if ($dir !== false) {
			$files = array();
			while (($file = @readdir($dir)) !== false) {
				if (!is_dir($file) && $file != '.' && $file != '..') {
					if (strlen($file) > 4 && strtolower(substr($file, -4, 4)) == '.xml') {
						$files[] = $file;
					}
			    }
			}
			sort($files);
			natcasesort($files);
		}
		return $files;
	}
}
?>