<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez ruben@gforgegroup.com
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

class XMLLogHelper {
	private $log;
	private $dateformat;
	
	function __construct($file, $dateformat = 'Y-m-d H:i:s') {
		$log = new DOMDocument();
		if (is_file($file)) {
			$log->load($file);
		} else {
			$log->loadXML($file);
		}
		$this->log = $log;
		$this->dateformat = $dateformat;
	}

    /**
     *  @return the build log name
     */
    public function getLogFileName() {
        return $this->getCruiseControlInfoProperty('logfile');
    }

    /**
     *  @return the label for this build
     */
    public function getLabel() {
        return $this->getCruiseControlInfoProperty('label');
    }
    
    private function getTimeFromString($date) {
    	if (strlen($date) == 14) {
    		$year = substr($date, 0, 4);
			$mon = substr($date, 4, 2);
			$day = substr($date, 6, 2);
			$hour = substr($date, 8, 2);
			$min = substr($date, 10, 2);
			$sec = substr($date, 12, 2);
			return mktime($hour, $min, $sec, $mon, $day, $year);
    	} else {
    		return 0;
    	}
    }

    public function getBuildTimestamp($getTime = 'no') {
    	$time = $this->getTimeFromString($this->getCruiseControlInfoProperty('cctimestamp'));
    	if ($getTime == null) {
    		return $time;
    	} else {
        	return date($this->dateformat, $time);
        }
    }

    /**
     *  @return true if the previous build was successful, false if it was not
     */
    public function wasPreviousBuildSuccessful() {
        return ($this->getCruiseControlInfoProperty('lastbuildsuccessful') == 'true');
    }

    /**
     *  @return errors if the build was necessary
     */
    public function getBuildInfo() {
    	$error = '';
    	$new = true;
    	$build = $this->log->getElementsByTagName('build');
    	if ($build->length > 0) {
			$error = $build->item(0)->getAttribute('error') . "\n";
			$messagesAux = $build->item(0)->getElementsByTagName('message');
			for ($i=0;$i< $messagesAux->length;$i++) {
				if ($messagesAux->item($i)->getAttribute('priority') == 'warn' ||
					$messagesAux->item($i)->getAttribute('priority') == 'error') {
					if ($new) {
						$error .= "\nDETAILS:\n";
						$new = false;
					}
					$error .= $messagesAux->item($i)->nodeValue . "\n";
				}
			}
			// If can't find details we'll use the regular output
			if ($new) {
				for ($i=0;$i< $messagesAux->length;$i++) {
					if ($new) {
						$error .= "\nDETAILS:\n";
						$new = false;
					}
					$error .= $messagesAux->item($i)->nodeValue . "\n";
				}				
			}
    	}
        return $error;
    }

    /**
     *  @return true if the build was necessary
     */
    public function wasBuildNecessary() {
    	$build = $this->log->getElementsByTagName('build');
    	if ($build->length > 0 && $build->item(0)->getAttribute('error') != '') {
			return ($build->item(0)->getAttribute('error') != 'No Build Necessary');

    	}
        return true;
    }

    public function getBuildParticipants() {
        $results = array();
        $modif = $this->log->getElementsByTagName('modifications');
        if ($this->isP4Modifications()) {
			$changeList = $modif->item(0)->getElementsByTagName('changelist');
			foreach($changeList as $changeElement) {
				$val = $changeElement->getAttribute('user');
				if ($val != null && strlen($val) > 0) {
					$results[$val] = true;
				}
			}
        } else {
        	$modifList = $modif->item(0)->getElementsByTagName('modification');
			foreach($modifList as $modifElement) {
				$val = $modifElement->getElementsByTagName('user');
				if ($val != null && ($val->length > 0) && strlen($val->item(0)->nodeValue) > 0) {
					$results[$val->item(0)->nodeValue] = true;
				}
			}
        }
        return $results;
    }

    private function isP4Modifications() {
    	$modif = $this->log->getElementsByTagName('modifications');
    	if ($modif->length > 0) {
    		$changelst = $modif->item(0)->getElementsByTagName('changelist');
    		if ($changelst != null && ($changelst->length > 0)) {
    			return true;
    		}
    	}
    	return false;
    }

    /**
     *  @return project name as defined in the ant build file
     */
    public function getProjectName() {
        return $this->getCruiseControlInfoProperty('projectname');
    }

    /**
     *  @return true if the build was successful, false otherwise
     */
    public function isBuildSuccessful() {
    	$build = $this->log->getElementsByTagName('build');
    	if ($build->length > 0) {
			if ($build->item(0)->getAttribute('error') == '') {
				return true;
    		}
    	}
    	return false;
    }

    public function isBuildFix() {
        return (!$this->wasPreviousBuildSuccessful() && $this->isBuildSuccessful());
    }

    public function getCruiseControlInfoProperty($propertyName) {
    	$info = $this->log->getElementsByTagName('info');
    	if ($info->length > 0) {
    		$properties = $info->item(0)->getElementsByTagName('property');
    		foreach($properties as $property) {
    			if ($property->getAttribute('name') == $propertyName) {
    				return $property->getAttribute('value');
    			}
    		}
    	}
    	return '';
    }
}
?>