<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez ruben@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('lib/html/ProjectTemplate.php');
require_once('lib/plugin/PluginResult.php');

class BuildTemplate extends ProjectTemplate {

	/**
	 * Header - this subclasses header and just passes params and data that is specific
	 * to this template
	 * @param array array of parameters
	 */
	function header($params) {
		$params['selectedbottomtab']='cruisecontrol';
		$Lang = Session::getSession()->getLanguage();
		if(!$params['project']->getPermission()->check(BuildPlugin::SECTION, 0, BuildPlugin::PERMLEVEL_READ)) {
			Throw new PermissionDeniedException();
		}
		if (isset($params['title'])) {
			$params['title']=' '.$Lang->getText('Build.StatusPageTitle').' > '.$params['title'];
		} else {
			Throw new exception('Missing title in CruiseControl template');
		}
		$this->addBreadCrumb('<a href="'.GFUrl::getUrl()->projectUrl($params['project'],'','build').'">'.$Lang->getText('Build.TabString').'</a>');	
		return parent::header($params);
	}

	/**
	 *      getPluginNavElements - this overrides parent class and adds its own
	 *      plugin listener
	 *      @return PluginResultSet
	 */
	function getPluginNavElements($params) {
		$prs = parent::getPluginNavElements($params);
		$tprs = ActionManager::Notify('GET_CRUISECONTROL_NAV_ELEMENTS',$params);
		foreach ($tprs as $pr) {
			$prs->addResult($pr);
		}
		return $prs;
	}

	/**
	 * getTemplateNavElements - this function can be overridden
	 * by subclasses so you can add navigation entries
	 * @return array - complex array of data.
	 */
	function getTemplateNavElements($params) {
		$ret = array();
		$elements = array();

		$pine = $this->getPluginNavElements($params);
		foreach ($pine as $result) {
			$ret[]=$result->getResult();
		}

		if (!isset($params['project'])) {
			echo 'missing project param';
		} else {
			if ($params['project']->getPermission()->check(BuildPlugin::SECTION, 0, BuildPlugin::PERMLEVEL_ADMIN)) {
					$elements[] = array('link'=>'/gf/project/'.$params['project']->getUnixName().'/build/admin/',
										'description'=>'Admin');
			}
		}
						
		if (count($elements) > 0) {
			$ret[] = array(
				'title' => 'Build',
				'elements'=> $elements
			);
		}

		return $ret;
	}
}

?>