<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a Cruisecontrol and saves it to the 
// database
//
require_once('plugins/build/lib/Cruisecontrol.php');
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$sess = Session::getSession();
$Lang = Language::getSessLang();
$Project = GFUrl::getUrl()->getProject();

if (!$Project->getPermission()->check(BuildPlugin::SECTION,0,BuildPlugin::PERMLEVEL_ADMIN)) {
	throw new PermissionDeniedException();
}

$errors = array();
$build_name = (isset($_POST['build_name']) && !empty($_POST['build_name'])) ? Validator::sanitizeString($_POST['build_name']) : false;
$build_desc = (isset($_POST['build_desc']) && !empty($_POST['build_desc'])) ? Validator::sanitizeString($_POST['build_desc']) : false;
$user_id = (isset($_POST['user_id']) && !empty($_POST['user_id'])) ? Validator::sanitizeInt($_POST['user_id']) : false;
$tracker = (isset($_POST['tracker']) && !empty($_POST['tracker'])) ? Validator::sanitizeInt($_POST['tracker']) : false;

if (!$tracker || $tracker == 100) {
	$errors[] = $Lang->getText('Build.InvalidTracker');
}
if (!$user_id) {
	$errors[] = $Lang->getText('Build.InvalidSubmitter');
}
if (!$build_name) {
	$errors[] = $Lang->getText('Build.EmptyBuildName');
}
if (!$build_desc) {
	$errors[] = $Lang->getText('Build.EmptyBuildDescription');
}

$cc_id = Validator::sanitizeInt($_POST['id']);
$Cruisecontrol = CruisecontrolPeer::retrieveByPk($cc_id);
if ($Cruisecontrol == null || !is_object($Cruisecontrol)) {
	$errors[] = $Lang->getText('Build.InvalidObject');
}

if (count($errors) == 0) {
	try {
		$Cruisecontrol->setProjectId( $Project->getProjectId() );
		$Cruisecontrol->setBuildName( $build_name );
		$Cruisecontrol->setBuildDescription( $build_desc );
		$Cruisecontrol->setSubmitterId( $user_id );
		
		$Cruisecontrol->save();

		// Edit asignee data
		$ass_objs = $Cruisecontrol->getAssignees();
		foreach($ass_objs as $obj) {
			$obj->delete();
		}
		$assignees = (isset($_POST['assigned_to']) ? $_POST['assigned_to'] : array());
		foreach($assignees as $ass) {
			$ass = Validator::sanitizeInt($ass);
			if ($ass != 100) {
				$assignee = new CruisecontrolAssignee();
				$assignee->setCruisecontrolId($cc_id);
				$assignee->setAssignee($ass);
				$assignee->save();
			}
		}

		// Edit publisher data
		$pub_objs = $Cruisecontrol->getPublisher();
		if (is_object($pub_objs)) {
			$pub_objs->delete();
		}

		$publisher = new CruisecontrolPublisher();
		$publisher->setCruisecontrolId($cc_id);
		$publisher->setTrackerId($tracker);
		$publisher->save();
		
		GFUrl::redirect(GFUrl::getUrl()->currentBaseURL('BuildBrowse'), array('feedback' => $Lang->getText('Build.EditFeedback')));
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());		// attach object errors to validation errors
		$feedback = '* '.implode('<br />* ', $errors);
		include('BuildEdit.php');
	}
} else {
	$feedback = '* '.implode('<br />* ', $errors);
	include('BuildEdit.php');
}

?>
