<?php

require_once('system/Cronjob.php');
require_once('plugins/calendar/lib/CalendarEventReminder.php');
require_once('plugins/calendar/lib/CalendarEventReminderPeer.php');
require_once('plugins/calendar/lib/CalendarEventNotification.php');

class QueueEventReminders extends Cronjob{
	const LOCK_FILE = '/tmp/gforge_queue_event_reminders.LCK';
	
	public function __construct(){
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}

	public function run(){
		$lock_handler = fopen(self::LOCK_FILE, "w+");
		if (!$lock_handler) {
			// Should not happen if we're root... we should be able to open the
			// file even if it's locked
			$this->logCron('Couldn\'t unlock file', array(self::LOCK_FILE));
			$this->commitLog();
			return;
		}
		
		if (!flock($lock_handler, LOCK_EX | LOCK_NB)) {
			// Couldn't acquire exclusive lock, means another process is running
			$this->logCron(Language::getSessText('This cron is already running', array(self::LOCK_FILE)));
			$this->commitLog();
			return;
		}

		$upper_limit = time() + 7.5 * 60;
		$lower_limit = time() - 7.5 * 60;		
		//Get events reminders set to be sent between 15 minutes
		$c = new Criteria();
		$c->add(CalendarEventReminderPeer::SEND_TIME, strftime('%Y-%m-%d %H:%M:%S %z', $upper_limit), Criteria::LESS_EQUAL);
		$c->addAnd(CalendarEventReminderPeer::SEND_TIME, strftime('%Y-%m-%d %H:%M:%S %z', $lower_limit), Criteria::GREATER_EQUAL);
		
		$reminders = CalendarEventReminderPeer::doSelect($c);

		foreach($reminders as $reminder){
			$notification = new CalendarEventNotification($reminder->getCalendarEvent());
			NotificationManager::getManager()->queue($notification);
		}
		
		//Delete reminders already queued and past reminders
		$sql = 'DELETE from calendar_event_reminder WHERE send_time <= \''. strftime('%Y-%m-%d %H:%M:%S %z', $upper_limit) . '\'';
		$con = Propel::getConnection(CalendarEventReminderPeer::DATABASE_NAME);

		$stmt = $con->createStatement();
		$stmt->executeUpdate($sql);

	}


}

$cronjob = new QueueEventReminders();
$cronjob->run();
		

?>
