<?php
require_once 'plugins/calendar/lib/om/BaseCalendar.php';
require_once 'plugins/tracker/lib/Tracker.php';
require_once 'plugins/tracker/lib/TrackerItem.php';
require_once 'plugins/calendar/lib/external/bennu/lib/bennu.inc.php';
require_once 'lib/GFUrl.php';

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.calendar.lib 
 */
class Calendar extends BaseCalendar {

	public function getStartingEventsInTimeFrame($start_date, $end_date){
		//Events starting in this time frame
		$c = new Criteria();
		$c->add(CalendarEventPeer::FROM_DATE, $start_date, Criteria::GREATER_EQUAL);
		$c->add(CalendarEventPeer::FROM_DATE, $end_date, Criteria::LESS_EQUAL);
		$c->addAscendingOrderByColumn(CalendarEventPeer::FROM_DATE);
		return $this->getCalendarEvents($c);
	}

	public function getEndingEventsInTimeFrame($start_date, $end_date){
		$c = new Criteria();
		//Events ending in this time frame
		$c->add(CalendarEventPeer::TO_DATE, $start_date, Criteria::GREATER_EQUAL);
		$c->add(CalendarEventPeer::TO_DATE, $end_date, Criteria::LESS_EQUAL);
		$c->addAscendingOrderByColumn(CalendarEventPeer::TO_DATE);
		return $this->getCalendarEvents($c);

	}

	
	public function getEventsInTimeFrame($start_date, $end_date){
		
		$starting_events = $this->getStartingEventsInTimeFrame($start_date, $end_date);
		$ending_events = $this->getEndingEventsInTimeFrame($start_date, $end_date);
		$starting_assigned_ti = $this->getAssignedTIStartingInTimeFrame($start_date, $end_date);

		$events= array();
		foreach($starting_events as $event){
			$key = 's'. $event->getCalendarEventId(); 
			$events[$key]['event'] = $event;
			$events[$key]['t'] = 's';
		}
		foreach($ending_events as $event){
			$key = 'e' . $event->getCalendarEventId() . strtotime($event->getToDate("%m/%d/%Y %H:%M")); 
			$events[$key]['event'] = $event;
			$events[$key]['t'] = 'e';
		}
			//ksort($events);

		return $events;
	}

	public function getAssignedTIStartingInTimeFrame($start_date, $end_date){
		$user = Session::getSession()->getUser();
		$user_id = $user->getUserId();
		$tis = array();
		$start_date = strftime("%Y-%m-%d", $start_date);
		$end_date = strftime("%Y-%m-%d", $end_date);


		$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
		$sql = "SELECT ti.* FROM tracker_item ti, tracker_item_assignee tia, tracker t
		WHERE tia.assignee = '$user_id'
		AND tia.tracker_item_id = ti.tracker_item_id
		AND t.tracker_id = ti.tracker_id
		AND ti.open_date >= '$start_date'
		AND ti.open_date <= '$end_date'
		AND t.datatype = " . Tracker::DATATYPE_TASK . "
		AND ti.status_id = " . Tracker::STATUS_OPEN;
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		$tis = TrackerItemPeer::populateObjects($rs);

		return $tis;
	}

	public function  getAssignedTIEndingInTimeFrame($start_date, $end_date){
		$user = Session::getSession()->getUser();
		$user_id = $user->getUserId();
		$tis=array();
		$start_date = strftime("%Y-%m-%d", $start_date);
		$end_date = strftime("%Y-%m-%d", $end_date);

		$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
		$sql = "SELECT ti.* FROM tracker_item ti, tracker_item_assignee tia, tracker t
		WHERE tia.assignee = '$user_id'
		AND tia.tracker_item_id = ti.tracker_item_id
		AND t.tracker_id = ti.tracker_id
		AND ti.close_date >= '$start_date'
		AND ti.close_date <= '$end_date'
		AND t.datatype = " . Tracker::DATATYPE_TASK . "
		AND ti.status_id = " . Tracker::STATUS_OPEN;

		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		$tis = TrackerItemPeer::populateObjects($rs);

		return $tis;
	}

	public function getAllAssignedTIInTimeFrame($start_date, $end_date){
		
		$starting_tis = $this->getAssignedTIStartingInTimeFrame($start_date, $end_date);
		$ending_tis = $this->getAssignedTIEndingInTimeFrame($start_date, $end_date);

		$tis = array();
		$i = 0;
		foreach($starting_tis as $ti){
			//Tasks don't have starting or ending times, so we must set one
			$key = 's' . $ti->getTrackerItemId();
			$tis[$key]['ti'] = $ti;
			$tis[$key]['t'] = 'ts';
			//$tis[$key]['sort_index'] = $ti->getOpenDate(null);
			$i++;
		}

		

		$i = 0;
		foreach($ending_tis as $ti){
			$key = 'e' . $ti->getCloseDate("%d/%m/%Y") . $ti->getTrackerItemId();
			$tis[$key]['ti'] = $ti;
			$tis[$key]['t'] = 'te';
			//$tis[$key]['sort_index'] = $ti->getCloseDate(null);
			$i++;
		}

		//usort($tis, array($this, 'cmp'));

		return $tis;
	}

	public function exportToICalendar(){

		$iCal = self::getICalendarObject();
				//Get events
		$events = $this->getCalendarEvents();

		$hostname = GfConf::get('system.hostName');
		foreach($events as $event){
			$iCal->add_component($event->createICalendarEvent());
		}


		//Get tasks

		if($this->getSection() == 'user'){
			$user_id = $this->getRefId();
			$sql = "SELECT ti.* FROM tracker_item ti, tracker_item_assignee tia, tracker t
			WHERE tia.assignee = '$user_id'
			AND tia.tracker_item_id = ti.tracker_item_id
			AND t.tracker_id = ti.tracker_id
			AND t.datatype = " . Tracker::DATATYPE_TASK;
		}
		$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
		$tasks  = TrackerItemPeer::populateObjects($rs);

		foreach($tasks as $task){
			$todo = new iCalendar_todo();
			$todo->add_property('uid', $task->getOpenDate('%Y%m%dT%H%M%S%s%z') . $task->getTrackerItemId() . '@' . $hostname);
			$todo->add_property('due', gmdate('Ymd',$task->getCloseDate(null)), array('value'=>'DATE'));
			$todo->add_property('summary', $task->getSummary());
			$todo->add_property('description', $task->getDetails());
			if($task->getStatusId() == 1){
				$todo->add_property('status', 'COMPLETED');
			} else {
				$todo->add_property('status', 'NEEDS-ACTION');
			}
			$todo->add_property('priority',$task->getPriority());

			$iCal->add_component($todo);
		}

		return $iCal->serialize();

	}

	public function getTimeZone(){

		if($this->getSection == 'user'){
			$tz = User::retrieveByPk($this->getRefId())->getTimezone();
		} else {
			$tz = 'UTC';
		}
		
		return $tz;
	}

	static function getICalendarObject(){
		$iCal = new iCalendar();
		$iCal->add_property('prodid','-//Gforge Group LLC//Gforge 6.0//EN');
		$iCal->add_property('calscale', 'GREGORIAN');
		$iCal->add_property('method', 'PUBLISH');

		return $iCal;
	}

	public function userIsAdmin(){
		if($this->getSection() == 'user'){
			$session = Session::getSession();
			if($session->isLoggedIn() && $this->getRefId() == $session->getUser()->getUserId()){
				return true;
			}
		}

		return false;
	}

	public function userCanCreate(){
		if($this->getSection() == 'user'){
			return $this->userIsAdmin();
		}

		return false;
	}



	private  function cmp($a,$b){
		if($a['sort_index'] < $b['sort_index']){
			return -1;
		} else if($a['sort_index'] == $b['sort_index']){
			return 0; 
		} else {
			return 1;
		}
	}

	

}


?>
