<?php
/**
*   This code shows the project and from here the admin can modify disk quota individually
*
**/
require_once('core/Language.php');
require_once('core/ProjectPeer.php');

require_once('html/templates/SiteAdminTemplate.php');
$theme = new SiteAdminTemplate();
$lang = Language::getSessLang();
$params = array(
        'action'   => isset($action) ? $action : '',
        'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
        'title'    => $lang->getText('Diskquota.Title'),
    );
$theme->header($params);
?>
<script>
$(document).ready(function() {
  $("[name='filter']").change(function(event) {
    var v = $(this).val();
    window.location =
      '<?php echo GFUrl::getUrl()->currentBaseURL('ManageDiskQuota'); ?>&filter=' + v;
  });
});
</script>
<?php

$filters = array(
  'all' => $lang->getText('Diskquota.AllProjects'),
  'overquota' => $lang->getText('Diskquota.ProjectsOverQuota')
);

if (array_key_exists('filter', $_REQUEST) && array_key_exists($_REQUEST['filter'], $filters)) {
  $selected_filter = Validator::sanitizeString($_REQUEST['filter']);
} else {
  $selected_filter = 'all';
}

echo "<strong>Filters: </strong>";
echo HTMLWidget::selectBoxFromAssoc($filters,
                                    'filter',
                                    $selected_filter,
                                    false,
                                    false);

//Columns to be used in pagination sort
$columns = array(
    'project_id'   => $lang->getText('Diskquota.ProjectId'),
    'project_name' => $lang->getText('Diskquota.ProjectName'),
    'unix_name'    => $lang->getText('Diskquota.ProjectUnixName'),
    'disk_used'    => $lang->getText('Diskquota.DiskUsage'),
    'disk_quota'   => $lang->getText('Diskquota.DiskQuota'),
	'percent_used' => $lang->getText('Diskquota.PercentUsed'),
	'setting_time' => $lang->getText('Diskquota.SettingTime'),
    0              => $lang->getText('Diskquota.Actions'),
);

$diskUsedSql = 'COALESCE((SELECT setting_value FROM setting WHERE setting_name = \'disk-used\' AND section = \'project\' AND ref_id = project.project_id)::float, \'0\')';
$diskQuotaSql = 'COALESCE((SELECT setting_value FROM setting WHERE setting_name = \'project-quota\' AND section = \'project\' AND ref_id = project.project_id)::float, \''.GFConf::get('diskquota.projectMaxQuota').'\')';
$timeSql = '(SELECT set_date FROM setting WHERE setting_name = \'disk-used\' AND section = \'project\' AND ref_id = project.project_id)';

$sql = "SELECT
            project_id, project_name, unix_name,
            {$diskUsedSql}::float AS disk_used,
            {$diskQuotaSql}::float AS disk_quota,
            (CASE WHEN {$diskQuotaSql}::float = 0 THEN 0
            ELSE ({$diskUsedSql}::float / {$diskQuotaSql}::float)
            END)
              AS percent_used,
            {$timeSql} AS setting_time
        FROM
            project
        WHERE
            is_template = false";

if ($selected_filter == 'overquota') {
  $sql .= " AND {$diskUsedSql}::float > {$diskQuotaSql}::float AND {$diskQuotaSql} <> '0'";
}
//Get the total number of projects
$total = 0;
$sqlCount = "SELECT COUNT(*) AS count FROM ($sql) foo";
$con = Propel::getConnection(ProjectPeer::DATABASE_NAME);
$stmtCount = $con->createStatement();
$rsCount = $stmtCount->executeQuery($sqlCount, ResultSet::FETCHMODE_ASSOC);
$rsCount->next();
$total = $rsCount->getInt('count');
$paginator = new Paginator('diskquotamng', -1, $total, ProjectPeer::PROJECT_ID, $columns);
$sqlPag = $paginator->getSql();

//Get messages
$sql .= $sqlPag;
$stmt = $con->createStatement();
$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

//Print headers
$theader = $paginator->getSortableTableHeaders($theme);

echo $theme->tableTop($theader);

if($total < 1){
    echo $theme->tableFullRow('<span class="none found">'.$lang->getText('Diskquota.None').'</span>');
} else {
    while ($rs->next()){
        $trow = array();
        $trow[] = $rs->getInt('project_id');
        $trow[] = $rs->getString('project_name');
        $trow[] = $rs->getString('unix_name');
        $trow[] = round($rs->getFloat('disk_used') / (1024*1024), 2).' MB';
        if ($rs->getFloat('disk_quota') != 0) {
          $trow[] = round($rs->getFloat('disk_quota') / (1024*1024), 2).' MB';
        } else {
          $trow[] = $lang->getText("Diskquota.Unlimited");
        }
	    $trow[] = number_format($rs->getString('percent_used')*100, 2).'%';
        $trow[] = date('Y-m-d H:i', strtotime($rs->getString('setting_time')));
        $trow[] = '[<a href="'.GFUrl::getUrl()->baseURL('EditDiskQuota','diskquota','admin', array('id' => $rs->getInt('project_id'))).'">'.$lang->getText('Diskquota.EditDiskQuota').'</a>]
         [<a href="'.GFUrl::getUrl()->baseURL('SaveDiskUsage','diskquota','admin', array('id' => $rs->getInt('project_id'), 'return_to_list' => '1')).'">'.$lang->getText('Diskquota.RefreshUsage').'</a>]';

        echo $theme->tableRow($trow);
    }
}

echo $theme->tableBottom();
if(isset($paginator) && is_object($paginator)) {
   echo '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
}
$theme->footer($params);
