<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Group.class,v 1.94 2005/09/06 18:13:29 danper Exp $
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('notification/GFNotification.php');
require_once("plugin/ActionManager.php");
require_once('core/ProjectPeer.php');
require_once('StringUtils.php');

class DocmanFileNotification implements GFNotification {
	private $docmanFile;
	private $project;
	private $users;
	private $isNew = false;
	private $changes;

	function __construct(DocmanFile $docmanFile, $filename='', $changes = null ) {
		$this->docmanFile = $docmanFile;
		$this->filename = $filename;
		$this->project = $this->docmanFile->getDocmanFolder()->getProject();
		$this->changes=null;
	}
	
	function setNew($isNew) {
		$this->isNew = $isNew;
	}
	
	/**
	 *	getNotificationMessage - construct a string of this message content.
	 *
	 *	@param	User
	 *	@return string
	 */
	function getNotificationMessage(User $User) {
		$url = GFUrl::getUrl();
		
		$body = 'The document #'.$this->docmanFile->getDocmanFileId(). ' - ' . $this->filename . ' was ';
		$body .= ($this->isNew ? 'created' : 'updated').'.';
		$body .= "\n\nFollow this link for more information:\n";
		$body .= $url->getFullUrl($url->projectUrl($this->project, '', 'docman', '', array('file_id' => $this->docmanFile->getDocmanFileId())),true);
		$body .= "\n\nDescription of the document: \n";
		$body .= $this->docmanFile->getDescription();		
		//If it's not a new file, we include the changes field from the latest version
		if (!$this->isNew) {
			$body .= "\n\nChanges: \n";
			$body .= (!is_null($this->changes)) ? $this->changes : $this->docmanFile->getLatestVersion()->getChanges();
		}

		// User is either monitoring this file or the whole docman
		$monitors_file = $this->docmanFile->userIsMonitoring($User);
		$monitors_docman = DocmanPlugin::userIsMonitoring($User, $this->project);
		
		$body .= "\n\n______________________________________________________________________".
				"\n\nYou received this email because you ".
				"\nrequested to be notified when changes were made to this document.".
				"\nIf you don't wish to be notified in the future, please".
				"\nlogin to " . $url->getFullUrl($url->baseUrl('Login', '', 'account'),true) . " and click this link: ";
		if ($monitors_file) {
			$body .= "\n" . $url->getFullUrl($url->getMonitorURL(DocmanFile::SECTION, $this->docmanFile->getPrimaryKey(), $url->projectUrl($this->project, '', 'docman'), false), true);
		} elseif ($monitors_docman) {
			$body .= "\n" . $url->getFullUrl($url->getMonitorURL(DocmanPlugin::SECTION, $this->project->getProjectId(), $url->projectUrl($this->project, '', 'docman'), false), true);
		}
					
		return $body;

	}
	
	/**
	 *	getNotificationSubject - construct a string of this message subject.
	 *
	 *	@param	User
	 *	@return string
	 */
	function getNotificationSubject(User $User) {
		return '['.$this->project->getName().'] '. $this->filename .' - Document '.($this->isNew ? 'created' : 'updated');
	}

	/**
	 *	getUsers - get an array of user objects
	 *	@return array
	 */
	public function getUsers() {
		return $this->docmanFile->getMonitoringUsers();
	}

	/**
	 *	getSection - retrieves the section name of the docman
	 *	@return string
	 */	
	function getSection() {
		return DocmanFile::SECTION;
	}
	
	/**
	 *	getRefId - retrieves the id of the docman file that the notification belongs to
	 *	@return int
	 */	
	function getRefId() {
		return $this->docmanFile->getPrimaryKey();
	}
}

?>
