<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-07-20
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
// The parent class
require_once 'plugins/docman/lib/om/BaseDocmanFolderPeer.php';

// The object class
include_once 'plugins/docman/lib/DocmanFolder.php';

require_once('plugins/docman/lib/DocmanFilePeer.php');
require_once('core/Filesystem.php');

/** 
 * The skeleton for this class was autogenerated by Propel  *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package plugins.docman.lib 
 */
class DocmanFolderPeer extends BaseDocmanFolderPeer {
	private static $_pkCache = array();

	/**
	 * Retrieve a single object by pkey or NULL if not found.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key
	 * @param	bool $useCache	Whether to use caching. This will increase performance if the same object
	 *							is accessed several times
	 * @param	Connection $con	The connection to use
	 * @return	DocmanFolder
	 */
	public static function retrieveByPk($pk, $useCache = true, $con = null) {
		if ($useCache && array_key_exists($pk, DocmanFolderPeer::$_pkCache)) {
			return DocmanFolderPeer::$_pkCache[$pk];
		}
		
		$object = parent::retrieveByPk($pk, $con);
		if ($useCache) {
			DocmanFolderPeer::$_pkCache[$pk] = $object;
		}
		return $object;
	}
	
	/**
	 * Removes an object from the cache. This is usually done after saving.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key of the object to remove
	 */
	public static function invalidate($pk) {
		if (array_key_exists($pk, DocmanFolderPeer::$_pkCache)) {
			unset(DocmanFolderPeer::$_pkCache[$pk]);
		}
	}

	/**
	 * Given an URL path for the folder structure (e.g. /docman/Folder%201/Folder%201.1) returns an
	 * array with the information of each subfolder
	 * @param 	string 	Path to parse
	 * @param 	Project Project object
	 * @param 	bool 	Whether the path includes a file name. If true, will not parse the last component of the path and will return it as a file name instead.
	 */
	public static function parseURLFolderPath($urlPath, Project $project) {
		$urlPath = explode('/', $urlPath);

		// Remove the empty value at the beginning and the end of the array (if neccessary)
		if ($urlPath[0] == '') {
			array_shift($urlPath);
		}
		
		if (count($urlPath) && $urlPath[count($urlPath)-1] == '') {
			array_pop($urlPath);
		}

		// urldecode all the elements in the array
		$urlPath = array_map('urldecode', $urlPath);

		$parent_folder_id = 0;

		$n = count($urlPath);

		$pathInfo = array();
		$fileInfo = array();

		for ($i = 0; $i < $n; $i++) {
			$folderName = $urlPath[$i];
			if (empty($folderName)) {
				continue;
			}

			// Special cases for the first part of the path. I can be 'docman' (which we don't consider) or
			// 'home' (Which is an alias for the 'incoming') folder, for historical reasons.
			if ($folderName == 'docman' && $i <= 1) {
				continue;
			} else if (strtolower($folderName) == 'pending queue') {
				$pathInfo[] = array(
								'folder_name' => $folderName,
								'docman_folder_id' => -1,
								'is_public' => false,
								'parent_folder_id' => 0,
							);				
			} else if ($folderName == 'home' && $i <= 1) {
				// See what the 'incoming' folder is for this project
				$c = new Criteria();
				$c->add(DocmanFolderPeer::FOLDER_NAME, 'incoming');
				$c->add(DocmanFolderPeer::PROJECT_ID, $project->getProjectId());
				$c->add(DocmanFolderPeer::PARENT_FOLDER_ID, 0);
				$folder = DocmanFolderPeer::doSelectOne($c);
				if (!$folder) {
					throw new Exception("'incoming' folder not found in docman");
				}

				$pathInfo[] = array(
								'folder_name' => $folder->getFolderName(),
								'docman_folder_id' => $folder->getDocmanFolderId(),
								'is_public' => $folder->getIsPublic(),
								'parent_folder_id' => $folder->getParentFolderId(),
							);
				// Note that we do NOT update the $parent_folder_id
			} else {
				$c = new Criteria();
				$c->setIgnoreCase(true);
				$c->add(DocmanFolderPeer::FOLDER_NAME, $folderName);
				$c->add(DocmanFolderPeer::PARENT_FOLDER_ID, $parent_folder_id);
				$c->add(DocmanFolderPeer::PROJECT_ID, $project->getProjectId());
				$folder = DocmanFolderPeer::doSelectOne($c);
				if (!$folder && $i < count($urlPath)-1) {
					// The last element of the path MAY be a file name
					throw new Exception("Folder '$folderName' not found in docman");
				} else if (!$folder) {
					// We are at the last element of the path and no matching folder was found. We need to see
					// if it is referring to a file name.
					$filename = $folderName;

					// See if we find the file in the right folder. This is a complicated query, we need to join
					// three tables.
					$c = new Criteria();
					$c->addJoin(DocmanFilePeer::DOCMAN_FILE_ID, DocmanFileFolderPeer::DOCMAN_FILE_ID);
					$c->add(DocmanFileFolderPeer::DOCMAN_FOLDER_ID, $parent_folder_id);
					$c->addJoin(DocmanFilePeer::DOCMAN_FILE_ID, DocmanFileVersionPeer::DOCMAN_FILE_ID);
					$c->add(FilesystemPeer::SECTION, DocmanFileVersion::SECTION);
					$c->addJoin(FilesystemPeer::REF_ID, DocmanFileVersionPeer::DOCMAN_FILE_VERSION_ID);
					$c->add(FilesystemPeer::FILE_NAME, $filename);
					$file = DocmanFilePeer::doSelectOne($c);
					if (!$file) {
						throw new Exception("Cannot find the file or folder '$filename' in docman.");
					}

					$fileInfo = array(
							'filename' => $filename,
							'docman_file_id' => $file->getPrimaryKey(),
						);
				} else {
					$pathInfo[] = array(
										'folder_name' => $folder->getFolderName(),
										'docman_folder_id' => $folder->getDocmanFolderId(),
										'is_public' => $folder->getIsPublic(),
										'parent_folder_id' => $folder->getParentFolderId(),
									);

					$parent_folder_id = $folder->getDocmanFolderId();
				}
			}
		}

		$return = array('path_info' => $pathInfo, 'file_info' => $fileInfo);

		return $return;
	}
	

	/**
	 * Give a project_id, verifies that the incoming folder in the docman plugin exists. If it doesn't, it creates it.
	 * @param 	int	Project Id
	 */
	public static function verifyAndCreateIncoming($project_id) {
		$project = ProjectPeer::retrieveByPk($project_id);
		if (!is_object($project)) {
			return;	
		}
		
		//Creates the default "incoming" folder
		$c = new Criteria();
		$c->add(DocmanFolderPeer::PROJECT_ID,$project_id);
		$c->add(DocmanFolderPeer::FOLDER_NAME,'incoming'); // default master folder
		$c->add(DocmanFolderPeer::PARENT_FOLDER_ID,0);
		$incoming = DocmanFolderPeer::doSelectOne($c);
		if (!is_object($incoming)) {
			$incoming = new DocmanFolder();
			$incoming->setFolderName('incoming');
			$incoming->setProject($project);
			$incoming->save();
		}	
	}

	public static function getRootFolderCount($project_id) {
		$total = 0;
		$c = new Criteria();
		$c->add(DocmanFolderPeer::PROJECT_ID,$project_id);
		$c->add(DocmanFolderPeer::PARENT_FOLDER_ID, 0);
		$c->addSelectColumn(DocmanFolderPeer::COUNT);
		$rs = DocmanFolderPeer::doSelectRS($c);  
  		if ($rs->next()) {
   			return $rs->getInt(1) - 1;
		}
		return 0;
	}

	public static function getUrlFromDocmanFolderId($docmanFolderId) {

		$docmanFolderId = intval($docmanFolderId);
		$ret = null;


		$con = Propel::getConnection();
		$stmt = $con->createStatement();
		$sql = "WITH RECURSIVE docman_folder_path(docman_folder_id, parent_folder_id, project_id, path) AS (
        SELECT docman_folder_id, parent_folder_id, project_id, folder_name::text as path FROM docman_folder AS
        df WHERE docman_folder_id=$docmanFolderId
        UNION ALL
        SELECT df.docman_folder_id, df.parent_folder_id, df.project_id, (df.folder_name || '/' || dfp.path::TEXT) as path
        FROM docman_folder df, docman_folder_path dfp WHERE df.docman_folder_id=dfp.parent_folder_id)
        SELECT path,project_id FROM docman_folder_path WHERE parent_folder_id=0";
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);


		if ($rs->next()) {
			$folder_names = explode('/',$rs->getString('path'));
			foreach ($folder_names as &$folder) {
				$folder = urlencode($folder);
			}

			$ret = '/'.implode('/', $folder_names);
		}
		return $ret;
	}
	
}
?>