<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Daniel Perez	<daniel@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugin/GFPlugin.php');
require_once('plugin/ActionManager.php');
require_once('plugin/PluginResult.php');
require_once('core/Language.php');
require_once('DocmanFolderPeer.php');
require_once('DocmanFileVersionPeer.php');
require_once('lib/GFExportImportUtils.php');
require_once('plugins/docman/lib/DocmanProjectSetting.php');
require_once('plugins/docman/lib/DocmanFileFolder.php');
require_once('plugins/docman/lib/FastJSON.php');

class DocmanPlugin implements GFPlugin {

	protected $project;
	CONST READPOST = 0; //LEGACY - DON"T USE AGAIN
	CONST NOT_ADMIN = 0;
	CONST ADMIN = 1;
	
	CONST SECTION = 'docman';

	/*function __construct($project) {
		$this->project = $project;
	}*/
	function __construct() {
	}	
	
	/**
	 *	this is called when the first Action is called. Action manager 
	 *	gets the listeners for each plugin.
	 */
	public function initialize() {
		$listenTo = array(
			'BUILD_PROJECT_TABS',
	//		'BUILD_PROJECT_SUMMARY',
			'GET_DOCMAN_NAV_ELEMENTS',
			'BUILD_SEARCH_COMBO',
			'DO_SEARCH',
			'GET_PROJECT_ADMIN_NAV_ELEMENTS',
			'PROJECT_APPROVE',
			'BUILD_MYPAGE_ELEMENTS',
			'CHECK_DOWNLOAD_PERMS',
			'GET_ROLE_CHOICES',
			'UPDATE_ROLE_CHOICES',
			'GET_OBSERVER_ROLE_CHOICES',
			'UPDATE_OBSERVER_SETTINGS',
			'RENDER_ASSOCIATION_LINK',
			'GET_ASSOCIATED_ITEM_INFO',
			'ASSOCIATION_SECTION',
			'ASSOCIATION_CHECK',
			'GET_DISK_USAGE',
			'GET_DISK_QUOTA_USAGE',
			'MONITOR_CHECK_PERMISSIONS',
			'MONITOR_GET_TEMPLATE',
			'BUILD_MYPAGE_MONITORS',
			'PROJECT_FULLEXPORT',
			'PROJECT_FULLIMPORT',
			'PROJECT_TPLEXPORT',
			'PROJECT_TPLIMPORT',
			'PROJECT_AUDIT_TRAIL',
			'SCM_COMMIT',
			'USER_PROJECT_DELETE',
			'USERGROUP_USER_REMOVE',
			'GET_PROJECT_SETTINGS_FORM',
			'SAVE_PROJECT_SETTINGS',
			'DO_TAG_SEARCH',
			'HANDLE_FILE_UPLOAD',
			'GET_DISCUSSION_MESSAGE',
			'DOCMAN_FILE_VERSION_SAVE',
			'DOCMAN_FILE_DELETE',
			'DOCMAN_FILE_VERSION_DELETE',
			'DOCMAN_FOLDER_SAVE',
			'DOCMAN_FOLDER_DELETE',
			'RENDER_DOWNLOAD_URL',
			'PROJECT_EVENT_PULL'
			);
		ActionManager::registerListeners($this, $listenTo);
	}

	/**
	 *	Different types of plugins are possible -
	 *	GFPlugin::SITE_PLUGIN = 'site'
	 *	GFPlugin::PROJECT_PLUGIN = 'project'
	 *	GFPlugin::USER_PLUGIN = 'user'
	 */
	public function getType() {
		return GFPlugin::PROJECT_PLUGIN;
	}	
	
	/**
	 *	get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	public function getName() {
		return 'docman';
	}

	/**
	 *	get the array of action names that this plugin answers to.
	 *	@return	array
	 */
	public function getListeners() {
	}

	/**
	 *	handle action has a case statement to redirect the action to the appropriate function.
	 *	@param string
	 *	@param mixed
	 *	@return PluginResult
	 */
	public function handleAction($listenerName, $params) {
		switch ($listenerName) {
			case 'BUILD_PROJECT_TABS': {
				return $this->_buildMainTabsListener($params);
				break;
			}
			case 'BUILD_PROJECT_SUMMARY': {
				return $this->_buildProjectSummary($params);
				break;
			}
			/*case 'GET_DOCMAN_NAV_ELEMENTS': {
				return $this->getDocmanNavElements();
				break;
			}*/
			case 'BUILD_SEARCH_COMBO': {
				return $this->buildSearchOptions($params);
				break;
			}
			case 'DO_SEARCH': {
				return $this->getSearchResults($params);
				break;
			}
			case 'GET_PROJECT_ADMIN_NAV_ELEMENTS': {
				return $this->getProjectAdminNavElements($params);
				break;
			}
			case 'PROJECT_APPROVE': {
				return $this->approveActions($params);
				break;
			}
			case 'BUILD_MYPAGE_ELEMENTS': {
				return $this->buildMyPageElements($params);
				break;
			}
			case 'CHECK_DOWNLOAD_PERMS': {
				return $this->checkDownloadPerms($params);
				break;
			}
			case 'GET_ROLE_CHOICES': {
				return $this->getRoleChoices($params);
				break;
			}
			case 'UPDATE_ROLE_CHOICES': {
				return $this->updateRoleChoices($params);
				break;
			}
			case 'GET_OBSERVER_ROLE_CHOICES': {
				return $this->getObserverRoleChoices($params);
				break;
			}
			case 'UPDATE_OBSERVER_SETTINGS': {
				return $this->updateObserverSettings($params);
				break;
			}
			case 'RENDER_ASSOCIATION_LINK': {
				return $this->renderAssociationLink($params);
				break;
			}
			case 'GET_ASSOCIATED_ITEM_INFO': {
				return $this->getAssociatedItemInfo($params);
				break;
			}
			case 'ASSOCIATION_SECTION': {
				return $this->getAssociationSection($params);
				break;
			}
			case 'ASSOCIATION_CHECK': {
				return $this->getAssociationCheck($params);
				break;
			}
			case 'GET_DISK_USAGE': {
				return $this->getDiskUsage($params);
				break;
			}
			case 'GET_DISK_QUOTA_USAGE': {
				return $this->getDiskQuotaUsage($params);
				break;
			}
			case 'MONITOR_CHECK_PERMISSIONS': {
				return $this->monitorCheckPermissions($params);
				break;
			}
			case 'MONITOR_GET_TEMPLATE': {
				return $this->monitorGetTemplate($params);
				break;
			}
			case 'BUILD_MYPAGE_MONITORS': {
				return $this->buildMyPageMonitors($params);
				break;
			}
			case 'PROJECT_FULLIMPORT':
			case 'PROJECT_FULLEXPORT': {
				return GFExportImportUtils::getSpecData($this);
				break;
			}
			case 'PROJECT_TPLIMPORT':
			case 'PROJECT_TPLEXPORT': {
				return GFExportImportUtils::getSpecData($this, 'specDataTpl.xml');
				break;
			}
			case 'PROJECT_AUDIT_TRAIL': {
				return $this->getProjectAuditTrail($params);
				break;
			}
			case 'SCM_COMMIT': {
				return $this->scmCommit($params);
				break;
			}
			case 'USER_PROJECT_DELETE': {
				return $this->deleteUserMonitors($params);
				break;
			}
			case 'USERGROUP_USER_REMOVE': {
				return $this->deleteUsergroupUserMonitors($params);
				break;
			}
			case 'GET_PROJECT_SETTINGS_FORM': {
				return $this->getProjectSettingsForm($params);
			}
			case 'SAVE_PROJECT_SETTINGS': {
				return $this->saveProjectSettings($params);
			}
			case 'DO_TAG_SEARCH': {
				return $this->getSearchByTagsResult($params);
				break;
			}
			case 'HANDLE_FILE_UPLOAD': {
				return $this->handleFileUpload($params);
				break;
			}
			case 'GET_DISCUSSION_MESSAGE': {
				return $this->getDiscussionMessage($params);
				break;
			}
			case 'DOCMAN_FILE_VERSION_SAVE': {
				$version = $params['item'];
				$file = $version->getDocmanFile();
				return $this->doDocmanFileActivity('save', $file, $params['is_new'], $version);
				break;
			}
			case 'DOCMAN_FILE_DELETE': {
				$file = $params['item'];
				return $this->doDocmanFileActivity('delete', $file);
				break;
			}
			// For project activity feed.  The file-level save and delete
			// are good enough for all new/update/new version cases,
			// but when a version is deleted (e.g., revert to previous version),
			// then the parent doc file may or may not also be deleted.
			// We need to catch the version delete event in this one case.
			case 'DOCMAN_FILE_VERSION_DELETE': {
				$version = $params['item'];
				$file = $version->getDocmanFile();
				return $this->doDocmanFileActivity('delete', $file, FALSE, $version);
				break;
			}
			case 'DOCMAN_FOLDER_SAVE': {
				$folder = $params['item'];
				return $this->doDocmanFolderActivity('save', $folder, $params['is_new']);
				break;
			}
			case 'DOCMAN_FOLDER_DELETE': {
				$folder = $params['item'];
				return $this->doDocmanFolderActivity('delete', $folder, FALSE);
				break;
			}
			case 'RENDER_DOWNLOAD_URL': {
				return $this->renderDownloadUrl($params);
				break;
			}
			case 'PROJECT_EVENT_PULL': {
				return $this->getProjectEvent($params);
				break;
			}
		}
	}

	private function deleteUsergroupUserMonitors($usergroup_user) {
		$con = Propel::getConnection();		

		$project_ids = $usergroup_user->getProjectMonitorsToDelete();
		$user_id = $usergroup_user->getMemberUserId();

		if ($project_ids != '') {
			$stmt = $con->createStatement();
			//Delete monitor for the projects
			$sql = "DELETE FROM monitor WHERE 
							user_id = ".$user_id." 
							AND
							(
								(section='".DocmanPlugin::SECTION."' AND ref_id IN (".$project_ids."))
							OR
								(section='".DocmanFile::SECTION."' AND ref_id IN 
									(
										SELECT docman_file_id as ref_id FROM docman_file, docman_folder, project WHERE 
									 	docman_file.docman_folder_id = docman_folder.docman_folder_id 
									 	AND
									 	docman_folder.project_id = project.project_id
									 	AND
									 	NOT (project.is_public = true AND docman_folder.is_public = true)
									 	AND
									 	docman_folder.project_id IN (".$project_ids.")
									 )
								)
							)";


			$stmt->executeQuery($sql);
		}		
	}

	private function deleteUserMonitors($params) {
		$user_id = $params['user']->getUserId();
		$project = $params['project'];

		if (is_object($project)) {
			if (!$project->isPublic()) {
				$con = Propel::getConnection();
				$stmt = $con->createStatement();
		
				$sql = "DELETE FROM monitor WHERE 
								user_id = ".$user_id." 
								AND
								(
									(section='".DocmanPlugin::SECTION."' AND ref_id = ".$project->getProjectId().")
								OR
									(section='".DocmanFile::SECTION."' AND ref_id IN 
										(SELECT docman_file_id as ref_id FROM docman_file, docman_folder WHERE 
										 docman_file.docman_folder_id = docman_folder.docman_folder_id AND
										 docman_folder.project_id = ".$project->getProjectId().")
									)
								)";
		
				$stmt->executeQuery($sql);
			}
		}
	}

	private function getAssociationCheck($params) {
		if ($params->getSection() == DocmanFile::SECTION) {
			$df=DocmanFilePeer::retrieveByPk($params->getRefId());
			if (!is_object($df) || !$df->userCanUpdate()) {
				throw new InvalidDataException(array('The document doesn\'t exist or you don\'t have permissions for it'));
			}
		} 
		if ($params->getToSection() == DocmanFile::SECTION) {
			$df=DocmanFilePeer::retrieveByPk($params->getToRefId());
			if (!is_object($df) || !$df->userCanUpdate()) {
				throw new InvalidDataException(array('The document doesn\'t exist or you don\'t have permissions for it'));
			}
		}
	}

	private function getAssociationSection($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$result->setResult(array(DocmanFile::SECTION,'Document'));
		return $result;
	}

	
	private function renderAssociationLink($params) {
		$ret=array();
		foreach ($params as $param) {
			if ($param[0]==DocmanFile::SECTION) {
				$ti=DocmanFilePeer::retrieveByPk($param[1]);
				if (is_object($ti)) {
					$ret[]=array(
						$ti->getPrimaryKey(),
						'Document: '.$ti->getDocumentLink(true),
						$param[2],
						$param[3]
					);
				}
			} else {
				$arr = explode("|", $param[2]);
				if (isset($arr[2]) && $arr[2] == DocmanFile::SECTION && $arr[0] == WallPost::SECTION) {
					$res = Association::renderNonPluginAssociationLink($param);
					if (is_array($res)) {
						$ret[] = $res;
					}
				}
 			}
		}
		$result = new PluginResult($this);
		$result->setResult($ret);
		return $result;
	}


	/**
	 *	Returns additional info for the associated item
	 *		 
	 *	@param array $params needed params to build the associated item info
	 * 	@return array with associated item info (Description, URL)
	 */	 
	private function getAssociatedItemInfo($params) {
		$ret=array();
		foreach ($params as $param) {
			if ($param[0]==DocmanFile::SECTION) {
				$ti=DocmanFilePeer::retrieveByPk($param[1]);
				if (is_object($ti)) {
					$ret[]=array(
					$param[2],
					$param[3],
					$param[4],
					$param[5],
					$param[6],
					'Document: '.$ti->getDescription(),
					$ti->getDocumentLink(false),
					);
				}
			}
		}
		$result = new PluginResult($this);
		$result->setResult($ret);
		return $result;
	}
	
	/**
	 * returns the part of the html text that goes into the my page area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageElements($params) {	
		
	}

	/**
	 *	return an sql statement for calculating disk usage
	 */
	function getDiskUsage($params, $search_contents_only = false) {
		$sql="SELECT df.project_id,sum(fs.file_size+length(fsi.vectors)) AS usage
			FROM docman_folder df JOIN docman_file fd USING (docman_folder_id) JOIN docman_file_version dfv USING (docman_file_id),
			filesystem fs JOIN filesystem_idx fsi USING (filesystem_id)
			WHERE dfv.docman_file_version_id=fs.ref_id
			AND fs.section='docmanfileversion'
			GROUP BY project_id";
		$result = new PluginResult($this);
		$result->setResult($sql);
		return $result;
	}

	function getDiskQuotaUsage($params) {
		$con = Propel::getConnection();
		$stmt = $con->createStatement();
		$project_id = $params['project_id'];
		$sql="SELECT df.project_id,sum(fs.file_size+length(fsi.vectors)) AS usage
			FROM docman_folder df JOIN docman_file fd USING (docman_folder_id) JOIN docman_file_version dfv USING (docman_file_id),
			filesystem fs JOIN filesystem_idx fsi USING (filesystem_id)
			WHERE dfv.docman_file_version_id=fs.ref_id AND df.project_id = '$project_id'
			AND fs.section='docmanfileversion'
			GROUP BY project_id";
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		if ($rs->getRecordCount() > 0) {
			$rs->next();
			$usage = $rs->getFloat('usage');
		} else {
			$usage = 0;
		}
		$resArray = array(
			'plugin_name' => $this->getName(),
			'usage'   => array('files' => $usage)
		);
		$result = new PluginResult($this);
		$result->setResult($resArray);
		return $result;
	}

	/**
	 *      take a search object and handle the search internally.
	 *      @param  GFSearch
	 *      @return string
	 */
	public function search(GFSearch $search) {
		if (!$search->getProjectId()) {
			return $this->globalSearch($search);
		}
		$files = array();
		$project = $search->params['project'];

		if (GFConf::get('system.dbtype') == 'postgres') {
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();
			//Gets folders
			$c = new Criteria();
			$c->add(DocmanFolderPeer::PROJECT_ID,$project->getProjectId());
			$Folders = DocmanFolderPeer::doSelect($c);
			//Get Folders that the user has at least read permissions
			$arr_folders = array();
			foreach ($Folders as $folder) {
				if ($folder->userCanRead()) {
					$arr_folders[] = $folder->getDocmanFolderId();
				}
			}
			$str_folders = implode(",",$arr_folders);
			
			if (!$str_folders) {
				$str_folders = 'NULL';		// Avoid an empty IN clause
			}
			
			if ($search->getStartDate()) {
				$date_sql = " AND dfv.create_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}
			if ($search->getSearchAttach()) {
				$attach_sql="			
				SELECT df.docman_file_id, rank(filesystem_idx.vectors, q) AS rank 
				FROM filesystem_idx, to_tsquery('".$words."') q, filesystem fs, docman_file_version dfv, docman_file df, docman_folder, docman_file_folder dff
				WHERE filesystem_idx.vectors @@ q 
				AND filesystem_idx.filesystem_id=fs.filesystem_id
				AND fs.section='docmanfileversion'
				AND fs.ref_id=dfv.docman_file_version_id
				AND dfv.docman_file_id=df.docman_file_id
				AND dfv.is_approved = 't'
				AND docman_folder.docman_folder_id IN (".$str_folders.") 
				$date_sql
				AND df.docman_file_id=dff.docman_file_id
				AND dff.docman_folder_id = docman_folder.docman_folder_id
				AND docman_folder.project_id=".$project->getProjectId();
			} else {
				$attach_sql='';
			}

			$sql="
			SELECT s.docman_file_id,df.description,sum(s.rank) AS rank FROM (
			$attach_sql
			";

			if ($attach_sql != '' && ((!isset($search->params['search_contents_only']) || !$search->params['search_contents_only']))) {
				$sql .= ' UNION ';
			}

			if (!isset($search->params['search_contents_only']) || !$search->params['search_contents_only']) {				
				$sql .= "
				SELECT df.docman_file_id, rank(filesystem_filename_idx.vectors, q) AS rank 
				FROM filesystem_filename_idx, to_tsquery('".$words."') q, filesystem fs, docman_file_version dfv, docman_file df, docman_folder, docman_file_folder dff
				WHERE filesystem_filename_idx.vectors @@ q 
				AND filesystem_filename_idx.filesystem_id=fs.filesystem_id
				AND fs.section='docmanfileversion'
				AND fs.ref_id=dfv.docman_file_version_id
				AND dfv.docman_file_id=df.docman_file_id
				AND dfv.is_approved = 't'
				AND docman_folder.docman_folder_id IN (".$str_folders.") 
				$date_sql
 				AND df.docman_file_id=dff.docman_file_id
				AND dff.docman_folder_id = docman_folder.docman_folder_id
				AND docman_folder.project_id=".$project->getProjectId(). " 

				UNION

				SELECT dfi.docman_file_id, rank(dfi.vectors, q) AS rank
				FROM docman_file_idx dfi, to_tsquery('".$words."') q, docman_file df, docman_folder, docman_file_folder dff
				WHERE dfi.vectors @@ q
				AND dfi.docman_file_id=df.docman_file_id
				AND dff.docman_file_id = df.docman_file_id				
				AND dff.docman_folder_id=docman_folder.docman_folder_id
				AND docman_folder.project_id=".$project->getProjectId()." 
				AND docman_folder.docman_folder_id IN (".$str_folders.") 

				UNION

				SELECT df.docman_file_id, rank(dfi.vectors, q) AS rank
				FROM docman_folder_idx dfi, to_tsquery('".$words."') q, docman_file df, docman_folder, docman_file_folder dff
				WHERE dfi.vectors @@ q
				AND dfi.docman_folder_id=dff.docman_folder_id
				AND dff.docman_folder_id = docman_folder.docman_folder_id
				AND df.docman_file_id = dff.docman_file_id
				AND docman_folder.project_id=".$project->getProjectId()." 
				AND docman_folder.docman_folder_id IN (".$str_folders.") 
	
				UNION
				
				SELECT df.docman_file_id, rank(dfvi.vectors, q) AS rank
				FROM docman_file_version_idx dfvi, to_tsquery('".$words."') q, docman_file df, docman_folder, docman_file_version dfv, docman_file_folder dff
				WHERE dfvi.vectors @@ q
				AND dfvi.docman_file_version_id = dfv.docman_file_version_id
				AND dfv.docman_file_id = df.docman_file_id
				AND df.docman_file_id = dff.docman_file_id
				AND dff.docman_folder_id = docman_folder.docman_folder_id
				AND dfv.is_approved = 't'
				AND docman_folder.project_id=".$project->getProjectId()." 
				AND docman_folder.docman_folder_id IN (".$str_folders.")";
			}
			
			$sql .= "
			$date_sql
			AND docman_folder.project_id=".$project->getProjectId().") s,docman_file df WHERE s.docman_file_id=df.docman_file_id 
			GROUP BY s.docman_file_id,df.description 
			ORDER BY rank DESC";

			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			$template = $search->getTemplate();

			$count = 0;
			$content = $template->tableTop(array(Language::getSessText('Docman.DescriptionName')));

			while ($rs->next()) {
				$trow = array();
				$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project,'',$this->getName(),'',array('file_id'=>$rs->getInt('docman_file_id'))) . '">' . $rs->getString('description').'</a>';
				$content .= $template->tableRow($trow);
				$files[] = $rs->getInt('docman_file_id');
				$count++;
			}
			$content .= $template->tableBottom();

			$title = Language::getSessText('Docman.SearchResultsTabTitle', array($count));

			if ($count == 0) {
				$content = Language::getSessText('Search.NoResults');
			}			

		} elseif (GFConf::get('system.dbtype') == 'mysql') {

		} elseif (GFConf::get('system.dbtype') == 'oracle') {

		}

		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content,
			'files' => $files
			);
	}

	/**
	 *	Searches across all projects
	 *	@param	GFSearch
	 *	@return string
	 */
	function globalSearch(GFSearch $search) {
		if (GFConf::get('system.dbtype') == 'postgres') {
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();
			if ($search->getStartDate()) {
				$date_sql = " AND dfv.create_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}

			$project_ids = $search->getProjectIds();
			$project_sql = '';
			if (!is_null($project_ids)){
				$project_sql = ' AND docman_folder.project_id IN ('. implode(',', $search->params['project_ids']).')';
			}

			if ($search->getSearchAttach()) {
				$attach_sql="UNION
				SELECT df.docman_file_id, rank(filesystem_idx.vectors, q) AS rank, headline(df.description, q) AS headline_description 
				FROM filesystem_idx, to_tsquery('".$words."') q, filesystem fs, docman_file_version dfv, docman_file df, docman_folder
				WHERE filesystem_idx.vectors @@ q 
				AND filesystem_idx.filesystem_id=fs.filesystem_id
				AND fs.section='docmanfileversion'
				AND fs.ref_id=dfv.docman_file_version_id
				AND dfv.docman_file_id=df.docman_file_id
				AND dfv.is_approved = 't'
				$date_sql
				AND df.docman_folder_id=docman_folder.docman_folder_id $project_sql";
			} else {
				$attach_sql='';
			}
			$sql="SELECT 
					project.project_name, 			-- 1
					project.unix_name, 				-- 2
					sum(s.rank) AS rank,			-- 3
					s.headline_description,			-- 4
					df.*							-- 5
			FROM (
				SELECT dfi.docman_file_id, rank(dfi.vectors, q) AS rank, headline(df.description, q) AS headline_description
				FROM docman_file_idx dfi, to_tsquery('".$words."') q, docman_file df
				WHERE dfi.vectors @@ q
				AND dfi.docman_file_id=df.docman_file_id
				$attach_sql
				UNION
				SELECT df.docman_file_id, rank(dfvi.vectors, q) AS rank, headline(df.description, q) AS headline_description
				FROM docman_file_version_idx dfvi, to_tsquery('".$words."') q, docman_file df, docman_file_version dfv, docman_folder, project
				WHERE dfvi.vectors @@ q
				AND dfvi.docman_file_version_id=dfv.docman_file_version_id
				AND dfv.docman_file_id=df.docman_file_id
				AND dfv.is_approved = 't'
				$date_sql
				AND docman_folder.project_id=project.project_id $project_sql
			) s, docman_file df, docman_folder, project WHERE s.docman_file_id=df.docman_file_id 
				AND df.docman_folder_id=docman_folder.docman_folder_id AND docman_folder.project_id=project.project_id
			GROUP BY project.project_name, project.unix_name, df.docman_file_id,df.docman_folder_id,df.description,df.is_locked, s.headline_description,df.locked_by
			ORDER BY rank DESC";

			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			$template = $search->getTemplate();
			
			$count = 0;
			$content = $template->tableTop(array(
						Language::getSessText('Project.ListNameTitle'),
						Language::getSessText('Docman.DescriptionName')
						));
			while ($rs->next()) {
				$docman_file = new DocmanFile();
				$docman_file->hydrate($rs, 5);
				
				if (!$docman_file->userCanRead()) {
					continue;
				}
				
				$count++;
				$project_unix_name = $rs->getString(2);
				$trow = array();
				$trow[] = '<a href="'.GFUrl::getUrl()->projectUrl($project_unix_name).'">'.$rs->getString(1).'</a>';
				$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project_unix_name,'',$this->getName(),'',array('file_id'=>$docman_file->getDocmanFileId())) . '">' . $rs->getString(4).'</a>';
				$content .= $template->tableRow($trow);
			}

			if ($count == 0) {
				$content .= $template->tableFullRow(Language::getSessText('Search.NoResults'));
			}
			$content .= $template->tableBottom();
			
			$title = Language::getSessText('Docman.SearchResultsTabTitle', array($count));

				

		} elseif (GFConf::get('system.dbtype') == 'mysql') {

		} elseif (GFConf::get('system.dbtype') == 'oracle') {

		}
		
		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content
			);
	}


	public function searchByTags(GFSearch $search){

		$words = $search->getWordsForQuery();
		$project_id = $search->getProjectId();
		$project_ids = $search->getProjectIds();
		$template = $search->getTemplate();

		//Clean key words from tsearch operators
		$words = preg_replace('/,+/',',', $words);
		$tsearch_operators = array('(', ')', '!', '|','&');
		$replacements = array('','','','',',');
		$words = str_replace($tsearch_operators, $replacements, $words);
		$words_arr = explode(',', $words);

		$wc = count($words_arr);
		if($wc == 0){
			return '';
		}

		for($i = 0; $i < $wc; $i++){
			$words_arr[$i] = "'" . $words_arr[$i] . "'";
		}
		$words = implode(',', $words_arr);

		if($project_id){
			$docman_prj = " AND docman_folder.project_id = $project_id ";
		} else if (!is_null($project_ids)) {
			$docman_prj = ' AND docman_folder.project_id IN (' . implode(',',$project_ids) .') ';
		} else {
			$docman_prj = ' ';
		}

		$docman_perms_sql = "SELECT DISTINCT docman_folder.docman_folder_id AS docman_folder_id
		FROM docman_file INNER JOIN docman_file_folder ON docman_file.docman_file_id = docman_file_folder.docman_file_id
		INNER JOIN docman_folder ON docman_folder.docman_folder_id = docman_file_folder.docman_folder_id
		WHERE docman_file.docman_file_id IN
		(SELECT tag_entry.ref_id FROM tag_entry WHERE
		section='docmanfile'
		AND tag_name IN ($words)
		GROUP BY tag_entry.ref_id
		HAVING count(tag_entry.ref_id) = $wc) $docman_prj";

		$con = Propel::getConnection();
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($docman_perms_sql, ResultSet::FETCHMODE_NUM);

		$docman_aux = null;
		if($rs->getRecordCount() > 0) {
			while($rs->next()){
				$docman_folders[] = $rs->getInt(1);
			}
			$dfrs = DocmanFolderPeer::retrieveByPks($docman_folders);
			$docman_folders = array();
			foreach($dfrs as $dfr){
				$setting = DocmanProjectSettingPeer::retrieveByPk($dfr->getProject()->getProjectId());
				if($dfr->getProject()->usesPlugin('docman') 
					&& $dfr->userCanRead() &&
					(!$setting || $setting->getShowTagsTab())){
					$docman_folders[] = $dfr->getDocmanFolderId();
				}
			}
			if(count($docman_folders) == 0){
				//no results
				//return $this->emptyTagSearch();
			}else {
				$docman_aux = ' AND docman_file.docman_folder_id IN ( '. implode(',', $docman_folders) .')';
			}
		} else {
			//empty result set
			//return $this->emptyTagSearch();
		}

		/*
		$docman_file_sql = "SELECT 'docmanfile' AS section, docman_file.docman_file_id AS ref_id,
		docman_file.description AS description, project.unix_name AS prj_unix_name,
		project.project_name AS prj_name
		FROM docman_file INNER JOIN docman_folder ON
		docman_file.docman_folder_id = docman_folder.docman_folder_id
		INNER JOIN project ON docman_folder.project_id = project.project_id
		WHERE docman_file.docman_file_id IN
		(SELECT tag_entry.ref_id FROM tag_entry WHERE section='docmanfile'
		AND tag_name IN ($words)
		GROUP BY tag_entry.ref_id
		HAVING COUNT(tag_entry.ref_id) = $wc) $docman_aux $docman_prj";
		*/
		$docman_file_sql = "SELECT 'docmanfile' AS section, docman_file.docman_file_id AS ref_id,
		docman_file.description AS description, project.unix_name AS prj_unix_name,
		project.project_name AS prj_name
		FROM docman_file INNER JOIN docman_file_folder ON
		docman_file.docman_file_id = docman_file_folder.docman_file_id
		INNER JOIN docman_folder ON docman_folder.docman_folder_id = docman_file_folder.docman_folder_id
		INNER JOIN project ON docman_folder.project_id = project.project_id
		WHERE docman_file.docman_file_id IN
		(SELECT tag_entry.ref_id FROM tag_entry WHERE section='docmanfile'
		AND tag_name IN ($words)
		GROUP BY tag_entry.ref_id
		HAVING COUNT(tag_entry.ref_id) = $wc) $docman_prj";


		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($docman_file_sql, ResultSet::FETCHMODE_ASSOC);

		$Lang = Session::getSession()->getLanguage();
		if(!$project_id){
			$content = $template->tableTop(array(
				$Lang->getText('Project.ListNameTitle'),
				$Lang->getText('Docman.DescriptionName')
			));
		} else{
			$content = $template->tableTop(array(
				$Lang->getText('Docman.DescriptionName')
			));
		}

		$count = 0;
		$files = array();
		if($rs->getRecordCount() > 0){
			while($rs->next()){
				$trow = array();
				if(!$project_id){
					$trow[] = '<a href="' . GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name')) . '">'. $rs->getString('prj_name') . '</a>';
				}

				$trow[] = '<a href="'. GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name'), '', 
					'docman', '', array('file_id'=>$rs->getInt('ref_id'))) . '">' . $rs->getString('description') . '</a>';
				$content .= $template->tableRow($trow);
				$files[] = $rs->getInt('ref_id');
				$count++;
			}
		}
		$title = $Lang->getText('Docman.SearchResultsTabTitle', array($count));
			
		if ($count == 0) {
			$content .= $template->tableFullRow($Lang->getText('Search.NoResults'));
		}
		$content .= $template->tableBottom();

		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content,
			'files' => $files
		);

	}

	private function emptyTagSearch(){
		$Lang = Session::getSession()->getLanguage();
		$count = 0;
		$title = $Lang->getText('Docman.SearchResultsTabTitle', array($count));
		$content = $Lang->getText('Search.NoResults');

		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content
		);
	}


	/**
	 * returns the options to print in the combobox
	 *
	 * @param unknown_type $params
	 * @return unknown
	 */
	private function buildSearchOptions($params) {
		$result = new PluginResult($this);
		
		if (isset($params['project'])) {
			if (!$params['project']->usesPlugin($this->getName())) {
				return;
			}
			$text = Language::getSessText('Docman.ThisProjectDocman');
		} else {
			$text = Language::getSessText('Docman.Documents');
		}
		
		$value = array(array(
			'value'	=> $this->getName(),
			'display'	=> $text
			));
		$result->setResult($value);
		return $result;		
	}
	
	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return resultset
	 */
	private function getSearchResults($params) {
		if (array_search($this->getName(), $params['search']) !== false) {
			$result = new PluginResult($this);
			$searcher = new GFSearch($params);
			$result->setResult($this->search($searcher));
			return $result;
		}
	}

	private function getSearchByTagsResult($params) {
		if (array_search($this->getName(), $params['search']) !== false) {
			$result = new PluginResult($this);
			$searcher = new GFSearch($params);
			$result->setResult($this->searchByTags($searcher));
			return $result;
		}
	}

	
	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return resultset
	 */
	private function checkDownloadPerms($params) {
		$section = $params['section'];
		$ref_id = $params['ref_id'];
		$result = new PluginResult($this);
		$result->setResult(false); // false by default, looking for true
		// if it's for another section, exit
		if ($section!=DocmanFileVersion::SECTION) {
			$result->setResult(false);
		} else {
			$docman_file_version = DocmanFileVersionPeer::retrieveByPK($ref_id);
			if (is_object($docman_file_version)) {
				$docman_file = $docman_file_version->getDocmanFile();
				if ($docman_file instanceof DocmanFile) {
					if (!$docman_file->userCanRead()) {
						$result->setResult(false);
					} else {
						$result->setResult(true);
					}
				}
			} else {
				$result->setResult(false);
			}
		}
		return $result;
	}

	/**
	 * returns an array of the fields where we want the search to be done upon
	 *
	 */
	private function getSearchFields() {
		return array();
	}	
	
	private function _buildMainTabsListener($params) {
		$project = $params['project'];
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$value = array(
				'text'	=> Language::getSessText('Docman.TabString'),
				'action'	=> GFUrl::getUrl()->projectURL($project,'','docman'),
				'id'	=> 'docman',
				'item_count' => DocmanFolderPeer::getRootFolderCount($project->getId()) + DocmanFilePeer::getRootFileCount($project->getId())
			);
		$result->setResult($value);
		return $result;
	}

	/**
	 *      getDocmanNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 * /
	private function getDocmanNavElements() {
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'Docman Title',
			'elements'=> array(
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 1'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 2'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 3')
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 *	get project uri to be used as root for constructing uris
	 *	@return string
	 */
	function getProjectPluginUri() {
		return GFUrl::getUrl()->projectURL($this->project,'','docman');
	}

	/**
	 * returns the part of the html text that goes into the project public area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function _buildProjectSummary($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);

		$project = $params['project'];
		$theme = $params['theme'];
		
		//$value = $theme->infoTableTop(Language::getSessText('Project.Documents'));
	
		$value = '<a href="'. GFUrl::getUrl()->projectURL($project,'','docman').'">';
		$value .= '<img src="/images/docman_doc.gif" width="16" height="16" border="0" alt="'.Language::getSessText('Docman.TabString').'" />';
		$value .= Language::getSessText('Docman.TabString').'</a>';		
		//$value .= $theme->infoTableBottom();

		$result->setResult($value);
		return $result;
	}

	function enableInProject(Project $project) {
		return;
	}

	function disableFromProject() {
		// unregister the actions
		ActionManager::unregisterListeners($this);
	}
	
	/**
	 *      getProjectAdminNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	private function getProjectAdminNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'Docs',
			'elements'=> array(
			array('link'=>'/gf/project/'.$params['project']->getUnixName().'/docman/admin/',
				'description'=>Language::getSessText('Docman.Admin'))
			)
		);
		$result->setResult($arr);
		return $result;
	}
	
	/**
	 * actions to perform on project approval
	 *
	 * @param $params array
	 * @return PluginResult
	 */
	private function approveActions($params) {
		$template_project = $params['template_project'];
		$project = $params['project'];
		$con = Propel::getConnection(UserPeer::DATABASE_NAME);

		if ( (is_object($project)) && (is_object($template_project))) {		
			// if the template project uses the docman plugin, initialize the folders
			if ($template_project->usesPlugin($this->getName())) {
				// get template's folders
				$c = new Criteria();
				$c->add(DocmanFolderPeer::PROJECT_ID, $template_project->getProjectId());
				$template_folders = DocmanFolderPeer::doSelect($c);

				if ($template_folders != null && is_array($template_folders)) {
					// initialize the maps for handling parents/childs
					$parent_map = array();
					$new_folder_map = array();

					// make the parent folder map
					foreach ($template_folders as $template_folder) {
						if ($template_folder->getParentFolderId() > 0) {
							$parent_map[$template_folder->getDocmanFolderId()] = $template_folder->getParentFolderId();
						}
					}

					//create each folder and update the new_folder_map
					foreach ($template_folders as $template_folder) {
						$doc_folder = new DocmanFolder();
						$doc_folder->setFolderName($template_folder->getFolderName());
						$doc_folder->setIsPublic($template_folder->getIsPublic());
						$doc_folder->setSortOrder($template_folder->getSortOrder());
						$doc_folder->setProject($project);
						$doc_folder->save(null, false);
						$new_folder_map[$template_folder->getDocmanFolderId()] = $doc_folder->getDocmanFolderId();
				
						// search the role_settings and let's see if for this item we have one set
						foreach ($params['role_settings'] as $role_setting) {
							if (strstr($role_setting['section'],$this->getName()) && $role_setting['ref_id']==$template_folder->getPrimaryKey()) {
								$role_id = $role_setting['role_id'];
								$section = $role_setting['section'];
								$ref_id = $doc_folder->getPrimaryKey();
								$value = $role_setting['value'];
								$stmt = $con->createStatement();

								// Note: when $doc_folder->save() is called, it creates default entries on the role_setting
								// table. We must override them.
								$sql = "DELETE FROM role_setting WHERE role_id='$role_id' AND section='$section' AND ref_id='$ref_id'";
								$rs = $stmt->executeQuery($sql);

								$sql = "insert into role_setting (role_id,section,ref_id,value) values ('$role_id','$section','$ref_id','$value')";
								$rs = $stmt->executeQuery($sql);
							}
						}
					}
					
					// assign the parents to the folders that need it
					foreach ($parent_map as $old_folder_id => $old_parent_id) {
						$doc_folder_id = $new_folder_map[$old_folder_id];
						$new_parent_id = $new_folder_map[$old_parent_id];
						$doc_folder = DocmanFolderPeer::retrieveByPk($doc_folder_id);
						if ($doc_folder != null && is_object($doc_folder)) {
							$doc_folder->setParentFolderId($new_parent_id);
							$doc_folder->save();
						}
					}
				}
				
				// Copy the project settings
				$con->createStatement()->executeQuery('DELETE FROM docman_project_setting WHERE project_id='.$project->getProjectId());
				$con->createStatement()->executeQuery('INSERT INTO docman_project_setting(project_id, show_associations_tab, show_tags_tab) 
									SELECT '.$project->getProjectId().', show_associations_tab, show_tags_tab
									FROM docman_project_setting WHERE project_id='.$template_project->getProjectId()
								);
			}
		}
		
	}

	/**
	 * returns the possible role settings available for this plugin
	 *
	 * @return array	the roles
	 */
	function getAvailableRoleSettings() {
		$return = array();
		$return[0][] = Language::getSessText('DocmanFolder.NoAccess'); //descriptions
		$return[1][] = DocmanFolder::PERMLEVEL_NOACCESS; // values
		$return[0][] = Language::getSessText('DocmanFolder.Read');
		$return[1][] = DocmanFolder::PERMLEVEL_READ;
		$return[0][] = Language::getSessText('DocmanFolder.ReadWrite');
		$return[1][] = DocmanFolder::PERMLEVEL_WRITE;
		//$return[0][] = Language::getSessText('DocmanFolder.Admin');
		//$return[1][] = DocmanFolder::PERMLEVEL_ADMIN;

		return $return;
	}

	/**
	 * returns the possible site-wide admin role settings for this plugin
	 *
	 * @return array	the roles
	 */
	function getAdminRoleSettings() {
		$return = array();
		$return[0][] = Language::getSessText('Docman.ProjectWideNonAdmin'); //descriptions
		$return[1][] = DocmanPlugin::NOT_ADMIN ; // values
		$return[0][] = Language::getSessText('Docman.ProjectWideAdmin');
		$return[1][] = DocmanPlugin::ADMIN;
		
		return $return;
	}

	/**
	 * returns the settings for the tracker status
	 *
	 * @return array
	 */
	function getStatusObserverSettings() {
		$return = array();
		$return[0][] = Language::getSessText('DocmanFolder.Private'); //descriptions
		$return[1][] = DocmanFolder::IS_PRIVATE; // values
		$return[0][] = Language::getSessText('DocmanFolder.Public');
		$return[1][] = DocmanFolder::IS_PUBLIC;
		return $return;
	}

	/**
	 * 	The getRoleChoices listener - this hook will fire
	 *	when a new role is added or one is going to be updated.
	 * 	It returns an array of role settings
	 *	@return	array
	 */
	function getRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$roles = array();
		$roles[0]['ref_id'] = 0;
		$roles[0]['section'] = Language::getSessText('Docman.Role.ProjectWide');
		$roles[0]['description'] = '<span id="docmanAdminDescription">'.
			Language::getSessText('Docman.Role.Description').'<span>';
		$roles[0]['subsection'] = '-';
		$roles[0]['settings'][0] = self::SECTION;
		$roles[0]['settings'][] = $this->getAdminRoleSettings();
		$c = new Criteria();
		$c->add(DocmanFolderPeer::PROJECT_ID,$params['project']->getPrimaryKey());
		$c->add(DocmanFolderPeer::PARENT_FOLDER_ID,0);
		$c->addAscendingOrderByColumn(DocmanFolderPeer::DOCMAN_FOLDER_ID);
		$folders = DocmanFolderPeer::doSelect($c);
		foreach ($folders as $folder) {
			$key = $folder->getPrimaryKey();
			$roles[$key]['ref_id'] = $key;
			$roles[$key]['section'] = Language::getSessText('DocmanFolder.Roles.Section'); // the displayed text in "section" field
			$roles[$key]['description'] = Language::getSessText('DocmanFolder.DocmanFolderWideAdmin.Description');
			if ($folder->getFolderName() == 'incoming' && $folder->getParentFolderId() == 0) {
				$roles[$key]['subsection'] = Language::getSessText('Docman.RootFolder'); // the displayed text in the "subsection" field	
			} else {
				$roles[$key]['subsection'] = $folder->getFolderName(); // the displayed text in the "subsection" field	
			}
			$roles[$key]['settings'][0] = DocmanFolder::SECTION; //$this->getName();// the section for the DB, one for all the settings
			$roles[$key]['settings'][] = $this->getAvailableRoleSettings();
		}
		
		$result = new PluginResult($this);
		$result->setResult($roles);
		return $result;
	}

	private function updateRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		
		$sections = array(self::SECTION,DocmanFolder::SECTION);

		$project = $params['project'];
		$data = $params['data'];
		$role = $params['role'];

		foreach ($sections as $section) {
			if (!isset($data[$section])) {
				return;
			}

			foreach ($data[$section] as $ref_id => $value) {
				$c = new Criteria();
				$c->add(RoleSettingPeer::ROLE_ID, $role->getPrimaryKey());
				$c->add(RoleSettingPeer::SECTION, $section);
				$c->add(RoleSettingPeer::REF_ID, $ref_id);
				$rolesetting = RoleSettingPeer::doSelectOne($c);
				if (!is_object($rolesetting)) {
					// Enter here if we have a role setting which we didn't have previously (e.g.: activate an inactive plugin)
					$rolesetting = new RoleSetting();
					$rolesetting->setSection($section);
					$rolesetting->setRefId($ref_id);
					$rolesetting->setRoleId($role->getPrimaryKey());
				}

				// Being a project admin overrides any other value
				if (isset($data['projectadmin']) && $data['projectadmin'][0] >= Project::PROJECT_ROLE_ADMIN) {
					$value = self::ADMIN;
				}
				$rolesetting->setValue($value);
				$rolesetting->save();						
			}	
		}
		return;
	}
	
	/**
	 *      The getObserverRoleChoices listener - this hook will fire
	 *      when the observer role is modified.
	 *      It returns an array of role settings or null
	 *      @return array or null
	 */
	function getObserverRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$choices = array();
		$c = new Criteria();
		$c->add(DocmanFolderPeer::PROJECT_ID,$params['project']->getPrimaryKey());
		$c->add(DocmanFolderPeer::PARENT_FOLDER_ID,0);
		$c->addAscendingOrderByColumn(DocmanFolderPeer::DOCMAN_FOLDER_ID);
		//retrieve the tracker items for the project
		$folders = DocmanFolderPeer::doSelect($c);
		$i = 0;
		foreach ($folders as $folder) {
			$key = $folder->getPrimaryKey();
			$choices[$i]['ref_id'] = $key;
			$choices[$i]['section'] = Language::getSessText('DocmanFolder.Privileges');
			if ($folder->getFolderName() == 'incoming' && $folder->getParentFolderId() == 0) {
				$choices[$i]['subsection'] = Language::getSessText('Docman.RootFolder'); // the displayed text in the "subsection" field	
			} else { 
				$choices[$i]['subsection'] = $folder->getFolderName();}
			$choices[$i]['description'] = Language::getSessText('DocmanFolder.DocmanFolderWideAdmin.Description');
			$choices[$i]['settings'][0] = 'folderpublic';
			$choices[$i]['settings'][] = $this->getStatusObserverSettings();
			$i++;
		}

		$result = new PluginResult($this);
		$result->setResult($choices);
		return $result;
	}

	/**
	 * Retrieves the concerning data and updates its observer role settings
	 *
	 * @param array $data
	 */
	function updateObserverSettings($params) {
		$project = $params['project'];
		$data = $params['data'];

		$onedata = current($data);
		while ($onedata != null) {
			$section = key($data);
			switch ($section) {
				case 'folderpublic' : {
					$oneitem = current($onedata);
					while ($oneitem != null) {
						$ref_id = key($onedata);
						$value = $oneitem;
						//retrieve the tracker and update it
						$c = new Criteria();
						$c->add(DocmanFolderPeer::PROJECT_ID,$project->getProjectId());
						$c->add(DocmanFolderPeer::DOCMAN_FOLDER_ID,$ref_id);

						try {
							$folder = DocmanFolderPeer::doSelectOne($c);
						} catch (Exception $e) {
							// something wrong happened
							echo $e->getMessage();
							next($onedata);
							$oneitem = current($onedata);
							continue;
						}
						$folder->setIsPublic($value);
						$folder->save();
						next($onedata);
						$oneitem = current($onedata);
					}
					break;
				}
				default : {
					//skip it
					continue;
				}
			}
			next($data);
			$onedata = current($data);
		}
	}

	/**
	 * Retrieve the corresponding setting (value) for the ref_id passed as argument, using the appropriate section
	 *
	 * @param int $ref_id
	 * @param string $section
	 * @return setting
	 */
	function getObserverSetting($ref_id,$section) {
		switch ($section) {
			case 'folderpublic' : {
				$c = new Criteria();
				$c->add(DocmanFolderPeer::DOCMAN_FOLDER_ID,$ref_id);
				try {
					$folder = DocmanFolderPeer::doSelectOne($c);
				} catch (Exception $e) {
					echo $e->getMessage();
					return '';
				}
				return $folder->getIsPublic();
				break;
			}
			default : {
				return '';
			}
		}
	}

	/**
	 * Auxiliar function to check if a user is monitoring the project's docman
	 */
	public function userIsMonitoring(User $user, Project $project) {
		$c = new Criteria();
		$c->add(MonitorPeer::SECTION, self::SECTION);
		$c->add(MonitorPeer::REF_ID, $project->getProjectId());
		$c->add(MonitorPeer::USER_ID, $user->getUserId());
		
		if (MonitorPeer::doCount($c) > 0) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Check if the user can monitor certain item
	 */	
	private function monitorCheckPermissions($params) {
		$section = $params['section'];
		$ref_id = $params['ref_id'];
		
		if ($section != self::SECTION && $section != DocmanFile::SECTION && $section != DocmanFolder::SECTION) {
			return;
		}
		$pr = new PluginResult($this);
		
		if ($section == self::SECTION) {
			$project = ProjectPeer::retrieveByPk($ref_id);
			if (!$project) {
				$result = false;
			} else {
				$perm = $project->getPermission();
				$result = $perm->check(self::SECTION, 0, self::NOT_ADMIN);
			}
		} elseif ($section == DocmanFile::SECTION) {
			$docmanFile = DocmanFilePeer::retrieveByPk($ref_id);
			if (!$docmanFile) {
				$result = false;
			} else {
				$result = $docmanFile->userCanRead();
			}
		} elseif ($section == DocmanFolder::SECTION) {
			$docmanFolder = DocmanFolderPeer::retrieveByPk($ref_id);
			if (!$docmanFolder) {
				$result = false;
			} else {
				$result = $docmanFolder->userCanRead();
			}
		}
		
		$pr->setResult($result);

		return $pr;
	}


	private function monitorGetTemplate($params){
		$section = $params['section'];
		$ref_id = $params['ref_id'];
		$project = $params['project'];
		$Lang = $params['lang'];

		if($section != self::SECTION && $section != DocmanFile::SECTION){
			return;
		}

		include_once('plugins/docman/lib/html/DocmanTemplate.php');
		$result['template'] = new DocmanTemplate();

		if($section == self::SECTION){
			$result['params'] = array(
				'project' => $project,
				'title'=>$Lang->getText('Monitor.UsersMonitoringPlugin'),
			);
		} else if($section == DocmanFile::SECTION){
			$result['params'] = array(
				'project' => $project,
				'title'=>$Lang->getText('Monitor.UsersMonitoringItem')
			);
		}

		$pr = new PluginResult($this);
		$pr->setResult($result);

		return $pr;
	}


	/**
	 * returns the part of the html text that goes into the my page area, for monitored items
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageMonitors($params) {	
		$con = Propel::getConnection(DocmanFilePeer::DATABASE_NAME);
		$theme = $params['theme'];

		//get the assigned documents for the user
		$user = Session::getSession()->getUser();
		$user_id = $user->getPrimaryKey();

		$sql = "(SELECT '0' as docman_file_id, p.unix_name as description, p.project_id
					FROM monitor m, project p
					WHERE m.user_id = '$user_id' AND m.section='".DocmanPlugin::SECTION."'
					AND m.ref_id=p.project_id
					AND p.project_id NOT IN (
						SELECT p.project_id
						FROM docman_file dfi, docman_folder dfo, monitor m, project p
						WHERE m.user_id = '$user_id' AND m.section='".DocmanFile::SECTION."'
						AND m.ref_id=dfi.docman_file_id AND dfi.docman_folder_id=dfo.docman_folder_id
						AND dfo.project_id=p.project_id)
					ORDER BY p.project_name ASC)
					UNION
					(SELECT dfi.docman_file_id, dfi.description, p.project_id
					FROM docman_file dfi, docman_folder dfo, monitor m, project p
					WHERE m.user_id = '$user_id' AND m.section='".DocmanFile::SECTION."'
					AND m.ref_id=dfi.docman_file_id AND dfi.docman_folder_id=dfo.docman_folder_id
					AND dfo.project_id=p.project_id
					ORDER BY p.project_name ASC)";

		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

		$value = '<h3>'.Language::getSessText('Docman.Monitoring').'</h3>';

		$theader = array();
		$theader[] = 'Project';
		$theader[] = Language::getSessText('My.DocmanFiles');
		$last_project = null;
		if ($rs->getRecordCount()>0) {
			$theader[] = '';
			$value .= $theme->tableTop($theader);
			$ttype=array();
			while($rs->next()) {
				$ttype[$rs->getInt('project_id')][] = '1';
			}
			$rs->beforeFirst();	
			while($rs->next()) {
				$trow = array();
				$tmod = array();
				$project = ProjectPeer::retrieveByPk($rs->getInt('project_id'));
				$project_name = $project->getProjectName();
				$project_unix_name = $project->getUnixName();
				$description = StringUtils::trimString($rs->getString('description'), 40);
				$docman_file_id = $rs->getString('docman_file_id');
				if ($last_project != $rs->getInt('project_id')) {
					$row = '<a href="' . GFUrl::getUrl()->projectURL($project_unix_name) . '" >' .  $project_name . '</a>';
					if (DocmanPlugin::userIsMonitoring($user, $project)) {
						$row .= '<br />';
						$row .= Language::getSessText('My.MonitoringDocman');
						$row .= ' (<a href="'.GFUrl::getUrl()->getMonitorUrl(DocmanPlugin::SECTION, $project->getProjectId(), null, false).'">'.Language::getSessText('My.StopMonitoringDocman').'</a>)';
					}
					$trow[] = $row;
					$tmod[]='rowspan="'.count($ttype[$rs->getInt('project_id')]).'" bgcolor="#FFFFFF" width="30%"';
				}
				$last_project = $rs->getString('project_id');	
				if ($rs->getString('docman_file_id') <> '0') {
					$trow[] = '<a href="' . GFUrl::getUrl()->projectURL($project_unix_name,'',$this->getName(),'',array('file_id'=>$docman_file_id)) . '" >' .  $description . '</a>';
					$tmod[] = '';
					$trow[] = '<a href="'.GFUrl::getUrl()->getMonitorUrl(DocmanFile::SECTION, $docman_file_id, null, false).'" >'.Language::getSessText('DocmanFile.StopMonitoring').'</a>';
					$tmod[] = 'align="right"';
				} else {
					$trow[] = '';
					$trow[] = '';
				}
				$value .= $theme->tableRow($trow, $tmod);
			}
		} else {
			$value .= $theme->tableTop($theader);
			$trow = array();
			$trow[] = Language::getSessText('My.NoMonitoredDocuments');
			$tmod[]='colspan="2"';
			$value .= $theme->tableRow($trow,$tmod);
		}
		$value .= $theme->tableBottom();
		
		$result = new PluginResult($this);
		$result->setResult($value);
		return $result;		
	}
	
	private function getProjectAuditTrail($params) {
		$project = $params['project'];
		$audit_rs = $params['audit_trail'];		// This is a recordset
		
		$section = $audit_rs->getString('section');
		$field_name = $audit_rs->getString('field_name');
		$new_value = $audit_rs->getString('new_value');
		
		if ($section != Project::SECTION) {
			// Not the project-level audit trail
			return;
		}
		
		switch ($field_name) {
		case DocmanFolder::SECTION:
			if ($new_value == '_deleted_') {
				$value = $audit_rs->getString('old_value');
				$desc = Language::getSessText('DocmanFolder.ProjectAudit.Deleted', array($value));
			} else {
				return;
			}
			break;
		case DocmanFile::SECTION:
			if ($new_value == '_deleted_') {
				$value = $audit_rs->getString('old_value');
				$desc = Language::getSessText('DocmanFile.ProjectAudit.Deleted', array($value));
			} else {
				return;
			}
			break;
		case DocmanFileVersion::SECTION:
			if ($new_value == '_deleted_') {
				$value = $audit_rs->getString('old_value');
				$desc = Language::getSessText('DocmanFileVersion.ProjectAudit.Deleted', array($value));
			} else {
				return;
			}
			break;
		default:
			return;
		}

		$result = new PluginResult($this);
		$result->setResult($desc);
		return $result;
	}

	/**
	 * Make the association (if necessary) between the SCM commit and the docman file
	 */
	private function scmCommit($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$scmcommit = $params['commit'];
		$log = $scmcommit->getMessageLog();
		
		// $params['associations'] has all the associations
		
		foreach ($params['associations'] as $association) {
			list($section, $ref_id) = $association;
			if ($section != DocmanFile::SECTION) {
				continue;
			}
			
			$docman_file_id = $ref_id;
	
			// see if the item belongs to the project
			$docman_file = DocmanFilePeer::retrieveByPk($docman_file_id);
			if (is_null($docman_file)) continue;
			
			if ($docman_file->getDocmanFolder()->getProjectId() != $project->getProjectId()) {
				// The referenced file is not part of this project
				continue;
			}
			
			// Message belongs to the project. Add entry to the association table.
			try {
				$association = new Association();
				$association->setSection(ScmCommit::SECTION);
				$association->setRefId($scmcommit->getPrimaryKey());
				$association->setToSection(DocmanFile::SECTION);
				$association->setToRefId($docman_file_id);
				$association->setComment($scmcommit->getMessageLog());
				$association->save();
			} catch (Exception $ex) {
				// This means the association is duplicated. Ignore the exception.
			}
		}
		
		return null;
	}
	
	function getProjectSettingsForm($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) {
			return;
		}
		
		$setting = DocmanProjectSettingPeer::retrieveByPk($project->getProjectId());
		if (!$setting) {
			$show_associations_tab = true;
			$show_tags_tab = true;
			$allow_unprivileged_upload = false;
		} else {
			$show_associations_tab = $setting->getShowAssociationsTab();
			$show_tags_tab = $setting->getShowTagsTab();
			$allow_unprivileged_upload = $setting->getAllowUnprivilegedUpload();
		}
		
		$form = array();
		$form['title'] = Language::getSessText('Docman.ProjectSettingsTitle');
		$form['contents'] = '
			<input type="checkbox" name="docman_show_associations_tab" '.($show_associations_tab ? 'checked="checked"' : '').'/> '.Language::getSessText('Tabs.ShowAssociationsTab').' <br />
			<input type="checkbox" name="docman_show_tags_tab" '.($show_tags_tab ? 'checked="checked"' : '').'/> '.Language::getSessText('Tabs.ShowTagsTab').' <br />
			<input type="checkbox" name="docman_allow_unprivileged_upload" '.($allow_unprivileged_upload ? 'checked="checked"' : '').'/> '.Language::getSessText('Docman.AllowUnprivilegedUpload').' <br />
		';
		
		$result = new PluginResult($this);
		$result->setResult($form);
		return $result;
	}
	
	function saveProjectSettings($params) {
		$project = $params['project'];
		$setting = DocmanProjectSettingPeer::retrieveByPk($project->getProjectId());
		if (!$setting) {
			$setting = new DocmanProjectSetting();
			$setting->setProject($project);
		}
		
		$setting->setShowAssociationsTab( (isset($_POST['docman_show_associations_tab']) ? true : false) );
		$setting->setShowTagsTab( (isset($_POST['docman_show_tags_tab']) ? true : false) );
		$setting->setAllowUnprivilegedUpload( (isset($_POST['docman_allow_unprivileged_upload']) ? true : false) );
		$setting->save();
	}
	
	public function handleFileUpload($params) {
		if ($params['section'] != $this->getName()) {
			return;
		}

		$uploaded_file = $params['uploaded_file'];
		$file_name = urldecode(StringUtils::removeInvalidChars($params['file_name'], false));
		$size = $params['size'];
		$type = $params['type'];
		$project_id = $params['ref_id'];		//[Marcelo] Shouldn't we put the folder_id here and get the project_id from there?

		$folder_id = $_REQUEST['folder_id'];
		$file_id = intval($_REQUEST['file_id']);

		$session = Session::getSession();
		$Lang = $session->getLanguage();
		$project = ProjectPeer::retrieveByPk($project_id);

		//Check if the file already exists
		$con = Propel::getConnection(DocmanFolderPeer::DATABASE_NAME);
		$con->begin();

		$stmt = $con->createStatement();

		if ($file_id != 0) {									
			$sql = "SELECT docman_file_id from docman_file WHERE docman_file_id = " . $file_id;
		} else {
			$sql = "SELECT docman_file.docman_file_id, docman_file_version.docman_file_version_id
						FROM filesystem, docman_file, docman_file_version, docman_file_folder
						WHERE (
										filesystem.section = 'docmanfileversion' AND 
										filesystem.file_name = '" . $file_name . "' AND
										docman_file_version.docman_file_version_id = filesystem.ref_id AND
										docman_file.docman_file_id = docman_file_version.docman_file_id AND
										docman_file_folder.docman_file_id = docman_file.docman_file_id AND
										docman_file_folder.docman_folder_id = " . $folder_id . "
										)";
		}

		try {				
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		} catch(Exception $e) {
			error_log($e->getMessage());
		}
		$docman_file_id = 0;
		if ($rs->getRecordCount() > 0) {
			while ($rs->next()) {
				if ($file_id != 0) {
					$docman_file_id = $rs->getInt('docman_file_id');
					break;
				} else {
					$docman_file = DocmanFilePeer::retrieveByPk($rs->getInt('docman_file_id'));
					if (is_object($docman_file)) {
						$latest_version = $docman_file->getLatestVersion(true);
						if (is_object($latest_version) && $latest_version->getDocmanFileVersionId() == $rs->getInt('docman_file_version_id')) {
							$docman_file_id = $rs->getInt('docman_file_id');
							break;
						}
					}
				}
			}
		}

		//Get the folder object
		$DocmanFolder = DocmanFolderPeer::retrieveByPk($folder_id, false);

		$file_name = stripslashes($file_name);

		//Create a new file version
		$dfv = new DocmanFileVersion();
		$dfv->setFilename($file_name);
		$dfv->setChanges(Validator::sanitizeString(''));
		$dfv->setCreatedBy($session->getUser()->getUserId());
		//if user doesn't have create permission, set the file in the pending queue
		if ($DocmanFolder->userCanCreate()) {
			$dfv->setIsApproved(true);
		} else {
			$dfv->setIsApproved(false);
		}

		if ($docman_file_id > 0) {
			$DocmanFile = DocmanFilePeer::retrieveByPk($docman_file_id, false);

			$dfv->setDocmanFileId($docman_file_id);
			$dfv->save();
		}	else {
			$DocmanFile = new DocmanFile();
			$DocmanFile->setDocmanFolderId($folder_id);
			$DocmanFile->setDescription($file_name);
			$DocmanFile->setIsLocked(false);
			$DocmanFile->setLockedBy(100);
			$DocmanFile->setFilename($file_name);
			$DocmanFile->addDocmanFileVersion($dfv);
			$DocmanFile->save(null, true);
				
			//Now, assign the new file to the incoming folder
			$docman_file_folder = new DocmanFileFolder();
			$docman_file_folder->setDocmanFolderId($folder_id);
			$docman_file_folder->setDocmanFileId($DocmanFile->getDocmanFileId());
			$docman_file_folder->save(null, $folder_id);		
		}		

		//Get file type
		$type = SystemUtils::getMimeType($file_name);

		// create the file in the FS
		$newfile = new Filesystem();
		$newfile->setRefId($dfv->getPrimaryKey());
		$newfile->setSection(DocmanFileVersion::SECTION);
		$newfile->setPostedBy($session->getUser()->getUserId());	
		$newfile->setFileType($type);
		$newfile->setFileName($file_name);
		try {
			$newfile->save();
		}
		catch(Exception $e) {
			error_log(print_r($e,true));
		}

		try {
		$newfile->setFileDataFromPath($uploaded_file);
		} catch(Exception $e) {
			error_log(print_r($e,true));
		}

		$con->commit();

		if ($dfv->getIsApproved()) {
			$feedback = $Lang->getText('DocmanFile.AddFeedback');
		} else {
			$feedback = $Lang->getText('DocmanFile.AddQueuedFeedback');
		}

		$file_id = $DocmanFile->getDocmanFileId();
		$author = UserPeer::retrieveByPk($dfv->getCreatedBy());
		
		$arr_file = array();
		$arr_file['file_id'] = $file_id;
		//$arr_file['folder_id'] = $folder->getDocmanFolderId();
		//$arr_file['file_name'] = $file_name;
		//$arr_file['file_type'] = $newfile->getFileType();
		$arr_file['file_size'] = StringUtils::bytesToSize($newfile->getFileSize(), 2);
		$arr_file['author'] = $author->getFullname();
		$arr_file['version_number'] = $dfv->getVersionNumber();
		$arr_file['download_url'] = GFUrl::getURL()->getFullURL(GFUrl::getURL()->downloadURL(DocmanFileVersion::SECTION,$dfv->getDocmanFileVersionId(),$newfile->getFilesystemId(),$newfile->getFileNameSafe()));
		$arr_file['is_approved'] = $dfv->getIsApproved();
			
		$result = new PluginResult($this);
		$result->setResult($arr_file);
		return $result;
	}


	private function getDiscussionMessage($params) {
		if ($params['section'] != DocmanFile::SECTION) {
			return ;
		}

		$docman_file_id = $params['refid'];
		$docman_file = DocmanFilePeer::retrieveByPk($docman_file_id);
		if (!is_object($docman_file)) {
			return;
		}

		$url = $docman_file->getDocumentLink(false);
		$title = Language::getSessText('DocmanDiscussion.DiscussingDocument') . ' ' . $docman_file->getLatestVersion()->getFile()->getFileName();
		$description = $docman_file->getDescription();
		$download_url = $docman_file->getDownloadUrl();
		$modification_date = $docman_file->getLatestVersion()->getCreateDate();
		$version_number = $docman_file->getLatestVersion()->getVersionNumber();

		$str_body = LinkRenderer::renderHtmlDocumentSummaryTemplate($url, $description, $title, $download_url, $modification_date, $version_number);
		
		$pr = new PluginResult($this);
		$pr->setResult($str_body);  // Mark the request as handled
		return $pr;
	}

	private function doDocmanFileActivity($action, $file, $new = FALSE, $version = null) {

		$user_id = 100;
		$user_name = 'Anonymous User';
		if (Session::getSession()) {
			$user = Session::getSession()->getUser();
			if ($user != null) {
				$user_id = $user->getUserId();
				$user_name =  $user->getUnixName();
			}
		}

		$verb = $action;
		if ($verb == 'save') {
			$verb = $new ? 'create' : 'update';
		}

		$source = 'File '.$file->getDescription();
		$folder = $file->getDocmanFolder();
		
		// Don't include folder name if it's the root folder
		if ($folder->getFolderName() != 'incoming' ||
			$folder->getParentFolder() != null) {
			$source .=' in '.$folder->getFolderName();
		}
		if ($version != null) {
			$source = "Version {$version->getVersionNumber()} of {$source}";
		}

		$url = $file->getDocumentUrl();
		if ($action == 'delete' && $version == null) {
			// Entire file was deleted
			$url = '';
		}

		$description = "{$user_name} {$verb}d {$source}";

		if ($version != null) {
			$extended_data = PropelJson::ShallowJson($version);
		} else {
			$extended_data = PropelJson::ShallowJson($file);
		}

		$project = $file->getDocmanFolder()->getProject();

		if ($project != null) {
			$event = array(
				'project_id' => $project->getProjectId(),
				'section' => self::SECTION,
				'ref_id' => $project->getPrimaryKey(),
				'user_id' => $user_id,
				'activity_date' => time(),
				'description' => $description,
				'url' => $url,
				'activity_type' => $verb,
				'extended_data' => $extended_data
			);

			$result = ActionManager::notify('ACTIVITY_LOG_EVENT', $event);
		}

		$ret = new PluginResult($this);
		$ret->setResult(true);

		return $ret;
	}

	private function doDocmanFolderActivity($action, $folder, $new = FALSE) {

		$user_id = 100;
		$user_name = 'Anonymous User';
		if (Session::getSession()) {
			$user = Session::getSession()->getUser();
			if ($user != null) {
				$user_id = $user->getUserId();
				$user_name =  $user->getUnixName();
			}
		}

		$verb = $action;
		if ($verb == 'save') {
			$verb = $new ? 'create' : 'update';
		}

		$parent = $folder->getParentFolder();
		$project = $folder->getProject();
		$source = 'Folder '.$folder->getFolderName();

		if ($folder->getParentFolder() != null) {
			$source .=' in '.$parent->getFolderName();
		}

		$url = $folder->getFolderUrl();
		if ($action == 'delete') {
			$url = '';
		}

		$description = "{$user_name} {$verb}d {$source}";

		$extended_data = PropelJson::ShallowJson($folder);

		if ($project != null) {
			$event = array(
				'project_id' => $project->getProjectId(),
				'section' => self::SECTION,
				'ref_id' => $project->getPrimaryKey(),
				'user_id' => $user_id,
				'activity_date' => time(),
				'description' => $description,
				'url' => $url,
				'activity_type' => $verb,
				'extended_data' => $extended_data
			);

			$result = ActionManager::notify('ACTIVITY_LOG_EVENT', $event);
		}

		$ret = new PluginResult($this);
		$ret->setResult(true);

		return $ret;
	}
	
	private function renderDownloadUrl($params) {
		if ($params['section'] != DocmanFileVersion::SECTION) {
			return ;
		}

		$docman_file_version_id = $params['ref_id'];
		$docman_file_version = DocmanFileVersionPeer::retrieveByPk($docman_file_version_id);
		if (!is_object($docman_file_version)) {
			return;
		}

		$docman_file = $docman_file_version->getDocmanFile();

		$url = $docman_file->getDocumentLink(false);
		$title = Language::getSessText('DocmanDiscussion.DiscussingDocument') . ' ' . $docman_file->getLatestVersion()->getFile()->getFileName();
		$description = $docman_file->getDescription();
		$download_url = $params['url'];
		$modification_date = $docman_file_version->getCreateDate();
		$version_number = $docman_file_version->getVersionNumber();

		$params = array();
		$params['docman_file_id'] = $docman_file->getDocmanFileId();
		$params['description'] = $description;
		$params['filename'] = $docman_file_version->getFile()->getFileName();
		$params['url'] = $url;
		$params['download_url'] = $download_url;
		$params['modification_date'] = $docman_file_version->getCreateDate();
		$params['version_number'] = $docman_file_version->getVersionNumber();

		$str_body = LinkRenderer::renderDocumentVersionDetailsTemplate($params);
		
		$pr = new PluginResult($this);
		$pr->setResult($str_body);  // Mark the request as handled
		return $pr;
		
	}

	function getProjectEvent($params) {
		if ($params['section'] != DocmanPlugin::SECTION) {
			return;
		}

		$action = $params['activity_type'];
		switch ($action) {
			case 'create':
				$activity_desc = $params['description'];
				$body = '<a href="'.$params['url'].'">'.$activity_desc.'</a>';
				break;
			case 'delete':
				$activity_desc = $params['description'];
				$body = $activity_desc;
				break;
			case 'update':
				$activity_desc = $params['description'];
				$body = '<a href="'.$params['url'].'">'.$activity_desc.'</a>';
				break;
			default:
				$body = $activity_desc;
			break;
		}

		$pr = new PluginResult($this);
		$pr->setResult($body);		// Mark the request as handled
		return $pr;
	}
}
