<?php

	class DocmanTreeRender {
		private $parser;
		private $paginator;
		private $nested_folders;
		private $rendered_rows;
		private $incoming_folder_id;
		
		function __construct ($nested_folders, $parser) {
			$this->nested_folders = $nested_folders;
			$this->parser = $parser;
			$this->rendered_table = "";

			//Get "home" folder
			$c = new Criteria();
			$c->add(DocmanFolderPeer::FOLDER_NAME, 'incoming');
			$c->add(DocmanFolderPeer::PROJECT_ID, GFUrl::getUrl()->getProject()->getProjectId());
			$docman_folder = DocmanFolderPeer::doSelectOne($c);
			$this->incoming_folder_id = $docman_folder->getDocmanFolderId();
		}

		
		function renderDocmanTree($parent_id, $folder_attributes) {		
			if (isset($this->nested_folders[$parent_id])) {
			for($i=0; $i < sizeof($this->nested_folders[$parent_id]); $i++) {
			 	$folder = $this->nested_folders[$parent_id][$i];
			 	if ($folder->userCanRead()) {
				 	$folder_attributes_thislevel =  $folder_attributes . ' folder_' . $folder->getDocmanFolderId() .  '="1"';
				 	$this->renderFolderRow($folder, $folder_attributes_thislevel);
				 	$this->renderDocmanTree($folder->getDocmanFolderId(), $folder_attributes_thislevel);
			 	}
			}
			}
		}


		function renderTable() {
			return '<table class="cls_docman_folder_table" id="docman_folder_table">
				  <tbody>'
				  . $this->rendered_rows .
				  '</tbody>
				  </table>';
				  
		}


		function renderFolderRow($folder, $folder_attributes) {
			//$folder_class = ($folder->getParentFolderId() != 0) ? 'child-of-node-' . $folder->getParentFolderId() : '';				
			$folder_public = '';
			if ($this->incoming_folder_id == $folder->getDocmanFolderId()) {
				$folder_class = 'folder_root';
				$folder_name = 'Home';
			} else {
				$folder_name = $folder->getFolderName();
				if ($folder->getParentFolderId() == 0) {
					$folder_class = 'child-of-node-' . $this->incoming_folder_id;
					if ($folder->isPublic()) {
						$folder_public = '&nbsp;<span style="font-size: 0.8em;">(public folder)</span>';
					} else {
						$folder_public = '&nbsp;<span style="font-size: 0.8em;">(private folder)</span>';
					}
				} else {
					$folder_class = 'child-of-node-' . $folder->getParentFolderId();	
				}
				
			}	
			
			if ($folder->getIsPublic()) {
				$is_public = '1';	
			} else {
				$is_public = '0';	
			}

			if ($folder->getTopParentFolderId() == $folder->getDocmanFolderId() && $folder->getFolderName() == 'incoming') {
				$this->rendered_rows = '
	      <tr id="node-' . $folder->getDocmanFolderId() . '" class="' . $folder_class . '" ' . $folder_attributes . ' ispublic="' . $is_public . '">
	        <td><span fname="' . $folder_name . '" id="folder-' . $folder->getDocmanFolderId() . '" class="folder menu_folder_root_panel"></span><span><a href="javascript:;" onclick="docmanFolder.loadSearchResult(\'' . $folder->getFolderName() . '\', false);" id="node-name' . $folder->getDocmanFolderId() . '">' . $folder_name . '</a></span>' . $folder_public . '</td>
	      </tr>' . $this->rendered_rows;
	    } else {
				$this->rendered_rows .= '
	      <tr id="node-' . $folder->getDocmanFolderId() . '" class="' . $folder_class . '" ' . $folder_attributes . ' ispublic="' . $is_public . '">
	        <td><span fname="' . $folder_name . '" id="folder-' . $folder->getDocmanFolderId() . '" class="folder child_folder menu_folder_panel"></span><span><a href="javascript:;" onclick="docmanFolder.loadSearchResult(\'' . $folder->getFolderName() . '\', false);" id="node-name' . $folder->getDocmanFolderId() . '">' . $folder_name . '</a></span>' . $folder_public . '</td>
	      </tr>';
	    }
		}
	
	}


	
	
	class DocmanTreeCheckboxRender {
		private $parser;
		private $paginator;
		private $nested_folders;
		private $rendered_tree;
		private $folder_ids;
		private $file_id;
		private $incoming_folder_id;

		function __construct ($nested_folders, $parser, $arr_folder_ids, $file_id) {
			$this->nested_folders = $nested_folders;
			$this->parser = $parser;
			$this->rendered_tree = "";
			$this->folder_ids = $arr_folder_ids;
			$this->file_id = $file_id;
			
			//Get "home" folder
			$c = new Criteria();
			$c->add(DocmanFolderPeer::FOLDER_NAME, 'incoming');
			$c->add(DocmanFolderPeer::PROJECT_ID, GFUrl::getUrl()->getProject()->getProjectId());
			$docman_folder = DocmanFolderPeer::doSelectOne($c);
			$this->incoming_folder_id = $docman_folder->getDocmanFolderId();
		}

		
		function renderFolderRow($parent_id, $level) {
			$thislevel_rendered_nodes = '';

			if (isset($this->nested_folders[$parent_id])) {
				$str_child_nodes = '';
				for($i=0; $i < sizeof($this->nested_folders[$parent_id]); $i++) {
				 	$folder = $this->nested_folders[$parent_id][$i];
				 	
				 	if ($folder->getDocmanFolderId() == $this->incoming_folder_id) {
				 		continue;
				 	}
				 	
				 	if ($folder->userCanRead()) {
					 	$str_child_nodes = $this->renderFolderRow($folder->getDocmanFolderId(), $level++);
				 	}
				
				$checked = '';
				$str_img_checked = 'unchecked';
				$str_checked_color = '#A3A3A3';

				if (in_array($folder->getDocmanFolderId(), $this->folder_ids)) {
					$checked = ' checked="yes"';
					$str_img_checked = 'checked';
					$str_checked_color = '#1C1C1C';
				}

				$thislevel_rendered_nodes .= '
				<ul>
					<li id="folder-assign-' . $folder->getDocmanFolderId() . '" style="color: ' . $str_checked_color . ' ;" is_checked="' . $str_img_checked . '"><a href="javascript:;" onclick="docmanFolder.assignFileToFolder(' . $folder->getDocmanFolderId() . ', ' . $this->file_id . ')"><img src="/themes/gforge5/images/checkbox_' . $str_img_checked . '.png" id="img-folder-assign-' . $folder->getDocmanFolderId() . '"></a>&nbsp;<label>' . $folder->getFolderName() . '</label>
						@child_nodes
					</li>
				</ul>';

				$thislevel_rendered_nodes = str_replace('@child_nodes', $str_child_nodes, $thislevel_rendered_nodes);
				}
				
			}
			
			return $thislevel_rendered_nodes;
		}

		
		function renderTree() {
			$checked = '';
			$str_img_checked = 'unchecked';
			$str_checked_color = '#A3A3A3';
			
			if (in_array($this->incoming_folder_id, $this->folder_ids)) {
				$checked = ' checked="yes"';
				$str_img_checked = 'checked';
				$str_checked_color = '#1C1C1C';
			}
			
			$incoming_folder = '
				<ul>
					<li id="folder-assign-' . $this->incoming_folder_id . '" style="color: ' . $str_checked_color . ' ;" is_checked="' . $str_img_checked . '"><a href="javascript:;" onclick="docmanFolder.assignFileToFolder(' . $this->incoming_folder_id . ', ' . $this->file_id . ')"><img src="/themes/gforge5/images/checkbox_' . $str_img_checked . '.png" id="img-folder-assign-' . $this->incoming_folder_id . '"></a>&nbsp;<label>Home</label>' .
						$this->renderFolderRow(0, 0)
					. '</li>
				</ul>';
			
			$this->rendered_tree = '<ul id="organizetree">' . $incoming_folder . '</ul>';
			return $this->rendered_tree;
		}

	
	}
	
	
	class DocmanFolderContentsRenderer {
		private $folder_id=0;
		private $project_id=0;
		private $docman_folders = array();
		private $docman_files_folder = array();
		private $docman_pending_files = array();
		private $folder_contents = array();
		private $incoming_id = 0;
		private $is_docman_admin = false;
		private $folder_permission = '';
		private $show_discuss_links = false;
		
		function __construct ($folder_id, $project_id, $incoming_id) {
			$this->folder_id = $folder_id;
			$this->project_id = $project_id;
			$this->incoming_id = $incoming_id;
			$this->folder_contents['folders'] = array();
			$this->folder_contents['files'] = array();
			$this->folder_contents['pending'] = array();
			
			//Get child folders
			if ($folder_id == 0) {
				$select_folder_id = $this->incoming_id;
			} else {
				$select_folder_id = $folder_id;
			}
			
			$this->folder_id = $select_folder_id;
			
			//Determine if user is docman admin
			$Project = ProjectPeer::retrieveByPk($project_id);
			$this->is_docman_admin = false;
			$perm = $Project->getPermission();
			if ($perm->check(DocmanPlugin::SECTION, 0, DocmanPlugin::ADMIN)) {
				$this->is_docman_admin = true;
			}
			
			//Get permission for this folder
			if ($select_folder_id != -1) {
				$docman_folder = DocmanFolderPeer::retrieveByPk($select_folder_id);
				if ($docman_folder->userCanCreate()) {
					$this->folder_permission = 'w';
				} else {
					if ($docman_folder->userCanRead()) {
						$this->folder_permission = 'r';
					}
				}
			} else {
				$this->folder_permission = 'w';
			}
			
			$c = new Criteria();
			$c->add(DocmanFolderPeer::PARENT_FOLDER_ID, $folder_id);
			$c->add(DocmanFolderPeer::PROJECT_ID, $project_id);
			$this->docman_folders = DocmanFolderPeer::doSelect($c);
			
			//Get child files
			$c = new Criteria();
			$c->add(DocmanFileFolderPeer::DOCMAN_FOLDER_ID, $select_folder_id);
			$this->docman_files_folder = DocmanFileFolderPeer::doSelect($c);
			
			if ($this->is_docman_admin) {
				//Get pending for approval files
				$c = new Criteria();
				$c->add(DocmanFileVersionPeer::IS_APPROVED, false);
				$c->addJoin(DocmanFileVersionPeer::DOCMAN_FILE_ID, DocmanFilePeer::DOCMAN_FILE_ID);
				$c->addJoin(DocmanFilePeer::DOCMAN_FOLDER_ID, DocmanFolderPeer::DOCMAN_FOLDER_ID);
				$c->add(DocmanFolderPeer::PROJECT_ID, $project_id);
				//$this->docman_pending_files = DocmanFilePeer::doSelect($c);
				$this->docman_pending_files = DocmanFileVersionPeer::doSelect($c);
			}

			$this->show_discuss_links = (Session::getSession() && Session::getSession()->isLoggedIn() && $Project && $Project->usesPlugin('forum')) ? 'true' : 'false';
		}
		
		function getFolderContents() {
			if ($this->folder_id == -1 && $this->is_docman_admin) {
				$this->renderPendingQueue();
			} else {
				$this->renderFolderFiles();
				$this->renderFolderSubfolders();
				if ($this->folder_id == $this->incoming_id) {
					$this->renderPendingQueue();
				}
			}
			return ($this->folder_contents);
		}
		

		function renderPendingQueue() {
			if (sizeof($this->docman_pending_files) > 0) {
				foreach($this->docman_pending_files as $docmanFileVersion) {
					$this->renderPendingFileVersions($docmanFileVersion);
				}
			}
		}

		
		function renderFolderFiles() {
			foreach($this->docman_files_folder as $docmanFileFolder) {
				if (!is_object($docmanFileFolder)) {
					continue;
				}
				$docmanFile = DocmanFilePeer::retrieveByPk($docmanFileFolder->getDocmanFileId());
				if (is_object($docmanFile)) {
					$this->renderFile($docmanFile);
				}
			}
		}

		
		function renderFile($file) {
			//Get download url
			$latest_version = $file->getLatestVersion();
			if (!is_object($latest_version)) {
				return;
			}
			$latestfile = $latest_version->getFile();

			$file_version = $file->getLatestVersion();
			$download_url = GFUrl::getURL()->getFullURL(GFUrl::getURL()->downloadUrlFromFS($latestfile,true));
			$strict_download_url = GFUrl::getURL()->getFullURL(GFUrl::getURL()->downloadUrlFromFS($latestfile,false));

			$c = new Criteria();
			$c->add(FilesystemPeer::SECTION, 'docmanfileversion');
			$c->add(FilesystemPeer::REF_ID, $file_version->getDocmanFileVersionId());
			$filesystem = FilesystemPeer::doSelectOne($c);

			$downloads = $file->getLatestVersion()->getFile()->getDownloadCount();
			$file_type = ' filetype="' . $filesystem->getFileType() . '"';

			$user = UserPeer::retrieveByPk($filesystem->getPostedBy());
			if (is_object($user)) {
				$author = $user->getFullname();
			}

			$str_user_lock = '';
			if ($file->getIsLocked()) {
				$is_locked = true;
				$lock_img = "locked.png";
				$lock_class = "locked";
				
				//Get user that holds the lock
				$user_lock_id = $file->getLockedBy();
				$user_lock = UserPeer::retrieveByPk($user_lock_id);
				if (is_object($user_lock)) {
					$str_user_lock = '<br/>Locked By: ' . $user_lock->getFullName() . ' (' . $user_lock->getUnixname() . ')</label>';
				}
				
			} else {
				$is_locked = false;
				$lock_img = "unlocked.png";
				$lock_class = "unlocked";
			}
					
			$user = Session::getSession()->getUser();
			$monitor = false;
			if (is_object($user)) {
				if ($file->userIsMonitoring($user)) {
					$monitor = true;
					$img_monitoring = 'monitoring_on.png';
				} else {
					$monitor = false;
					$img_monitoring = 'monitoring_off.png';
				}
			} else {
				$img_monitoring = 'monitoring_off.png';
			}
					
			//Get file type icon
			$file_type_icon = "html";
			if (strpos($filesystem->getFileType(), "html") > 0) {
				$file_type_icon = "document";
			} else {
				if (strpos($filesystem->getFileType(), "pdf") > 0) {
					$file_type_icon = "pdf";
				} else {
					if (strpos($filesystem->getFileType(), "msword") > 0) {
						$file_type_icon = "document";
					} else {
						if (strpos($filesystem->getFileType(), "excel") > 0 || strpos($filesystem->getFileType(), "officedocument.spreadsheetml") > 0) {
							$file_type_icon = "document";
						} else {
							if (strpos($filesystem->getFileType(), "mage") > 0) {
								$file_type_icon = "graphic";
							} else {
								$file_type_icon = "document";
							}
						}
					}
				}
			}	
				
			$docman_file_array = array();
			$docman_file_array['file_id'] = $file->getDocmanFileId();
			$docman_file_array['description'] = $file->getDescription();
			$docman_file_array['file_name'] = $filesystem->getFileName();
			$docman_file_array['file_name_safe'] = $filesystem->getFileNameSafe();
			$docman_file_array['file_version'] = $file->getVersionsCount();
			$docman_file_array['file_type'] = $filesystem->getFileType();
			$docman_file_array['file_type_icon'] = $file_type_icon;
			$docman_file_array['file_size'] = StringUtils::bytesToSize($filesystem->getFileSize(),2);
			$docman_file_array['file_size_raw'] = $filesystem->getFileSize();
			$docman_file_array['downloads'] = $downloads;
			$docman_file_array['monitor'] = $monitor;
			$docman_file_array['img_monitoring'] = $img_monitoring;
			$docman_file_array['is_locked'] = $is_locked;
			$docman_file_array['lock_class'] = $lock_class;
			$docman_file_array['lock_img'] = $lock_img;
			$docman_file_array['str_user_lock'] = $str_user_lock;
			$docman_file_array['author'] = $author;
			$docman_file_array['download_url'] = $download_url;
			$docman_file_array['strict_download_url'] = $strict_download_url;
			$docman_file_array['is_pending'] = false;
			$docman_file_array['permission'] = $this->folder_permission;
			$docman_file_array['discuss'] = $this->show_discuss_links;
			
			if ($is_pending == false) {			
				$docman_file_array['is_pending'] = false;
				$this->folder_contents['files'][] = $docman_file_array;
			} else {
				$docman_file_array['is_pending'] = true;
				$this->folder_contents['pending'][] = $docman_file_array;
			}
		}


		function renderPendingFileVersions($file_version) {
			//Get download url
			$download_url = GFUrl::getURL()->getFullURL(GFUrl::getURL()->downloadUrlFromFS($file_version->getFile(),true));
			$strict_download_url = GFUrl::getURL()->getFullURL(GFUrl::getURL()->downloadUrlFromFS($file_version->getFile(),false));

			$c = new Criteria();
			$c->add(FilesystemPeer::SECTION, 'docmanfileversion');
			$c->add(FilesystemPeer::REF_ID, $file_version->getDocmanFileVersionId());
			$filesystem = FilesystemPeer::doSelectOne($c);

			$filesystem = $file_version->getFile();
			$file = $file_version->getDocmanFile();
			$downloads = $filesystem->getDownloadCount();
			$file_type = ' filetype="' . $filesystem->getFileType() . '"';

			$user = UserPeer::retrieveByPk($filesystem->getPostedBy());
			if (is_object($user)) {
				$author = $user->getFullname();
			}

			$str_user_lock = '';
			if ($file->getIsLocked()) {
				$is_locked = true;
				$lock_img = "locked.png";
				$lock_class = "locked";

				//Get user that holds the lock
				$user_lock_id = $file->getLockedBy();
				$user_lock = UserPeer::retrieveByPk($user_lock_id);
				if (is_object($user_lock)) {
					$str_user_lock = '<br/>Locked By: ' . $user_lock->getFullName() . ' (' . $user_lock->getUnixname() . ')</label>';
				}

			} else {
				$is_locked = false;
				$lock_img = "unlocked.png";
				$lock_class = "unlocked";
			}

			$user = Session::getSession()->getUser();
			$monitor = false;
			if (is_object($user)) {
				if ($file->userIsMonitoring($user)) {
					$monitor = true;
					$img_monitoring = 'monitoring_on.png';
				} else {
					$monitor = false;
					$img_monitoring = 'monitoring_off.png';
				}
			} else {
				$img_monitoring = 'monitoring_off.png';
			}

			//Get file type icon
			$file_type_icon = "html";
			if (strpos($filesystem->getFileType(), "html") > 0) {
				$file_type_icon = "document";
			} else {
				if (strpos($filesystem->getFileType(), "pdf") > 0) {
					$file_type_icon = "pdf";
				} else {
					if (strpos($filesystem->getFileType(), "msword") > 0) {
						$file_type_icon = "document";
					} else {
						if (strpos($filesystem->getFileType(), "excel") > 0 || strpos($filesystem->getFileType(), "officedocument.spreadsheetml") > 0) {
							$file_type_icon = "document";
						} else {
							if (strpos($filesystem->getFileType(), "mage") > 0) {
								$file_type_icon = "graphic";
							} else {
								$file_type_icon = "document";
							}
						}
					}
				}
			}

			$docman_file_array = array();
			$docman_file_array['file_id'] = $file_version->getDocmanFileVersionId();
			$docman_file_array['file_name'] = $filesystem->getFileName();
			$docman_file_array['file_name_safe'] = $filesystem->getFileNameSafe();
			$docman_file_array['file_version'] = 0;
			$docman_file_array['file_type'] = $filesystem->getFileType();
			$docman_file_array['file_type_icon'] = $file_type_icon;
			$docman_file_array['file_size'] = StringUtils::bytesToSize($filesystem->getFileSize(),2);
			$docman_file_array['file_size_raw'] = $filesystem->getFileSize();
			$docman_file_array['downloads'] = $downloads;
			$docman_file_array['monitor'] = $monitor;
			$docman_file_array['img_monitoring'] = $img_monitoring;
			$docman_file_array['is_locked'] = $is_locked;
			$docman_file_array['lock_class'] = $lock_class;
			$docman_file_array['lock_img'] = $lock_img;
			$docman_file_array['str_user_lock'] = $str_user_lock;
			$docman_file_array['author'] = $author;
			$docman_file_array['download_url'] = $download_url;
			$docman_file_array['strict_download_url'] = $strict_download_url;
			$docman_file_array['is_pending'] = $is_pending;
			$docman_file_array['permission'] = $this->folder_permission;
			$docman_file_array['discuss'] = $this->show_discuss_links;

			$docman_file_array['is_pending'] = true;
			$this->folder_contents['pending'][] = $docman_file_array;
		}

	
		function renderFolderSubfolders() {
			foreach($this->docman_folders as $docmanFolder) {
				if (($docmanFolder->getDocmanFolderId() != $this->incoming_id) && ($docmanFolder->userCanRead() || $docmanFolder->isPublic())) {	
					$this->renderFolder($docmanFolder);
				}
			}
		}
	

		function renderFolder($docmanFolder) {
			$user = Session::getSession()->getUser();
			$monitor = false;
			if (is_object($user)) {
				if ($docmanFolder->userIsMonitoring($user)) {
					$monitor = true;
					$img_monitoring = 'monitoring_on.png';
				} else {
					$monitor = false;
					$img_monitoring = 'monitoring_off.png';
				}
			} else {
				$img_monitoring = 'monitoring_off.png';
			}
			
			$docman_folder_array = array();
			$docman_folder_array['folder_id'] = $docmanFolder->getDocmanFolderId();
			$docman_folder_array['folder_name'] = $docmanFolder->getFolderName();
			$docman_folder_array['is_public'] = $docmanFolder->getIsPublic();
			$docman_folder_array['img_monitoring'] = $img_monitoring;
			//$docman_folder_array['is_locked'] = $is_locked;
			if ($docmanFolder->userCanUpdate()) {
				$docman_folder_array['permission'] = 'u';
			} else {
				if ($docmanFolder->userCanCreate()) {
					$docman_folder_array['permission'] = 'w';
				} else {
					$docman_folder_array['permission'] = 'r';
				}
			}
			$this->folder_contents['folders'][] = $docman_folder_array;
		}


		
	}





	class DocmanFolderSearchRenderer {
		private $file_ids = array();
		private $folder_contents = array();

		function __construct ($file_ids) {
			$this->file_ids = $file_ids;
			$this->folder_contents['files'] = array();
			$this->folder_contents['folders'] = array();
			$this->folder_contents['pending'] = array();
		}
		
		
		function renderSearchResults() {
			for($i=0; $i < sizeof($this->file_ids); $i++) {
				$docmanFile = DocmanFilePeer::retrieveByPk($this->file_ids[$i]);		
				$this->renderFile($docmanFile);
			}
			
			return $this->folder_contents;
		}

		
		function renderFile($file) {				
			//Get download url
			$latestfile = $file->getLatestVersion()->getFile();
			$file_version = $file->getLatestVersion();
			$download_url = GFUrl::getURL()->getFullURL(GFUrl::getURL()->downloadUrlFromFS($latestfile,true));
			$strict_download_url = GFUrl::getURL()->getFullURL(GFUrl::getURL()->downloadUrlFromFS($latestfile, false));

			$c = new Criteria();
			$c->add(FilesystemPeer::SECTION, 'docmanfileversion');
			$c->add(FilesystemPeer::REF_ID, $file_version->getDocmanFileVersionId());
			$filesystem = FilesystemPeer::doSelectOne($c);

			$downloads = $file->getLatestVersion()->getFile()->getDownloadCount();
			$file_type = ' filetype="' . $filesystem->getFileType() . '"';

			$user = UserPeer::retrieveByPk($filesystem->getPostedBy());
			if (is_object($user)) {
				$author = $user->getFullname();
			}

			$str_user_lock = '<label id="locked_by_label_' . $file->getDocmanFileId()  . '" style="font-size: .8em; color: #A2616E;"></label>';
			if ($file->getIsLocked()) {
				$is_locked = true;
				$lock_img = "locked.png";
				$lock_class = "locked";
				
				//Get user that holds the lock
				$user_lock_id = $file->getLockedBy();
				$user_lock = UserPeer::retrieveByPk($user_lock_id);
				if (is_object($user_lock)) {
					$str_user_lock = '<label id="locked_by_label_' . $file->getDocmanFileId()  . '" style="font-size: .8em; color: #A2616E;"><br/>Locked By: ' . $user_lock->getFullName() . ' (' . $user_lock->getUnixname() . ')</label>';
				}
				
			} else {
				$is_locked = false;
				$lock_img = "unlocked.png";
				$lock_class = "unlocked";
			}
					
			if ($file->userIsMonitoring($user)) {
				$monitor = true;
				$img_monitoring = 'monitoring_on.png';
			} else {
				$monitor = false;
				$img_monitoring = 'monitoring_off.png';
			}
					
			//Get file type icon
			$file_type_icon = "html";
			if (strpos($filesystem->getFileType(), "html") > 0) {
				$file_type_icon = "document";
			} else {
				if (strpos($filesystem->getFileType(), "pdf") > 0) {
					$file_type_icon = "pdf";
				} else {
					if (strpos($filesystem->getFileType(), "msword") > 0) {
						$file_type_icon = "document";
					} else {
						if (strpos($filesystem->getFileType(), "excel") > 0 || strpos($filesystem->getFileType(), "officedocument.spreadsheetml") > 0) {
							$file_type_icon = "document";
						} else {
							if (strpos($filesystem->getFileType(), "mage") > 0) {
								$file_type_icon = "graphic";
							} else {
								$file_type_icon = "document";
							}
						}
					}
				}
			}
					
			
				
			$docman_file_array = array();
			$docman_file_array['file_id'] = $file->getDocmanFileId();
			$docman_file_array['file_name'] = $filesystem->getFileName();
			$docman_file_array['file_name_safe'] = $filesystem->getFileNameSafe();
			$docman_file_array['file_version'] = $file->getVersionsCount();
			$docman_file_array['file_type'] = $filesystem->getFileType();
			$docman_file_array['file_type_icon'] = $file_type_icon;
			$docman_file_array['file_size'] = StringUtils::bytesToSize($filesystem->getFileSize(),2);
			$docman_file_array['file_size_raw'] = $filesystem->getFileSize();
			$docman_file_array['downloads'] = $downloads;
			$docman_file_array['monitor'] = $monitor;
			$docman_file_array['img_monitoring'] = $img_monitoring;
			$docman_file_array['is_locked'] = $is_locked;
			$docman_file_array['lock_class'] = $lock_class;
			$docman_file_array['lock_img'] = $lock_img;
			$docman_file_array['str_user_lock'] = $str_user_lock;
			$docman_file_array['author'] = $author;
			$docman_file_array['download_url'] = $download_url;
			$docman_file_array['strict_download_url'] = $strict_download_url;
			$docman_file_array['is_pending'] = false;
			
			//Get all folder for which this file belongs to
			$c = new Criteria();
			$c->add(DocmanFileFolderPeer::DOCMAN_FILE_ID, $file->getDocmanFileId());
			$docman_file_folders = DocmanFileFolderPeer::doSelect($c);
			
			$docman_file_array['folders'] = array();
			foreach($docman_file_folders as $docman_file_folder) {
				//Get the folder object
				$docman_folder = DocmanFolderPeer::retrieveByPk($docman_file_folder->getDocmanFolderId());
				if ($docman_folder->userCanRead()) {
					$arr_docman_folder = array();
					$arr_docman_folder['folder_id'] = $docman_folder->getDocmanFolderId();
					$arr_docman_folder['folder_name'] = $docman_folder->getFolderName();
					$docman_file_array['folders'][] = $arr_docman_folder;
				}
			}

			if ($file->checkMaximumAccessType(DocmanFolder::PERMLEVEL_WRITE)) {
				$docman_file_array['permission'] = 'w';
				$this->folder_contents['files'][] = $docman_file_array;
			} else {
				if ($file->checkMaximumAccessType(DocmanFolder::PERMLEVEL_READ)) {
					$docman_file_array['permission'] = 'r';
					$this->folder_contents['files'][] = $docman_file_array;
				}
			}


		}


		
	}



?>

