<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugins/docman/lib/DocmanFolderPeer.php');
require_once('plugins/docman/lib/DocmanFilePeer.php');
require_once('plugins/docman/lib/DocmanFileFolderPeer.php');
require_once('plugins/docman/lib/DocmanFileVersionPeer.php');
require_once('lib/core/FilesystemPeer.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('plugins/docman/lib/DocmanTreeRenderer.php');
require_once('plugins/docman/lib/FastJSON.php');

$session = Session::getSession();
$Lang = $session->getLanguage();
$project=GFUrl::getUrl()->getProject();

require_once('plugins/docman/lib/html/DocmanTemplate.php');
$theme = new DocmanTemplate();

if (!Session::getSession()->isLoggedIn()) {
	throw new PermissionDeniedException();
}

$operation = Validator::sanitizeString($_REQUEST['operation']);
$folder_id = Validator::sanitizeInt($_REQUEST['folder_id']);
$file_id = Validator::sanitizeInt($_REQUEST['file_id']);

$folder = DocmanFolderPeer::retrieveByPk($folder_id);
if (!is_object($folder)) {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = "Invalid Folder ID";
	echo FastJSON::encode($arr_feedback);
	exit;
}

if (!$folder->userCanCreate()) {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = "You don't have write permissions on that folder";
	echo FastJSON::encode($arr_feedback);
	exit;
}


$file = DocmanFilePeer::retrieveByPk($file_id);
if (!is_object($file)) {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = "Invalid File ID";
	echo FastJSON::encode($arr_feedback);
	exit;
}

//Get association object
$c = new Criteria();
$c->add(DocmanFileFolderPeer::DOCMAN_FOLDER_ID,$folder_id);
$c->add(DocmanFileFolderPeer::DOCMAN_FILE_ID,$file_id);
$docman_file_folder = DocmanFileFolderPeer::doSelectOne($c);

if($operation == 'add') {
	//If association does not exist, create it
	if (!is_object($docman_file_folder)) {	
		try {
			$docman_file_folder = new DocmanFileFolder();
			$docman_file_folder->setDocmanFolderId($folder_id);
			$docman_file_folder->setDocmanFileId($file_id);
			$docman_file_folder->save();
			
			TagEntryPeer::addTags(DocmanFile::SECTION, $folder_id, $folder->getFolderName());
					
			header('HTTP/1.1 200');
			$arr_feedback = array();
			echo FastJSON::encode($arr_feedback);
			exit;
		} catch(Exception $e) {
			header('HTTP/1.1 401');
			$arr_feedback = array();
			$arr_feedback["error_message"] = "";
			echo FastJSON::encode($arr_feedback);
			exit;
		}
	}
} elseif($operation == 'remove') {
		if (is_object($docman_file_folder)) {
			try {
				//Make sure this isn't the only folder-file relation
				$c = new Criteria();
				$c->add(DocmanFileFolderPeer::DOCMAN_FILE_ID, $file_id);
				$docman_file_folders = DocmanFileFolderPeer::doSelect($c);
				error_log("count: " . sizeof($docman_file_folders));
				if (sizeof($docman_file_folders) <= 1) {
					header('HTTP/1.1 401');
					$arr_feedback = array();
					$arr_feedback["error_message"] = "You can't delete all the associations";
					echo FastJSON::encode($arr_feedback);
					exit;	
				}
				
				$docman_file_folder->delete();
			
				//Remove tag, if it exists
        $tag = TagEntryPeer::findTag(DocmanFile::SECTION, $file_id, $folder->getFolderName());
				
				if(is_null($tag) || !is_object($tag)){
					//Don't do anything
				} else {
					$tag->delete();
				}
			
				header('HTTP/1.1 200');
				$arr_feedback = array();
				echo FastJSON::encode($arr_feedback);
				exit;
			} catch(Exception $e) {
				header('HTTP/1.1 401');
				$arr_feedback = array();
				$arr_feedback["error_message"] = "";
				echo FastJSON::encode($arr_feedback);
				exit;	
			}
		}		
} else {
	header('HTTP/1.1 401');
	$arr_feedback = array();
	$arr_feedback["error_message"] = "";
	echo FastJSON::encode($arr_feedback);
	exit;
}


?>
