<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

//
// This code lists Docman
//
require_once('plugins/docman/lib/html/DocmanTemplate.php');
require_once('plugins/docman/lib/InfoParser.php');
require_once('plugins/docman/lib/DocmanTreeRenderer.php');
require_once('plugins/docman/lib/DocmanFolder.php');
require_once('plugins/docman/lib/DocmanFolderPeer.php');
require_once('plugins/docman/lib/DocmanFile.php');
require_once('plugins/docman/lib/DocmanFilePeer.php');
require_once('lib/exceptions/PermissionDeniedException.php');
require_once('html/Paginator.php');
require_once('html/HTMLWidget.php');
require_once('html/ProjectTemplate.php');
require_once('external/Browser.php');
require_once('plugins/docman/lib/FastJSON.php');

$session = Session::getSession();
$Lang = $session->getLanguage();
$template = new ProjectTemplate();

//Get current user data
$user = $session->getUser();
$user_full_name = '';
$user_unix_name = '';
if (is_object($user)) {
	$user_full_name = $user->getFullName();
	$user_unix_name = $user->getUnixName();
}

$theme = new DocmanTemplate();
$project = GFUrl::getUrl()->getProject();

//Parse url
$url = GFUrl::getUrl();
$url->parseRequest();

//Get document id to display
$file_id = isset($_GET['file_id']) ? intval($_GET['file_id']) : 0;
$docmanfolder_id = isset($_GET['docmanfolder_id']) ? intval($_GET['docmanfolder_id']) : 0;

//Get breadcrumb data from url
if ($docmanfolder_id) {
	$breadcrumb_data = DocmanFolderPeer::parseURLFolderPath(DocmanFolderPeer::getUrlFromDocmanFolderId($docmanfolder_id), $project);
} else {
	$breadcrumb_data = DocmanFolderPeer::parseURLFolderPath($url->getRelativeURI(), $project);
}

//If there's a filename in the url, get the download url for that file
$download_url = '';
if (isset($breadcrumb_data['file_info']['filename'])) {
	$file = DocmanFilePeer::retrieveByPk($breadcrumb_data['file_info']['docman_file_id']);
	$file_version = $file->getLatestVersion();
	$latestfile = $file_version->getFile();
	$download_url = GFUrl::getURL()->getFullURL(GFUrl::getURL()->downloadURL(DocmanFileVersion::SECTION,$file_version->getDocmanFileVersionId(),$latestfile->getFilesystemId(),$latestfile->getFileNameSafe()));
}

//Determine user global perms
$perm = $project->getPermission();
if ($perm->check(DocmanPlugin::SECTION, 0, DocmanPlugin::ADMIN)) {
	$docman_wide_perms = 'true';
} else {
	$docman_wide_perms = 'false';
}

//Determine browser version
$browser = new Browser();
$browser_name = strtolower($browser->getBrowser());
$browser_version = floatval($browser->getVersion());
$can_drag_drop = false;
$dragdrop_browser = false;
$dragdrop_mode = false;

//Determine if it's an HTML5 compliant browser and allow drag'n drop
if ( ($browser_name == 'firefox' && $browser_version >= 4) || ($browser_name == 'chrome' || $browser_name == 'safari') ) {
	//if ($file_id == 0) {
	if ($file_id >= 0) {
		$can_drag_drop = true;
	}
} else {
	$dragdrop_browser = true;
}

//Creates the default "incoming" folder
$c = new Criteria();
$c->add(DocmanFolderPeer::PROJECT_ID,$project->getProjectId());
$c->add(DocmanFolderPeer::FOLDER_NAME,'incoming'); // default master folder
$c->add(DocmanFolderPeer::PARENT_FOLDER_ID,0);
$incoming = DocmanFolderPeer::doSelectOne($c);
if (!is_object($incoming)) {
	$incoming = new DocmanFolder();
	$incoming->setFolderName('incoming');
	$incoming->setProject($project);
	$incoming->save();
}

//Check if unprivileged upload is available
$setting = DocmanProjectSettingPeer::retrieveByPk($project->getProjectId());
if (is_object($setting)) {
	$allow_unprivileged_upload = $setting->getAllowUnprivilegedUpload();
} else {
	$allow_unprivileged_upload = false;
}

//Get current folder
if (sizeof($breadcrumb_data['path_info']) > 0)  {
	$current_folder = DocmanFolderPeer::retrieveByPk($breadcrumb_data['path_info'][sizeof($breadcrumb_data['path_info'])-1]['docman_folder_id']);
	if (!is_object($current_folder)) {
		$current_folder = $incoming;
	}
} else {
	$current_folder = $incoming;
}

//Get permission on the incoming (root) folder
if ($incoming->userCanCreate()) {
	$incoming_folder_perms = 'w';
} else {
	if ($incoming->userCanRead()) {
		$incoming_folder_perms = 'r';
	} else {
		$incoming_folder_perms = 'n';
	}
}

//Get permission on the current folder
if ($current_folder->userCanCreate()) {
	$current_folder_permission = 'w';
} else {
	if ($current_folder->userCanRead()) {
		$current_folder_permission = 'r';
	} else {
		$current_folder_permission = 'n';
	}
}

$redirect_to_root = false;
//If the user does NOT have read permission on the current folder (entered from the url), redirect him to the root folder
if ($current_folder_permission == 'n') {
	$redirect_to_root = true;
}


$params = array(
		'project' => $project,
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('Docman.BrowsePageTitle'),
		'enableUploads' => true,
		);
$theme->header($params);
$lang = Session::getSession()->getLanguage();


$parser = new InfoParser($project);
$nested_folders = $parser->getNestedFolders();

$folders = $nested_folders;
$docmanRenderer = new DocmanTreeRender($nested_folders, $parser);
$docmanRenderer->renderDocmanTree(0, '');
$rendered_table = $docmanRenderer->renderTable();
	
$flat_tree = false;

?>
<script type="text/javascript">

	function namespace(namespaceString) {
	    var parts = namespaceString.split('.'),
	        parent = window,
	        currentPart = '';    
	
	    for(var i = 0, length = parts.length; i < length; i++) {
	        currentPart = parts[i];
	        parent[currentPart] = parent[currentPart] || {};
	        parent = parent[currentPart];
	    }
	
	    return parent;
	}
	


</script>
<div style='display:none; visibility:hidden'>
<?php
	$CKEditor = new GFCKEditor(GFCKEditor::FULL);
	$CKEditor->editor('messagea', '');
	$CKEditor->setConnectorURLs($project->getProjectId());
?>
</div>

<?php
	include("js/DocmanCommonFunctions.php");
	include("js/DocmanFolderFunctions.php");
	include("js/DocmanFileFunctions.php");
?>

<script type="text/javascript">
	var docman_data = {
						unprivileged_upload : <?php $unpriv = ($allow_unprivileged_upload) ? 'true' : 'false'; echo $unpriv; ?>,
						root : {
							folder_id : <?php echo $incoming->getDocmanFolderId(); ?>,
							permission : '<?php echo $incoming_folder_perms; ?>',
							is_public : <?php echo $incoming->getIsPublic() ? 'true' : 'false'; ?>
						},
						current : {
							folder_id : 0,
							permission : '',
							is_public : false
						},
						folders : new Object(),
						files : new Object(),
						working : false
					};

	var show_file_id = <?php echo $file_id; ?>;
	var user_full_name = '<?php echo $user_full_name; ?>';
	var user_unix_name = '<?php echo $user_unix_name; ?>';
	var incoming_folder = <?php echo $incoming->getDocmanFolderId(); ?>;
	var incoming_folder_perms = '<?php echo $incoming_folder_perms; ?>';
	docman_data.current.folder_id = <?php echo $current_folder->getDocmanFolderId(); ?>;
	docman_data.current.permission = '<?php echo $current_folder_permission; ?>';
	var folder_stack = new Array();
	var docman_wide_perms = '';
	var base_docman_uri;
	var breadcrumb_data;
	var spinner_opts;
	var spinner_target;
	var redirect_to_root = '<?php echo $redirect_to_root; ?>';
	var docman_mode = 'navigate';
	var user_is_logged_in = <?php echo ((Session::getSession()->isLoggedIn()) ? 'true' : 'false'); ?>;
	var can_drag_drop = <?php echo ($can_drag_drop) ? 'true' : 'false'; ?>;
	function setUploadWidgetParameter() {
		FileUploadWidgetSetParameter('folder_id', docman_data.current.folder_id);
	}

	// Custom handler for file uploads
	function customFileUploadHandler(file) {
		docmanFile.handleFileUpload(file);
	}
	

var internal_action = false;

(function(window,undefined){

    // Prepare
    var History = window.History; // Note: We are using a capital H instead of a lower h
    if ( !History.enabled ) {
         // History.js is disabled for this browser.
         // This is because we can optionally choose to support HTML4 browsers or not.
        return false;
    }

    // Bind to StateChange Event
    History.Adapter.bind(window,'statechange',function(){ // Note: We are using statechange instead of popstate
        var State = History.getState(); // Note: We are using History.getState() instead of event.state
        History.log(State.data, State.title, State.url);

	      folder_stack = State.data.folder_stack;
				if (folder_stack.length > 0) {
					var folder_id = folder_stack[folder_stack.length-1].folder_id;	 
					if (folder_id != docman_data.current.folder_id) {
						docmanFolder.loadFolderFiles(folder_id, null, null, null, true);
					}
				}
    });

})(window);


$(document).ready(function() {
	$.ajaxSetup({
	    cache: false
	});
	
	if (incoming_folder_perms == 'n') {
		var params_dialog = {title: "Docman permission denied", perm_description : 'read'};
		docman.statusConfirm('docman-home-permission-dialog-template', params_dialog, {prevent_auto_fade : true});
		return;
	}
	
		var download_url = '<?php echo $download_url; ?>';

		if (download_url != '') {
			window.open (download_url,"Download","status=1");
		}
		
		//Get breadcrumb data
		breadcrumb_data = <?php echo FastJSON::encode($breadcrumb_data); ?>;

		//Build base docman uri
		base_docman_uri = '/gf/project/<?php echo $project->getUnixName(); ?>/docman/';

		docman_wide_perms = <?php echo $docman_wide_perms; ?>;

		var upload_url = '<?php echo GFUrl::getURL()->baseURL('', '', 'upload', array('section' => 'docman', 'ref_id' => $project->getProjectId())); ?>';
		FileUploadWidgetInit('widget_container', 'fileupload', upload_url, ['folder_id', 'file_id'], can_drag_drop);

    //Loads the files
    if (show_file_id != 0) {
    	docmanFile.loadFile(show_file_id);
  	} else {
	    if (docman_data.current.permission != 'n') {
		    if (breadcrumb_data.path_info.length > 0) {
		    	docmanFolder.loadBreadcrumbsFromUrl(breadcrumb_data);
		    } else {
		    	docmanFolder.loadFolderFiles(docman_data.current.folder_id);
		    }
			} else {
				var redirect = '<?php echo GFUrl::getUrl()->projectURL($project,'','docman','', array()); ?>';
				var params_dialog = {title: "Docman Permission", perm_description : 'read'};
	   		docman.statusConfirm('folder-permission-dialog-template', params_dialog, {prevent_auto_fade : true, redirect : redirect});
			}
		}

  	createFolderMenu();
  	createFolderMenuPanel();
  	createFileMenu();
  	
		spinner_opts = {
		  lines: 16, // The number of lines to draw
		  length: 10, // The length of each line
		  width: 5, // The line thickness
		  radius: 20, // The radius of the inner circle
		  color: '#000', // #rgb or #rrggbb
		  speed: 1, // Rounds per second
		  trail: 79, // Afterglow percentage
		  shadow: false, // Whether to render a shadow
		  hwaccel: false, // Whether to use hardware acceleration
		  className: 'spinner', // The CSS class to assign to the spinner
		  zIndex: 2e9, // The z-index (defaults to 2000000000)
		  top: 25, // Top position relative to parent in px
		  left: 'auto' // Left position relative to parent in px
		};
		spinner_target = document.getElementById('docman_files');	
  	
   
    //$(".menu_file").disableContextMenuItems();

    $(".cls_docman_folder_table").treeTable({
      initialState: "expanded"
    });   

		//Binds enter key to doSearch function on the search bar
		$('#words').keypress(function(e) {
			if (e.which == 13) {
				jQuery(this).blur();
				doSearch();
				return false;
			}
		});

<?php
	if (!$flat_tree && $can_drag_drop) {
?>
	docman.activateTreeRows();
<?php
	}
?>
    'use strict';

<?php
	if (!$flat_tree) {
?>
    // Initialize the jQuery File Upload widget:
<?php
				
		if ($can_drag_drop) {
?>
		$('#fileupload').fileupload({
	        dropZone: $(document)
		});


<?php
		} else {
?>
		$('#fileupload').fileupload({
	    optionsdropZone: $('fake_drop')
		});

<?php
		} 
?>

    // Load existing files:
    $.getJSON($('#fileupload form').prop('action'), function (files) {
        var fu = $('#fileupload').data('fileupload');
        fu._adjustMaxNumberOfFiles(-files.length);
    });

    // Open download dialogs via iframes, to prevent aborting current uploads:
    $('#fileupload .files a:not([target^=_blank])').live('click', function (e) {
        e.preventDefault();
        $('<iframe style="display:none;"></iframe>')
            .prop('src', this.href)
            .appendTo('body');
    });

<?php
	}
?>
  });

</script>
</head>
<body>

<div id="dialog-confirm" title="Test" style="display: none;">
	<p><span class="ui-icon ui-icon-alert" style="float:left; margin:0 7px 20px 0;"></span>These items will be permanently deleted and cannot be recovered. Are you sure?</p>
</div>

<div id="docman-modal-dialog" title="" style="display: none;">
</div>
<div id="docman-modal-dialog2" title="" style="display: none;">
</div>
<div id="docman-modal-dialog3" title="" style="display: none;">
</div>

<div id="fake_drop" style="display: none;">
</div>

<?php
	//Generate tooltip text;
	$tooltip_home = $Lang->getText('Docman.TooltipHome');
	$tooltip_organizefolders = $Lang->getText('Docman.TooltipOrganizeFolders');
	$tooltip_addfileinline = $Lang->getText('Docman.TooltipAddFileInline');
	$tooltip_createnewfolder = $Lang->getText('Docman.TooltipCreateNewFolder');
?>

<div>
	<div style="float: left; margin-top: 4px;">
		<a href="javascript:;" onclick="docmanFolder.jumpToFolderFromBreadcrumb(incoming_folder);" onMouseOver="tooltip('<?php echo $tooltip_home; ?>')" onmouseout="tooltipexit()"><img src="/themes/gforge5/images_docman/home_icon.png" /></a>
		<?php
			if ($docman_wide_perms == 'true') {
		?>
		&nbsp;&nbsp;
		<a id="trigger1" style="color: rgb(255, 255, 255);" href="javascript:;" onclick="docman.openOrganizeFolders();"  onMouseOver="tooltip('<?php echo $tooltip_organizefolders; ?>')" onmouseout="tooltipexit()"><img src="/themes/gforge5/images_docman/browse_folders_icon.png" /></a>
		<?php
			}
		?>
		&nbsp;&nbsp;
		<a id="trigger1" style="color: rgb(255, 255, 255);" href="javascript:;" onclick="docmanFolder.createFolder(docman_data.current.folder_id, 'Current Folder');"  onMouseOver="tooltip('<?php echo $tooltip_createnewfolder; ?>')" onmouseout="tooltipexit()"><img src="/themes/gforge5/images_docman/create_new_folder.png" /></a>
		&nbsp;&nbsp;
		<a href="javascript:;" onclick="showUpload();"  onMouseOver="tooltip('<?php echo $tooltip_addfileinline; ?>')" onmouseout="tooltipexit()"><img src="/themes/gforge5/images_docman/add_file_inline_icon.png" /></a>
	</div>

	<div id="docman_container" style="margin-left: 50px; float: left;">
		<div id="search">
			<form style="display: inline;" action="" method="POST">
				<table>
					<tr>
						<td>
							<span class="search_bar">
								<input id="words" name="words" type="text" size="40" placeholder="" />
							</span>
						</td>
						<td>
							<?php echo HTMLWidget::button("button",$lang->getSessText('SearchBox.search'),"do_search","","doSearch()"); ?>
						</td>
						<td>
							<input type="checkbox" id="search_contents_only" /><label><?php echo $lang->getSessText('Docman.SearchContentsOnly'); ?></label>
						</td>
					</tr>		
				</table>
			</form>	
		</div>
		<br/>
	</div>

</div>

<div style="clear: both;"></div>

<?php
if ($can_drag_drop) {
	$str_hint = $Lang->getText('Docman.DragDropSupported');
} else {
	if ($dragdrop_browser) {
		$str_hint = $Lang->getText('Docman.DragDropNotSupported');
	} elseif($dragdrop_mode) {
		$str_hint = $Lang->getText('Docman.DragDropNotAvailable');
	}
}
if (!empty($str_hint)) {
	$str_hint .= '<br>';
}
$str_hint .= $Lang->getText('Docman.Rightclick');
?>
<div class="infobox" style="margin-left: 0px;"><?php echo $str_hint; ?></div>

<?php
		include("forms/DocmanFolderDialogs.php");
		include("forms/DocmanFileDialogs.php");
?>

<div id="organize_folders" style="display: none;">
	<div id="organize_folders_inner" style="border: 1px #BDBDBD solid; padding-left: 20px;">
<?php 
	echo $rendered_table;
?>
	</div>
</div>



<div id="folder_breadcrumbs">
</div>

<div id="docman_files" class="" style="min-height: 70px;">
	<table width="100%" cellspacing="0" cellpadding="8" summary="" class="tablesorter" id="docman_table">
		<tbody> 
			<tr class="th">
				<th></th>
				<th>Name</th>
				<th></th>
				<th></th>
				<th>Size</th>
				<th>Type</th>
				<th>Author</th>
				<th>Version</th>
				<th>Downloads</th>
				</tr>
		</tbody>
	</table>
</div>

<!-- Note: this conflicts with the fileupload_wrapper div defined in Layout.php -->
<!--div id="fileupload_wrapper" style="display: none;" class="fileupload">
	<div id="widget_inline" style="display: none;">
		Click <a href="javascript:;" onclick="docmanFile.addFileInline();"  onMouseOver="tooltip('<?php echo $tooltip_addfileinline; ?>')" onmouseout="tooltipexit()">here</a> to create a file inline.
	</div>
	<br/>
	<div id="widget_container">
	</div>
</div-->
<?php
	$theme->footer($params);
?>

