<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('plugins/forum/lib/om/BaseForum.php');
require_once("plugin/ActionManager.php");
require_once('plugin/ActionManager.php');
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('core/Monitor.php');
require_once('plugins/forum/lib/ForumThread.php');
require_once('plugins/forum/lib/ForumMessage.php');
require_once('lib/core/Monitor.php');
require_once('plugins/userblog/lib/UserBlog.php');

/**
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.forum.lib
 */
class Forum extends BaseForum
{

    const SECTION = 'forum';
    const NOACCESS = -1;
    const READ = 0;
    const POST = 1;
    const ADMIN = 2;
    const NOANON = 0;
    const ANON = 1;
    const ISPRIVATE = 0;
    const ISPUBLIC = 1;
    const NO_MODERATION = 0;
    const MODERATE_NON_PROJECT_MEMBERS = 1;
    const MODERATE_NON_FORUM_ADMINS = 2;


    /** Name of the preference that holds the style in which the user prefers to view the forums */
    const PREFERENCE_THREAD_STYLE = 'forum_thread_style';
    const PREFERENCE_THREAD_LIMIT = 'forum_thread_limit';
    const PREFERENCE_SORT_ORDER = 'forum_sort_order';
    const PREFERENCE_SORT_BY = 'forum_sort_by';

    private $monitoring_users = null;
    private $force_save = false;
    private $forum_admins = null;

    /**
     * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
     * This method was added by propel_customizer.php.
     * 
     * @param Connection $con
     * @return void
     * @throws PropelException
     * @see Forum::setDeleted()
     * @see Forum::isDeleted()
     */
    function delete($con = null)
    {
        if (!$this->userCanDelete())
        {
            throw new PermissionDeniedException();
        }

        //delete monitors
        $crit = new Criteria();
        $crit->add(MonitorPeer::SECTION, self::SECTION);
        $crit->add(MonitorPeer::REF_ID, $this->getForumId());
        MonitorPeer::doDelete($crit);

        ActionManager::notify("FORUM_DELETE", $this);
        parent::delete($con);
    }

    function userCanDelete()
    {
        return $this->userIsAdmin();
    }

    /**
     * The forums are linked to many sections, so we must check if the forum is readable
     * by consulting the appropiate section
     * @todo
     */
    function userCanRead()
    {

        if ($this->userIsAdmin())
        {
            return true;
        }

        switch ($this->getSection())
        {
            case Project::SECTION:
                /** @var Project $project  */
                $project = ProjectPeer::retrieveByPk($this->getRefId());
                if (!is_object($project))
                {
                    return false;
                }

                if ($project->isPublic() && $this->isPublic())
                {
                    return true;
                }

                $perm = $project->getPermission();
                if ($perm->check(Forum::SECTION, 0, Forum::READ))
                {
                    return true;
                } else
                {
                    if ($perm->check(Forum::SECTION, $this->getForumId(), Forum::READ))
                    {
                        return true;
                    } else
                    {
                        return false;
                    }
                }
                break;

            case Mailman::SECTION:
                $c = new Criteria();
                $c->addJoin(MailmanPeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
                $c->add(MailmanPeer::MAILMAN_ID, $this->getRefId());
                /** @var Mailman[] $mailman  */
                $mailman = MailmanPeer::doSelectJoinProject($c);
                if (count($mailman) > 0)
                {
                    return $mailman[0]->userCanRead();
                } else
                {
                    return false;
                }
                break;

            case WikiPage::SECTION:
                /** @var WikiPage $wiki_page  */
                $wiki_page = WikiPagePeer::retrieveByPk($this->getRefId());
                if (!is_object($wiki_page))
                {
                    return false;
                }
                return $wiki_page->userCanRead();
                break;

            case News::SECTION:
                /** @var News $news  */
                $news = NewsPeer::retrieveByPk($this->getRefId());
                if (!$news)
                {
                    return false;
                }
                $project = $news->getProject();
                if ($project->isPublic())
                {
                    return true;
                }
                $user = Session::getSession()->getUser();
                if (!$user)
                {
                    return false;
                }
                return $user->isProjectMember($project);
                break;

            case UserBlog::SECTION :
                return Session::getSession()->isLoggedIn();
                break;

            case TrackerItem::SECTION:
                $tracker = TrackerItemPeer::retrieveByPk($this->getRefId());
                return $tracker->userCanRead();
                break;

            case DocmanFolder::SECTION:
                $docman = DocmanFolderPeer::retrieveByPk($this->getRefId());
                return $docman->userCanRead();
                break;

            case DocmanFile::SECTION;
                $docmanFile = DocmanFilePeer::retrieveByPk($this->getRefId());
                return $docmanFile->userCanRead();
                break;

            case FrsRelease::SECTION;
                $frsRelease = FrsReleasePeer::retrieveByPk($this->getRefId());
                return $frsRelease->userCanRead();
                break;

            default:
                return false;
        }
    }

    /**
     * If this person is an admin for this object
     * @todo	If it is not a project forum, call the appropiate section to check permissions
     */
    public function userIsAdmin()
    {
        if ($this->getSection() == 'project')
        {
            $project = ProjectPeer::retrieveByPk($this->getRefId());
            if (is_object($project))
            {
                $perm = $project->getPermission();
//				if ( ($perm->check(Forum::SECTION, $this->getForumId() , Forum::ADMIN)) || ($perm->check('forumadmin',0,1)) ) { // normal permission or project/site admin
                if ($perm->check(Forum::SECTION, 0, Forum::ADMIN))
                {
                    return true;
                }
                if ($perm->check(Forum::SECTION, $this->getForumId(), Forum::ADMIN))
                {
                    return true;
                }
            }
        } else if ($this->getSection() == 'mailman')
        {
            $c = new Criteria();
            $c->addJoin(MailmanPeer::PROJECT_ID, ProjectPeer::PROJECT_ID);
            $c->add(MailmanPeer::MAILMAN_ID, $this->getRefId());

            // This makes one query instead of two queries (one for mailman + 1 for project)
            $mailman = MailmanPeer::doSelectJoinProject($c);
            if (count($mailman) > 0)
            {
                // This should return only one object
                $project = $mailman[0]->getProject();
                return $project->userIsAdmin();
            }
        } else if ($this->getSection() == 'wikipage')
        {
            $wiki_page = WikiPagePeer::retrieveByPk($this->getRefId());
            if (!is_object($wiki_page))
            {
                return false;
            }
            return $wiki_page->userIsAdmin();
        } else if (Session::getSession()->isSiteAdmin())
        {
            return true;
        }

        return false;
    }

    /**
     * @param Connection $con
     * @return void
     * @throws PropelException
     */
    function save($con = null)
    {
        if (!$this->force_save)
        {
            // TODO: check if user can create forum.
            // $this->force_save is used when we must create for example a forum
            // for a news on the fly
        }


        // Validate all the data
        $errors = array();
        if (!$this->section)
        {
            $errors['section'] = Language::getSessText('Forum.EmptySection');
        }
        if (is_null($this->ref_id))
        {  // ref_id could be zero
            $errors['ref_id'] = Language::getSessText('Forum.InvalidRefId');
        }
        if (!$this->forum_name)
        {
            $errors['forum_name'] = Language::getSessText('Forum.EmptyForumName');
        } elseif ($this->isNew())
        {
            // check for duplicates
            $c = new Criteria();
            $c->add(ForumPeer::SECTION, $this->section);
            $c->add(ForumPeer::REF_ID, $this->ref_id);
            $c->add(ForumPeer::FORUM_NAME, $this->forum_name);
            $f = ForumPeer::doSelectOne($c);
            if (!is_null($f))
            {
                $errors['forum_name'] = Language::getSessText('Forum.DuplicatedForumName', array($this->forum_name));
            }
        }
        if ($this->send_all_posts_to)
        {
            $invalid_emails = Validator::validateEmailList($this->send_all_posts_to);
            if ((count($invalid_emails) > 0) && ($this->send_all_posts_to != 'NULL'))
            { // the db accepts NULL in this field
                $invalid_str = implode(', ', $invalid_emails);
                $errors['send_all_posts_to'] = Language::getSessText('Forum.InvalidEmailList', array($invalid_str));
            }
        }

        if (count($errors) > 0)
        {
            throw new InvalidDataException($errors);
        }

        // So far so good
        ActionManager::notify("FORUM_SAVE_BEFORE", $this);
        parent::save($con);

        $this->force_save = false;
        ActionManager::notify("FORUM_SAVE", $this);
    }

    /**
     * Just a shorthand for getIsPublic
     * @return bool
     */
    public function isPublic()
    {
        return $this->getIsPublic();
    }

    /**
     * Tells if a particular user can create a thread in this forum
     * @return bool
     */
    public function userCanPost(User $user = null)
    {
        if ($this->userIsAdmin())
        {
            return true;
        }

        switch ($this->getSection())
        {
            case Project::SECTION:
                $project = ProjectPeer::retrieveByPk($this->getRefId());
                if (!is_object($project))
                {
                    return false;
                }

                if ($project->isPublic() && $this->isPublic())
                {
                    return true;
                }

                $perm = $project->getPermission();
                if ($perm->check(Forum::SECTION, 0, Forum::POST))
                {
                    return true;
                } else
                {
                    if ($perm->check(Forum::SECTION, $this->getForumId(), Forum::POST))
                    {
                        return true;
                    } else
                    {
                        return false;
                    }
                }
                break;

            case Mailman::SECTION:
                return false;
                break;

            case WikiPage::SECTION:
                require_once('plugins/wiki/lib/WikiPlugin.php');
                require_once('plugins/wiki/lib/WikiPagePeer.php');
                $wiki_page = WikiPagePeer::retrieveByPk($this->getRefId());
                if (!is_object($wiki_page))
                {
                    return false;
                }
                return $wiki_page->isPublic();
                break;

            case News::SECTION:
                // Any user can post if it's a public project or if they are a member of the project
                $news = NewsPeer::retrieveByPk($this->getRefId());
                if (!$news)
                {
                    return false;
                }
                $project = $news->getProject();
                if ($project->isPublic())
                {
                    return true;
                }
                $user = Session::getSession()->getUser();
                if (!$user)
                {
                    return false;
                }
                return $user->isProjectMember($project);
                break;

            case UserBlog::SECTION:
                return Session::getSession()->isLoggedIn();
                break;

            case Tracker::SECTION:
                //Discussions should be allowed for members with read access to tracker items
                $tracker = TrackerPeer::retrieveByPk($this->getRefId());
                return $tracker->userCanRead();
                break;

	        case Trackeritem::SECTION:
		        //Discussions should be allowed for members with read access to tracker items
		        $tracker_item = TrackerItemPeer::retrieveByPk($this->getRefId());
		        return $tracker_item->userCanRead();
		        break;

            case DocmanFolder::SECTION:
                $docman = DocmanFolderPeer::retrieveByPk($this->getRefId());
                return $docman->userCanRead();
                break;

            case DocmanFile::SECTION:
                $docmanFile = DocmanFilePeer::retrieveByPk($this->getRefId());
                return $docmanFile->userCanRead();
                break;

            default:
                return false;
        }
    }

    /**
     * returns array of user objects who are forum admins for this forum
     */
    public function getForumAdmins()
    {
        if (!is_array($this->forum_admins))
        {
            $this->forum_admins = array();
            switch ($this->getSection())
            {
                case 'project':
                    {
                        $project = ProjectPeer::retrieveByPk($this->getRefId());

                        if (is_object($project))
                        {
                            $members = $project->getMembers();
                            foreach ($members as $user)
                            {
                                $perm = $project->getUserPermission($user);
                                if ($perm->check(Forum::SECTION, 0, Forum::ADMIN) || $perm->check(Forum::SECTION, $this->getForumId(), Forum::ADMIN))
                                {
                                    $this->forum_admins[] = $user;
                                }
                            }
                        }
                    }
            }
        }
        return $this->forum_admins;
    }

    /**
     * Tells if the user is monitoring this forum
     * @return bool
     */
    public function userIsMonitoring(User $user)
    {
        $c = new Criteria();
        $c->add(MonitorPeer::SECTION, self::SECTION);
        $c->add(MonitorPeer::REF_ID, $this->getForumId());
        $c->add(MonitorPeer::USER_ID, $user->getUserId());
        $mon = MonitorPeer::doSelectOne($c);
        return (!is_null($mon)) ? true : false;
    }

    /**
     * Returns an array of user objects corresponding to the ones that are monitoring this forum or thread or project
     *
     * @return array	Users
     */
    function getMonitoringUsers()
    {
        if (is_null($this->monitoring_users))
        {
            $this->monitoring_users = MonitorPeer::getMonitoringUsers(self::SECTION, $this->getForumId());
        }
        return $this->monitoring_users;
    }

    /**
     * Starts a thread as the current session's user
     */
    public function startThread($thread_name, $text)
    {
        $thread = new ForumThread();
        $message = new ForumMessage();

        //	$con=Propel::getConnection();
        //	$con->begin();

        $session = Session::getSession();
        if ($session->isLoggedIn())
        {
            $thread->setCreatedBy($session->getUser()->getUserId());
            $message->setCreatedBy($session->getUser()->getUserId());
        }
        $thread->setThreadName($thread_name);
        $message->setSubject($thread_name);
        $message->setBody($text);

        $thread->addForumMessage($message);
        $this->addForumThread($thread);
        $thread->save();

        //$this->save();
        //
		//      handle the files
        //
		if (array_key_exists('addfile', $_FILES))
        {
            for ($i = 0; $i < 5; $i++)
            {
                $error = $_FILES['addfile']['error'][$i];
                $filename = $_FILES['addfile']['name'][$i];

                if ($error > 0 && $error != 4)
                {
                    $fileerrors = StringUtils::getFileUploadErrorString($error, $filename);
                    throw new InvalidDataException($fileerrors);
                    continue;
                }

                $tmp_name = $_FILES['addfile']['tmp_name'][$i];
                if (!is_uploaded_file($tmp_name))
                {
                    continue;
                }
                $size = $_FILES['addfile']['size'][$i];
                $type = $_FILES['addfile']['type'][$i];

                try
                {
                    $fs = new Filesystem();
                    $fs->setSection(ForumMessage::SECTION);
                    $fs->setRefId($message->getForumMessageId());

                    $fs->setFileName($filename);
                    //$fs->setFileNameSafe();
                    $fs->setFileType($type);
                    $fs->setFileSize($size);
                    $fs->setPostedBy($session->getUser()->getUserId());
                    $fs->save();
                    $fs->setFileDataFromPath($tmp_name);
                } catch (Exception $e)
                {
                    // rollback if we couldn't create a file (write permissions)
                    //$con->rollback();
                    $err = array($e->getMessage());
                    throw new InvalidDataException($err);
                }
            }
        }
        //$con->commit();
        return $message;
    }

    /**
     * 	The UNIX time when the person last hit "save my place".
     * 	@return	int	The unix time.
     */
    public function getSavedDate($user)
    {
        $last_week = (time() - (60 * 60 * 24 * 7)) . '|' . (time() - (60 * 60 * 24 * 7));
        if (!isset($user) || !($user instanceof User))
        {
            return $last_week;
        }

        $saved = $user->getPreference('forumsvdt_' . $this->getForumId());
        if (!$saved)
            return $last_week;

        return $saved;
    }

    /**
     * Set the UNIX time in which all the messages where marked as 'read'
     * @param	User $user	User
     * @param	int $time	(optional) UNIX time to set. 0 means now.
     */
    public function setSavedDate(User $user, $time = 0)
    {
        if (!$time)
            $time = time();
        $user->setPreference('forumsvdt_' . $this->getForumId(), $time);
    }

    /**
     * Alias for ForumThreadPeer::getLatestThreads
     * @param	Criteria $c
     */
    public function getLatestThreads(Criteria $c)
    {
        $c->add(ForumThreadPeer::FORUM_ID, $this->forum_id);
        $c->add(ForumThreadPeer::MESSAGE_COUNT, 0, Criteria::GREATER_THAN);
        return ForumThreadPeer::getLatestThreads($c);
    }

    /**
     * Public function getLatestMessagesThreaded
     */
    public function getLatestMessagesThreaded()
    {
        return ForumMessagePeer::getLatestMessagesThreaded($this->getForumId());
    }

    public function getAllMessagesThreaded()
    {
        return ForumMessagePeer::getAllMessagesThreaded($this->getForumId());
    }

    /**
     * Get this forum's history sorted by year/month
     */
    public function getHistory()
    {
        return ForumMessagePeer::getForumHistory($this->getForumId());
    }

    /**
     * Get all the messages that were posted in this month/year
     */
    public function getMonthHistory($month, $year)
    {
        return ForumMessagePeer::getMonthHistory($this->getForumId(), $month, $year);
    }

    public function forceSave()
    {
        $this->force_save = true;
    }

    public function getTopThreads($limit = 5)
    {
        return ForumThreadPeer::getTopThreads($this, $limit);
    }

    public function getId()
    {
        return $this->getForumId();
    }

    public function getName()
    {
        return $this->getForumName();
    }

    public function getProject()
    {
        $project = null;
        switch ($this->getSection())
        {
            case "project":
                $project = ProjectPeer::retrieveByPk($this->getRefId());
                break;

            case "trackeritem":
                $tracker_item = TrackerItemPeer::retrieveByPk($this->getRefId());
                $project = $tracker_item->getTracker()->getProject();
                break;

            case "docmanfile":
                $docman_file = DocmanFilePeer::retrieveByPk($this->getRefId());
                $project = $docman_file->getDocmanFolder()->getProject();
                break;
        }

        return $project;
    }

    public function getForumUrl()
    {
        //https://gforge.com/gf/project/test/forum/?action=ForumBrowse&forum_id=133
        $url = '';
        $project = $this->getProject();
        if ($project)
        {
            $url = GFUrl::getURL()->projectURL($project, 'ForumBrowse', 'forum', '', array('forum_id' => $this->getForumId()));
        }
        return $url;
    }

    public function getForumLink($full = false)
    {
        if ($this->getSection() == 'project')
        {
            $url = $this->getForumUrl();
            if ($full)
            {
                return '<a href="' . $url . '">' . $this->getName() . '</a>';
            } else
            {
                return $url;
            }
        } else
        {
            throw new Exception('ForumLink only supported for project forums');
        }
    }

}
