<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('plugins/forum/lib/om/BaseForumMessage.php');
require_once("plugin/ActionManager.php");
require_once('plugin/ActionManager.php');
require_once('lib/core/Filesystem.php');
require_once('plugins/forum/lib/ForumMessageNotification.php');
require_once('notification/NotificationManager.php');
require_once('StringUtils.php');

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.forum.lib 
 */
class ForumMessage extends BaseForumMessage {
	const SECTION = 'forummessage';
	
	private $invalid_data = array();

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see ForumMessage::setDeleted()
	 * @see ForumMessage::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException();
		}

		// Delete the children messages recursively
		$c = new Criteria();
		$c->add(ForumMessagePeer::PARENT_FORUM_MESSAGE_ID, $this->getForumMessageId());
		$children = ForumMessagePeer::doSelect($c);
		foreach ($children as $child) {
			$child->delete();
		}
		
		if ($this->isMainThreadMessage()) {
			// This means it's the first message in the thread. The whole thread must be deleted.
			$thread = $this->getForumThread();
			ActionManager::notify("FORUM_MESSAGE_DELETE", $this);
			parent::delete($con);
			$thread->delete();
		} else {
			ActionManager::notify("FORUM_MESSAGE_DELETE", $this);
			parent::delete($con);
		}
	}
	
	function userCanDelete() {
		$sess=Session::getSession();
		if ($sess->isLoggedIn() && $this->getCreatedBy()==$sess->getUser()->getUserId()) {
			return true;
		}
		return $this->getForumThread()->getForum()->userIsAdmin();
	}

	function userCanUpdate() {
		$sess=Session::getSession();
		if ($sess->isLoggedIn() && $this->getCreatedBy()==$sess->getUser()->getUserId()) {
			return true;
		}
		return $this->getForumThread()->getForum()->userIsAdmin();
	}
	
	function userCanCreate() {
		$sess=Session::getSession();
		if ($sess->isLoggedIn() && $this->getCreatedBy()==$sess->getUser()->getUserId()) {
			return true;
		}
		return $this->getForumThread()->getForum()->userCanPost($sess->getUser());
	}
	
	function userCanRead() {
		return $this->getForumThread()->getForum()->userCanRead();
	}

	function approvePendingMessage() {
		if (!$this->getForumThread()->getForum()->userIsAdmin()) {
			throw new PermissionDeniedException();
		}
/*
		$this->getForumThread()->setMessageCount($this->getForumThread()->getMessageCount()+1);
		$this->getForumThread()->getForum()->setMessageCount($this->getForumThread()->getForum()->getMessageCount()+1);
		//if this is first message in thread, must also increment thread count in forum
		if ($this->getForumThread()->getMessageCount() == 0) {
			$this->getForumThread()->getForum()->setThreadCount(1);
		}
*/
		$this->setIsApproved(true);
//		$this->getForumThread()->getForum()->save();
//		$this->getForumThread()->save();
		$this->save();
		//send out the notice
		$notification = new ForumMessageNotification($this);
		NotificationManager::getManager()->queue($notification);
	}

	function sendPendingNotice() {
		$emails=array();
		$admins=$this->getForumThread()->getForum()->getForumAdmins();
		foreach ($admins as $user) {
			$emails[]=$user->getEmail();
		}
		if (count($emails)) {
			$proj=ProjectPeer::retrieveByPk($this->getForumThread()->getForum()->getRefId());
			if (is_object($proj)) {
				$subj=Language::getSessText('ForumMessage.PendingSubject').'- '.$proj->getUnixName().'-'.$this->getForumThread()->getForum()->getForumName();
				$url=GFUrl::getUrl()->getFullUrl(GFUrl::getUrl()->projectUrl($proj, 'PendingMessageBrowse', 'forum','admin',array('id'=>$this->getForumThread()->getForum()->getForumId())),true);
				$body=Language::getSessText('ForumMessage.PendingBody')."\n\n";
				$body.="Subject: " . $this->getSubject() . "\n\n";
				$body.="Message: " . StringUtils::getPlainUTF8FromHtml($this->getBody()) ."\n\n";
				$body.="Message URL: \n\n".$url;
			}
			SystemUtils::sendMail($subj,$body,implode(',',$emails));
		}
	}

	function save($con=null, $notification = true) {
		if (count($this->invalid_data) > 0) {
			throw new InvalidDataException($this->invalid_data);
		}
		
		// See if the thread is locked
		if ($this->getForumThread()->getIsLocked() && !$this->getForumThread()->getForum()->userIsAdmin()) {
			throw new PermissionDeniedException();
		}
		
		//moderation code
		$is_moderated=false;
		$modlevel = $this->getForumThread()->getForum()->getModerationLevel();
		if ($this->isNew() && $modlevel > Forum::NO_MODERATION) {
			//moderate only users who are not members of this project
		//	if ($modlevel == MODERATE_NON_PROJECT_MEMBERS) {
				if ($this->getForumThread()->getForum()->userIsAdmin()) {
					$is_moderated=false;
				} else {
					$is_moderated=true;
					//a rule will increment thread message count, so decrement it first
					$this->sendPendingNotice();
					$this->setIsApproved(false);
				}
		//	}
		}

		// only notify about project forums
		$notify = ($notification && $this->isNew()) && ($this->getForumThread()->getForum()->getSection() == 'project');

		$con = Propel::getConnection(ForumMessagePeer::DATABASE_NAME);
		
		$con->begin();
		ActionManager::notify("FORUM_MESSAGE_SAVE_BEFORE", $this);
		parent::save($con);
		ActionManager::notify("FORUM_MESSAGE_SAVE", $this);
		
		if ($notify && !$is_moderated) {
			// At this point checking the date would return an invalid value since
			// $this->post_date doesn't get updated after saving the record. This is a small
			// hack to send the correct time on the notification email
			$this->post_date = time();
			
			// Notify there was a new message in the forum
			$notification = new ForumMessageNotification($this);
			NotificationManager::getManager()->queue($notification);

			// This sends notification to the e-mail addresses defined in
			// "send_all_posts_to" field
			$notification->notifyAllPosts();
		}
		$con->commit();
/*
		if ($is_moderated) {
echo "thread msg: ".$this->getForumThread()->getMessageCount();
echo "forum thread/msg: ".$this->getForumThread()->getForum()->getThreadCount().'/'.$this->getForumThread()->getForum()->getMessageCount();
			$this->getForumThread()->setMessageCount($this->getForumThread()->getMessageCount()-1);
			$this->getForumThread()->getForum()->setMessageCount($this->getForumThread()->getForum()->getMessageCount());
			//if this is first message in thread, must also decrement thread count in forum
			if ($this->getForumThread()->getMessageCount() == 0) {
				$this->getForumThread()->getForum()->setThreadCount($this->getForumThread()->getForum()->getThreadCount());
			}
echo "thread msg: ".$this->getForumThread()->getMessageCount();
echo "forum thread/msg: ".$this->getForumThread()->getForum()->getThreadCount().'/'.$this->getForumThread()->getForum()->getMessageCount();
exit;
			$this->getForumThread()->getForum()->save();
			$this->getForumThread()->save();
		}
*/
	}
	
	public function setSubject($subject) {
		if (strlen(trim($subject)) == 0) {
			$this->invalid_data['subject'] = Language::getSessText('ForumMessage.EmptySubject');
			return;
		}

		parent::setSubject($subject);
		unset($this->invalid_data['subject']);
	}
	
	public function setBody($body) {
		if (strlen(trim($body)) == 0) {
			$this->invalid_data['body'] = Language::getSessText('ForumMessage.EmptyBody');
			return;
		}

		parent::setBody($body);
		unset($this->invalid_data['body']);
	}
	
	/**
	 * Returns true if it's the latest top message (not reply of other message) in the thread 
	 */
	public function isMainThreadMessage() {
		if ($this->getParentForumMessageId() == 0) {
			$c = new Criteria();
			$c->add(ForumMessagePeer::FORUM_MESSAGE_ID, $this->getForumMessageId(), Criteria::NOT_EQUAL);
			$c->add(ForumMessagePeer::FORUM_THREAD_ID, $this->getForumThreadId());
			$c->add(ForumMessagePeer::PARENT_FORUM_MESSAGE_ID, 0);
			$count = ForumMessagePeer::doCount($c);
			if ($count == 0) {
				return true;
			}
		}
		return false;
	}

	public function getForumMessageUrl() {
		//https://gforge.com/gf/project/test/forum/?action=ForumBrowse&forum_id=133&_forum_action=MessageReply&thread_id=3570&message_id=11365
		$project=ProjectPeer::retrieveByPk($this->getForumThread()->getForum()->getRefId());
		$url=GFUrl::getURL()->projectURL($project,'ForumBrowse','forum','',array('forum_id'=>$this->getForumThread()->getForumId(),'_forum_action'=>'MessageReply','thread_id'=>$this->getForumThreadId(),'message_id'=>$this->getForumMessageId()));
		return $url;
	}
	
	public function getForumMessageLink($full=false) {
		if ($this->getForumThread()->getForum()->getSection() == 'project') {
			$url=$this->getForumMessageUrl();
			if ($full) {
				return '<a href="'.$url.'">'.$this->getSubject().'</a>';
			} else {
				return $url;
			}
		} else {
			throw new Exception('ForumMessageLink only supported for project forums');
		}
	}

	public function deleteAttachment($file_id){
		$fs = FileSystemPeer::retrieveByPk($file_id);
		if (!is_object($fs) ||
			$fs->getSection() != self::SECTION ||
			$fs->getRefId() != $this->getForumMessageId()){
			throw new InvalidDataException(array('File not found!'));
		}
		if(!$this->userCanUpdate()) {
			throw new PermissionDeniedException();
		}
		$fs->delete();
	}
	
	/**
	 * Attaches a file to the item.
	 * @param 	mixed $file 	Either a string to the path where the file is in the filesystem or the file as stored in the $_FILES array.
	 * @return 	Filesystem 		The Filesystem object
	 */
	public function attachFile($file, $index=0) {
		$fs = new Filesystem();
		$fs->setSection(self::SECTION);
		$fs->setRefId($this->getPrimaryKey());
		$fs->setPostedBy(Session::getSession()->getUser()->getUserId());

		if (is_string($file)) {
			// $file is specified by a FS path
			if (!is_file($file)) {
				return false;
			}

			$fs->setFileDataFromLocalFile($file);
		} else if (is_array($file)) {
			// $file comes from $_FILES
			$fs->setFileDataFromUploadedFile($file);
		}

		//TODO: handle $_FILES
		return $fs;
	}			
}
?>
