<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
// The parent class
require_once 'plugins/forum/lib/om/BaseForumMessagePeer.php';

// The object class
include_once 'plugins/forum/lib/ForumMessage.php';

/**
 * The skeleton for this class was autogenerated by Propel on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package plugins.forum.lib
 */
class ForumMessagePeer extends BaseForumMessagePeer {

	private static $_pkCache = array();

	/**
	 * Retrieve a single object by pkey or NULL if not found.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key
	 * @param	bool $useCache	Whether to use caching. This will increase performance if the same object
	 * 							is accessed several times
	 * @param	Connection $con	The connection to use
	 * @return	ForumMessage
	 */
	public static function retrieveByPk($pk, $useCache = true, $con = null) {
		if ($useCache && array_key_exists($pk, ForumMessagePeer::$_pkCache)) {
			return ForumMessagePeer::$_pkCache[$pk];
		}

		$object = parent::retrieveByPk($pk, $con);
		if ($useCache) {
			ForumMessagePeer::$_pkCache[$pk] = $object;
		}
		return $object;
	}

	/**
	 * Removes an object from the cache. This is usually done after saving.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key of the object to remove
	 */
	public static function invalidate($pk) {
		if (array_key_exists($pk, ForumMessagePeer::$_pkCache)) {
			unset(ForumMessagePeer::$_pkCache[$pk]);
		}
	}

	/**
	 * Gets all messages on the specified thread with PK greater than the $since parameter, and less than the $before parameter.
	 * A max of MAX_MESSAGES records will be returned, in ascending date/time order.
	 * If both since and before params are zero, then the most recent ($MAX_MESSAGES) messages are retrieved.
	 * @param ForumThread $thread Thread on which to query for messages
	 * @param int $since Only include messages greater than this ID
	 * @param int $before Only include messages less than this ID
	 * @return array A list of zero or more messages that meet the criteria, including joined user data.
	 */
	public static function getMessagesByThreadSince(ForumThread $thread, $since = 0, $before = 0) {

		$MAX_MESSAGES = 25;

		$c = new Criteria();
		$c->add(self::FORUM_THREAD_ID, $thread->getForumThreadId());
		if ($since > 0) {
			$c->add(self::FORUM_MESSAGE_ID, $since, Criteria::GREATER_THAN);
			$c->addAscendingOrderByColumn(self::FORUM_MESSAGE_ID);
		} else if ($before > 0) {
			$c->add(self::FORUM_MESSAGE_ID, $before, Criteria::LESS_THAN);
			$c->addDescendingOrderByColumn(self::FORUM_MESSAGE_ID);
		} else {
			$c->addDescendingOrderByColumn(self::FORUM_MESSAGE_ID);
		}
		$c->setLimit($MAX_MESSAGES);

		$result = self::doSelectJoinUser($c);

		usort($result, array('ForumMessagePeer', 'sortByMessageId'));

		return $result;
	}

	public static function sortByMessageId($a, $b) {
		$ida = $a->getForumMessageId();
		$idb = $b->getForumMessageId();

		if ($ida == $idb) return 0;
		if ($ida < $idb) return -1;
		return 1;
	}

	/**
	 * Given a forum_id, returns the messages posted in the last weeks
	 * @param	int $forum_id	ID of the forum
	 * @return	array			Recursive array of ForumMessage
	 */
	public static function getLatestMessages($forum_id) {
		$today_sql = "'" . date('Y-m-d', time() + (60 * 60 * 24)) . "'";
		$last_weeks_sql = "'" . date('Y-m-d', time() - (60 * 60 * 24 * 7 * 6)) . "'";

		// We need to retrieve the tree and make it efficient... so we skip propel
		// and do plain SQL for now. This will allow us to build the data structures
		// WARNING - PostgreSQL-specific query, we will have to modify the date_parth
		// thing for database compatibility
		$con = Propel::getConnection(self::DATABASE_NAME);
		$sql = '	SELECT fm.*, u.*
					FROM forum_message AS fm, forum_thread AS ft, "user" AS u
					WHERE fm.forum_thread_id=ft.forum_thread_id AND
					ft.forum_id=' . $forum_id . ' AND fm.created_by=u.user_id AND
					AND fm.is_approved=true AND
					ft.most_recent_date BETWEEN ' . $last_weeks_sql . ' AND ' . $today_sql . '
					ORDER BY ft.most_recent_date DESC';
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		// This emulates a doSelectJoinUser (User information will be displayed, so
		// we try to get the info in just one query)
		$messages = array();
		$startcol = (self::$numColumns - self::$numLazyLoadColumns) + 1;
		while ($rs->next()) {
			$message = new ForumMessage();
			$message->hydrate($rs);

			$user = new User();
			$user->hydrate($rs, $startcol);
			$message->setUser($user);

			$messages[] = $message;
		}

		return $messages;
	}

	public static function getThreadMessagesFlat($thread_id, $paginator=null) {
		$c = new Criteria();
		$c->addAscendingOrderByColumn(self::POST_DATE);
		$c->add(self::FORUM_THREAD_ID, $thread_id);
		$c->add(self::IS_APPROVED, true);
		
		if (is_object($paginator)) {
			$paginator->addPageClauses($c);
		}
		
		return self::doSelect($c);
	}


	public static function getThreadMessagesThreaded($thread_id, $paginator=null) {
		$c = new Criteria();
		$c->addAscendingOrderByColumn(self::POST_DATE);
		$c->add(self::FORUM_THREAD_ID, $thread_id);
		$c->add(self::IS_APPROVED, true);
		
		if (is_object($paginator)) {
			$paginator->addPageClauses($c);
		}
		
		$messages_aux = self::doSelect($c);

		return self::buildThread($messages_aux);
	}

	/**
	 * Given a forum_id, returns the messages posted in the last weeks in a threaded fashion
	 * @param	int $forum_id	ID of the forum
	 * @return	array			Recursive array of ForumMessage
	 */
	public static function getLatestMessagesThreaded($forum_id) {
		$messages_aux = self::getLatestMessages($forum_id);


		return self::buildThread($messages_aux);
	}



	public static function getAllMessagesThreaded($forum_id) {
		$c = new Criteria();
		$c->addJoin(self::FORUM_THREAD_ID, ForumThreadPeer::FORUM_THREAD_ID);
		$c->add(ForumThreadPeer::FORUM_ID, $forum_id);
		$c->add(ForumThreadPeer::MESSAGE_COUNT, 0, Criteria::GREATER_THAN);
		$c->addAscendingOrderByColumn(self::POST_DATE);
		$messages_aux = self::doSelect($c);

		return self::buildThread($messages_aux);
	}

	/**
	 * This method gets an array of messages and nests them
	 */
	private function buildThread($messages_aux) {
		// build the structures that will help us with recursion later
		$children = array();
		$messages = array();
		foreach ($messages_aux as $message) {
			$parent_id = $message->getParentForumMessageId();
			$message_id = $message->getForumMessageId();

			if (!isset($children[$parent_id]))
				$children[$parent_id] = array();
			$children[$parent_id][] = $message_id;

			$messages[$message_id] = $message;
		}

		$result = array();
		self::buildThreadedRecursive($result, $messages, $children, 0, 0);

		return $result;
	}

	/**
	 * This functions builds a list with the messages in the order and level they should be
	 * displayed
	 * Note: the three first variables MUST be passed as reference. Ampersands are needed
	 * because they are arrays, not objects
	 */
	private static function buildThreadedRecursive(&$result, &$messages, &$children, $parent_id, $level) {
		if (!array_key_exists($parent_id, $children))
			return;
		$arr = $children[$parent_id];
		$n = count($arr);
		for ($i = 0; $i < $n; $i++) {
			$children_id = $arr[$i];
			$result[] = array("message" => $messages[$children_id], "level" => $level);
			// now append this children's children (if any)
			if (array_key_exists($children_id, $children)) {
				self::buildThreadedRecursive($result, $messages, $children, $children_id, ($level + 1));
			}
		}
	}

	/**
	 * Returns the history of a given forum
	 */
	public static function getForumHistory($forum_id) {
		$con = Propel::getConnection(self::DATABASE_NAME);

		// WARNING - PostgreSQL-specific query, we will have to modify the date_parth
		// thing for database compatibility
		$sql = "SELECT date_part('year', fm.post_date) AS year, 
			date_part('month', fm.post_date) AS month,
			COUNT(fm.forum_message_id) AS total
			FROM forum_message AS fm, forum_thread AS ft
			WHERE fm.forum_thread_id=ft.forum_thread_id AND
			ft.forum_id='$forum_id'
			GROUP BY year,month
			ORDER BY year DESC, month ASC";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		$res = array();
		$curr_year = 0;
		$last_month = 0;

		while ($rs->next()) {
			$year = $rs->getInt(1);
			$month = $rs->getInt(2);
			$count = $rs->getInt(3);

			if ($curr_year != $year) {
				$res[$year] = array();
				$curr_year = $year;
				$last_month = 0;
			}

			// these are months with no posts
			$last_month++;
			for (; $last_month < $month; $last_month++) {
				$res[$year][$last_month] = 0;
			}

			assert('$month == $last_month');

			$res[$year][$month] = $count;
		}

		return $res;
	}

	/**
	 * Returns the history of a given forum in the given month/year
	 */
	public static function getMonthHistory($forum_id, $month, $year) {
		$con = Propel::getConnection(self::DATABASE_NAME);

		// WARNING - PostgreSQL-specific query, we will have to modify the date_parth
		// thing for database compatibility
		$sql = '	SELECT fm.*, u.*
					FROM forum_message AS fm, forum_thread AS ft, "user" as u
					WHERE fm.forum_thread_id=ft.forum_thread_id AND
					fm.created_by=u.user_id AND	ft.forum_id=' . $forum_id . ' AND
					date_part(\'month\', fm.post_date)=' . $month . ' AND
					date_part(\'year\', fm.post_date)=' . $year . '
					ORDER BY post_date DESC
					';
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		$messages = array();
		$startcol = (self::$numColumns - self::$numLazyLoadColumns) + 1;
		while ($rs->next()) {
			$message = new ForumMessage();
			$message->hydrate($rs);

			$user = new User();
			$user->hydrate($rs, $startcol);
			$message->setUser($user);

			$messages[] = $message;
		}

		return $messages;
	}

	static function doSelectProjectsWithPendingForumMessages($project_ids = null) {
		if (is_array($project_ids)) {
			$ids = implode(',', $project_ids);
			$sqlprj = "forum.ref_id IN ($ids) AND";
		} else {
			$sqlprj = '';
		}

		$sql = "SELECT ref_id FROM forum INNER JOIN forum_thread ON 
		forum.forum_id=forum_thread.forum_id 
		INNER JOIN forum_message ON 
		forum_thread.forum_thread_id = forum_message.forum_thread_id 
		WHERE $sqlprj  is_approved=false AND section='project'";

		$con = Propel::getConnection(ForumMessagePeer::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		$ret = array();
		while ($rs->next()) {
			//We didn't use DISTINCT in the query since it is not very efficient,
			//so we filter here instead
			if (!in_array($rs->getInt(1), $ret)) {
				$ret[] = $rs->getInt(1);
			}
		}
		return $ret;
	}

	/**
	 * Given a thread_id, returns the last message posted
	 * @param	int $thread_id	ID of the thread
	 * @return	object	Message object
	 */
	public static function getLatestMessageByThread($thread_id) {
		$con = Propel::getConnection(self::DATABASE_NAME);
		$sql = '	SELECT * FROM forum_message WHERE forum_thread_id = ' . $thread_id . ' ORDER BY forum_message_id DESC LIMIT 1';
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		$message = null;
		while ($rs->next()) {
			$message = new ForumMessage();
			$message->hydrate($rs);
		}

		return $message;
	}

	/**
	 * Creates a message on a thread, and persists it
	 */
	public static function addMessage($user, $thread, $str_subject, $str_message) {
		$message = new ForumMessage();
		$message->setBody($user->getUnixName() . ' ' . $str_message);
		$message->setCreatedBy($user->getUserId());
		$message->setUser($user);
		$message->setForumThread($thread);
		$message->setPostDate(time());
		$message->setSubject($str_subject);
		$message->setPostDate(time());
		$message->save();
	}

	public static function getDeletedMessagesSince($since_timestamp) {

		$ret = array();
		$section = ForumMessage::SECTION;
		$timestamp = date('c', $since_timestamp);
		$sql = "select distinct(ref_id) from audit_trail
			where section = '$section' and field_name = '_deleted_'
			and change_date > '$timestamp'
			order by ref_id ASC";

		$con = Propel::getConnection(self::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		while ($rs->next()) {
			$ret[] = $rs->getInt(1);
		}
		return $ret;
	}

}

?>
