<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('plugins/forum/lib/om/BaseForumThread.php');
require_once("plugin/ActionManager.php");
require_once('plugin/ActionManager.php');
require_once('plugins/forum/lib/ForumMessage.php');

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.forum.lib 
 */
class ForumThread extends BaseForumThread {
	
	const SECTION = 'forumthread';
	
	private $invalid_data = array();
	
	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see ForumThread::setDeleted()
	 * @see ForumThread::isDeleted()
	 */
	function delete($con = null) {
		//Delete associated preferences
		UserPreferencePeer::deleteDiscussionPreferences($this->getForumThreadId());

		ActionManager::notify("FORUM_THREAD_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @param boolean $isView indicates if the save method has been called when viewing a thread (true) or actually updating a thread (false)
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null, $isView = false) {
		if (count($this->invalid_data) > 0) {
			throw new InvalidDataException($this->invalid_data);
		}

		$new = $this->isNew();
		parent::save($con);

		$this->setNew($new); // fake new status for activity event
		if ($isView) {
			ActionManager::notify("FORUM_THREAD_VIEW", $this);
		} else {
			ActionManager::notify("FORUM_THREAD_SAVE", $this);
		}
		$this->setNew(FALSE);
	}
	
	/**
	 * Get a list of the messages in this thread, flat-style, sorted by date
	 * @param	int $limit	How many messages to return (0=all)
	 * @param	int $offset		Offset from where to start retrieving the messages
	 */
	public function getThreadMessagesFlat($paginator) {
		return ForumMessagePeer::getThreadMessagesFlat($this->getForumThreadId(), $paginator);
	}
	
	/**
	 * Get a list of the messages in this thread, threaded-style, sorted by date
	 */
	public function getThreadMessagesThreaded($paginator=null) {
		return ForumMessagePeer::getThreadMessagesThreaded($this->getForumThreadId(), $paginator);
	}
	
	public function setThreadName($name) {
		if (strlen(trim($name)) == 0) {
			$this->invalid_data['thread_name'] = Language::getSessText('ForumThread.EmptyThreadName');
			return;
		}

		parent::setThreadName($name);
		unset($this->invalid_data['thread_name']);
	}
	
	/**
	 * If the creator of the thread is the "undefined" user (id=100), try to get
	 * the name of the creator from the field mail_sender in the FIRST message in the
	 * thread. (This is only used in mailman)
	 */
	public function getUndefinedUserName() {
		$c = new Criteria();
		$c->add(ForumMessagePeer::FORUM_THREAD_ID, $this->getForumThreadId());
		$c->addAscendingOrderByColumn(ForumMessagePeer::FORUM_MESSAGE_ID);
		$message = ForumMessagePeer::doSelectOne($c);
		if (!is_null($message)) {
			return $message->getMailSender();
		} else {
			return null;
		}
	}
	
	public function setIsSticky($v) {
		if (!$this->getForum()->userIsAdmin()) {
			throw new PermissionDeniedException();
		}
		parent::setIsSticky($v);
	}

	public function setIsLocked($v) {
		if (!$this->getForum()->userIsAdmin()) {
			throw new PermissionDeniedException();
		}
		parent::setIsLocked($v);
	}

    public function getForumThreadUrl()
    {
		//http://gforge.com/gf/project/gforge5/forum/?_forum_action=ForumMessageBrowse&thread_id=5367&action=ForumBrowse&forum_id=8595
        $url = '';
        $project = $this->getForum()->getProject();
        if ($project)
        {
            $url = GFUrl::getURL()->projectURL($project, 'ForumBrowse', 'forum', '', array('forum_id' => $this->getForumId(), '_forum_action' => 'ForumMessageBrowse', 'thread_id' => $this->getForumThreadId()));
        }
        return $url;
    }

    public function getForumThreadLink($full = false)
    {
        if ($this->getSection() == 'project')
        {
            $url = $this->getForumUrl();
            if ($full)
            {
                return '<a href="' . $url . '">' . $this->getName() . '</a>';
            } else
            {
                return $url;
            }
        } else
        {
            throw new Exception('ForumThreadLink only supported for project forums');
        }
    }

}
?>
