<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
// The parent class
require_once 'plugins/forum/lib/om/BaseForumThreadPeer.php';

// The object class
include_once 'plugins/forum/lib/ForumThread.php';

/**
 * The skeleton for this class was autogenerated by Propel on:
 *
 * [Sun Nov 13 22:49:46 2005]
 *
 *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package plugins.forum.lib
 */
class ForumThreadPeer extends BaseForumThreadPeer {

	private static $_pkCache = array();

	/**
	 * Retrieve a single object by pkey or NULL if not found.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key
	 * @param	bool $useCache	Whether to use caching. This will increase performance if the same object
	 * 							is accessed several times
	 * @param	Connection $con	The connection to use
	 * @return	ForumThread
	 */
	public static function retrieveByPk($pk, $useCache = true, $con = null) {
		if ($useCache && array_key_exists($pk, ForumThreadPeer::$_pkCache)) {
			return ForumThreadPeer::$_pkCache[$pk];
		}

		$object = parent::retrieveByPk($pk, $con);
		if ($useCache) {
			ForumThreadPeer::$_pkCache[$pk] = $object;
		}
		return $object;
	}

	/**
	 * Removes an object from the cache. This is usually done after saving.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key of the object to remove
	 */
	public static function invalidate($pk) {
		if (array_key_exists($pk, ForumThreadPeer::$_pkCache)) {
			unset(ForumThreadPeer::$_pkCache[$pk]);
		}
	}

	/**
	 * Returns the threads active in the last weeks in the forum
	 * @return	array	Array of ForumThread
	 */
	public static function getLatestThreads(Criteria $c) {
		$today_sql = date('Y-m-d', time() + (60 * 60 * 24));
		$last_weeks_sql = date('Y-m-d', time() - (60 * 60 * 24 * 7 * 6));  // Last six weeks
		// This emulates a BETWEEN using Propel's Criterion objects
		$criterion = $c->getNewCriterion(self::MOST_RECENT_DATE, $today_sql, Criteria::LESS_EQUAL);
		$criterion->addAnd($c->getNewCriterion(self::MOST_RECENT_DATE, $last_weeks_sql, Criteria::GREATER_EQUAL));
		$c->add($criterion);

		$c->addDescendingOrderByColumn(self::MOST_RECENT_DATE);

		return self::doSelectJoinUser($c);
	}

	/**
	 * Returns the current thread for chat.  If the right one does not exist, it is created.
	 * @param	Forum $forum The forum in which the thread should be found or created.
	 * @param string $threadName The short name for the thread to be used/created
	 * @param string $section The section name
	 * @param int $refid The ref_id
	 * @param mixed @user The User instance or the user_id, used to set the topic starter
	 * @return ForumThread The existing or new thread matching the current date as its name.
	 */
	public static function getNamedThread(Forum $forum, $user = 100, $thread_name = 'Discussion') {
		$c = new Criteria();
		$c->add(self::THREAD_NAME, $thread_name, Criteria::EQUAL);
		$c->add(self::FORUM_ID, $forum->getForumId(), Criteria::EQUAL);
		$c->setLimit(1);
		$threads = self::doSelect($c);

		//Check $user param
		if (is_int($user)) {
			$user_id = $user;
		} else if (is_object($user) && $user instanceof User) {
			$user_id = $user->getUserId();
		} else {
			$user_id = 100;
		}

		if (count($threads) == 0) {
			$thread = new ForumThread();
			$thread->setForum($forum);
			$thread->setCreatedBy($user_id);
			$thread->setThreadName($thread_name);

			$params = array(
				'section' => $forum->getSection(),
				'refid' => $forum->getRefId()
			);
			$res = ActionManager::notify('GET_DISCUSSION_MESSAGE', $params);

			$str_body = '';
			foreach ($res as $pluginresult) {
				$str_body = $pluginresult->getResult();
				break;
			}

			if ($str_body != '') {
				$m = new ForumMessage();
				$m->setBody($str_body);
				$m->setCreatedBy($user_id);
				$m->setForumThread($thread);
				$m->setPostDate(time());
				$m->setSubject($forum->getForumName());
				$m->setPostDate(time());
			} else {
				//Adds special message the explains how to use at-mentions
				$unix_name = 'someuser';
				$str_body = Language::getSessText('Forum.ChatUseAtMentions', array($unix_name));
				$m = new ForumMessage();
				$m->setBody($str_body);
				$m->setCreatedBy(100);
				$m->setForumThread($thread);
				$m->setPostDate(time());
				$m->setSubject($forum->getForumName());
				$m->setPostDate(time());
			}

			$thread->addForumMessage($m);
			$thread->save();

			//Persist the event to the DB (create a message)
			ForumPeer::logChatEvent($thread, ForumPeer::CHAT_EVENT_CREATE);
		} else {
			$thread = $threads[0];
		}
		return $thread;
	}

	/**
	 * Returns the latest modified threads
	 * @param	int $numthreads	Number of threads to retrieve
	 */
	public static function getTopThreads(Forum $forum, $numthreads = 5) {
		// We must retrieve several objects, so this query must be optimized
		$sql = '
			SELECT ft.*, fm.*, u.*
			FROM forum_thread AS ft, forum_message AS fm, "user" as u,
				(SELECT ft.forum_thread_id, ft.most_recent_date, MAX(fm.forum_message_id) AS last_message_id
				FROM forum_thread AS ft, forum_message AS fm
				WHERE ft.forum_id=' . $forum->getForumId() . ' AND fm.forum_thread_id=ft.forum_thread_id
				GROUP BY ft.forum_thread_id, ft.most_recent_date
				ORDER BY ft.most_recent_date DESC
				LIMIT ' . $numthreads . ' OFFSET 0
				) AS last_thread_data
			WHERE ft.forum_thread_id=last_thread_data.forum_thread_id AND
			fm.forum_message_id=last_thread_data.last_message_id AND
			u.user_id=fm.created_by
		';

		$con = Propel::getConnection(self::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);

		$res = array();
		while ($rs->next()) {
			$item = array();
			$startcol1 = (self::$numColumns - self::$numLazyLoadColumns) + 1;
			$startcol2 = $startcol1 + (ForumMessagePeer::$numColumns - ForumMessagePeer::$numLazyLoadColumns);

			$thread = new ForumThread();
			$thread->hydrate($rs);

			$last_message = new ForumMessage();
			$last_message->hydrate($rs, $startcol1);

			$user = new User();
			$user->hydrate($rs, $startcol2);

			$item['thread'] = $thread;
			$item['latest_message'] = $last_message;
			$item['user'] = $user;

			$res[] = $item;
		}

		return $res;
	}

	public static function GetMentionedItemDiscussions(ForumThread $thread) {

		$sql = "
		select
			toth.*
		from
			association assoc
		inner join
			forum_message msg
		on
			assoc.section = 'forummessage' and assoc.ref_id = msg.forum_message_id
		inner join
			forum_thread th
		on
			msg.forum_thread_id = th.forum_thread_id
		inner join
			forum tof
		on
			assoc.to_section = tof.section and assoc.to_ref_id = tof.ref_id
		inner join
			forum_thread toth
		on
			toth.forum_id = tof.forum_id
		where
			th.forum_thread_id = {$thread->getPrimaryKey()}

		UNION

		select
			th.*
		from
			association assoc
		inner join
			forum_message msg
		on
			assoc.section = 'forummessage' and assoc.ref_id = msg.forum_message_id
		inner join
			forum_thread th
		on
			msg.forum_thread_id = th.forum_thread_id
		inner join
			forum tof
		on
			assoc.to_section = tof.section and assoc.to_ref_id = tof.ref_id
		inner join
			forum_thread toth
		on
			toth.forum_id = tof.forum_id
		where
			toth.forum_thread_id = {$thread->getPrimaryKey()}";

		$con = Propel::getConnection(self::DATABASE_NAME);
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
		$ret = self::populateObjects($rs);

		return $ret;
	}

}

?>