<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code lists FrsRelease
//
require_once('plugins/frs/lib/FrsReleasePeer.php');
require_once('plugins/frs/lib/FrsRelease.php');
require_once('lib/core/FilesystemPeer.php');
require_once('html/HTMLWidget.php');
require_once('html/Paginator.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

require_once('plugins/frs/lib/html/FrsTemplate.php');
$theme = new FrsTemplate();

$Project = GFUrl::getUrl()->getProject();
$FrsPackage = FrsPackagePeer::retrieveByPk($_REQUEST['frs_package_id']);
if (!is_object($FrsPackage) || $FrsPackage->getProject()->getProjectId() != $Project->getProjectId()) {
	throw new PermissionDeniedException();
}
if (!$FrsPackage->userCanRead()) {
	throw new PermissionDeniedException();
}

$params = array(
		'project' => $Project,
		'action' => isset($action) ? $action : '',
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('FrsRelease.BrowsePageTitle'),
		'frspackage' => $FrsPackage
		);
$theme->header($params);


$columns = array(
		'release_name' => $Lang->getText('FrsRelease.ReleaseNameName'),
		'release_date' => $Lang->getText('FrsRelease.ReleaseDateName'),
		'file_name'=>$Lang->getText('FrsFile.FileNameName'),
		'file_size'=>$Lang->getText('FrsFile.FileSizeName'),
		'download_count'=>$Lang->getText('FrsFile.DownloadCountName')
		);

//make paginator code
$total=0;
$default_sort_by='release_name';
$sqltmp="SELECT frsr.frs_release_id,frsr.release_name,frsr.release_date,fs.filesystem_id,fs.file_name_safe,fs.file_size,fs.download_count 
FROM frs_release frsr, filesystem fs 
WHERE fs.section='".FrsRelease::SECTION."'
AND fs.ref_id=frsr.frs_release_id
AND frsr.is_released=true
AND frsr.frs_package_id='".Validator::sanitizeInt($_GET['frs_package_id'])."'
AND frsr.status_id=".FrsRelease::STATUS_ACTIVE;

//get count of matching items for paginator
$sqlCount = "SELECT count(*) AS count FROM ($sqltmp) foo";

$con=Propel::getConnection(FrsPackagePeer::DATABASE_NAME);
$stmtCount = $con->createStatement();
$rsCount = $stmtCount->executeQuery($sqlCount);
if ($rsCount->next()) {
	$total = $rsCount->getInt('count');
}

$paginator = new Paginator('br_pkgrls', -1, $total, $default_sort_by, $columns);
$sqlAux = $paginator->getSQL();

//get the items to put into paginator
$sql="$sqltmp $sqlAux";

$stmt = $con->createStatement();
$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

$sort = $paginator->getSortingOptions();
$sort_by = $sort['sort_by'];
$sort_order = $sort['order'];

//iterate the files in the release to group them by release
$releaserow=array();
foreach ($rs as $row) {
	$releaserow[$row['frs_release_id']][] = $row;
}

$theader = $paginator->getSortableTableHeaders($theme);
echo $theme->tableTop($theader);

$forum_prefix = $Project->getProjectName().'/'.$FrsPackage->getPackageName();

if ($total < 1) {
	// No releases with files available. List the releases instead.
	$releases = $FrsPackage->getFrsReleases();

	$trows = array();
	$tmods = array();

	foreach ($releases as $release) {
		if (!$release->userCanRead()) {
			// User doesn't have access to the release
		} else {
			$discuss = HTMLWidget::discuss(
					$Project,
					$Lang->getText('Frs.DiscussRelease'),
					FrsRelease::SECTION, $release->getFrsReleaseId(),
					$forum_prefix.'/'.$release->getReleaseName()
			);
			$trow = array();
			$tmod = array();
			$trow[] = $release->getReleaseName().$discuss;
			$tmod[] = '';
			$trow[] = $Lang->getText('FrsRelease.NoFilesForRelease');
			$tmod[] = 'colspan="4"';
			$trows[] = $trow;
			$tmods[] = $tmod;
		}
	}
	
	if (count($trows) == 0) {
		echo $theme->tableFullRow($Lang->getText('FrsRelease.NoneFound'));
	} else {
		for ($i = 0; $i < count($trows); $i++) {
			echo $theme->tableRow($trows[$i], $tmods[$i]);
		}
	}
} else {
	//top
	$last_release='';
	//list releases, grouping by release name
	foreach ($releaserow as $itema) {
	foreach ($itema as $item) {
		$trow = array();
		$tmod = array();
		if ($item['frs_release_id'] != $last_release || $sort_by != $default_sort_by) {
			$discuss = HTMLWidget::discuss(
				$Project,
				$Lang->getText('Frs.DiscussRelease'),
				FrsRelease::SECTION, $item['frs_release_id'],
				$forum_prefix.'/'.$item['release_name']
			);

			$trow[] = '<strong><a href="'.GFUrl::getUrl()->currentBaseUrl('FrsReleaseView').'&amp;release_id='.$item['frs_release_id'].'">'.$item['release_name'].'</a></strong>'.$discuss;
			//if using non-standard sorting, the grouping and rowspaning will not work
			if ($sort_by != $default_sort_by) {
				$tmod[]='rowspan="1"';
			} else {
				$tmod[]='rowspan="'.count($releaserow[$item['frs_release_id']]).'" bgcolor="#FFFFFF"';
			}

			$trow[] = $item['release_date'];
			if ($sort_by != $default_sort_by) {
				$tmod[]='rowspan="1"';
			} else {
				$tmod[]='rowspan="'.count($releaserow[$item['frs_release_id']]).'" bgcolor="#FFFFFF"';
			}
			$last_release=$item['frs_release_id'];
		}
		$url = GFUrl::getURL()->downloadURL(FrsRelease::SECTION,$item['frs_release_id'],$item['filesystem_id'],$item['file_name_safe']);
		if ( (Session::getSession()->isLoggedIn()) || ( (!Session::getSession()->isLoggedIn()) && (!$FrsPackage->getRequireLogin())  ) ) {
			// download link. 
			$trow[] =  '<a href="'.$url.'">'.$item['file_name_safe'].'</a><br />';
			$tmod[] = '';
		} else {
			$trow[] =  '<a href="'.GFUrl::getURL()->baseURL('Login','account','',array('redirect'=>$url)).'">'.$item['file_name_safe'] .' - ('.$Lang->getText('Common.RequireLogin').')</a><br />';
			$tmod[] = '';
		}
		$trow[] = SystemUtils::getReadableSize($item['file_size'], 2);
		$tmod[] = '';
		$trow[] = $item['download_count'];
		$tmod[] = '';
		echo $theme->tableRow($trow,$tmod);
		unset($trow);
		unset($tmod);
	}
	}
}
	//echo $theme->listPaginatorRow($paginator);
	if ($FrsPackage->userIsAdmin()) {
		$button_enabled = true;
	} else {
		$button_enabled = false;
	}
	echo $theme->tableBottom(array(GFUrl::getURL()->projectUrl($Project,'FrsPackageEdit','frs','admin',array('frs_package_id'=>$FrsPackage->getFrsPackageId()))=>$Lang->getText('FrsPackage.ManagePackage')),array($button_enabled));

	echo $theme->tableBottom();
	if(isset($paginator) && is_object($paginator)) {
		echo '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
	}


$theme->footer($params);
?>
