<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-13
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugins/frs/lib/FrsPackagePeer.php');
require_once('lib/html/HTMLWidget.php');
require_once('html/templates/ProjectReportingTemplate.php');

$session = Session::getSession();
$Project = GFUrl::getUrl()->getProject();
$project_id = $Project->getProjectId();
$Lang = $session->getLanguage();

$theme = new ProjectReportingTemplate();

$con = Propel::getConnection(FrsPackagePeer::DATABASE_NAME);

// First, we'll make a map with the subprojects names
$Subprojects = $Project->getAllSubprojects();
$subprj_map = array();
foreach ($Subprojects as $Subroject) {
	$subprj_map[$Subroject->getProjectId()] = $Subroject->getProjectName();
}

$users = array();
$users[0] = 'All';
foreach ($Project->getMembers() as $member) {
	$users[$member->getUserId()] = $member->getFullName();
}

// Get all packages and children's packages
$c = new Criteria();
$c->add(FrsPackagePeer::PROJECT_ID, $project_id);
$c->addOr(FrsPackagePeer::PROJECT_ID, implode(',', $Project->getAllSubprojectIds()), Criteria::IN);
$packagesObj = FrsPackagePeer::doSelect($c);

$packages = array();
$packages[0] = 'All';
foreach ($packagesObj as $package) {
	if ($package->getProjectId() != $project_id) {
		$packages[$package->getFrsPackageId()] = $subprj_map[$package->getProjectId()] . ' &gt; ' . $package->getPackageName();
	} else {
		$packages[$package->getFrsPackageId()] = $package->getPackageName();
	}
}

$package_id = (isset($_REQUEST['frs_package_id']) ? Validator::sanitizeInt($_REQUEST['frs_package_id']) : 0);
$begin=((isset($_REQUEST['start_date']) && strlen($_REQUEST['start_date']) > 5) ? $_REQUEST['start_date'] : 0 );
$end=((isset($_REQUEST['end_date']) && strlen($_REQUEST['end_date']) > 5) ? $_REQUEST['end_date'] : 0 );
$subprojects=((isset($_REQUEST['subprojects'])) ? Validator::sanitizeInt($_REQUEST['subprojects']) : 0 );

if ($begin=='0') {
	$begin = (time()-(2*86400));
} else {
	$begin = strtotime($begin);
}
if ($end=='0') {
	$end = time();
} else {
	$end = strtotime($end)+86399;
}
if ($begin > $end) {
	$endtmp=$end;
	$end=$begin;
	$begin=$endtmp;
}

$params = array(
		'project' => $Project,
		'feedback' => (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('FrsReporting.Title')
	);
$theme->header($params);

?>
<script language="javascript" type="text/javascript" src="/datetimepicker.js">
</script>
<form action="<?php echo $_SERVER['REQUEST_URI']; ?>" method="post">
<table border="0" cellspacing="0" cellpadding="3" bgcolor="#E0E0E0">
<tr>
	<td><strong><?php echo $Lang->getText('FrsReporting.FilterByPackage') ?></strong></td>
	<td><?php echo HTMLWidget::selectBoxFromAssoc($packages,'frs_package_id',$package_id, false, false) ?></td>
	<td><input id="startdate" name="start_date" value="<?php echo date('M d Y',$begin); ?>" size="11" maxlength="11" /><a href="javascript:NewCal('startdate','MMMddyyyy')"><img src="/themes/gforge5/images/cal.gif" width="16" height="16" border="0" alt="Pick a date"/></a></td>
	<td><input id="enddate" name="end_date" value="<?php echo date('M d Y',$end); ?>" size="11" maxlength="11" /><a href="javascript:NewCal('enddate','MMMddyyyy')"><img src="/themes/gforge5/images/cal.gif" width="16" height="16" border="0" alt="Pick a date"/></a></td>
	<td valign="middle" rowspan="2"><?php echo HTMLWidget::button("submit","" .   $Lang->getText('Common.Submit')  . " ","submit","","")?></td>
</tr>
<tr>
	<td colspan="5">
		<strong><?php echo $Lang->getText('ProjectReporting.IncludeSubprojects') ?></strong> 
		<input type="checkbox" name="subprojects" value="1"<?php echo ($subprojects == 1) ? ' checked' : '';?>/>
	</td>
</tr>
</table>
</form>
<?php
$imgquerystring = '&amp;frs_package_id='.$package_id.'&amp;start_date='.date('Y-m-d', $begin);
$imgquerystring .= '&amp;end_date='.date('Y-m-d', $end).'&amp;subprojects='.$subprojects;

echo '<br /><img src="/gf/project/'.$Project->getUnixName().'/frs/graphs/?action=DownloadsByPackage'.$imgquerystring.'" height="250" width="400" alt="" /> &nbsp;';
echo '<img src="/gf/project/'.$Project->getUnixName().'/frs/graphs/?action=DownloadsByRelease'.$imgquerystring.'" height="250" width="400" alt="" />';
echo '<br /> &nbsp; <br /><img src="/gf/project/'.$Project->getUnixName().'/frs/graphs/?action=DownloadsByFile'.$imgquerystring.'" height="250" width="400" alt="" /> &nbsp;';
echo '<img src="/gf/project/'.$Project->getUnixName().'/frs/graphs/?action=DownloadsByMonth'.$imgquerystring.'" height="250" width="400" alt="" />';

echo '<br />';
echo '<br />';

$sqltime = 'AND fsd.download_date BETWEEN \''.date('Y-m-d', $begin).' 00:00:00\' AND \''.date('Y-m-d', $end).' 23:59:59.999\'';

// If there's a valid project_id, we'll filter by project all the queries
if ($package_id > 0) {
	$sqlpkg = "AND frsp.frs_package_id = '$package_id'";
} else {
	$sqlpkg = '';
}


/* *****************************
 * Annonymous Downloads Per Package
 *******************************/

$sql = "SELECT COUNT(*) as downloads, frsp.package_name, to_date(to_char(fsd.download_date, 'YYYY/MM/DD'), 'YYYY/MM/DD') as download_date
FROM filesystem_download fsd, filesystem fs, frs_release frsr, frs_package frsp
WHERE fsd.user_id = 100
AND fsd.filesystem_id = fs.filesystem_id
AND fs.section = 'frsrelease'
AND fs.ref_id = frsr.frs_release_id
AND frsr.frs_package_id = frsp.frs_package_id
$sqltime
$sqlpkg
AND frsp.project_id = $project_id
GROUP BY frsp.frs_package_id, frsp.package_name, to_date(to_char(fsd.download_date, 'YYYY/MM/DD'), 'YYYY/MM/DD')";

echo '<h3>'.$Lang->getText('FrsReporting.AnnonymousDownloadsPerPackage').'</h3>';
$theader = array($Lang->getText('FrsReporting.Package'), $Lang->getText('FrsReporting.DownloadDate'), $Lang->getText('FrsReporting.Downloads'));
echo $theme->tableTop($theader);

$stmt = $con->createStatement();
$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

$itemscount = 0;
while($rs->next()) {
	$trow = array();
	$trow[] = $rs->getString('package_name');
	$trow[] = $rs->getString('download_date');
	$trow[] = $rs->getInt('downloads');

	echo $theme->tableRow($trow);
	unset($trow);
	$itemscount++;

}

if ($itemscount==0) {
	echo $theme->tableFullRow($Lang->getText('FrsReporting.NoneFound'));
}
echo $theme->tableBottom();
echo '<br /><br />';


/* *****************************
 * Annonymous Downloads Per Package
 *******************************/

$sql = "SELECT COUNT(*) as downloads, frsp.frs_package_id, frsp.package_name,
frsr.frs_release_id, frsr.release_name, to_date(to_char(fsd.download_date, 'YYYY/MM/DD'), 'YYYY/MM/DD') as download_date
FROM filesystem_download fsd, filesystem fs, frs_release frsr, frs_package frsp
WHERE fsd.user_id = 100
AND fsd.filesystem_id = fs.filesystem_id
AND fs.section = 'frsrelease'
AND fs.ref_id = frsr.frs_release_id
AND frsr.frs_package_id = frsp.frs_package_id
$sqltime
$sqlpkg
AND frsp.project_id = $project_id
GROUP BY frsp.frs_package_id, frsp.package_name, frsr.frs_release_id, frsr.release_name, to_date(to_char(fsd.download_date, 'YYYY/MM/DD'), 'YYYY/MM/DD')
ORDER BY frsp.package_name";

echo '<h3>'.$Lang->getText('FrsReporting.AnnonymousDownloadsPerRelease').'</h3>';
$theader = array($Lang->getText('FrsReporting.Package'),
				 $Lang->getText('FrsReporting.Release'),
				 $Lang->getText('FrsReporting.DownloadDate'),
				 $Lang->getText('FrsReporting.Downloads'));
echo $theme->tableTop($theader);

$stmt = $con->createStatement();
$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

$lastPackage=0;
$itemscount = 0;

$rows = array();
$pids = array();
while($rs->next()) {
	$pid = $rs->getInt('frs_package_id');

	if (!isset($pids[$pid])) {
		$pids[$pid] = 0;
	}

	$pids[$pid]++;
	$rows[] = array('pid'=>$pid,'package_name'=>$rs->getString('package_name'),
					'release_name'=>$rs->getString('release_name'), 'download_date'=>$rs->getString('download_date'), 'downloads'=>$rs->getString('downloads'));
}

foreach ($rows as $row) {
	$tmod = array();
	$trow = array();
	
	if ($row['pid'] != $lastPackage) {
		$tmod[]='rowspan="'.$pids[$row['pid']].'" bgcolor="#FFFFFF"';
		$lastPackage = $row['pid'];
		$trow[] = $row['package_name'];
	}
	$tmod[] = '';
	$tmod[] = '';
	$trow[] = $row['release_name'];
	$trow[] = $row['download_date'];
	$trow[] = $row['downloads'];
	echo $theme->tableRow($trow,$tmod);
	unset($tmod);
	unset($trow);

	$itemscount++;
}

if ($itemscount==0) {
	echo $theme->tableFullRow($Lang->getText('FrsReporting.NoneFound'));
}
echo $theme->tableBottom();
echo '<br /><br />';


/* *****************************
 * Downloads By User
 *******************************/

$sql = "SELECT u.user_id, frsp.frs_package_id, frsp.package_name, frsr.frs_release_id, frsr.release_name,
fs.file_name, to_date(to_char(fsd.download_date, 'YYYY/MM/DD'), 'YYYY/MM/DD') as download_date, COUNT(*) as downloads
FROM filesystem_download fsd, filesystem fs, frs_release frsr, frs_package frsp, \"user\" u
WHERE fsd.filesystem_id = fs.filesystem_id
AND fs.section = 'frsrelease'
AND fs.ref_id = frsr.frs_release_id
AND frsr.frs_package_id = frsp.frs_package_id
AND fsd.user_id = u.user_id
AND fsd.user_id <> 100
$sqltime
$sqlpkg
AND frsp.project_id = $project_id
GROUP BY u.user_id, frsp.frs_package_id, frsp.package_name, frsr.frs_release_id, frsr.release_name, fs.file_name, to_date(to_char(fsd.download_date, 'YYYY/MM/DD'), 'YYYY/MM/DD')
ORDER BY u.user_id, frsr.frs_release_id, frsp.frs_package_id, fs.file_name, frsp.package_name, frsr.release_name";

echo '<h3>'.$Lang->getText('FrsReporting.DownloadsByUser').'</h3>';
$theader = array($Lang->getText('FrsReporting.User'), $Lang->getText('FrsReporting.Package'),
				 $Lang->getText('FrsReporting.Release'), $Lang->getText('FrsReporting.File'),
				$Lang->getText('FrsReporting.DownloadDate'),
				$Lang->getText('FrsReporting.Downloads'));
echo $theme->tableTop($theader);

$stmt = $con->createStatement();
$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

$lastUser=0;
$lastPackage=0;
$lastRelease=0;
$itemscount = 0;

$rows = array();
$uids = array();
$pids = array();
$rids = array();

while($rs->next()) {
	$uid = $rs->getInt('user_id');
	$pid = $rs->getInt('frs_package_id');
	$rid = $rs->getInt('frs_release_id');

	if (!isset($uids[$uid])) {
		$uids[$uid] = 0;
	}
	if (!isset($users[$uid])) {
		$userObj = UserPeer::retrieveByPK($uid);
		$users[$uid] = $userObj->getFullName();
	}
	
	if (!isset($pids[$pid])) {
		$pids[$pid] = array();
	}
	if (!isset($pids[$pid][$uid])) {
		$pids[$pid][$uid] = 0;
	}

	if (!isset($rids[$rid])) {
		$rids[$rid] = array();
	}
	if (!isset($rids[$rid][$uid])) {
		$rids[$rid][$uid] = 0;
	}

	$uids[$uid]++;
	$pids[$pid][$uid]++;
	$rids[$rid][$uid]++;
	$rows[] = array('uid'=>$uid,'user_name'=>$users[$uid], 
					'pid'=>$pid, 'package_name'=>$rs->getString('package_name'),
					'rid'=>$rid, 'release_name'=>$rs->getString('release_name'),
					'file_name'=>$rs->getString('file_name'),
					'download_date'=>$rs->getString('download_date'),
					'downloads'=>$rs->getString('downloads'));

}

foreach ($rows as $row) {
	$tmod = array();
	$trow = array();
	
	if ($row['uid'] != $lastUser) {
		$tmod[]='rowspan="'.$uids[$row['uid']].'" bgcolor="#FFFFFF"';
		$lastUser = $row['uid'];
		$lastPackage = 0;
		$lastRelease = 0;
		$trow[] = $row['user_name'];
	}
	if ($row['pid'] != $lastPackage) {
		$tmod[]='rowspan="'.$pids[$row['pid']][$row['uid']].'" bgcolor="#FFFFFF"';
		$lastPackage = $row['pid'];
		$lastRelease = 0;
		$trow[] = $row['package_name'];
	}

	if ($row['rid'] != $lastRelease) {
		$tmod[]='rowspan="'.$rids[$row['rid']][$row['uid']].'" bgcolor="#FFFFFF"';
		$lastRelease = $row['rid'];
		$trow[] = $row['release_name'];
	}

	$tmod[] = '';
	$tmod[] = '';
	$trow[] = $row['file_name'];
	$trow[] = $row['download_date'];
	$trow[] = $row['downloads'];
	echo $theme->tableRow($trow,$tmod);
	unset($tmod);
	unset($trow);

	$itemscount++;
}

if ($itemscount==0) {
	echo $theme->tableFullRow($Lang->getText('FrsReporting.NoneFound'));
}
echo $theme->tableBottom();
echo '<br /><br />';

$theme->footer($params);

?>
