#!/usr/bin/php
<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 * @date 2006-10-02
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

if ($argc != 5){
	echo "Usage: ".$argv[0]. " -u <username> -p <password>\n";
	exit(1);
}

$ldap_user = '';
$ldap_password = '';

if ($argv[1] == '-u') {
	$ldap_user = $argv[2];
} else if ($argv[3] == '-u') {
	$ldap_user = $argv[4];
}

if ($argv[1] == '-p') {
	$ldap_password = $argv[2];
} else if ($argv[3] == '-p') {
	$ldap_password = $argv[4];
}

if (strlen($ldap_user) == 0 || strlen($ldap_password) == 0) {
	echo "Usage: ".$argv[0]. " -u <username> -p <password>\n";
	exit(1);
}


/** ****************************************************************************** */
echo "Including /etc files... ";

checkInit();

echo "OK\n";

/** ****************************************************************************** */
echo "Checking configuration... ";
if (! GFConf::get('system.usesLdap', false)) {
	$lib_dir = GFConf::get('system.libDir', '/opt/gforge');
	echo "LDAP plugin is not active. Please enable LDAP: cd $lib_dir/bin && ./gf-config set 'system.usesLdap' true\n";
	exit(1);
}

$conn = Propel::getConnection(SettingPeer::DATABASE_NAME);
if  (!$conn) {
	echo "Could not connect to database!\n";
	exit(1);
}


$stmt = $conn->createStatement();
$sql = "SELECT COUNT(*) FROM plugin WHERE plugin_name='ldap'";
$res = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

list($c) = $res->fetch();
if ($c != 1) {
	echo "LDAP plugin not found in database. Please execute the following query:\n";
	echo "INSERT INTO plugin(plugin_name, plugin_desc, plugin_type, plugin_order) ";
	echo "VALUES('ldap', 'LDAP Authentication', 'site', 2);\n";
	exit(1);
}

echo "OK\n";

/** ****************************************************************************** */

echo "Checking LDAP support for PHP... ";
if (!function_exists('ldap_connect')) {
	echo "LDAP support not found!\n";
	exit(1);
}

echo "OK\n";

/** ****************************************************************************** */

echo "Testing LDAP connection... ";

$server = GFConf::get('ldap.ldapServer');
$port = GFConf::get('ldap.ldapPort');
$connection = ldap_connect($server,$port);
if (!$connection) {
	echo "Couldn't connect to $server:$port!\n";
	exit(1);
}

$ldap_version = GFConf::get('ldap.version', false);
if ($ldap_version) {
	ldap_set_option($connection, LDAP_OPT_PROTOCOL_VERSION, $ldap_version);
}

// Windows 2003 support
ldap_set_option($connection, LDAP_OPT_REFERRALS, 0);

$useTLS = GFConf::get('ldap.useTLS', false);
if ($useTLS) {
	echo "Starting TLS...\n";
	if (!@ldap_start_tls($connection)) {
		echo 'Error starting TLS: '.ldap_error($connection)."\n";
		exit(1);
	}
}

echo "OK\n";

/** ****************************************************************************** */

$authenticatedBind = GFConf::get('ldap.authenticatedBind', false);
if ($authenticatedBind) {
	$ldap_bind_user = GFConf::get('ldap.authenticatedBindUser', false);
	if (!$ldap_bind_user) {
		$ldap_bind_user = $ldap_user;
		$ldap_bind_password = $ldap_password;
	} else {
		$ldap_bind_password = GFConf::get('ldap.authenticatedBindPass', '');
	}

	echo "Binding to LDAP server as user ".$ldap_bind_user."... ";

	$bind = ldap_bind($connection, $ldap_bind_user, $ldap_bind_password);
	if (!$bind) {
		echo 'Invalid password for authenticatedBindUser "'.$ldap_bind_user.'"'."\n";
		exit(1);
	}

	echo "OK\n";
}

$userNameAttribute = GFConf::get('ldap.usernameAttribute', 'uid');  // maybe default to AD=userPrincipalName?
$filter = "$userNameAttribute=$ldap_user";
$base_dn = GFConf::get('ldap.baseDn', '');

echo "Looking for user on LDAP directory with filter \"$filter\" using base DN $base_dn... ";

$result = ldap_search($connection, $base_dn, $filter);
if (!$result) {
	echo 'Couldn\'t search LDAP server using base DN "'.$base_dn.'"'."\n";
	exit(1);
}
echo "OK\n";

echo "Retrieving entries from result... ";
$info = ldap_get_entries($connection, $result);

if ($info['count'] == 0) {
	echo 'User "'.$ldap_user.'" is not in LDAP database'."\n";
	exit(1);
}

$user_dn = $info[0]['dn'];
echo "Binding as specified user (user DN=\"$user_dn\")... ";

$bind = ldap_bind($connection, $user_dn, $ldap_password);

if (!$bind) {
	echo "Couldn't bind using DN $user_dn and the specified password!\n";
	exit(1);
}

echo "OK\n";

/** ****************************************************************************** */

$fields = array(GFConf::get('ldap.usernameAttribute', 'uid'),
				GFConf::get('ldap.emailAttribute', 'email'),
				GFConf::get('ldap.firstnameAttribute', 'givenname'),
				GFConf::get('ldap.lastnameAttribute', 'sn')
				);

echo "Fetching user information... ";
$result = ldap_search($connection, $base_dn, $filter, $fields);
if (!$result) {
	$field_list = implode(',', $fields);
	echo "Couldn't fetch the specified fields ($field_list) for the user!\n";
	exit(1);
}
echo "OK\n";

echo "Retrieving entries from result (2)... ";
$info = ldap_get_entries($connection, $result);
echo "Information fetched from LDAP server:\n";
print_r($info);

/** ****************************************************************************** */

echo "Everything looks fine!\n";


function checkInit() {
	require_once('initialize.php');
	require_once('GFConf.php');
}
