<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Daniel Perez	<daniel@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugin/GFPlugin.php');
require_once('plugin/ActionManager.php');
require_once('plugin/PluginResult.php');
require_once('core/Language.php');
require_once('core/DeletedMailingListPeer.php');
require_once('MailmanPeer.php');
require_once('lib/core/ProjectPluginPeer.php');
require_once('lib/core/PluginPeer.php');
require_once('lib/GFExportImportUtils.php');

class MailmanPlugin implements GFPlugin {

	const SECTION = 'mailman';

	//protected $project;

	/*function __construct($project) {
		$this->project = $project;
	}*/
	function __construct() {
	}
	
	/**
	 *	this is called when the first Action is called. Action manager 
	 *	gets the listeners for each plugin.
	 */
	public function initialize() {				
		$listenTo = array(
			'BUILD_PROJECT_TABS',
//			'BUILD_PROJECT_SUMMARY',
			'GET_PROJECT_ADMIN_NAV_ELEMENTS',
			'GET_MAIL_ALIASES',
			'PROJECT_APPROVE',
			'DAEMON_PROJECT_APPROVE',
			'DAEMON_PROJECT_DELETE',
			'DAEMON_LIST_CREATE',
			'DAEMON_LIST_EDIT',
			'DAEMON_LIST_DELETE',
			'BUILD_MYPAGE_ELEMENTS',
			'BUILD_SEARCH_COMBO',
			'DO_SEARCH',
			'GET_DISK_USAGE',
			'PROJECT_FULLEXPORT',
			'PROJECT_FULLIMPORT',
			'PROJECT_TPLEXPORT',
			'PROJECT_TPLIMPORT',
			//'BUILD_MYPAGE_MONITORS'
			);
	//		'GET_MAILMAN_NAV_ELEMENTS'
	//		);
	
		ActionManager::registerListeners($this, $listenTo);
	}

	/**
	 *	Different types of plugins are possible -
	 *	GFPlugin::SITE_PLUGIN = 'site'
	 *	GFPlugin::PROJECT_PLUGIN = 'project'
	 *	GFPlugin::USER_PLUGIN = 'user'
	 */
	public function getType() {
		return GFPlugin::PROJECT_PLUGIN;
	}	

	/**
	 * returns the part of the html text that goes into the my page area, for monitored items
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 * /
	private function buildMyPageMonitors($params) {	
		$con = Propel::getConnection(ForumPeer::DATABASE_NAME);
		$theme = $params['theme'];

		//get the assigned project forums for the user
		$user = Session::getSession()->getUser();
		$user_id = $user->getPrimaryKey();
		$sql = "select f.forum_id, f.forum_name, p.project_id from forum f, monitor m, mailman ma, project p 
		where m.user_id = '$user_id' and f.section='". $this->getName() . "' and m.section='forum' 
		 and m.ref_id = f.forum_id and ma.mailman_id = f.ref_id and ma.project_id = p.project_id";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		
		$theader = array();
		$theader[] = Language::getSessText('My.ListsForums');
// TODO -> create a page or something to "stop monitoring" and put a link in the table	
		$value = $theme->tableTop($theader);		
		
		if ($rs->getRecordCount()>0) {
			while($rs->next()) {
				$trow = array();
				$forum_name = $rs->getString('forum_name');
				$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'ForumThreadBrowse','forum','',array('forum_id'=>$rs->getInt('forum_id'))) . '" >' .  $forum_name . '</a>';
				$value .= $theme->tableRow($trow);
			}
		} else {
			$trow = array();
			$trow[] = Language::getSessText('My.NoMonitoredListsForums');
			$value .= $theme->tableRow($trow);
		}
		$value .= $theme->tableBottom();
		
		$result = new PluginResult($this);
		$result->setResult($value);
		return $result;		
	}		
	*/
	
	/**
	 *	get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	public function getName() {
		return 'mailman';
	}

	/**
	 *	get the array of action names that this plugin answers to.
	 *	@return	array
	 */
	public function getListeners() {
	}

	/**
	 *	handle action has a case statement to redirect the action to the appropriate function.
	 *	@param string
	 *	@param mixed
	 *	@return PluginResult
	 */
	public function handleAction($listenerName, $params) {			
		switch ($listenerName) {
			case 'BUILD_PROJECT_TABS': {
				return $this->_buildMainTabsListener($params);
				break;
			}
			case 'BUILD_PROJECT_SUMMARY': {
				return $this->_buildProjectSummary($params);
				break;
			}
			case 'GET_PROJECT_ADMIN_NAV_ELEMENTS': {
				return $this->getProjectAdminNavElements($params);
				break;
			}
			case 'GET_MAIL_ALIASES': {
				return $this->getMailAliases($params);
				break;
			}
			case 'PROJECT_APPROVE': {
				return $this->approveActions($params);
				break;
			}
			case 'DAEMON_LIST_CREATE':
			case 'DAEMON_PROJECT_APPROVE': {
				return $this->daemonCreateMailmanLists($params);
				break;
			}
			case 'DAEMON_LIST_EDIT':{
				return $this->privatizeList($params);
				break;
			}
			case 'DAEMON_PROJECT_DELETE': {
				return $this->processDeletedMailingLists();
				break;
			}
			case 'DAEMON_LIST_DELETE':{
				return $this->deleteMailingList($params);
				break;
			}
			case 'BUILD_MYPAGE_ELEMENTS': {
				return $this->buildMyPageElements($params);
				break;
			}
			case 'BUILD_MYPAGE_MONITORS': {
				return $this->buildMyPageMonitors($params);
				break;
			}
			case 'BUILD_SEARCH_COMBO': {
				return $this->buildSearchOptions($params);
				break;
			}
			case 'DO_SEARCH': {
				return $this->getSearchResults($params);
				break;
			}
			case 'GET_DISK_USAGE': {
				return $this->getDiskUsage($params);
				break;
			}
			case 'PROJECT_FULLIMPORT':
			case 'PROJECT_FULLEXPORT': {
				return GFExportImportUtils::getSpecData($this);
				break;
			}
			case 'PROJECT_TPLIMPORT':
			case 'PROJECT_TPLEXPORT': {
				return GFExportImportUtils::getSpecData($this, 'specDataTpl.xml');
				break;
			}
		}
	}

	/**
	 * returns the part of the html text that goes into the my page area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageElements($params) {	
		
	}

	private function _buildMainTabsListener($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$project = $params['project'];
		
		$result = new PluginResult($this);
		$value = array(
				'text'	=> Language::getSessText('Mailman.TabString'),
				'action'	=> GFUrl::getUrl()->projectURL($project,'','mailman'),
				'id'	=> 'mailman',
				'item_count' => MailmanPeer::getMailingListCount($project->getId())
			);
		$result->setResult($value);
		return $result;
	}	

	/**
	 *      getMailmanNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 * /
	private function getMailmanNavElements() {
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'Mailman Links',
			'elements'=> array(
			array('link'=>'/gf/project/',
				'description'=>'Title 1'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 2'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 3')
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 *      getProjectAdminNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	private function getProjectAdminNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'Mailman',
			'elements'=> array(
			array('link'=>'/gf/project/'.$params['project']->getUnixName().'/mailman/admin/',
				'description'=>Language::getSessText('Mailman.Admin'))
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 *	get project uri to be used as root for constructing uris
	 *	@return string
	 */
	function getProjectPluginUri() {
		return GFUrl::getUrl()->currentBaseUrl(); //projectURL($this->project,'','mailman');
	}

	/**
	 * returns the part of the html text that goes into the project public area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 * /
	private function _buildProjectSummary($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);

		$project = $params['project'];
		$theme = $params['theme'];
		
		//$value = $theme->infoTableTop(Language::getSessText('Project.Lists'));
		
		//get the count of lists
		$c = new Criteria();
		$c->add(MailmanPeer::PROJECT_ID,$project->getPrimaryKey());
		$list_count = MailmanPeer::doCount($c);
		
		$value = '<hr size="1" /><a href="'. GFUrl::getUrl()->projectURL($project,'','mailman').'">';
		$value .= $theme->image('ic/mail16b.png','20','20',array('alt'=>Language::getSessText('Project.ShortMailman')));
		$value .= Language::getSessText('Project.Mailman').'</a>&nbsp;&nbsp;( ' . $list_count . ' ' . Language::getSessText('Project.PublicLists') . ' )';
		//$value .= $theme->infoTableBottom();
		
		$result->setResult($value);
		return $result;
	}
	*/
	function enableInProject(Project $project) {
		return;
	}

	function disableFromProject() {
		// unregister the actions
		ActionManager::unregisterListeners($this);
	}

	/**
	 * Retrieves the required mail aliases for mailman lists
	 *
	 * @return array  mail aliases
	 */
	function getMailAliases() {
	
		$aliases = array();
		$result = new PluginResult($this);
						
		$arr_aliases = MailmanPeer::doSelect(new Criteria());
		foreach ($arr_aliases as $alias) {
			$alias_name = $alias->getListName();
			
			$mailmandir = GFConf::get('mailman.mailmanDir');
			$mailmanbin = SystemUtils::buildFilePath($mailmandir, '/mail/mailman');

			if (file_exists($mailmanbin)) {
				// Mailman 2.1						
				$aliases[] = array('name' => $alias_name, 'command' => $mailmanbin.' post '.$alias_name);
				$aliases[] = array('name' => $alias_name.'-admin', 'command' => $mailmanbin.' admin '.$alias_name);
				$aliases[] = array('name' => $alias_name.'-bounces', 'command' => $mailmanbin.' bounces '.$alias_name);
				$aliases[] = array('name' => $alias_name.'-confirm', 'command' => $mailmanbin.' confirm '.$alias_name);
				$aliases[] = array('name' => $alias_name.'-join', 'command' => $mailmanbin.' join '.$alias_name);
				$aliases[] = array('name' => $alias_name.'-leave', 'command' => $mailmanbin.' leave '.$alias_name);
				$aliases[] = array('name' => $alias_name.'-owner', 'command' => $mailmanbin.' owner '.$alias_name);
				$aliases[] = array('name' => $alias_name.'-request', 'command' => $mailmanbin.' request '.$alias_name);
				$aliases[] = array('name' => $alias_name.'-subscribe', 'command' => $mailmanbin.' subscribe '.$alias_name);
				$aliases[] = array('name' => $alias_name.'-unsubscribe', 'command' => $mailmanbin.' unsubscribe '.$alias_name);
			} else {
				// Mailman < 2.1	

				$mailmanbin = $mailmanbin = SystemUtils::buildFilePath($mailmandir, '/mail/wrapper');
				$aliases[] = array('name' => $alias_name, 'command' => $mailmanbin.' post '.$alias_name);
				$aliases[] = array('name' => $alias_name.'-admin', 'command' => $mailmanbin.' mailowner '.$alias_name);
				$aliases[] = array('name' => $alias_name.'-request', 'command' => $mailmanbin.' mailcmd '.$alias_name);
				$aliases[] = array('name' => $alias_name.'-owner', 'alias' => $alias_name.'-admin');
			}
			
		}		
		$result->setResult($aliases);
		return $result;

	}
	
	/**
	 * actions to perform on project approval
	 *
	 * @param $params array
	 * @return PluginResult
	 */
	private function approveActions($params) {
		$template_project = $params['template_project'];
		$project = $params['project'];
		
		$user = Session::getSession()->getUser();
		
		if ( (is_object($project)) && (is_object($template_project))) {
			// get the lists for the template project
			$c = new Criteria();
			$c->add(MailmanPeer::PROJECT_ID,$template_project->getProjectId());
			$template_lists = MailmanPeer::doSelect($c);
			foreach ($template_lists as $template_list) {
				// Create new lists for the new project, copy them from the project template
				// with custom data
				$new_list = new Mailman();
				
				$new_list->setProjectId($project->getProjectId());

				// Extract the project name from the unix name
				$name = $template_list->getListName();
				preg_match('/^'.$template_project->getUnixName().'-(.*)/i', $name, $matches);
				$list_name = $project->getUnixName().'-'.$matches[1];
				$new_list->setListName($list_name);

				// Generate a new password for the list
				$password = substr(md5($list_name.microtime()), 0, 8);
				$new_list->setListPassword($password);

				$new_list->setIsPublic($template_list->getIsPublic());

				//get userid of the user that registered the project
				$crit = new Criteria();
				$crit->add(UserProjectPeer::PROJECT_ID, $project->getProjectId(), Criteria::EQUAL);
				$userproject = UserProjectPeer::doSelectOne($crit);
				
				if (is_object($userproject)) {
					$new_list->setCreatedBy($userproject->getUserId());
					$new_list->setStatus(Mailman::STATUS_REQUESTED);
					$new_list->setListDescription($template_list->getListDescription());
					$new_list->save();
				}
			}
		}		
	}	
	
	private function buildSearchOptions($params) {
		$result = new PluginResult($this);
		$value = array();
		
		if (isset($params['project'])) {
			if (!$params['project']->usesPlugin($this->getName())) {
				return;
			}
			$text = Language::getSessText('Mailman.ThisProjectLists');
		} else {
			$text = Language::getSessText('Mailman.Lists');
		}

		$value[] = array(
			'value'	=> $this->getName(),
			'display'	=> $text
		);
		$result->setResult($value);
		return $result;		
	}	

	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return PluginResult
	 */	
	private function getSearchResults($params) {
		if (array_search($this->getName(), $params['search']) !== false) {
			$result = new PluginResult($this);
			$searcher = new GFSearch($params);
			$result->setResult($this->search($searcher));
			return $result;			
		}
	}
	
	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return PluginResult
	 */
	function search(GFSearch $search) {
		if (!$search->getProjectId()) {
			return $this->globalSearch($search);
		}

		$project = $search->params['project'];

		if (GFConf::get('system.dbtype') == 'postgres') {
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();

			if ($search->getStartDate()) {
				$date_sql = " AND forum_message.post_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}
			
			$sql = "SELECT forum.forum_id, forum.ref_id, forum.forum_name, 
			forum_message.subject, forum_message.forum_message_id, 
			forum_message.created_by, forum_message.post_date, forum_thread.thread_name, forum_thread.forum_thread_id 
			FROM forum, forum_thread, forum_message, forum_message_idx, to_tsquery('".$words."') as q, mailman 
			WHERE 
			vectors @@ q AND forum_message.forum_message_id = forum_message_idx.forum_message_id 
			$date_sql
			AND forum_thread.forum_thread_id = forum_message_idx.forum_thread_id 
			AND forum.forum_id = forum_thread.forum_id 
			AND forum_thread.forum_thread_id = forum_message.forum_thread_id 
			AND forum.section = 'mailman'
			AND forum.ref_id = mailman.mailman_id 
			AND mailman.project_id= '".$search->getProjectId()."'
			ORDER BY rank(vectors, q) DESC";
			
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			$template = $search->getTemplate();

			$count = 0;
			$content = $template->tableTop(array(
						Language::getSessText('Forum.ThreadMessage'),
						Language::getSessText('Forum.Author'),
						Language::getSessText('ForumMessage.PostDateName'))
					);

			while ($rs->next()) {
				$ftid = $rs->getInt('forum_thread_id');
				$trow = array();
				$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project,'ListThreads',$this->getName(),'',array('mailman_id' => $rs->getInt('ref_id'), '_forum_action'=>'MessageReply', 'message_id' => $rs->getInt('forum_message_id'))) . '">' . $rs->getString('subject').'</a>';
				$trow[] = UserPeer::retrieveByPK($rs->getInt('created_by'))->getName();
				$trow[] = $rs->getDate('post_date');
				$content .= $template->tableRow($trow);
				$count++;
			}
			$content .= $template->tableBottom();

			$title = Language::getSessText('Mailman.SearchResultsTabTitle', array($count));

			if ($count == 0) {
				$content = Language::getSessText('Search.NoResults');
			}
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}

		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content
			);
	}
	
	/**
	 *	Searches across all projects
	 *	@param	GFSearch
	 *	@return string
	 */
	private function globalSearch($search) {
		if (GFConf::get('system.dbtype') == 'postgres') {
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();

			if ($search->getStartDate()) {
				$date_sql = " AND forum_message.post_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}

			$project_sql='';
			$project_ids = $search->getProjectIds();
			if (!is_null($project_ids)) {
				$project_sql= ' AND mailman.project_id IN ('.implode(',',$project_ids).')';
			}
			
			$sql = "SELECT project.project_name, 		-- 1
						project.unix_name, 				-- 2
						headline(forum_message.subject, q),		-- 3
						forum_message.forum_message_id, -- 4
						forum_message.created_by, 		-- 5
						forum_message.post_date, 		-- 6
						headline(forum_thread.thread_name, q),	-- 7
						forum_thread.forum_thread_id,	-- 8
						forum.*							-- 9
			FROM project, forum, forum_thread, forum_message, forum_message_idx, to_tsquery('".$words."') as q, mailman 
			WHERE 
			vectors @@ q AND forum_message.forum_message_id = forum_message_idx.forum_message_id
			$date_sql
			$project_sql
			AND forum_thread.forum_thread_id = forum_message_idx.forum_thread_id 
			AND forum.forum_id = forum_thread.forum_id 
			AND forum_thread.forum_thread_id = forum_message.forum_thread_id 
			AND forum.section = 'mailman'
			AND forum.ref_id = mailman.mailman_id 
			AND mailman.project_id=project.project_id
			ORDER BY rank(vectors, q) DESC";
			
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			$template = $search->getTemplate();

			$count = 0;
			$content = $template->tableTop(array(
						Language::getSessText('Project.ListNameTitle'),
						Language::getSessText('Forum.ThreadMessage'),
						Language::getSessText('Forum.Author'),
						Language::getSessText('ForumMessage.PostDateName'))
					);
			//$template->tableFullRow( Language::getSessText('Mailman.SearchResultsFor') );
			while ($rs->next()) {
				$forum = new Forum();
				$forum->hydrate($rs, 9);
				
				if (!$forum->userCanRead()) {
					continue;
				}

				$count++;
				$project_unix_name = $rs->getString(2);
				$ftid = $rs->getInt(8);
				$trow = array();
				$trow[] = '<a href="'.GFUrl::getUrl()->projectUrl($project_unix_name).'">'.$rs->getString(1).'</a>';
				$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project_unix_name,'ListThreads',$this->getName(),'',array('mailman_id' => $forum->getRefId(), '_forum_action'=>'MessageReply', 'message_id' => $rs->getInt(4))) . '">' . $rs->getString(3).'</a>';
				$trow[] = UserPeer::retrieveByPK($rs->getInt(5))->getName();
				$trow[] = $rs->getDate(6);
				$content .= $template->tableRow($trow);
			}
			if ($count == 0) {
				$content .= $template->tableFullRow(Language::getSessText('Search.NoResults'));
			}
			$content .= $template->tableBottom();
			
			$title = Language::getSessText('Mailman.SearchResultsTabTitle', array($count));
			

		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}
		
		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content
			);
	}

	/**
	 *      return an sql statement for calculating disk usage
	 */
	function getDiskUsage($params) {
		$con = Propel::getConnection(MailmanPeer::DATABASE_NAME);
		$IterateSql = "SELECT mailman_id, list_name FROM mailman";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($IterateSql, ResultSet::FETCHMODE_ASSOC);

		if ($rs->getRecordCount() > 0) {
			$archivesdir = GFConf::get('mailman.mailmanListsDir');
			while ($rs->next()) {
				$filesize = 0;
				$listname = $rs->getString('list_name');
				$mailman_id = $rs->getInt('mailman_id');
				$mbox = $archivesdir.'/archives/private/'.$listname.'.mbox/'.$listname.'.mbox';
				$arch = $archivesdir.'/archives/private/'.$listname.'/';
				if (file_exists($mbox)) {
					$filesize = filesize($mbox);
				}
				if (file_exists($arch)) {
					$filesize += SystemUtils::getDirSize($arch);
				}
				$stmt->executeQuery("UPDATE mailman SET bytes_used={$filesize} WHERE mailman_id={$mailman_id}");
			}
		}

		$sql = "SELECT project_id, SUM(bytes_used) AS usage FROM mailman GROUP BY project_id ORDER BY project_id";
		$result = new PluginResult($this);
		$result->setResult($sql);
		return $result;
	}	

	private function daemonCreateMailmanLists($prjUnixName) {
		GFDaemon::logDebugMessage("MailmanPlugin ({$this->getName()}): Processing daemon message");
		$project = ProjectPeer::retrieveByUnixName($prjUnixName, false);
		if (!$project || !$project->usesPlugin($this->getName())) {
			return;
		}
		$c = new Criteria();
		$c->add(MailmanPeer::PROJECT_ID, $project->getProjectId());
		$lists = MailManPeer::doSelectJoinUser($c);
		if (!$lists) {
			return;
		}

		$active_lists = array();
		foreach ($lists as $list) {
			exec(GFConf::get('mailman.mailmanDir').'/bin/list_lists -b ', $active_lists);
			if(!in_array($list->getListName(), $active_lists)){
				//Create new list
				GFDaemon::logDebugMessage("Creating Mailing List: {$list->getListName()}\n");
				$cmd = escapeshellcmd(GFConf::get('mailman.mailmanDir')."/bin/newlist -q ".$list->getListName()." ".
				$list->getUser()->getEmail()." ".$list->getListPassword());
				$output = array();
				$failed = SystemUtils::execute($cmd, $output);
				if($failed) {
					GFDaemon::logDebugMessage('Failed to create '.$list->getListName()." with command '$cmd', skipping. Message was: ". implode($output));
					continue;
				}	

				$list->setStatus(Mailman::STATUS_CREATED);
				$list->save();  

				// send the password to the creator of the list
				SystemUtils::sendMail(GFConf::get('system.name') . ' - Password for mailing list ' . $list->getListName(),
				'You are receiving this email because you created the '.$list->getListName().' mailing list on '.GFConf::get('system.hostName')."\n".
				'The password for the list is : ' . $list->getListPassword() . " \n Please be careful at handling the password and do not give it to anyone.",
				UserPeer::retrieveByPk($list->getCreatedBy())->getEmail());
				//privatize 

				GFDaemon::logDebugMessage("Privatizing {$list->getListName()}");
				$output = array();
				$command = escapeshellcmd(GFConf::get('mailman.mailmanDir').'/bin/config_list -i '.GFConf::get('system.pluginsPath').'/mailman/cronjobs/privatize_list.py '.
				$list->getListName()).' 2>&1';
				$privatizeFailed = SystemUtils::execute($command, $output);
				if($privatizeFailed){
					syslog(LOG_ERR, "Privatizing list {$list->getListName()} has failed " . implode($output). "\n");
				}
			}
		}
	}

			

	private function privatizeList($list_name){
		$c = new Criteria();
		$c->add(MailmanPeer::LIST_NAME, $list_name);
		$list = MailmanPeer::doSelectOne($c);

		if(!$list || $list->getIsPublic()){
			return;
		}

		syslog(LOG_INFO, "Privatizing {$list->getListName()} \n");
		$output = array();
		$command = escapeshellcmd(GFConf::get('mailman.mailmanDir').'/bin/config_list -i '.GFConf::get('system.pluginsPath').'/mailman/cronjobs/privatize_list.py '.
		$list->getListName()).' 2>&1';
		$privatizeFailed = SystemUtils::execute($command, $output);

		if($privatizeFailed){
			syslog(LOG_ERR, "Privatizing list {$list->getListName()} has failed " . implode($output). "\n");
		}

	}

	private function deleteMailingList($list_name){
		$c = new Criteria();
		$c->add(DeletedMailingListPeer::MAILING_LIST_NAME, $list_name);
		$deleted_list = DeletedMailingListPeer::doSelectOne($c);
		if(!is_object($deleted_list)){
			return;
		}

		if ($this->deleteList($deleted_list->getMailingListName()) == 0) {
			$deleted_list->delete();
		}

	}

	/**
	 * When a project is removed, mailman lists records from
	 * mailman_list are moved into deleted_mailing_list table
	 * This method gets called by apache and then proceed to
	 * delete each list using the daemon privileges
	 */
	private function processDeletedMailingLists() {
		$c = new Criteria();
		$c->addSelectColumn(DeletedMailingListPeer::MAILING_LIST_NAME);
		$rs = DeletedMailingListPeer::doSelectRS($c);
		while($rs->next()) {
			GFDaemon::notify('DAEMON_LIST_DELETE',$rs->getString(1));
		}

	}

	private function deleteList($list_name) {
		$output = array();
		$failed = 0;
		$rmlist_cmd = GFConf::get('mailman.mailmanDir')."/bin/rmlist -a ";
		exec($rmlist_cmd . ' ' . escapeshellcmd($list_name), $output, $failed);

		if($failed){
			syslog(LOG_ERR,"Could not remove the list $list_name, message was:" . implode($output) . "\n");
		} else {
			syslog(LOG_INFO,"Deleted mailing list: $list_name\n");
		}

		return $failed;
	}
}
