<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Daniel Perez	<daniel@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugin/GFPlugin.php');
require_once('plugin/ActionManager.php');
require_once('plugin/PluginResult.php');
require_once('plugins/forum/lib/ForumPeer.php');
require_once('core/Language.php');
require_once('NewsPeer.php');
require_once('NewsCountPeer.php');
require_once('lib/GFExportImportUtils.php');

class NewsPlugin implements GFPlugin {
	const NOACCESS = -1;
	const READ = 0;
	const WRITE = 1;
	const ADMIN = 2;

	protected $project;
	
	CONST SECTION = 'news';

	CONST COMMENTS_THREADS_LIST = 'plugins/news/wwwlib/forms/NewsCommentsThreads-tpl.php';
	CONST COMMENTS_THREAD = 'plugins/news/wwwlib/forms/NewsCommentsThread-tpl.php';
	CONST COMMENTS_THREAD_ADD = 'plugins/news/wwwlib/forms/NewsThreadAdd-tpl.php';
	CONST COMMENTS_LIST = 'plugins/news/wwwlib/forms/NewsComment-tpl.php';
	CONST COMMENT_REPLY = 'plugins/news/wwwlib/forms/MessageReplyForm-tpl.php';
	CONST COMMENT_EDIT = 'plugins/news/wwwlib/forms/MessageEditForm-tpl.php';
	

	/*function __construct($project) {
		$this->project = $project;
	}*/
	function __construct() {
	}	
	
	/**
	 *	this is called when the first Action is called. Action manager 
	 *	gets the listeners for each plugin.
	 */
	public function initialize() {
		ActionManager::registerListeners($this, $this->getListeners());
	}

	/**
	 *	Different types of plugins are possible -
	 *	GFPlugin::SITE_PLUGIN = 'site'
	 *	GFPlugin::PROJECT_PLUGIN = 'project'
	 *	GFPlugin::USER_PLUGIN = 'user'
	 */
	public function getType() {
		return GFPlugin::PROJECT_PLUGIN;
	}	
	
	/**
	 *	get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	public function getName() {
		return 'news';
	}

	/**
	 *	get the array of action names that this plugin answers to.
	 *	@return	array
	 */
	public function getListeners() {
		return array(
			'BUILD_PROJECT_TABS',
			'GET_PROJECT_ADMIN_NAV_ELEMENTS',
			'BUILD_PROJECT_NEWS_SUMMARY',
			'BUILD_SEARCH_COMBO',
			'DO_SEARCH',
			'PROJECT_APPROVE',
			'BUILD_MYPAGE_ELEMENTS',
			'MONITOR_GET_TEMPLATE',
			'MONITOR_CHECK_PERMISSIONS',
			'SITE_ADMIN_INDEX',
			'PROJECT_FULLEXPORT',
			'PROJECT_FULLIMPORT',
			'PROJECT_TPLEXPORT',
			'PROJECT_TPLIMPORT',
			'USER_PROJECT_DELETE',
			'USERGROUP_USER_REMOVE',
	//		'BUILD_FRONTPAGE_ELEMENTS'
			'BUILD_MYPAGE_MONITORS',
			'GET_ROLE_CHOICES',
			'UPDATE_ROLE_CHOICES',
			'DO_TAG_SEARCH',
			'RENDER_PLUGIN_LINK'
//			'GET_NEWS_NAV_ELEMENTS'
			);
	}

	/**
	 * returns the part of the html text that goes into the my page area, for monitored items
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageMonitors($params) {	
		$con = Propel::getConnection(NewsPeer::DATABASE_NAME);
		$theme = $params['theme'];

		//get the assigned items for the user
		$user = Session::getSession()->getUser();
		$user_id = $user->getPrimaryKey();
/*
		// news forums
		$sql = "select f.forum_id, f.forum_name, p.project_id from forum f, monitor m, news n, project p 
		where m.user_id = '$user_id' and f.section='". $this->getName() . "' and m.section='forum' 
		 and m.ref_id = f.forum_id and n.news_id = f.ref_id and n.project_id = p.project_id";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		
		$theader = array();
		$theader[] = Language::getSessText('My.NewsForums');
// TODO -> create a page or something to "stop monitoring" and put a link in the table
		$value = $theme->tableTop($theader);		
		
		if ($rs->getRecordCount()>0) {
			while($rs->next()) {
				$trow = array();
				$forum_name = $rs->getString('forum_name');
				$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'ForumThreadBrowse','forum','',array('forum_id'=>$rs->getInt('forum_id'))) . '" >' .  $forum_name . '</a>';
				$value .= $theme->tableRow($trow);
			}
		} else {
			$trow = array();
			$trow[] = Language::getSessText('My.NoMonitoredNewsForums');
			$value .= $theme->tableRow($trow);
		}
		$value .= $theme->tableBottom();
*/
		// news forums
		$sql = "SELECT p.project_name, p.project_id FROM monitor m, project p 
		WHERE m.user_id = '$user_id' AND m.section='news' AND m.ref_id = p.project_id";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

		$value = '<h3>'.Language::getSessText('News.Monitoring').'</h3>';

		$theader = array();
		$theader[] = Language::getSessText('News.Project');
		$theader[] = '';
		$value .= $theme->tableTop($theader);		
	
		if ($rs->getRecordCount()>0) {
			while($rs->next()) {
				$trow = array();
				$tmod = array();
				$project_name = $rs->getString('project_name');
				$project_id = $rs->getString('project_id');
				$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'','news','',array()) . '" >' .  $project_name . '</a>';
				$tmod[] = 'width="30%"';
				$trow[] = '<a href="'.GFUrl::getUrl()->getMonitorUrl(NewsPlugin::SECTION, $project_id , null, false).'" >'.Language::getSessText('WikiPage.StopMonitoring').'</a>';					
				$tmod[] = '';
				$value .= $theme->tableRow($trow, $tmod);
			}
		} else {
			$value .= $theme->tableFullRow(Language::getSessText('My.NoMonitoredNewsForums'));
		}
		$value .= $theme->tableBottom();
		
		$result = new PluginResult($this);
		$result->setResult($value);
		return $result;		
	}		

	/**
	 *	handle action has a case statement to redirect the action to the appropriate function.
	 *	@param string
	 *	@param mixed
	 *	@return PluginResult
	 */
	public function handleAction($listenerName, $params) {
		switch ($listenerName) {
			case 'BUILD_PROJECT_TABS': {
				return $this->_buildMainTabsListener($params);
				break;
			}
			case 'GET_PROJECT_ADMIN_NAV_ELEMENTS': {
				return $this->getProjectAdminNavElements($params);
				break;
			}
			case 'BUILD_PROJECT_NEWS_SUMMARY': {
				return $this->_buildProjectSummary($params);
				break;
			}
			case 'BUILD_SEARCH_COMBO': {
				return $this->buildSearchOptions($params);
				break;
			}
			case 'DO_SEARCH': {
				return $this->getSearchResults($params);
				break;
			}				
			case 'PROJECT_APPROVE': {
				return $this->approveActions($params);
				break;
			}			
			case 'BUILD_MYPAGE_ELEMENTS': {
				return $this->buildMyPageElements($params);
				break;
			}
			case 'BUILD_MYPAGE_MONITORS': {
				return $this->buildMyPageMonitors($params);
				break;
			}
			case 'MONITOR_CHECK_PERMISSIONS': {
				return $this->monitorCheckPermissions($params);
				break;
			}
			case 'MONITOR_GET_TEMPLATE': {
				return $this->monitorGetTemplate($params);
			}
			case 'SITE_ADMIN_INDEX': {
				return $this->getPluginSiteAdminelements($params);
				break;
			}
			case 'PROJECT_FULLIMPORT':
			case 'PROJECT_FULLEXPORT': {
				return GFExportImportUtils::getSpecData($this);
				break;
			}
			case 'PROJECT_TPLIMPORT':
			case 'PROJECT_TPLEXPORT': {
				return GFExportImportUtils::getSpecData($this, 'specDataTpl.xml');
				break;
			}
			case 'USER_PROJECT_DELETE': {
				return $this->deleteUserMonitors($params);
				break;
			}
			case 'USERGROUP_USER_REMOVE': {
				return $this->deleteUsergroupUserMonitors($params);
				break;
			}
			case 'GET_ROLE_CHOICES': {
				return $this->getRoleChoices($params);
				break;
			}
			case 'UPDATE_ROLE_CHOICES': {
				return $this->updateRoleChoices($params);
				break;
			}
			case 'DO_TAG_SEARCH':{
				return $this->getSearchByTagsResult($params);
				break;
			}
			case 'RENDER_PLUGIN_LINK': {
				return $this->renderPluginLink($params);
				break;
			}
			/*case 'BUILD_FRONTPAGE_ELEMENTS': {
				return $this->buildFrontPageElements($params);
				break;
			}*/
		}
	}


	private function deleteUsergroupUserMonitors($usergroup_user) {
		$con = Propel::getConnection();		

		$project_ids = $usergroup_user->getProjectMonitorsToDelete();
		$user_id = $usergroup_user->getMemberUserId();

		if ($project_ids != '') {
			$stmt = $con->createStatement();
			//Delete monitor for the projects
			$sql = "DELETE FROM monitor WHERE section='".NewsPlugin::SECTION."' AND ref_id IN 
							(
								SELECT project_id FROM project WHERE
								project_id IN (".$project_ids.")
								AND
								project.is_public = false
							)
							AND user_id = ".$user_id;
			$stmt->executeQuery($sql);
		}		
	}


	private function deleteUserMonitors($params) {
		$user_id = $params['user']->getUserId();
		$project = $params['project'];

		if (is_object($project)) {
			if (!$project->isPublic()) {
				$con = Propel::getConnection();
				$stmt = $con->createStatement();
		
				$sql = "DELETE FROM monitor WHERE section='".NewsPlugin::SECTION."' AND ref_id = ".$project->getProjectId()."
								AND user_id = ".$user_id;
		
				$stmt->executeQuery($sql);
			}
		}
	}

	/**
	 * returns the part of the html text that goes into the my page area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageElements($params) {	

	}
	
	/**
	 * returns the part of the html text that goes into the front page
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 * /
	private function buildFrontPageElements($params) {	
		$c = new Criteria();
		$c->add(NewsPeer::IS_APPROVED,true);
		$approved = NewsPeer::doSelect($c);
		$result = new PluginResult($this);
		$template = $params['template'];
		$res = $template->infoTableMiddle(Language::getSessText('News.LatestNews'));
		$res .= '<ul>';
		foreach ($approved as $approved) {
			$res .= '<li> (' . $approved->getPostDate() . ') - ' . $approved->getSummary() . '</li>';
		}
		$result->setResult($res);
		return $result;
	}	

	/**
	 * Build Admin Site link
	 *
	 */	
	private function getPluginSiteAdminElements($params) {
		$arr = array(
			'title' => Language::getSessText('News.TabString'),
			'elements' => array('line' => '<a href="'.GFUrl::getUrl()->baseURL('NewsApprove','','admin').'">'.Language::getSessText('News.ApproveRejectFPNews').'</a>')
		);
		$pr = new PluginResult($this);
		$pr->setResult($arr);
		return $pr;
	}
		
	
	/**
	 * returns the part of the html text that goes into the site admin area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildSiteAdminElements($params) {	
		$result = new PluginResult($this);
		$ret = '<p><strong>' . Language::getSessText('News.TabString') .'</strong></p>
				<ul>
					<li><a href="'. GFUrl::getUrl()->projectURL($params['project'],'',$this->getName(),'admin') .'">'. Language::getSessText('News.Approve') . '</a></li>
				</ul>';
		$result->setResult($ret);
		return $result;		
	}	

	/**
	 * returns the part of the html text that goes into the project public area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */	
	private function _buildProjectSummary($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);

		$project = $params['project'];
		$theme = $params['theme'];
		
		$theader=array(Language::getSessText('Project.News'));
		$value=$theme->tableTop($theader);

		$c = new Criteria();
		$c->setLimit(5);
		$c->add(NewsPeer::PROJECT_ID,$project->getPrimaryKey());
		$c->addDescendingOrderByColumn(NewsPeer::POST_DATE);
		$posts = NewsPeer::doSelect($c);
		
		$url = GFUrl::getUrl();

		if (count($posts) < 1) {
			$value .= $theme->tableFullRow(Language::getSessText('News.NoneFound'));
			$value .= '</td></tr></table>';
		} else {
			ob_start();	
			require("plugins/news/wwwlib/forms/NewsProjectSummary-tpl.php");
			$value = ob_get_clean();
		}
/*
		foreach ($news as $new) {
			$trow=array();
			$trow[]='<strong><a href="'.$url->projectUrl($project, '', 'news', '', array('id' => $new->getPrimaryKey())).'">'.$new->getSummary().'</a></strong><br />'.
			$new->getUser()->getUserLink().'<br />'.
			$new->getPostDate('Y-m-d') .'<p>'.
			nl2br(StringUtils::addHtmlLinks($new->getShortDetails())).'</p>';
//$new->getUser()->getUserLink() .' - '. $new->getPostDate();
			$value .= $theme->tableRow($trow,array(),true);
			unset($trow);
		}

		$value .=$theme->tableBottom();
		//$value .= $theme->tableBottom();
*/
		$result->setResult($value);
		return $result;
	}	
	
	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return PluginResult
	 */
	public function search(GFSearch $search) {
		if (!$search->getProjectId()) {
			return $this->globalSearch($search);
		}

		$project = $search->params['project'];

		if (GFConf::get('system.dbtype') == 'postgres') {		
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();

			if ($search->getStartDate()) {
				$date_sql = " AND news.post_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}

			$sql = "SELECT headline(news.summary, q) as summary,
			news.post_date,
			news.submitted_by,
			news.news_id
			FROM news, news_idx, to_tsquery('$words') AS q 
			WHERE (news.project_id='".$search->getProjectId()."'
			$date_sql
			AND news_idx.news_id = news.news_id
			AND (news_idx.vectors @@ q))
			ORDER BY rank(news_idx.vectors, q) DESC";
			
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			$template = $search->getTemplate();

			$content = $template->tableTop(array(
				Language::getSessText('News.SummaryName'),
				Language::getSessText('News.SubmittedByName'),
				Language::getSessText('News.PostDateName'))
				);
			$count = 0;

			while ($rs->next()) {
				$newsid = $rs->getInt('news_id');
				$trow = array();
				$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project,null,$this->getName(),'',array('id'=>$newsid)) . '">' . $rs->getString('summary').'</a>';
				$trow[] = UserPeer::retrieveByPK($rs->getInt('submitted_by'))->getName();
				$trow[] = $rs->getDate('post_date');
				$content .= $template->tableRow($trow);
				$count++;
			}
			$content .= $template->tableBottom();

			$title = Language::getSessText('News.SearchResultsTabTitle', array($count));

			if ($count == 0) {
				$content = Language::getSessText('Search.NoResults');
			}			
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}

		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content
			);	
	}

	/**
	 *	Searches across all projects
	 *	@param	GFSearch search
	 *	@return string
	 */
	private function globalSearch($search) {
		if (GFConf::get('system.dbtype') == 'postgres') {		
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();

			if ($search->getStartDate()) {
				$date_sql = " AND news.post_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}

			$project_sql = '';
			$project_ids = $search->getProjectIds();
			if (!is_null($project_ids)) {
				$project_sql = ' AND project.project_id IN ('. implode(',', $search->params['project_ids']) .')';
			}

			$sql = "SELECT project.project_name, 					-- 1
				project.unix_name, 									-- 2
				headline(news.summary, q) as headline_summary,		-- 3
				news.*												-- 4
			FROM news, news_idx, to_tsquery('$words') AS q, project
			WHERE (news.project_id=project.project_id
			$date_sql $project_sql
			AND news_idx.news_id = news.news_id
			AND (news_idx.vectors @@ q))
			ORDER BY rank(news_idx.vectors, q) DESC";

			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			/** @var SiteTemplate $template */
			$template = $search->getTemplate();
			
			$content = $template->tableTop(array(
							Language::getSessText('Project.ListNameTitle'),
							Language::getSessText('News.SummaryName'),
							Language::getSessText('News.SubmittedByName'),
							Language::getSessText('News.PostDateName'))
						);
			$count = 0;
			while ($rs->next()) {
				$news = new News();
				$news->hydrate($rs, 4);
				
				if (!$news->userCanRead()) {
					continue;
				}
				
				$count++;
				$project_unix_name = $rs->getString(2);
				$newsid = $news->getNewsId();
				$trow = array();
				$trow[] = '<a href="'.GFUrl::getUrl()->projectUrl($project_unix_name).'">'.$rs->getString(1).'</a>';
				$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project_unix_name,null,$this->getName(),'',array('id'=>$newsid)) . '">' . $rs->getString(3).'</a>';
				$trow[] = UserPeer::retrieveByPK($news->getSubmittedBy())->getName();
				$trow[] = $news->getPostDate();
				$content .= $template->tableRow($trow);
			}

			$title = Language::getSessText('News.SearchResultsTabTitle', array($count));
			
			if ($count == 0) {
				$content .= $template->tableFullRow(Language::getSessText('Search.NoResults'));
			}


			$content .= $template->tableBottom();
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}
		
		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content
			);		
	}

	/**
	 *	Searches by tag on a given project or across all projects
	 *	@param	GFSearch
	 *	@return string
	 */
	public function searchByTags(GFSearch $search){

		$words = $search->getWordsForQuery();
		$project_id = $search->getProjectId();
		$project_ids = $search->getProjectIds();
		$template = $search->getTemplate();

		//Clean key words from tsearch operators
		$words = preg_replace('/,+/',',', $words);
		$tsearch_operators = array('(', ')', '!', '|','&');
		$replacements = array('','','','',',');
		$words = str_replace($tsearch_operators, $replacements, $words);
		$words_arr = explode(',', $words);

		$wc = count($words_arr);
		if($wc == 0){
			return '';
		}
		for($i = 0; $i < $wc; $i++){
			$words_arr[$i] = "'" . $words_arr[$i] . "'";
		}
		$words = implode(',', $words_arr);

		//Project
		if($project_id){
			$news_prj = " AND news.project_id=$project_id";
		} else if (!is_null($project_ids)){
			$news_prj = ' AND news.project_id IN ('. implode(',', $project_ids) . ') ';
		} else {
			$news_prj = ' ';
		}

		//Get all the projects ids involved
		$projects_sql = "SELECT DISTINCT project.project_id AS project_id 
		FROM news NATURAL JOIN project
		WHERE news.news_id IN
		(SELECT tag_entry.ref_id FROM tag_entry WHERE section='news'
		AND tag_name IN ($words) 
		GROUP BY tag_entry.ref_id
		HAVING COUNT(ref_id) =  $wc) $news_prj";
		$con = Propel::getConnection();
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($projects_sql, ResultSet::FETCHMODE_NUM);
		$project_aux = '';
		//Get projects and check permissions
		if($rs->getRecordCount() > 0){
			$project_ids = array();
			while($rs->next()){
				$project_ids[] = $rs->getInt(1);
			}
			$prjs = ProjectPeer::retrieveByPks($project_ids);
			
			//If user can read, add the project id to the project_aux sql sentence
			$project_ids = array();
			foreach($prjs as $prj){
				if(is_object($prj) && $prj->usesPlugin('news')){
						if($prj->isPublic() || 
						$prj->getPermission()->check(self::SECTION,0,self::READ)){
							$project_ids[] = $prj->getProjectId();
						}
				}
			}
			if(count($project_ids) == 0){
				return $this->emptyTagSearch();
			} else {
				$project_aux = ' AND news.project_id IN (' . implode(',', $project_ids) .')';
			}
		};

		$news_sql = "SELECT 'blog' AS section,news.news_id AS ref_id,
		news_id AS news_id, news.post_date AS post_date, news.submitted_by AS author,
		news.summary AS description, project.unix_name AS prj_unix_name,
		project.project_name AS prj_name
		FROM news NATURAL JOIN project
		WHERE news_id IN 
		(SELECT tag_entry.ref_id FROM tag_entry WHERE section='news'
		AND tag_name IN ($words) 
		GROUP BY tag_entry.ref_id
		HAVING COUNT(ref_id) =  $wc) $project_aux $news_prj";
		
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($news_sql, ResultSet::FETCHMODE_ASSOC);

		
		$Lang = Session::getSession()->getLanguage();
		if(!$project_id){
			$content = $template->tableTop(array(
				$Lang->getText('Project.ListNameTitle'),
				$Lang->getText('News.SummaryName'),
				$Lang->getText('News.SubmittedByName'),
				$Lang->getText('News.PostDateName'))
			);
		} else {
				$content = $template->tableTop(array(
				$Lang->getText('News.SummaryName'),
				$Lang->getText('News.SubmittedByName'),
				$Lang->getText('News.PostDateName'))
			);
		}

		$count = 0;
		if($rs->getRecordCount() > 0){

			while($rs->next()){
				$trow = array();

				if(!$project_id){
					$trow[] = '<a href="' . GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name')) . '">'. $rs->getString('prj_name') . '</a>';
				}
				$trow[] = '<a href="'. GFUrl::getUrl()->projectUrl($rs->getString('prj_unix_name'), null, 'news','',array('id'=>$rs->getInt('ref_id'))) . '">'.$rs->getString('description')."</>";
				$trow[] = UserPeer::retrieveByPK($rs->getInt('author'))->getName();
				$trow[] = $rs->getDate('post_date');
				$content .= $template->tableRow($trow);
				$count++;
			}

		}


		$title = $Lang->getText('News.SearchResultsTabTitle', array($count));
			
		if ($count == 0) {
			$content .= $template->tableFullRow($Lang->getText('Search.NoResults'));
		}

		$content .= $template->tableBottom();
		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content
		);

	}

	private function emptyTagSearch(){
		$Lang = Session::getSession()->getLanguage();
		$count = 0;
		$title = $Lang->getText('News.SearchResultsTabTitle', array($count));
		$content = $Lang->getText('Search.NoResults');

		return array(
			'title' => $title,
			'count' => $count,
			'content' => $content
		);

	}


	
	/**
	 * returns the options to print in the combobox
	 *
	 * @param unknown_type $params
	 * @return unknown
	 */
	private function buildSearchOptions($params) {
		$result = new PluginResult($this);
		
		if (isset($params['project'])) {
			if (!$params['project']->usesPlugin($this->getName())) {
				return;
			}
			$text = Language::getSessText('News.ThisProjectNews');
		} else {
			$text = Language::getSessText('News.Blogs');
		}
		
		$value = array();
		// only give result if were in a project
		$value[] = array(
			'value'	=> $this->getName(),
			'display'	=> $text
			);
		
		$result->setResult($value);
		return $result;		
	}
	
	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return resultset
	 */
	private function getSearchResults($params) {
		if (array_search($this->getName(), $params['search']) !== false) {
			$result = new PluginResult($this);
			$searcher = new GFSearch($params);
			$result->setResult($this->search($searcher));
			return $result;
		}
	}

	/**
	 * returns the tag search results for the given input
	 *
	 * @param array $params
	 * @return resultset
	 */

	private function getSearchByTagsResult($params){
		if (array_search($this->getName(), $params['search']) !== false) {
			$result = new PluginResult($this);
			$searcher = new GFSearch($params);
			$result->setResult($this->searchByTags($searcher));
			return $result;
		}
	}
	
	
	/**
	 * returns an array of the fields where we want the search to be done upon
	 *
	 */
	private function getSearchFields() {
		return array(NewsPeer::SUMMARY,NewsPeer::DETAILS);
	}	
	
	private function _buildMainTabsListener($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$project = $params['project'];
		
		$result = new PluginResult($this);
		$value = array(
				'text'	=> Language::getSessText('News.TabString'),
				'action'	=> GFUrl::getUrl()->projectURL($project,'','news'),
				'id'	=> 'news',
				'item_count' => NewsPeer::getNewsCount($project->getId())
			);
		$result->setResult($value);
		return $result;
	}	

	/**
	 *      getNewsNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 * / 
	private function getNewsNavElements() {
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'News Title',
			'elements'=> array(
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 1'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 2'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 3')
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 *      getProjectAdminNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	private function getProjectAdminNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'News',
			'elements'=> array(
			array('link'=>'/gf/project/'.$params['project']->getUnixName().'/news/admin/',
				'description'=>Language::getSessText('News.Admin'))
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 *	get project uri to be used as root for constructing uris
	 *	@return string
	 */
	function getProjectPluginUri() {
		return GFUrl::getUrl()->projectURL($this->project,'','news');
	}
	
	function enableInProject(Project $project) {
		return;
	}

	function disableFromProject() {
		// unregister the actions
		ActionManager::unregisterListeners($this);
	}
	
	/**
	 * actions to perform on project approval
	 *
	 * @param $params array
	 * @return PluginResult
	 */
	private function approveActions($params) {
		$template_project = $params['template_project'];
		$project = $params['project'];

		$con = Propel::getConnection(UserPeer::DATABASE_NAME);

		if ( (is_object($project)) && (is_object($template_project))) {
			foreach ($params['role_settings'] as $role_setting) {
				if (strstr($role_setting['section'],$this->getName())) {
					$role_id = $role_setting['role_id'];
					$section = $role_setting['section'];
					$ref_id = $project->getProjectId();
					$value = $role_setting['value'];
					$sql = "insert into role_setting (role_id,section,ref_id,value) values ('$role_id','$section','$ref_id','$value')";
					$stmt = $con->createStatement();
					$rs = $stmt->executeQuery($sql);						
				}
			}		
		}
	}
	
	/**
	 * Check if the user can monitor certain item
	 */	
	private function monitorCheckPermissions($params) {
		$lang = Session::getSession()->getLanguage();

		$section = $params['section'];
		$ref_id = $params['ref_id'];
		
		if ($section != 'news') return null;		// only deal with news monitor
		$pr = new PluginResult($this);
		
		// DO NOT CONFUSE: ref_id points to the project ID, not to the news
		// ID. This is done because we want to monitor *all* the project's news.
		$project = ProjectPeer::retrieveByPk($ref_id);
		if (is_null($project) || !$project->userCanRead()) {
			$pr->setResult(false);
		} else {
			$pr->setResult(true);
		}

		return $pr;
	}

	private function monitorGetTemplate($params){
		$section = $params['section'];
		$ref_id = $params['ref_id'];
		$project = $params['project'];
		$Lang = $params['lang'];

		if($section != self::SECTION){
			return;
		}

		include_once('plugins/news/lib/html/NewsTemplate.php');
		$result['template' ]=new NewsTemplate();
		$result['params'] = array(
			'project' => $project,
			'title'=>$Lang->getText('Monitor.UsersMonitoringPlugin')
		);

		$pr = new PluginResult($this);
		$pr->setResult($result);

		return $pr;
	}
	
	public function getFormPaths() {
		$arr_paths = array();

		$arr_paths['threads_list'] = NewsPlugin::COMMENTS_THREADS_LIST;
		$arr_paths['thread'] = NewsPlugin::COMMENTS_THREAD;
		$arr_paths['thread_add'] = NewsPlugin::COMMENTS_THREAD_ADD;
		$arr_paths['comments_list'] = NewsPlugin::COMMENTS_LIST;
		$arr_paths['comment_reply'] = NewsPlugin::COMMENT_REPLY;
		$arr_paths['comment_edit'] = NewsPlugin::COMMENT_EDIT;

		return $arr_paths;	
	}
	
	
	/**
	 * returns the possible role settings available for this plugin
	 *
	 * @return array	the roles
	 */
	function getAvailableRoleSettings() {
		$return = array();
		$return[0][] = Language::getSessText('News.NoAccess'); //descriptions
		$return[1][] = NewsPlugin::NOACCESS; // values
		$return[0][] = Language::getSessText('News.Read');
		$return[1][] = NewsPlugin::READ;
		$return[0][] = Language::getSessText('News.Write');
		$return[1][] = NewsPlugin::WRITE;
		$return[0][] = Language::getSessText('News.Admin');
		$return[1][] = NewsPlugin::ADMIN;
		
		return $return;
	}

	
	/**
	 * 	The getRoleChoices listener - this hook will fire
	 *	when a new role is added or one is going to be updated.
	 * 	It returns an array of role settings
	 *	@return	array
	 */
	function getRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$project = $params['project'];

		$roles = array();
		$roles[0]['ref_id'] = $project->getProjectId();
		$roles[0]['section'] = Language::getSessText('News.TabString',array($this->getname()));
		$roles[0]['description'] = Language::getSessText('News.Roles.Description');
		$roles[0]['subsection'] = '-';
		$roles[0]['settings'][0] = self::SECTION;
		$roles[0]['settings'][] = $this->getAvailableRoleSettings();
		
		$result = new PluginResult($this);
		$result->setResult($roles);
		return $result;
	}


	
	private function updateRoleChoices($params) {
		if (!is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		
		$section = self::SECTION;

		$project = $params['project'];
		$data = $params['data'];
		$role = $params['role'];
		
		if (!isset($data[$section])) {
			return;
		}
		
		foreach ($data[$section] as $ref_id => $value) {
			$c = new Criteria();
			$c->add(RoleSettingPeer::ROLE_ID, $role->getPrimaryKey());
			$c->add(RoleSettingPeer::SECTION, $section);
			//$c->add(RoleSettingPeer::REF_ID, $project->getProjectId());
			$rolesetting = RoleSettingPeer::doSelectOne($c);
			if (!is_object($rolesetting)) {
				// Enter here if we have a role setting which we didn't have previously (e.g.: activate an inactive plugin)
				$rolesetting = new RoleSetting();
				$rolesetting->setSection($section);
				$rolesetting->setRefId($project->getProjectId());
				$rolesetting->setRoleId($role->getPrimaryKey());
			}
			
			// Being a project admin overrides any other value
			if (isset($data['projectadmin']) && $data['projectadmin'][0] >= Project::PROJECT_ROLE_ADMIN) {
				$value = self::ADMIN;
			}			
			
			$rolesetting->setValue($value);
			$rolesetting->save();						
		}	
		
		return;
	}


	function renderPluginLink($params) {

		if ($params['gf_url']->getPlugin()->getName() != TrackerPlugin::SECTION) {
			return '';
		}

		$url = $params['url'];
		$arr_query = $params['query_string'];
		$res = '';

		switch ($arr_query['action']) {

			default:
				$project = GFUrl::getUrl()->getProject();
				$id = (isset($arr_query['id'])) ? $arr_query['id'] : 0; //Get Blog item
				$blog_post = NewsPeer::retrieveByPk($id);
				$str_blog_post = '';
				if (is_object($blog_post)) {
					$str_blog_post = ' - Summary: ' . $blog_post->getSummary();
				}
				
				
				if (is_object($project)) {
					$message = $project->getProjectName() . ' - ' . ucfirst(NewsPlugin::SECTION) . $str_blog_post;
					$res = LinkRenderer::renderGFUrlHtml($message, $url);
				}
			break;
		}
		
		
		$pr = new PluginResult($this);
		$pr->setResult($res);		// Mark the request as handled
		return $pr;
	}
}
