<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('plugins/news/lib/html/NewsTemplate.php');
require_once('lib/plugin/PluginResult.php');

class NewsAdminTemplate extends NewsTemplate {

	/**
	 * Header - this subclasses header and just passes params and data that is specific
	 * to this template
	 * @param array array of parameters
	 */
	function header($params) {
		$params['selectedbottomtab']='news';
		$Lang = Session::getSession()->getLanguage();		

		if(!isset($params['project']) || !is_object($params['project'])) {
			Throw new exception('Missing project object in news template');
		} 
		if (!Session::getSession()->isLoggedIn()) {
			Throw new PermissionDeniedException();
		}
		if (!$params['project']->getPermission()->check(NewsPlugin::SECTION,$params['project']->getProjectId(),NewsPlugin::WRITE)) {
			Throw new PermissionDeniedException();
		}
		if (!isset($params['title'])) {
			Throw new exception('Missing title in news template');
		}
		$params['title']=$Lang->getText('Tabs.Admin').' > '.$params['title'];
		
		$this->addBreadCrumb('<a href="'.GFUrl::getUrl()->projectUrl($params['project'],'','news','admin').'">'.$Lang->getText('Tabs.Admin').'</a>');		
		return parent::header($params);
	}

	/**
	 *	getPluginNavElements - this overrides parent class and adds its own 
	 *	plugin listener
	 *	@return	PluginResultSet
	 */
	function getPluginNavElements($params) {
		$prs = parent::getPluginNavElements($params);
	       	$tprs = ActionManager::Notify('GET_NEWS_NAV_ELEMENTS',$params);
		foreach ($tprs as $pr) {
			$prs->addResult($pr);
		}
		return $prs;
	}

	/**
	 * getTemplateNavElements - this function can be overridden
	 * by subclasses so you can add navigation entries
	 * @return array - complex array of data.
	 */
	function getTemplateNavElements($params) {
		$url = GFurl::getUrl();
		$lang = Session::getSession()->getLanguage();
		$user = Session::getSession()->getUser();
		$loggedin = Session::getSession()->isLoggedIn();
		$project = $params['project'];

		$pine = $this->getPluginNavElements($params);
		$ret = array();
		foreach ($pine as $result) {
			$ret[]=$result->getResult();
		}
		
		$elements = array();
		
		if (!is_null($user)) {
			if (News::userIsMonitoringProjectNews($user, $project)) {
				$elements[] = array('link' => $url->getMonitorUrl(News::SECTION, $project->getPrimaryKey(), null, false),
					'description' => $lang->getText('News.StopMonitor'));
			} else {
				$elements[] = array('link' => $url->getMonitorUrl(News::SECTION, $project->getPrimaryKey(), null, true),
					'description' => $lang->getText('News.Monitor'));
			}
			
				$elements[] = array('link' => GFUrl::getUrl()->projectURL($project,'UsersMonitoringBrowse','','usersmonitoring',array('section'=>News::SECTION,'ref_id'=>$project->getPrimaryKey(),'description'=>'Blog','type'=>'plugin','return_to'=>$_SERVER['REQUEST_URI'])),
					'description' => $lang->getText('Monitor.UsersMonitoringPlugin'));
		}

		//show admin link for all forums
		if ($loggedin) {
			$news = new News;
			$news->setProjectId($project->getProjectId());
			if ($project->getPermission()->isAdmin() || $news->userCanCreate()) {
				$elements[] = array('link' => $url->projectURL($project,'','news','admin', ''),
					'description' => $lang->getText('Tabs.Admin'));
			}
		}

		if (count($elements) > 0) {
			$ret[] = array(
				'title' => $lang->getText('News.TabString'),
				'elements'=> $elements
			);
		}

		return $ret;
	}

}

?>
