<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('lib/html/ProjectTemplate.php');
require_once('lib/plugin/PluginResult.php');
require_once('plugins/news/lib/News.php');

class NewsTemplate extends ProjectTemplate {

	/**
	 * Header - this subclasses header and just passes params and data that is specific
	 * to this template
	 * @param array array of parameters
	 */
	function header($params) {
		$params['selectedbottomtab']='news';
		$Lang = Session::getSession()->getLanguage();
		if (isset($params['title'])) {
			$params['title']=' '.$Lang->getText('News.TabString').' > '.$params['title'];
		} else {
			Throw new exception('Missing title in news template');
		}
		if (!isset($params['project'])) {
			throw new Exception('Missing project in news template');
		}
		
		//add a breadcrumb for the thread if we're in one
		if (isset($_REQUEST['thread_id'])) {
			$ft=ForumThreadPeer::retrieveByPk($_REQUEST['thread_id']);
			if (is_object($ft) && $ft->getForum()->getRefId()== $_REQUEST['id']) {
				$this->addBreadCrumb('<a href="'.GFUrl::getUrl()->projectUrl($params['project'],'NewsThreadView','news','',array( '_forum_action'=>'ForumMessageBrowse', 'thread_id'=>$_REQUEST['thread_id'], 'id'=>$_REQUEST['id'])).'">'.$ft->getThreadName().'</a>');
			}
		}
		if (isset($params['news'])) {
				$this->addBreadCrumb('<a href="'.GFUrl::getUrl()->projectUrl($params['project'],'NewsThreadView','news','',array( 'id'=>$params['news']->getNewsId())).'">'.$params['news']->getSummary().'</a>');
		}

		$this->addBreadCrumb('<a href="'.GFUrl::getUrl()->projectUrl($params['project'],'','news').'">'.$Lang->getText('News.TabString').'</a>');
		return parent::header($params);
	}

	/**
	 *      getPluginNavElements - this overrides parent class and adds its own
	 *      plugin listener
	 *      @return PluginResultSet
	 */
	function getPluginNavElements($params) {
		$prs = parent::getPluginNavElements($params);
		$tprs = ActionManager::Notify('GET_NEWS_NAV_ELEMENTS',null);
		foreach ($tprs as $pr) {
			$prs->addResult($pr);
		}
		return $prs;
	}
	
	/**
	 * getTemplateNavElements - this function can be overridden
	 * by subclasses so you can add navigation entries
	 * @return array - complex array of data.
	 */
	function getTemplateNavElements($params) {
		$url = GFurl::getUrl();
		$lang = Session::getSession()->getLanguage();
		$user = Session::getSession()->getUser();
		$loggedin = Session::getSession()->isLoggedIn();
		$project = $params['project'];

		$pine = $this->getPluginNavElements($params);
		$ret = array();
		foreach ($pine as $result) {
			$ret[]=$result->getResult();
		}
		
		$elements = array();
		
		if (!is_null($user)) {
			if (News::userIsMonitoringProjectNews($user, $project)) {
				$elements[] = array('link' => $url->getMonitorUrl(News::SECTION, $project->getPrimaryKey(), null, false),
					'description' => $lang->getText('News.StopMonitor'));
			} else {
				$elements[] = array('link' => $url->getMonitorUrl(News::SECTION, $project->getPrimaryKey(), null, true),
					'description' => $lang->getText('News.Monitor'));
			}
			
				$elements[] = array('link' => GFUrl::getUrl()->projectURL($project,'UsersMonitoringBrowse','','usersmonitoring',array('section'=>News::SECTION,'ref_id'=>$project->getPrimaryKey(),'description'=>'Blog','type'=>'plugin','return_to'=>$_SERVER['REQUEST_URI'])),
					'description' => $lang->getText('Monitor.UsersMonitoringPlugin'));
		}

		//show admin link for all forums
		if ($loggedin) {
			$news = new News;
			$news->setProjectId($project->getProjectId());
			if ($project->getPermission()->isAdmin() || $news->userCanCreate()) {
				$elements[] = array('link' => $url->projectURL($project,'','news','admin', ''),
					'description' => $lang->getText('Tabs.Admin'));
			}
		}

		if (count($elements) > 0) {
			$ret[] = array(
				'title' => $lang->getText('News.TabString'),
				'elements'=> $elements
			);
		}

		return $ret;
	}

	function generateRSSLinks() {
		$url = GFUrl::getUrl();
		$project = $url->getProject();
    $project_id = $project->getProjectId();

		$links_arr = parent::generateRSSLinks();

		$links_arr['news']['title'] = GFConf::get('system.name').' - '.$project->getName().' - News';
		$links_arr['news']['url'] = $url->getFullURL($url->baseURL('Rss', '', '', array('section' => 'projectnews', 'project_id' => $project_id)));

		return $links_arr;
	}
}

?>
