<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a News and saves it to the 
// database
//
require_once('plugins/news/lib/News.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$News = NewsPeer::retrieveByPk($_POST['id']);
if (!is_object($News) || !$News->userCanUpdate()) {
	throw new PermissionDeniedException();
}
$errors = array();
$language = Language::getSessLang();
//update
if(isset($_POST['submit'])) {	
	if (strlen($_POST['summary'])==0) {
		$errors['invalid_summary'] = Language::getSessText('News.SummaryInvalid');
	}
	if (strlen($_POST['details'])==0) {
		$errors['invalid_details'] = Language::getSessText('News.DetailsInvalid');
	}
	if (count($errors)==0) {
		if($session->isSiteAdmin()) {
			$News->setIsApproved($_POST['is_approved']);
		} else {
			$News->setIsApproved(0);
		}
		$News->setSummary( Validator::sanitizeString($_POST['summary']) );
		$News->setDetails( Validator::sanitizeHtml($_POST['details']) );
		try {
			$News->save();
		} catch (InvalidDataException $ex) {
			$errors = array_merge($errors, $ex->getErrors());		
		}
		if(count($errors) == 0) {
			GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL(''), array('feedback' => $language->getText('News.EditFeedback')));
		} else {
			$feedback = implode('<br />', $errors);
			include('NewsEdit.php');
		}		
	} else {
		$feedback = implode('<br />', $errors);
		include('NewsEdit.php');		
	}


}
//delete
if(isset($_POST['delete'])) {
	try {	
		$News->delete();
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());		
	}

	if(count($errors) == 0) {
		GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL(''), array('feedback' => $language->getText('News.DeleteFeedback')));
	} else {
		$feedback = implode('<br />', $errors);
		include('NewsEdit.php');
	}
}
?>