<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('plugin/GFPlugin.php');
require_once('plugin/ActionManager.php');
require_once('plugin/PluginResult.php');
require_once('core/Language.php');
require_once('lib/GFUrl.php');
require_once('plugins/projectrating/lib/UserProjectRating.php');
require_once('plugins/projectrating/lib/html/RatingWidget.php');

class ProjectRatingPlugin implements GFPlugin {

	function __construct() {
	}

	public function initialize() {
		ActionManager::registerListeners($this, $this->getListeners());
	}

	/**
	 *	Different types of plugins are possible -
	 *	GFPlugin::SITE_PLUGIN = 'site'
	 *	GFPlugin::PROJECT_PLUGIN = 'project'
	 *	GFPlugin::USER_PLUGIN = 'user'
	 */
	public function getType() {
		return GFPlugin::SITE_PLUGIN;
	}

	/**
	 *	Get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	public function getName() {
		return 'projectrating';
	}

	public function getListeners() {
		return array(
			'PROJECT_INFO_PANELS',
			'PROJECT_LISTING_HEADER_COLS',
			'PROJECT_LISTING_COLS',
		);
	}

	public function handleAction($listenerName, $params) {
		switch ($listenerName) {
		case 'PROJECT_INFO_PANELS':
			return $this->projectInfoPanels($params);
		case 'PROJECT_LISTING_HEADER_COLS':
			return $this->projectListingHeaderCols($params);
		case 'PROJECT_LISTING_COLS':
			return $this->projectListingCols($params);
		}
	}

	private function projectInfoPanels($params) {
		$project = $params['project'];
		$lang = Language::getSessLang();
		
		$panel = array();
		$panel['title'] = Language::getSessText('ProjectRating.ProjectInfoPanelTitle');

		$contents = '<div align="center"><table border="0" cellspacing="0" cellpadding="0">';
		
		$rating = $project->getAverageRating();
		
		if (!$rating) {
			$contents .= '<tr><td colspan="3" align="center">'.$lang->getText('ProjectRating.NotRated').'</td></tr>';
		} else {
			$contents .= '<tr>';
			$contents .= '<td align="right">'.Language::getSessText('ProjectRating.AverageRatingTitle').':'.'</td>';
			$contents .= '<td align="center">'.RatingWidget::getWidget('project_rating', $rating, true).'</td>';
			$url = GFUrl::getUrl()->projectUrl($project, 'ProjectRatingComments', $this->getName());
			$contents .= '<td align="left"><a href="'.$url.'">'.Language::getSessText('ProjectRating.SeeComments').'</a></td>';
			$contents .= '</tr>';
		}

		// If the user is logged in, show his rating or allow him to vote
		if (Session::getSession()->isLoggedIn()) {
			$user = Session::getSession()->getUser();
			$userRating = UserProjectRatingPeer::retrieveByPk($user->getUserId(), $project->getProjectId());
			$url = GFUrl::getUrl()->projectUrl($project, '', $this->getName());

			if (!$userRating) {				// The user hasn't rated this project yet
				$contents .= '<tr><td colspan="3" align="center"><a href="'.$url.'">'.$lang->getText('ProjectRating.RateProject').'</a></td></tr>';
			} else {
				$contents .= '<tr>';
				$contents .= '<td align="right">'.Language::getSessText('ProjectRating.YourRating').':'.'</td>';
				$contents .= '<td align="center">'.RatingWidget::getWidget('user_rating', $userRating->getRating(), true).'</td>';
				$contents .= '<td align="left"><a href="'.$url.'">'.$lang->getText('ProjectRating.ChangeRating').'</a></td>';
				$contents .= '</tr>';
			}
		}
		
		
		$contents .= '</table></div>';

		
		$panel['contents'] = $contents;

		$pr = new PluginResult($this);
		$pr->setResult(array($panel));			// We only show one panel but we could show more
		return $pr;
	}

	private function projectListingHeaderCols($params) {
		$cols = array(
			'average_rating' => Language::getSessText('ProjectRating.RatingTitle'),
		);
		
		$pr = new PluginResult($this);
		$pr->setResult($cols);
		return $pr;
	}
	
	private function projectListingCols($params) {
		$project = $params['project'];
		
		$rating = $project->getAverageRating();
		if (!$rating) {
			$col = Language::getSessText('ProjectRating.NotRated');
		} else {
			$url = GFUrl::getUrl()->projectUrl($project, 'ProjectRatingComments', $this->getName());
			$col = '';
			$col .= RatingWidget::getWidget('project_rating_'.$project->getProjectId(), $rating, true);
			$col .= '&nbsp;&nbsp;<a href="'.$url.'">'.Language::getSessText('ProjectRating.SeeComments').'</a>';
		}
		
		$cols = array(
			$col,
		);

		$pr = new PluginResult($this);
		$pr->setResult($cols);
		return $pr;

	}


	/**
	 *	Take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return PluginResult
	 */
	public function search(GFSearch $search) {
		return null;
	}

}

?>
