<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-01-03
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/scm/lib/om/BaseProjectScm.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Wed Jan  4 05:51:25 2006]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.scm.lib 
 */
class ProjectScm extends BaseProjectScm implements GFPermission {
	
	/** Value we have to add to the project's ID to get the unix group ID (this
	    will be reimplemented with sequences in the future) */
	const UNIX_GROUP_ADD = 50000;
	private $errors = array();

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		if ( $this->getProject()->userIsAdmin()) { // user is admin (site/project admin)
			return true;
		}
		/*
		//requires manual customization
		$perm = $this->getProject()->getPermission();
		if ($perm->check(self::SECTION, SELF::ADMIN)) {
			return true;
		}
		*/
		
	}

	/**
	 * Specifies if the current user can create an object in the database
	 *
	 * @return bool
	 */
	public function userCanCreate() {
		return $this->userIsAdmin();
	}

	/**
	 * Specifies if the current user can read an object in the database
	 *
	 * @return bool
	 */
	public function userCanRead() {
		$project = $this->getProject();
		if (!$project->isPublic() || !$this->getEnableAnonread()) {
			// Private SCM -- Check for permissions
			return $project->getPermission()->check(ScmPlugin::SECTION, 0, ScmPlugin::READ);
		} else {
			return true;
		}
	}	

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 */
	public function userCanUpdate() {
		return $this->userIsAdmin();
	}
	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 * @todo Program the logic
	 */
	public function userCanDelete() {
		return $this->userIsAdmin();
	}


	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see ProjectScm::setDeleted()
	 * @see ProjectScm::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete ProjectScm object: permission denied");
		}
		ActionManager::notify("PROJECT_SCM_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException("Could not create ProjectScm object: permission denied");
		} else if (!$this->userCanUpdate()) {
			throw new PermissionDeniedException("Could not update ProjectScm object: permission denied");
		}
		
		if (!$this->controlled && empty($this->hostname)) {
			$this->errors['hostname'] = Language::getSessText('ProjectScm.EmptyHostname');
		} else {
			unset($this->errors['hostname']);
		}
		
		if (count($this->errors) > 0) {
			throw new InvalidDataException($this->errors);
		}
		
		parent::save($con);
		ActionManager::notify("PROJECT_SCM_SAVE", $this);
	}
	
	function setExternalBrowseUrl($url) {
		if (!empty($url) && !Validator::validateUrl($url)) {
			$this->errors['external_browse_url'] = Language::getSessText('ProjectScm.InvalidBrowseUrl');
			return;
		}
		
		parent::setExternalBrowseUrl($url);
		unset($this->errors['external_browse_url']);
	}
	
	/**
	 * Retrieves the SCM UNIX ID for the project
	 */
	public function getUnixGroupId() {
		return ($this->getProject()->getPrimaryKey() + self::UNIX_GROUP_ADD);
	}

}
?>