<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: SCMRepository.php 488 2006-02-26 15:56:16Z tperdue $
 * @author Marcelo Mottalli	marcelo@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('SystemUtils.php');

/**
 * This class builds a CVSRepository or SVNRepository object, according to the
 * project's SCM plugin
 */
class SCMRepositoryFactory {
	/**
	 * @return SCMRepository
	 */
	public static function getRepository(Project $project) {
		$scmplugin = $project->getExclusiveTypePlugin('scm');
		if (!$scmplugin) return null;
		
		$plugin_name = $scmplugin->getName();
		
		if ($plugin_name == 'scmcvs') {
			include_once('plugins/scmcvs/lib/CVSRepository.php');
			return new CVSRepository($project);
		} elseif ($plugin_name == 'scmsvn') {
			include_once('plugins/scmsvn/lib/SVNRepository.php');
			return new SVNRepository($project);
		} else if ($plugin_name == 'scmperforce') {
			include_once('plugins/scmperforce/lib/PerforceRepository.php');
			return new PerforceRepository($project);
		} elseif ($plugin_name == 'scmclearcase') {
			include_once('plugins/scmclearcase/lib/ccRepository.php');
			return new ClearCaseRepository($project);
		} elseif ($plugin_name == 'scmgit') {
			include_once('plugins/scmgit/lib/GITRepository.php');
			return new GitRepository($project);
		} else {
			// don't know how to handle other types
			return null;
		}
	}
}
?>