<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once 'plugins/scm/lib/om/BaseScmCommit.php';
require_once("exceptions/PermissionDeniedException.php");
require_once("GFPermission.interface.php");
require_once("plugin/ActionManager.php");
require_once('plugins/tracker/lib/TrackerPeer.php');
require_once('plugins/tracker/lib/TrackerItemPeer.php');

/** 
 * The skeleton for this class was autogenerated by Propel  on:
 *
 * [Fri Jan  6 05:19:32 2006]
 *
 * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.scm.lib 
 */
class ScmCommit extends BaseScmCommit implements GFPermission {
	
	const ERROR_INVALID_ID = 'Tracker item ID doesn\'t exist';
	const ERROR_COMMIT_RULE = 'You are violating a commit rule';
	const ERROR_PERMISSION_DENIED = 'You don\'t have permission on this tracker item';
	const MIN_TIME_ENTRY = 0;

	const STATUS_UPDATE_INVALID_ALIAS_MESSAGE = 'The status alias you entered is invalid';
	const STATUS_UPDATE_CANT_TRANSITION_MESSAGE = 'There\'s workflow set on this tracker and the transition you are requesting is invalid';


	const STATUS_UPDATE_INVALID_ALIAS = 1;
	const STATUS_UPDATE_CANT_TRANSITION = 2;
	const STATUS_UPDATE_COMMIT_ACTION = 3;
	const STATUS_UPDATE_NO_NEXT_STATUS = 4;

	/**
	 * Specifies if the current user can create an entry in the database for this object
	 *
	 * @return bool	
	 */
	public function userCanCreate() {
		$prj = $this->getProject();
		if(is_object($prj)) {
			return $prj->getPermission()->check(ScmPlugin::SECTION, 0, ScmPlugin::WRITE);
		}
		return false;
	}

	/**
	 * If this person is an admin for this object
	 * @return boolean
	 */
	private function userIsAdmin() {
		$prj = $this->getProject();
		if(is_object($prj)) {
			return $prj->userIsAdmin();
		}
		return false;	
	}

	/**
	 * Specifies if the current user can update this object's entry in the database
	 *
	 * @return bool
	 */
	public function userCanUpdate() {
		$prj = $this->getProject();
		if(is_object($prj)) {
			return $prj->userIsAdmin();
		}
		return false;
	}

	
	/**
	 * Specifies if the current user can delete this object's entry from the database
	 *
	 * @return bool
	 */
	public function userCanDelete() {
		$prj = $this->getProject();
		if(is_object($prj)) {
			return $prj->userIsAdmin();
		}
		return false;
	}

	/**
	 * Specifies if the current user has permissions to access this object
	 *
	 * @return bool
	 */
	public function userCanRead() {		
		$scm = ProjectScmPeer::retrieveByPk($this->getProjectId());
		if (is_object($scm)) {
			return $scm->userCanRead();
		}
		return false;
	}

	const SECTION = 'scmcommit';

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see ScmCommit::setDeleted()
	 * @see ScmCommit::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException("Could not delete ScmCommit object: permission denied");
		}
		ActionManager::notify("SCM_COMMIT_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null, $checkPermissions=true) {
		//Some plugins check permissions in the hook scripts,
		//so the permission check can be ignored here
		if($checkPermissions){
			if ($this->isNew() && !$this->userCanCreate()) {
				throw new PermissionDeniedException("Could not create ScmCommit object: permission denied");
			} else if (!$this->isNew() && !$this->userCanUpdate()) {
				throw new PermissionDeniedException("Could not update ScmCommit object: permission denied");
			}
		}
		$new = $this->isNew();
		parent::save($con);
		
		$this->setNew($new);  // Fake new flag for activity log purposes
		ActionManager::notify("SCM_COMMIT_SAVE", $this);
		$this->setNew(FALSE);

	}

	/**
	 * @param string $log
	 * @return array
	 */
	private static function getTrackerItemIdCandidatesInLog($log) {
		preg_match_all('/\\[#([0-9]+)[,\\]]/', $log, $matches);
		return (isset($matches[1])) ? $matches[1] : array();
	}


	/**
	 * @param string $log
	 * @return bool
	 */
	public static function messageContainsID($log){
		if (preg_match('/\\[#([0-9]+)[,\\]]/', $log) === 1) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @param string $ref the branch name
	 * @return bool
	 */
	public static function refNameContainsID($ref) {
		if (preg_match('/\#[0-9]+/', $ref) === 1) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @param string $ref the branch name
	 * @return array the ids
	 */
	public static function getTrackerItemIdCandidatesInRefName($ref){
		preg_match_all('/#[0-9]+/', $ref, $matches);
		return (isset($matches[0])) ? str_replace('#','',$matches[0]) : array();
	}

	/**
	 * @param array $ids
	 * @param Project $project
	 * @return TrackerItem[] (array of TrackerITem)
	 */
	public static function getAssociatedTrackerItems($ids, $project) {

		if (!count($ids) || is_null($project))	{
			return array();
		}

		$project_ids = (($project->getParentProjectId()) ? explode(',',$project->getRootPath()) : array($project->getProjectId()));

		$c = new Criteria();
		$c->addJoin(TrackerItemPeer::TRACKER_ID, TrackerPeer::TRACKER_ID);
		$c->add(TrackerPeer::PROJECT_ID, $project_ids, Criteria::IN);
		$c->add(TrackerItemPeer::TRACKER_ITEM_ID, $ids, Criteria::IN);

		return TrackerItemPeer::doSelect($c);


	}

	public static function processBranchNameTIDs($ref, $project) {
		$ret = array();
		$invalidIds = array();
		$validIds = array();

		$ids = self::getTrackerItemIdCandidatesInRefName($ref);
		$tis = self::getAssociatedTrackerItems($ids, $project);

		/** @var TrackerItem[] $tids */
		$tids = array();
		foreach ($tis as $ti) {
			$tids[$ti->getId()] = $ti;
		}

		foreach ($ids as $id) {
			if(!array_key_exists($id, $tids)) {
				$invalidIds[$id] = ScmCommit::ERROR_INVALID_ID;
			} elseif (!$tids[$id]->userCanCommit()) {
				$invalidIds[$id] = ScmCommit::ERROR_COMMIT_RULE;
			} elseif(!$tids[$id]->userCanRead()) {
				$invalidIds[$id] = ScmCommit::ERROR_PERMISSION_DENIED;
			} else {
				$validIds[$id] = intval($id);
			}
		}

		$ret['invalidIds'] = $invalidIds;
		$ret['validIds'] = $validIds;

		return $ret;

	}

	/**
	 * @param string $log (assume format hash SP message)
	 * @param Project $project
	 *
	 * @return array
	 *
	 * Structure = array('hash'=>
	 * array(
	 *    'hash'=>hash
	 *    'log' => string log message
	 *    'invalidIds'=> array('id'=>message)
	 *    'validIds' => array('id'=>tid)
	 *  )
	 * )
	 *
	 */

	public static function processCommitMsgTIDs($log, $project) {

		$ret = array();
		$tmp = explode(" ", $log);

		$ret['hash'] = $tmp[0];
		$ret['log'] = $log;


		$invalidIds = array();
		$validIds = array();

		$ids = self::getTrackerItemIdCandidatesInLog($log);
		$tis = self::getAssociatedTrackerItems($ids, $project);

			/** @var TrackerItem[] $tids */
		$tids = array();
		foreach ($tis as $ti) {
			$tids[$ti->getId()] = $ti;
		}

		foreach ($ids as $id) {
			if(!array_key_exists($id, $tids)) {
				$invalidIds[$id]= ScmCommit::ERROR_INVALID_ID;
			} elseif (!$tids[$id]->userCanCommit()) {
				$invalidIds[$id]= ScmCommit::ERROR_COMMIT_RULE;
			} elseif(!$tids[$id]->userCanUpdate()) {
				$invalidIds[$id]= SCMCOmmit::ERROR_PERMISSION_DENIED;
			} else {
				$validIds[$id] = $id;
			}
		}

		$arr_statuses = self::getStatusFromMessages($log);
        $arr_time_entries = self::getTimeEntriesFromMessages($log);

		$ret['invalidIds'] = $invalidIds;
		$ret['validIds'] = $validIds;
		$ret['statuses'] = $arr_statuses;
        $ret['time_entries'] = $arr_time_entries;

		return $ret;
	}


	public static function checkTrackerItemIds($log, $project, &$invalid_ids=array(), &$messages=array(), $bol_validate_perm=false, $has_valid_tids=false) {
		$invalid_ids = array();
		$messages = array();

		//want to be able to commit to trackers in current project or any parent project
		$project_ids = (($project->getParentProjectId()) ? explode(',',$project->getRootPath()) : array($project->getProjectId()));
		preg_match_all('/\\[#([0-9]+)[,\\]]/', $log, $matches);
		
		$tracker_item_ids = (isset($matches[1]) ? $matches[1] : array());

		if  (count($tracker_item_ids) == 0) {
			$messages[] = "No tracker item ID specified";
			return false;
		}

		if (count($tracker_item_ids) > 0) {
			foreach($tracker_item_ids as $tracker_item_id) {
			$c = new Criteria();
			$c->addJoin(TrackerItemPeer::TRACKER_ID, TrackerPeer::TRACKER_ID);
			$c->add(TrackerPeer::PROJECT_ID, $project_ids, Criteria::IN);
				$c->add(TrackerItemPeer::TRACKER_ITEM_ID, $tracker_item_id);
				$TrackerItem = TrackerItemPeer::doSelectOne($c);

				if (!is_object($TrackerItem)) {
					$invalid_ids[] = $tracker_item_id;
					$messages[$tracker_item_id] = ScmCommit::ERROR_INVALID_ID;
				} else {
					if (!$TrackerItem->userCanCommit()) {
						$invalid_ids[] = $TrackerItem->getTrackerItemId();
						$messages[$TrackerItem->getTrackerItemId()] = ScmCommit::ERROR_COMMIT_RULE;
					} else if ($bol_validate_perm && !$TrackerItem->userCanUpdate()) {
							$invalid_ids[] = $TrackerItem->getTrackerItemId();
							$messages[$TrackerItem->getTrackerItemId()] = ScmCommit::ERROR_PERMISSION_DENIED;

					}
						
				}
			}
		}

		if(count($invalid_ids) == 0) {
			return true;
		} else {
			return false;
		}
		
	}
	
	public function getScmCommitUrl($full=false) {
		$scmplugin = $this->getProject()->getExclusiveTypePlugin('scm');
		if (is_null($scmplugin)) return '';		// shouldn't happen... means SCM was disabled in the project
		
		$url = GFUrl::getUrl();
		$address = GFUrl::getURL()->projectURL($this->getProject(), 'ScmCommitDetail', $scmplugin->getName(), '', array('scm_commit_id'=>$this->getScmCommitId()));
		if ($full) {
			$address = $url->getFullUrl($address);
		}
		
		return $address;

	}
	
	public function getScmCommitLink($full=false) {
		return '<a href="'.$this->getScmCommitUrl($full).'">#'.$this->getScmCommitId().'</a>';
	}

	public function getTrackerItems(){
		$ti_scm_commits = $this->getTrackerItemScmCommits();
		$tis = array();
		foreach($ti_scm_commits as $ti_scm_commit){
			$tis[] = $ti_scm_commit->getTrackerItem();
		}
		return $tis;
	}
	

	public static function getStatusFromMessages($commit_message) {
		$matches = array();
		preg_match_all("/\\[#([0-9]+),(.*,|)[ ]*(\d*[a-zA-Z]{1,}[ a-zA-Z0-9]*)[ ]*[,]*.*\\]/", $commit_message, $matches);
		
		$arr_tid_status = array();
		
		for($i=0; $i < sizeof($matches[1]); $i++) {
			$tid = $matches[1][$i];
			$status = '';
			
			if ($matches[3][$i] != '') {
				$status = $matches[3][$i];
			} elseif ($matches[4][$i] != '') {
				$status = $matches[4][$i];
			}
			
			$arr_tid_status[$tid] = trim($status);
		}
			
		$arr_tid_status_valid = array();
		$arr_tid_status_invalid = array();
			
		//Validate statuses
		foreach($arr_tid_status as $tid => $status) {
            $tracker_item = TrackerItemPeer::retrieveByPk($tid);

            if (!is_object($tracker_item)) {
                continue;
            }

            //Get current status name
            $curr_status_name = $tracker_item->getStatusAlias();

            //Get current status object
            $curr_status = $tracker_item->getTracker()->getStatusByAlias($curr_status_name);

			$tracker_item = TrackerItemPeer::retrieveByPk($tid);
			if (is_object($tracker_item)) {

				//Check if there are any commit actions and abort status setting
				$c = new Criteria();
				$c->add(TrackerCommitTriggerPeer::TRACKER_ID, $tracker_item->getTracker()->getTrackerId());
				$trigger = TrackerCommitTriggerPeer::doSelectOne($c);

				if (is_object($trigger)) {
					$arr_tid_status_invalid[$tid]['status'] = $status;
					$arr_tid_status_invalid[$tid]['errcode'] = ScmCommit::STATUS_UPDATE_COMMIT_ACTION;
				} else {
					$arr_statuses = $tracker_item->getTracker()->getStatusAliases(true);

					if ($status != "next") {
	                    //Get new status object
	                    $new_status = $tracker_item->getTracker()->getStatusByAlias($status);

						if (in_array(strtolower($status), $arr_statuses) && is_object($new_status)) {
	                        //Verify if status can he updated
	                        if ($tracker_item->canTransition($curr_status, $new_status)) {
	                            $arr_tid_status_valid[$tid] = $status;
	                        } else {
	                            $arr_tid_status_invalid[$tid]['status'] = $status;
		                        $arr_tid_status_invalid[$tid]['errcode'] = ScmCommit::STATUS_UPDATE_CANT_TRANSITION;
	                        }
						} else {
							$arr_tid_status_invalid[$tid]['status'] = $status;
							$arr_tid_status_invalid[$tid]['errcode'] = ScmCommit::STATUS_UPDATE_INVALID_ALIAS;
						}
					} else {
						//Get next status from the sequence
						$is_valid = false;
						$pos = 0;
						for($i=0; $i < count($arr_statuses); $i++) {
							if ($arr_statuses[$i] == strtolower($curr_status_name) && $i < count($arr_statuses)) {
								$pos = $i+1;
								$is_valid = true;
								break;
							}
						}

						if ($is_valid) {
							$arr_statuses = $tracker_item->getTracker()->getStatusAliases();

	                        if ($pos < count($arr_statuses)) {
	                            $status = $arr_statuses[$pos];
	                            //Get new status object
	                            $new_status = $tracker_item->getTracker()->getStatusByAlias($status);

	                            //Verify if status can he updated
	                            if ($tracker_item->canTransition($curr_status, $new_status)) {
	                                $arr_tid_status_valid[$tid] = $status;
	                            } else {
		                            $arr_tid_status_invalid[$tid]['status'] = $status;
		                            $arr_tid_status_invalid[$tid]['errcode'] = ScmCommit::STATUS_UPDATE_CANT_TRANSITION;
	                            }
	                        } else {
		                        $arr_tid_status_invalid[$tid]['status'] = $curr_status_name;
		                        $arr_tid_status_invalid[$tid]['errcode'] = ScmCommit::STATUS_UPDATE_NO_NEXT_STATUS;
	                        }
						} else {
							$arr_tid_status_invalid[$tid]['status'] = $curr_status_name;
							$arr_tid_status_invalid[$tid]['errcode'] = ScmCommit::STATUS_UPDATE_NO_NEXT_STATUS;
						}
					}
				}
			} 
		}
		
		$ret['invalid'] = $arr_tid_status_invalid;
		$ret['valid'] = $arr_tid_status_valid;

		return $ret;
	}


	/**
	 * Returns an array with the time entry settings retrieved from a commit message. Classifies the time entry as valid/invalid
	 *
	 * @param string $commit_message
	 * @return array
	 */
	public static function getTimeEntriesFromMessages($commit_message) {
		$matches = array();
		preg_match_all("/\\[#([0-9]+),([ ]*\d*[a-zA-Z][ a-zA-Z0-9]*[ ]*,|)[ ]*((?:[1-9]\d*|0)?(?:\.\d+)?)[ ]*/", $commit_message, $matches);

		$arr_tid_time_entries_valid = array();
        $arr_tid_time_entries_invalid = array();
		
		for($i=0; $i < sizeof($matches[1]); $i++) {
			$tid = $matches[1][$i];

			if ($matches[3][$i] != '') {
				$time_entry = $matches[3][$i];

	            if (floatval($time_entry) > ScmCommit::MIN_TIME_ENTRY) {
				    $arr_tid_time_entries_valid[$tid] = trim($time_entry);
                } else {
	                $arr_tid_time_entries_invalid[$tid] = trim($time_entry);
                }
			}
		}


        $arr_tid_time_entries = array();
        $arr_tid_time_entries['valid'] = $arr_tid_time_entries_valid;
        $arr_tid_time_entries['invalid'] = $arr_tid_time_entries_invalid;

		return $arr_tid_time_entries;
	}


    /**
     * Returns a string containing the available status names for a tracker item
     *
     * @param int $tid
     * @return string
     */
    public static function getStatusesByTid($tid) {
        $tracker_item = TrackerItemPeer::retrieveByPk($tid);
		$arr_aliases = array();
        if (is_object($tracker_item)) {
            $arr_statuses = $tracker_item->getTracker()->getStatusAliases();

	        foreach ($arr_statuses as $status_alias) {
		        if ($status_alias != '') {
			        $arr_aliases[] = $status_alias;
		        }
	        }

	        if (count($arr_aliases) > 0) {
                return implode(",", $arr_aliases);
	        } else {
		        return Language::getSessText('ScmCommit.NoStatusAliases');
	        }
        } else {
            return '';
        }
    }

}
?>