<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

//
// This code shows a form for editing ScmModuleUser
//
require_once('plugins/scm/lib/ScmModulePeer.php');
require_once('plugins/scm/lib/ScmModule.php');
require_once('exceptions/PermissionDeniedException.php');

$lang = Session::getSession()->getLanguage();
$url = GFUrl::getUrl();
$project = $url->getProject();

if (isset($_REQUEST['id'])) {
	$ScmModule = ScmModulePeer::retrieveByPk($_REQUEST['id']);
	if (!is_object($ScmModule) || !$ScmModule->userCanUpdate()) {
		
	}
} elseif (isset($_REQUEST['path'])) {
	// Try to create the module on the fly if it doesn't exist
	$c = new Criteria();
	$c->add(ScmModulePeer::PROJECT_ID, $project->getProjectId());
	$c->add(ScmModulePeer::PATH, $_REQUEST['path']);
	$ScmModule = ScmModulePeer::doSelectOne($c);
	if (is_null($ScmModule)) {
		$ScmModule = new ScmModule();
		$ScmModule->setProject($project);
		$ScmModule->setPath($_REQUEST['path']);
		$ScmModule->save();
	}
} else {
	throw new PermissionDeniedException();
}
$obj = $ScmModule;

$params['feedback'] = isset($feedback) ? $feedback : '';

$theme->header($params);

echo '<p><h3>'.$lang->getText('ScmModuleUser.Edit', array($ScmModule->getPath())).'</h3></p>';
echo $lang->getText('ScmModuleUser.EditDescription');

if (!GFConf::get('system.useDaemon')) {
	echo '<br />';
	echo '<i>'.$lang->getText('ScmModuleUser.DaemonDisabledNotice').'</i>';
}

echo '<br /><br />';
if (($repos->getType() == 'git' || $repos->getType() == 'svn') && 
	$repos->getAccessType() == ScmRepository::ACCESS_TYPE_SSH) 
{ 
	echo '<p>'.$lang->getText('ScmModuleUser.NoAccessDisabledSSH') .'<p>';
}

if ($repos->hasAnonymousAccess()) { 
	echo '<p>'.$lang->getText('ScmModuleUser.NoAccessDisabledAnonymous') .'<p>';
}

$perms = $repos->getModulePermissions($ScmModule->getScmModuleId());

$theader = array();
$theader[] = $lang->getText('ScmModuleUser.User');
$theader[] = $lang->getText('ScmModuleUser.ScmAccess');
$theader[] = $lang->getText('ScmModuleUser.DirAccess');
echo $theme->tableTop($theader);

echo '<form method="post" action="'.$url->currentBaseURL('ScmModuleUserEditAction').'">';
echo '<input type="hidden" name="id" value="'.$ScmModule->getScmModuleId().'" />';
if (isset($_REQUEST['redirect_to'])) {
	echo '<input type="hidden" name="redirect_to" value="'.Validator::sanitizeString($_REQUEST['redirect_to']).'" />';
}

foreach ($perms as $perm) {
	$trow = array();
	
	$user = $perm['user'];
	$scm_access = $perm['scm_access'];
	$module_access = $perm['module_access'];
	
	$trow[] = $user->getFullName();

	$str_scm_access = '';
	if ($scm_access == ScmPlugin::NOACCESS) {
		$str_scm_access = $lang->getText('Scm.NoAccess');
	} else if ($scm_access == ScmPlugin::READ) {
		$str_scm_access = $lang->getText('Scm.Read');
	} else if ($scm_access == ScmPlugin::WRITE) {
		$str_scm_access = $lang->getText('Scm.Write');
	}
	$trow[] = $str_scm_access;
	
	// Inherit access from the repository permissions
	if (is_null($module_access)) {
		// Acess level is not specified
		$module_access = $scm_access;
	}
	
	$select = '<select name="access['.$user->getUserId().']">';
	// Only show this option for SVN
	if ($repos->getType() == 'svn' && $repos->getAccessType() != ScmRepository::ACCESS_TYPE_SSH) {
		$select .= '<option value="'.ScmPlugin::NOACCESS.'" '.($module_access == ScmPlugin::NOACCESS ? 'selected="selected"' : '').'>'.$lang->getText('ScmModuleUser.NoAccess').'</option>'; 
	}
			
	$select .= '<option value="'.ScmPlugin::READ.'" '.($module_access == ScmPlugin::READ ? 'selected="selected"' : '').'>'.$lang->getText('ScmModuleUser.Read').'</option>';
	$select .= '<option value="'.ScmPlugin::WRITE.'" '.($module_access == ScmPlugin::WRITE ? 'selected="selected"' : '').'>'.$lang->getText('ScmModuleUser.Write').'</option>';
	$select .= '</select>';

	$trow[] = $select;
	echo $theme->tableRow($trow);
}

echo $theme->tableBottom();
echo '<div align="center">';
echo HTMLWidget::button("submit",$lang->getText('Common.Submit'),"submit","submit","");
echo HTMLWidget::button("button",$Lang->getText('Common.Back'),"cancel","","javascript:history.go(-1);");
echo "</div>";
echo '</form>';

$theme->footer($params);
?>
