<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: aclcheck.php 816 2006-04-18 17:36:20Z marcelo $
 * @author Marcelo Mottalli	marcelo@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('initialize.php');
require_once('GFConf.php');
require_once('core/Project.php');
require_once('plugins/scm/lib/ScmCommit.php');
require_once('plugins/scm/lib/ScmPreference.php');

if ($argc != 3) {
	echo 'Usage: '.basename(__FILE__).' <project UNIX name> <log message file>';
	exit(1);
}

// Get the project object
$project_name = $argv[1];
$project = ProjectPeer::retrieveByUnixName($project_name);
if (is_null($project)) {
	echo 'Invalid project name: '.$project_name."\n";
	exit(1);
}

ActionManager::notify("SCMCVS_PRECOMMIT", $project);

// Retrieve the SCM preferences for this project
$preferences = ScmPreferencePeer::retrieveByPk($project->getProjectId());
if (is_null($preferences)) {
	// No preferences specified... let it proceed
	exit(0);
}

if (!$preferences->getRequireTrackerItemId()) {
	// doesn't require a tracker item ID... let it proceed
	exit(0);
}

// Retrieve the user name from the environment
$user_arr = posix_getpwuid(posix_geteuid());
$user_name = $user_arr['name'];
if (!$user_name) {
	echo 'Couldn\'t retrieve user name';
	exit(1);
}

// Allow root to commit
if ($user_name == 'root') {
	exit(0);
} elseif ($user_name == 'anonymous') {
	echo 'You cannot commit as the anonymous user.';
	exit(1);
}

// Start the session as the user that's committing
$user = UserPeer::retrieveByUnixName($user_name);
if (is_null($user)) {
	echo 'Invalid user name: '.$user_name."\n";
	exit(1);
}
Session::getSession()->setUser($user);


// In this point, the log file must include at least one tracker item ID

$log_file = $argv[2];
if (!file_exists($log_file)) {
	echo 'File with log message '.$log_file.' doesn\'t exist';
	exit(1);
}

$log = file_get_contents($log_file);

$invalid_ids = array();
$messages = array();

$arr_statuses = ScmCommit::getStatusFromMessages($log);
$arr_time_entries = ScmCommit::getTimeEntriesFromMessages($log);
$tids_valid = ScmCommit::checkTrackerItemIds($log, $project, $invalid_ids, $messages);

if (!$tids_valid || count($arr_statuses['invalid']) > 0 || count($arr_time_entries['invalid']) > 0) {
    if (!$tids_valid) {
        if (count($invalid_ids) > 0) {
            echo 'The following Tracker Item Ids are not valid for this Project or you are violating commit rules:'."\n";
            echo 'Details: ' . "\n";
            foreach($messages as $id => $message) {
                echo '[#'.$id.'] = ' . $message . "\n";
            }
        } else {
            echo 'Commit not related to any Tracker Item.'."\n";
        }
        echo "\n";
        echo 'Please specify one or more valid Tracker Item Ids in the commit '."\n";
        echo 'log as "[#nnnn]". For example:'."\n";
        echo '$ cvs commit -m "Fixed [#2708]"'."\n";
    }

    if (count($arr_statuses['invalid']) > 0) {
        echo "\nThe commit message contains invalid status update references:\n";
        foreach($arr_statuses['invalid'] as $tid => $status) {
            echo " > Id " . $tid . ", status: " . $status . "\n";
            echo " > Available statuses: " . ScmCommit::getStatusesByTid($tid) . "\n";
        }
    }

    if (count($arr_time_entries['invalid']) > 0) {
        echo "\nThe commit message contains invalid time entry references:\n";
        foreach($arr_statuses['invalid'] as $tid => $time_entry) {
            echo " > Id " . $tid . ", time entry: " . $time_entry . "\n";
	        echo " > Time entries must be greater than: " . ScmCommit::MIN_TIME_ENTRY;
        }
    }

	exit(1);
}

exit(0);
?>
