<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version $Id$
 * @author Daniel A. Perez daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('initialize.php');

require_once('GFConf.php');
require_once('GFUrl.php');
require_once('plugins/scmcvs/lib/CVSRepository.php');
require_once('plugins/scm/lib/ScmCommitWrapper.php');


/**
 * PostCvsCommit - posts the commit to the scmcommit script
 *
 */
class PostCvsCommit {
	
	private $debug,$debug_file; // debugging vars
	private $use_ssl,$default_domain,$cvs_version;
	private $argc,$argv;
	
	/**
	 * Constructor
	 *
	 * Initialization of variables
	 * 
	 */	
	function __construct($argc,$argv) {
		$this->debug = GFConf::get('scmcvs.debug');
		$this->debug_file = GFConf::get('scmcvs.debugFile');
		$this->use_ssl = GFConf::get('scmcvs.useSsl');
		$this->default_domain = GFConf::get('system.hostName');
		$this->cvs_version = GFConf::get('scmcvs.binaryVersion');

		$this->argc = $argc;
		$this->argv = $argv;
		if ($this->debug) {
			$name = $this->debug_file;
			$this->debug_file = fopen($this->debug_file, "a+");
			chmod($name,0666);
		}
	}

	/**
	 * Parse input and get the Log message.
	 *
	 * @param   string   $Input Input from stdin.
	 *
	 * @return  array    Array of lines of Log Message.
	 */
	private function getLog($Input) {
		$Lines = explode("\n", $Input);
		$ii = count($Lines);
		$Logging=false;
		$Log = '';
		for ( $i=0; $i < $ii ; $i++ ) {
			if ($Logging==true)
				$Log.=$Lines[$i]."\n";
			if ($Lines[$i]=='Log Message:')
				$Logging=true;
		}
		return trim($Log);
	}
	
	/**
	 * Usage - prints the error and message of how to use the file
	 *
	 */
	private function usage() {
		$arg = $this->argv[0];
		if ($this->cvs_version == "1.12" ) {
			echo "Usage: $arg <Repository> <Path> [<File> <VersionFrom> <VersionTo>]xN\n";
		} 
		if ($this->cvs_version == "1.11" ) {
			echo "Usage: $arg <Repository> [<File>,<VersionFrom>,<VersionTo>]xN\n";
		}
		exit(0);		
	}
	
	/**
	 * Main procedure
	 *
	 */
	function run() {
		
		$files = array();
		
		if ($this->debug) {
			echo "Arguments count: ".$this->argc."\n";
			echo "Arguments passed to post.php:\n";
			print_r($this->argv);
		}
		
		switch ($this->cvs_version) {
			case "1.11" : {
				if ($this->argc != 3 ) {
					$this->usage();
				}
				$repository      = $this->argv[1];
				$parameters = explode(' ,,', $this->argv[2]);
				$path = $parameters[0];
				for($i = 1; $i < count($parameters); $i++) { 
					$filesInformation = explode(',', trim($parameters[$i], ','));
					$files[] = array(
						'path' => $path."/".$filesInformation[0],
						'prev_version' => (isset($filesInformation[1])) ? $filesInformation[1] : '',
						'new_version' => (isset($filesInformation[2])) ? $filesInformation[2] : ''
						);
				}
				break;
			}
			case "1.12" : {
				if ($this->argc < 6 ) {
					$this->usage();
				}	
				if ( (($this->argc - 3) % 3 ) != 0 ) {
					echo "There should be 3 params + 3*N, instead of $this->argc\n";
					$this->usage();
				}
				$NumFiles= (($this->argc-3) / 3 ); // 3 Fixed params + 3 * File
				$repository      = $this->argv[1];
				$path            = $this->argv[2];
				for ( $i=0; $i < $NumFiles; $i++ ) {
					$files[] = array(
						'path' => $path."/".$this->argv[3 + 3*$i],
						'prev_version' => (isset($this->argv[4 + 3*$i])) ? $this->argv[4 + 3*$i] : '',
						'new_version' => (isset($this->argv[5 + 3*$i])) ? $this->argv[5 + 3*$i] : ''
					);
				}
				break;
			} 
			default : {
				die("You must specify the cvs version in the config file.\n");
			}
		}
		
		$userarray = posix_getpwuid ( posix_geteuid ( ) );
		$username = $userarray['name'];
		$input = file_get_contents ("php://stdin" );
		$log = $this->getLog($input);
		
		if ($this->debug) {
			echo "Vars filled\n";
			echo "username: ".$username." \n";
			echo "log:  ".$log." \n";
			fwrite($this->debug_file,"Vars filled:\n");
			fwrite($this->debug_file,"username: ".$username."\n");
			fwrite($this->debug_file,"log: ".$log."\n");
			fclose($this->debug_file);
		}

		/* Generate the ScmCommitWrapper object */
		// Extract the project name (the last component of the path)
		$repository = preg_replace('/\\/*$/', '', $repository);		// remove trailing /
		preg_match('/([^\\/]*)$/', $repository, $matches);
		$project_name = $matches[1];
		
		$project = ProjectPeer::retrieveByUnixName($project_name);
		$repos = new CVSRepository($project);
		$user = UserPeer::retrieveByUnixName($username);
		
		$wrapper = new ScmCommitWrapper($repos, $user, $log, $files);
		$wrapper->generateCommitData();
	}	
}

$poster = new PostCvsCommit($argc,$argv);
$poster->run();
?>
