<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a ProjectScm and saves it to the 
// database
//
require_once('plugins/scm/lib/ProjectScm.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('plugins/scmcvs/lib/CVSRepository.php');
require_once('plugins/scm/lib/ScmPreferencePeer.php');
require_once('plugins/scm/lib/ScmCommitFilterProject.php');


if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$session = Session::getSession();
$Lang = $session->getLanguage();
$strU = new StringUtils(StringUtils::FULL_HTML);

$project = GFUrl::getUrl()->getProject();
$cvsplugin = $project->getPluginByName('scmcvs');
if (is_null($cvsplugin)) {
	throw new InvalidPluginException('scmcvs');
}

$repos = new CVSRepository($project);
$ProjectScm = $repos->getScmConfiguration();

// $ProjectScm can be null if SCM is not currently configured for this project
if (!is_object($ProjectScm)) {
	if (!$project->getPermission()->isAdmin()) {
		throw new PermissionDeniedException();
	} else {
		// this means it's the first time we're configuring SCM for this project
		$ProjectScm = new ProjectScm();
		$ProjectScm->setProjectId($project->getProjectId());
	}
} else if (!$ProjectScm->userCanUpdate()) {
	throw new PermissionDeniedException();
}


$scm_pref = ScmPreferencePeer::retrieveByPk($project->getProjectId());
$scm_pref->setProjectId($project->getProjectId());

try {
	if (isset($_POST['require_tracker_item_id'])) {
		$scm_pref->setRequireTrackerItemId(true);
	} else {
		$scm_pref->setRequireTrackerItemId(false);
	}
	if (isset($_POST['mail_commits'])) {
		if ($scm_pref->getMailCommits() == false) {
			$notify = true;
		}
		$scm_pref->setMailCommits(true);
	} else {
		if ($scm_pref->getMailCommits() == true) {
			$notify = true;
		}
		$scm_pref->setMailCommits(false);
	}
	$scm_pref->save();

	$ProjectScm->setRootDirectory(null);


	if(GFConf::get('system.useDaemon')){
		GFDaemon::notify('DAEMON_ACCESS_TYPE_UPDATE', $project->getUnixName());
	}
	
	if (!$ProjectScm->getControlled()) {
		$ProjectScm->setExternalBrowseUrl(Validator::sanitizeString($_POST['external_browse_url']));
	}
	
	$ProjectScm->setBrowseText( $strU->cleanHtml($_POST['browse_text']) );
	$ProjectScm->setAccessText( $strU->cleanHtml($_POST['access_text']) );
	$ProjectScm->save();
	
	$feedback = $Lang->getText('ProjectScm.ChangesSaved');
} catch (InvalidDataException $ex) {
	$errors = $ex->getErrors();
	$feedback = '* '.implode('<br />* ', $errors);
}

include('ProjectScmEdit.php');

?>
