<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: aclcheck.php 816 2006-04-18 17:36:20Z marcelo $
 * @author Marcelo Mottalli	marcelo@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once('initialize.php');
require_once('GFConf.php');
require_once('SystemUtils.php');
require_once('core/Project.php');
require_once('plugins/scm/lib/ScmPreference.php');
require_once('plugins/scmgit/lib/GitReceive.php');

//Get Project
$cwd = getcwd();
$path = explode('/', $cwd);
$project_name = $path[count($path)-1];
$project = ProjectPeer::retrieveByUnixName($project_name);
if (!$project) {
	echo "Invalid project name: $project_name\n";
	exit(1);
}

$gitRepo = new GitRepository($project);

// Get the committing user
if ($gitRepo->getAccessType() == GITRepository::ACCESS_TYPE_SSH) {
	// Retrieve the user name from the environment
	$user_arr = posix_getpwuid(posix_geteuid());
	$user_name = $user_arr['name'];
} else {
	// Committing through HTTP
	$user_name = (isset($_ENV["REMOTE_USER"]) ? $_ENV["REMOTE_USER"] : "");
	//For some reason, this seems to work on gforge.com (git 1.7.4) but it doesn't
	//on local Centos6 with official git packages (git 1.7.1), so if $user_name has not been
	// set, we use getenv()
	if (!$user_name) {
		$user_name = getenv("REMOTE_USER");
	}
}

	if (!$user_name) {
	echo "Couldn't retrieve username from environment: $user_name";
		exit(1);
	}
	
	if ($user_name == 'root') {
	//We allow the push, no further check done (but we won't be creating associations in post_receive eithr
		exit(0);
	}
	
	$user = UserPeer::retrieveByUnixName($user_name);
if (!$user) {
	echo "Invalid username $user_name\n";
		exit(1);
	}

$gitReceive = new GitReceive($project, $user, GitReceive::PRE_SCRIPT);

echo "* Starting Gforge pre-receive hook\n\n";
echo "\tBranches to be processed: " . implode(' ', $gitReceive->getRefs())."\n\n";
$scmPref = ScmPreferencePeer::retrieveByPk($project->getId());
if($scmPref && $scmPref->getRequireTrackerItemId()) {
	$gitReceive->parseIds();
	echo "\tValidating logs, please wait...\n";
	if (!$gitReceive->allLogsValid()) {
		echo $gitReceive->getInvalidCommitsMessages();
			exit(1);	
		} 			
	
		}
		
if ($gitRepo->getAccessType() == GitRepository::ACCESS_TYPE_SSH && !$gitReceive->allValidAcls()) {
			exit(1);
		}


exit(0);

?>

