<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/initialize.php');
require_once('system/Cronjob.php');
require_once('core/ProjectPeer.php');
require_once('core/PluginPeer.php');
require_once('core/ProjectPluginPeer.php');
require_once('plugins/scm/lib/ProjectScmPeer.php');
require_once('plugins/scmgit/lib/GITRepository.php');
require_once('GFConf.php');

class CreateGitRepositoriesCronjob extends Cronjob {
	private $repositoryPath, $gitRoot;
	
	const PLUGIN_NAME = "scmgit";
	
	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}
	
	public function run() {
		$this->cleanup();
		$this->gitRoot = GFConf::get('scmgit.localGitRoot');
		
		
		// Loop through and create/update each repository for every project that uses the Git plugin
		if (is_dir($this->gitRoot)) {
			$this->updateGitRepositories();
		} elseif (is_file($this->gitRoot)) {
			$this->logCron($this->gitRoot." exists but is a file");
			$this->commitLog();
			exit(1);
		} else {
			// Need to create the Git root
			if (mkdir($this->gitRoot)) {
				chmod($this->gitRoot, 0755);
				$this->updateGitRepositories();
			} else {
				$this->logCron("Unable to create ".$this->gitRoot." directory");
				$this->commitLog();
				exit(1);
			}
		}
	}

	private function updateGitRepositories() {
		// lets get all projects that use the plugin
		$c = new Criteria();
		$c->add(PluginPeer::PLUGIN_NAME, self::PLUGIN_NAME);
		$project_plugins = ProjectPluginPeer::doSelectJoinPlugin($c);
		
		foreach ($project_plugins as $project_plugin) {
			$project = $project_plugin->getProject(); // get the project
			$gitplugin = $project->getPluginByName('scmgit');
			
			$repos = new GITRepository($project);
			// GIT not configured for this project
			if (is_null($repos) || !$repos->isConfigured()) {
				continue;
			}
			
			try {
				if($project->isActive()){
					$repos->setupRepository();
				}
			} catch (Exception $e) {
				$this->logCron($e->getMessage());
			}
		}
		
		$this->commitLog();
	}
	
	public function cleanup() {
		$c = new Criteria();	
		$c->add(DeletedProjectPeer::ISDELETED, 0);	
		$deletedProjects = DeletedProjectPeer::doSelect($c);
		$gitroot = GFConf::get('scmgit.localGitRoot');

		$deletedRoot = SystemUtils::buildFilePath($gitroot, '/.deleted');
		if (!is_dir($deletedRoot)) {
			mkdir($deletedRoot);
		}

		$output = array();
		
		foreach ($deletedProjects as $deletedProject){
			$deletedUnixName = $deletedProject->getUnixGroupName();

			$reposPath = SystemUtils::buildFilePath($gitroot, $deletedUnixName);
			if (!is_dir($reposPath)) {
				// Repository doesn't exist
				continue;
			}

			$deletedPath = SystemUtils::buildFilePath($deletedRoot, '/'.$deletedUnixName);

			SystemUtils::execute('mv -f '.$reposPath.' '.$deletedPath, $output);
			$deletedProject->delete();
		}	

		SystemUtils::execute('chown -h -R root:root '.$deletedRoot, $output);
		SystemUtils::execute('chmod -R o-rwx '.$deletedRoot, $output);
	}
}

if(!defined("TEST_RUN")){
	$cronjob = new CreateGitRepositoriesCronjob();
	$cronjob->run();
}
?>
