<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id:
 * @author Federico Tarantini	federico@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugin/GFPlugin.php');
require_once('plugin/ActionManager.php');
require_once('plugin/PluginResult.php');
require_once('plugins/scm/lib/ScmPlugin.php');
require_once('core/Language.php');



class ScmgitPlugin extends ScmPlugin {
	function __construct() {
	}	
	
	/**
	 *	Different types of plugins are possible -
	 *	GFPlugin::SITE_PLUGIN = 'site'
	 *	GFPlugin::PROJECT_PLUGIN = 'project'
	 *	GFPlugin::USER_PLUGIN = 'user'
	 */
	public function getType() {
		return GFPlugin::PROJECT_PLUGIN;
	}	
	
	/**
	 *	get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	public function getName() {
		return 'scmgit';
	}

	public function getListeners() {
		$listeners = parent::getListeners();
		$listeners[] = 'GET_SCMGITADMIN_NAV_ELEMENTS';
		$listeners[] = 'PURGE_DELETED_PROJECT';
		return $listeners;
	}

	function handleAction($listenername,$params) {
		switch ($listenername) {
			case 'GET_SCMGITADMIN_NAV_ELEMENTS':
				return $this->getScmAdminNavElements($params); 
				break;
			case 'PURGE_DELETED_PROJECT':
				return $this->purgeDeletedProject($params); 
				break;
			default:
				return parent::handleAction($listenername, $params);
				break;
		}
	}
	
	/**
	 *	get project uri to be used as root for constructing uris
	 *	@return string
	 */
	function getProjectPluginUri() {
		return GFUrl::getUrl()->projectURL($this->project,'','scmgit');
	}

	protected function buildMainTabsListener($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$result = new PluginResult($this);
		$value = array(
				'text'	=> Language::getSessText('Scmgit.TabString'),
				'action'	=> GFUrl::getUrl()->projectURL($project,'','scmgit'),
				'id'	=> 'git'
			);
		$result->setResult($value);
		return $result;
	}

	/**
	 *      getScmNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	protected function getScmNavElements($params) {
		$lang = Language::getSessLang();
		
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$url = GFurl::getUrl();
		$lang = Session::getSession()->getLanguage();
		$user = Session::getSession()->getUser();
		$loggedin = Session::getSession()->isLoggedIn();
		$project = $params['project'];		
		
		$elements = array(
						array(
							'link'=> GFUrl::getUrl()->projectUrl($project,'AccessInfo','scmgit'),
							'description' => $lang->getText('Scmgit.AccessInfo')
						),
 						array(
							'link'=> GFUrl::getUrl()->projectUrl($project,'ScmCommitBrowse','scmgit'),
							'description' => $lang->getText('Scm.BrowseCommits')
						),						
						array(
							'link'=> GFUrl::getUrl()->projectUrl($project,'ScmStats','scmgit'),
							'description' => $lang->getText('Scmgit.Stats')
						),
						array(
							'link'=> 'http://www.kernel.org/pub/software/scm/git/docs/',
							'description' => $lang->getText('Scmgit.ReferenceTitle'),
							'target' => '_blank'
						),
					);
				
		if ($loggedin) {
			if ($project->getPermission()->isAdmin()) {
				$elements[] = array('link' => $url->projectURL($project,'','scmgit','admin', ''),
					'description' => $lang->getText('Tabs.Admin'));
			}
		}				
		
		$result = new PluginResult($this);
		$arr = array(
			'title'=> 'Scmgit',
			'elements'=> $elements
			);
		$result->setResult($arr);
		return $result;
	}
	
	protected function buildProjectSummary($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$project = $params['project'];
		$theme = $params['theme'];
		
		$result = new PluginResult($this);
		$stats = $repos->getStats();
		
		$value = 'hola';

		$result->setResult($value);
		return $result;
	}

	/**
	 * 	The getObserverRoleChoices listener - this hook will fire
	 *	when the observer role is modified.
	 * 	It returns an array of role settings or null
	 *	@return	array or null
	 */
	function getObserverRoleChoices($params) {	
		return parent::getObserverRoleChoices($params);
	}
	
	/**
	 * Retrieves the concerning data and updates its observer role settings
	 *
	 * @param array	$data
	 */	
	function updateObserverSettings($data) {	
		return parent::updateObserverSettings($data);
	}		

	/**
	 *      getProjectAdminNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	protected function getProjectAdminNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$lang = Session::getSession()->getLanguage();
		$url = GFUrl::getUrl();

		$result = new PluginResult($this);
		$arr = array(
			'title'=> $lang->getText('Scmgit.TabString'),
			'elements'=> array(
			array('link'=> $url->projectUrl($params['project'], '', $this->getName(), 'admin'),
				'description'=> $lang->getText('Scmgit.Admin'))
			)
		);
		$result->setResult($arr);
		return $result;
	}
	
	protected function getScmAdminNavElements($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$url = GFurl::getUrl();
		$lang = Session::getSession()->getLanguage();
		$user = Session::getSession()->getUser();
		$loggedin = Session::getSession()->isLoggedIn();
		$project = $params['project'];		
		
		$elements = array(
						array(
							'link'=> $url->projectUrl($project,'ScmModuleBrowse','scmgit', 'admin'),
							'description' => $lang->getText('Scmgit.BrowseModules')
						),
						array(
							'link'=> $url->projectUrl($project,'CodeSearchModuleBrowse','scmgit', 'admin'),
							'description' => $lang->getText('CodeSearchModuleGit.NavTitle')
						),
						array(
							'link'=> $url->projectUrl($project,'ScmCommitFilterProjectBrowse','scmgit', 'admin'),
							'description' => $lang->getText('ScmCommitFilter.NavTitle')
						),
					);
				
		$result = new PluginResult($this);
		$arr = array(
			'title'=> $lang->getText('Scmgit.AdminNavTitle'),
			'elements'=> $elements
			);
		$result->setResult($arr);
		return $result;
	}
	
	protected function purgeDeletedProject($params) {
		// See if there's a managed repository for the deleted project
		$deleted_project = $params['deleted_project'];
		$unix_name = $deleted_project->getUnixGroupName();
		
		$repository = SystemUtils::buildFilePath(GFConf::get('scmgit.localGitRoot'), '/'.$unix_name);
		if (is_dir($repository)) {
			$this->purgeRepository($repository, GFConf::get('scmgit.localGitRoot').'/.deleted');
		}
	}

}
