<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: ProjectScmEditAction.php 3226 2008-08-06 17:58:08Z bigdisk $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a ProjectScm and saves it to the 
// database
//
require_once('plugins/scm/lib/ProjectScm.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('plugins/scmgit/lib/GITRepository.php');
require_once('plugins/scm/lib/ScmPreferencePeer.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$session = Session::getSession();
$Lang = $session->getLanguage();
$strUtils = new StringUtils(StringUtils::FULL_HTML);

$project = GFUrl::getUrl()->getProject();
$gitplugin = $project->getPluginByName('scmgit');
if (is_null($gitplugin)) {
	throw new InvalidPluginException('scmgit');
}

$repos = new GitRepository($project);
$ProjectScm = $repos->getScmConfiguration();
$notify = false;

// $ProjectScm can be null if SCM is not currently configured for this project
if (!is_object($ProjectScm)) {
	if (!$project->getPermission()->isAdmin()) {
		throw new PermissionDeniedException();
	} else {
		// this means it's the first time we're configuring SCM for this project
		$ProjectScm = new ProjectScm();
		$ProjectScm->setProjectId($project->getProjectId());
	}
} else if (!$ProjectScm->userCanUpdate()) {
	throw new PermissionDeniedException();
}

$scm_pref = ScmPreferencePeer::retrieveByPk($project->getProjectId(), false);
$scm_pref->setProjectId($project->getProjectId());

try {
	if (isset($_POST['require_tracker_item_id'])) {
		$scm_pref->setRequireTrackerItemId(true);
	} else {
		$scm_pref->setRequireTrackerItemId(false);
	}
	if (isset($_POST['mail_commits'])) {
		if ($scm_pref->getMailCommits() == false) {
			$notify = true;
		}
		$scm_pref->setMailCommits(true);
	} else {
		if ($scm_pref->getMailCommits() == true) {
			$notify = true;
		}
		$scm_pref->setMailCommits(false);
	}
	if (GFConf::get('scm.allowChangeScmAccess')) {
		if ($_POST['use_ssh'] == "true") {
			if ($ProjectScm->getUseSsh() == false) {
				$notify = true;
			}
			$ProjectScm->setUseSsh(true);
		} else {
			if ($ProjectScm->getUseSsh() == true) {
				$notify = true;
			}
			$ProjectScm->setUseSsh(false);
		}
	}
	$scm_pref->save();

	$ProjectScm->setRootDirectory(null);
	
	if (!$ProjectScm->getControlled()) {
		$ProjectScm->setExternalBrowseUrl(Validator::sanitizeString($_POST['external_browse_url']));
	}

	$ProjectScm->setBrowseText( $strUtils->cleanHtml($_POST['browse_text']) );
	$ProjectScm->setAccessText( $strUtils->cleanHtml($_POST['access_text']) );
	$ProjectScm->save();

	if ($notify && GFConf::get('system.useDaemon')) {
		GFDaemon::notify('DAEMON_ACCESS_TYPE_UPDATE', $project->getUnixName());
	}
	
	$feedback = $Lang->getText('ProjectScm.ChangesSaved');
} catch (InvalidDataException $ex) {
	$errors = $ex->getErrors();
	$feedback = '* '.implode('<br />* ', $errors);
}


include('ProjectScmEdit.php');

?>
