#!/usr/bin/php
<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
/**
 * Checks that all the text files committed to the repository have UNIX newline characters. To use it, 
 * add the following SCM commit filter in SVN:
 * /opt/gforge/plugins/scmsvn/bin/check_newlines_svn.php $PROJECT $PATH $FILE
 */
require_once('initialize.php');
require_once('GFConf.php');
require_once('SystemUtils.php');


// Everything must be written to stderr
function log_error($error) {
	file_put_contents("php://stderr", $error."\n");
}

if ($argc != 4) {
	log_error("Usage: $argv[0] <project name> <path to repository> <txn>");
	exit(1);
}

$projectName = $argv[1];
$repositoryPath = $argv[2];
$txn = $argv[3];

// Get a list of the changed files using the "svnlook" command
$svnlook = GFConf::get('scmsvn.svnpath').'/svnlook';
$cmd = 'LC_ALL="en_US.UTF-8" '.$svnlook.' changed -t '.escapeshellarg($txn).' '.escapeshellarg($repositoryPath).' | sed \'s/[A-Z]*   //\'';
SystemUtils::execute($cmd, $changed);

$invalid_files = array();

foreach ($changed as $file) {
	if (checkNL($file)) {
		$invalid_files[] = $file;
	}
}

if (count($invalid_files) > 0) {
	log_error("The following files contain invalid newline characters: \n".implode("\n", $invalid_files)."\nPlease correct them and commit again.");
	exit(1);
}

exit(0);

// Returns true if the file has DOS newlines
function checkNL($file) {
	global $svnlook, $txn, $repositoryPath;
	
	// File extensions to check for DOS chars
	$text_extensions = array('txt', 'php', 'inc', 'js', 'java');
	
	// Extract the extension from the file (note: files without extension are assumed to be text)
	preg_match('/\\.([^\\.]*)$/', $file, $res);
	if (isset($res[1])) {
		$extension = $res[1];
		$is_text_file = false;
		foreach ($text_extensions as $text_extension) {
			if (strcasecmp($extension, $text_extension) == 0) {
				$is_text_file = true;
			}
		}
		if (!$is_text_file) {
			return false;
		}
	}
	
	$cmd = 'LC_ALL="en_US.UTF-8" '.$svnlook.' cat -t '.escapeshellarg($txn).' '.escapeshellarg($repositoryPath).' /'.$file;
	$output = array();
	SystemUtils::execute($cmd, $output, false);
	
	if (strpos($output, "\n\r") !== false) {
		// Found a DOS newline
		return true;
	}
	
	return false;
}

?>
