<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Daniel Perez	<daniel@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugin/GFPlugin.php');
require_once('plugin/ActionManager.php');
require_once('plugin/PluginResult.php');
require_once('plugins/scm/lib/ScmPlugin.php');
require_once('core/Language.php');
require_once('plugins/scmsvn/lib/SVNRepository.php');


class ScmsvnPlugin extends ScmPlugin {
	/*function __construct(Project $project) {
		parent::__construct($project);
	}*/
	function __construct() {
	}	
	
	/**
	 *	Different types of plugins are possible -
	 *	GFPlugin::SITE_PLUGIN = 'site'
	 *	GFPlugin::PROJECT_PLUGIN = 'project'
	 *	GFPlugin::USER_PLUGIN = 'user'
	 */
	public function getType() {
		return GFPlugin::PROJECT_PLUGIN;
	}
	
	public function getListeners() {
		$listeners = parent::getListeners();
		$listeners[] = 'GET_SCMSVNADMIN_NAV_ELEMENTS';
		$listeners[] = 'GET_SCM_SERVER_TYPES';
		return $listeners;
	}	
	
	function handleAction($listenername,$params) {
		switch ($listenername) {
			case 'GET_SCMSVNADMIN_NAV_ELEMENTS':
				return $this->getScmAdminNavElements($params); 
				break;
			case 'GET_SCM_SERVER_TYPES':
				return $this->getScmServerType($params); 
				break;
			default:
				return parent::handleAction($listenername, $params);
				break;
		}
	}
	
	/**
	 *	get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	public function getName() {
		return 'scmsvn';
	}

	/**
	 *	get project uri to be used as root for constructing uris
	 *	@return string
	 */
	function getProjectPluginUri() {
		return GFUrl::getUrl()->projectURL($this->project,'','scmsvn');
	}

	protected function buildMainTabsListener($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$result = new PluginResult($this);
		$value = array(
				'text'	=> Language::getSessText('Scmsvn.TabString'),
				'action'	=> GFUrl::getUrl()->projectURL($project,'','scmsvn'),
				'id'	=> 'svn'
			);
		$result->setResult($value);
		return $result;
	}

	/**
	 *      getScmNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	protected function getScmNavElements($params) {
		$lang = Language::getSessLang();
		
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$url = GFurl::getUrl();
		$lang = Session::getSession()->getLanguage();
		$user = Session::getSession()->getUser();
		$loggedin = Session::getSession()->isLoggedIn();
		$project = $params['project'];		
		
		$elements = array(
						array(
							'link'=> GFUrl::getUrl()->projectUrl($project,'AccessInfo','scmsvn'),
							'description' => $lang->getText('Scmsvn.AccessInfo')
						),
						array(
							'link'=> GFUrl::getUrl()->projectUrl($project,'','scmsvn'),
							'description' => $lang->getText('Scmsvn.Browse')
						),
 						array(
							'link'=> GFUrl::getUrl()->projectUrl($project,'ScmCommitBrowse','scmsvn'),
							'description' => $lang->getText('Scm.BrowseCommits')
						),						
						array(
							'link'=> GFUrl::getUrl()->projectUrl($project,'ScmStats','scmsvn'),
							'description' => $lang->getText('Scmsvn.Stats')
						),
						array(
							'link'=> 'http://svnbook.red-bean.com/en/1.2/index.html',
							'description' => $lang->getText('Scmsvn.ReferenceTitle'),
							'target' => '_blank'
						),
					);
				
		if ($loggedin) {
			if ($project->getPermission()->isAdmin()) {
				$elements[] = array('link' => $url->projectURL($project,'','scmsvn','admin', ''),
					'description' => $lang->getText('Tabs.Admin'));
			}
		}				
		
		$result = new PluginResult($this);
		$arr = array(
			'title'=> 'SVN',
			'elements'=> $elements
			);
		$result->setResult($arr);
		return $result;
	}
	
	protected function buildProjectSummary($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$project = $params['project'];
		$theme = $params['theme'];
		
		$repos = new SVNRepository($project);
		if (!$repos->isCreated()) {
			return null;
		}

		$result = new PluginResult($this);
		$stats = $repos->getStats();
		
		
		$value = '<hr size="1" /><a href="'. GFUrl::getUrl()->projectURL($project,'','scmsvn').'">';
		$value .= $theme->image('ic/ofolder15.png','15','13',array('alt'=>Language::getSessText('Project.TabString'), 'border' => '0'));
		$value .= '&nbsp;'.Language::getSessText('Scmsvn.TabString').'</a>&nbsp;&nbsp;';
		$value .= '('.Language::getSessText('Scm.ScmSummary', array($stats['checkouts'], $stats['adds'], $stats['commits'])).')';
		
		$result->setResult($value);
		return $result;
	}

	/**
	 * 	The getObserverRoleChoices listener - this hook will fire
	 *	when the observer role is modified.
	 * 	It returns an array of role settings or null
	 *	@return	array or null
	 */
	function getObserverRoleChoices($params) {	
		return parent::getObserverRoleChoices($params);
	}
	
	/**
	 * Retrieves the concerning data and updates its observer role settings
	 *
	 * @param array	$data
	 */	
	function updateObserverSettings($data) {	
		return parent::updateObserverSettings($data);
	}		

	/**
	 *      getProjectAdminNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	protected function getProjectAdminNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}

		$lang = Session::getSession()->getLanguage();
		$url = GFUrl::getUrl();

		$result = new PluginResult($this);
		$arr = array(
			'title'=> $lang->getText('Scmsvn.TabString'),
			'elements'=> array(
			array('link'=> $url->projectUrl($params['project'], '', $this->getName(), 'admin'),
				'description'=> $lang->getText('Scmsvn.Admin'))
			)
		);
		$result->setResult($arr);
		return $result;
	}
	

	protected function getScmAdminNavElements($params) {
		$project = $params['project'];
		if (!$project->usesPlugin($this->getName())) return null;

		$url = GFurl::getUrl();
		$lang = Session::getSession()->getLanguage();
		$user = Session::getSession()->getUser();
		$loggedin = Session::getSession()->isLoggedIn();
		$project = $params['project'];		
		
		$elements = array(
						array(
							'link'=> $url->projectUrl($project,'ScmModuleBrowse','scmsvn', 'admin'),
							'description' => $lang->getText('Scmsvn.BrowseModules')
						),
						array(
							'link'=> $url->projectUrl($project,'CodeSearchModuleBrowse','scmsvn', 'admin'),
							'description' => $lang->getText('CodeSearchModuleSVN.NavTitle')
						),
						array(
							'link'=> $url->projectUrl($project,'ScmCommitFilterProjectBrowse','scmsvn', 'admin'),
							'description' => $lang->getText('ScmCommitFilter.NavTitle')
						),
					);
				
		$result = new PluginResult($this);
		$arr = array(
			'title'=> $lang->getText('Scmsvn.AdminNavTitle'),
			'elements'=> $elements
			);
		$result->setResult($arr);
		return $result;
	}

	protected function getScmServerType($params) {
		$pr = new PluginResult($this);
		$pr->setResult('SVN');
		return $pr;
	}

}
?>
