<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('GFUrl.php');
require_once('plugins/scmsvn/lib/html/ScmSVNTemplate.php');
require_once('plugins/scmsvn/lib/SVNRepository.php');


$project = GFUrl::getUrl()->getProject();
$svnplugin = $project->getPluginByName('scmsvn');
if (is_null($svnplugin)) {
	throw new InvalidPluginException('scmsvn');
}

$session = Session::getSession();
$lang = $session->getLanguage();

$unix_name = 'developername';
if ($session->isLoggedIn()) {
	$user = $session->getUser();
	if (is_object($user)) {
		$unix_name = $user->getUnixname();
	}
}

$template = new ScmsvnTemplate();

$params = array(
	'selectedbottomtab'	=> 'scmsvn',
	'project' => $project,
	'action' => isset($action) ? $action : '',
	'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
	'title' => $lang->getText('Scmsvn.AccessInfo')
	);

$template->header($params);

// See if SCM is configured for this project
$repos = new SVNRepository($project);
if ($repos->isConfigured()) {
	if($repos->getAccessType() == SCMRepository::ACCESS_TYPE_SSH){
		$useSSH = true;
	} else {
		$useSSH = false;
	}
	$projectScm = $repos->getScmConfiguration();

	if (!$projectScm->getControlled()) {
		//@todo	This will be used when we add support for external SCM
		//die("UNSUPPORTED");
	}
	
	$root = GFConf::get('scmsvn.localSVNRoot');
	if (GFConf::get('system.splitScmServers')) {
		$hostname = $projectScm->getScmServer()->getHostname();
	} else {
		$hostname = GFConf::get('system.hostName');
	}
	// full SVNROOT is something like /svnroot/project_name

	$custom_access_text = $projectScm->getAccessText();
	if (!empty($custom_access_text)) {
?>
<p>
	<h3><?php echo $lang->getText('Scmsvn.AccessTextTitle') ?></h3>
</p>
<p>
	<?php echo $custom_access_text ?>
</p>
<?php
	}

	if ($projectScm->getControlled()) {		// Only display the access info if access is controlled
		if ($project->getIsPublic() && $projectScm->getEnableAnonread()) {
?>
<p>
	<h3><?php echo $lang->getText('Scmsvn.AnonymousAccess'); ?></h3>
</p>
<p>
	<?php echo $lang->getText('Scmsvn.AnonymousAccessDescription'); ?>
</p>
<p>
<?php
	if ($useSSH) {
		echo '<tt>svn checkout svn+ssh://anonymous@'.$hostname.$repos->getRoot().'</tt><br />'."\n";
	} else {
		echo '<tt>svn checkout '.GFUrl::getUrlPrefix() . $hostname .'/svn/'.$repos->getRepositoryName().'</tt><br />'."\n";
	}
?>
<br /><br />
</p>
<?php 	} /*if ($projectScm->getEnableAnonread()) */ ?>
<p>
	<h3><?php echo ($useSSH ? $lang->getText('Scmsvn.DeveloperAccessSSH') : $lang->getText('Scmsvn.DeveloperAccess')); ?></h3>
</p>
<p>
	<?php echo $lang->getText('Scmsvn.DeveloperAccessDescription'); ?>
</p>
<p>
	<?php
		if ($useSSH) {
			echo '<tt>svn checkout svn+ssh://<i>'.$unix_name.'</i>@'.$hostname.$repos->getRoot().'</tt><br />'."\n";
		} else {
			echo '<tt>svn checkout --username <i>'.$unix_name.'</i> '.GFUrl::getUrlPrefix() . $hostname .'/svn/'.$repos->getRepositoryName().'</tt><br />'."\n";
		}
	?>
</p>

<?php
	} /* $projectScm->getControlled() */
} else {		// SCM is not configured for this project
?>
	<a href="<?php echo GFUrl::getUrl()->projectURL($project, '', 'scmsvn', 'admin'); ?>"><?php echo $lang->getText('Scm.Admin'); ?></a>
	<p><h3><?php echo $lang->getText('Scm.AdminTitle'); ?></h3></p>
	<p><?php echo $lang->getText('Scm.NotConfigured') ?></p>

<?php
}
$template->footer($params);
?>
