<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Daniel Perez	<daniel@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugin/GFPlugin.php');
require_once('plugin/ActionManager.php');
require_once('plugin/PluginResult.php');
require_once('core/Language.php');
require_once('SnippetPeer.php');

class SnippetPlugin implements GFPlugin {

	function __construct() {
	}	
	
	/**
	 *	this is called when the first Action is called. Action manager 
	 *	gets the listeners for each plugin.
	 */
	public function initialize() {
		$listenTo = array(
			'BUILD_MAIN_TABS',
			'CHECK_DOWNLOAD_PERMS'
			);
		ActionManager::registerListeners($this, $listenTo);
	}

	/**
	 *	Different types of plugins are possible -
	 *	GFPlugin::SITE_PLUGIN = 'site'
	 *	GFPlugin::PROJECT_PLUGIN = 'project'
	 *	GFPlugin::USER_PLUGIN = 'user'
	 */
	public function getType() {
		return GFPlugin::SITE_PLUGIN;
	}	
	
	/**
	 *	get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	public function getName() {
		return 'snippet';
	}

	/**
	 *	get the array of action names that this plugin answers to.
	 *	@return	array
	 */
	public function getListeners() {
	}

	/**
	 *	handle action has a case statement to redirect the action to the appropriate function.
	 *	@param string
	 *	@param mixed
	 *	@return PluginResult
	 */
	public function handleAction($listenerName, $params) {
		switch ($listenerName) {
			case 'BUILD_MAIN_TABS': {
				return $this->_buildMainTabsListener($params);
				break;
			}
			case 'CHECK_DOWNLOAD_PERMS': {
				return $this->checkDownloadPerms($params);
				break;
			}			
		}
	}

	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return PluginResult
	 */
	public function search(GFSearch $search) {
	}
	

	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return resultset
	 */
	private function checkDownloadPerms($params) {
		$section = $params['section'];
		$ref_id = $params['ref_id'];
		$result = new PluginResult($this);
		$result->setResult(false); // false by default, looking for true
		// if its for another section, exit
		if ($section!=$this->getName()) {
			$result->setResult(false);
		} else {
			$c = new Criteria();
			$c->add(SnippetPeer::SNIPPET_ID,$ref_id);
			$snippet = SnippetPeer::doSelectOne($c);
			if (is_object($snippet)) {
				$result->setResult(true); // snippets are public
			}
		}
		return $result;
	}	
	
	private function _buildMainTabsListener($params) {
		$result = new PluginResult($this);
		$value = array(
				'text'	=> Language::getSessText('Snippet.TabString'),
				'action'	=> GFUrl::getUrl()->baseUrl('',$this->getName()),
				'id'	=> 'snippet'
			);
		$result->setResult($value);
		return $result;
	}
}

?>
