<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a Snippet and saves it to the 
// database
//
require_once('plugins/snippet/lib/Snippet.php');
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
require_once('lib/core/TroveLink.php');
require_once('html/TroveRenderer.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$c = new Criteria();
$c->add(SnippetPeer::SNIPPET_ID,$_REQUEST['snippet_id']);
$Snippet = SnippetPeer::doSelectOne($c);
if (!is_object($Snippet) || !$Snippet->userCanUpdate()) {
	throw new PermissionDeniedException();
}

$errors = array();
$Lang = Language::getSessLang();

//
// TODO Validate the values passed by $_POST. Store the errors in $error like
// $errors["field_name"] = error_description
//


$name = Validator::sanitizeString($_POST['name']);
if (strlen($name)==0) {
	$errors['name'] = Language::getSessText('Snippet.EmtpyName');
}
if (!isset($_POST['trove_programminglanguage'])) {
	$errors[] = Language::getSessText('Snippet.InvalidProg');
}	
if (!isset($_POST['trove_topic'])) {
	$errors[] = Language::getSessText('Snippet.InvalidTrove');
}
if (strlen($_POST['description'])==0) {
	$errors[] = Language::getSessText('Snippet.InvalidDesc');
}

// get the files
$c = new Criteria();
$c->add(FilesystemPeer::SECTION,Snippet::SECTION);
$c->add(FilesystemPeer::REF_ID,$Snippet->getSnippetId());
$fsos = FilesystemPeer::doSelect($c);
$has = count($fsos);


if (count($errors) == 0) {
	
	try {
		$Snippet->setSnippetName( $name );
		$Snippet->setSnippetDescription( Validator::sanitizeString($_POST['description']) );
		
		$Snippet->save();
		//
		//      handle the files
		//
		for ($i=0; $i<5; $i++) {
			$error=$_FILES['addfile']['error'][$i];
			$filename=$_FILES['addfile']['name'][$i];

			if ($error > 0 && $error != 4) {
				$fileerrors = StringUtils::getFileUploadErrorString($error, $filename);
				throw new InvalidDataException($fileerrors);
				continue;
			}

			$tmp_name=$_FILES['addfile']['tmp_name'][$i];
			if (!is_uploaded_file($tmp_name)) {
				continue;
			}
			$size=$_FILES['addfile']['size'][$i];
			$type=$_FILES['addfile']['type'][$i];

			$fs = new Filesystem();
			$fs->setSection(Snippet::SECTION);
			$fs->setRefId($Snippet->getSnippetId());

			$fs->setFileName( $filename );
			//$fs->setFileNameSafe();
			$fs->setFileType( $type );
			$fs->setFileSize( $size );
			$fs->setPostedBy( $session->getUser()->getUserId() );
			$fs->save();
			$fs->setFileDataFromPath( $tmp_name );
		}

		if (isset($_POST['deletefile']) && is_array($_POST['deletefile'])) {
			foreach ($_POST['deletefile'] as $file_id) {
	//TODO audit trail

				$fs = FilesystemPeer::retrieveByPK($file_id);
				if (!is_object($fs)
					|| $fs->getSection() != Snippet::SECTION
					|| $fs->getRefId() != $Snippet->getSnippetId()) {
					echo "Invalid File Delete";
				} else {
					$fs->delete();
				}
			}
		}

		$crit = new Criteria();
		$crit->add(TroveLinkPeer::SECTION, 'snippet');
		$crit->add(TroveLinkPeer::REF_ID, $Snippet->getSnippetId());
		TroveLinkPeer::doDelete($crit);
		
		// save trove information
		$renderer = new TroveRenderer();
		$renderer->setSection('snippet');
		$renderer->setRefId($Snippet->getSnippetId());
		$renderer->savePOSTData();
		
		$url = GFUrl::getUrl();
		GFUrl::redirect($url->currentBaseURL('SnippetBrowse'), array('feedback' => $Lang->getText('Snippet.EditFeedback'),'id'=>$Snippet->getSnippetId()));
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());               // attach object errors to validation errors
		$feedback = '* '.implode('<br />* ', $errors);
		//include('SnippetEdit.php');
		GFUrl::redirect($url->currentBaseURL('SnippetEdit'), array('feedback' => $feedback,'snippet_id'=>$Snippet->getSnippetId()));
	}
} else {
	$feedback = implode('<br />', $errors);
	//include('SnippetEdit.php');
	GFUrl::redirect($url->currentBaseURL('SnippetEdit'), array('feedback' => $feedback,'snippet_id'=>$_REQUEST['snippet_id']));
}

?>
