<?php
require_once('initialize.php');
require_once('MailParser.php');
require_once('session/Session.php');
require_once('core/User.php');
require_once('lib/core/Setting.php');
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('lib/StringUtils.php');

$session = Session::getSession();
$mail = new MailParser("php://stdin");

if($mail->isAutoResponse()) {
	//Ignore auto reponses
	exit(0);
}

$from = $mail->getFrom();
$body = $mail->getBody();
$subject = $mail->getSubject();
$users = UserPeer::retrieveByEmail($from);
//Extract alias
list($tracker_alias, $hostname) = explode('@',$mail->getTo());
$tracker_alias = Validator::sanitizeString($tracker_alias);
//Get Setting
$c = new criteria();
$c->add(SettingPeer::SETTING_NAME,Tracker::EMAIL_ALIAS);
$c->add(SettingPeer::SETTING_VALUE, $tracker_alias);
$setting = SettingPeer::doSelectOne($c);

try {
	if (!count($users)) {
		throw new Exception(Language::getSessText('Gateway.NoUser', array($from)));
	}

	if (!count($users)) {
		throw new Exception(Language::getSessText('Gateway.NoUser', array($from)));
	}

	/** @var User $user */
	$user = $users[0];
	$session->setUser($user);


	if (!$setting) {
        throw new Exception(Language::getSessText('Tracker.InvalidAlias', array($from)));
	}

	//Get tracker
	$tracker = TrackerPeer::retrieveByPk($setting->getRefId());

	if (!$tracker) {
		throw new InvalidDataException(Language::getSessText('Tracker.InvalidTracker'));
	}


	$project = $tracker->getProject();
	if (!$project || !$project->isActive() || !$project->usesPlugin('tracker')) {
		throw new PermissionDeniedException(Language::getSessText('Project.CantRead'));
	}

	$summary = Validator::sanitizeString($subject, false);
	if (StringUtils::containsHTML($body)) {
		$str = new StringUtils(StringUtils::BASIC_HTML);
		$details = $str->cleanHtml($body);
	} else {
		$details = StringUtils::HTMLFromPlainString($body);
	}

	if (!$details) {
		throw new InvalidDataException(Language::getSessText('Tracker.NoDetails'));
	}

	if (!$summary) {
		throw new InvalidDataException(Language::getSessText('Tracker.NoSummary'));
	}


	$ti = new TrackerItem();
	$ti->setTracker($tracker);
	if (!$ti->userCanCreate()) {
		throw new  PermissionDeniedException(Language::getSessText('Tracker.CantCreate'));
	}


	$ti->setDetails($details);
	$ti->setSummary($summary);
	$ti->setPriority(Tracker::PRIORITY_MEDIUM);
	$ti->setSubmittedBy($user->getUserId());
	$ti->setOpenDate(time());
	if ($tracker->getDatatype() == Tracker::DATATYPE_TASK && $tracker->getCloseDateIsEnabled()) {
		//We need to set the default close date to a week from now for task trackers
		$ti->setCloseDate(time() + (60*60*24*7));
	}
	$ti->setLastModifiedDate(time());

	//If it's a support tracker, then check the user's group membership
	$member_group_id = 0;
	if ($tracker->getDatatype() == Tracker::DATATYPE_SUPPORT) {
		$project_groups = $project->getMemberGroups();
		$project_group_ids = array();
		foreach($project_groups as $project_group) {
			$project_group_ids[] = $project_group->getUserId();
		}
		$member_groups = $user->getMemberGroups($project_group_ids);
		$groups_count = sizeof($member_groups);

		if ($groups_count > 1) {
			//Send an email back to the sender
			$subject = \Language::getSessText('Tracker.TrackerItemCannotBeCreatedSubject');
			$body = \Language::getSessText('Tracker.TrackerItemCannotBeCreatedBody');
			//Set recipient to original email sender
			$to = $from;
			$reply_to = $from = SystemUtils::getFromAddress();
			SystemUtils::sendMail($subject, html_entity_decode($body), $to, false, $from, $reply_to);
			exit;
		} else {
			if (sizeof($member_groups) > 0) {
				//Get member group to autoassign the TI to
				$member_group = $member_groups[0];
				$member_group_id = $member_group->getUserId();
			}
		}
	}

	//Get Tracker extra fields and check if there is a status field
	/** @var TrackerExtraField[] $tefs */
	$tefs = $tracker->getTrackerExtraFields();

	$statusField = null;
	$memberGroupField = null;
	foreach ($tefs as $tef) {
		if ($tef->getFieldType() == TrackerExtraField::TYPE_STATUS) {
			$statusField = $tef;
		} elseif ($tef->getFieldType() == TrackerExtraField::TYPE_MEMBERS) {
			$memberGroupField = $tef;
		}
		if ($statusField != null && $memberGroupField != null) {
			break;
		}
	}

	$arrExtraFields = array();
	if ($statusField) {
		$defaultValue = $statusField->getDefaultValue();
		if ($defaultValue) {
			$arrExtraFields[$statusField->getId()]=$defaultValue;
		} else {
			//No default set, grab the first open element and set it
			$elements = $statusField->getTrackerExtraFieldElements();
			$openValue =null;
			foreach ($elements as $el) {
				if($el->getStatusId() == Tracker::STATUS_OPEN){
					$openValue=$el->getElementId();
					break;
				}
			}
			$arrExtraFields[$statusField->getId()]=$openValue;
		}
	}
	if ($memberGroupField && $member_group_id > 0) {
		$arrExtraFields[$memberGroupField->getId()][0] = $member_group_id;
	}

	$ti->addExtraFields($arrExtraFields);
	$ti->save();

	if (GFConf::get('system.useDaemon')) {
		$attachments = $mail->getAttachments();

		foreach ($attachments as $attachment) {
			$ti->attachFileFromEmail($attachment,$user->getUserId());
		}
	}

	//Monitor item
	$ti->monitorItem($user);

	$subject = $message = \Language::getSessText('Tracker.TrackerItemCreated', array($ti->getId(), $ti->getTracker()->getTrackerName()));
	//Set recipient to original email sender
	$to = $from;
	//Set Sender as tracker item address
	$from = SystemUtils::getFromAddress('trackeritem+'.$ti->getTrackerItemId());
	$reply_to = $from;

	$autoresponse_instructions = $ti->getTracker()->getAutoresponseInstructions();
	if ($autoresponse_instructions != '') {
		$params['tracker_item'] = $ti;
		$params['body'] = $autoresponse_instructions;
		$prs = ActionManager::Notify('PROCESS_EMAIL_BODY',$params);
		$autoresponse_instructions = '';
		if ($prs->count() > 0) {
			foreach ($prs as $pluginresult) {
				$autoresponse_instructions = $pluginresult->getResult();
				if ($autoresponse_instructions != '') {
					break;
				}
			}
		}

		$message .= "\n\n" . str_replace("@ticketNumber", $ti->getTrackerItemId(), $autoresponse_instructions);	
	} else {
		$message .= "\n" . \Language::getSessText('TrackerItem.SummaryName') . ': '. $ti->getSummary() . "\n";
		$message .= \Language::getSessText('TrackerItem.DetailsName') . ': '. $ti->getDetails() . "\n\n";
		$message .= \Language::getSessText('Tracker.TrackerItemUrl') . "\n";
		$message .= GFUrl::getUrl()->getFullURL(
				GFUrl::getUrl()->projectURL($tracker->getProject(),'TrackerItemEdit', 'tracker', '',
				array('tracker_id' => $tracker->getTrackerId(),'tracker_item_id' => $ti->getId())), true) . "\n";
	}
	SystemUtils::sendMail($subject, html_entity_decode($message), $to, false, $from, $reply_to);

}  catch (Exception $ex) {
	$message = $ex->getMessage();
	if (is_array($message)) {
		$message = implode(PHP_EOL, $message);
	}

    if (GFConf::get('system.debugIncomingEmail')) {
        $mail->dumpRawEmail();
    }

	$subject = Language::getSessText('Gateway.ErrorSubject');
	$message = Language::getSessText('Gateway.ErrorResponse', array($ex->getMessage(), $body));

	SystemUtils::sendMail($subject, $message,$from);
}
