<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-07-20
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once 'plugins/tracker/lib/om/BaseTrackerExtraFieldElementTransition.php';
require_once("plugin/ActionManager.php");
require_once('lib/SystemUtils.php');
require_once('plugins/tracker/lib/TrackerTransitionScript.php');
require_once('plugins/tracker/lib/TrackerEfeTransitionEnabledScript.php');

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.tracker.lib 
 */
class TrackerExtraFieldElementTransition extends BaseTrackerExtraFieldElementTransition {
	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see TrackerExtraFieldElementTransition::setDeleted()
	 * @see TrackerExtraFieldElementTransition::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->getTrackerExtraFieldElementRelatedByElementId()->getTrackerExtraField()->getTracker()->userIsAdmin()) {
			throw new PermissionDeniedException();
		}
		ActionManager::notify("TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if (!$this->getTrackerExtraFieldElementRelatedByElementId()->getTrackerExtraField()->getTracker()->userIsAdmin()) {
			throw new PermissionDeniedException();
		}
		parent::save($con);
		ActionManager::notify("TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_SAVE", $this);
	}


	function notify($TrackerItem) {
		// Get all the email addresses that need notification
    $emails = array();
    $c = new Criteria();
    $c->add(TrackerExtraFieldElementTransitionNotifyPeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, 
      $this->getTrackerExtraFieldElementTransitionId());
    $notifys = TrackerExtraFieldElementTransitionNotifyPeer::doSelect($c);
    foreach ($notifys as $notify) {
      if ($notify->getSection() == 'role') {
        $c = new Criteria();
        $c->add(UserProjectRolePeer::ROLE_ID, $notify->getRefId());
        $uprs = UserProjectRolePeer::doSelect($c);
        foreach ($uprs as $upr) {
          $up = $upr->getUserProject();
          if (is_object($up)) {
            $u = $up->getUser();
            if (is_object($u)) {
              if ($u->getIsGroup()) {
                $c = new Criteria();
                $c->add(UsergroupUserPeer::USERGROUP_ID, $u->getUserId());
                $c->addJoin(UserPeer::USER_ID, UsergroupUserPeer::MEMBER_USER_ID);
                $members = UserPeer::doSelect($c);
                foreach ($members as $m) {
                  $emails[] = $m->getEmail();
                }
              } else {
                $emails[] = $u->getEmail();
              }
            }
          }
        }
      } else if ($notify->getSection() == 'members') {
        $c = new Criteria();
        $c->add(TrackerExtraFieldDataPEer::TRACKER_EXTRA_FIELD_ID, 
          $notify->getRefId());
        $c->add(TrackerExtraFieldDataPeer::TRACKER_ITEM_ID, $TrackerItem->getTrackerItemId());
        $d = TrackerExtraFieldDataPeer::doSelectOne($c);
        if (is_object($d)) {
          $u = UserPeer::retrieveByPk($d->getFieldData());
          $c = new Criteria();
          $c->add(UsergroupUserPeer::USERGROUP_ID, $u->getUserId());
          $c->addJoin(UserPeer::USER_ID, UsergroupUserPeer::MEMBER_USER_ID);
          $members = UserPeer::doSelect($c);
          foreach ($members as $m) {
            $emails[] = $m->getEmail();
          }
        }
      }
    }

    if (count($emails) == 0 || strlen($this->getNotificationMessage()) == 0) {
      // Don't send any mail if no recipient or no message specified
      return;
    }

		$subject='Tracker Item #'.$TrackerItem->getTrackerItemId().' Was Modified';
		$body=$this->getNotificationMessage();
		
		// URL for the link
		$url = GFUrl::getURL()->projectURL($TrackerItem->getTracker()->getProject(), 'TrackerItemEdit', 'tracker', '', array('tracker_item_id'=>$TrackerItem->getTrackerItemId()));
		$url = GFUrl::getFullUrl($url, true);
		
		// Since the email is not in HTML, we have to replace the "&amp;" in the URL for regular ampersands
		$url = str_replace('&amp;', '&', $url);
		
		$body .= "\n\nModify the item here:\n\n".$url;

 		SystemUtils::sendMail($subject,$body,$emails);
	}
	
	public function executeScripts() {
		// Get the scripts assigned to this workflow transition
		$c = new Criteria();
		$c->add(TrackerEfeTransitionEnabledScriptPeer::TRACKER_EXTRA_FIELD_ELEMENT_TRANSITION_ID, $this->getPrimaryKey());
		$c->addAscendingOrderByColumn(TrackerEfeTransitionEnabledScriptPeer::EXEC_ORDER);
		$c->addJoin(TrackerTransitionScriptPeer::TRACKER_TRANSITION_SCRIPT_ID, TrackerEfeTransitionEnabledScriptPeer::TRACKER_TRANSITION_SCRIPT_ID);
		$scripts = TrackerTransitionScriptPeer::doSelect($c);
		
		foreach ($scripts as $script) {
			$res = $script->executeScript($this);
		}
	}
}
?>
