<?php

require_once('notification/GFNotification.php');

/**
 *
 * @property Tracker $Tracker
 * @property User $user
 * @property TrackerItem $TrackerItem
 * @property TrackerItemMessage $TrackerItemMessage
 * @property Project $Project
 */

class TrackerItemMentionNotification implements GFNotification{

	const TRACKER_ITEM = 1;
	const TRACKER_ITEM_MESSAGE = 2;
	private $user;
	private $TrackerItem;
	private $TrackerItemMessage;
	private $type;
	private $sender;


	/**
	 * @param User $user
	 * @param TrackerItem | TrackerItemMessage $object
	 * @param User $sender
	 * @throws InvalidDataException
	 */
	function __construct($user, $object, $sender) {

		if (is_a($object, 'TrackerItem')) {
			$this->type = self::TRACKER_ITEM;
			$this->TrackerItem = $object;
			$this->TrackerItemMessage = null;
		} else if (is_a($object,'TrackerItemMessage')) {
			/** @var TrackerItemMessage $object */
			$this->type = self::TRACKER_ITEM_MESSAGE;
			$this->TrackerItem = $object->getTrackerItem();
			$this->TrackerItemMessage = $object;
		} else {
			throw new InvalidDataException("I expect either a TrackerItem or a TrackerItemMessage object");
		}

		$this->user = $user;
		$this->sender = $sender;

	}

	private function getGenericMessage() {

		$body = '';

		$ti = $this->TrackerItem;
		$tim = $this->TrackerItemMessage;

		$url = GFUrl::getUrl();

		$body .= $this->getGenericSubject();
		$body .= "\n\n";

		if ($this->type == self::TRACKER_ITEM) {
			$body .= StringUtils::getPlainUTF8FromHtml($ti->getDetails(false));
		} else {
			$body .= StringUtils::getPlainUTF8FromHtml($tim->getBody(false));
		}

		$body .= "\n";

		$body .=	"\n\n----------------------------------------------------------------------";
		$body .= "\n\n To respond simply reply this email or visit:\n".

				"\n".$url->getFullUrl($url->projectUrl($this->TrackerItem->getTracker()->getProject(), 'TrackerItemEdit',
				'tracker', '', array('tracker_id' => $this->TrackerItem->getTrackerId(),
					'tracker_item_id' => $this->TrackerItem->getPrimaryKey())), true).
			"\n\n";

		return $body;
	}

	public function getNotificationMessage(User $user) {
		return $this->getGenericMessage();
	}

	private function getGenericSubject() {
		$user = $this->sender;
		$userName = $user->getFirstName() . " " . $user->getLastName() . "({$user->getUnixName()})";
		$ti = $this->TrackerItem;

		return "User $userName mentioned you in Tracker Item [#{$ti->getId()}]: {$ti->getSummary()} \n";
	}

	public function getNotificationSubject(User $user) {
		return $this->getGenericSubject();
	}

	public function getUsers() {
		return array($this->user);
	}

	public function getSection() {
		return TrackerItem::SECTION;
	}

	public function getRefId() {
		return $this->TrackerItem->getPrimaryKey();
	}

	public function getMessageId() {
		if ($this->TrackerItemMessage) {
			return $this->TrackerItemMessage->getTrackerItemMessageId();
		} else {
			return $this->TrackerItem->getId();
		}
	}

	public function getReferences() {
		return null;
	}

	public function getInReplyTo() {
		return null;
	}

}
