<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugins/tracker/lib/TrackerItemQuery.php');
require_once('lib/exceptions/NotLoggedInException.php');
require_once('lib/GFSearch.php');

class TrackerItemQuickQuery extends TrackerItemQuery {
	
	private $show_tree;

	function __construct(Tracker $Tracker) {
		parent::__construct($Tracker);
		$this->setupItems();
	}

	private function setupItems() {
		$fields = array();
        $preference_name = 'quickquery'.$this->Tracker->getTrackerId();
		if (isset($_REQUEST['quickquery'])) {
			if (!Session::getSession()->isLoggedin()) {
				throw new NotLoggedInException();
			}
			if (isset($_REQUEST['tracker_item_id']) && intval($_REQUEST['tracker_item_id']) > 0) {
				$fields['tracker_item_id']=intval($_REQUEST['tracker_item_id']);
				//if the ID was given, ignore the other fields
			} else {
				$fields['priority']=intval($_REQUEST['priority']);
				$fields['tracker_sprint']=intval($_REQUEST['tracker_sprint']);
				$fields['assigned_to']=intval($_REQUEST['assigned_to']);
				$fields['waiting_for']=intval($_REQUEST['waiting_for']);
				$fields['summary']=Validator::sanitizeString(StringUtils::stripHtmlTags(addslashes($_REQUEST['summary'])));
				$fields['submitted_by']=Validator::sanitizeString(StringUtils::stripHtmlTags($_REQUEST['submitted_by']), true);
				if (isset($_REQUEST['extra_field'])) 
					$fields['extrafields']=$_REQUEST['extra_field'];
				//error_log("EXTRA FIELD: " .print_r($_REQUEST['extra_field'], true));
			}
			$fields['sortcol']=Validator::sanitizeString($_REQUEST['sortcol']);
			$fields['sortord']=Validator::sanitizeString($_REQUEST['sortord']);
            $usr = Session::getSession()->getUser();
            $usr->setPreference($preference_name,serialize($fields));
		} elseif (Session::getSession()->isLoggedin()) {
            //logged in user may have a saved preference
			$usr = Session::getSession()->getUser();
            $pref = $usr->getPreference($preference_name);
			if ($pref && strlen($pref) > 0) {
                try {
                    $fields = unserialize($pref);
                } catch (Exception $ex) {
                    error_log("Caught an exception while unserializing quick query pref for {$usr->getUnixName()} in tracker {$this->Tracker->getProject()->getUnixName()}/{$this->Tracker->getTrackerName()}: {$ex->getMessage()}");
                    $usr->deletePreference($preference_name);
                }
			}
		}
		
		// Logic to see whether we should display the query as a tree or as a flat listing. Show a flat
		// list if there is some filter set.
		$show_tree = true;
		$show_tree = $show_tree && (!isset($fields['tracker_item_id']) || $fields['tracker_item_id'] == 0);		// Priority not set
		$show_tree = $show_tree && (!isset($fields['priority']) || $fields['priority'] == 0);		// Priority not set
		$show_tree = $show_tree && (!isset($fields['assigned_to']) || $fields['assigned_to'] == 0);		// Priority not set
		$show_tree = $show_tree && (!isset($fields['summary']) || empty($fields['summary']));		// Priority not set
		$show_tree = $show_tree && (!isset($fields['submitted_by']) || empty($fields['submitted_by']));
		if (isset($fields['extrafields'])) {
			foreach ($fields['extrafields'] as $field_id => $value) {
				$show_tree = $show_tree && empty($value);
			}
		}
		
		$this->show_tree = $show_tree;
		
		if (isset($fields['tracker_item_id']) && $fields['tracker_item_id'] > 0) {
			$this->tracker_item_id=$fields['tracker_item_id'];
		}
		if (isset($fields['assigned_to'])) {
			$this->assignee=$fields['assigned_to'];
		}
		if (isset($fields['summary'])) {
			$this->summary=$fields['summary'];
		}
		if (isset($fields['waiting_for'])) {
			$this->waitingFor=$fields['waiting_for'];
		}
		if (isset($fields['submitted_by']) && !empty($fields['submitted_by'])) {
			$search = new GFSearch(array('words'=>$fields['submitted_by']));
			$this->submittedBy = $search->getWordsForQuery();
		}
		if (isset($fields['sortcol'])) {
			$this->sortcol=$fields['sortcol'];
		}
		if (isset($fields['sortord'])) {
			$this->sortord=$fields['sortord'];
		}
		if (isset($fields['extrafields'])) {
			$this->extra_fields=$fields['extrafields'];
		}
		if (isset($fields['priority'])) {
			$this->priority=$fields['priority'];
		}

    if (isset($fields['tracker_sprint'])) {
			$this->tracker_sprint=$fields['tracker_sprint'];
		}
	}
	
	public function showTree() {
		return $this->show_tree;
	}

}

?>
