<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2005-11-11
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
// The parent class
require_once 'plugins/tracker/lib/om/BaseTrackerQueryPeer.php';

// The object class
include_once 'plugins/tracker/lib/TrackerQuery.php';

/** 
 * The skeleton for this class was autogenerated by Propel on:
 *
 * [Wed Nov 16 20:42:33 2005]
 *
 *  You should add additional methods to this class to meet the
 *  application requirements.  This class will only be generated as
 *  long as it does not already exist in the output directory.
 *
 * @package plugins.tracker.lib 
 */
class TrackerQueryPeer extends BaseTrackerQueryPeer {
	private static $_pkCache = array();

	/**
	 * Retrieve a single object by pkey or NULL if not found.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key
	 * @param	bool $useCache	Whether to use caching. This will increase performance if the same object
	 *							is accessed several times
	 * @param	Connection $con	The connection to use
	 * @return	TrackerQuery
	 */
	public static function retrieveByPk($pk, $useCache = true, $con = null) {
		if ($useCache && array_key_exists($pk, TrackerQueryPeer::$_pkCache)) {
			return TrackerQueryPeer::$_pkCache[$pk];
		}
		
		$object = parent::retrieveByPk($pk, $con);
		if ($useCache) {
			TrackerQueryPeer::$_pkCache[$pk] = $object;
		}
		return $object;
	}
	
	/**
	 * Removes an object from the cache. This is usually done after saving.
	 * This method was added by propel_customizer.php.
	 *
	 * @param	mixed $pk	The primary key of the object to remove
	 */
	public static function invalidate($pk) {
		if (array_key_exists($pk, TrackerQueryPeer::$_pkCache)) {
			unset(TrackerQueryPeer::$_pkCache[$pk]);
		}
	}

        /**
         * Adds the standard set of basic queries to the specified tracker.
         * NOTE: No checking is done to see if this tracker already has any or all of these queries.  They are always added.
         * @param int $tracker_id Unique identifier for the Tracker to which queries will be added
         * @return boolean TRUE if all went well, FALSE otherwise (e.g., invalid ID provided).
         */
        public static function addStandardQueries($tracker_id, $user = null) {
            
            $con = Propel::getConnection();

            $con->begin();

            $tracker = TrackerPeer::retrieveByPk($tracker_id);
            
            if ($tracker == null) {
                error_log("addStandardQueries($tracker_id) does not exist, exit!");
                return FALSE;
            }
            
            if ($user == null) {
              $user = UserPeer::retrieveByPk(100);  // Nobody user
            }
            
            if ($user == null) {
                error_log("addStandardQueries($tracker_id) Nobody user does not exist, exit!");
                return FALSE;
            }
            
            // Recently Modified
            $query = new TrackerQuery();
            $query->setQueryName('Recently Modified');
            $query->setUser($user);
            $query->addTrackerQueryDisplayColumn(TrackerQueryDisplayColumn::create(977, 0));
            $query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_MODDATE, 0, '1 week ago'.TrackerItemQuery::DATE_RANGE_DELIMITER.'tomorrow'));
            $query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTCOL, 0, 'last_modified_date'));
            $query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTORD, 0, 'DESC'));
            if (self::saveQuery($query, $tracker_id, $con) == FALSE) return FALSE;
            
            // Do we have an extra field for status in this tracker?
            $statusField = null;
            $extraFields = $tracker->getTrackerExtraFields();
            foreach ($extraFields as $candidate) {
                if ($candidate->getFieldType() == TrackerExtraField::TYPE_STATUS) {
                    $statusField = $candidate;
                    break;
                }
            }

			// All of these standard queries depend on having a status field.
			// If this tracker doesn't have one for some reason, then we
			// won't add them.
            if ($statusField != null) {
	            // All Open
                $query = new TrackerQuery();
                $query->setQueryName('All Open');
                $query->setUser($user);
                $query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_EXTRAFIELD, $statusField->getTrackerExtraFieldId(), TrackerItemQuery::STATUS_ANY_OPEN));
                $query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTORD, 0, 'DESC'));
                $query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTCOL, 0, 'close_date'));
                if (self::saveQuery($query, $tracker_id, $con) == FALSE) return FALSE;

				// All Closed - same as open, but with different status ID criteria
                $query = new TrackerQuery();
                $query->setQueryName('All Closed');
                $query->setUser($user);
                $query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_EXTRAFIELD, $statusField->getTrackerExtraFieldId(), TrackerItemQuery::STATUS_ANY_CLOSED));
                $query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTORD, 0, 'DESC'));
                $query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTCOL, 0, 'close_date'));
                if (self::saveQuery($query, $tracker_id, $con) == FALSE) return FALSE;

				// My open items
				$query = new TrackerQuery();
				$query->setQueryName('Your Open Items');
				$query->setUser($user);
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_ASSIGNEE, 0, TrackerItemQuery::CURRENT_USER));
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_EXTRAFIELD, $statusField->getTrackerExtraFieldId(), TrackerItemQuery::STATUS_ANY_OPEN));
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTORD, 0, 'ASC'));
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTCOL, 0, 'priority'));
				if (self::saveQuery($query, $tracker_id, $con) == FALSE) return FALSE;

				// Unassigned
				$query = new TrackerQuery();
				$query->setQueryName('Unassigned');
				$query->setUser($user);
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_ASSIGNEE, 0, 100));  // 100 = Nobody special record
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_EXTRAFIELD, $statusField->getTrackerExtraFieldId(), TrackerItemQuery::STATUS_ANY_OPEN));
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTORD, 0, 'ASC'));
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTCOL, 0, 'priority'));
				if (self::saveQuery($query, $tracker_id, $con) == FALSE) return FALSE;

				// All assigned
				$query = new TrackerQuery();
				$query->setQueryName('Assigned Work');
				$query->setUser($user);
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_ASSIGNEE, 0, TrackerItemQuery::ANY_USER));  //Anyone EXCEPT Nobody
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_EXTRAFIELD, $statusField->getTrackerExtraFieldId(), TrackerItemQuery::STATUS_ANY_OPEN));
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTORD, 0, 'ASC'));
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTCOL, 0, 'priority'));
				if (self::saveQuery($query, $tracker_id, $con) == FALSE) return FALSE;

				// Stale Items
				$query = new TrackerQuery();
				$query->setQueryName('Stale Items');
				$query->setUser($user);
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_EXTRAFIELD, $statusField->getTrackerExtraFieldId(), TrackerItemQuery::STATUS_ANY_OPEN));
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_MODDATE, 0, '1970-01-01'.TrackerItemQuery::DATE_RANGE_DELIMITER.'30 days ago'));
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTORD, 0, 'ASC'));
				$query->addTrackerQueryField(TrackerQueryField::create(TrackerItemQuery::FIELD_SORTCOL, 0, 'priority'));
				if (self::saveQuery($query, $tracker_id, $con) == FALSE) return FALSE;
			}

            try {
                $con->commit();
            } catch (Exception $ex) {
                error_log("addStandardQueries($tracker_id) caught an exception: {$ex->getMessage()}");
                error_log($ex->getTraceAsString());
                $con->rollback();
                return FALSE;
            }
            
            return TRUE;
        }
        
        private static function saveQuery(TrackerQuery $query, $tracker_id, $con = null) {
            try {
                $query->setTrackerId($tracker_id);
                $query->setIsShared(true);
                $query->save($con);
                return TRUE;
            } catch (Exception $ex) {
                error_log("saveQuery({$query->getQueryName()}, $tracker_id) caught an exception: {$ex->getMessage()}");
                error_log($ex->getTraceAsString());
                return FALSE;
            }
        }
}
?>
