<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('plugins/tracker/lib/Tracker.php');
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('plugins/tracker/lib/TrackerExtraField.php');
require_once('plugins/tracker/lib/TrackerExtraFieldData.php');
require_once('lib/html/HTMLWidget.php');
require_once('plugins/frs/lib/FrsReleasePeer.php');

class TrackerExtraFieldRenderer {

	private $Tracker;
    private $intrinsic_fields;

	function __construct(Tracker $Tracker) {
		$this->Tracker = $Tracker;

        //Get intrinsic fields settings
        $this->intrinsic_fields = $this->Tracker->getIntrinsicFieldsSettings();
	}

	/**
	 * @param $criteria
	 * @param TrackerItemQuery TrackerItemQuery
	 * @return mixed
	 */
	function getQuerySelected($criteria, $TrackerItemQuery) {
		$fields = $this->Tracker->getTrackerExtraFields($criteria);
		$selected = $TrackerItemQuery->getExtraFields();
		$selected['PRIORITY'] = $TrackerItemQuery->getPriority();
		if (!$selected['PRIORITY']) {
			$selected['PRIORITY'] = array();
		}
		$selected['WAITING_FOR'] = $TrackerItemQuery->getWaitingFor();
		$selected['TRACKERSPRINT'] = $TrackerItemQuery->getTrackerSprint();
		if (!$selected['TRACKERSPRINT']) {
			$selected['TRACKERSPRINT'] = array();
		}
		$selected['ASSIGNEDTO'] = $TrackerItemQuery->getAssignedTo();
		if (!$selected['ASSIGNEDTO']) {
			$selected['ASSIGNEDTO'] = array();
		}
		foreach ($fields as $field) {
			if (!isset($selected[$field->getTrackerExtraFieldId()][0])) {
				$selected[$field->getTrackerExtraFieldId()][0] = '';
			}
		}
		return $selected;
	}

	function getDefaultSelected($criteria) {
		$fields = $this->Tracker->getTrackerExtraFields($criteria);
		$selected = array();
		foreach ($fields as $field) {
			//
			//	may not have an element selected for default
			//
			$selected[$field->getTrackerExtraFieldId()][] = '';
		}
		$selected['PRIORITY'] = '';
		$selected['TRACKERSPRINT'] = '';
		$selected['ASSIGNEDTO'] = array(0);
		return $selected;
	}

	function getTrackerItemSelected(TrackerItem $TrackerItem, Criteria $criteria) {
		$TrackerExtraFieldData = $TrackerItem->getTrackerExtraFieldDatas();
		$selected = array();
		foreach ($TrackerExtraFieldData as $tefd) {
			$selected[$tefd->getTrackerExtraFieldId()][] = $tefd->getFieldData();
		}
		$fields = $this->Tracker->getTrackerExtraFields($criteria);
		foreach ($fields as $field) {
			//
			//	may not have an element selected for default
			//
	    if (!isset($selected[$field->getTrackerExtraFieldId()][0])) {
				switch ($field->getFieldType()) {
					case TrackerExtraField::TYPE_TEXTAREA:
					case TrackerExtraField::TYPE_DATE:
					case TrackerExtraField::TYPE_NUMERIC:
					case TrackerExtraField::TYPE_SERIAL:
					case TrackerExtraField::TYPE_TEXT: {
							$selected[$field->getTrackerExtraFieldId()][] = '';
							break;
						}
					default: {
							$selected[$field->getTrackerExtraFieldId()][] = 100;
						}
				}
			}
		}
		$selected['PRIORITY'] = $TrackerItem->getPriority();
		$assignees = $TrackerItem->getTrackerItemAssignees();
		$assign = array();
		foreach ($assignees as $assignee) {
			$assign[] = $assignee->getAssignee();
		}
		$selected['ASSIGNEDTO'] = $assign;
		$selected['TRACKERSPRINT'] = $TrackerItem->getSprintId();
		return $selected;
	}

	function renderFieldsReadOnly($TrackerItem) {
		$criteria = new Criteria();
		$selected = $this->getTrackerItemSelected($TrackerItem, $criteria);
		$template = $this->generateTemplate($criteria);
		$fields = $this->Tracker->getTrackerExtraFields($criteria);

		//assignee
		$crit = new Criteria();
		$crit->add(UserPeer::USER_ID, $selected['ASSIGNEDTO'], Criteria::IN);
		$assignees = UserPeer::doSelect($crit);
		$str = '';
		foreach ($assignees as $assignee) {
			$str .= $assignee->getUserLink() . '<br />';
		}
		$template = str_replace('<!--EFASSIGNEDTO-->', $str, $template);

		//priority
		$priorities = $this->Tracker->getPriorities();
		$str = $priorities[$selected['PRIORITY']];
		$template = str_replace('<!--EFPRIORITY-->', $str, $template);

		//
		//	Custom fields
		//
	foreach ($fields as $field) {
			$sel = $selected[$field->getTrackerExtraFieldId()];
			switch ($field->getFieldType()) {
				case TrackerExtraField::TYPE_SELECT:
				case TrackerExtraField::TYPE_RADIO:
				case TrackerExtraField::TYPE_STATUS:

				// Don't need to check permissions and show/hide here
				// It's read-only, doesn't show additional groups,
				// and the current user can only see the TI if it's in their group already.
				case TrackerExtraField::TYPE_MEMBERS:
				case TrackerExtraField::TYPE_RELEASE: {
//MONO HARD CODED TIE TO FRS
						if (isset($selected[$field->getTrackerExtraFieldId()])) {
							if ($field->getFieldType() == TrackerExtraField::TYPE_RELEASE) {
								$element = FrsReleasePeer::retrieveByPk($selected[$field->getTrackerExtraFieldId()][0]);
								if (is_object($element)) {
									$str = $element->getReleaseName();
								} else {
									$str = 'N/A';
								}
							} else if ($field->getFieldType() == TrackerExtraField::TYPE_MEMBERS) {
								$element = UserPeer::retrieveByPk($selected[$field->getTrackerExtraFieldId()][0]);
								if (is_object($element)) {
									$str = $element->getUserLink() . '<br />';
								} else {
									$str = 'N/A';
								}
							} else {
								$element = TrackerExtraFieldElementPeer::retrieveByPk($selected[$field->getTrackerExtraFieldId()][0]);
								if (is_object($element)) {
									$str = $element->getElementName();
								} else {
									$str = 'N/A';
								}
							}
						} else {
							$str = 'N/A';
						}
						break;
					}
				case TrackerExtraField::TYPE_CHECKBOX:
				case TrackerExtraField::TYPE_MULTISELECT: {

						$crit = new Criteria();
						$crit->add(TrackerExtraFieldElementPeer::ELEMENT_ID, $selected[$field->getTrackerExtraFieldId()], Criteria::IN);
						$elements = TrackerExtraFieldElementPeer::doSelect($crit);
						$str = '';
						foreach ($elements as $element) {
							$str .= $element->getElementName() . '<br />';
						}
						break;
					}
				case TrackerExtraField::TYPE_TEXT:
				case TrackerExtraField::TYPE_DATE:
				case TrackerExtraField::TYPE_NUMERIC:
				case TrackerExtraField::TYPE_SERIAL:
				case TrackerExtraField::TYPE_TEXTAREA: {

						$str = $selected[$field->getTrackerExtraFieldId()][0];

						break;
					}
			}//end switch
			$template = str_replace('<!--EF' . $field->getTrackerExtraFieldId() . '-->', $str, $template);
			unset($str);
			unset($sel);
		}
		return $template;
	}

	function renderFields($mode, $TrackerItem = null, $TrackerItemQuery = null, $user_selected = array()) {
                switch ($mode) {
			case 'QUERY': {
					$criteria = new Criteria();
					$criteria->add(TrackerExtraFieldPeer::FIELD_TYPE, explode(',', TrackerExtraField::QUERY_TYPES), Criteria::IN);
					$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
					$defaultSelected = $this->getQuerySelected($criteria, $TrackerItemQuery);
					break;
				}
			case 'SUBMIT': {
					$criteria = new Criteria();
					$criteria->add(TrackerExtraFieldPeer::SHOW_ON_SUBMIT, 'true');
					$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
					$defaultSelected = $this->Tracker->getTrackerDefaultSelected($criteria);
					break;
				}
			case 'EDIT': {
					$criteria = new Criteria();
					$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
					$defaultSelected = $this->getTrackerItemSelected($TrackerItem, $criteria);
					break;
				}
			case 'MASSUPDATE': {
					$criteria = new Criteria();
					$criteria->add(TrackerExtraFieldPeer::FIELD_TYPE, explode(',', TrackerExtraField::INT_TYPES), Criteria::IN);
					$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
					$defaultSelected = $this->getDefaultSelected($criteria);
					break;
				}
			case 'QUICKBROWSE': {
					$criteria = new Criteria();
					$criteria->add(TrackerExtraFieldPeer::USE_IN_QUICK_BROWSE, 'true');
					$criteria->add(TrackerExtraFieldPeer::FIELD_TYPE, explode(',', TrackerExtraField::INT_TYPES), Criteria::IN);
					$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
					$defaultSelected = $this->getQuerySelected($criteria, $TrackerItemQuery);
					break;
				}
		}

		// Merge the selected values with the selected by default
		// Note that $user_selected comes from $_POST so it has a very different format than
		// $defaultSelected
		$selected = $defaultSelected;

		if (isset($user_selected['priority'])) {
			$selected['PRIORITY'] = $user_selected['priority'];
		}
		if (isset($user_selected['assigned_to'])) {
			$selected['ASSIGNEDTO'] = $user_selected['assigned_to'];
		}

        if (isset($user_selected['tracker_sprint'])) {
			$selected['TRACKERSPRINT'] = $user_selected['tracker_sprint'];
		}

		if (isset($user_selected['waiting_for'])) {
			$selected['WAITING_FOR'] = $user_selected['waiting_for'];
		}

		if (array_key_exists('extra_field', $user_selected) && is_array($user_selected['extra_field'])) {
			// Extra field data is set
			foreach ($user_selected['extra_field'] as $extra_field_id => $value) {
				if (is_array($value)) {
					// for multiselect fields
					$selected[$extra_field_id] = $value;
				} else {
					$selected[$extra_field_id][0] = $value;
				}
			}
		}

		$template = $this->generateTemplate($criteria, $mode);

		//kind of getting hacky, but we want to re-use the template generator...
		if ($mode == 'QUICKBROWSE') {
			//strip the <tr> tags - everything should be on one row
			$template = str_replace('<tr>', '', $template);
			$template = str_replace('</tr>', '', $template);
			$template = preg_replace('/<td.*?>/', '<li class="quickBrowse">', $template);
			$template = str_replace('</td>', '</li>', $template);
		}
		$fields = $this->Tracker->getTrackerExtraFields($criteria);


		//
		//	Assignee
		//
		switch ($mode) {
			case 'QUICKBROWSE':
				$assignees = $this->getHistoricalAssignees(false, true); // include special "any user" selection but not "current user"
				$str = HTMLWidget::select(
						'assigned_to',
						$assignees['assignees'],
						$selected['ASSIGNEDTO'],
						HTMLWidget::OPTION_ANY,
						true, Language::getSessText('Tracker.Nobody'),
						true, Language::getSessText('Options.Any'),
						$assignees['names']
				);
				break;

			case 'MASSUPDATE':
				$techs = $this->Tracker->getTechnicians();
				$str = HTMLWidget::select(
						'assigned_to',
						$techs,
						'',
						HTMLWidget::OPTION_ANY,
						! $this->isUserInList($techs, HTMLWidget::OPTION_NONE), Language::getSessText('Tracker.Nobody'),
						true, Language::getSessText('Tracker.NoChange')
				);
				break;

			case 'SUBMIT':
				if ($this->Tracker->userIsAdmin()) {
					$techs = $this->Tracker->getTechnicians();
					$str = HTMLWidget::multiselect(
							'assigned_to[]',
							$techs,
							$selected['ASSIGNEDTO'],
							HTMLWidget::OPTION_NONE,
							! $this->isUserInList($techs, HTMLWidget::OPTION_NONE), Language::getSessText('Tracker.Nobody')
					);
				} else {
					$str = '(auto)';
				}
				break;

			case 'EDIT':
				if ($this->Tracker->userIsAdmin()) {
					$techs = $this->Tracker->getTechnicians();
					$str = HTMLWidget::multiselect(
							'assigned_to[]',
							$techs,
							$selected['ASSIGNEDTO'],
							HTMLWidget::OPTION_NONE,
							true, Language::getSessText('Tracker.Nobody')
					);
				} else {
					$technicians_assigned = $TrackerItem->getAssignedUsers();
					$str = HTMLWidget::multiselect(
							'assigned_disabled',
							$technicians_assigned,
							$selected['ASSIGNEDTO'],
							HTMLWidget::OPTION_NONE,
							false, '',
							false, '',
							10,
							true
					);
				}
				break;

			case 'QUERY':
				$assignees = $this->getHistoricalAssignees(true, true);  // include special "current user" and "any user" selections
				$str = HTMLWidget::multiselect(
						'assigned_to[]',
						$assignees['assignees'],
						$selected['ASSIGNEDTO'],
						HTMLWidget::OPTION_NONE,
						true, Language::getSessText('Tracker.Nobody'),
						false, null,
						10,
						false, $assignees['names']
				);
				break;

			default:
				$techs = $this->Tracker->getTechnicians();
				$str = HTMLWidget::multiselect(
						'assigned_to[]',
						$techs,
						$selected['ASSIGNEDTO'],
						HTMLWidget::OPTION_NONE,
						true, Language::getSessText('Tracker.Nobody'),
						false, null,
						5
				);
				break;
		}

        $session = Session::getSession();
        $project = GFUrl::getUrl()->getProject();
        //If it's a TI submission and the user has tech level perms, then render the assignees field
        if ($mode != 'SUBMIT' || ($session->isLoggedIn() && $project->getPermission()->check(Tracker::SECTION, $this->Tracker->getTrackerId(), Tracker::PERMLEVEL_TECH_ADMIN))) {
            $template = str_replace('<!--EFASSIGNEDTO-->', $str, $template);
        }
		//
		//	priority
		//
        if ($this->intrinsic_fields[Tracker::TRACKER_FIELD_PRIORITY]['is_enabled']) {
            switch ($mode) {
                case 'QUERY':
                    $str = HTMLWidget::multiSelectFromAssoc('priority[]', $this->Tracker->getPriorities(), $selected['PRIORITY'], 100, false);
                    break;

                case 'QUICKBROWSE':
                    $str = HTMLWidget::selectBoxFromAssoc($this->Tracker->getPriorities(), 'priority', $selected['PRIORITY'], false, false, null, true, null);
                    break;

                case 'MASSUPDATE':
                    $str = HTMLWidget::selectBoxFromAssoc($this->Tracker->getPriorities(), 'priority', HTMLWidget::OPTION_ANY, false, false, null, true, 'No Change');
                    break;

	            case 'EDIT':
		            $str = HTMLWidget::selectBoxFromAssoc($this->Tracker->getPriorities(), 'priority', $selected['PRIORITY'], false, false);
		            break;

                default:
	                if ($this->intrinsic_fields[Tracker::TRACKER_FIELD_PRIORITY]['on_submit']) {
                        $str = HTMLWidget::selectBoxFromAssoc($this->Tracker->getPriorities(), 'priority', $selected['PRIORITY'], false, false);
	                } else {
		                $str = '';
	                }
                    break;
            }
            $template = str_replace('<!--EFPRIORITY-->', $str, $template);
        }

		//Waiting for
		if ($this->Tracker->getDatatype() == Tracker::DATATYPE_SUPPORT && $this->intrinsic_fields[Tracker::TRACKER_FIELD_WAITING_FOR]['is_enabled']) {
			switch($mode) {
				case 'QUERY':
				case 'QUICKBROWSE':
					$waiting_for_values = array(
						TrackerItem::WAITINGFOR_ASSIGNEE=>$session->getLanguage()->getText('TrackerItem.WaitingFor.Assignee'),
						TrackerItem::WAITINGFOR_SUBMITTER=>'Waiting for customer');
					$str = HTMLWidget::selectBoxFromAssoc($waiting_for_values,'waiting_for', $selected['WAITING_FOR'],
					false, true, $text_100 = null, $show_any = true);
					break;
				default:
					$str ='';
			}
			$template = str_replace('<!--EFWAITINGFOR-->',$str, $template);
		}

		//
		//	sprint
		//
		if ($this->intrinsic_fields[Tracker::TRACKER_FIELD_SPRINT_ID]['is_enabled']) {
			$str = '';
                        $skip_closed = TrackerSprintPeer::userIsAdmin($this->Tracker->getProject()->getId()) ? false : true;
			switch ($mode) {
				case 'QUICKBROWSE':
					if ($this->intrinsic_fields[Tracker::TRACKER_FIELD_SPRINT_ID]['on_browse']) {
						$str = HTMLWidget::selectBoxFromAssoc(
							$this->Tracker->getProject()->getSprintList(false),  // false = include closed sprints
							"tracker_sprint",
							$selected['TRACKERSPRINT'],
							false, true, null, true, 'Any'
						);
					}
					break;
				case 'QUERY':
					$str = HTMLWidget::multiSelectFromAssoc(
						"tracker_sprint[]",
						$this->Tracker->getProject()->getSprintList(false),  // false = include closed sprints
						$selected['TRACKERSPRINT'],
						false, true, null, true, 'Any'
					);
					break;
				case 'MASSUPDATE':
					$str = HTMLWidget::selectBoxFromAssoc(
						$this->Tracker->getProject()->getSprintList($skip_closed),
						"tracker_sprint", HTMLWidget::OPTION_ANY, false, true, null, true, 'No Change');
					break;
				case 'SUBMIT':
					$str = HTMLWidget::selectBoxFromAssoc(
						$this->Tracker->getProject()->getSprintList($skip_closed),
						"tracker_sprint", $selected['TRACKERSPRINT'], false, true, null, false, null);
				case 'EDIT':
					if ($this->intrinsic_fields[Tracker::TRACKER_FIELD_SPRINT_ID]['on_submit']) {
						$str = HTMLWidget::selectBoxFromAssoc(
							$this->Tracker->getProject()->getSprintList($skip_closed), 'tracker_sprint',
							$selected['TRACKERSPRINT']);
					}
					break;
			}
			$template = str_replace('<!--EFSPRINT-->', $str, $template);
		}


		//
		//	Custom fields
		//
		foreach ($fields as $field) {
			$str = '';
			switch ($mode) {
				case 'QUERY':
					//queries use multi-select boxes for everything EXCEPT date and numeric
					if ($field->getFieldType() == TrackerExtraField::TYPE_STATUS) {
						$show_none = false;
					} else {
						$show_none = true;
					}
					switch ($field->getFieldType()) {
						//date. numeric and serial fields have a 'range'
						case TrackerExtraField::TYPE_DATE: {
								$datevalues = explode(TrackerItemQuery::DATE_RANGE_DELIMITER, $selected[$field->getTrackerExtraFieldId()][0]);
								$selected[$field->getTrackerExtraFieldId()] = $datevalues;
								$str = '<input type="text" id="extra_field' . $field->getTrackerExtraFieldId() . '_0" name="extra_field[' . $field->getTrackerExtraFieldId() . '][0]" value="' . (isset($selected[$field->getTrackerExtraFieldId()][0]) ? $selected[$field->getTrackerExtraFieldId()][0] : '') . '" size="20" maxlength="50"/><a href="javascript:NewCal(\'extra_field' . $field->getTrackerExtraFieldId() . '_0\',\'MMMddyyyy\')"><img src="/themes/gforge5/images/cal.gif" alt="Pick a date" border="0" height="16" width="16"></a><a href="javascript:TestCal(\'extra_field' . $field->getTrackerExtraFieldId() . '_0\')" title="Test this date"><img src="/themes/gforge5/images/checkbox_checked.png" width="16" height="16" border="0" alt="Test this date"/></a>&nbsp;<input type="text" id="extra_field' . $field->getTrackerExtraFieldId() . '_1" name="extra_field[' . $field->getTrackerExtraFieldId() . '][1]" value="' . (isset($selected[$field->getTrackerExtraFieldId()][1]) ? $selected[$field->getTrackerExtraFieldId()][1] : '') . '" size="11" maxlength="11"/><a href="javascript:NewCal(\'extra_field' . $field->getTrackerExtraFieldId() . '_1\',\'MMMddyyyy\')"><img src="/themes/gforge5/images/cal.gif" alt="Pick a date" border="0" height="16" width="16"></a><a href="javascript:TestCal(\'extra_field' . $field->getTrackerExtraFieldId() . '_1\')" title="Test this date"><img src="/themes/gforge5/images/checkbox_checked.png" width="16" height="16" border="0" alt="Test this date"/></a>';
								break;
							}
						case TrackerExtraField::TYPE_SERIAL: {
								$str = '<input type="text" name="extra_field[' . $field->getTrackerExtraFieldId() . '][0]" value="' . (isset($selected[$field->getTrackerExtraFieldId()][0]) ? $selected[$field->getTrackerExtraFieldId()][0] : '') . '" size="5"/>&nbsp;<input type="text" name="extra_field[' . $field->getTrackerExtraFieldId() . '][1]" value="' . (isset($selected[$field->getTrackerExtraFieldId()][1]) ? $selected[$field->getTrackerExtraFieldId()][1] : '') . '" size="5" />';
								break;
							}
						case TrackerExtraField::TYPE_NUMERIC: {
								$str = '<input type="text" name="extra_field[' . $field->getTrackerExtraFieldId() . '][0]" value="' . (isset($selected[$field->getTrackerExtraFieldId()][0]) ? $selected[$field->getTrackerExtraFieldId()][0] : '') . '" size="' . $field->getAttribute1() . '" maxlength="' . $field->getAttribute2() . '"/>&nbsp;<input type="text" name="extra_field[' . $field->getTrackerExtraFieldId() . '][1]" value="' . (isset($selected[$field->getTrackerExtraFieldId()][1]) ? $selected[$field->getTrackerExtraFieldId()][1] : '') . '" size="' . $field->getAttribute1() . '" maxlength="' . $field->getAttribute2() . '"/>';
								break;
							}
						case TrackerExtraField::TYPE_STATUS: {
								$elements = $this->getStatusElements($field, TRUE);
								$str = HTMLWidget::multiselect('extra_field[' . $field->getTrackerExtraFieldId() . '][]', $elements, $selected[$field->getTrackerExtraFieldId()], null, $show_none);
								break;
							}
						case TrackerExtraField::TYPE_MEMBERS: {
							// We hide member fields for folks with only the members permission
							if ($this->userHasMemberPermission()) {
								continue;
							}
							// Otherwise, continue through to default and render the field
						}
						default: {
								$elements = $field->getTrackerExtraFieldElements();
								$str = HTMLWidget::multiselect('extra_field[' . $field->getTrackerExtraFieldId() . '][]', $elements, $selected[$field->getTrackerExtraFieldId()], null, $show_none);
								break;
							}
					}
					break;

				case 'QUICKBROWSE':
					if ($field->getFieldType() == TrackerExtraField::TYPE_MEMBERS) {
						// We hide member fields for folks with only the members permission
						if ($this->userHasMemberPermission()) {
							continue;
						}
						// Otherwise, continue through to default and render the field
					}
					//mass update and browse always use select boxes
					if ($field->getFieldType() == TrackerExtraField::TYPE_STATUS) {
						$elements = $this->getStatusElements($field, TRUE);
						$show_none = false;
          } else {
            $elements = $field->getTrackerExtraFieldElements();
						$show_none = true;
          }
					$str = HTMLWidget::select('extra_field[' . $field->getTrackerExtraFieldId() . ']', $elements, $selected[$field->getTrackerExtraFieldId()], HTMLWidget::OPTION_ANY, $show_none, null, true, null);
					break;

				case 'MASSUPDATE':
					if ($field->getFieldType() == TrackerExtraField::TYPE_MEMBERS) {
						// We hide member fields for folks with only the members permission
						if ($this->userHasMemberPermission()) {
							continue;
						}
						// Otherwise, continue through to default and render the field
					}
					if ($field->getFieldType() == TrackerExtraField::TYPE_STATUS) {
						$show_none = false;
					} else {
						$show_none = true;
					}
					$str = HTMLWidget::select('extra_field[' . $field->getTrackerExtraFieldId() . ']', $field->getTrackerExtraFieldElements(), '', HTMLWidget::OPTION_ANY, $show_none, null, true, 'No Change');
					break;

				default:
					switch ($field->getFieldType()) {
						case TrackerExtraField::TYPE_SELECT:
							switch ($mode) {
								case 'EDIT':
									// Only show the available items defined by the workflow
									// Note that we use $defaultSelected instead of $selected. This must be done
									// in case the user selects the "none" value.
									$selected_value = $defaultSelected[$field->getTrackerExtraFieldId()][0];

									// Bring in the proposed new value if it's set
									if (array_key_exists($field->getTrackerExtraFieldId(), $selected) &&
										count($selected[$field->getTrackerExtraFieldId()]) > 0) {
										$proposed_value = $selected[$field->getTrackerExtraFieldId()][0];
									}

									// Get the list of valid settings for this field, based on any existing workflow state.
									// This returns the whole list if the field is not workflow-enabled.
									$available_values = $field->getTrackerExtraFieldElementsWorkflow($selected_value);

									// Can we accept the new proposed value, based on workflow-allowed values?
									if (in_array($proposed_value, array_map( function($i) { return $i->getElementId(); }, $available_values))) {
										$selected_value = $proposed_value;
									}

									// Show the "none" option if this isn't a workflow field,
									// OR if it is workflow, only if it's currently selected.
									if ($field->getUseWorkflow() &&
										$selected_value != TrackerExtraFieldElement::ELEMENT_NONE) {
										$show_none = FALSE;
									} else {
										$show_none = TRUE;
									}

									$str = HTMLWidget::select('extra_field[' . $field->getTrackerExtraFieldId() . ']', $available_values, $selected_value, null, $show_none);
									break;

								case 'SUBMIT':
									// Show the "none" value only if the field doesn't use workflow
									$show_none = !$field->getUseWorkflow();
									$str = HTMLWidget::select('extra_field[' . $field->getTrackerExtraFieldId() . ']', $field->getTrackerExtraFieldElements(), $selected[$field->getTrackerExtraFieldId()][0], null, $show_none);
									break;

								default:
									$str = HTMLWidget::select('extra_field[' . $field->getTrackerExtraFieldId() . ']', $field->getTrackerExtraFieldElements(), $selected[$field->getTrackerExtraFieldId()][0]);
									break;
							}
						case TrackerExtraField::TYPE_SELECT:
							break;

						case TrackerExtraField::TYPE_RELEASE:
							break;

						case TrackerExtraField::TYPE_CHECKBOX:
							break;

						case TrackerExtraField::TYPE_RADIO:
							break;
					}
					break;
			}


			if ($mode == 'QUERY') {
				
			} elseif ($mode == 'MASSUPDATE') {

			} elseif ($mode == 'QUICKBROWSE') {
				
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_SELECT) {
				if ($mode == 'EDIT') {
					
				} elseif ($mode == 'SUBMIT') {

				} else {
					
				}
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_RELEASE) {
				$str = HTMLWidget::select('extra_field[' . $field->getTrackerExtraFieldId() . ']', $field->getTrackerExtraFieldElements(), $selected[$field->getTrackerExtraFieldId()][0]);
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_CHECKBOX) {
				$str = HTMLWidget::checkbox('extra_field[' . $field->getTrackerExtraFieldId() . '][]', $field->getTrackerExtraFieldElements(), $selected[$field->getTrackerExtraFieldId()]);
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_RADIO) {
				if ($mode == 'EDIT') {
					// Only show the available items defined by the workflow
					$selected_value = $defaultSelected[$field->getTrackerExtraFieldId()][0];
					$available_values = $field->getTrackerExtraFieldElementsWorkflow($selected_value);

					if ($field->getUseWorkflow()) {
						// Show the "none" value only if it is the current value
						$show_none = ($selected_value == TrackerExtraFieldElement::ELEMENT_NONE);
					} else {
						$show_none = true;
					}

					$str = HTMLWidget::radioSet('extra_field[' . $field->getTrackerExtraFieldId() . ']', $available_values, $selected_value, null, $show_none);
				} elseif ($mode == 'SUBMIT') {
					// Show the "none" value only if the field doesn't use workflow
					$show_none = !$field->getUseWorkflow();
					$str = HTMLWidget::radioSet('extra_field[' . $field->getTrackerExtraFieldId() . ']', $field->getTrackerExtraFieldElements(), $selected[$field->getTrackerExtraFieldId()][0], null, $show_none);
				} else {
					$str = HTMLWidget::radioSet('extra_field[' . $field->getTrackerExtraFieldId() . ']', $field->getTrackerExtraFieldElements(), $selected[$field->getTrackerExtraFieldId()][0]);
				}
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_MULTISELECT
					|| $field->getFieldType() == TrackerExtraField::TYPE_MEMBERS) {
				if ($field->getFieldType() == TrackerExtraField::TYPE_MEMBERS) {
					// We hide member fields for folks with only the members permission
					if ($this->userHasMemberPermission()) {
						continue;
					}
					// Otherwise, continue through to default and render the field
				}
				$str = HTMLWidget::multiselect('extra_field[' . $field->getTrackerExtraFieldId() . '][]', $field->getTrackerExtraFieldElements(), $selected[$field->getTrackerExtraFieldId()]);
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_TEXT) {
				$str = '<input type="text" name="extra_field[' . $field->getTrackerExtraFieldId() . ']" value="' . $selected[$field->getTrackerExtraFieldId()][0] . '" size="' . $field->getAttribute1() . '" maxlength="' . $field->getAttribute2() . '"/>';
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_TEXTAREA) {
				ob_start();
				$CKEditor = new GFCKEditor();
				$CKEditor->config['width']= $field->getAttribute2()*10;
				$CKEditor->textareaAttributes = array('style'=>'overflow:hidden;','rows'=>$field->getAttribute1(), 'id'=>'extra_field['.$field->getTrackerExtraFieldId().']', 'name'=>'extra_field['.$field->getTrackerExtraFieldId().']','onkeyup'=>'checkRow(this)');
				$CKEditor->editor('extra_field['.$field->getTrackerExtraFieldId().']',  html_entity_decode ($selected[$field->getTrackerExtraFieldId()][0]));
				$str = ob_get_contents();
				ob_end_clean();
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_STATUS) {

				if ($mode == 'EDIT') {

					// Only show the available items defined by the workflow
					// Note that we use $defaultSelected instead of $selected. This must be done
					// in case the user selects the "none" value.
					$selected_value = $defaultSelected[$field->getTrackerExtraFieldId()][0];

					// Bring in the proposed new value if it's set
					if (array_key_exists($field->getTrackerExtraFieldId(), $selected) &&
						count($selected[$field->getTrackerExtraFieldId()]) > 0) {
						$proposed_value = $selected[$field->getTrackerExtraFieldId()][0];
					}

					$available_values = $field->getTrackerExtraFieldElementsWorkflow($selected_value);

					// Can we accept the new proposed value, based on workflow-allowed values?
					if (in_array($proposed_value, array_map( function($i) { return $i->getElementId(); }, $available_values))) {
						$selected_value = $proposed_value;
					}

					$str = HTMLWidget::select('extra_field[' . $field->getTrackerExtraFieldId() . ']', $available_values, $selected_value, null, false);
				} else {
					$str = HTMLWidget::select('extra_field[' . $field->getTrackerExtraFieldId() . ']', $field->getTrackerExtraFieldElements(), $selected[$field->getTrackerExtraFieldId()][0], null, false);
				}
				//$str = HTMLWidget::select('extra_field['.$field->getTrackerExtraFieldId().']',$field->getTrackerExtraFieldElements(),$selected[$field->getTrackerExtraFieldId()][0],null,false);
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_DATE) {

				$str = '<input type="text" id="extra_field' . $field->getTrackerExtraFieldId() . '" name="extra_field[' . $field->getTrackerExtraFieldId() . ']" value="' . $selected[$field->getTrackerExtraFieldId()][0] . '" size="11" maxlength="11"/><a href="javascript:NewCal(\'extra_field' . $field->getTrackerExtraFieldId() . '\',\'MMMddyyyy\')"><img src="/themes/gforge5/images/cal.gif" alt="Pick a date" border="0" height="16" width="16"></a>';
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_NUMERIC) {
				//size=attr
				//maxlength=attr2
				$field_size = ($field->getAttribute1() > 0) ? $field->getAttribute1() : 5;
				$field_maxlength = ($field->getAttribute2() > 0) ? $field->getAttribute2() : 5;
				$str = '<input type="text" name="extra_field[' . $field->getTrackerExtraFieldId() . ']" value="' . $selected[$field->getTrackerExtraFieldId()][0] . '" size="' . $field_size . '" maxlength="' . $field_maxlength . '"/>';
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_MEMBERS) {
				// We hide member fields for folks with only the members permission
				if ($this->userHasMemberPermission()) {
					continue;
				}
				// Otherwise, continue through to default and render the field
				$str = HTMLWidget::multiselect('extra_field[' . $field->getTrackerExtraFieldId() . '][]', $field->getTrackerExtraFieldElements(), $selected[$field->getTrackerExtraFieldId()]);
			} elseif ($field->getFieldType() == TrackerExtraField::TYPE_SERIAL) {
				// The serial field shows up as read-only
				$str = $selected[$field->getTrackerExtraFieldId()][0];
			}
			$template = str_replace('<!--EF' . $field->getTrackerExtraFieldId() . '-->', $str, $template);
		}
		return $template;
	}

	function generateTemplate($criteria = null, $mode = false) {
		$Lang = Session::getSession()->getLanguage();

		$return = "<!-- Start Extra Fields Rendering -->\n<tr>";

		if ($this->intrinsic_fields[Tracker::TRACKER_FIELD_PRIORITY]['is_enabled']) {
			if (($mode == 'SUBMIT' && $this->intrinsic_fields[Tracker::TRACKER_FIELD_PRIORITY]['on_submit']) || $mode != 'SUBMIT') {
				$return .= '<td width="50%" valign="top"><strong>' . $Lang->getText('TrackerItem.PriorityName') . ':</strong><br /><!--EFPRIORITY--></td>';
			} else {
				$return .= '';
			}
		}

		// sprint
		if ($this->intrinsic_fields[Tracker::TRACKER_FIELD_SPRINT_ID]['is_enabled']) {
			if ( ($mode == 'EDIT' && $this->intrinsic_fields[Tracker::TRACKER_FIELD_SPRINT_ID]['on_submit']) ||
				($mode == 'QUICKBROWSE' && $this->intrinsic_fields[Tracker::TRACKER_FIELD_SPRINT_ID]['on_browse']) ||
				$mode != 'EDIT' && $mode != 'QUICKBROWSE') {
				$return .= '<td width="50%" valign="top"><strong>' . $Lang->getText('TrackerSprint.TrackerSprintName') . ':</strong><br /><!--EFSPRINT--></td>';
			}
		}

		$return .= "</tr>\n<tr>";

        $session = Session::getSession();
		$project = GFUrl::getUrl()->getProject();
		//If it's a TI submission and the user has tech level perms, then render the assignees field
		if ($mode != 'SUBMIT' || ($session->isLoggedIn() && $project->getPermission()->check(Tracker::SECTION, $this->Tracker->getTrackerId(), Tracker::PERMLEVEL_TECH_ADMIN))) {
			$return .= '<td width="50%" valign="top"><strong>' . $Lang->getText('TrackerItem.AssignedTo') . ':</strong><br /><!--EFASSIGNEDTO--></td>';
		}

		$criteria->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
		$fields = $this->Tracker->getTrackerExtraFields($criteria);
		$count = count($fields);
		// bumped to one because sprint reflows the fields
		$col_count = 1;
		$k = 0;

		foreach ($fields as $field) {
			if ($mode == 'SUBMIT' || $mode == 'EDIT') {
				if ($mode == 'EDIT' && $field->getUseWorkflow()) {
					// Field is required because it has workflow rules
					$req = '<span class="workflow_field">*</span>';
				} elseif ($field->getIsRequired()) {
					$req = '<span class="required">*</span>';
				} else {
					$req = '';
				}
			} else {
				$req = '';
			}

			switch ($field->getFieldType()) {
				case TrackerExtraField::TYPE_MEMBERS:
					// We hide member fields for folks with only the members permission
					if ($this->userHasMemberPermission()) {
						continue;
					}
					// Otherwise, continue through to default and render the field
				case TrackerExtraField::TYPE_SELECT:
				case TrackerExtraField::TYPE_MULTISELECT:
				case TrackerExtraField::TYPE_STATUS:
				case TrackerExtraField::TYPE_DATE: {
					//in query mode, the date is two fields entered as a range
					if ($mode == 'QUERY' && $field->getFieldType() == TrackerExtraField::TYPE_DATE) {
						$req = '&nbsp;(Between)';
					}
				}
				case TrackerExtraField::TYPE_CHECKBOX:
				case TrackerExtraField::TYPE_RADIO:
				case TrackerExtraField::TYPE_RELEASE: {
					$return .= '
					<td width="50%" valign="top"><strong>' . $field->getFieldName() . '</strong>:' . $req . '<br /><!--EF' . $field->getTrackerExtraFieldId() . '--></td>';
					break;
				}
				case TrackerExtraField::TYPE_NUMERIC: {
					//in query mode, the number is two fields entered as a range
					if ($mode == 'QUERY' && $field->getFieldType() == TrackerExtraField::TYPE_NUMERIC) {
						$req = '&nbsp;(Between)';
					}
				}
				case TrackerExtraField::TYPE_SERIAL: {
					//in query mode, the serial is two fields entered as a range
					if ($mode == 'QUERY' && $field->getFieldType() == TrackerExtraField::TYPE_SERIAL) {
						$req = '&nbsp;(Between)';
					}
				}
				case TrackerExtraField::TYPE_TEXT: {
					//text fields might be really wide, so need a row to themselves.
					if ($field->getAttribute1() > 30) {
						$colspan = 2;
						if ($col_count == 1) {
							$return .= "<td>&nbsp;</td>\n</tr>\n<tr>";
						}
					} else {
						$colspan = 1;
					}

					$return .= '
					<td width="' . (50 * $colspan) . '%" colspan="' . $colspan . '" valign="top"><strong>' . $field->getFieldName() .
						'</strong>:' . $req . '<br /><!--EF' . $field->getTrackerExtraFieldId() . '--></td>';
					if ($colspan == 2) {
						//this is an ugly hack, we are reseting the column
						//counter to avoid problems with following fields
						$col_count = 1;
						$return .=' </tr>';
					}

					break;
				}
				case TrackerExtraField::TYPE_TEXTAREA: {
					//text areas might be really wide, so need a row to themselves.
					if ($field->getAttribute2() > 30) {
						$colspan = 2;
						//is this the second column?
						if ($col_count == 1) {
							$return .= "<td>&nbsp;</td>\n</tr>\n<tr>";
						}
					} else {
						$colspan = 1;
					}

					$return .= '
					<td width="' . (50 * $colspan) . '%" colspan="' . $colspan . '" valign="top"><strong>' . $field->getFieldName()
						. '</strong>:' . $req . '<br /><!--EF' . $field->getTrackerExtraFieldId() . '--></td>';
					if ($colspan == 2) {
						//this is an ugly hack, we are reseting the column
						//counter to avoid problems with following fields
						$col_count = 1;
						$return .=' </tr>';
					}

					break;
				}
			}//end switch
			$col_count++;
			//we've done two columns - if there are more to do, start a new row
			if (($col_count == 2) && ($k != $count - 1)) {
				$col_count = 0;
				$return .= "</tr>\n<tr>";
			}
			$k++;
		}
		if ($col_count == 1) {
			$return .= "\n<td>&nbsp;</td>";
		}
		$return .= "\n</tr><!-- End Extra Fields Rendering -->";
		//Waiting for
		if ($this->Tracker->getDatatype() == Tracker::DATATYPE_SUPPORT &&
			$this->intrinsic_fields[Tracker::TRACKER_FIELD_WAITING_FOR]['is_enabled']) {
			if ($mode == 'QUERY' || ($mode == 'QUICKBROWSE' && $this->intrinsic_fields[Tracker::TRACKER_FIELD_WAITING_FOR]['on_browse'])) {
				$return .= '<tr><td width="50%" valign="top"><strong>' .
					$Lang->getText('TrackerItem.WaitingForName').':</strong><br /><!--EFWAITINGFOR--></td></tr>';
			}
		}

		return $return;
	}

	/**
	 * Gets the list of "historical" assignees (that is, it also returns the users that are not assignees but belong to a tracker item, since
	 * they may have been removed from the project at some point). It returns both the list of user objects and an array with the names to display
	 * on the widget. It appends an asterisk to those assignees which are no longer assignees in this tracker,
	 */
	private function getHistoricalAssignees($includeCurrentUserSpecial = false, $includeAnyUserSpecial = false) {
		$names = array();
		$technicians = $this->Tracker->getTechnicians(true, $includeCurrentUserSpecial);
		foreach ($technicians as $technician) {
			if ($technician->isUndefinedUser())
				continue;

			$perm = new Permission($this->Tracker->getProject(), $technician);
			if (!$perm->check(Tracker::SECTION, $this->Tracker->getPrimaryKey(), Tracker::PERMLEVEL_TECH)) {
				$names[] = $technician->getName() . ' [*]';
			} else {
				$names[] = $technician->getName();
			}
		}

		if ($includeCurrentUserSpecial) {
			$specialUser = new User();

			$specialUser->setFirstname('(current user)');
			$specialUser->setUserId(TrackerItemQuery::CURRENT_USER);

			array_unshift($technicians, $specialUser);
			array_unshift($names, '(current user)');
		}

		if ($includeAnyUserSpecial) {
			$specialUser = new User();

			$specialUser->setFirstname('(any user)');
			$specialUser->setUserId(TrackerItemQuery::ANY_USER);

			array_unshift($technicians, $specialUser);
			array_unshift($names, '(any user)');
		}

		return array('assignees' => $technicians, 'names' => $names);
	}

	private function getStatusElements(TrackerExtraField $field, $includeSpecials = FALSE) {
		$elements = $field->getTrackerExtraFieldElements();
		// [#9241]  Allow "any open" and "any closed" options for Status queries
		if ($includeSpecials == TRUE) {
			$efe = new TrackerExtraFieldElement();
			$efe->setElementName('(any closed status)');
			$efe->setElementId(TrackerItemQuery::STATUS_ANY_CLOSED);
			array_unshift($elements, $efe);

			$efe = new TrackerExtraFieldElement();
			$efe->setElementName('(any open status)');
			$efe->setElementId(TrackerItemQuery::STATUS_ANY_OPEN);
			array_unshift($elements, $efe);
		}
		return $elements;
	}

	private function isUserInList($list, $user_id = 0) {
		foreach ($list as $u) {
			if ($u->getUserId() == $user_id) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Determine if the user has Member permission (and only that permission) for the current tracker.
	 * This condition is used to control showing/hiding of group-type fields
	 * @return boolean true if the user has only the member permission for this tracker.  Otherwise, false.
	 */
	private function userHasMemberPermission() {
    return $this->Tracker->userHasMemberPermission();
  }
}
