<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('lib/html/ProjectTemplate.php');
require_once('lib/plugin/PluginResult.php');

class TrackerTemplate extends ProjectTemplate {

	/**
	 * Header - this subclasses header and just passes params and data that is specific
	 * to this template
	 * @param array array of parameters
	 */
	function header($params) {
		$params['selectedbottomtab']='tracker';
		$Lang = Session::getSession()->getLanguage();
		if (isset($params['tracker']) && is_object($params['tracker'])) {
			if (isset($params['title'])) {
				$params['title']=$Lang->getText('Tracker.TabString').' > '.$params['tracker']->getTrackerName().' > '.$params['title'];
			} else {
				Throw new exception('Missing title in tracker template');
			}
		} else {
			//Throw new exception('Missing tracker object in tracker template');
		}
		if (isset($params['tracker'])) {
			if (!$params['tracker']->userCanRead()) {
				throw new PermissionDeniedException();
			}
			//hack to suppress this breadcrumb if you're in a specific tracker
			if (!isset($params['istrackeradmin'])) {
				$this->addBreadCrumb('<a href="'.GFUrl::getUrl()->projectUrl($params['project'],'TrackerItemBrowse','tracker','',array('tracker_id'=>$params['tracker']->getTrackerId())).'">'.$params['tracker']->getTrackerName().'</a>');
			}

			$this->addBreadCrumb('<a href="'.GFUrl::getUrl()->projectUrl($params['project'],'','tracker').'">'.$Lang->getText('Tracker.TabString').'</a>');
		}
		return parent::header($params);
	}

	/**
	 *	getPluginNavElements - this overrides parent class and adds its own 
	 *	plugin listener
	 *	@return	PluginResultSet
	 */
	function getPluginNavElements($params) {
		$prs = parent::getPluginNavElements($params);
	       	$tprs = ActionManager::Notify('GET_TRACKER_NAV_ELEMENTS',$params);
		foreach ($tprs as $pr) {
			$prs->addResult($pr);
		}
		return $prs;
	}

	/**
	 * getTemplateNavElements - this function can be overridden
	 * by subclasses so you can add navigation entries
	 * @return array - complex array of data.
	 */
	function getTemplateNavElements($params) {
		$url = GFurl::getUrl();
		$lang = Session::getSession()->getLanguage();
		$user = Session::getSession()->getUser();
		$loggedin = Session::getSession()->isLoggedIn();
		$project = $params['project'];

		$pine = $this->getPluginNavElements($params);
		$ret = array();
		foreach ($pine as $result) {
			$ret[]=$result->getResult();
		}
		
		$elements = array();
		

		if (!isset($params['project'])) {
			echo 'missing project param';
		}

		$c = new Criteria();
		$c->add(TrackerPeer::PROJECT_ID,$project->getProjectId());
		$trackers = TrackerPeer::doSelect($c);

		foreach ($trackers as $tracker) {
			//if you have no access to this tracker, don't show it in nav
			if (!$tracker->userCanRead()) {
				continue;
			}
			$subelements = array();
			$subelements[] = array('link'=>'/gf/project/'.$params['project']->getUnixName().'/tracker/?action=TrackerItemBrowse&amp;tracker_id='.$tracker->getTrackerId(),
			'description'=> $lang->getText('Tracker.Browse'));
			$subelements[] = array('link'=>'/gf/project/'.$params['project']->getUnixName().'/tracker/?action=TrackerItemAdd&amp;tracker_id='.$tracker->getTrackerId(),
			'description'=> $lang->getText('TrackerItem.AddNew'));
			$subelements[] = array('link'=>'/gf/project/'.$params['project']->getUnixName().'/tracker/?action=TrackerQueryBrowse&amp;tracker_id='.$tracker->getTrackerId(),
			'description'=> $lang->getText('Tracker.Query'));
			$subelements[] = array('link'=>'/gf/project/'.$params['project']->getUnixName().'/tracker/?action=EmailGateway&amp;tracker_id='.$tracker->getTrackerId(),
			'description'=> $lang->getText('Tracker.EmailGatewayTitle'));

			/*if (!(isset($params['tracker_item']) && is_object($params['tracker_item']))) {		// tracker is set but trackeritem isn't*/
				if (!is_null($user)) {
					if ($tracker->userIsMonitoring($user)) {
						$subelements[] = array(
							'link' => $url->getMonitorUrl(Tracker::SECTION, $tracker->getPrimaryKey(), $_SERVER['REQUEST_URI'], false),
							'description' => $lang->getText('Tracker.StopMonitor'),
							true);
					} else {
						$subelements[] = array(
							'link' => $url->getMonitorUrl(Tracker::SECTION, $tracker->getPrimaryKey(), $_SERVER['REQUEST_URI'], true),
							'description' => $lang->getText('Tracker.Monitor'));
					}
					
					$subelements[] = array('link' => GFUrl::getUrl()->projectURL($project,'UsersMonitoringBrowse','','usersmonitoring',array('section'=>Tracker::SECTION,'ref_id'=>$tracker->getPrimaryKey(),'description'=>$tracker->getTrackerName(),'type'=>'item','return_to'=>$_SERVER['REQUEST_URI'])),
						'description' => $lang->getText('Monitor.UsersMonitoringTracker'));
				}
			//}

			/**
			 * @var Tracker $tracker
			 */
			if ($tracker->getDatatype() == Tracker::DATATYPE_TASK && $tracker->getCloseDateIsEnabled()) {
				$subelements[] = array('link'=>'/gf/project/'.$params['project']->getUnixName().'/tracker/?action=GanttView&amp;tracker_id='.$tracker->getTrackerId().'" onClick="window.open(this.href, \'Gantt_View\', \'scrollbars=yes,resizable=yes,toolbar=no,height=570,width=790\'); return false',
				'description'=> $lang->getText('Tracker.GanttView'));
			}

			$subelements[] = array('link' => $url->projectUrl($params['project'], 'TrackerExtraFieldElementTransitionBrowse', 'tracker', '', array('tracker_id' => $tracker->getTrackerId())),
			'description' => 'Workflow rules');

			if ($loggedin) {
				if ($project->getPermission()->check(Tracker::SECTION, $tracker->getTrackerId(), TrackerPlugin::TECHADMIN)) {
					$subelements[] = array('link' => $url->projectURL($project, 'TrackerEdit','tracker', 'admin',array('tracker_id' => $tracker->getTrackerId())),
					'description' => $lang->getText('Tabs.Admin'));
				}
			}
			
			
			
			if(isset($params['tracker']) && $params['tracker']->getTrackerId()==$tracker->getTrackerId()) {
				if (isset($params['tracker_item']) && is_object($params['tracker_item'])) {
					$tracker_item = $params['tracker_item'];
					if (!is_null($user)) {
						$subelements[] = array('link' => GFUrl::getUrl()->projectURL($project,'UsersMonitoringBrowse','','usersmonitoring',array('section'=>TrackerItem::SECTION,'ref_id'=>$tracker_item->getPrimaryKey(),'description'=>$tracker_item->getSummary(),'type'=>'item','return_to'=>$_SERVER['REQUEST_URI'])),
							'description' => $lang->getText('Monitor.UsersMonitoringItem'));
					}
				}
				$expanded=true;
			} else {
				$expanded=false;
			}

			$elements[] = array('link'=>'/gf/project/'.$params['project']->getUnixName().'/tracker/?action=TrackerItemBrowse&amp;tracker_id='.$tracker->getTrackerId(),
			'description'=> $tracker->getTrackerName(),
			'expandables' => $subelements,
			'expanded' => $expanded);
		}

		if (count($elements) > 0) {
			if(isset($params['tracker'])) {
				$title = $params['tracker']->getTrackerName();
			}
			else {
				$title = $lang->getText('Tracker.TabString');
			}
			$ret[] = array(
			'title'=>$title,
			'elements'=> $elements
			);
		}
		//return $ret;

		
		
		/*if (count($elements) > 0) {
			$ret[] = array(
			'title'=>Language::getSessText('Tracker.TabString'),
			'elements'=> $elements
			);
		}*/
		return $ret;
	}

	function generateRSSLinks() {
		$url = GFUrl::getUrl();
		$project = $url->getProject();
                $project_id = $project->getProjectId();

//		if (is_object($project) && $project->isPublic()) {
			// Retrieve the tracker objects array
			$crit = new Criteria();
			$crit->add(TrackerPeer::PROJECT_ID, $project->getProjectId());
			$crit->addAscendingOrderByColumn(TrackerPeer::DATATYPE);
			$arr = TrackerPeer::doSelect($crit);
//		} else {
//			$arr = array();
//		}

		$links_arr = parent::generateRSSLinks();

		if(is_array($arr) && count($arr) > 0){
			$i = 0;
			foreach ($arr as $item) {
				if ($item->userCanRead()) {
					$links_arr['tracker'.$i]['title'] = GFConf::get('system.name').' - '.$project->getName().' - Tracker: '.$item->getTrackerName();
					$links_arr['tracker'.$i]['url'] = $url->getFullURL($url->baseURL('Rss', '', '', array('section' => Tracker::SECTION, 'ref_id' => $item->getTrackerID(), 'project_id' => $project_id)));
					$i++;
				}
			}
		}

		return $links_arr;
	}

}

?>
