<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code creates a new TrackerItem and saves it to the 
// database
//
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('plugins/tracker/lib/TrackerItemMentionNotification.php');
require_once('Validator.php');
require_once('lib/core/Filesystem.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('exceptions/NotLoggedInException.php');
require_once('plugins/tracker/lib/TrackerItemAssignee.php');
require_once('plugins/tracker/lib/TrackerExtraFieldData.php');
require_once('core/Language.php');
require_once('lib/html/UploadHandler.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	error_log('PermissionDeniedException: 1, method = '.$_SERVER['REQUEST_METHOD']);
	throw new PermissionDeniedException();
}

//force login
$sess = Session::getSession();
if (!$sess->isLoggedIn()) {
	throw new NotLoggedInException();
}
$Project = GFUrl::getUrl()->getProject();
$tracker_id=Validator::sanitizeInt($_REQUEST['tracker_id']);
//validate that this is a legitimate tracker for this project and user can add items
$Tracker = TrackerPeer::retrieveByPk($tracker_id);
$TrackerItem = new TrackerItem();
$Tracker->addTrackerItem($TrackerItem);
if (!$Tracker || !is_object($Tracker) || $Tracker->getProject()->getProjectId() != $Project->getProjectId() || !$TrackerItem->userCanCreate()) {

	if (!$Tracker) $msg = '!$Tracker';
	if (!is_object($Tracker)) $msg = '!is_object($Tracker)';
	if ($Tracker->getProject()->getProjectId() != $Project->getProjectId()) $msg = '$tracker->project != $project';
	if (!$TrackerItem->userCanCreate()) $msg = 'userCanCreate = false';
	$counter++;
	error_log("PermissionDeniedException: 2, $msg");
	throw new PermissionDeniedException();
}
$redirect = GFUrl::getUrl()->currentBaseURL('TrackerItemBrowse');
if (array_key_exists("sprint_id", $_REQUEST)) {
	// adding into defined sprint, should redirect to sprint browse on success
 	$sprint_id = Validator::sanitizeInt($_REQUEST['sprint_id']);
 	$redirect = GFUrl::getFullURL(GFUrl::getUrl()->projectURL($Project, 'TrackerSprintView', 'tracker','', array('sprint_id'=>$sprint_id)));
} elseif (array_key_exists("tracker_sprint", $_REQUEST)) {// adding to tracker
 	$sprint_id = Validator::sanitizeInt($_REQUEST['tracker_sprint']);
}
if ($sprint_id > 0 && $sprint_id != TrackerSprint::TRACKER_SPRINT_NONE) {
	$TrackerSprint = TrackerSprintPeer::retrieveByPk($sprint_id, $Project->getProjectId());
        if (!$TrackerSprint->userIsAdmin() && $TrackerSprint->getSprintStatus() == TrackerSprint::STATUS_CLOSED) {
            throw new PermissionDeniedException();
        } else {
            if (!is_object($TrackerSprint)) {
                    throw new PermissionDeniedException();
            }
            $TrackerItem->setTrackerSprint($TrackerSprint);
        }
}

$errors = array();
$language = Language::getSessLang();
$stringUtils = new StringUtils();

//TODO: do some verification, add error strings to $errors

if (count($errors) == 0) {
//	$TrackerItem->setTrackerId( Validator::sanitizeInt($_POST['tracker_id']) );
//	$TrackerItem->setStatusId( Validator::sanitizeInt($_POST['status_id']) );
	$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
	$con->begin();
	$TrackerItem->setPriority( Validator::sanitizeInt($_POST['priority']) );
	$TrackerItem->setSubmittedBy( $sess->getUser()->getUserId() );
	if ($Tracker->getDatatype()==Tracker::DATATYPE_TASK) {
		if (isset($_POST['start_date']) && strlen($_POST['start_date']) > 5) {
			$TrackerItem->setOpenDate(Validator::sanitizeDate($_POST['start_date']));
		}
		if (isset($_POST['end_date']) && strlen($_POST['end_date']) > 5) {
			$TrackerItem->setCloseDate(Validator::sanitizeDate($_POST['end_date']));
		}
	}
	$TrackerItem->setSummary(Validator::sanitizeString($_POST['summary'], false) );
	$TrackerItem->setDetails( $stringUtils->cleanHtml($_POST['details']) );

	try {
		//
		//	Admins can set an assignee at submit time
		//
		if ($Tracker->userIsAdmin()) {
			if (!isset($_POST['assigned_to'])) {
				$assigned_to=array('100');
			} else {
				$assigned_to=$_POST['assigned_to'];
			}
		} else {
			$assigned_to=$TrackerItem->determineAutoAssignId($_POST['extra_field']);
			if ($assigned_to===false) {
				$assigned_to=array('100');
			}
		}
		$TrackerItem->addAssignees($assigned_to);
		$TrackerItem->addExtraFields($_POST['extra_field']); //Always call addExtraFields, since it will ensure that default values are applied when the extra field was not submitted
		$TrackerItem->setLastModifiedDate(time());
		$TrackerItem->save();
		ActionManager::notify('TRACKER_ITEM_ADD_ACTION',$TrackerItem);

		//Set monitoring
		$monitor = isset($_POST['monitor']) ? true : false;
		$setting_name =  Tracker::MONITORING_SETTING_NAME . '_' . $session->getUser()->getUserId();
		if ($monitor) {
			$TrackerItem->monitorItem($sess->getUser());
			//Save the setting for setting the monitor checkbox on
			SettingPeer::SaveSetting(Tracker::SECTION, $Tracker->getTrackerId(), $setting_name, 'true', $Project->getProjectId());
		} else {
			//Save the setting for setting the monitor checkbox off
			SettingPeer::SaveSetting(Tracker::SECTION, $Tracker->getTrackerId(), $setting_name, 'false', $Project->getProjectId());

		}

		// Set up info for user wall mentions, if needed
		$user = $sess->getUser();
		$userUrl = GFUrl::getUrl()->userURL($user);
		$userlink = "<a href=\"{$userUrl}\" target=\"_new\" title=\"User profile for {$user->getUnixName()}\">{$user->getUnixName()}</a>";
		$tracker_item_id = $TrackerItem->getTrackerItemId();
		$tracker_item_url = GFUrl::getUrl()->currentBaseURL('TrackerItemEdit', array('tracker_item_id'=>$tracker_item_id));
		$link = "<a href=\"{$tracker_item_url}\" title=\"View details\" >Tracker Item {$tracker_item_id}</a>";

		// Check/send @mentions
		$to_users = StringUtils::getMentionedUsers($TrackerItem->getDetails());
		if (count($to_users) > 0) {
		    $wall_comment = "$userlink mentioned you in $link: <br /><br />{$TrackerItem->getSummary()}";
		    ForumPeer::sendUserWallMentions($to_users, $user->getUserId(), $wall_comment, '', false);

			foreach($to_users as $key => $unix_name) {
				$u = UserPeer::retrieveByUnixName($unix_name);
				if(is_null($u)) {
					continue;
				}
				if(!$Tracker->userIsMonitoring($u) && !$TrackerItem->userIsMonitoring($u)) {
					$tiMentionNotification = new TrackerItemMentionNotification($u, $TrackerItem, $user);
					NotificationManager::getManager()->queue($tiMentionNotification);
				}
			}
		}
		
		// Check/send assignee notifications
		$to_users = array();
		foreach ($assigned_to as $assignee) {
			if (strlen($assignee) > 0   // skip empty item(?)
				&& $assignee != '100'  // skip Nobody user
				//&& $assignee != $user->getUserId()  // skip yourself
				&& !in_array($assignee, $to_users)) {  // skip duplicates(?)
				$to_users[] = $assignee;
			}
		}
		if (count($to_users) > 0) {
		    $wall_comment = "$userlink assigned $link to you: <br /><br />{$TrackerItem->getSummary()}";
		    ForumPeer::sendUserWallMentions($to_users, $user->getUserId(), $wall_comment, '', false);
		}

		// Now handle the files uploaded using the upload widget.
		$uploadHandler = new UploadHandler();
		$files = $uploadHandler->getTmpUploadedFiles();
		foreach ($files as $file) {
			$TrackerItem->attachFile($file, 0, false);
		}
		$uploadHandler->purgeTmpDir();

		$con->commit();
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());		// attach object errors to validation errors
	}

}
	
if (count($errors) > 0) {
	$feedback = '* '.implode('<br />* ', $errors);
	$con->rollback();
	//GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerItemAdd'), array('feedback' => $feedback, 'tracker_id'=>$tracker_id));
	// Include the file instead of redirecting so we can save the data from $_POST
	include('TrackerItemAdd.php');
} else {
	$feedback = $language->getText('TrackerItem.AddFeedback', array($tracker_item_id));
	GFUrl::getUrl()->redirect($redirect, array('feedback' => $feedback, 'tracker_id'=>$tracker_id));
}

?>
