<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code lists TrackerItem
//
require_once('plugins/tracker/lib/TrackerItemPeer.php');
require_once('plugins/tracker/lib/TrackerItemSavedQuery.php');
require_once('plugins/tracker/lib/TrackerItemQuickQuery.php');
require_once('plugins/tracker/lib/TrackerQuery.php');
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('plugins/tracker/lib/html/TrackerExtraFieldRenderer.php');
require_once('exceptions/PermissionDeniedException.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

require_once('plugins/tracker/lib/html/TrackerTemplate.php');
$theme = new TrackerTemplate();

$Project = GFUrl::getUrl()->getProject();
$Tracker = TrackerPeer::retrieveByPk($_REQUEST['tracker_id']);
if (!$Tracker || !is_object($Tracker) || $Tracker->getProject()->getProjectId() != $Project->getProjectId() || !$Tracker->userCanRead()) {
        throw new PermissionDeniedException();
}

//Get intrinsic fields settings
$intrinsic_fields = $Tracker->getIntrinsicFieldsSettings();

$params = array(
		'project' => $Project,
		'tracker' => $Tracker,
		'action' => isset($action) ? $action : '',
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('TrackerItem.BrowsePageTitle')
		);
$theme->header($params);

$tqid = Validator::sanitizeInt($_GET['tracker_query_id']);

if (isset($_GET['tracker_query_id'])) {
	$link_csv = GFUrl::getUrl()->currentBaseUrl('TrackerQueryCSV', array('tracker_id' => $Tracker->getTrackerId(), 'tracker_query_id'=>$tqid));
	$link_xls = GFUrl::getUrl()->currentBaseUrl('TrackerQueryExcel', array('tracker_id' => $Tracker->getTrackerId(), 'tracker_query_id'=>$tqid));
	$link_xml = GFUrl::getUrl()->currentBaseUrl('TrackerQueryXML', array('tracker_id' => $Tracker->getTrackerId(), 'tracker_query_id'=>$tqid));
} else {
	$link_csv = GFUrl::getUrl()->currentBaseUrl('TrackerQueryCSV', array('tracker_id' => $Tracker->getTrackerId()));
	$link_xls = GFUrl::getUrl()->currentBaseUrl('TrackerQueryExcel', array('tracker_id' => $Tracker->getTrackerId()));
	$link_xml = GFUrl::getUrl()->currentBaseUrl('TrackerQueryXML', array('tracker_id' => $Tracker->getTrackerId()));
}

?>
<script>
	var link_csv = '<?php echo htmlspecialchars_decode($link_csv); ?>';
	var link_xls = '<?php echo htmlspecialchars_decode($link_xls); ?>';
	var link_xml = '<?php echo htmlspecialchars_decode($link_xml); ?>';
	var url_mass_delete = '<?php echo GFUrl::getUrl()->currentBaseUrl('TrackerItemMassDelete', array('tracker_id' => $Tracker->getTrackerId())); ?>';

	function tiMassDelete() {
		var selected = [];
		$('#massupdateform input:gt(1):checked').each(function() {
			var ti = $(this).attr('value');
			selected.push(ti);
		});

		var tids = selected.join();
		document.location.href = unescape(url_mass_delete + '&tids_delete=' + tids);
	}

	function executeQuery() {
		if ($("#querynav").prop("selectedIndex") == 0) {
			document.quickbrowse.submit();
		} else {
			document.location.href = $("#querynav").val();
		}
	}

	function produceQueryOutput(sel) {
		var str_url = '';
		switch(sel) {
			case 'csv':
				str_url = link_csv;
				break;

			case 'xls':
				str_url = link_xls;
				break;

			case 'xml':
				str_url = link_xml;
				break;
		}

		if (str_url != '') {
			document.location.href = str_url;
		}
	}

	$(document).ready(function() {
    $('#select_all').on('click', function () {
      $("table > tbody:eq(1) tr:gt(3):not(:last) td > label > input").prop('checked', this.checked);
    });
    $("table > tbody:eq(1) tr:gt(3):not(:last) td > label > input").on('click', function() {
			if (!this.clicked) {
				$('#select_all').prop('checked', false);
			}
		});

		function updateQuickBrowse(goToPage) {
			if ($("#querynav").prop("selectedIndex") == 0) {
				$("#quickBroweSection").show();
			} else {
				$("#quickBroweSection").hide();
			}
		}

		$("#querynav").on('change', function() {
			updateQuickBrowse(true);
		});

		$("#query_output").on('change', function() {
			produceQueryOutput();
		});

		$("#quickBroweSection").hide();
        updateQuickBrowse(false);
	});
</script>
<?php
/* There are 3 ways of retrieving the tracker items:
 * 1) Running a query
 * 2) "Remembering" the last query used,
 * 3) Using the "Quick query" menu.
 * 
 * The logic goes like this: if a query is run, remember the query_id and the current page so the
 * user can come back to the same page if he, for example, adds a new tracker item.
 * If a quick query is run, forget the query_id and remember the parameters of the quick query instead.
 */

// Saved query logic
$preference_name = 'trkquery'.$Tracker->getTrackerId();
if ($session->isLoggedIn()) {
	$user = $session->getUser();
	if (!isset($_GET['quickquery']) && !isset($_GET['tracker_query_id']) && !isset($_GET['forget_query'])) {
		$saved_query = $user->getPreference($preference_name);
		if ($saved_query) {
            try {
                // Emulate a query using the saved query information
                $saved_query = unserialize($saved_query);
                $_REQUEST['tracker_query_id'] = $_GET['tracker_query_id'] = $saved_query['tracker_query_id'];
                $_REQUEST['show'] = $_GET['show'] = $saved_query['show'];
                $_REQUEST['start'] = $_GET['start'] = $saved_query['start'];
            } catch (Exception $ex) {
                error_log("Caught an exception while unserializing query prefs for {$user->getUnixName()} in tracker {$Tracker->getProject()->getUnixName()}/{$Tracker->getTrackerName()}: {$ex->getMessage()}");
        		$user->deletePreference($preference_name);
            }
		}
	} else {
		// If using quick query, delete the saved query ID
		$user->deletePreference($preference_name);
	}
}

if (isset($_GET['tracker_query_id'])) {
	if (!$session->isLoggedIn()) {
		throw new PermissionDeniedException();
	}
	$tracker_query_id = Validator::sanitizeInt($_GET['tracker_query_id']);
	$tsq = TrackerQueryPeer::retrieveByPk($tracker_query_id);
	if (!$tsq) {
		// The query got deleted for some reason
		$tq = new TrackerItemQuickQuery($Tracker);
	} elseif (!$tsq->getIsShared() && $tsq->getUserId() != $session->getUser()->getUserId()) {
		throw new PermissionDeniedException();
	} else {
		$tq = new TrackerItemSavedQuery($Tracker,$tsq, $session->getUser());
	}
} else {
	$tq = new TrackerItemQuickQuery($Tracker);
}

//
//	build the query
//
$show=((isset($_REQUEST['show'])) ? intval($_REQUEST['show']) : 25 );
if (!isset($_GET['start'])) {
	$start=0;
} else {
	$start=$_GET['start'];
}
$totalcount=$tq->getTrackerItemCount();
$arr=$tq->getTrackerItems($show,$start);

// Save the tracker_query info, if any
if (isset($_GET['tracker_query_id']) && $session->isLoggedIn()) {
	$saved_query = array('tracker_query_id' => $_GET['tracker_query_id'], 'show' => $show, 'start' => $start);
	try {
		$session->getUser()->setPreference($preference_name, serialize($saved_query));
	}  catch (Exception $e) {
		//Log a warning, avoid displaying an error message which is not fatal or critical
		syslog(LOG_WARNING, $e->getMessage());
	}
}

//show a list of saved queries
$str_savedqueries = '';
if ($session->isLoggedIn()) {
	// Retrieve the objects array
	$crit = new Criteria();
	$cton1=$crit->getNewCriterion(TrackerQueryPeer::USER_ID, $session->getUser()->getID());
	$cton2=$crit->getNewCriterion(TrackerQueryPeer::IS_SHARED, true);
	$cton1->addOr($cton2);
	$crit->add($cton1);
	$queries = $Tracker->getTrackerQuerys($crit);
    usort($queries, 'sortQueries');

	include('TrackerQueryTicklers.php');

	echo '<form id="quickbrowse" name="quickbrowse" action="'.GFUrl::getUrl()->currentBaseUrl().'" method="get">';

	echo '<input type="hidden" name="action" value="TrackerItemBrowse" />';
	echo '<input type="hidden" name="tracker_id" value="'.$Tracker->getTrackerId().'" />';

	$str_savedqueries = '<table>
    <tr><td valign="top" width="120"><h4>' . $Lang->getText("Tracker.ChooseQuery") . '</h4></td><td valign="top"><select style="margin-bottom: 4px;" sorted="sorted" name="querynav" id="querynav" class="">
      <option value="'.GFUrl::getUrl()->currentBaseUrl('TrackerItemBrowse', array('tracker_id' => $Tracker->getTrackerId(), 'forget_query' => 1)).'">'.$Lang->getText('Tracker.Filter').'</option>
      <option value="'.GFUrl::getUrl()->currentBaseUrl('TrackerQueryAdd').'&amp;tracker_id='.$Tracker->getTrackerId().'">'.$Lang->getText('Tracker.FilterAdd').'</option>'."\n";
	$url=GFUrl::getUrl()->currentBaseUrl('TrackerItemBrowse').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;tracker_query_id=';

	foreach ($queries as $query) {
		$str_savedqueries .= '      <option value="'.$url.$query->getTrackerQueryId().'" '.((isset($_REQUEST['tracker_query_id']) && $_REQUEST['tracker_query_id']==$query->getTrackerQueryId()) ? 'selected="selected"': '').'>'.$query->getQueryName().'</option>'."\n";
	}
	$str_savedqueries .= '</select>';


	$title_na = "N/A";
	$title_csv = "CSV";
	$title_xls = "XLS";
	$title_xml = "XML";

	$str_savedqueries .= '&nbsp;&nbsp;&nbsp;';
  $str_savedqueries .= '<div style="display: inline; min-width: 300px;">';
  $str_savedqueries .= HTMLWidget::button("button",$Lang->getText('Common.Browse'),"browse","","executeQuery();", true, "button", "position: relative");
  $str_savedqueries .= '&nbsp;';
	$str_savedqueries .= HTMLWidget::button("button",$title_csv,"csv","","produceQueryOutput('csv');", true, "button", "position: relative");
	$str_savedqueries .= HTMLWidget::button("button",$title_xls,"csv","","produceQueryOutput('xls');", true, "button", "position: relative");
	$str_savedqueries .= HTMLWidget::button("button",$title_xml,"xml","","produceQueryOutput('xml');", true, "button", "position: relative");
  $str_savedqueries .= '</div>';
	$str_savedqueries .= '</td>'."\n";

	$str_savedqueries .= '</tr></table>';
	echo $str_savedqueries;
}

//
//	Show the quick-browse pop-ups
//

echo '<input type="hidden" name="quickquery" id="quickquery_id" value="1" />';
echo '<ul class="quickBrowse" id="quickBroweSection">';
echo '<li class="quickBrowse"><strong>'.$Lang->getText('TrackerItem.TrackerItemIdName').'</strong><br /><input type="text" name="tracker_item_id" value="'.$tq->getTrackerItemId().'" size="4" maxlength="8" /></li>';
echo '<li class="quickBrowse"><strong>'.$Lang->getText('TrackerItem.SummaryName').'</strong><br /><input type="text" name="summary" value="'.$tq->getSummary().'" size="10" maxlength="40" /></li>';
echo '<li class="quickBrowse"><strong>'.$Lang->getText('TrackerItem.SubmittedByName').'</strong><br /><input type="text" name="submitted_by" value="'.$tq->getSubmittedBy().'" size="10" maxlength="40" /></li>';
$tefr = new TrackerExtraFieldRenderer($Tracker);
echo $tefr->renderFields('QUICKBROWSE',null,$tq);
echo '<li class="quickBrowse"><strong>'.$Lang->getText('Tracker.SortCol').'</strong><br />'.HTMLWidget::selectBoxFromAssoc($tq->getAvailableSortColumns(),'sortcol',$tq->getSortCol(),false,false).'</li>';
echo '<li class="quickBrowse"><strong>'.$Lang->getText('Tracker.SortOrd').'</strong><br />'.HTMLWidget::selectBoxFromAssoc($tq->getAvailableSortOrd(),'sortord',$tq->getSortOrd(),false,false).'</li>';
//echo '</ul>';
echo '<br /><br />';
//echo '<ul class="quickBrowse">';
echo '</ul>';
echo '</form>';
//echo '<br/><br/>';

function sortQueries(TrackerQuery $a, TrackerQuery $b) {
    $aname = $a->getQueryName();
    $bname = $b->getQueryName();
    if ($aname == $bname) {
        return 0;
    }
    return ($aname < $bname) ? -1 : 1;
}




$IS_ADMIN=$Tracker->userIsAdmin();
$IS_TASK=$Tracker->getDatatype()==Tracker::DATATYPE_TASK;

$priorityTexts = Tracker::getPriorities();

//
//	Show the browse instructions. 
//
if (strlen($Tracker->getBrowseInstructions()) > 0) {
	echo "<br />".$Tracker->getBrowseInstructions(false)."<br />";
}

//
//	Pagination code
//
echo "<br />";
$pages = $totalcount / $show;
$currentpage = intval($start / $show);
$skipped_pages=false;
if ($totalcount > 0)
  echo "<strong>" . $Lang->getText("Paginator.Page") . ": </strong>";

for ($j=0; $j<$pages; $j++) {
  if ($pages > 20) {
    if ((($j > 4) && ($j < ($currentpage-5))) || (($j > ($currentpage+5)) && ($j < ($pages-5)))) {
      if (!$skipped_pages) {
        $skipped_pages=true;
        echo "....&nbsp;";
      }
      continue;
    } else {
      $skipped_pages=false;
    }
  }
  if ($j == $currentpage) {
    echo '<strong>'.($j+1).'</strong>&nbsp;';
  } else {
    if (isset($_GET['tracker_query_id'])) {
      echo '<a href="'.GFUrl::getUrl()->currentBaseUrl('TrackerItemBrowse').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;tracker_query_id='.$tracker_query_id.'&amp;start='.($j*$show).'">'.($j+1).'</a>&nbsp;';
    } else {
      echo '<a href="'.GFUrl::getUrl()->currentBaseUrl('TrackerItemBrowse').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;start='.($j*$show).'">'.($j+1).'</a>&nbsp;';
    }
  }
}

  $tr = $Lang->getText("Paginator.TotalRecords");
  echo "<strong>(". $totalcount . " " . $tr . ")</strong>";

//
//	Show the list of items that match the query.
//

	$theader = array();

  $theader[] = '<input id="select_all" type="checkbox">' . 
    $Lang->getText('TrackerItem.TrackerItemIdName');
	$theader[] = $Lang->getText('TrackerItem.SummaryName');
//$theader[] = $Lang->getText('TrackerItem.StatusIdName');
	if ($tq->showColumn('priority') && $intrinsic_fields[Tracker::TRACKER_FIELD_PRIORITY]['is_enabled'] && $intrinsic_fields[Tracker::TRACKER_FIELD_PRIORITY]['on_browse'])
		$theader[] = $Lang->getText('TrackerItem.PriorityName');
	if ($tq->showColumn('assigned_to'))
		$theader[] = $Lang->getText('TrackerItem.AssignedTo');
	if ($tq->showColumn('submitted_by'))
		$theader[] = $Lang->getText('TrackerItem.SubmittedByName');
  if ($tq->showColumn('waiting_for') && $intrinsic_fields[Tracker::TRACKER_FIELD_WAITING_FOR]['is_enabled'] && $intrinsic_fields[Tracker::TRACKER_FIELD_WAITING_FOR]['on_browse'])
    $theader[] = $Lang->getText('TrackerItem.WaitingForName');
	$efs=$Tracker->getTrackerExtraFields();
	foreach ($efs as $ef) {
		$efcache[$ef->getTrackerExtraFieldId()]=$ef;
		if ($ef->getUseInQuickBrowse()) {
			$theader[] = $ef->getFieldName();
		}
	}
  if ($tq->showColumn('tracker_sprint') && $intrinsic_fields[Tracker::TRACKER_FIELD_SPRINT_ID]['is_enabled'] && $intrinsic_fields[Tracker::TRACKER_FIELD_SPRINT_ID]['on_browse'])
    $theader[] = $Lang->getText('TrackerSprint.Name');

	if ($tq->showColumn('open_date') && $intrinsic_fields[Tracker::TRACKER_FIELD_OPEN_DATE]['is_enabled'] && $intrinsic_fields[Tracker::TRACKER_FIELD_OPEN_DATE]['on_browse'])
		$theader[] = $Lang->getText('TrackerItem.OpenDateName');
	if ($tq->showColumn('close_date') && $intrinsic_fields[Tracker::TRACKER_FIELD_CLOSE_DATE]['is_enabled'] && $intrinsic_fields[Tracker::TRACKER_FIELD_CLOSE_DATE]['on_browse'])
		$theader[] = $Lang->getText('TrackerItem.CloseDateName');
	if ($tq->showColumn('last_modified_date'))
		$theader[] = $Lang->getText('TrackerItem.LastModifiedDateName');

	echo '<form id="massupdateform" action="'.GFUrl::getUrl()->currentBaseUrl('TrackerItemMassUpdateAction').'&amp;tracker_id='.$Tracker->getTrackerId().'" method="post">';
	
	// Display the "Add new tracker item" button on the top of the table (if necessary)
	$widgets_left = array();
	$widgets_right = array();

	if (count($arr) >= 0 && $Tracker->userCanRead() && Session::getSession()->isLoggedIn()) {
		$link = GFUrl::getUrl()->currentBaseUrl('TrackerItemAdd', array('tracker_id' => $Tracker->getTrackerId()));
		$title = $Lang->getText('TrackerItem.AddNew');
		$widgets_left[] = HTMLWidget::button("button", $title , "", "", 'window.location=\'' . $link . '\';', true, "button", "", true);
	}

	echo $theme->tableTop($theader, false, '', 'tabular', "", true, $widgets_left, $widgets_right);


//	$max = (($totalcount > ($start + $show)) ? ($start+$show) : $totalcount );
//echo "max: $max";
if (count($arr) == 0 ) {
	echo $theme->tableFullRow($Lang->getText('TrackerItem.NoneFound'));
} else {
	$editurl=GFUrl::getUrl()->currentBaseUrl('TrackerItemEdit');
	$deleteurl=GFUrl::getUrl()->currentBaseUrl('TrackerItemDelete');
	for ($i=0; $i<count($arr); $i++) {
		$trow = array();
		$tmod = array();
    if ($IS_ADMIN) {
      $trow []= '<label style="white-space: nowrap" for="ti' . $i . '">'
        . '<input type="checkbox" name="tid[]" value="' . $arr[$i]->getTrackerItemId() . '" id="ti' . $i . '" />'
        . $arr[$i]->getTrackerItemId() . '</label>';
    } else {
      $trow []= $arr[$i]->getTrackerItemId();
    }
		$tmod[]='';
		
		// Text that will be used on the tooltip
		if ($intrinsic_fields[Tracker::TRACKER_FIELD_DETAILS]['is_enabled']) {
			$tooltip = StringUtils::trimString($arr[$i]->getDetails(), 500);
			$tooltip = StringUtils::generateTooltip($tooltip);
			$tooltip = 'onmouseover="tooltip(\''.$tooltip.'\');" onmouseout="tooltipexit()"';
		}
		
		$trow[]='<a href="'. $editurl .'&amp;tracker_item_id='.$arr[$i]->getTrackerItemID() .'&amp;start='. $start .'"' . $tooltip . '>'. htmlentities($arr[$i]->getSummary(false), ENT_COMPAT,'UTF-8').'</a>';
		$tmod[]='';

		//$trow[]=$arr[$i]->getStatusId();

		if ($tq->showColumn('priority') && $intrinsic_fields[Tracker::TRACKER_FIELD_PRIORITY]['is_enabled'] && $intrinsic_fields[Tracker::TRACKER_FIELD_PRIORITY]['on_browse']) {
			$priority = $arr[$i]->getPriority();
			$trow[] = $priorityTexts[$priority];
			$tmod[] = 'bgcolor="'.$theme->getPriorityColor($priority).'"';
		}
		if ($tq->showColumn('assigned_to')) {
			$asses = $arr[$i]->getTrackerItemAssignees();
			$assignees = '';
			foreach ($asses as $ass) {
				$assignees .= $ass->getUser()->getUserLink().'<br />';
			}
			$trow[]=$assignees;
			$tmod[]='';
			unset($assignees);
		}
		if ($tq->showColumn('submitted_by')) {
			$trow[]=UserPeer::retrieveByPk($arr[$i]->getSubmittedBy())->getUserLink();
			$tmod[]='';
		}
    if ($tq->showColumn('waiting_for') && $intrinsic_fields[Tracker::TRACKER_FIELD_WAITING_FOR]['is_enabled'] && $intrinsic_fields[Tracker::TRACKER_FIELD_WAITING_FOR]['on_browse']) {
      $trow[]= $arr[$i]->getWaitingForString();
      $tmod[]='';
    }
		$colarray=array();
		$efdatas=$arr[$i]->getTrackerExtraFieldDatas();

		foreach($efdatas as $efdata) {
			if (is_object($efcache[$efdata->getTrackerExtraFieldId()])) {
				$colarray[$efdata->getTrackerExtraFieldId()][]=$efcache[$efdata->getTrackerExtraFieldId()]->translateFieldDataToName($efdata->getFieldData());
			}
		}
		reset($efs);
		foreach ($efs as $ef) {
			if ($ef->getUseInQuickBrowse()) {
				if (isset($colarray[$ef->getTrackerExtraFieldId()]) && count($colarray[$ef->getTrackerExtraFieldId()]) > 1) {
					$trow[] = implode('<br />',$colarray[$ef->getTrackerExtraFieldId()]);
					$tmod[]='';
				} else {
					if (isset($colarray[$ef->getTrackerExtraFieldId()])) {
						if($ef->getFieldType() == TrackerExtraField::TYPE_DATE){
							if ($colarray[$ef->getTrackerExtraFieldId()][0] != "") {
								$dt = date('Y-m-d',  strtotime($colarray[$ef->getTrackerExtraFieldId()][0]));
								$trow[] = $dt;
							} else {
								$trow[] = '--';
							}
						} else {
							$trow[] =  $colarray[$ef->getTrackerExtraFieldId()][0];
						}
							$tmod[]='';
					} else {
						$trow[] = '--';
						$tmod[]='';
					}
				}
			}
		}
		if ($tq->showColumn('tracker_sprint') && $intrinsic_fields[Tracker::TRACKER_FIELD_SPRINT_ID]['is_enabled'] && $intrinsic_fields[Tracker::TRACKER_FIELD_SPRINT_ID]['on_browse']) {
      $sprint = $arr[$i]->getTrackerSprint();
      if (is_object($sprint)) {
        $trow[]=$sprint->getSprintTitleText();
        $tmod[]='';
      } else {
        $trow[]=$Lang->getText('Common.None');
        $tmod[]='';
      }
    }
		if ($tq->showColumn('open_date') && $intrinsic_fields[Tracker::TRACKER_FIELD_OPEN_DATE]['is_enabled'] && $intrinsic_fields[Tracker::TRACKER_FIELD_OPEN_DATE]['on_browse']) {
			$trow[]=$arr[$i]->getOpenDate('Y-m-d');
			$tmod[]='';
		}
		if ($tq->showColumn('close_date') && $intrinsic_fields[Tracker::TRACKER_FIELD_CLOSE_DATE]['is_enabled'] && $intrinsic_fields[Tracker::TRACKER_FIELD_CLOSE_DATE]['on_browse']) {
			$trow[]=(($arr[$i]->getStatusId()==Tracker::STATUS_CLOSED || $IS_TASK) ? $arr[$i]->getCloseDate('Y-m-d') : '&nbsp;');
			$tmod[]='';
		}
		if ($tq->showColumn('last_modified_date')) {
			$trow[]=$arr[$i]->getLastModifiedDate('Y-m-d');
			$tmod[]='';
		}

//echo "trow count: ".count($trow);
		echo $theme->tableRow($trow,$tmod,false);
		unset($trow);
		unset($tmod);
	}
}

if ($Tracker->userCanRead() && Session::getSession()->isLoggedIn()) {
	$button_enabled = true;
} else {
	$button_enabled = false;
}

if ($Tracker->userCanDelete() && Session::getSession()->isLoggedIn()) {
	$mass_delete_enabled = true;
} else {
	$mass_delete_enabled = false;
}

echo $theme->tableBottom(array(GFUrl::getUrl()->currentBaseUrl('TrackerItemAdd').'&amp;tracker_id='.$Tracker->getTrackerId()=>$Lang->getText('TrackerItem.AddNew')),array($button_enabled), "", false, true, true, array(true, false));

	//
	//	Pagination code
	//
	$pages = $totalcount / $show;
	$currentpage = intval($start / $show);
	$skipped_pages=false;
  if ($totalcount > 0)
    echo "<strong>" . $Lang->getText("Paginator.Page") . ": </strong>";
	for ($j=0; $j<$pages; $j++) {
		if ($pages > 20) {
			if ((($j > 4) && ($j < ($currentpage-5))) || (($j > ($currentpage+5)) && ($j < ($pages-5)))) {
				if (!$skipped_pages) {
					$skipped_pages=true;
					echo "....&nbsp;";
				}
				continue;
			} else {
				$skipped_pages=false;
			}
		}
		if ($j == $currentpage) {
			echo '<strong>'.($j+1).'</strong>&nbsp;';
		} else {
			if (isset($_GET['tracker_query_id'])) {
				echo '<a href="'.GFUrl::getUrl()->currentBaseUrl('TrackerItemBrowse').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;tracker_query_id='.$tracker_query_id.'&amp;start='.($j*$show).'">'.($j+1).'</a>&nbsp;';
			} else {
				echo '<a href="'.GFUrl::getUrl()->currentBaseUrl('TrackerItemBrowse').'&amp;tracker_id='.$Tracker->getTrackerId().'&amp;start='.($j*$show).'">'.($j+1).'</a>&nbsp;';
			}
		}
	}

  $tr = $Lang->getText("Paginator.TotalRecords");
  echo "<strong>(". $totalcount . " " . $tr . ")</strong>";

	//
	//	Mass update code
	//
if ($IS_ADMIN) {
	echo '<br/><br/><form><fieldset class="massUpdate"><legend><strong>&nbsp;' . $Lang->getText("Tracker.MassUpdate") . '&nbsp;</strong></legend><table width="100%" style="clear:left;">';
	$tefr = new TrackerExtraFieldRenderer($Tracker);
	echo $tefr->renderFields('MASSUPDATE');
	$start = $currentpage * $show;
	echo '<input name="start" type="hidden" value="'.($start).'" />';
	echo '<tr><td>'.HTMLWidget::button("submit",$Lang->getText('Tracker.MassUpdate'),"browse","","");
	if ($mass_delete_enabled) {
		echo HTMLWidget::button("button",$Lang->getText('TrackerItem.MassDelete'),"massdelete","", "tiMassDelete()");
	}
	echo '</td></tr></table></fieldset></form>';
}
$theme->footer($params);
?>
