<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code shows a form for editing TrackerItemMessage
//
require_once('plugins/tracker/lib/TrackerItemScmCommit.php');
require_once('plugins/tracker/lib/TrackerItemScmCommitPeer.php');
require_once('plugins/scm/lib/ScmCommitFile.php');
require_once('plugins/scm/lib/RepositoryBrowser.php');
require_once('plugins/scm/lib/SCMRepositoryFactory.php');


function getScmBrowser($repository) {
	$browser = null;

	if (!is_null($repository)) {		// Project doesn't have a repository
		try {
			$browser = new RepositoryBrowser($repository);
		} catch (Exception $e) {
			// SCM other than CVS or SVN
			$browser = null;
		}
	}

	return $browser;
}

function commitTableHeader($Lang, $theme) {
	$theader = array();
	$theader[] = $Lang->getText('TrackerItemScmCommit.ModifiedFile');
	$theader[] = $Lang->getText('TrackerItemScmCommit.CommitDate');
	$theader[] = $Lang->getText('TrackerItemScmCommit.PreviousVersion');
	$theader[] = $Lang->getText('TrackerItemScmCommit.NewVersion');
	$theader[] = $Lang->getText('TrackerItemScmCommit.MessageLog');
	$theader[] = $Lang->getText('TrackerItemScmCommit.SubmittedByName');

	return $theme->tableTop($theader);
}

function commitTableBody($Lang, $theme, $browser, $arr = array()) {

	$str_commits = '';

	if (count($arr) < 1) {
		$str_commits .=  $theme->tableFullRow($Lang->getText('TrackerItemScmCommit.NoneFound'));
	} else {
		foreach ($arr as $item) {
			$files = $item->getScmCommitFiles();

			foreach ($files as $file) {
				$prevVersion = $file->getPrevVersion();
				$newVersion = $file->getNewVersion();

				$trow = array();
				if (!is_null($browser)) {
					$trow[] = '<a href="'.$browser->getBrowsePathUrl($file).'">'.$file->getModifiedFile().'</a>';
				} else {
					$trow[] = $file->getModifiedFile();
				}
				$trow[] = $item->getCommitDate();

				if (strtolower($newVersion) == 'none'  || strtolower($newVersion == 'deleted') || empty($newVersion) ||
					strstr($newVersion, '0000000')) {
					// This happens when the file is deleted. In that case, we don't have a diff.
					$trow[] = $prevVersion;
				} elseif ($prevVersion == 'none' || empty($prevVersion) || strstr($prevVersion, '0000000')) {
					$trow[] = '';
				} else {
					if (!is_null($browser)) {
						// Show diff link
						$trow[] = '<a href="'.$browser->getDiffUrl($file->getModifiedFile(), $prevVersion, $newVersion).'">'.$Lang->getText('TrackerItemScmCommit.DiffTo').' '.StringUtils::trimString($prevVersion, 10).'</a>';
					} else {
						$trow[] = StringUtils::trimString($prevVersion, 10);
					}
				}

				$trow[] = StringUtils::trimString($file->getNewVersion(), 10);
				$log = $item->getMessageLog();
				// Log is trimmed because CVS import logs may be huge
				$trow[] = StringUtils::trimString($log, 256);

				$user = $item->getUser();
				if ($user->isUndefinedUser()) {
					$trow[] = $Lang->getText('Common.UnknownUser');
				} else {
					$trow[] = $user->getUserLink();
				}
				$str_commits .= $theme->tableRow($trow);
			}
		}
	}
	return $str_commits;
}

function commitTable($Lang, $theme, $browser, $item) {

	if (is_array($item)) {
		$arr = $item;
	} else {
		$arr = array($item);
	}

	return commitTableHeader($Lang, $theme) .
			commitTableBody($Lang, $theme, $browser, $arr)  .
			$theme->tableBottom();
}

function createCommitsContentArray($Lang, $theme, $TrackerItem) {

	$ti_commits = $TrackerItem->getTrackerItemScmCommitsJoinScmCommit();
	$commits = array();
	foreach ($ti_commits as $ti_commit) {
		$commits[] = $ti_commit->getScmCommit();
	}

    $project = $TrackerItem->getTracker()->getProject();
	$repository = null;
	$browser = null;

    if (count($commits) > 0) {
        $repository = SCMRepositoryFactory::getRepository($project);
        $browser = getScmBrowser($repository);
    }

	$arr_commits = array();
	$arr_commits['html'] = commitTable($Lang, $theme, $browser, $commits);
	$arr_commits['count'] = sizeof($commits);

	return $arr_commits;
}
?>
