<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code shows a form for editing TrackerItem
//
require_once('lib/core/TagEntry.php');
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('plugins/tracker/lib/html/TrackerExtraFieldRenderer.php');
require_once('plugins/frs/lib/FrsRelease.php');
require_once('lib/html/UploadHandler.php');
require_once('lib/core/UserPreference.php');
require_once('TrackerItemCommits.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

require_once('plugins/tracker/lib/html/TrackerTemplate.php');
$theme = new TrackerTemplate();

$Project = GFUrl::getUrl()->getProject();
$TrackerItem = TrackerItemPeer::retrieveByPk(Validator::sanitizeInt($_REQUEST['tracker_item_id']));
$user = Session::getSession()->getUser();

$start = (isset($_REQUEST['start'])) ? intval($_REQUEST['start']) : 0;

if (!is_object($TrackerItem)) {
	throw new PermissionDeniedException();
}
$Tracker=$TrackerItem->getTracker();
if (!is_object($TrackerItem) || !$TrackerItem->userCanRead()) {
	throw new PermissionDeniedException();
}

//Get intrinsic fields settings
$intrinsic_fields = $TrackerItem->getTracker()->getIntrinsicFieldsSettings();

//if the Tracker is of Support type, get the autclose setting for this TI
$autoclose = $TrackerItem->getAutocloseEnabled();

$waiting_for = $TrackerItem->getWaitingFor();
if ($waiting_for == TrackerItem::WAITINGFOR_ASSIGNEE) {
  $waiting_for_opposite = TrackerItem::WAITINGFOR_SUBMITTER;
} else {// default select option is 'assignee'
  $waiting_for_opposite = TrackerItem::WAITINGFOR_ASSIGNEE;
}

// Make sure the tracker belongs to the current project
if ($Tracker->getProjectId() != $Project->getProjectId()) {
	throw new PermissionDeniedException();
}

if ($session->isLoggedIn()) {
	$IS_TECH=$Project->getPermission()->check(Tracker::SECTION, $Tracker->getTrackerId(), Tracker::PERMLEVEL_TECH);
	$IS_SUBMITTER = ($session->getUser()->getUserId() == $TrackerItem->getSubmittedBy());
} else {
	$IS_TECH=false;
	$IS_SUBMITTER=false;
}

$templates = array(
	'classic' => 'TrackerItemEdit-tpl.php',
	'tabbed' => 'TrackerItemEditTabs-tpl.php',
	'one-page' => 'TrackerItemEditTabs-tpl.php'
);

// Template selection logic

$template = '';
$default_template = 'classic';
$pref_key = 'ti-template-'.$TrackerItem->getTrackerId();

// Top priority - did the user select a specific template?
if (isset($_REQUEST['view'])) {
	$temp = $_REQUEST['view'];
	if (array_key_exists($temp, $templates)) {
		$template = $temp;
		if ($user instanceof User) {
			UserPreferencePeer::setUserPref($user->getUserId(), $pref_key, $template);
		}
	}
}

// No selection in the request -
// does the user have a preference set for this Tracker?
if ($template == '' && $user instanceof User) {
	$template = UserPreferencePeer::getUserPreferenceValue($user->getUserId(), $pref_key, '');
}

// No preference for the user -
// Is there one for the Tracker itself? (future feature)
if ($template == '') {
	$template = SettingPeer::GetSetting(Tracker::SECTION, $Tracker->getTrackerId(), 'edit-template', '');
}

if ($template == '' ||
	! array_key_exists($template, $templates)) {
	$template = $default_template;
}

$template_file = realpath(__DIR__).'/forms/'.$templates[$template];

if (! file_exists($template_file)) {
	$message = "Template file $template_file does not exist!";
	error_log($message);
	throw new Exception($message);
}
// END Template selection logic

$tefr = new TrackerExtraFieldRenderer($Tracker);

$start_date=((isset($_REQUEST['start_date'])) ? $_REQUEST['start_date'] : $TrackerItem->getOpenDate('M d Y'));
$end_date=((isset($_REQUEST['end_date'])) ? $_REQUEST['end_date'] : $TrackerItem->getCloseDate('M d Y'));

$uploadHandler = new UploadHandler(null, TrackerItem::SECTION, $TrackerItem->getPrimaryKey());

$params = array(
		'project' => $Project,
		'tracker' => $Tracker,
		'tracker_item' => $TrackerItem,
		'user' => $user,
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'action' => isset($action) ? $action : '',
		'title' => $Lang->getText('TrackerItem.EditPageTitle'),
		'enableUploads' => ($IS_TECH || $IS_SUBMITTER ? true : false),
		'uploadHandler' => $uploadHandler,
		'start_date' => $start_date,
		'end_date' => $end_date,
		'tefr' => $tefr,
        'tab-style' => ($template == 'tabbed') ? 'tabber' : 'inline',
		'activity' => ($template != 'classic') ? gatherActivityList($TrackerItem) : false,
		'related' => ($template != 'classic') ? gatherRelatedList($TrackerItem) : false
		);

include($template_file);

function gatherActivityList(TrackerItem $TrackerItem) {
	// Assemble the unified activity view data
	$activity_list = array();
	$activity_count = array();
	$all_users = array();

	// Messages = follow-ups
	$followups = $TrackerItem->getTrackerItemMessages();
	foreach ($followups as $followup) {
		$activity_list[] = array(
			'type' => 'followup',
			'title'=> 'Added a follow-up',
			'date' => $followup->getAddDate(),
			'user' => $followup->getUser(),
			'item' => $followup
		);
	}
	$activity_count['followup'] = count($followups);

	// SCM commits for this tracker item
	$tics = $TrackerItem->getTrackerItemScmCommitsJoinScmCommit();
	foreach ($tics as $tic) {
		$commit = $tic->getScmCommit();
		$activity_list[] = array(
			'type' => 'commit',
			'title'=> 'Added an SCM commit',
			'date' => $commit->getCommitDate(),
			'user' => $commit->getUser(),
			'item' => $commit
		);
	}
	$activity_count['commit'] = count($tics);


	// Add uploaded files to activity list
	$fs_files = FilesystemPeer::getRelatedFiles(TrackerItem::SECTION, $TrackerItem->getTrackerItemId());
	foreach ($fs_files as $file) {
		$activity_list[] = array(
			'type' => 'file',
			'title'=> 'Added a file',
			'date' => $file->getUploadDate(),
			'user' => $file->getUser(),
			'item' => $file
			);
	}
	$activity_count['file'] = count($fs_files);


	// Set up objects required for SCM display, if needed
	if (count($tics) > 0) {
		$project = $TrackerItem->getTracker()->getProject();
		$repository = SCMRepositoryFactory::getRepository($project);
		$browser = null;

		if (!is_null($repository)) {		// Project doesn't have a repository
			try {
				$browser = new RepositoryBrowser($repository);
			} catch (Exception $e) {
				// SCM other than CVS or SVN
				$browser = null;
			}
		}
	}

	// Audit trail - changes to this tracker item
	$audits = AuditTrailPeer::getAuditTrail(TrackerItem::SECTION, $TrackerItem->getTrackerItemId());
	// First, we need to group these by timestamp
	$audit_list = array();
	foreach ($audits as $audit) {
		// Don't include _created_ and _deleted_ entries
		if (substr($audit->getFieldName(), 0, 1) == '_') continue;

		// Sort audit records into date/time buckets (changesets)
		$key = $audit->getChangeDate();
		if (! array_key_exists($key, $audit_list)) {
			$audit_list[$key] = array(
				'type' => 'audit',
				'title'=> 'Updated this Tracker Item',
				'date' => $key,
				'user' => $audit->getUser(),
				'item' => array()
			);
		}
		$audit_list[$key]['item'][] = $audit;
	}
	// Now add the consolidated audit records as activity
	$activity_list = array_merge($activity_list, array_values($audit_list));
	$activity_count['audit'] = count($audit_list);


	// Time entry for this tracker item
	$time_entries = $TrackerItem->getTrackerItemTimes();
	foreach ($time_entries as $time_entry) {
		$activity_list[] = array(
			'type' => 'time',
			'title'=> 'Recorded time on this Tracker Item',
			'date' => $time_entry->getDateEntry(),
			'user' => $time_entry->getUser(),
			'item' => $time_entry
		);
	}
	$activity_count['time'] = count($time_entries);

	// After gathering all information, sort the entire list by date
	usort($activity_list, function($TrackerItem_a, $TrackerItem_b) {
		$a = $TrackerItem_a['date'];
		$b = $TrackerItem_b['date'];

		if ($a == $b) {
			return 0;
		}
		return ($a < $b) ? -1 : 1;
	});

	foreach ($activity_list as $activity) {
		if (array_key_exists('user', $activity)) {
			$u = $activity['user'];
			if ($u instanceof User) {
				$all_users[$u->getUnixName()] = $u;
			}
		}
	}

	$ret = array(
		'activity_list' => $activity_list,
		'activity_count' => $activity_count,
		'all_users' => 		$all_users
	);

	return $ret;
}

/**
 *
 * @param TrackerItem $TrackerItem  The TI being gathered for
 * @return array The list of related item array elements to be displayed
 */
function gatherRelatedList(TrackerItem $TrackerItem) {
	$related_list = array();
	$related_count = array();

	// This TI depends on...
	$depends_on = array();
	$temp = $TrackerItem->getTrackerItemRelationsRelatedByTrackerItemIdJoinTrackerItemRelatedByRelatedTrackerItemId();
	foreach ($temp as $rel) {
		$depends_on[] = $rel->getTrackerItemRelatedByRelatedTrackerItemId();
	}

	// Other TIs depend on this...
	$depended_on = array();
	$temp = $TrackerItem->getTrackerItemRelationsRelatedByRelatedTrackerItemIdJoinTrackerItemRelatedByTrackerItemId();
	foreach ($temp as $rel) {
		$depended_on[] = $rel->getTrackerItemRelatedByTrackerItemId();
	}
	$related_count['dependency'] = count($depended_on) + count($depends_on);
	$related_list['dependency'] = array(
		'depended_on'=> $depended_on,
		'depends_on' => $depends_on
	);

	// This TI is a duplicate of...
	$duplicates_of = array();
	$temp = $TrackerItem->getTrackerItemDuplicatesRelatedByDuplicateOfTrackerItemId();
	foreach ($temp as $rel) {
		$duplicates_of[] = $rel->getTrackerItemRelatedByTrackerItemId();
	}

	// Other TIs are duplicates of this...
	$duplicates = array();
	$temp = $TrackerItem->getTrackerItemDuplicatesRelatedByTrackerItemId();
	foreach ($temp as $rel) {
		$duplicates[] = $rel->getTrackerItemRelatedByDuplicateOfTrackerItemId();
	}
	$related_count['duplicate'] = count($duplicates_of) + count($duplicates);
	$related_list['duplicate'] = array(
		'duplicate-of' => $duplicates_of,
		'duplicates' => $duplicates
	);

	// This TI is associated to...
	$crit = new Criteria();
		$crit0 = $crit->getNewCriterion(AssociationPeer::SECTION, TrackerItem::SECTION);
		$crit1 = $crit->getNewCriterion(AssociationPeer::REF_ID, $TrackerItem->getTrackerItemId());
		// Perform AND at level 1 ($crit0 $crit1 )
		$crit0->addAnd($crit1);

		$crit2 = $crit->getNewCriterion(AssociationPeer::TO_SECTION, TrackerItem::SECTION);
		$crit3 = $crit->getNewCriterion(AssociationPeer::TO_REF_ID, $TrackerItem->getTrackerItemId());
		// Perform AND at level 1 ($crit2 $crit3 )
		$crit2->addAnd($crit3);

		// Perform OR at level 0 ($crit0 $crit2 )
		$crit0->addOr($crit2);

		$crit->add($crit0);

	$assoc = AssociationPeer::doSelect($crit);
	$assoc_array = array();
	foreach ($assoc as $assoc_item) {
		$key = $assoc_item->getSection().'-'.$assoc_item->getRefId();
		if ($assoc_item->getSection() == TrackerItem::SECTION &&
			$assoc_item->getRefId() == $TrackerItem->getTrackerItemId()) {
			$key = $assoc_item->getToSection().'-'.$assoc_item->getToRefId();
		}
		if (! array_key_exists($key, $assoc_array)) {
			$assoc_array[$key] = explode('-', $key);
		}
	}
	$related_count['association'] = count($assoc_array);
	$related_list['association'] = array_values($assoc_array);

	$tags = TagEntryPeer::getTagValues(TrackerItem::SECTION, $TrackerItem->getTrackerItemId());
	$related_count['tag'] = count($tags);
	$related_list['tag'] = $tags;

	return array(
		'related_list' => $related_list,
		'related_count' => $related_count
	);
}
?>
