<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a TrackerItem and saves it to the 
// database
//
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('plugins/tracker/lib/TrackerItemMentionNotification.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
require_once('core/FilesystemPeer.php');
require_once('plugins/tracker/lib/TrackerItemTime.php');
require_once('plugins/tracker/lib/TrackerItemRelation.php');
require_once('plugins/tracker/lib/TrackerItemDuplicate.php');
require_once('plugins/tracker/lib/TrackerItemMessage.php');
require_once('lib/html/UploadHandler.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

//force login
$sess = Session::getSession();
if (!$sess->isLoggedIn()) {
	throw new NotLoggedInException();
}
$tracker_item_id = (isset($_REQUEST['tracker_item_id'])) ? intval($_REQUEST['tracker_item_id']) : 0;
$start = (isset($_REQUEST['start'])) ? intval($_REQUEST['start']) : 0;
$Project = GFUrl::getUrl()->getProject();
$stringUtils = new StringUtils();

$TrackerItem = TrackerItemPeer::retrieveByPk($tracker_item_id);
if (!$TrackerItem || !is_object($TrackerItem)) {
	throw new Exception('No Such Item');
}
$Tracker = $TrackerItem->getTracker();
//validate that this is a legitimate trackeritem for this tracker and user can add items
if (!$TrackerItem || !is_object($TrackerItem) || $Tracker->getProject()->getProjectId() != $Project->getProjectId()) {
	throw new PermissionDeniedException();
}

// Make sure the tracker belongs to the current project
if ($Tracker->getProjectId() != $Project->getProjectId()) {
	throw new PermissionDeniedException();
}

$intrinsic_fields = $TrackerItem->getTracker()->getIntrinsicFieldsSettings();

//
//	Determine level of permission they have
//
$ADMIN = $Project->getPermission()->check(Tracker::SECTION, $Tracker->getTrackerId(), Tracker::PERMLEVEL_ADMIN);
$TECHONLY = !$ADMIN && $Project->getPermission()->check(Tracker::SECTION, $Tracker->getTrackerId(), Tracker::PERMLEVEL_TECH);
$SUBMITTERONLY = !$ADMIN && !$TECHONLY && ($sess->getUser()->getUserId() == $TrackerItem->getSubmittedBy());
$READONLY = !$ADMIN && !$TECHONLY && !$SUBMITTERONLY && $TrackerItem->userCanRead();

if (!$READONLY && !$ADMIN && !$TECHONLY && !$SUBMITTERONLY) {
	throw new PermissionDeniedException();
}


$errors = array();
$language = Language::getSessLang();

//
// TODO Validate the values passed by $_POST. Store the errors in $error like
// $errors["field_name"] = error_description
//

$uploadHandler = new UploadHandler(null, TrackerItem::SECTION, $TrackerItem->getPrimaryKey());

try {
	$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
	$con->begin();

	$uploadHandler->handleDeletedFiles();

	if (isset($_POST['deletefile']) && is_array($_POST['deletefile'])) {
		foreach ($_POST['deletefile'] as $file_id) {
			$TrackerItem->deleteAttachment($file_id);
		}
	}

	// Set up info for user wall mentions, if needed
	$user = $sess->getUser();
	$userUrl = GFUrl::getUrl()->userURL($user);
	$userlink = "<a href=\"{$userUrl}\" target=\"_new\" title=\"User profile for {$user->getUnixName()}\">{$user->getUnixName()}</a>";
	$tracker_item_id = $TrackerItem->getTrackerItemId();
	$tracker_item_url = GFUrl::getUrl()->currentBaseURL('TrackerItemEdit', array('tracker_item_id'=>$tracker_item_id));
	$link = "<a href=\"{$tracker_item_url}\" title=\"View details\" >Tracker Item {$tracker_item_id}</a>";

  //Updating all waiting for fields
  $waiting_for = Validator::sanitizeInt($_POST['waiting_for']);
  if ($waiting_for != TrackerItem::WAITINGFOR_SUBMITTER) {
    $waiting_for = TrackerItem::WAITINGFOR_ASSIGNEE;// default is assignee
  }
  $TrackerItem->setWaitingFor($waiting_for);

  $followUpAdded = false;
	//
	//	Add message
	//
	if (isset($_POST['body']) && strlen($_POST['body']) > 0) {
		$tim = new TrackerItemMessage();
		//$TrackerItem->addTrackerItemMessage($tim);
		$tim->setTrackerItemId($TrackerItem->getTrackerItemId());
		$tim->setBody($stringUtils->cleanHtml($_POST['body']));
		$tim->setSubmittedBy($sess->getUser()->getUserId());
		$tim->save();
		$followUpAdded = true;

    		// Check/send @mentions
		$to_users = StringUtils::getMentionedUsers($tim->getBody());
		if (count($to_users) > 0) {
			$msg = $tim->getBody();
			if (strlen($msg) > 140) {
				//$msg = substr($msg, 140).'...';
			}
			$wall_comment = "$userlink mentioned you in $link: <br /><br />$msg";
			//error_log("TrackerItemEditAction posting to walls for $link, mentions: " . explode(', ', $to_users));
			ForumPeer::sendUserWallMentions($to_users, $user->getUserId(), $wall_comment, '', false);

			//Queue notifications

			foreach ($to_users as $key => $unix_name) {
				$u = UserPeer::retrieveByUnixName($unix_name);
				if (is_null($u)) {
					continue;
				}
				if (!$Tracker->userIsMonitoring($u) && !$TrackerItem->userIsMonitoring($u)) {
					$tiMentionNotification = new TrackerItemMentionNotification($u, $tim, $user);
					NotificationManager::getManager()->queue($tiMentionNotification);
				}
			}
		}
	}
	if ($READONLY) {
		//can't do anything except add message. Quit here
		$TrackerItem->unprivilegedSave();
		$con->commit();
		GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerItemBrowse', array('feedback' => $language->getText('TrackerItem.EditFeedback', array($TrackerItem->getTrackerItemId())), 'tracker_id' => $Tracker->getTrackerId())));
		exit;
	}
	//
	//	handle the files
	//
	if (array_key_exists('addfile', $_FILES)) {
		for ($i = 0; $i < 5; $i++) {
			$error = $_FILES['addfile']['error'][$i];
			$filename = $_FILES['addfile']['name'][$i];

			if ($error > 0 && $error != 4) {
				$fileerrors = StringUtils::getFileUploadErrorString($error, $filename);
				throw new InvalidDataException($fileerrors);
				continue;
			}

			$tmp_name = $_FILES['addfile']['tmp_name'][$i];
			if (!is_uploaded_file($tmp_name)) {
				continue;
			}
			$size = $_FILES['addfile']['size'][$i];
			$type = $_FILES['addfile']['type'][$i];
			$fs = new Filesystem();
			$fs->setSection(TrackerItem::SECTION);
			$fs->setRefId($TrackerItem->getTrackerItemId());

			$fs->setFileName($filename);
			//$fs->setFileNameSafe();
			$fs->setFileType($type);
			$fs->setFileSize($size);
			$fs->setPostedBy($sess->getUser()->getUserId());
//TODO audit trail
			$fs->save();
			$fs->setFileDataFromPath($tmp_name);
		}
	}
	if ($SUBMITTERONLY) {
		//can't do anything except add message and add files
		$TrackerItem->unprivilegedSave();
		$con->commit();
		GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerItemBrowse', array('feedback' => $language->getText('TrackerItem.EditFeedback', array($TrackerItem->getTrackerItemId())), 'tracker_id' => $Tracker->getTrackerId())));
		exit;
	}

	//$TrackerItem->setTrackerId( Validator::sanitizeInt($_POST['tracker_id']) );
	//$TrackerItem->setStatusId( Validator::sanitizeInt($_POST['status_id']) );
	$TrackerItem->setPriority(Validator::sanitizeInt($_POST['priority']));

    if ($intrinsic_fields[Tracker::TRACKER_FIELD_SPRINT_ID]['on_submit']) {
      $sprint_id = isset($_POST['tracker_sprint']) ? Validator::sanitizeInt($_POST['tracker_sprint']) : null;
      $NewSprint = TrackerSprintPeer::retrieveByPk($sprint_id,$Project->getProjectId());

      if ($TrackerItem->canChangeSprintTo($NewSprint) == false) {
          throw new PermissionDeniedException();
      }
      if ($sprint_id == TrackerSprint::TRACKER_SPRINT_NONE)
      {
          //none
          $TrackerItem->setSprintId(null);
      } else
      {
          $TrackerItem->setSprintId($sprint_id);
      }
    }
	//$TrackerItem->setSubmittedBy( Validator::sanitizeInt($_POST['submitted_by']) );
	//$TrackerItem->setOpenDate( Validator::sanitizeDate($_POST['open_date']) );
	//$TrackerItem->setCloseDate( Validator::sanitizeDate($_POST['close_date']) );
	if ($Tracker->userIsAdmin() && $Tracker->getProject()->isUserMember($TrackerItem->getSubmittedBy())) {
		$TrackerItem->setSubmittedBy( Validator::sanitizeInt($_POST['submitted_by']) );
	}
	if ($Tracker->getDatatype() == Tracker::DATATYPE_TASK) {
		if (isset($_POST['start_date']) && strlen($_POST['start_date']) > 5) {
			$TrackerItem->setOpenDate(Validator::sanitizeDate($_POST['start_date']));
		}
		if (isset($_POST['end_date']) && strlen($_POST['end_date']) > 5) {
			$TrackerItem->setCloseDate(Validator::sanitizeDate($_POST['end_date']));
		}
	}
	$TrackerItem->setSummary(html_entity_decode($_POST['summary']));
	$TrackerItem->setLastModifiedDate(time());
	$TrackerItem->setLastModifiedBy($sess->getUser()->getUserId());
	if (isset($_POST['details'])) {
		$TrackerItem->setDetails($stringUtils->cleanHtml($_POST['details']));
	}
	if (!isset($_POST['assigned_to'])) {
		$assigned_to = array(User::UNDEFINED_USER_ID);
	} else {
		$assigned_to = $_POST['assigned_to'];
		// Remove nobody user if set and its not the only user assigned.
		$remove_assigned_to = array_search(User::UNDEFINED_USER_ID, $assigned_to);
		if ($remove_assigned_to !== false && count($assigned_to) > 1) {
			array_splice($assigned_to, $remove_assigned_to, 1);
		}
	}
	if ($TrackerItem->getTracker()->userIsAdmin()) {
		$TrackerItem->updateAssignees($assigned_to);

		// Check/send assignee notifications
		$to_users = array();
		//new_field_values propery in TrackerItem will be set only when a new assignee as been added
		if(isset($TrackerItem->new_field_values['assignee']) && count($TrackerItem->new_field_values['assignee'])) {

			foreach ($assigned_to as $assignee) {
				if (strlen($assignee) > 0   // skip empty item(?)
					&& $assignee != '100'  // skip Nobody user
					&& $assignee != $user->getUserId()  // skip yourself
					&& !in_array($assignee, $to_users)) {  // skip duplicates(?)
				$to_users[] = $assignee;
			}
		}
		}
		if (count($to_users) > 0) {
			$wall_comment = "$userlink assigned $link to you";
			//error_log("TrackerItemEditAction posting to walls for $link, assignees: " . implode(', ', $to_users));
			ForumPeer::sendUserWallMentions($to_users, $user->getUserId(), $wall_comment, '', false);
		}
	}

	if (array_key_exists('extra_field', $_POST)) {
		$TrackerItem->updateExtraFields($_POST['extra_field']);
	}

	if (isset($_REQUEST['related_items']) && !empty($_REQUEST['related_items'])) {
		$rels = explode(',', $_REQUEST['related_items']);
		$parsed_rels = array();
		for ($i = 0; $i < count($rels); $i++) {
			$id = intval($rels[$i]);
			if (!$id)
				continue;
			$parsed_rels[] = $id;
		}
		$TrackerItem->addRelations($parsed_rels);
	}
	if (isset($_REQUEST['del_relation']) && !empty($_REQUEST['del_relation'])) {
		$TrackerItem->deleteRelations($_REQUEST['del_relation']);
	}
	if (isset($_REQUEST['duplicate_items']) && !empty($_REQUEST['duplicate_items'])) {
		$dups = explode(',', $_REQUEST['duplicate_items']);
		$parsed_dups = array();
		for ($i = 0; $i < count($dups); $i++) {
			$id = intval($dups[$i]);
			if (!$id)
				continue;
			$parsed_dups[] = $id;
		}
		$TrackerItem->addDuplicates($parsed_dups);
	}
	if (isset($_REQUEST['del_duplicate']) && !empty($_REQUEST['del_duplicate'])) {
		$TrackerItem->deleteDuplicates($_REQUEST['del_duplicate']);
	}
	ActionManager::notify('TRACKER_ITEM_EDIT_ACTION', $TrackerItem);

	$TrackerItem->save(null, TRUE, $followUpAdded);

	$con->commit();

	//if TI save was successful, verify if the tracker is of support type and save the autoclose setting
	if ($TrackerItem->getTracker()->getDatatype() == Tracker::DATATYPE_SUPPORT) {
		$autoclose = isset($_REQUEST['disable_autoclose']) ? false : true;
		$TrackerItem->setAutocloseEnabled($autoclose);
	}

	$feedback = $language->getText('TrackerItem.EditFeedback', array($TrackerItem->getTrackerItemId()));
	GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerItemBrowse', array('feedback' => $feedback, 'tracker_id' => $Tracker->getTrackerId(), 'start' => $start)));
} catch (InvalidDataException $ex) {
	$con->rollback();
	$feedback = '* ' . implode('<br />* ', $ex->getErrors());
	include('TrackerItemEdit.php');
	//GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerItemEdit', array('feedback' => $feedback,'tracker_id'=>$Tracker->getTrackerId(), 'tracker_item_id'=>$tracker_item_id)));
}
?>
