<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code creates a new TrackerQuery and saves it to the 
// database
//
require_once('plugins/tracker/lib/TrackerQuery.php');
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
require_once('lib/exceptions/NotLoggedInException.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$session = Session::getSession();

//must be logged in
if (!$session->isLoggedIn()) {
	throw new NotLoggedInException();
}

$Project = GFUrl::getUrl()->getProject();
$Tracker = TrackerPeer::retrieveByPk($_REQUEST['tracker_id']);

$TrackerQuery = new TrackerQuery();
$TrackerQuery->setTracker($Tracker);
$TrackerQuery->setUser($session->getUser());
$TrackerQuery->setIsShared( isset($_REQUEST['is_shared']) ? true : false );

$errors = array();
$language = Language::getSessLang();

//TODO: do some verification, add error strings to $errors

if (count($errors) == 0) {
	$TrackerQuery->setQueryName( Validator::sanitizeString($_POST['query_name']) );
	$assigned_to=(( isset($_POST['assigned_to']) ) ? $_POST['assigned_to'] : array() );
	$extra_field=(( isset($_POST['extra_field']) ) ? $_POST['extra_field'] : array() );
	$moddate=(( isset($_POST['moddatestart']) && isset($_POST['moddateend'])  ) ? $_POST['moddatestart'].TrackerItemQuery::DATE_RANGE_DELIMITER.$_POST['moddateend'] : '' );
	$opendate=(( isset($_POST['opendatestart']) &&  isset($_POST['opendateend']) ) ? $_POST['opendatestart'].TrackerItemQuery::DATE_RANGE_DELIMITER.$_POST['opendateend'] : '' );
	$closedate=(( isset($_POST['closedatestart']) && isset($_POST['closedateend']) ) ? $_POST['closedatestart'].TrackerItemQuery::DATE_RANGE_DELIMITER.$_POST['closedateend'] : '' );
	$sortord=(( isset($_POST['sortord']) ) ? $_POST['sortord'] : '' );
	$sortcol=(( isset($_POST['sortcol']) ) ? $_POST['sortcol'] : '' );
	$fieldlist=(( isset($_POST['fieldlist']) ) ? $_POST['fieldlist'] : array() );
	$priority=(( isset($_POST['priority']) ) ? $_POST['priority'] : array() );
        $sprint=(( isset($_POST['tracker_sprint']) ) ? $_POST['tracker_sprint'] : array() );
        $waiting_for=(( isset($_POST['waiting_for']) ) ? $_POST['waiting_for'] : null );
	$efs_display = ((isset($_POST['efs_display'])) ? $_POST['efs_display'] : array());


	try {
		$TrackerQuery->setQueryFields($assigned_to, $extra_field, $moddate, $opendate, $closedate, $sortord, $sortcol, $fieldlist, $priority, $sprint, $waiting_for);
		$TrackerQuery->setDisplayExtraFields($efs_display);
		$TrackerQuery->save();
		GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerQueryBrowse'), array('feedback' => $language->getText('TrackerQuery.AddFeedback'),'tracker_id'=>$Tracker->getTrackerId()));
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());		// attach object errors to validation errors
	}

}
	
if (count($errors) > 0) {
	$feedback = '* '.implode('<br />* ', $errors);
	include('TrackerQueryAdd.php');
} else {
}

?>
