<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a TrackerQuery and saves it to the 
// database
//
require_once('plugins/tracker/lib/TrackerQuery.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

//must be logged in
if (!$session->isLoggedIn()) {
	throw new NotLoggedInException();
}

$Project = GFUrl::getUrl()->getProject();
$Tracker = TrackerPeer::retrieveByPk($_REQUEST['tracker_id']);

$crit = new Criteria();
if (! $Tracker->userIsAdmin()) {
    // [#9315] Only filter on user id if the user is not admin for this tracker
    // Any tracker admin can edit any shared query
    $crit->add(TrackerQueryPeer::USER_ID, $session->getUser()->getID());
}
$crit->add(TrackerQueryPeer::TRACKER_QUERY_ID,$_REQUEST['tracker_query_id']);
$arr = $Tracker->getTrackerQuerys($crit);
$TrackerQuery=$arr[0];
if (!is_object($TrackerQuery)) {
	throw new PermissionDeniedException();
}

$errors = array();
$language = Language::getSessLang();

//
// TODO Validate the values passed by $_POST. Store the errors in $error like
// $errors["field_name"] = error_description
//

if (count($errors) == 0) {
//	$TrackerQuery->setTrackerId( Validator::sanitizeInt($_POST['tracker_id']) );
//	$TrackerQuery->setUserId( Validator::sanitizeInt($_POST['user_id']) );
	$assigned_to=(( isset($_POST['assigned_to']) ) ? $_POST['assigned_to'] : array() );
	$extra_field=(( isset($_POST['extra_field']) ) ? $_POST['extra_field'] : array() );
	$moddate=(( isset($_POST['moddatestart']) && isset($_POST['moddateend'])  ) ? $_POST['moddatestart'].TrackerItemQuery::DATE_RANGE_DELIMITER.$_POST['moddateend'] : '' );
	$opendate=(( isset($_POST['opendatestart']) &&  isset($_POST['opendateend']) ) ? $_POST['opendatestart'].TrackerItemQuery::DATE_RANGE_DELIMITER.$_POST['opendateend'] : '' );
	$closedate=(( isset($_POST['closedatestart']) && isset($_POST['closedateend']) ) ? $_POST['closedatestart'].TrackerItemQuery::DATE_RANGE_DELIMITER.$_POST['closedateend'] : '' );
	$sortord=(( isset($_POST['sortord']) ) ? $_POST['sortord'] : '' );
	$sortcol=(( isset($_POST['sortcol']) ) ? $_POST['sortcol'] : '' );
	$fieldlist=(( isset($_POST['fieldlist']) ) ? $_POST['fieldlist'] : array() );
	$priority=(( isset($_POST['priority']) ) ? $_POST['priority'] : array() );
	$sprint=(( isset($_POST['tracker_sprint']) ) ? $_POST['tracker_sprint'] : array() );
	$waiting_for=(( isset($_POST['waiting_for']) ) ? $_POST['waiting_for'] : null );
	$efs_display=(( isset($_POST['efs_display']) ) ? $_POST['efs_display'] : array() );

	try {
                //error_log("TrackerQueryEditAction setting date criteria fields: ($moddate), ($opendate), ($closedate)");
		$TrackerQuery->setQueryFields($assigned_to, $extra_field, $moddate, $opendate, $closedate, $sortord, $sortcol, $fieldlist, $priority, $sprint, $waiting_for);
		$TrackerQuery->setDisplayExtraFields($efs_display);
		$TrackerQuery->setQueryName( Validator::sanitizeString($_POST['query_name']) );
		$TrackerQuery->setIsShared( isset($_REQUEST['is_shared']) ? true : false);
	//	$TrackerQuery->setIsSelected( Validator::sanitizeBool($_POST['is_selected']) );
	
		$TrackerQuery->save();
		GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerQueryBrowse'), array('feedback' => $language->getText('TrackerQuery.EditFeedback'),'tracker_id'=>$Tracker->getTrackerId()));
	} catch(InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());
		$feedback = implode('<br />', $errors);
		include('TrackerQueryEdit.php');
	}
} else {
	$feedback = implode('<br />', $errors);
	include('TrackerQueryEdit.php');
}

?>
