<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code lists TrackerItem
//
require_once('plugins/tracker/lib/TrackerItemPeer.php');
require_once('plugins/tracker/lib/TrackerItemSavedQuery.php');
require_once('plugins/tracker/lib/TrackerItemQuickQuery.php');
require_once('plugins/tracker/lib/TrackerQuery.php');
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('lib/ExcelXML.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

$Project = GFUrl::getUrl()->getProject();
$Tracker = TrackerPeer::retrieveByPk($_REQUEST['tracker_id']);

if (isset($_GET['tracker_query_id'])) {
	if (!$session->isLoggedIn()) {
		throw new PermissionDeniedException();
	}
	$tsq = TrackerQueryPeer::retrieveByPk($_GET['tracker_query_id']);
	if (!$tsq->getIsShared() && $tsq->getUserId() != $session->getUser()->getUserId()) {
		throw new PermissionDeniedException();
	}
	$tq = new TrackerItemSavedQuery($Tracker, $tsq, $session->getUser());
} else {
	$tq = new TrackerItemQuickQuery($Tracker);
}

if (is_a($tq,'TrackerItemSavedQuery')) {
	$filename = StringUtils::removeInvalidChars(str_replace(' ', '_',$tsq->getQueryName()));
} else {
	$filename = 'quickquery';
}

$excel = new ExcelXML($filename);

$totalcount=$tq->getTrackerItemCount();
$arr=$tq->getTrackerItems();

$header = array();
$header[] = $Lang->getText('TrackerItem.TrackerItemIdName');
$header[] = $Lang->getText('TrackerItem.SummaryName');
if ($tq->showColumn('priority'))
	$header[] = $Lang->getText('TrackerItem.PriorityName');
if ($tq->showColumn('assigned_to'))
	$header[] = $Lang->getText('TrackerItem.AssignedTo');
if ($tq->showColumn('submitted_by'))
	$header[] = $Lang->getText('TrackerItem.SubmittedByName');
	
$efs=$Tracker->getTrackerExtraFields();
usort($efs, array($tq,'sortByDisplayOrder'));
foreach ($efs as $ef) {
	$efcache[$ef->getTrackerExtraFieldId()]=$ef;
	if ($tq->showColumn($ef->getId())) {
		$header[] = $ef->getFieldName();
	}
}

if ($tq->showColumn('open_date'))
	$header[] = $Lang->getText('TrackerItem.OpenDateName');
if ($tq->showColumn('close_date'))
	$header[] = $Lang->getText('TrackerItem.CloseDateName');
if ($tq->showColumn('last_modified_date'))
	$header[] = $Lang->getText('TrackerItem.LastModifiedDateName');

$excel->setHeader($header);

for ($i=0; $i<count($arr); $i++) {
	$excel->openRow();
	$excel->setCellNumber($arr[$i]->getTrackerItemId());
	$excel->setCellString($arr[$i]->getSummary());
	if ($tq->showColumn('priority'))
		$excel->setCellNumber($arr[$i]->getPriority());
	if ($tq->showColumn('assigned_to')) {
		$asses = $arr[$i]->getTrackerItemAssignees();
		$assignees = array();
		foreach ($asses as $ass) {
			$assignees[] = trim($ass->getUser()->getUnixName());
		}
		$excel->setCellString(implode(':', $assignees));
	}
	if ($tq->showColumn('submitted_by'))
		$excel->setCellString(UserPeer::retrieveByPk($arr[$i]->getSubmittedBy())->getUnixName());

	// Begin Extra Fields //
	$colarray=array();
	$efdatas=$arr[$i]->getTrackerExtraFieldDatas();
	foreach($efdatas as $efdata) {
		//Members type needs an special treatment
		$field_type = $efcache[$efdata->getTrackerExtraFieldId()]->getFieldType();
		if($field_type == TrackerExtraField::TYPE_MEMBERS){
			$member = UserPeer::retrieveByPk($efdata->getFieldData());
			$colarray[$efdata->getTrackerExtraFieldId()][] = $member->getUnixName();
		} else {
			$colarray[$efdata->getTrackerExtraFieldId()][]=$efcache[$efdata->getTrackerExtraFieldId()]->translateFieldDataToName($efdata->getFieldData());
		}
	}
	reset($efs);
	foreach ($efs as $ef) {
		if ($tq->showColumn($ef->getId())) {
			if (isset($colarray[$ef->getTrackerExtraFieldId()]) && count($colarray[$ef->getTrackerExtraFieldId()]) > 1) {
				$excel->setCellString(implode(':',$colarray[$ef->getTrackerExtraFieldId()]));
			} else {
				if (isset($colarray[$ef->getTrackerExtraFieldId()])) {
					if ($ef->getFieldType() == TrackerExtraField::TYPE_DATE) {
						$excel->setCellDate(date('Y-m-d', strtotime($colarray[$ef->getTrackerExtraFieldId()][0])));
					} else {
						$excel->setCellString($colarray[$ef->getTrackerExtraFieldId()][0]);
					}
				} else {
						$excel->setCellString('None');
				}
			}
		}
	}
	// End Extra Fields //

	if ($tq->showColumn('open_date'))
		$excel->setCellDate($arr[$i]->getOpenDate('Y-m-d'));
	if ($tq->showColumn('close_date'))
		$excel->setCellDate($arr[$i]->getCloseDate('Y-m-d'));
	if ($tq->showColumn('last_modified_date'))
		$excel->setCellDate($arr[$i]->getLastModifiedDate('Y-m-d'));

	$excel->closeRow();
}

header("Content-Type: application/vnd.ms-excel");
header("Expires: 0");
header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
header("content-disposition: attachment;filename=$filename.xls");
echo $excel->getXML();
?>
