<?php
// NOTE: This script has dependencies!
// $queries A list of TrackerQuery objects that the logged-in user can select from.
// $Tracker The currently-viewed GForge tracker
// $session (optional) The current user's session

if (GFConf::get('tracker.enableTicklers') && Session::getSession()->isLoggedIn()) {

    if (! $session) {
        $session = Session::getSession();
    }
    $tracker_id = $Tracker->getTrackerId();
    $add_url = GFUrl::getUrl()->currentBaseURL('TicklerAddAction', array('tracker_id' => $tracker_id));
    $preference_prefix = "ti-tq-{$tracker_id}";
    $tickler_prefs = UserPreferencePeer::getUserPrefsByUserAndPrefix(Session::getSession()->getUser()->getUserId(), $preference_prefix);

    $id_list = array();
    foreach ($tickler_prefs as $pref) {
        $id_list[] = intval($pref->getPreferenceValue());
    }

    $tickler_queries = TrackerQueryPeer::retrieveByPKs($id_list);

    $new_queries = $queries;

    echo "<div class=\"ti-tickler-list\">\n";

    foreach ($tickler_queries as $tickler_query) {
        try {
            if (!$tickler_query->getIsShared() && $tickler_query->getUserId() != $session->getUser()->getUserId()) {
                // Not shared, not theirs, skip it
                throw new PermissionDeniedException();
            }
            $tickler_saved_query = new TrackerItemSavedQuery($tickler_query->getTracker(),$tickler_query, $session->getUser());

            $tickler_full_title = $tickler_query->getQueryName();
            $tickler_title = $tickler_full_title;  //StringUtils::mbtrimString($tickler_full_title, 15);
            $ti_count = $tickler_saved_query->getTrackerItemCount();
            $query_url = GFUrl::getUrl()->currentBaseURL('TrackerItemBrowse', array('tracker_id' => $tracker_id, 'tracker_query_id' => $tickler_query->getPrimaryKey()));
            $remove_url = GFUrl::getUrl()->currentBaseURL('TicklerDeleteAction', TrackerPlugin::SECTION);
?>
	<div class="ti-tickler">
        <a href="<?php echo $query_url ?>" title="<?php echo $tickler_full_title ?>">
            <div class="tickler-title">&nbsp;<?php echo $tickler_title ?>&nbsp;</div>
            <div class="tickler-count">&nbsp;<?php echo $ti_count ?>&nbsp;</div>
            <div class="tickler-remove">
                <form action="<?php echo $remove_url ?>" method="POST">
                    <input type="hidden" name="tracker_id" value="<?php echo $tracker_id ?>" />
                    <input type="hidden" name="tracker_query_id" value="<?php echo $tickler_query->getPrimaryKey() ?>" />
                    <input type="submit" name="submit" value="X" title="<?php echo Language::getSessLang('Tracker.RemoveTickler') ?>" />
                </form>
            </div>
        </a>
	</div>
<?php
        } catch (\Exception $ex) {
            error_log("Exception building a TI tickler ({$tickler_query->getQueryName()}, ID {$tickler_query->getPrimaryKey()}): {$ex->getMessage()}");
        }
        // Remove this query from the list of remaining choices for a new tickler
        for ($i = count($new_queries) - 1; $i > -1; $i--) {
            $q = $new_queries[$i];
            if (! $q || $q->getPrimaryKey() == $tickler_query->getPrimaryKey()) {
                unset($new_queries[$i]);
            }  // if (PK)
        }  // for (new_queries)
    } // foreach (query)

    // Only include queries for this tracker
    $new_queries = array_filter($new_queries, function($item) use($tracker_id) { 
        return ($item->getTracker()->getTrackerId() == $tracker_id);
    });
?>
    <div class="ti-tickler ti-tickler-add">
        <div class="tickler-title">Add a Tickler</div>
        <div class="tickler-form">
            <form method="POST" action="<?php echo $add_url ?>">
                <input type="hidden" name="tracker_id" value="<?php echo $tracker_id ?>" />
                <select name="tracker_query_id" style="width: 7em;">
                    <option value="0"><?php echo Language::getSessText('Tracker.Query') ?></option>
                    <?php foreach ($new_queries as $nq) {
                        echo "<option value=\"{$nq->getPrimaryKey()}\" title=\"{$nq->getQueryName()}\">{$nq->getQueryName()}</option>\n";
                    } ?>
                </select>
                <input type="submit" name="submit" value="<?php echo Language::getSessText('submit.add') ?>" />
            </form>
        </div>
    </div>
</div>
<script type="text/javascript">
    $(document).ready(function() {
        $('.ti-tickler').not('.ti-tickler-add').hover(function() {
            $(this).addClass('over');
        }, function() {
            $(this).removeClass('over');
        });
    });
</script>
<?php
} // if (enableTrackers)
