<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('plugins/tracker/lib/TrackerSprintPeer.php');
require_once('plugins/tracker/lib/TrackerSprint.php');
require_once('plugins/tracker/lib/html/TrackerSprintTemplate.php');
require_once('html/Paginator.php');

$session = Session::getSession();
$user = $session->getUser();

/** @var Project $Project */
$Project = GFUrl::getUrl()->getProject();
$preference_key = 'tk_sp_filter-'.$Project->getProjectId();

$theme = new TrackerSprintTemplate();
$lang = Language::getSessLang();
$params = array(
        'project'   => $Project,
        'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
        'title'    => 'Browse',
);
$theme->header($params);
$isSprintAdmin = $Project->getPermission()->check(TrackerSprint::SECTION, $Project->getProjectId(), TrackerSprint::PERMLEVEL_ADMIN)|| $Project->userIsAdmin() || Session::getSession()->getUser()->isSiteAdmin();
//Columns to be used in pagination sort
$columns = array(
    'sprint_progression'    => $lang->getText('TrackerSprint.SprintNameName'),
    'sprint_status'         => $lang->getText('TrackerSprint.SprintStatusName'),
    'sprint_completion'     => $lang->getText('TrackerSprint.SprintCompletionName'),
	  'sprint_start'          => $lang->getText('TrackerSprint.SprintStartName'),
	  'sprint_end'            => $lang->getText('TrackerSprint.SprintEndName'),
);
if($isSprintAdmin){
	$columns []= $lang->getText("TrackerSprint.ActionColumn");
}

$statuses = TrackerSprint::getStatuses();
$status_options = array_slice($statuses, 0, 2, true) + array(
	TrackerSprint::STATUS_IN_PROGRESS.'|'.TrackerSprint::STATUS_NOT_STARTED => $lang->getText('TrackerSprint.StatusInProgressOrNotStarted')
) + array_slice($statuses, 2, count($statuses)-2, true);
$sprint_status = 100;  // default value

$filter_from_pref = false;
if (array_key_exists('sprint_status', $_REQUEST)) {
	$sprint_status_input = $_REQUEST['sprint_status'];
	if ($_REQUEST['sprint_status'] === '') {
		$user->deletePreference($preference_key);		
	}
} else {
	$saved_filter = $user->getPreference($preference_key);
	if ($saved_filter) {
		try {
			$saved_filter = unserialize($saved_filter);
			$sprint_status_input = $saved_filter['sprint_status'];
			$filter_from_pref = true;
		} catch (Exception $ex) {
			error_log("Caught an exception while unserializing query prefs for {$user->getUnixName()} ($preference_key): {$ex->getMessage()}");
			$user->deletePreference($preference_key);
			$sprint_status_input = '';
		}
	} else {
		$sprint_status_input = '';
	}
}

// Don't look at non-integer (e.g., empty) sprint_status
$status_filter = array();
if ($sprint_status_input !== '') {
	$selected_status = array();
	foreach (explode('|', $sprint_status_input) as $status) {
		$sanitized_status = Validator::sanitizeInt($status);
		if (array_key_exists($sanitized_status, $statuses)) {
		  $status_filter[] = "tracker_sprint.sprint_status = $sanitized_status";
		  $selected_status[] = $sanitized_status;
		}
	}
}

$ti_status_open = Tracker::STATUS_OPEN;

$sql = "SELECT 
  tracker_sprint.sprint_id, tracker_sprint.sprint_name, 
  tracker_sprint.sprint_status, tracker_sprint.sprint_progression,
  COALESCE(open_ti.open_count, 0) as open_count,
  COALESCE(total_ti.total_count, 0) as total_count,
  tracker_sprint.sprint_start, tracker_sprint.sprint_end
FROM
    tracker_sprint
LEFT JOIN
    (select sprint_id, count(*) as open_count from tracker_item where status_id = {$ti_status_open} group by sprint_id) open_ti
ON
    tracker_sprint.sprint_id = open_ti.sprint_id
LEFT JOIN
    (select sprint_id, count(*) as total_count from tracker_item group by sprint_id) total_ti
ON
    tracker_sprint.sprint_id = total_ti.sprint_id
WHERE
    tracker_sprint.project_id = {$Project->getProjectId()}
";

if (!empty($status_filter)) {
	$sql .= "\nAND (".implode(' OR ', $status_filter).")";
	$sprint_status = implode('|', $selected_status);
	if (!$filter_from_pref) {		
		$user->setPreference($preference_key, serialize(array('sprint_status' => $sprint_status)));
	}
}

// calcuate total before getting full results, for pagination
$total = 0; 
$sqlCount = "SELECT COUNT(*) AS count FROM ($sql) foo";
$con = Propel::getConnection(ProjectPeer::DATABASE_NAME);
$stmtCount = $con->createStatement($sql);
$rsCount = $stmtCount->executeQuery($sqlCount, ResultSet::FETCHMODE_ASSOC);
$rsCount->next();
$total = $rsCount->getInt('count');
$paginator = new Paginator('tracker_sprint', -1, $total, TrackerSprintPeer::SPRINT_ID, $columns);
$sqlPag = $paginator->getSql();

$sql .= $sqlPag;
$stmt = $con->createStatement();
$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

echo '<style>'
. 'form#quickbrowse select { margin: auto .5em .5em auto; }'
. '</style>';

echo '<form id="quickbrowse" name="quickbrowse" action="'.GFUrl::getUrl()->currentBaseUrl().'" method="get">';
  echo '<input type="hidden" name="action" value="TrackerSprintBrowse" />';

  // Cribbed from TrackerItemBrowse
  echo '<table><tr>'
  . '<td valign="top"><h4>'. $lang->getText("TrackerSprint.StatusName") . '</h4></td>'
  . '<td valign="top">' . HTMLWidget::selectBoxFromAssoc($status_options, "sprint_status", $sprint_status, false, false, null, true)
  . '<div style="display: inline; min-width: 300px;">'
  . HTMLWidget::button('submit',$lang->getText('Common.Browse'),'browse','','executeQuery();', true, 'button')
  . '</div></td></tr></table>';
echo '</form>';

//Print headers
$theader = $paginator->getSortableTableHeaders($theme);

echo $theme->tableTop($theader);

$label_start = $lang->getText('TrackerSprint.Start');
$label_close = $lang->getText('TrackerSprint.Close');
$label_delete = $lang->getText('Delete');
$label_open = $lang->getText('Tracker.OpenCount');

if($total < 1){
    echo $theme->tableFullRow('<span class="none found">'.$lang->getText('TrackerItem.NoneFound').'</span>');
} else {
    while ($rs->next()){

        $id = $rs->getInt('sprint_id');
      
        $trow = array();

        $url = GFUrl::getFullURL(
          GFUrl::getUrl()->projectURL($Project, 'TrackerSprintView', 'tracker', '', array('sprint_id'=>$id))
        );

        // sprint name
        $trow[]="<a href=\"{$url}\">"
        . TrackerSprint::TitleForSprint($rs->getString("sprint_progression"), $rs->getString("sprint_name"), $lang)
        . '</a>';

        // sprint status
        $trow[] = $statuses[$rs->getInt('sprint_status')];

        // percent complete
        $open_raw = $rs->getString('open_count');
        $total_raw = $rs->getString("total_count");
        $closed_raw = $total_raw - $open_raw;
	    $percent_complete = round( ($closed_raw / $total_raw) * 100);

        $trow[] = "{$percent_complete}% ($closed_raw / $total_raw)";

            // sprint start
            $trow[] = $rs->getString('sprint_start');

            // sprint end
            $trow[] = $rs->getString('sprint_end');

	    if ($isSprintAdmin) {

		    // sprint start link
        $actions = '';
        if ($rs->getInt('sprint_status') == TrackerSprint::STATUS_NOT_STARTED) {
          $actions .='<a class="action" href="'. GFUrl::getFullURL(GFUrl::getUrl()->projectURL(
            $Project,
            'TrackerSprintStartAction',
            'tracker',
            '',
            array('sprint_id'=>$id))) . '">'. $label_start .'</a>';
		} else if ($rs->getInt('sprint_status') == TrackerSprint::STATUS_IN_PROGRESS) {
	        $actions .='<a class="action" href="'. GFUrl::getFullURL(GFUrl::getUrl()->projectURL(
	        $Project,
	        'TrackerSprintCloseAction',
	        'tracker',
	        '',
	        array('sprint_id'=>$id))) . '">'. $label_close .'</a>';
        }
        
        // sprint delete
        $actions .='<a class="action" href="'. GFUrl::getFullURL(GFUrl::getUrl()->projectURL(
          $Project, 
          'TrackerSprintDelete', 
          'tracker',
          '',
          array('sprint_id'=>$id))) . '">'. $label_delete .'</a>';

        $trow[] = $actions;

	    }

        echo $theme->tableRow($trow);
    }
}

$arr_buttons = array();
if ($isSprintAdmin) {
	$arr_buttons = array(GFUrl::getFullURL(GFUrl::getUrl()->projectURL($Project, 'TrackerSprintAdd', 'tracker','',array())) => $lang->getText('TrackerSprint.AddNew'));
}
echo $theme->tableBottom($arr_buttons);

if(isset($paginator) && is_object($paginator)) {
  $txt = $paginator->getPaginator();
  echo '<div class="paginator">';
  if (empty($txt)) {//no records or just one page
    echo "<strong>" . $lang->getText("Paginator.Page") . ": 1</strong> ";
    echo "<strong>(". $total . " " . $lang->getText("Paginator.TotalRecords") . ")</strong>";
  }
  echo $txt . '</div><br />';
}
$theme->footer($params);

function percentComplete($open_raw, $total_raw) {
  $open = intval($open_raw);
  $total = intval($total_raw);

  $pct = (1 - ($open/$total));

  return round($pct * 100, 0);
}