<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code lists TrackerItem
//
require_once('plugins/tracker/lib/TrackerItemPeer.php');
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('plugins/tracker/lib/TrackerSprint.php');
require_once('plugins/tracker/lib/TrackerSprintPeer.php');
require_once('plugins/tracker/lib/html/TrackerSprintTemplate.php');

function saveFilterOptions($filterOptions) {
	$user = Session::getSession()->getUser();
	$project = GFUrl::getUrl()->getProject();
	if (!is_object($project) || !is_object($user)) {
		return;
	}

	$preference_name = 'sprintquery_' . $project->getProjectId();
	$user->setPreference($preference_name, serialize($filterOptions));
}

function getFilterOptions($sortCols, $sortArray) {
	$filterOptions = array();

	$user = Session::getSession()->getUser();
	$project = GFUrl::getUrl()->getProject();
	if (is_object($project) && is_object($user)) {
		$preference_name = 'sprintquery_' . $project->getProjectId();
		$userPreference = $user->getPreference($preference_name);
		$filterOptions = unserialize($userPreference);
		if ($filterOptions == FALSE) {
			$filterOptions = array();
		}
	}

	if (!isset($filterOptions['search'])) {
		$filterOptions['search'] = null;
	}
	if (!isset($filterOptions['status'])) {
		$filterOptions['status'] = -1;
	}
	if (!isset($filterOptions['assignee'])) {
		$filterOptions['assignee'] = null;
	}
	if (!isset($filterOptions['submitted_by'])) {
		$filterOptions['submitted_by'] = null;
	}
	if (!isset($filterOptions['tracker'])) {
		$filterOptions['tracker'] = null;
	}
	if (!isset($filterOptions['sortcol'])) {
		$filterOptions['sortcol'] = 'tracker_item_id';
	}
	if (!isset($filterOptions['sortord'])) {
		$filterOptions['sort'] = 'ASC';
	}

        if (isset($_REQUEST['search'])) {
            $search = Validator::sanitizeString($_REQUEST['search']);
            if ($search === '') {
                $search = null;
            }
        } else {
            $search = null;
        }
        if (isset($_REQUEST['status'])) {
            // Status set, validate
            $status = Validator::sanitizeInt($_REQUEST['status']);
            // If status is invalid (note that '0' is a valid status value)
            if ($_REQUEST['status'] !== '0' && $status === 0) {
                $status = -1;
            }
        } else {
            // Status not set
            $status = -1;
        }
        if (isset($_REQUEST['assignee'])) {
            $assignee = Validator::sanitizeInt($_REQUEST['assignee']);
            if ($assignee === 0) {
                $assignee = null;
            }
        } else {
            $assignee = null;
        }
        if (isset($_REQUEST['submitted_by'])) {
            $submitted_by = Validator::sanitizeInt($_REQUEST['submitted_by']);
            if ($submitted_by === 0) {
                $submitted_by = null;
            }
        } else {
            $submitted_by = null;
        }
        if (isset($_REQUEST['tracker'])) {
            $tracker = Validator::sanitizeInt($_REQUEST['tracker']);
            if ($tracker === 0) {
                $tracker = null;
            }
        } else {
            $tracker = null;
        }
        if (isset($_REQUEST['sortcol'])) {
            $sortcol = $_REQUEST['sortcol'];
            if (!array_key_exists($sortcol, $sortCols)) {
                $sortcol = 'tracker_item_id';
            }
        } else {
            $sortcol = null;
        }
        if (isset($_REQUEST['sortord'])) {
            $sortord = $_REQUEST['sortord'];
            if (!array_key_exists($sortord, $sortArray)) {
                $sortord = 'ASC';
            }
        } else {
            $sortord = null;
        }
        
	$filterOptions['search'] = ($search != null) ? $search : (isset($_REQUEST['search']) ? null : $filterOptions['search']);
	$filterOptions['status'] = ($status != -1) ? $status : (isset($_REQUEST['status']) ? -1 : $filterOptions['status']);
	$filterOptions['assignee'] = ($assignee != null) ? $assignee : (isset($_REQUEST['assignee']) ? null : $filterOptions['assignee']);
	$filterOptions['submitted_by'] = ($submitted_by != null) ? $submitted_by : (isset($_REQUEST['submitted_by']) ? null : $filterOptions['submitted_by']);
	$filterOptions['tracker'] = ($tracker != null) ? $tracker : (isset($_REQUEST['tracker']) ? null : $filterOptions['tracker']);
	$filterOptions['sortcol'] = ($sortcol != null) ? $sortcol : $filterOptions['sortcol'];
	$filterOptions['sortord'] = ($sortord != null) ? $sortord : $filterOptions['sortord'];

	//Save the options
	saveFilterOptions($filterOptions);

	return $filterOptions;
}

function getAllAssignees($Project) {
  $assignees = array();
  $trackers = $Project->getTrackers();
  foreach ($trackers as $tracker) {
    foreach ($tracker->getTechnicians() as $tech) {
      if (!array_key_exists($tech->getUserId(), $assignees)) {
        $assignees[$tech->getUserId()] = $tech;
      }
    }
  }
  return array_values($assignees);
}

$session = Session::getSession();
$Lang = $session->getLanguage();

$theme = new TrackerSprintTemplate();

$Project = GFUrl::getUrl()->getProject();

$TrackerSprint = TrackerSprintPeer::retrieveByPk($_REQUEST['sprint_id'],$Project->getProjectId());
if (!$TrackerSprint || !is_object($TrackerSprint)) {
  throw new NotFoundException();
}
$sprint_id = $TrackerSprint->getSprintId();

if (!$TrackerSprint->userCanRead()) {
  throw new PermissionDeniedException();
}

$ProjectSprints = $Project->getSprintList();

// $ProjectSprints will not include any closed sprints.
// If the currently-selected sprint is not in the list,
// then we need to add it to the list for display,
// otherwise things look funky.
if ($TrackerSprint != null) {
  if (! array_key_exists($TrackerSprint->getSprintId(), $ProjectSprints)) {
    $ProjectSprints[$TrackerSprint->getSprintId()] = $TrackerSprint->getSprintTitleText();
  }  
}

$allAssignees = getAllAssignees($Project);

$action = isset($_REQUEST['action']) ? $_REQUEST['action'] : null;

$sortCols = array(
	'tracker_item_id'    => $Lang->getText('TrackerItem.TrackerItemIdName'),
	'summary'            => $Lang->getText('TrackerItem.SummaryName'),
	'priority'           => $Lang->getText('TrackerItem.PriorityName'),
	'submitted_by'       => $Lang->getText('TrackerItem.SubmittedByName'),
	'open_date'          => $Lang->getText('TrackerItem.OpenDateName'),
	'close_date'         => $Lang->getText('TrackerItem.CloseDateName'),
	'last_modified_date' => $Lang->getText('TrackerItem.LastModifiedDateName'),
);

$sortArray = array(
	'ASC'  => $Lang->getText('Tracker.SortAsc'),
	'DESC' => $Lang->getText('Tracker.SortDesc')
);

$params = array(
		'project' => $Project,
		'action' => isset($action) ? $action : '',
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('TrackerSprint.ViewPageTitle')
		);
$theme->header($params);
?>
<script type="text/javascript">
$(document).ready(function() {

  // select/deselect all functionality
  $('#select_all').on('click', function () {
      $("table > tbody:eq(1) tr:gt(3):not(:last) td > label > input").prop('checked', this.checked);
    });
  $("table > tbody:eq(1) tr:gt(3):not(:last) td > label > input").on('click', function() {
    if (!this.clicked) {
      $('#select_all').prop('checked', false);
    }
  });

  function doFilter() {
    $("form.filter").submit();
  }
  function doMassUpdate() {
    $("form.ti-table").submit();
  }
  $("#submit-filter").click(function() {
    doFilter();
  });
  $("#massupdate").click(function() {
    doMassUpdate();
  });

	//Opens the Edit form in a modal dialog
	$(".edit").click(function() {
		$( "#sprint-edit-container" ).dialog({
			esizable: true,
			height:'auto',
			width: 680,
			title: "Edit",
			modal: true,
			buttons: {},
			Cancel: function() {
				$( this ).dialog( "close" );
			}
		});
	});

	//Opens the burndown graph in a modal dialog
	$(".burndown").click(function() {
		$( "#burndown-chart-container" ).dialog({
			esizable: true,
			height:'auto',
			width: 880,
			title: "Burndown chart",
			modal: true,
			buttons: {},
			Cancel: function() {
				$( this ).dialog( "close" );
			}
		});
	});


});

</script>

<?php
$filterOptions = getFilterOptions($sortCols, $sortArray);

$search = $filterOptions['search'];

$statuses = array(
  Tracker::STATUS_OPEN => $Lang->getText("tracker.status.open"),
  Tracker::STATUS_CLOSED => $Lang->getText("tracker.status.closed")
);
$statusSelect = HTMLWidget::selectBoxFromAssoc(
  $statuses,
  'status',
  $filterOptions['status'],
  false,
  false, '',
  true
);

$c = new Criteria();
$c->setDistinct();
$c->addJoin(TrackerItemAssigneePeer::TRACKER_ITEM_ID, TrackerItemPeer::TRACKER_ITEM_ID);
$c->addJoin(TrackerItemAssigneePeer::ASSIGNEE, UserPeer::USER_ID);
$c->add(TrackerItemPeer::SPRINT_ID, $TrackerSprint->getSprintId());
$assignees = UserPeer::doSelect($c);
$assigneeSelect = HTMLWidget::select(
  'assignee',
  $assignees,
  $filterOptions['assignee'],
  HTMLWidget::OPTION_ANY,
  false, Language::getSessText('Tracker.Nobody'),
  true, Language::getSessText('Options.Any')
);

$c = new Criteria();
$c->setDistinct();
$c->addJoin(TrackerItemPeer::SUBMITTED_BY, UserPeer::USER_ID);
$c->add(TrackerItemPeer::SPRINT_ID, $TrackerSprint->getSprintId());
$submitted = UserPeer::doSelect($c);
$submittedSelect = HTMLWidget::select(
  'submitted_by',
  $submitted,
  $filterOptions['submitted_by'],
  HTMLWidget::OPTION_ANY,
  true, Language::getSessText('Tracker.Nobody'),
  true, Language::getSessText('Options.Any')
);

$trackers = $Project->getTrackers();
// tracker acl
for ($i = 0; $i < count($trackers); $i++) {
  if (!$trackers[$i]->userCanRead()) {
    array_splice($trackers, $i, 1);
  }
}
$trackerSelect = HTMLWidget::select(
  'tracker',
  $trackers,
  $filterOptions['tracker'],
  HTMLWidget::OPTION_ANY,
  false, Language::getSessText('Tracker.Nobody'),
  true, Language::getSessText('Options.Any')
);

$sortBySelect = HTMLWidget::selectBoxFromAssoc(
  $sortCols, 'sortcol', $filterOptions['sortcol'], false, false
);

$sortOrderSelect = HTMLWidget::selectBoxFromAssoc(
  $sortArray, 'sortord', $filterOptions['sortord'], false, false
);

$burndownUrl = GFUrl::getURL()->projectURL($Project, 'Burndown', 'graphs', '', array('sprint_id' => $sprint_id));

?>
<table>
  <tbody>
    <tr>
      <td style="vertical-align: top;padding-left:0px;">
        <div id="sprint-select-container" style="margin: 0 0 3em 0;">
          <form class="sprint-select" action="<?php echo GFUrl::getUrl()->currentBaseURL(); ?>" method="get">
            <div>
              <strong><?php echo $Lang->getText('TrackerSprint.Name'); ?>:</strong> 
              <?php echo HTMLWidget::selectBoxFromAssoc($ProjectSprints, "sprint_id", $sprint_id, false, false); ?> 
              <a href="javascript:;" class="edit" style="margin-left:5px;" title="<?php echo $Lang->getText('Common.Edit'); ?>">
                  <?php echo $Lang->getText('Common.Edit'); ?>
                </a>
            </div>
            <input type="hidden" name="action" value="TrackerSprintView" />
          </form>
        </div>
      </td>
      <td style="vertical-align: top;">
        <h4 style="margin: 0 0 1em 0;">
          Percent Complete: <?php echo $TrackerSprint->getPercentComplete(); ?>%
          (<?php echo $TrackerSprint->getTrackerItemCount(Tracker::STATUS_CLOSED) ?> / 
          <?php echo $TrackerSprint->getTrackerItemCount() ?>)
        </h4>
        <img src="<?php echo $burndownUrl; ?>"
             style="height: 48px; cursor: pointer;" 
             title="Burndown - Click to toggle larger chart"
	         class="burndown"
         />
      </td>
    </tr>
    <tr>
      <td style="vertical-align: top;" colspan="2">
        <div style="display: none; margin: 1em 0 1em 0;" id="sprint-edit-container">
          <h4><?php echo $Lang->getText('TrackerSprint.DetailsLabel'); ?></h4>
          <form class="sprint-edit" action="<?php echo GFUrl::getUrl()->currentBaseURL('TrackerSprintEditAction') ?>" method="post">
            <input type="hidden" name="sprint_id" value="<?php echo $TrackerSprint->getSprintId(); ?>" />
            <ul class="quickBrowse" style="display: inline-block;">
              <li class="quickBrowse"><strong><?php echo $Lang->getText('TrackerSprint.SprintNameName'); ?></strong><br /><input type="text" name="sprint_name" value="<?php echo $TrackerSprint->getSprintName(); ?>" /></li>
              <li class="quickBrowse"><strong><?php echo $Lang->getText('TrackerSprint.SprintStatusName'); ?></strong><br /><?php echo HTMLWidget::selectBoxFromAssoc(TrackerSprint::getStatuses(), "sprint_status", $TrackerSprint->getSprintStatus(), false, false); ?></li>
              <li class="quickBrowse"><strong><?php echo $Lang->getText('TrackerSprint.SprintStartName'); ?></strong><br /><input type="text" id="sprint_start_date" name="sprint_start_date" value="<?php echo date("M j Y", strtotime($TrackerSprint->getSprintStart())); ?>"  /><a href="javascript:NewCal('sprint_start_date','MMMddyyyy')"><img src="/themes/gforge5/images/cal.gif" width="16" height="16" border="0" alt="Pick a date"/></a></li>
              <li class="quickBrowse"><strong><?php echo $Lang->getText('TrackerSprint.SprintEndName'); ?></strong><br/><input type="text" id="sprint_end_date" name="sprint_end_date" value="<?php echo date("M j Y", strtotime($TrackerSprint->getSprintEnd())); ?>"  /><a href="javascript:NewCal('sprint_end_date','MMMddyyyy')"><img src="/themes/gforge5/images/cal.gif" width="16" height="16" border="0" alt="Pick a date"/></a></li><br /><br />
              <li class="quickBrowse"><?php echo HTMLWidget::button("submit", $Lang->getText("TrackerSprint.Save"), "submit", "", ""); ?></li>
            </ul>
          </form>
        </div>
      </td>
    </tr>
    <tr>
      <td style="vertical-align: top;" colspan="2">
        <div id="burndown-chart-container" style="display: none; margin: 1em 0 1em 0">
          <h4>
            Percent Complete: <?php echo $TrackerSprint->getPercentComplete(); ?>%
            (<?php echo $TrackerSprint->getTrackerItemCount(Tracker::STATUS_CLOSED) ?> / 
            <?php echo $TrackerSprint->getTrackerItemCount() ?>)
          </h4>
          <img src="<?php echo $burndownUrl; ?>&full=1" style="margin: 1em 2em 2em 2em;" alt="Burndown for this Sprint" />
        </div>
      </td>
    </tr>
    <tr>
      <td style="vertical-align: top;padding-left:0px;" colspan="2">
        <h4><?php echo $Lang->getText('TrackerSprint.TrackerItemFilter') ?></h4>
        <form method="GET">
          <input type="hidden" name="action" value="TrackerSprintView">
          <input type="hidden" name="sprint_id" value="<?php echo $TrackerSprint->getSprintId(); ?>">
          <ul class="quickBrowse" style="display: inline-block;">
            <li class="quickBrowse"><strong>Keywords:</strong><br/><input type="text" name="search" value="<?php echo $search ?>"/></li>
            <li class="quickBrowse"><strong>Status: </strong><?php echo $statusSelect; ?></li>
            <li class="quickBrowse"><strong>Assignee: </strong><?php echo $assigneeSelect; ?></li>
            <li class="quickBrowse"><strong>Submitted By: </strong><?php echo $submittedSelect; ?></li>
            <li class="quickBrowse"><strong>Tracker: </strong><?php echo $trackerSelect; ?></li>
            <li class="quickBrowse"><strong>Sort By: </strong><?php echo $sortBySelect; ?></li>
            <li class="quickBrowse"><strong>Order: </strong><?php echo $sortOrderSelect; ?></li>


            <br /><br />
            <li class="quickBrowse"><?php echo HTMLWidget::button("submit", "Submit", "submit-filter", "submit-filter"); ?></li>
          </ul>
        </form>
      </td>
    </tr>
  </tbody>
</table>

<?php

$filterSql = "SELECT ti.tracker_item_id FROM tracker_item AS ti LEFT JOIN tracker_item_assignee AS tia ON ti.tracker_item_id = tia.tracker_item_id";
$filterSqlWheres = array();

if ($filterOptions['status'] != -1) {
  $filterSqlWheres []= "status_id = {$filterOptions['status']}";
}
if ($filterOptions['submitted_by'] != null) {
  $filterSqlWheres []= "submitted_by = {$filterOptions['submitted_by']}";
}
if ($filterOptions['tracker'] != null) {
  $filterSqlWheres []= "tracker_id = {$filterOptions['tracker']}";
}
if ($filterOptions['assignee'] != null) {
  $filterSqlWheres []= "assignee = {$filterOptions['assignee']}";
}
if (count($filterSqlWheres) > 0) {
  $filterSql .= " WHERE ti.sprint_id = {$sprint_id} AND ";
  $filterSql .= implode(" AND ", $filterSqlWheres);
}

$keywordSql = "SELECT ti.tracker_item_id FROM tracker_item AS ti 
  LEFT JOIN tracker_item_assignee as tia ON ti.tracker_item_id = tia.tracker_item_id
  LEFT JOIN \"user\" AS au ON tia.assignee = au.user_id
  LEFT JOIN \"user\" AS su ON ti.submitted_by = su.user_id
";
$keywordSqlWheres = array();

$keywords = explode(" ", $search);

foreach ($keywords as $word) {
  if ($word != ""){ 
    $keywordSqlWheres []= "summary ILIKE '%$word%'";
    $keywordSqlWheres []= "details ILIKE '%$word%'";
    $keywordSqlWheres []= "su.firstname ILIKE '%$word%'";
    $keywordSqlWheres []= "su.lastname ILIKE '%$word%'";
    $keywordSqlWheres []= "au.firstname ILIKE '%$word%'";
    $keywordSqlWheres []= "au.lastname ILIKE '%$word%'";
  }
}

$sql = "SELECT * FROM tracker_item WHERE sprint_id = {$sprint_id} AND tracker_item_id IN($filterSql)";
if (count($keywordSqlWheres) > 0) {
  $keywordSql .= " WHERE ti.sprint_id = {$sprint_id} AND ("  . implode(" OR ", $keywordSqlWheres).')';
  $sql .= " AND tracker_item_id IN ($keywordSql)";
}
// Sorting works backwards for priority
// e.g., high priority = low number in the database
// Switch only for the SQL call, leave intact for display purposes!
if ($filterOptions['sortcol'] == 'priority') {
  $sqlSort = ($filterOptions['sortord'] == 'ASC') ? 'DESC' : 'ASC';
} else {
  $sqlSort = $filterOptions['sortord'];
}
$sql .= " ORDER BY {$filterOptions['sortcol']} {$sqlSort}";
$con = Propel::getConnection(TrackerItemPeer::DATABASE_NAME);
$stmt = $con->createStatement();
$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
$arr = TrackerItemPeer::populateObjects($rs);

echo "<br />";
$totalCount = count($arr);
if (!isset($_GET['start'])) {
	$start=0;
} else {
	$start=$_GET['start'];
}
$pageSize=((isset($_REQUEST['show'])) ? intval($_REQUEST['show']) : 25 );

// Only include parameters that are set
$query_params = array_filter(array(
 'sprint_id' => $TrackerSprint->getSprintId(),
 'search' => $search,
 'assignee' => $filterOptions['assignee'],
 'status' => $filterOptions['status'],
 'submitted_by' => $filterOptions['submitted_by'],
 'tracker' => $filterOptions['tracker'],
 'show' => $pageSize,
 'start' => $start,
 'sortcol' => $filterOptions['sortcol'],
 'sortord' => $filterOptions['sortord']
), 'strlen');

$pager = renderPagerContent($totalCount, $pageSize, $query_params, $Lang);
echo "<br />$pager";

//
//	Show the list of items that match the query.
//

$this_url = GFUrl::getUrl()->currentBaseURL('TrackerSprintView').'&'.http_build_query($query_params);
echo "<form method='POST' class='ti-table' action='" . GFUrl::getUrl()->currentBaseURL('TrackerSprintMassUpdateAction', array('return' => $this_url)) . "'>";

$theader = array();

$theader[] = '<input id="select_all" type="checkbox">' . 
  $Lang->getText('TrackerItem.TrackerItemIdName');
$theader[] = $Lang->getText('TrackerItem.SummaryName');
$theader[] = $Lang->getText('TrackerItem.PriorityName');
$theader[] = $Lang->getText('Tracker.TabString');
$theader[] = $Lang->getText('TrackerSprint.StatusName');
$theader[] = $Lang->getText('TrackerItem.AssignedTo');
$theader[] = $Lang->getText('TrackerItem.SubmittedByName');
$theader[] = $Lang->getText('TrackerItem.OpenDateName');
$theader[] = $Lang->getText('TrackerItem.CloseDateName');
$theader[] = $Lang->getText('TrackerItem.LastModifiedDateName');

// Display the "Add new tracker item" button on the top of the table (if necessary)
$widgets_left = array();
$widgets_right = array();

if ($totalCount >= 0 && $TrackerSprint->userCanRead() && Session::getSession()->isLoggedIn()) {
  $link = GFUrl::getUrl()->currentBaseUrl('TrackerItemAdd', array('sprint_id' => $TrackerSprint->getSprintId()));
  $title = $Lang->getText('TrackerItem.AddNew');
  $widgets_left[] = HTMLWidget::button("button", $title , "", "", 'window.location=\'' . $link . '\';', true, "button", "", true);
}

echo $theme->tableTop($theader, false, '', 'tabular', "", true, $widgets_left, $widgets_right);

if ($totalCount == 0 ) {
	echo $theme->tableFullRow($Lang->getText('TrackerItem.NoneFound'));
} else {
	$editurl=GFUrl::getUrl()->currentBaseUrl('TrackerItemEdit');
	$deleteurl=GFUrl::getUrl()->currentBaseUrl('TrackerItemDelete');

  $currentpage = currentPage($start, $pageSize);
	for ($i=$currentpage*$pageSize; $i<$start * $pageSize + $pageSize; $i++) {
    if (is_object($arr[$i])) {
      $trow = array();
      $tmod = array();

      //Traker Item ID
        if (true) {
          $trow []= '<label style="white-space: nowrap" for="ti' . $i . '">'
            . '<input type="checkbox" name="tid[]" value="' . $arr[$i]->getTrackerItemId() . '" id="ti' . $i . '" />'
            . $arr[$i]->getTrackerItemId() . '</label>';
        } else {
          $trow []= $arr[$i]->getTrackerItemId();
        }
      $tmod[]='';
      
      //Tracker Item Summary
      $trow[]='<a href="'. $editurl .'&amp;tracker_item_id='.$arr[$i]->getTrackerItemID() .'&amp;start='. $start .'"' . $tooltip . '>'. htmlentities($arr[$i]->getSummary(false), ENT_COMPAT,'UTF-8').'</a>';
      $tmod[]='';

      $priorityTexts = Tracker::getPriorities();

      //Tracker Item Priority
      $priority = $arr[$i]->getPriority();
      $trow[] = $priorityTexts[$priority];
      $tmod[] = 'bgcolor="'.$theme->getPriorityColor($priority).'"';

      //Tracker Name
      $trow[]=$arr[$i]->getTracker()->getTrackerName();
      $tmod[]='';

      //Tracker Item Status
      $s = $arr[$i]->getStatusId();
      $statuses = Tracker::getStatuses();
      $trow[]= $statuses[$s] . " (" . $arr[$i]->getStatusName() . ")";
      $tmod[]='';

      //Tracker Item Assignees
      $asses = $arr[$i]->getTrackerItemAssignees();
      $assignees = '';
      foreach ($asses as $ass) {
        $assignees .= $ass->getUser()->getUserLink().'<br />';
      }
      $trow[]=$assignees;
      $tmod[]='';
      unset($assignees);

      //Tracker Item Submitter
      $trow[]=UserPeer::retrieveByPk($arr[$i]->getSubmittedBy())->getUserLink();
      $tmod[]='';

      //Tracker Item Open Date
      $trow[]=$arr[$i]->getOpenDate('Y-m-d');
      $tmod[]='';

      //Tacker Item Close Date
      $trow[]=(($arr[$i]->getStatusId()==Tracker::STATUS_CLOSED || $IS_TASK) ? $arr[$i]->getCloseDate('Y-m-d') : '&nbsp;');
      $tmod[]='';

      //Tracker Item Last Modified Date
      $trow[]=$arr[$i]->getLastModifiedDate('Y-m-d');
      $tmod[]='';

      echo $theme->tableRow($trow,$tmod,false);
      unset($trow);
      unset($tmod);
    }
	}
}

if ($TrackerSprint->userCanRead() && Session::getSession()->isLoggedIn()) {
	$button_enabled = true;
  $widgets_left = array(GFUrl::getUrl()->currentBaseUrl('TrackerItemAdd').'&amp;sprint_id='.$TrackerSprint->getSprintId()=>$Lang->getText('TrackerItem.AddNew'));
} else {
	$button_enabled = false;
  $widgets_left = array();
}

if ($TrackerSprint->userCanDelete() && Session::getSession()->isLoggedIn()) {
	$mass_delete_enabled = true;
} else {
	$mass_delete_enabled = false;
}

echo $theme->tableBottom($widgets_left,array($button_enabled), "", false, true, true, array(true, false));

  echo "<br />$pager";

  echo "<input type=\"hidden\" name=\"sprint_id\" value=\"{$sprint_id}\" />";

	echo '<br/><br/><fieldset class="massUpdate"><legend><strong>&nbsp;' . $Lang->getText("Tracker.MassUpdate") . '&nbsp;</strong></legend><table width="100%" style="clear:left;">';


  echo '<td width="50%" valign="top"><strong>' . $Lang->getText('TrackerItem.PriorityName') . ':</strong><br />';
  echo HTMLWidget::selectBoxFromAssoc(Tracker::getPriorities(), 'priority', HTMLWidget::OPTION_ANY, false, false, null, true, 'No Change');
  echo "</td>";

  echo '<td width="50%" valign="top"><strong>' . $Lang->getText('TrackerSprint.TrackerSprintName') . ':</strong><br />';
  echo HTMLWidget::selectBoxFromAssoc($ProjectSprints, "new_sprint_id", HTMLWidget::OPTION_ANY, false, true, null, true, 'No Change');
  echo "</td>";

  echo "<td>&nbsp;</td></tr><tr>";

  echo '<td width="50%" valign="top"><strong>' . $Lang->getText('TrackerItem.AssignedTo') . ':</strong><br />';
  echo HTMLWidget::select(
						'assignee',
            $allAssignees,
						'',
						HTMLWidget::OPTION_ANY,
            false,
            Language::getSessText('Tracker.Nobody'),
						true, Language::getSessText('Tracker.NoChange')
				);
  echo "</td>";

	echo '<tr><td>'.HTMLWidget::button("submit",$Lang->getText('Tracker.MassUpdate'),"","massupdate","");
	echo '</td></tr></table></fieldset>';

echo "</form>";

$theme->footer($params);

?>
<script type="text/javascript">
$(document).ready(function() {
  var select = $("#tracker_id");
  select.remove();
  select.removeAttr("style");
  select.insertBefore(".buttonLinkLeft > .buttondiv > .button");

  // Remove default onclick listener
  $(".buttonLinkLeft > .buttondiv > .button > a").removeAttr("onclick");

  $(".buttonLinkLeft > .buttondiv > .button").click(function(event) {
    event.preventDefault();
    var tracker_id = $(this).siblings("#tracker_id").val();
    var url = "<?php echo GFUrl::getUrl()->currentBaseUrl('TrackerItemAdd').'&sprint_id='.$TrackerSprint->getSprintId(); ?>&tracker_id=" + tracker_id;
    location.href = url;
  });

  $('#sprint-select-container').on('change', 'select', function() {
    $('form.sprint-select').submit();
  });
});
</script>
<script language="javascript" type="text/javascript" src="/datetimepicker.js"></script>
<?php

$trackers = $Project->getTrackers();
$select = "<select id='tracker_id' class='table-dropdown' style='display: none;'>";
foreach ($trackers as $tracker) {
  $select .= "<option value='" . $tracker->getTrackerId() . "'>" . $tracker->getTrackerName() . "</option>";
}
$select .= "</select>";
echo $select;

/**
 * Render a block of HTML for the paging links.  Query params should include the following:
 * sprint_id - int ID for the sprint being viewed
 * search - string criteria for filtering against TI summary, etc.
 * assignee - int ID of the current assignee
 * submitted_by in ID of the submitter
 * tracker - int ID of the Tracker to filter by
 * start - int row number in the result list to start display at
 * @param  int $totalcount Total # of records in the list, across all pages
 * @param  int $page_size # of records to show on one page
 * @param  array $query_params Array of query string parameters used to filter the list.
 * @return string The rendered HTML.
 */
function renderPagerContent($totalcount, $page_size, $query_params, $Lang) {
  $ret = '';

  $pages = pageCount($totalcount, $page_size);
  $currentpage = currentPage($query_params['start'], $page_size);
  $skipped_pages=false;
  $base_url = GFUrl::getUrl()->currentBaseUrl('TrackerSprintView');
  $total_text = $Lang->getText("Paginator.TotalRecords");
  $page_num_text = $Lang->getText("Paginator.Page");

  if ($totalcount > 0) {
    $ret .= "<strong>{$page_num_text}: </strong>";
  }

  for ($j=0; $j<$pages; $j++) {
    $j_plus_one = $j + 1;
    if ($pages > 20) {
      if ((($j > 4) && ($j < ($currentpage - 5))) || (($j > ($currentpage + 5)) && ($j < ($pages - 5)))) {
        if (!$skipped_pages) {
          $skipped_pages=true;
          $ret .= "....&nbsp;";
        }
        continue;
      } else {
        $skipped_pages=false;
      }
    }
    if ($j == $currentpage) {
      $ret .= "<strong>{$j_plus_one}</strong>&nbsp;";
    } else {
      $query_params['start'] = $j * $page_size;
      $query_string = http_build_query($query_params);
      $ret .= "<a href=\"{$base_url}&{$query_string}\">{$j_plus_one}</a>&nbsp;";
    }
  }

  $ret .= "<strong>($totalcount $total_text)</strong>";

  return $ret;
}

function currentPage($offset, $pageSize) {
  if ($pageSize == 0) $pageSize = 25;
  return intval($offset / $pageSize);
}

function pageCount($totalRecords, $pageSize) {
  $ret = $totalRecords / $pageSize;
  return $ret;
}

function getThisViewUrl($query_params) {

  $ret = GFUrl::getUrl()->currentBaseURL('TrackerSprintView', $query_params);
  return $ret;
}