<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-13
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/html/HTMLWidget.php');
require_once('html/templates/ProjectReportingTemplate.php');

$session = Session::getSession();
$project = GFUrl::getUrl()->getProject();
$project_id = $project->getProjectId();
$Lang = $session->getLanguage();
$user = $session->getUser();
$theme = new ProjectReportingTemplate();

$con = Propel::getConnection(TrackerPeer::DATABASE_NAME);

$subprojects=((isset($_REQUEST['subprojects'])) ? Validator::sanitizeInt($_REQUEST['subprojects']) : 0 );

$all_ids = array();
$all_names = array();
if ($subprojects == 1) {
	$all_ids = $project->getAllSubprojectIds();
	foreach ($project->getAllSubprojects() as $subPrj) {
		$all_names[$subPrj->getProjectId()] = $subPrj->getProjectName();
	}
	$all_ids[] = $project_id;
	$all_names[$project->getProjectId()] = $project->getProjectName();
} else {
	$all_ids[] = $project_id;
}

// get all trackers for this project
$c = new Criteria();
$c->add(TrackerPeer::PROJECT_ID, array_values($all_ids), Criteria::IN);
$trackersObj = TrackerPeer::doSelect($c);

// Create an array with all the trackers (id=>name)
$trackers = array();
$trackersType = array();
$trackers[0] = $Lang->getText('TrackerReporting.AllTrackers');
foreach($trackersObj as $trk) {
	if ($subprojects == 0 || $trk->getProjectId() == $project_id) {
		$trackers[$trk->getTrackerId()]=$trk->getTrackerName();
		$trackersType[$trk->getTrackerId()]=$trk->getDataType();
	}
}

foreach($trackersObj as $trk) {
	if ($subprojects == 1 && $trk->getProjectId() != $project_id) {
		$trackers[$trk->getTrackerId()]=$all_names[$trk->getProjectId()] . ' &gt; ' . $trk->getTrackerName();
		$trackersType[$trk->getTrackerId()]=$trk->getDataType();
	}
}

// Save the project_id that came in the request, only if it's valid
$tracker_id = (isset($_REQUEST['tracker_id']) && 
			  $_REQUEST['tracker_id'] > 0 &&
			  array_key_exists($_REQUEST['tracker_id'], $trackers) === true) ? intval($_REQUEST['tracker_id']) : 0;

			  
// Get Project members			  
$users = array();
$users[0] = 'All';
foreach ($project->getMembers() as $member) {
	$users[$member->getUserId()] = $member->getFullName();
}

$user_id=((isset($_REQUEST['user_id'])) ? Validator::sanitizeInt($_REQUEST['user_id']) : 0 );

$sqlmine = '';
$sqlmineaux = '';
if ($user_id) {
	$sqlmine .= "AND (ti.submitted_by = $user_id OR
				ti.tracker_item_id IN (SELECT tracker_item_id FROM tracker_item_assignee
				WHERE assignee=$user_id))";
	$sqlmineaux .= "AND tia.assignee ='$user_id'";
}




//Priorities
$Priorities = Tracker::getPriorities();
$Priorities[0] = 'All';
$priority=((isset($_REQUEST['priority'])) ? Validator::sanitizeInt($_REQUEST['priority']) : 0 );

if ($priority)	{
	$sqlmine .= " AND ti.priority = $priority";
	$sqlmineaux .= " AND ti.priority = $priority";
	
}

// Status
$sts = Tracker::getStatuses();
$Statuses = array (0=>'All');

foreach($sts as $k => $f) {
	$Statuses[$k+1] = $f;
}

//array_unshift($Statuses, 'All');
$status = ((isset($_REQUEST['status'])) ? Validator::sanitizeInt($_REQUEST['status']) : 0 );

if ($status > 0)	{
	//$sqlmine = " AND ti.status_id = " . ($status - 1);
	$sqlmineaux .= " AND ti.status_id = " . ($status - 1);
	
}


// Check if we want to filter only the user's items
/*$onlymine = (isset($_REQUEST['onlymine']) && $_REQUEST['onlymine'] == 1) ? true : false;

// If "onlymine", we'll filter the results accordingly
if (!$onlymine) {
	$sqlmine = '';
	$sqlmineaux = '';
} else {
	$sqlmine = "AND (ti.submitted_by = ".$user->getUserId()." OR
				ti.tracker_item_id IN (SELECT tracker_item_id FROM tracker_item_assignee
				WHERE assignee=".$user->getUserId()."))";
	$sqlmineaux = "AND tia.assignee ='".$user->getUserId()."'";
}*/

$params = array(
		'project' => $project,
		'feedback' => (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('TrackerReporting.Title')
	);
$theme->header($params);

?>
<br />
<form action="<?php echo $_SERVER['REQUEST_URI']; ?>" method="post">
<table border="0" cellspacing="0" cellpadding="3" bgcolor="#E0E0E0">
<tr>
	<td><strong><?php echo $Lang->getText('TrackerReporting.FilterByTracker') ?></strong></td>
	<td><?php echo HTMLWidget::selectBoxFromAssoc($trackers,'tracker_id',$tracker_id, false, false) ?></td>
</tr>
<!-- <tr>
	<td colspan="2"><strong><?php //echo $Lang->getText('TrackerReporting.IncludeOnlyMine') ?></strong>
	<input type="checkbox" name="onlymine" value="1" <?php //if ($onlymine) echo 'checked' ?> /></td>
</tr> !-->
<tr>
	<td><strong><?php echo $Lang->getText('TrackerReporting.FilterByUser') ?></strong></td>
	<td><?php echo HTMLWidget::selectBoxFromAssoc($users,'user_id',$user_id, false, false) ?></td>
</tr>
<tr>
	<td><strong><?php echo $Lang->getText('TrackerReporting.FilterByPriority') ?></strong></td>
	<td><?php echo HTMLWidget::selectBoxFromAssoc($Priorities,'priority',$priority, false, false) ?></td>
</tr>
<tr>
	<td><strong><?php echo $Lang->getText('TrackerReporting.FilterByStatus') ?></strong></td>
	<td><?php echo HTMLWidget::selectBoxFromAssoc($Statuses,'status',$status, false, false) ?></td>
</tr>
<tr>
	<td colspan="2">
		<strong><?php echo $Lang->getText('ProjectReporting.IncludeSubprojects') ?></strong> 
		<input type="checkbox" name="subprojects" value="1"<?php echo ($subprojects == 1) ? ' checked' : '';?>/>
	</td>
	<td valign="middle" rowspan="3"><?php echo HTMLWidget::button("submit","" .   $Lang->getText('Common.Submit')  . " ","submit","","")?></td>
</tr>
</table>
</form>
<?php
$imgquerystring = '&amp;tracker_id='.$tracker_id.'&amp;subprojects='.$subprojects;
if($user_id) {
	$imgquerystring .= '&amp;user_id='.$user_id;
}
if($priority) {
	$imgquerystring .= '&amp;priority='.$priority;
}
if($status)	{
	$imgquerystring .= '&amp;status='.$status;
}

echo '<br /><img src="/gf/project/'.$project->getUnixName().'/tracker/graphs/?action=OpenItemsByPriority'.$imgquerystring.'" height="250" width="400" alt="" /> &nbsp;';
if ($subprojects == 1 && $tracker_id == 0) {
	echo '<img src="/gf/project/'.$project->getUnixName().'/tracker/graphs/?action=OpenItemsByProject'.$imgquerystring.'" height="250" width="400" alt="" />';
} else {
	echo '<img src="/gf/project/'.$project->getUnixName().'/tracker/graphs/?action=OpenItemsByTracker'.$imgquerystring.'" height="250" width="400" alt="" />';
}
echo '<br /> &nbsp; <br /><img src="/gf/project/'.$project->getUnixName().'/tracker/graphs/?action=OpenItemsByUser'.$imgquerystring.'" height="250" width="400" alt="" /> &nbsp;';
echo '<img src="/gf/project/'.$project->getUnixName().'/tracker/graphs/?action=AgeOfItems'.$imgquerystring.'" height="250" width="400" alt="" />';
echo '<br /> &nbsp; <br /><img src="/gf/project/'.$project->getUnixName().'/tracker/graphs/?action=ItemsByDate'.$imgquerystring.'" height="250" width="400" alt="" />';
echo '<img src="/gf/project/'.$project->getUnixName().'/tracker/graphs/?action=ItemsByAssignee'.$imgquerystring.'" height="250" width="600" alt="" />';

echo '<br />';
echo '<br />';

/* *****************************
 * Late TASKS by Id and Date
 *******************************/

//	get a list of tasks that are past due
if ($tracker_id > 0) {
	if ($trackersType[$tracker_id] == Tracker::DATATYPE_TASK) {
		$sql = "SELECT ti.*	FROM tracker_item ti
		WHERE ti.tracker_id='".$tracker_id."' 
		AND ti.status_id='".Tracker::STATUS_OPEN."'
		$sqlmine
		AND ti.close_date < now() ORDER BY ti.tracker_id ASC, ti.close_date ASC";
	} else {
		$sql = '';
	}
} else {
	$sql="SELECT ti.* FROM tracker_item ti, tracker t 
		WHERE t.datatype='".Tracker::DATATYPE_TASK."' 
		AND t.tracker_id=ti.tracker_id 
		AND t.project_id IN (".implode(',',$all_ids).")
		AND ti.status_id='".Tracker::STATUS_OPEN."'
		$sqlmine
		AND ti.close_date < now() ORDER BY ti.tracker_id ASC, ti.close_date ASC";
}
if ($sql != '') {
	echo '<h3>'.$Lang->getText('TrackerReporting.LateTasks').'</h3>';
	$theader = array('Tracker',$Lang->getText('TrackerReporting.LateTasks'), $Lang->getText('TrackerReporting.DueDate'));
	echo $theme->tableTop($theader);

	$stmt = $con->createStatement();
	$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
	$tis=TrackerItemPeer::populateObjects($rs);
	$itemscount = 0;
	
	reset($tis);
	$trackercount=array();
	//iterate the list and put into array which will be 
	//useful for getting a rowspan count
	foreach ($tis as $ti) {
		$trackercount[$ti->getTrackerId()][]=$ti;
	}
	
	reset($tis);
	$lastTracker=0;
	foreach ($tis as $ti) {
		$trow=array();
		$tmod=array(); //extra param to tableRow to give us a rowspan
		if ($ti->getTrackerId() != $lastTracker) {
			$tmod[]='rowspan="'.count($trackercount[$ti->getTrackerId()]).'" bgcolor="#FFFFFF"';
			if ($subprojects == 1) {
				$trow[]=$all_names[$ti->getTracker()->getProjectId()] . ' &gt; ' . $ti->getTracker()->getTrackerName();
			} else {
				$trow[]=$ti->getTracker()->getTrackerName();
			}
			$lastTracker=$ti->getTrackerId();
		}
		$tmod[]='';
		$item_project = $ti->getTracker()->getProject();
		$trow[]='<a href="'.GFUrl::getURL()->projectURL($item_project, 'TrackerItemEdit', 'tracker', '', array('tracker_item_id'=>$ti->getTrackerItemId())).'">'. $ti->getSummary().'</a>';
		
		$tmod[]='';
		$trow[]=$ti->getCloseDate();
		echo $theme->tableRow($trow,$tmod);
		unset($tmod);
		unset($trow);
		$itemscount++;
	}
	
	if ($itemscount==0) {
		echo $theme->tableFullRow($Lang->getText('TrackerReporting.NoneFound'));
	}
	echo $theme->tableBottom();
	echo '<br /><br />';
}

/* *****************************
 * LATE TRACKER ITEMS
 *******************************/

//	get a list of issues that are past due
if ($tracker_id > 0) {
	if ($trackersType[$tracker_id] == Tracker::DATATYPE_TRACKER || $trackersType[$tracker_id] == Tracker::DATATYPE_SUPPORT) {
		$sql = "SELECT ti.*	FROM tracker_item ti, tracker t
		WHERE ti.tracker_id='".$tracker_id."' 
		AND t.tracker_id=ti.tracker_id
		AND ti.status_id='".Tracker::STATUS_OPEN."'
		$sqlmine
		AND ti.open_date < (now() - (t.due_period*'1 second'::interval) ) ORDER BY ti.tracker_id ASC, ti.close_date ASC";
	} else {
		$sql = '';
	}
} else {
	$sql="SELECT ti.* FROM tracker_item ti, tracker t 
		WHERE t.datatype IN (".Tracker::DATATYPE_TRACKER.','.Tracker::DATATYPE_SUPPORT .")
		AND t.tracker_id=ti.tracker_id 
		AND t.project_id IN (".implode(',',$all_ids).") 
		AND ti.status_id='".Tracker::STATUS_OPEN."'
		$sqlmine
		AND ti.open_date < (now() - (t.due_period*'1 second'::interval) ) ORDER BY ti.tracker_id ASC, ti.close_date ASC";
}

if ($sql != '') {

	echo '<h3>'.$Lang->getText('TrackerReporting.LateItems').'</h3>';
	$theader = array('Tracker',$Lang->getText('TrackerReporting.LateItems'), $Lang->getText('TrackerReporting.DueDate'));
	echo $theme->tableTop($theader);
	
	$stmt = $con->createStatement();
	$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
	$tis=TrackerItemPeer::populateObjects($rs);
	$itemscount = 0;
	
	reset($tis);
	$trackercount=array();
	//iterate the list and put into array which will be 
	//useful for getting a rowspan count
	foreach ($tis as $ti) {
		$trackercount[$ti->getTrackerid()][]=$ti;
	}
	
	reset($tis);
	$lastTracker=0;
	$duePeriod=0;
	foreach ($tis as $ti) {
		$trow=array();
		$tmod=array(); //extra param to tableRow to give us a rowspan
		if ($ti->getTrackerId() != $lastTracker) {
			$tmod[]='rowspan="'.count($trackercount[$ti->getTrackerid()]).'" bgcolor="#FFFFFF"';
			if ($subprojects == 1) {
				$trow[]=$all_names[$ti->getTracker()->getProjectId()] . ' &gt; ' . $ti->getTracker()->getTrackerName();
			} else {
				$trow[]=$ti->getTracker()->getTrackerName();
			}
			$lastTracker=$ti->getTrackerid();
			$duePeriod=$ti->getTracker()->getDuePeriod();
		}
		$tmod[]='';
		$item_project = $ti->getTracker()->getProject();
		$trow[]='<a href="'.GFUrl::getURL()->projectURL($item_project, 'TrackerItemEdit', 'tracker', '', array('tracker_item_id'=>$ti->getTrackerItemId())).'">'. $ti->getSummary().'</a>';
		//$trow[]=$ti->getName();
		
		$tmod[]='';
		$auxdue = (strtotime($ti->getOpenDate()) + $duePeriod);
		$trow[]= date('Y-m-d H:i:s', $auxdue);
		echo $theme->tableRow($trow,$tmod);
		unset($tmod);
		unset($trow);
		$itemscount++;
	}
	
	if ($itemscount==0) {
		echo $theme->tableFullRow($Lang->getText('TrackerReporting.NoneFound'));
	}
	echo $theme->tableBottom();
	echo '<br /><br />';
}

/* *****************************
 * Open Items by Assignee
 *******************************/

//	get a list of items by user
if ($tracker_id > 0) {
	
	$sql="SELECT tia.*,ti.*
		FROM tracker_item ti, tracker_item_assignee tia
		WHERE ti.tracker_id='".$tracker_id."'
		AND ti.tracker_item_id = tia.tracker_item_id 
		$sqlmineaux
		ORDER BY tia.assignee ASC, ti.priority DESC, ti.open_date ASC, ti.tracker_id ASC";
} else {
	$sql="SELECT tia.*,ti.*
		FROM tracker_item ti, tracker t, tracker_item_assignee tia
		WHERE t.tracker_id=ti.tracker_id 
		AND t.project_id IN (".implode(',',$all_ids).") 
		AND ti.tracker_item_id = tia.tracker_item_id 
		$sqlmineaux
		ORDER BY tia.assignee ASC, ti.priority DESC, ti.open_date ASC, ti.tracker_id ASC";
}

echo '<h3>'.$Lang->getText('TrackerReporting.OpenByAssignee').'</h3>';

$itemscount = 0;
$theader = array($Lang->getText('TrackerReporting.User'),
			$Lang->getText('TrackerReporting.ByAssignee'),
			$Lang->getText('TrackerReporting.OpenDate'));

echo $theme->tableTop($theader);

$stmt = $con->createStatement();
$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
$tias = TrackerItemAssigneePeer::populateObjects($rs);
$lastuser = '';

reset($tias);
$usercount=array();

//iterate the list and put into array which will be 
//useful for getting a rowspan count
foreach ($tias as $tia) {
	$usercount[$tia->getAssignee()][] = $tia;
}

$lastUser=0;
foreach ($tias as $tia) {
	$trow=array();
	$tmod=array(); //extra param to tableRow to give us a rowspan
	if ($tia->getAssignee() != $lastUser) {
		$tmod[]='rowspan="'.count($usercount[$tia->getAssignee()]).'" bgcolor="#FFFFFF"';
		$trow[]=$tia->getUser()->getFullName();
		$lastUser=$tia->getAssignee();
	}
	$tmod[]='';
	$ti = $tia->getTrackerItem();
	$item_project = $ti->getTracker()->getProject();
	$trow[]='<a href="'.GFUrl::getURL()->projectURL($item_project, 'TrackerItemEdit', 'tracker', '', array('tracker_item_id'=>$ti->getTrackerItemId())).'">'. $item_project->getName() . ' > ' . $ti->getSummary().'</a>';
	$tmod[]='';
	$trow[]=$tia->getTrackerItem()->getOpenDate();
	echo $theme->tableRow($trow,$tmod);
	unset($tmod);
	unset($trow);
	$itemscount++;
}

if ($itemscount==0) {
	echo $theme->tableFullRow($Lang->getText('TrackerReporting.NoneFound'));
}

echo $theme->tableBottom();
echo '<br /><br />';

?>
