<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code creates a new TrackerExtraFieldElementTransition and saves it to the 
// database
//
require_once('plugins/tracker/lib/TrackerCommitReassignTrigger.php');
require_once('plugins/tracker/lib/TrackerCommitTrigger.php');
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$session = Session::getSession();
$Lang = $session->getLanguage();
$project = GFUrl::getUrl()->getProject();
$tracker = TrackerPeer::retrieveByPk($_REQUEST['tracker_id']);
if (!is_object($tracker) || !$tracker->userIsAdmin()) {
	throw new PermissionDeniedException();
}

try {
	$c = new Criteria();
	$c->add(TrackerCommitReassignTriggerPeer::TRACKER_ID, $tracker->getTrackerId());
	$tcrts = TrackerCommitReassignTriggerPeer::doSelect($c);
	foreach ($tcrts as $tcrt) {
		$tcrt->delete();
	}
	
	if (isset($_REQUEST['assign_to_role_id']) && is_array($_REQUEST['assign_to_role_id']) && count($_REQUEST['assign_to_role_id']) > 0) {
		foreach ($_REQUEST['assign_to_role_id'] as $role) {
			if (empty($role)) {
				continue;
			} else {
				$tcrt = new TrackerCommitReassignTrigger();
				$tcrt->setTracker($tracker);
				$tcrt->setAssignToRoleId($role);
				$tcrt->save();
			}
		}
	}

	// Save the transition trigger (if any)
	$c = new Criteria();
	$c->add(TrackerCommitTriggerPeer::TRACKER_ID, $tracker->getTrackerId());
	if (isset($_POST['change_tracker_extra_field_id']) && !empty($_POST['change_tracker_extra_field_id'])) {
		$trigger = TrackerCommitTriggerPeer::doSelectOne($c);
		if (!empty($trigger)) {
			// we delete and re-create the item so the changes can be easily
			// copied to the "templated" elements
			$trigger->delete();
		}
		
		$trigger = new TrackerCommitTrigger();
		$trigger->setTrackerId($tracker->getTrackerId());
		$trigger->setChangeTrackerExtraFieldId($_POST['change_tracker_extra_field_id']);
		$trigger->setToElementId($_POST['to_element_id']);
		$trigger->save();
	} else {
		TrackerCommitTriggerPeer::doDelete($c);
	}
	

	GFUrl::redirect(GFUrl::getUrl()->currentBaseURL('TrackerEdit'), array('feedback' => $Lang->getText('TrackerCommit.EditFeedback'),'tracker_id'=>$_POST['tracker_id']));
} catch (InvalidDataException $ex) {
	$errors = $ex->getErrors();		// attach object errors to validation errors
	$feedback = '* '.implode('<br />* ', $errors);
	
	include('TrackerCommitEdit.php');

}
?>
