<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code lists TrackerExtraField
//
require_once('plugins/tracker/lib/TrackerExtraFieldPeer.php');
require_once('plugins/tracker/lib/TrackerExtraField.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

require_once('plugins/tracker/lib/html/TrackerAdminTemplate.php');
$theme = new TrackerAdminTemplate();

$Project = GFUrl::getUrl()->getProject();
$Tracker = TrackerPeer::retrieveByPk($_REQUEST['tracker_id']);
if (!is_object($Tracker) || !$Tracker->userIsAdmin() || 
	$Tracker->getProject()->getProjectId() != $Project->getProjectId()) {
	throw new PermissionDeniedException();
}

$params = array(
		'project' => $Project,
		'tracker' => $Tracker,
		'action' => isset($action) ? $action : '',
		'feedback' => (isset($_REQUEST['feedback'])) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('TrackerExtraField.BrowsePageTitle')
		);
$theme->header($params);


//Get intrinsic fields data
$intrinsic_fields = $Tracker->getIntrinsicFieldsSettings();

// Retrieve the objects array
$crit = new Criteria();
$crit->add(TrackerExtraFieldPeer::TRACKER_ID, $Tracker->getTrackerId());
$crit->addAscendingOrderByColumn(TrackerExtraFieldPeer::FIELD_ORDER);
$arr = TrackerExtraFieldPeer::doSelect($crit);

echo '<p><div class="infobox">'.$Lang->getText('Tracker.AutoAssignDescription').'<br />'.$Lang->getText('Tracker.TransitionCommitRulesDescription').'<br />'.$Lang->getText('Tracker.WorkflowEnabledIndicator').'</div></p>';

$theader = array();

$theader[] = $Lang->getText('TrackerExtraField.FieldNameName');
$theader[] = $Lang->getText('TrackerExtraField.FieldTypeName');
$theader[] = $Lang->getText('TrackerExtraField.AutoAssign');
$theader[] = $Lang->getText('TrackerExtraField.UseWorkFlow');
$theader[] = $Lang->getText('TrackerExtraField.UseCommitRule');
$theader[] = $Lang->getText('TrackerExtraField.IsEnabledName');
$theader[] = $Lang->getText('TrackerExtraField.IsRequiredName');
$theader[] = $Lang->getText('TrackerExtraField.ShowOnSubmitName');
$theader[] = $Lang->getText('TrackerExtraField.UseInQuickBrowseName');
$theader[] = $Lang->getText('TrackerExtraField.FieldOrder');
$theader[] = $Lang->getText('TrackerExtraField.Actions');
?>
<form action="<?php echo GFUrl::getUrl()->currentBaseURL('TrackerExtraFieldBrowseAction') ?>" method="post">
<input type="hidden" name="tracker_id" value="<?php echo $Tracker->getTrackerId(); ?>" />
<?php
echo $theme->tableTop($theader,false,'class="jsSortTBL"');
if (count($arr) < 1) {
	echo $theme->tableFullRow($Lang->getText('TrackerExtraField.NoneFound'));
} else {
	//these vars are used to determine if any field has workflow or auto-assign
	//if not, then the bottom row is set to true
	$was_auto_assign=false;
//	$was_workflow=false;
//	$was_commit_rule=false;

	//Some of the columns of checkboxes are disabled in Express, nothing is disabled in AS
	if (GFConf::get('system.licenseType')=='advanced') {
		$disable_str_1='';
	} else {
		$disable_str_1=' disabled="disabled"';
	}
	if (GFConf::get('system.licenseType')=='express') {
		$disable_str_2=' disabled="disabled"';
	} else {
		$disable_str_2='';
	}
	

	echo '<tbody id="jsExtraFieldList">';

    //Adds instrinsic fields
    foreach($intrinsic_fields as $field_name => $field) {
        $trow = array();

        $trow[] = $field['field_name'].'&nbsp;'.$Lang->getText('TrackerExtraField.IntrinsicField');
        $tmod[] = 'class="moveHandle"';

        $trow[] = $Lang->getText('TrackerExtraField.FieldType'.$field['field_type']);
        $tmod[] = '';

        $trow[] = '&nbsp;';
        $tmod[] = '';

        $trow[] = '&nbsp;';
        $tmod[] = '';

        $trow[] = '&nbsp;';
        $tmod[] = '';

        $trow[] = '<input type="checkbox" class="toggle-enabled" name="is_enabled_'.$field_name.'" '.(($field['is_enabled']) ? 'checked="checked"' : '' ).'" size="3" maxlength="3" '.$disable_str_2.'/>';
        $tmod[] = '';

        $trow[] = '<input type="checkbox" class="toggle-group" name="is_required_'.$field_name.'" '.(($field['is_required']) ? 'checked="checked"' : '' ).'" size="3" maxlength="3" '.$disable_str_2.'/>';
        $tmod[] = '';

        $trow[] = '<input type="checkbox" class="toggle-group" name="show_on_submit_'.$field_name.'" '.(($field['on_submit']) ? 'checked="checked"' : '' ).'" size="3" maxlength="3" '.$disable_str_2.'/>';
        $tmod[]='';

        $trow[]='<input type="checkbox" class="toggle-group" name="show_on_browse_'.$field_name.'" '.(($field['on_browse']) ? 'checked="checked"' : '' ).'" size="3" maxlength="3" '.$disable_str_2.'/>';
        $tmod[]='';

        $trow[]='';
        $tmod[]='';

        $trow[]='';
        $tmod[]='';

        echo $theme->tableRow($trow, $tmod, false, 'id="jsExtraFieldList_EFOrder'.$field_name.'"');
    }

	/** @var TrackerExtraField[] $arr */
	foreach ($arr as $item) {
		$trow = array();
		$tmod = array();

		$trow[]=$item->getFieldName();
		$tmod[]='class="moveHandle"';
		$fieldExtended = '';
		if ($item->getFieldType() == TrackerExtraField::TYPE_MEMBERS) {
			switch ($item->getAttribute1()) {
				case TrackerExtraField::MEMBER_USERS + TrackerExtraField::MEMBER_GROUPS:
					$fieldExtended = ' ('.$Lang->getText('TrackerExtraField.MemberUsersAndGroups').')'; break;
				case TrackerExtraField::MEMBER_GROUPS:
					$fieldExtended = ' ('.$Lang->getText('TrackerExtraField.MemberGroups').')'; break;
				// Default to legacy behavior, users only
				default:
					$fieldExtended = ' ('.$Lang->getText('TrackerExtraField.MemberUsers').')'; break;
			}
		}
		$trow[]=$Lang->getText('TrackerExtraField.FieldType'.$item->getFieldType()).$fieldExtended;
		$tmod[]='';

		if ($item->getFieldType() == TrackerExtraField::TYPE_STATUS 
			|| $item->getFieldType() == TrackerExtraField::TYPE_RADIO
			|| $item->getFieldType() == TrackerExtraField::TYPE_SELECT) {
			if ($item->getUseAutoAssign()) {
				$was_auto_assign=true;
			}
			$trow[]='<input type="radio" name="auto_assign_to" value="'.$item->getTrackerExtraFieldID().'" '.(( $item->getUseAutoAssign()) ? 'checked="checked"' : '').$disable_str_1.'/>';
			$tmod[]='';
		} else {
			$trow[]='&nbsp;';
			$tmod[]='';
		}
		if ($item->getFieldType() == TrackerExtraField::TYPE_STATUS 
			|| $item->getFieldType() == TrackerExtraField::TYPE_RADIO
			|| $item->getFieldType() == TrackerExtraField::TYPE_SELECT) {
			$contents = '<input type="checkbox" name="use_workflow['.$item->getTrackerExtraFieldID().']" value="1" '.(( $item->getUseWorkFlow()) ? 'checked="checked"' : '').$disable_str_1.'/>';
			
                        $transitions = false;
						$tooltipDescription = '';
                        if ($item->getUseWorkflow()) {
	                        $transitions_count = 0;
                            //Looking for transitions
                            if (count($item->getTrackerExtraFieldElements()) > 0){
                                foreach ($item->getTrackerExtraFieldElements() as $trackerExtraFielElement){
                                    if ($trackerExtraFielElement->getTrackerExtraFieldElementTransitionsRelatedByElementId()){
                                        $transitions = true;
                                        foreach ($trackerExtraFielElement->getTrackerExtraFieldElementTransitionsRelatedByElementId() as $transition){
                                            $tooltipDescription .= $trackerExtraFielElement->getElementName().' => '.$transition->getTrackerExtraFieldElementRelatedByToElementId()->getElementName().'</br>';
	                                        $transitions_count++;
                                        }
                                    }
                                }
                            }

	                        $tooltipDescription = '';
	                        //Preprocesses tooltip string
	                        if ($transitions_count > 0) {
	                            $tooltipDescription = $Lang->getSessText('TrackerExtraField.DefineTooltip', array($transitions_count));
	                            $tooltipDescription = StringUtils::generateTooltip($tooltipDescription);
	                        }

                            //If user defined some transition it shows a blue ball and a transitions resume tooltip
                            if ($transitions){
                                $contents .= "<span onMouseOver=\"tooltip('{$tooltipDescription}')\" onmouseout=\"tooltipexit()\"><a href=\"".GFUrl::getUrl()->currentBaseUrl('WorkflowMatrix', array('tracker_extra_field_id' => $item->getTrackerExtraFieldId())).'">'.$Lang->getText('WorkflowMatrix.Define')."</a><img class=\"workflowBallTooltip\" src=\"/images/blue-ball.gif\"/></span>"; 
                            } else {
                            //If the user didn't define some transition it shows a grey ball tooltip with a "No transitions defined" note   
                                $contents .= "<span onMouseOver=\"tooltip('No transitions defined')\" onmouseout=\"tooltipexit()\"><a href=\"".GFUrl::getUrl()->currentBaseUrl('WorkflowMatrix', array('tracker_extra_field_id' => $item->getTrackerExtraFieldId())).'">'.$Lang->getText('WorkflowMatrix.Define')."</a><img class=\"workflowBallTooltip\" src=\"/images/grey-ball.gif\"/></span>";
                            }
                        }
                        
                            
			$trow[]=$contents;
			$tmod[]='';
		} else {
			$trow[]='&nbsp;';
			$tmod[]='';
		}
		if ($item->getFieldType() == TrackerExtraField::TYPE_STATUS 
			|| $item->getFieldType() == TrackerExtraField::TYPE_RADIO
			|| $item->getFieldType() == TrackerExtraField::TYPE_SELECT) {
			$str = '<input type="checkbox" name="use_commit_rule['.$item->getTrackerExtraFieldID().']" value="1" '.(( $item->getUseCommitRule()) ? 'checked="checked"' : '').$disable_str_1.'/>';
			if ($item->getUseCommitRule()) {
				$url = GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldEdit', array('tracker_extra_field_id' => $item->getTrackerExtraFieldID(), 'tracker_id' => $item->getTrackerID()));
				$str .= ' <a href="'.$url.'">'.$Lang->getText('TrackerExtraField.DefineCommitRules').'</a>';
			}
			
			$trow[]=$str;
			$tmod[]='';
		} else {
			$trow[]='&nbsp;';
			$tmod[]='';
		}

        $trow[] = '&nbsp;';
        $tmod[] = '';

		// Serial fields dont use is_required
  		if($item->getFieldType() == TrackerExtraField::TYPE_SERIAL){
  			$trow[]='&nbsp';
  			$tmod[]='';
  		} else {
			$trow[]='<input type="checkbox" name="is_required['.$item->getTrackerExtraFieldID().']" '.(($item->getIsRequired()) ? 'checked="checked"' : '' ).'" size="3" maxlength="3" '.$disable_str_2.'/>';
			$tmod[]='';
		}
		//A serial field can't be shown on submit
		if($item->getFieldType() == TrackerExtraField::TYPE_SERIAL){
  			$trow[]='&nbsp';
  			$tmod[]='';
  		} else { 
			$trow[]='<input type="checkbox" name="show_on_submit['.$item->getTrackerExtraFieldID().']" '.(($item->getShowOnSubmit()) ? 'checked="checked"' : '' ).'" size="3" maxlength="3" '.$disable_str_2.'/>';
		}
		$trow[]='<input type="checkbox" name="show_on_browse['.$item->getTrackerExtraFieldID().']" '.(($item->getUseInQuickBrowse()) ? 'checked="checked"' : '' ).'" size="3" maxlength="3" '.$disable_str_2.'/>';
		$tmod[]='';
		$trow[]='<input type="text" id="EFOrder'.$item->getTrackerExtraFieldID().'" name="field_order['.$item->getTrackerExtraFieldID().']" value="'.$item->getFieldOrder().'" size="3" maxlength="3" />';
		$tmod[]='';

		if ($item->getAlias() == 'zduration'
			|| $item->getAlias() == 'zpercentcomplete' 
			|| $item->getAlias() == 'zestimatedeffort') {
            $str_edit='';
			$str_delete='';
		} else {
            $str_edit='<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldEdit').'&amp;tracker_id='.$item->getTrackerID().'&amp;tracker_extra_field_id='.$item->getTrackerExtraFieldID() .'">'. $Lang->getText('Edit') .'</a>&nbsp;&nbsp;';
            if ($item->getFieldType() != TrackerExtraField::TYPE_STATUS) {
                $str_delete='<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldDelete').'&amp;tracker_id='.$item->getTrackerID().'&amp;tracker_extra_field_id='.$item->getTrackerExtraFieldID() .'">'. $Lang->getText('Delete') .'</a>&nbsp;&nbsp;';
            } else {
                $str_delete='';
            }
		}

        $trow[] = $str_edit . $str_delete;
        $tmod[]='';

		echo $theme->tableRow($trow, $tmod, false, 'id="jsExtraFieldList_EFOrder'.$item->getTrackerExtraFieldID().'"');
	}

	echo '</tbody>';

	$trow = array ($Lang->getText('TrackerExtraField.AutoAssignDont'),'&nbsp;','<input type="radio" name="auto_assign_to" value="" '.(($was_auto_assign) ? '' : 'checked="checked"').'/>','&nbsp;','&nbsp;','&nbsp;','&nbsp;','&nbsp;','&nbsp;','&nbsp;','&nbsp;');
	echo $theme->tableRow($trow);

}
echo $theme->tableBottom();
?><br />
<?php echo HTMLWidget::button("submit", $Lang->getText('Common.Submit')); ?>

<?php
echo '<br />';
echo '<br />';
echo '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldAdd').'&amp;tracker_id='.$Tracker->getTrackerID().'&amp;field_type='.TrackerExtraField::TYPE_CHECKBOX .'">'. $Lang->getText('TrackerExtraField.AddCheckbox') .'</a><br />';
echo '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldAdd').'&amp;tracker_id='.$Tracker->getTrackerID().'&amp;field_type='.TrackerExtraField::TYPE_MULTISELECT .'">'. $Lang->getText('TrackerExtraField.AddMultiSelect') .'</a><br />';
echo '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldAdd').'&amp;tracker_id='.$Tracker->getTrackerID().'&amp;field_type='.TrackerExtraField::TYPE_SELECT .'">'. $Lang->getText('TrackerExtraField.AddSelect') .'</a><br />';
echo '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldAdd').'&amp;tracker_id='.$Tracker->getTrackerID().'&amp;field_type='.TrackerExtraField::TYPE_RADIO .'">'. $Lang->getText('TrackerExtraField.AddRadio') .'</a><br />';
echo '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldAdd').'&amp;tracker_id='.$Tracker->getTrackerID().'&amp;field_type='.TrackerExtraField::TYPE_TEXT .'">'. $Lang->getText('TrackerExtraField.AddTextField') .'</a><br />';
echo '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldAdd').'&amp;tracker_id='.$Tracker->getTrackerID().'&amp;field_type='.TrackerExtraField::TYPE_TEXTAREA .'">'. $Lang->getText('TrackerExtraField.AddTextArea') .'</a><br />';
echo '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldAdd').'&amp;tracker_id='.$Tracker->getTrackerID().'&amp;field_type='.TrackerExtraField::TYPE_RELEASE .'">'. $Lang->getText('TrackerExtraField.AddRelease') .'</a><br />';
echo '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldAdd').'&amp;tracker_id='.$Tracker->getTrackerID().'&amp;field_type='.TrackerExtraField::TYPE_DATE .'">'. $Lang->getText('TrackerExtraField.AddDate') .'</a><br />';
echo '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldAdd').'&amp;tracker_id='.$Tracker->getTrackerID().'&amp;field_type='.TrackerExtraField::TYPE_NUMERIC .'">'. $Lang->getText('TrackerExtraField.AddNumeric') .'</a><br />';

// Add Serial Field
echo '<a href="'. GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldAdd').'&amp;tracker_id='.$Tracker->getTrackerID().'&amp;field_type='.TrackerExtraField::TYPE_SERIAL . '">' . $Lang->getText('TrackerExtraField.AddSerial'). '</a><br />';

// Add Multi Select member
echo '<a href="' . GFUrl::getUrl()->currentBaseUrl('TrackerExtraFieldAdd').'&amp;tracker_id='.$Tracker->getTrackerID().'&amp;field_type='.TrackerExtraField::TYPE_MEMBERS . '">' . $Lang->getText('TrackerExtraField.AddMembers') . '</a><br />';
?>

<script type="text/javascript">

//Binds on-change events to the "enabled" checkbox for intrinsic fields
$("input.toggle-enabled").bind('change', function() {
	var val = this.checked;
	if (!val) {
		$(this).parent().parent().find(".toggle-group").each(function() {
			this.checked = false; // toggle the checkbox
			$(this).attr('disabled','disabled');
		})
	} else {
		$(this).parent().parent().find(".toggle-group").each(function() {
			$(this).removeAttr('disabled');
		})
	}
});

window.onload=window.onload=function(){oldload();convertTrees();window.setTimeout('expandableExpandItem()',100);docmanExpandTree();}
</script>
<?php
$theme->footer($params);
?>
