<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a TrackerExtraField and saves it to the 
// database
//
require_once('plugins/tracker/lib/TrackerExtraField.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$TrackerExtraField = TrackerExtraFieldPeer::retrieveByPk($_POST['tracker_extra_field_id']);
$type = $TrackerExtraField->getFieldType();
$Tracker = $TrackerExtraField->getTracker();
if (!is_object($Tracker) || !$Tracker->userIsAdmin()) {
	throw new PermissionDeniedException();
}

$errors = array();
$language = Language::getSessLang();

//
// TODO Validate the values passed by $_POST. Store the errors in $error like
// $errors["field_name"] = error_description
//

$Project = GFUrl::getUrl()->getProject();
$keys = array_keys($_POST['element_name']);
$errors=array();

$arr_element_name = array();
$arr_alias = array();
$element_name_repeated = false;
$alias_repeated = false;
$statuses_valid = true;
$valid_status_ids = array_keys(Tracker::getStatuses());
foreach ($keys as $element_id) {
    $element_name = trim(Validator::sanitizeString($_POST['element_name'][$element_id]));
    $alias = trim(strtolower(Validator::sanitizeString($_POST['alias'][$element_id])));

    if (in_array($element_name, $arr_element_name)) {
        $element_name_repeated = true;
    } else {
        $arr_element_name[] = $element_name;
    }

	if ($alias != '') {
	    if (in_array($alias, $arr_alias)) {
	        $alias_repeated = true;
	    } else {
	        $arr_alias[] = $alias;
	    }
	}
  if ($type == TrackerExtraField::TYPE_STATUS) {
    $status_id = Validator::sanitizeInt($_POST['status'][$element_id]);
    if (!in_array($status_id, $valid_status_ids)) {
      $statuses_valid = false;
    }
  }
}

if ($element_name_repeated) {
    $errors[] = $language->getText('TrackerExtraFieldElement.ElementNameNotUnique');
}

if ($alias_repeated) {
    $errors[] = $language->getText('TrackerExtraFieldElement.AliasNotUnique');
}

if (!$statuses_valid) {
    $errors[] = $language->getText('TrackerExtraFieldElement.InvalidStatus');
}

if (!$element_name_repeated && !$alias_repeated && $statuses_valid) {
    foreach ($keys as $element_id) {
        try {
            $TrackerExtraFieldElement = TrackerExtraFieldElementPeer::retrieveByPk($element_id);
            $Tracker=$TrackerExtraFieldElement->getTrackerExtraField()->getTracker();
            if (!is_object($Tracker) || !$Tracker->userIsAdmin()
                || $Tracker->getProject()->getProjectId() != $Project->getProjectId()) {
                throw new PermissionDeniedException();
            }

            $TrackerExtraFieldElement->setElementName( Validator::sanitizeString($_POST['element_name'][$element_id]) );
            $alias = Validator::sanitizeString($_POST['alias'][$element_id]);
            if (!preg_match('/^[a-zA-Z0-9_-]*$/i', $alias)) {
                throw new InvalidDataException(array($language->getText('TrackerExtraFieldElement.AliasNotValid')));
            }

            $TrackerExtraFieldElement->setAlias( $alias );

        //	$TrackerExtraFieldElement->setStatusId( Validator::sanitizeInt($_POST['status_id'][]) );
            if ($type == TrackerExtraField::TYPE_STATUS) {
              $TrackerExtraFieldElement->setStatusId(
                Validator::sanitizeInt($_POST['status'][$element_id])
              );
            }

            if (isset($_POST['default_selected']) && $_POST['default_selected']==$element_id) {
                $selected=1;
            } else {
                $selected=0;
            }
            $TrackerExtraFieldElement->setDefaultSelected( $selected );
            if (isset($_POST['can_commit']) && isset($_POST['can_commit'][$element_id]) && $_POST['can_commit'][$element_id]=='1' ) {
                $TrackerExtraFieldElement->setCanCommit(true);
            } else {
                $TrackerExtraFieldElement->setCanCommit(false);
            }
            $TrackerExtraFieldElement->setSortOrder( Validator::sanitizeInt($_POST['sort_order'][$element_id]) );
            if ($TrackerExtraFieldElement->getTrackerExtraField()->getUseAutoAssign()) {
                $TrackerExtraFieldElement->setAutoAssignTo(Validator::sanitizeInt($_POST['auto_assign_to'][$element_id]));
            } else {
                $TrackerExtraFieldElement->setAutoAssignTo(100);
            }

            $TrackerExtraFieldElement->save();
        } catch (InvalidDataException $ex) {
            $errors = array_merge($errors, $ex->getErrors());               // attach object errors to validation errors
            break;
        }
    }
}


if (count($errors) == 0) {
    if ($type != TrackerExtraField::TYPE_STATUS) {
	
        if ($type == TrackerExtraField::TYPE_TEXT) {
            $attribute1 = (isset($_POST['attribute1']) && !empty($_POST['attribute1'])) ? Validator::sanitizeInt($_POST['attribute1']) : 50;
            $attribute2 = (isset($_POST['attribute2']) && !empty($_POST['attribute2'])) ? Validator::sanitizeInt($_POST['attribute2']) : 50;
            $default_value = (isset($_POST['default_value']) ? Validator::sanitizeString($_POST['default_value']) : '');
        } elseif ($type == TrackerExtraField::TYPE_TEXTAREA) {
            $attribute1 = (isset($_POST['attribute1']) && !empty($_POST['attribute1'])) ? Validator::sanitizeInt($_POST['attribute1']) : 5;
            $attribute2 = (isset($_POST['attribute2']) && !empty($_POST['attribute2'])) ? Validator::sanitizeInt($_POST['attribute2']) : 40;
        } elseif ($type == TrackerExtraField::TYPE_NUMERIC) {
            $default_value = (isset($_POST['default_value']) && is_numeric($_POST['default_value'])) ? Validator::sanitizeString($_POST['default_value']) : '';
            $attribute1 = (isset($_POST['attribute1'])) ? Validator::sanitizeInt($_POST['attribute1']) : 0;
            $attribute2 = (isset($_POST['attribute2'])) ? Validator::sanitizeInt($_POST['attribute2']) : 0;
        } else {
            $attribute1 = (isset($_POST['attribute1'])) ? Validator::sanitizeInt($_POST['attribute1']) : 0;
            $attribute2 = (isset($_POST['attribute2'])) ? Validator::sanitizeInt($_POST['attribute2']) : 0;
            $default_value = '';
        }

        $TrackerExtraField->setFieldName( Validator::sanitizeString($_POST['field_name']) );
        $TrackerExtraField->setAttribute1($attribute1);
        $TrackerExtraField->setAttribute2($attribute2);
        $TrackerExtraField->setIsRequired( isset($_POST['is_required']) ? 1 : 0 );
        //$TrackerExtraField->setAlias( Validator::sanitizeString($_POST['alias']) );
        $TrackerExtraField->setFieldOrder( Validator::sanitizeInt($_POST['field_order']) );
        $TrackerExtraField->setUseInQuickBrowse( isset($_POST['use_in_quick_browse']) ? 1 : 0 );
        $TrackerExtraField->setShowOnSubmit( isset($_POST['show_on_submit']) || isset($_POST['is_required'])  ? 1 : 0 );
        $TrackerExtraField->setDefaultValue($default_value);

        try {
            $TrackerExtraField->save();
            GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerExtraFieldBrowse'), array('feedback' => $language->getText('TrackerExtraField.EditFeedback'),'tracker_id'=>$Tracker->getTrackerId()));
        } catch (InvalidDataException $ex) {
            $errors = array_merge($errors, $ex->getErrors());               // attach object errors to validation errors
            $feedback = implode('<br />', $errors);
            include('TrackerExtraFieldEdit.php');
        }
    } else {
        GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerExtraFieldBrowse'), array('feedback' => $language->getText('TrackerExtraField.EditFeedback'),'tracker_id'=>$Tracker->getTrackerId()));
    }
} else {
	$feedback = implode('<br />', $errors);
	include('TrackerExtraFieldEdit.php');
}
?>
