<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code creates a new TrackerExtraFieldElement and saves it to the 
// database
//
require_once('plugins/tracker/lib/TrackerExtraFieldElement.php');
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}


$TrackerExtraField = TrackerExtraFieldPeer::retrieveByPk($_POST['tracker_extra_field_id']);
$Tracker = $TrackerExtraField->getTracker();
if (!is_object($Tracker) || !$Tracker->userIsAdmin()) {
	throw new PermissionDeniedException();
}

$TrackerExtraFieldElement = new TrackerExtraFieldElement();
$TrackerExtraFieldElement->setTrackerExtraField($TrackerExtraField);

$errors = array();
$language = Language::getSessLang();

//TODO: do some verification, add error strings to $errors

if (count($errors) == 0) {
	try {
		$element_name = isset($_POST['element_name']) ? trim(Validator::sanitizeString($_POST['element_name'])) : "";
        $alias = isset($_POST['alias']) ? trim(strtolower(Validator::sanitizeString($_POST['alias']))) : "";

        //Validate alias format
        if (!preg_match('/^[a-zA-Z0-9_-]*$/i', $alias)) {
            $errors[] = $language->getText('TrackerExtraFieldElement.AliasNotValid');
        }

        //Verify the element name is unique
        if (!TrackerExtraFieldPeer::isElementNameUnique($element_name, $TrackerExtraField->getTrackerExtraFieldId())) {
            $errors[] = $language->getText('TrackerExtraFieldElement.ElementNameNotUnique');
        }

        //Verify the alias is unique
        if (!TrackerExtraFieldPeer::isAliasUnique($alias, $TrackerExtraField->getTrackerExtraFieldId())) {
            $errors[] = $language->getText('TrackerExtraFieldElement.AliasNotUnique');
        }

        $TrackerExtraFieldElement->setAlias($alias);
        $TrackerExtraFieldElement->setElementName($element_name);
		$TrackerExtraFieldElement->setStatusId( isset($_POST['status_id']) ? Validator::sanitizeInt($_POST['status_id']) : Tracker::STATUS_OPEN );
		//$TrackerExtraFieldElement->setDefaultSelected( Validator::sanitizeString($_POST['default_selected']) );
		//$TrackerExtraFieldElement->setSortOrder( Validator::sanitizeInt($_POST['sort_order']) );

        if (count($errors) > 0) {
            throw new InvalidDataException(array());
        } else {
            $TrackerExtraFieldElement->save();
            GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerExtraFieldEdit'), array('feedback' => $language->getText('TrackerExtraFieldElement.AddFeedback'),'tracker_extra_field_id'=>$TrackerExtraField->getTrackerExtraFieldId()));
        }
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());		// attach object errors to validation errors
		$feedback = '* '.implode('<br />* ', $errors);
		include('TrackerExtraFieldElementAdd.php');
	}
}
?>
