<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version		$Id$
 * @author		Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This	file is	part of	GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugins/tracker/lib/TrackerTransitionScript.php');
require_once('GFUrl.php');

if (!GFConf::get('tracker.enableWorkflowScripts')) {
	throw new PermissionDeniedException();
}

$session = Session::getSession();
$lang = $session->getLanguage();
$url = GFUrl::getUrl();

// If $project is null, we're on the site admin, editing the global scripts
$project = $url->getProject();

if (is_null($project)) {
	include_once('lib/html/templates/SiteAdminTemplate.php');
	$theme = new SiteAdminTemplate();
} else {
	include_once('lib/html/templates/ProjectAdminTemplate.php');
	$theme = new ProjectAdminTemplate();
}

$params = array(
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $lang->getText('Tracker.WorkflowScriptsTitle')
		);
if (!is_null($project)) {
	if (!$project->getEnableWorkflowScripts()) {
		throw new PermissionDeniedException();
	}
	$params['project'] = $project;
	$scripts = TrackerTransitionScriptPeer::getProjectScripts($project);
} else {
	$scripts = TrackerTransitionScriptPeer::getGlobalScripts();
}
$theme->header($params);



$columns = array(
	$lang->getText('Tracker.WorkflowScriptDescriptionTitle'),
	$lang->getText('Tracker.WorkflowScriptType'),
	$lang->getText('Tracker.WorkflowScriptsScriptTitle'),
	);

if ($session->isSiteAdmin() || (!is_null($project) && $project->userIsAdmin())) {
	$columns[] = $lang->getText('Common.Edit');
	$columns[] = $lang->getText('Common.Delete');
}

echo $theme->tableTop($columns);

$count = 0;
foreach ($scripts as $script) {
	$count++;
	
	$trow = array();
	$tmod = array();
	$trow[] = $script->getDescription();
	$tmod[] = '';	
	$trow[] = ($script->getScriptType() == TrackerTransitionScript::SCRIPT_TYPE_LOCAL ? $lang->getText('Tracker.WorkflowScriptTypeLocal') : $lang->getText('Tracker.WorkflowScriptTypeUploaded'));
	$tmod[] = '';
	$trow[] = $script->getScriptName();
	$tmod[] = '';
	
	if ($session->isSiteAdmin() || (!is_null($project) && $project->userIsAdmin())) {
		// Don't show edit/delete buttons for global scripts in the project page
		if (!$project || ($project && $script->getProjectId())) {
			$trow[] = '<a href="'.$url->currentBaseUrl('TrackerTransitionScriptForm', array('tracker_transition_script_id' => $script->getPrimaryKey())).'">'.$lang->getText('Common.Edit').'</a>';
			$tmod[] = '';
			$trow[] = '<a href="'.$url->currentBaseUrl('TrackerTransitionScriptDelete', array('tracker_transition_script_id' => $script->getPrimaryKey())).'">'.$lang->getText('Common.Delete').'</a>';
			$tmod[] = '';
		} else if ($project) {
			$trow[] = $lang->getText('Tracker.WorkflowScriptSiteWideScript');
			$tmod[] = 'colspan="2"';
		}
	}
	
	echo $theme->tableRow($trow, $tmod);
}

if ($count == 0) {
	echo $theme->tableFullRow($lang->getText('Tracker.WorkflowScriptsNoneFound'));
}

echo $theme->tableBottom(array($url->currentBaseUrl('TrackerTransitionScriptForm') => $lang->getText('Tracker.WorkflowScriptAdd')));

$theme->footer($params);
?>
