<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('plugins/tracker/lib/TrackerTransitionScript.php');
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');

$errors = array();
$language = Language::getSessLang();
$url = GFUrl::getUrl();
$project = $url->getProject();

if ($_SERVER['REQUEST_METHOD'] != 'POST' || !GFConf::get('tracker.enableWorkflowScripts') || ($project && !$project->getEnableWorkflowScripts())) {
	throw new PermissionDeniedException();
}

$tracker_transition_script_id = (isset($_REQUEST['tracker_transition_script_id']) ? Validator::sanitizeInt($_REQUEST['tracker_transition_script_id']) : null);

if ($tracker_transition_script_id) {
	// Editing
	$script = TrackerTransitionScriptPeer::retrieveByPk($tracker_transition_script_id);
} else {
	// Adding
	$script = new TrackerTransitionScript();
}

$errors = array();

try {
	$script->setDescription(Validator::sanitizeString($_REQUEST['description']));
	if ($project) {
		$script->setProject($project);
	}
	
	$scriptType = (isset($_REQUEST['script_type']) ? Validator::sanitizeInt($_REQUEST['script_type']) : 0);
	if ($scriptType == TrackerTransitionScript::SCRIPT_TYPE_LOCAL) {
		$script->setScriptPath(Validator::sanitizeString($_REQUEST['script_path']));
	} elseif ($scriptType == TrackerTransitionScript::SCRIPT_TYPE_UPLOADED) {
		$script->setUploadedScript($_FILES['uploaded_script']);
	}
	
	$script->save();
} catch (InvalidDataException $ex) {
	$errors = $ex->getErrors();
}

if (count($errors) > 0) {
	$feedback = '* '.implode('<br />* ', $errors);
	include('TrackerTransitionScriptForm.php');
} else {
	GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerTransitionScriptBrowse'), array('feedback' => $language->getText('Tracker.WorkflowScriptAddFeedback')));
}

?>
