<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('lib/html/association.php');
require_once('lib/html/TagsRenderer.php');
require_once('lib/html/HTMLWidget.php');
require_once('lib/html/GFCKEditor.php');

require_once('plugins/tracker/lib/TrackerItemScmCommit.php');
require_once('plugins/tracker/lib/TrackerItemScmCommitPeer.php');
require_once('plugins/scm/lib/ScmCommitFile.php');
require_once('plugins/scm/lib/RepositoryBrowser.php');
require_once('plugins/scm/lib/SCMRepositoryFactory.php');

if (! array_key_exists('extra_css', $params) || ! is_array($params['extra_css'])) {
	$params['extra_css'] = array();
}
$params['extra_css'][] = $theme->getThemeRoot().'/tracker-item-tabbed.css';

$theme->header($params);

// These functions are display-related, so still belong in this template
function stripExtraTableElements($html) {

	// Why does this output include extra p elements before
	// and after the table??
	$start = stripos($html, '<table');
	if ($start !== FALSE) {
		$html = substr($html, $start);
	}

	$end = strripos($html, '</table>', -1);
	if ($end !== FALSE) {
		$html = substr($html, 0, $end + 8);
	}

	return $html;
}

function writeAuditTable($Lang, $theme, $items) {

	$ret = $theme->tableTop(array(
		$Lang->getText('AuditTrail.FieldNameName'),
		$Lang->getText('AuditTrail.OldValueName'),
		$Lang->getText('AuditTrail.NewValueName')
	));

	foreach ($items as $item) {
		$ret .= $theme->tableRow(array(
			$item->getFieldName(),
			$item->getOldValue(),
			$item->getNewValue()
		));
	}

	$ret .= $theme->tableBottom();

	return stripExtraTableElements($ret);
}

function writeCommitItem($Lang, $theme, $browser, $item) {
	$ret = commitTable($Lang, $theme, $browser, $item);
	return stripExtraTableElements($ret);
}

function writeFileItem($Lang, $theme, $item) {

	$su = new StringUtils();

	$url = GFUrl::getUrl()->downloadUrlFromFS($item, false);
	$img_url = $theme->getImgUrl('download.png');
	$file_size = $su->bytesToSize($item->getFileSize());

	$ret = "<p class=\"file-item\"><a href=\"{$url}\" title=\"{$item->getFileName()}\"><img class=\"download\" src=\"{$img_url}\" />{$item->getFileName()}</a>({$item->getFileType()}, {$file_size})</p>";

	return $ret;
}

function writeTimeEntry($Lang, $item) {

	return $Lang->getText('TrackerItemTime.Activity', array(
		$item->getUser()->getFullName(),
		$item->getHours(),
		$item->getTrackerTimeCode()->getTimeName()
	));
}

function writeTI($ti) {

	$ret = "<p class=\"tracker-item-ref\">
		<a href=\"{$ti->getTrackerItemUrl()}\" title=\"{$ti->getDetails()}\">
			[#{$ti->getTrackerItemId()}]: {$ti->getSummary()}</a>
		&nbsp;{$ti->getStatusName()}
		</p>
	";

	return $ret;
}

function writeAssociation($ti, $assoc) {

	$title_arr = PluginPeer::getRelatedItemTitle($assoc[0], $assoc[1]);

	$ret = '';
	if (count($title_arr) > 0) {
		$ret = "<p class=\"association-ref\"><a href=\"{$title_arr['url']}\" title=\"{$title_arr['title_full']}\">{$title_arr['title']}</a></p>";
	}

	return $ret;
}

function tagLink($Lang, $ti, $tag) {

	$tracker = $ti->getTracker();
	$params = array(
		'section' => TrackerItem::SECTION,
		'ref_id' => $ti->getTrackerItemId(),
		'tag_name' => $tag,
		'subsection_id' => $tracker->getTrackerId()
	);
	$title = $Lang->getText('Tag.RelatedItemsLink');
	$url = GFUrl::getUrl()->projectURL($tracker->getProject(), 'TagSearch', 'tag', '', $params);
	$ret = "<a class=\"tag-link\" href=\"$url\" title=\"$title\">$tag</a>";

	return $ret;
}

// Make sure key context vars are set
$Lang = Session::getSession()->getLanguage();

// Extract vars from params array
$Tracker = $params['tracker'];
$TrackerItem = $params['tracker_item'];
$user = $params['user'];

$tefr = $params['tefr'];
$uploadHandler = $params['uploadHandler'];
$start_date = $params['start_date'];
$end_date = $params['end_date'];

$activity_count = $params['activity']['activity_count'];
$activity_list = $params['activity']['activity_list'];
$all_users = $params['activity']['all_users'];
$related_count = $params['related']['related_count'];
$related_list = $params['related']['related_list'];
?>

<div class="tracker-item">

<?php include('TrackerItemEdit-TemplateChooser-tpl.php'); ?>

<div class="discuss-link">
	<?php echo  HTMLWidget::discuss(
		$Tracker->getProject(),
		'Discuss this tracker item',
		TrackerItem::SECTION,
		$TrackerItem->getTrackerItemId(),
		$TrackerItem->getSummary());
	?>
</div>

<?php if (! is_null($user)) { ?>
	<script language="javascript" type="text/javascript">
		$(document).on( 'click', 'input.action-submit', function(){
			var selected = $(this).prev().find(":selected");
			if (undefined !== selected) {
				alert($(selected).val());
			}
		});
	</script>

	<div class="tracker-item-actions">

<?php
	if ($TrackerItem->userIsMonitoring($user)) {
		$text = $Lang->getText('TrackerItem.StopMonitor');
		$url = GFUrl::getUrl()->getMonitorUrl(TrackerItem::SECTION, $TrackerItem->getPrimaryKey(), null, false);
	} else {
		$text = $Lang->getText('TrackerItem.Monitor');
		$url = GFUrl::getUrl()->getMonitorUrl(TrackerItem::SECTION, $TrackerItem->getPrimaryKey(), null, true);
	}
?>
		<select>
<?php if ($Tracker->userIsAdmin()) { ?>
			<option value="<?php echo GFUrl::getUrl()->currentBaseUrl('TrackerItemMove', array('tracker_item_id' => $TrackerItem->getTrackerItemId())) ?>">
				<?php echo $Lang->getText('TrackerItem.MoveItem'); ?>
			</option>
			<option value="<?php echo GFUrl::getUrl()->currentBaseUrl('TrackerItemDelete', array('tracker_item_id' => $TrackerItem->getTrackerItemId())) ?>">
				<?php echo $Lang->getText('TrackerItem.DeleteItem'); ?>
			</option>
			<option value="<?php echo GFUrl::getUrl()->currentBaseUrl('TrackerItemClone', array('tracker_item_id' => $TrackerItem->getTrackerItemId())) ?>">
				<?php echo $Lang->getText('TrackerItem.Clone'); ?>
			</option>
			<option value="<?php echo GFUrl::getUrl()->currentBaseUrl('TrackerItemMerge', array('tracker_item_id' => $TrackerItem->getTrackerItemId())) ?>">
				<?php echo $Lang->getText('TrackerItem.MergeItem'); ?>
			</option>
<?php } // if ($Tracker->userIsAdmin()) ?>
			<option value="$url"><?php echo $text ?></option>
		</select>
		<input type="button" class="action-submit" style="min-width: 40px;" value="<?php echo $Lang->getText('Common.Go'); ?>" />
	</div>
<?php
} // if ! isnull($user) ?>

<h3 style="padding-top: 6px;">&nbsp;[#<?php echo $TrackerItem->getPrimaryKey() ?>] <?php echo htmlspecialchars($TrackerItem->getSummary(false)) ?></h3>
<div class="tracker-item-subhead">
	<!-- This is the submitter's avatar-->
	<div class="avatar-box">
		<?php
		$submitUser = UserPeer::retrieveByPk($TrackerItem->getSubmittedBy());
		if (GFConf::get('system.displayAvatar')) {
			$url = GFUrl::getUrl()->userURL($submitUser);
			echo "<a href=\"$url\" title=\"{$submitUser->getUnixName()}\">{$submitUser->getAvatarHTML()}</a>";
			if (array_key_exists($submitUser->getUnixName(), $all_users)) unset($all_users[$submitUser->getUnixName()]);
		} ?>
		<div class="label">
			<?php echo $Lang->getText('TrackerItem.SubmittedByOn', array($submitUser->getUserLink(), $TrackerItem->getCreateDate('r')));?>
		</div>
	</div>

	<!-- This is the list of assigned users, for open items only -->
	<?php
	if ($TrackerItem->getStatusId() == Tracker::STATUS_OPEN) { ?>
		<div class="avatar-box">
	<?php
		$assignees = $TrackerItem->getAssignedUsers();
		$assignee_list = array();
		if (count($assignees) == 0) {
			$assignee_list[] = $Lang->getText('Tracker.Nobody');
		} else {
			foreach ($assignees as $assignee) {
				if (GFConf::get('system.displayAvatar')) {
					$url = GFUrl::getUrl()->userURL($assignee);
					echo "<a href=\"$url\" title=\"{$assignee->getUnixName()}\">{$assignee->getAvatarHTML()}</a>";
				}
				$assignee_list[] = $assignee->getUserLink();
				if (array_key_exists($assignee->getUnixName(), $all_users)) unset($all_users[$assignee->getUnixName()]);
			}
		} ?>
			<div class="label">
				<?php echo $Lang->getText('TrackerItem.AssignedToLabel') . ' '. implode(', ', $assignee_list);?>
			</div>
		</div>
	<?php } else { ?>

	<!-- This is the avatar for closed by user, for closed items only -->
		<div class="avatar-box">
			<?php
			$closedBy = $TrackerItem->getClosedByUser();
			if ($closedBy instanceof User) {
				if (array_key_exists($closedBy->getUnixName(), $all_users)) unset($all_users[$closedBy->getUnixName()]);
				if (GFConf::get('system.displayAvatar')) {
					$url = GFUrl::getUrl()->userURL($closedBy);
					echo "<a href=\"$url\" title=\"{$closedBy->getUnixName()}\">{$closedBy->getAvatarHTML()}</a>";
				} ?>
				<div class="label">
					<?php echo $Lang->getText('TrackerItem.ClosedByOn', array($closedBy->getUserLink(), $TrackerItem->getCloseDate('r')));?>
				</div>
			<?php } ?>
		</div>
	<?php } ?>

	<!-- This is the list of all users with activity on the item, not including the users above -->
	<?php if (count($all_users) > 0) { ?>
		<div class="label">
			<br /><?php echo $Lang->getText('TrackerItem.WorkedOnBy', array(count($all_users)));?>:<br />
		</div>
		<div class="avatar-box">
			<?php
			foreach ($all_users as $involved) {
				if ($involved instanceof User) {
					if (GFConf::get('system.displayAvatar')) {
						$url = GFUrl::getUrl()->userURL($involved);
						echo "<a href=\"$url\" title=\"{$involved->getUnixName()}\">{$involved->getAvatarHTML()}</a>";
					} ?>
					<div class="label">
						<?php echo "<a href=\"$url\" title=\"{$involved->getUnixName()}\">{$involved->getFullname()}</a>"; ?>
					</div>
			<?php } ?>
		<?php } ?>
		</div>
		
	<?php } ?>
</div>

<script language="javascript" type="text/javascript" src="/datetimepicker.js"></script>

<?php echo $uploadHandler->getJavascriptSetup(); ?>

<SCRIPT LANGUAGE="JavaScript" type="text/javascript">

function checkRows(textArea){

	if (/*navigator.appName.indexOf("Microsoft Internet Explorer") == 0 ||*/ navigator.appVersion.match("MSIE 6"))
	{
		textArea.style.overflow = 'visible';
		return;
	}

	while (
		textArea.rows > 1 &&
		textArea.scrollHeight < textArea.offsetHeight
	){
		textArea.rows--;
	}

	while (textArea.scrollHeight > textArea.offsetHeight)
	{
		textArea.rows++;
	}

	textArea.rows++;

	return;
}

function cancelClick(el) {
	if ( navigator.appVersion.match("MSIE 7") ) {
		el.href = "java"+"script:;";
	}
	return false;
}
</SCRIPT>

<script type="text/javascript">
	$(document).ready(function() {
		$('.filter-header', '.tracker-item').on("change", "input.toggle", function(event) {
			var t = $(this).val();
			var selector = "."+t;
			if ($(this).is(":checked")) {
				$(selector, $(this).parent().parent()).fadeIn('slow');
			} else {
				$(selector, $(this).parent().parent()).fadeOut('fast');
			}
		});
	});
</script>

<form name="TrackerItemEdit" action="<?php echo GFUrl::getUrl()->currentBaseURL('TrackerItemEditAction') ?>" method="post" enctype="multipart/form-data">
	<input type="hidden" name="tracker_item_id" value="<?php echo $_REQUEST['tracker_item_id']; ?>" />
	<input type="hidden" name="start" value="<?php echo $start; ?>" />

<div id="trackerTabs" class="<?php echo $params['tab-style'] ?>" style="max-width: 960px;">
	<div class="tabbertab" title="<?php echo $Lang->getText('TrackerItem.DetailsName') ?>" id="detailstab" data-alt-title="<?php echo $Lang->getText('TrackerItem.DetailsName') ?>">
		<table border="0" width="100%">
			<tr>
				<td><strong><?php echo $Lang->getText('TrackerItem.SummaryName'); ?></strong>&nbsp;<br />
				<?php
				$summary = htmlentities($TrackerItem->getSummary(false), ENT_COMPAT, 'UTF-8');
				if ($IS_TECH) { ?>
					<input type="text" name="summary" value="<?php echo $summary; ?>" size="70" maxlength="255" />
				<?php } else {
					echo $summary;
				} ?></td>
				<td style="padding-top: 12px;">
					<?php echo HTMLWidget::button("submit","" .   $Lang->getText('submit.edit') . " ","submit","","");?>
					<?php echo HTMLWidget::button("button","" .   $Lang->getText('Common.Back') . " ","cancel","","window.location = '" .   GFUrl::getUrl()->projectURL(GFUrl::getUrl()->getProject(),'TrackerItemBrowse','tracker','',array('tracker_id' => $Tracker->getTrackerId())) . "'; ")?>
				</td>
			</tr>
			<tr id='details_readonly' class='row_visible'>
				<?php if ($IS_TECH) { ?>
						<td colspan="2"><strong><?php echo $Lang->getText('TrackerItem.DetailsName'); ?></strong>
						<a href="javascript:void(0)" class='enable_link' onclick="e1=document.getElementById('details_textarea');e1.className='row_visible';
						e2=document.getElementById('details_readonly');e2.className='row_hidden';checkRows(document.getElementById('details'));
						return cancelClick(this);"> (Edit)</a><br />

						<span style="margin: .5em 0 1em 12px; float:left;width: 700px;" id="details_readonly"><?php echo wordwrap($TrackerItem->getDetails(false));?></span></td>
				<?php } else {?>
						<td colspan="2"><strong><?php echo $Lang->getText('TrackerItem.DetailsName'); ?></strong><br />
						<?php
						echo wordwrap($TrackerItem->getDetails(false),80);
					} ?></td>
			</tr>
			<tr id='details_textarea' class='row_hidden'>
				<td colspan="2"><strong><?php echo $Lang->getText('TrackerItem.DetailsName'); ?></strong><br />
				<?php if ($IS_TECH) {
					$CKEditor = new GFCKEditor();
					$CKEditor->config['width']='700';
					$CKEditor->textareaAttributes = array('style'=>'overflow:hidden;','rows'=>7,'cols'=>70, 'id'=>'details', 'name'=>'details','onkeyup'=>'checkRow(this)');
					$CKEditor->editor("details",$TrackerItem->getDetails(false));
				?>
				<?php } else {
					//echo '<pre>'.wordwrap($TrackerItem->getDetails(false),80).'</pre>';
					echo wordwrap($TrackerItem->getDetails(false),80);
				} ?>
			</tr>
			<tr>
				<td colspan="2"><strong><?php echo $Lang->getText('TrackerItem.SubmittedByName'); ?>:</strong><br />
				<?php if ($Tracker->userIsAdmin()) {
					echo HTMLWidget::selectBoxFromUsers($Tracker->getProject()->getMembers(), 'submitted_by', $TrackerItem->getSubmittedBy());
				} else {
					echo UserPeer::retrieveByPk($TrackerItem->getSubmittedBy())->getUserLink();
				} ?>
				</td>
			</tr>
			<?php if (GFConf::get('system.displayAvatar') && !$Tracker->userIsAdmin()) { ?>
			<tr>
				<td>
					<?php echo UserPeer::retrieveByPk($TrackerItem->getSubmittedBy())->getAvatarHTML(); ?>
				</td>
			</tr>
			<?php } ?>
			<?php if ($Tracker->getDatatype()==Tracker::DATATYPE_TASK) { ?>
			<tr>
				<td><strong><?php echo $Lang->getText('TrackerItem.StartDate'); ?></strong><span class="required">*</span><br />
				<?php if ($IS_TECH) { ?>
				<input type="text" id="task_start" name="start_date" value="<?php echo $start_date; ?>" size="11" maxlength="11" /><a href="javascript:NewCal('task_start','MMMddyyyy')"><img src="/themes/gforge5/images/cal.gif" width="16" height="16" border="0" alt="Pick a date"/></a>
				<?php } else {
					echo $start_date;
				} ?>
				</td>
				<td><strong><?php echo $Lang->getText('TrackerItem.EndDate'); ?></strong><span class="required">*</span><br />
				<?php if ($IS_TECH) { ?>
				<input type="text" id="task_end" name="end_date" value="<?php echo $end_date; ?>" size="11" maxlength="11" /><a href="javascript:NewCal('task_end','MMMddyyyy')"><img src="/themes/gforge5/images/cal.gif" width="16" height="16" border="0" alt="Pick a date"/></a>
				<?php } else {
					echo $end_date;
				} ?>
				</td>
			</tr>
			<?php } else { ?>
			<tr>
				<td><strong><?php echo $Lang->getText('TrackerItem.OpenDateName'); ?></strong><br />
				<?php echo $TrackerItem->getOpenDate(); ?></td>
				<td><?php if ($TrackerItem->getStatusId()==Tracker::STATUS_CLOSED) { ?><strong><?php echo $Lang->getText('TrackerItem.CloseDateName'); ?></strong><br />
				<?php echo $TrackerItem->getCloseDate(); } ?></td>
			</tr>
			<?php } ?>
			<?php  echo (($IS_TECH) ? $tefr->renderFields('EDIT', $TrackerItem,null,$_POST) : $tefr->renderFieldsReadOnly($TrackerItem) ); ?>
			<tr>
				<td>&nbsp;</td>
				<td style="padding-top: 12px;">
					<?php echo HTMLWidget::button("submit","" .   $Lang->getText('submit.edit') . " ","submit","","");?>
					<?php echo HTMLWidget::button("button","" .   $Lang->getText('Common.Back') . " ","cancel","","window.location = '" .   GFUrl::getUrl()->projectURL(GFUrl::getUrl()->getProject(),'TrackerItemBrowse','tracker','',array('tracker_id' => $Tracker->getTrackerId())) . "'; ")?>
				</td>
			</tr>
		</table>

	</div>
	<div class="tabbertab" title="<?php echo $Lang->getText('TrackerItem.ActivityName') ?>" id="activitytab" data-alt-title="<?php echo $Lang->getText('TrackerItem.ActivityName') ?>">
		<div class="filter-header">
			<input type="checkbox" checked="checked" class="toggle activity-type" id="activity-filter-followup" value="followup" />
			<label for="activity-filter-followup" class="unselectable"><?php echo $Lang->getText('TrackerItemMessage.BodyName')." ({$activity_count['followup']})" ?></label>

			<input type="checkbox" checked="checked" class="toggle activity-type" id="activity-filter-file" value="file" />
			<label for="activity-filter-file" class="unselectable"><?php echo $Lang->getText('TrackerItem.Tabs.Files')." ({$activity_count['file']})" ?></label>

			<input type="checkbox" checked="checked" class="toggle activity-type" id="activity-filter-audit" value="audit" />
			<label for="activity-filter-audit" class="unselectable"><?php echo $Lang->getText('TrackerItem.Tabs.Changes')." ({$activity_count['audit']})" ?></label>

			<input type="checkbox" checked="checked" class="toggle activity-type" id="activity-filter-commit" value="commit" />
			<label for="activity-filter-commit" class="unselectable"><?php echo $Lang->getText('TrackerItem.Tabs.Commits')." ({$activity_count['commit']})" ?></label>

			<input type="checkbox" checked="checked" class="toggle activity-type" id="activity-filter-time" value="time" />
			<label for="activity-filter-time" class="unselectable"><?php echo $Lang->getText('TrackerItem.Tabs.TimeTracking')." ({$activity_count['time']})" ?></label>
		</div>
		<div class="activity-list">
			<?php foreach ($activity_list as $activity_item) { ?>
				<div class="activity-item <?php echo $activity_item['type']; ?>">
					<div class="activity-date"><?php echo $activity_item['date']; ?></div>
					<?php if (GFConf::get('system.displayAvatar')) { ?>
						<div class="activity-user">
							<?php
								$activity_user = $activity_item['user'];
								$url = GFUrl::getUrl()->userURL($activity_user);
								echo "<a href=\"$url\" title=\"{$activity_user->getUnixName()}\">{$activity_user->getAvatarHTML()}</a>";
							?>
						</div>
					<?php } ?>
					<div class="activity-title"><?php echo $activity_item['user']->getUserLink().' '.$activity_item['title']; ?></div>
					<div class="activity-content">
						<?php
						switch ($activity_item['type']) {
							case 'followup': {
								$content = wordwrap($activity_item['item']->getBody(false),80);
								echo $content;
								break;
							}
							case 'audit': {
								echo writeAuditTable($Lang, $theme, $activity_item['item']);
								break;
							}
							case 'commit': {
								echo writeCommitItem($Lang, $theme, $browser, $activity_item['item']);
								break;
							}
							case 'file': {
								echo writeFileItem($Lang, $theme, $activity_item['item']);
								break;
							}
							case 'time': {
								echo writeTimeEntry($Lang, $activity_item['item']);
								break;
							}
							default: {
								echo 'Unknown activity type: '.$activity_item['type'];
								break;
							}
						} ?>
					</div>
				</div>
			<?php } ?>
		</div>
	</div>
	<div class="tabbertab" title="<?php echo $Lang->getText('TrackerItem.RelatedName') ?>" id="relatedtab" data-alt-title="<?php echo $Lang->getText('TrackerItem.RelatedName') ?>">
		<div class="filter-header">
			<input type="checkbox" checked="checked" class="toggle related-type" id="related-filter-dependency" value="dependency" />
			<label for="related-filter-dependency" class="unselectable"><?php echo $Lang->getText('TrackerItem.Tabs.Dependencies')." ({$related_count['dependency']})" ?></label>

			<input type="checkbox" checked="checked" class="toggle related-type" id="related-filter-duplicate" value="duplicate" />
			<label for="related-filter-duplicate" class="unselectable"><?php echo $Lang->getText('TrackerItem.Tabs.Duplicates')." ({$related_count['duplicate']})" ?></label>

			<input type="checkbox" checked="checked" class="toggle related-type" id="related-filter-association" value="association" />
			<label for="related-filter-association" class="unselectable"><?php echo $Lang->getText('Associations')." ({$related_count['association']})" ?></label>

			<input type="checkbox" checked="checked" class="toggle related-type" id="related-filter-tag" value="tag" />
			<label for="related-filter-tag" class="unselectable"><?php echo $Lang->getText('Tag.TabHeader')." ({$related_count['tag']})" ?></label>
		</div>
		<div class="related-list">
			<?php if (count($related_list['dependency']['depends_on']) > 0) { ?>
				<div class="related-item dependency">
					<img class="related-item-icon" src="<?php echo $theme->getImgUrl('depends-on.png'); ?>" alt="<?php echo $Lang->getText('TrackerItem.DependentOn'); ?>" />
					<div class="related-title"><?php echo $Lang->getText('TrackerItem.DependentOn'); ?></div>
					<div class="related-content">
					<?php foreach ($related_list['dependency']['depends_on'] as $dep) { ?>
						<?php echo writeTI($dep); ?>
					<?php } // foreach (depends_on) ?>
					</div>
				</div>
			<?php } // if (count(depends_on)) ?>

			<?php if (count($related_list['dependency']['depended_on']) > 0) { ?>
				<div class="related-item dependency">
					<img class="related-item-icon" src="<?php echo $theme->getImgUrl('depended-on.png'); ?>" alt="<?php echo $Lang->getText('TrackerItem.DependentOnThis'); ?>" />
					<div class="related-title"><?php echo $Lang->getText('TrackerItem.DependentOnThis'); ?></div>
					<div class="related-content">
					<?php foreach ($related_list['dependency']['depended_on'] as $dep) { ?>
						<?php echo writeTI($dep); ?>
					<?php } // foreach (depended_on) ?>
					</div>
				</div>
			<?php } // if (count(depended_on)) ?>

			<?php if (count($related_list['duplicate']['duplicates']) > 0) { ?>
				<div class="related-item duplicate">
					<img class="related-item-icon" src="<?php echo $theme->getImgUrl('duplicated-by.png'); ?>" alt="<?php echo $Lang->getText('TrackerItem.HasDuplicates'); ?>" />
					<div class="related-title"><?php echo $Lang->getText('TrackerItem.HasDuplicates'); ?></div>
					<div class="related-content">
					<?php foreach ($related_list['duplicate']['duplicates'] as $dup) { ?>
						<?php echo writeTI($dup); ?>
					<?php } // foreach (duplicates) ?>
					</div>
				</div>
			<?php } // if (count(depends_on)) ?>

			<?php if (count($related_list['duplicate']['duplicate-of']) > 0) { ?>
				<div class="related-item duplicate">
					<img class="related-item-icon" src="<?php echo $theme->getImgUrl('duplicate-of.png'); ?>" alt="<?php echo $Lang->getText('TrackerItem.DuplicateOf'); ?>" />
					<div class="related-title"><?php echo $Lang->getText('TrackerItem.DuplicateOf'); ?></div>
					<div class="related-content">
					<?php foreach ($related_list['duplicate']['duplicate-of'] as $dup) { ?>
						<?php echo writeTI($dup); ?>
					<?php } // foreach (duplicate-of) ?>
					</div>
				</div>
			<?php } // if (count(depended_on)) ?>

			<?php if (count($related_list['association']) > 0) { ?>
				<div class="related-item association">
					<img class="related-item-icon" src="<?php echo $theme->getImgUrl('association.png'); ?>" alt="<?php echo $Lang->getText('Associations.List'); ?>" />
					<div class="related-title"><?php echo $Lang->getText('Associations.List'); ?></div>
					<div class="related-content">
					<?php foreach ($related_list['association'] as $assoc) { ?>
						<?php echo writeAssociation($TrackerItem, $assoc); ?>
					<?php } // foreach (association) ?>
					</div>
				</div>
			<?php } // if (count(association)) ?>

			<?php if (count($related_list['tag']) > 0) { ?>
				<div class="related-item tag">
					<img class="related-item-icon" src="<?php echo $theme->getImgUrl('tag.png'); ?>" alt="<?php echo $Lang->getText('Tag.TabLabel'); ?>" />
					<div class="related-title"><?php echo $Lang->getText('Tag.TabLabel'); ?></div>
					<div class="related-content">
						<p class="tag-list">
						<?php foreach ($related_list['tag'] as $tag) { ?>
							<?php echo tagLink($Lang, $TrackerItem, $tag); ?>
						<?php } // foreach (association) ?>
						</p>
					</div>
				</div>
			<?php } // if (count(tag)) ?>

		</div>
	</div>
</div>
<br />
<br />	<!-- Extra br solves a problem with IE6/7 rendering -->
<?php ActionManager::notify('TRACKER_ITEM_EDIT_FORM',$TrackerItem); ?>
</form>

</div>
<?php
$theme->footer($params);

