<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once 'plugins/userblog/lib/om/BaseUserBlog.php';
require_once("plugin/ActionManager.php");
require_once("exceptions/PermissionDeniedException.php");

/** 
 * The skeleton for this class was autogenerated by Propel  * You should add additional methods to this class to meet the
 * application requirements.  This class will only be generated as
 * long as it does not already exist in the output directory.
 *
 * @package plugins.userblog.lib 
 */
class UserBlog extends BaseUserBlog {
	const SECTION = 'userblog';
	const CONTENT_LIMIT = 1000;

	/**
	 * Removes this object from datastore and sets delete attribute. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 * 
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 * @see UserBlog::setDeleted()
	 * @see UserBlog::isDeleted()
	 */
	function delete($con = null) {
		if (!$this->userCanDelete()) {
			throw new PermissionDeniedException();
		}
		ActionManager::notify("USER_BLOG_DELETE", $this);
		parent::delete($con);		
	}

	/**
	 * Stores the object in the database using the Propel methods. Informs the plugin manager.
	 * This method was added by propel_customizer.php.
	 *
	 * @param Connection $con
	 * @return void
	 * @throws PropelException
	 */
	function save($con = null) {
		if ($this->isNew() && !$this->userCanCreate()) {
			throw new PermissionDeniedException();
		} else if (!$this->isNew() && !$this->userCanUpdate()) {
			throw new PermissionDeniedException();
		}

		parent::save($con);
		ActionManager::notify("USER_BLOG_SAVE", $this);
	}


	function userCanUpdate() {
		$user_session = Session::getSession()->getUser();
		if (is_object($user_session) && $user_session->getUserId() == $this->getUserId()) {
			return true;
		} else {
			return false;
		}
	}


	function userCanCreate() {
		$user_session = Session::getSession()->getUser();
		if (is_object($user_session) && $user_session->getUnixName() == GFUrl::getUrl()->getUser()->getUnixName()) {
			return true;
		} else {
			return false;
		}
	}

	function userCanDelete() {
		return $this->userCanUpdate();
	}
	
	function getShortContent($addLink=false) {
		$content = $this->getContent();
		$break = " ";

 		if(strlen($content) <= UserBlog::CONTENT_LIMIT) 
 			return $content; 

 		if(false !== ($breakpoint = strpos($content, $break, UserBlog::CONTENT_LIMIT))) { 
 			if($breakpoint < strlen($content) - 1) { 
 				$content = substr($content, 0, $breakpoint) . '...';
 				if ($addLink) {
 					$content .= '<a href="' . GFUrl::getUrl()->currentBaseURL('',array('id'=>$this->getUserBlogId())) . '">(see more)</a>'; 
 				}
 			} 
		}
		return $content;
	}

}
?>