<?php

require_once('plugins/calendar/lib/CalendarEvent.php');
require_once('plugins/calendar/lib/CalendarEventReminder.php');

Validator::validatePOSTAction();


$user=GFUrl::getUrl()->getUser();
$lang= Language::getSessLang();

if($user->getUserId() != $_POST['ref_id']){
	throw new PermissionDeniedException();
}


$errors = array();

//Required fields
$calendar_id = $_POST['calendar_id'];
$from_date = Validator::sanitizeDate($_POST['from_date']);
$to_date = Validator::sanitizeDate($_POST['to_date']);
$from_hour = Validator::sanitizeString($_POST['from_hour_hour'] . ':' . $_POST['from_hour_minute']);
$to_hour = Validator::sanitizeString($_POST['to_hour_hour'] . ':' . $_POST['to_hour_minute']);
$subject = Validator::sanitizeString($_POST['subject']);
$is_public = (isset($_POST['is_public']) && $_POST['is_public'] == true) ? true : false;
$redirect_to = $_POST['redirect_to'];
$start_date = $_POST['start_date'];
$reminder = Validator::sanitizeInt($_POST['reminder']);
$occurrence_to = isset($_POST['occurrence_to']) ? Validator::sanitizeDate($_POST['occurrence_to']) : null;


try{
	$cal_event = new CalendarEvent();
	$cal_event->setCalendarId($calendar_id);
	$cal_event->setFromDate($from_date . ' ' . $from_hour);
	$cal_event->setToDate($to_date . ' ' . $to_hour);
	$cal_event->setSubject($subject);
	if(isset($_POST['description'])){
		$cal_event->setDescription(Validator::sanitizeString($_POST['description']));
	}
	if(isset($_POST['is_public'])){
		$cal_event->setIsPublic(true);
	}
	if(isset($_POST['occurrence'])){
		$cal_event->setOccurrence(Validator::sanitizeInt($_POST['occurrence']));
	} else {
		$cal_event->setOccurrence(0);
	}
	if(!is_null($occurrence_to)){
		$cal_event->setOccurrenceTo($occurrence_to);
	}
	if(isset($_POST['is_public']) && $_POST['is_public'] == true){
		$cal_event->setIsPublic(true);
	}

	$cal_event->save();

} catch(InvalidDataException $e){
	$errors = array_merge($errors, $e->getErrors());
}
if(count($errors) > 0){
	$feedback = '* ' . implode('<br />* ', $errors);
	include('UsercalendarEventAdd.php');
} else {
	if($reminder){
		$er = new CalendarEventReminder();
		$er->setNewReminder($cal_event, $reminder);
		$er->save();
	}
	GFUrl::redirect(GFUrl::getUrl()->currentBaseUrl($redirect_to, array('start_date'=>$start_date, 'feedback'=>$lang->getText('UserCalendar.EventAddFeedback'))));
}

?>
