<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Group.class,v 1.94 2005/09/06 18:13:29 danper Exp $
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('notification/GFNotification.php');
require_once("plugin/ActionManager.php");
require_once('core/ProjectPeer.php');
require_once('StringUtils.php');

class WikiPageNotification implements GFNotification {
	private $page;
	private $project;
	private $users;
	private $isNew = false;

	function __construct(WikiPage $page) {
		$this->page = $page;
		$this->project = ProjectPeer::retrieveByPk($page->getRefId());
	}
	
	function setNew($isNew) {
		$this->isNew = $isNew;
	}
	
	/**
	 *	getNotificationMessage - construct a string of this message content.
	 *
	 *	@param	User
	 *	@return string
	 */
	function getNotificationMessage(User $User) {
		$url = GFUrl::getUrl();
		
		$body = 'The Wiki page "'.$this->page->getPageName().'" was ';
		$body .= ($this->isNew ? 'created' : 'updated').'.';
		$body .= "\n\nFollow this link for more information:\n";
		$body .= $url->getFullUrl($url->projectUrl($this->project, '', 'wiki', '', array('pagename' => $this->page->getPageName())), true);
		
		// User is either monitoring this page or the whole wiki
		$monitors_page = $this->page->userIsMonitoring($User);
		$monitors_wiki = WikiPlugin::userIsMonitoring($User, $this->project);
		
		$body .= "\n\n______________________________________________________________________".
				"\n\nYou received this email because you ".
				"\nrequested to be notified when changes were made to this page.".
				"\nIf you don't wish to be notified in the future, please".
				"\nlogin to " . $url->getFullUrl($url->baseUrl('Login', '', 'account'), true) . " and click this link: ";
		if ($monitors_page) {
			$body .= "\n" . $url->getFullUrl($url->getMonitorURL(WikiPage::SECTION, $this->page->getPrimaryKey(), $url->projectUrl($this->project, '', 'wiki'), false), true);
		} elseif ($monitors_wiki) {
			$body .= "\n" . $url->getFullUrl($url->getMonitorURL(WikiPlugin::SECTION, $this->project->getPrimaryKey(), $url->projectUrl($this->project, '', 'wiki'), false), true);
		}
					
		return $body;

	}
	
	/**
	 *	getNotificationSubject - construct a string of this message subject.
	 *
	 *	@param	User
	 *	@return string
	 */
	function getNotificationSubject(User $User) {
		return '['.$this->project->getName().'] Wiki page "'.$this->page->getPageName().'" '.($this->isNew ? 'created' : 'updated');
	}

	/**
	 *	getUsers - get an array of user objects
	 *	@return array
	 */
	public function getUsers() {
		return $this->page->getMonitoringUsers();
	}

	/**
	 *	getSection - retrieves the section name of the wiki
	 *	@return string
	 */	
	function getSection() {
		return WikiPage::SECTION;
	}
	
	/**
	 *	getRefId - retrieves the id of the wiki page that the notification belongs to
	 *	@return int
	 */	
	function getRefId() {
		return $this->page->getPrimaryKey();
	}
}

?>
