<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   
 * @author Daniel A. Perez	daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('html/SiteTemplate.php');
require_once('lib/core/ProjectPeer.php');
require_once('lib/core/UserPeer.php');
require_once('lib/core/User.php');
require_once('lib/core/RolePeer.php');
require_once('lib/core/Role.php');
require_once('lib/core/UserPeer.php');
require_once('html/HTMLWidget.php');
require_once('plugins/wiki/lib/Wiki.php');
require_once('plugins/wiki/lib/WikiProjectSettingPeer.php');
require_once('plugins/wiki/lib/html/WikiAdminTemplate.php');

$theme = new WikiAdminTemplate();
$project = GFUrl::getUrl()->getProject();
$lang = Session::getSession()->getLanguage();

if (array_key_exists('pagename',$_REQUEST)) {
	$pagename = Validator::sanitizeString($_REQUEST['pagename']);
} else {
	$pagename = null;
}
if (array_key_exists('pageid',$_REQUEST)) {
	$pageid = Validator::sanitizeInt($_REQUEST['pageid']);
} else {
	$pageid = null;
}
if (array_key_exists('version',$_REQUEST)) {
	$version = Validator::sanitizeInt($_REQUEST['version']);
} else {
	$version = null;
}
if (array_key_exists('_action',$_REQUEST)) {
	$action = Validator::sanitizeString($_REQUEST['_action']);
} else {
	$action = null;
}
$section = Project::SECTION; // This should be changed if were in the user wiki wwwlib, for example
$ref_id = $project->getProjectId(); // This should be changed if were in the user wiki wwwlib, for example

$wiki = new Wiki($section,$ref_id,$theme,$project);
$wiki->setAdmin();


if ($action) {
	switch ($action) {
		case 'updatestatistics' : {
			$wiki->updateStatistics();
			$pagecontent = $wiki->displayFrontPage('<font color="red">'.$lang->getText('Wiki.StatisticsUpdated').'</font>');
			break;
		}
	}
} elseif ($pageid) {
	// display page or display create-page page
	if ($version) {
		$pagecontent = $wiki->displayPageById($pageid,$version);
	} else {
		$pagecontent = $wiki->displayPageById($pageid);
	}
} elseif ($pagename) {
	// display page or display create-page page
	if ($version) {
		$pagecontent = $wiki->displayPage($pagename,$version);
	} else {
		$pagecontent = $wiki->displayPage($pagename);
	}
} else {	
	$wikiSettings = WikiProjectSettingPeer::retrieveByPk($project->getProjectId());
	$action = GFUrl::getUrl()->currentBaseURL('WikiSettingsAction');
	$submitButton = HTMLWidget::button("submit",   
																			$lang->getText('ProjectScmEditForm.submit'),
																			"submit","","");

	$textWikiSelected = '';
	$htmlWikiSelected = '';
	
	if($wikiSettings->getWikiType() == WikiProjectSetting::TEXT_WIKI)	{		
		$textWikiSelected = "checked='checked'";
	} else {
		$htmlWikiSelected = "checked='checked'";
	}

	if(isset($_REQUEST['firstTimeSetup'])){
		$pagecontent = "
		<br/>
		<h3>Define Wiki Type</h3><br/>
		<div class='infobox'>
			{$lang->getText('Wiki.Admin.WikiFirstTime')}
		</div> ";
	
	} else {
		$pagecontent = "
		<br/>
		<h3>Edit Wiki Type</h3><br/>
		<div class='infobox'>
			{$lang->getText('Wiki.Admin.WikiTypeTip')}
		</div>	";		
	}
	
	$pagecontent .= "
		<form action='$action' method='POST'>
			<table border='0'>
				<tr><td>
				<label>
					<input type='radio' name='wikiType' value='html' $htmlWikiSelected /> 
					Use HTML Based Wiki
				</label>			
				</td></tr>	
				<tr><td>
					<label>
						<input type='radio' name='wikiType' value='text' $textWikiSelected /> 
						Use Text Based Wiki
					</label>
				</td></tr>				
				<tr><td align='right'>	
				$submitButton	
				</td></tr>
			</table>		
		</form>
	";
}

$params = array(
		'project' => $project,
		'action' => isset($action) ? $action : '',
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => 'Wiki Admin',
		'wiki' => $wiki,
		'page' => $wiki->getPage()
		);
$theme->header($params);

echo $pagecontent;

$theme->footer();
?>
