<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez	ruben@gforgegroup.com
 * @date 2006-03-16
 *
 * SOAP plugin for GForge5.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
require_once 'session/LoginFactory.php';
require_once('exceptions/SessionException.php');
 
$server->register(
	'login',
	array('userid'=>'xsd:string','passwd'=>'xsd:string'),
	array('loginResponse'=>'xsd:string'),
	$uri,
	$uri.'#login'
);

$server->register(
	'logout',
	array('session_ser'=>'xsd:string'),
	array('logoutResponse'=>'xsd:string'),
	$uri,
	$uri.'#logout'
);

/**
 * login - Logs in a SOAP client
 * 
 * @param	string	userid	The user's unix id
 * @param	string	passwd	The user's passwd in clear text
 *
 * @return	string	the session hash
 */
function login($userid, $passwd) {
	$login = LoginFactory::getLogin();
	$ip = $_SERVER['REMOTE_ADDR'];
	try {
		$userSession = $login->doLogin($userid, $passwd);
		$session = Session::getSession();
		$session->setUserSession($userSession, $ip);
		$sessionHash = $session->getSessionHash();
		return $sessionHash;

	} catch (Exception $e) {
		$error = $e->getMessage();
		return (new soap_fault('1001', 'login', "Unable to login with user \"$userid\".\nReason: $error", $error));
	}
}

/**
 * logout - Logs out a SOAP client
 *
 * @param 	string	session_ser	The session hash
 */
function logout($session_ser) {
	try {
		$login = LoginFactory::getLogin();
		$session = continueSession($session_ser);
		$session->terminate();
		$login->doLogout();
	   	return 'OK';
	} catch (Exception $e) {
		$error = $e->getMessage();
		return (new soap_fault('', 'logut', 'Unable to log out: ', $error));
	}
}

/**
 * continueSession - Returns a previous session given the Hash
 *
 * @param 	session	session	The session
 */
 function continueSession($sessionHash) {
 	$session = Session::getSession();
	$ip = $_SERVER['REMOTE_ADDR'];
	$session->setup($ip, $sessionHash);
	return $session;
}

?>
