<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling AuditTrail

require_once('core/AuditTrail.php');
require_once('core/AuditTrailPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'AuditTrail',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'audit_trail_id' => array('name'=>'audit_trail_id', 'type' => 'xsd:int'),
		'change_date' => array('name'=>'change_date', 'type' => 'xsd:string'),
		'user_id' => array('name'=>'user_id', 'type' => 'xsd:int'),
		'section' => array('name'=>'section', 'type' => 'xsd:string'),
		'ref_id' => array('name'=>'ref_id', 'type' => 'xsd:int'),
		'field_name' => array('name'=>'field_name', 'type' => 'xsd:string'),
		'new_value' => array('name'=>'new_value', 'type' => 'xsd:string'),
		'old_value' => array('name'=>'old_value', 'type' => 'xsd:string'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfAuditTrail',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:AuditTrail[]')),
	'tns:AuditTrail'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getAuditTrails',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'field_name' => 'xsd:string'
	), 
	array('getAuditTrails' => 'tns:ArrayOfAuditTrail'),
	$uri, 
	$uri.'#getAuditTrails',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of AuditTrail
 * @param	$section	Filter by section
 * @param	$ref_id	Filter by ref_id
 * @param	$field_name	Filter by field_name
 */
function getAuditTrails($sessionHash, $tracker_item_id = 0, $field_name = '') {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$TrackerItem = TrackerItemPeer::retrieveByPK($tracker_item_id);
	if (is_null($TrackerItem) || !$TrackerItem->userCanRead()) {
		return new soap_fault('', 'getAuditTrails','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$c = new Criteria();
	// we only support Tracker Item
	$c->add(AuditTrailPeer::SECTION, 'trackeritem');

	if ($tracker_item_id) {
		$c->add(AuditTrailPeer::REF_ID, $tracker_item_id);
	}
	if ($field_name != '') {
		$c->add(AuditTrailPeer::FIELD_NAME, $field_name);
	}
	
	$objs = AuditTrailPeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = AuditTrailToSOAP($obj);
	}
	return $res;
}

$server->register(
	'getAuditTrailArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'audit_trail_id' => 'tns:ArrayOfInt',
	), 
	array('getAuditTrail' => 'tns:ArrayOfAuditTrail'),
	$uri, 
	$uri.'#getAuditTrail',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of AuditTrail
 * @param	$audit_trail_id	Retrieve the AuditTrail array identified by the array of audit_trail_id
 */
function getAuditTrailArray($sessionHash, $audit_trail_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$ret = array();

	$crit=new Criteria();
        $crit->add(AuditTrailPeer::AUDIT_TRAIL_ID,$audit_trail_idArr,Criteria::IN);
	$objs = AuditTrailPeer::doSelect($crit);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = AuditTrailToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getAuditTrail',
	array(
		'sessionHash'	=> 'xsd:string',
		'audit_trail_id' => 'xsd:int',
	), 
	array('getAuditTrail' => 'tns:AuditTrail'),
	$uri, 
	$uri.'#getAuditTrail',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single AuditTrail
 * @param	$audit_trail_id	Retrieve the AuditTrail identified by this audit_trail_id
 */
function getAuditTrail($sessionHash, $audit_trail_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = AuditTrailPeer::retrieveByPK($audit_trail_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->userCanRead()) {
		return new soap_fault('', 'getAuditTrail','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return AuditTrailToSOAP($obj);
}

function AuditTrailToSOAP(&$obj) {
	$ret = array();
	$ret['audit_trail_id'] = $obj->getAuditTrailId();
	$ret['change_date'] = $obj->getChangeDate();
	$ret['user_id'] = $obj->getUserId();
	$ret['section'] = $obj->getSection();
	$ret['ref_id'] = $obj->getRefId();
	$ret['field_name'] = $obj->getFieldName();
	$ret['new_value'] = $obj->getNewValue();
	$ret['old_value'] = $obj->getOldValue();
	return $ret;
}
?>
