<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling CountryCode

require_once('core/CountryCode.php');
require_once('core/CountryCodePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'CountryCode',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'country_name' => array('name'=>'country_name', 'type' => 'xsd:string'),
		'ccode' => array('name'=>'ccode', 'type' => 'xsd:string'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfCountryCode',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:CountryCode[]')),
	'tns:CountryCode'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getCountryCodes',
	array(
		'sessionHash'	=> 'xsd:string'
	), 
	array('getCountryCodes' => 'tns:ArrayOfCountryCode'),
	$uri, 
	$uri.'#getCountryCodes',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve the list of CountryCode
 */
function getCountryCodes($sessionHash) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$c = new Criteria();
	$objs = CountryCodePeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = CountryCodeToSOAP($obj);
	}
	
	return $res;
}

$server->register(
	'getCountryCodeArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'ccode' => 'tns:ArrayOfString',
	), 
	array('getCountryCode' => 'tns:ArrayOfCountryCode'),
	$uri, 
	$uri.'#getCountryCode',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of CountryCode
 * @param	$ccode	Retrieve the CountryCode array identified by the array of ccode
 */
function getCountryCodeArray($sessionHash, $ccodeArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$ret = array();

	$crit=new Criteria();
        $crit->add(CountryCodePeer::CCODE,$ccodeArr,Criteria::IN);
	$objs = CountryCodePeer::doSelect($crit);
	foreach ($objs as $obj) {
		$ret[] = CountryCodeToSOAP($obj);
	}
	return $ret;
}

$server->register(
	'getCountryCode',
	array(
		'sessionHash'	=> 'xsd:string',
		'ccode' => 'xsd:string',
	), 
	array('getCountryCode' => 'tns:CountryCode'),
	$uri, 
	$uri.'#getCountryCode',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single CountryCode
 * @param	$ccode	Retrieve the CountryCode identified by this ccode
 */
function getCountryCode($sessionHash, $ccode) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = CountryCodePeer::retrieveByPK($ccode);
	if (is_null($obj)) {
		return null;
	}
	return CountryCodeToSOAP($obj);
}


function CountryCodeToSOAP(&$obj) {
	$ret = array();
	$ret['country_name'] = $obj->getCountryName();
	$ret['ccode'] = $obj->getCcode();
	return $ret;
}
?>