<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling Language

require_once('core/Language.php');
require_once('core/LanguagePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'Language',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'language_id' => array('name'=>'language_id', 'type' => 'xsd:int'),
		'name' => array('name'=>'name', 'type' => 'xsd:string'),
		'filename' => array('name'=>'filename', 'type' => 'xsd:string'),
		'language_code' => array('name'=>'language_code', 'type' => 'xsd:string'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfLanguage',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:Language[]')),
	'tns:Language'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getLanguages',
	array(
		'sessionHash'	=> 'xsd:string'
	), 
	array('getLanguages' => 'tns:ArrayOfLanguage'),
	$uri, 
	$uri.'#getLanguages',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve the list of Languages
 */
function getLanguages($sessionHash) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$c = new Criteria();
	$objs = LanguagePeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = LanguageToSOAP($obj);
	}
	
	return $res;
}

$server->register(
	'getLanguageArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'language_id' => 'tns:ArrayOfInt',
	), 
	array('getLanguage' => 'tns:ArrayOfLanguage'),
	$uri, 
	$uri.'#getLanguage',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of Language
 * @param	$language_id	Retrieve the Language array identified by the array of language_id
 */
function getLanguageArray($sessionHash, $language_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$ret = array();

	$crit=new Criteria();
        $crit->add(LanguagePeer::LANGUAGE_ID,$language_idArr,Criteria::IN);
	$objs = LanguagePeer::doSelect($crit);
	foreach ($objs as $obj) {
		$ret[] = LanguageToSOAP($obj);
	}
	return $ret;
}

$server->register(
	'getLanguage',
	array(
		'sessionHash'	=> 'xsd:string',
		'language_id' => 'xsd:int',
	), 
	array('getLanguage' => 'tns:Language'),
	$uri, 
	$uri.'#getLanguage',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single Language
 * @param	$language_id	Retrieve the Language identified by this language_id
 */
function getLanguage($sessionHash, $language_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = LanguagePeer::retrieveByPK($language_id);
	if (is_null($obj)) {
		return null;
	}
	return LanguageToSOAP($obj);
}


function LanguageToSOAP(&$obj) {
	$ret = array();
	$ret['language_id'] = $obj->getLanguageId();
	$ret['name'] = $obj->getName();
	$ret['filename'] = $obj->getFilename();
	$ret['language_code'] = $obj->getLanguageCode();
	return $ret;
}
?>