<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling CruisecontrolPublisher

require_once('plugins/build/lib/CruisecontrolPublisher.php');
require_once('plugins/build/lib/CruisecontrolPublisherPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'CruisecontrolPublisher',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'cruisecontrol_id' => array('name'=>'cruisecontrol_id', 'type' => 'xsd:int'),
		'tracker_id' => array('name'=>'tracker_id', 'type' => 'xsd:int'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfCruisecontrolPublisher',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:CruisecontrolPublisher[]')),
	'tns:CruisecontrolPublisher'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getCruisecontrolPublishers',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'xsd:int',
		'tracker_id' => 'xsd:int',
	), 
	array('getCruisecontrolPublishers' => 'tns:ArrayOfCruisecontrolPublisher'),
	$uri, 
	$uri.'#getCruisecontrolPublishers',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of CruisecontrolPublisher
 * @param	$cruisecontrol_id	Filter by cruisecontrol_id
 * @param	$tracker_id	Filter by tracker_id
 */
function getCruisecontrolPublishers($sessionHash, $cruisecontrol_id, $tracker_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$crit = new Criteria();
	
	if (!is_null($cruisecontrol_id) && $cruisecontrol_id > 0) {
		$crit->add(CruisecontrolPublisherPeer::CRUISECONTROL_ID, $cruisecontrol_id);
	}
	if (!is_null($tracker_id) && $tracker_id > 0) {
		$crit->add(CruisecontrolPublisherPeer::TRACKER_ID, $tracker_id);
	}
	
	$objs = CruisecontrolPublisherPeer::doSelect($crit);
	$res = array();
	foreach ($objs as $obj) {
		//if ($obj->userCanRead()) {
			$res[] = CruisecontrolPublisherToSOAP($obj);
	//	}
	}
	return $res;
}



$server->register(
	'addCruisecontrolPublisher',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'xsd:int',
		'tracker_id' => 'xsd:int',
	), 
	array('addCruisecontrolPublisher' => 'xsd:int'),
	$uri, 
	$uri.'#addCruisecontrolPublisher',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a CruisecontrolPublisher
 * @param	$cruisecontrol_id
 * @param	$tracker_id
 */
function addCruisecontrolPublisher($sessionHash, $cruisecontrol_id, $tracker_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = new CruisecontrolPublisher();
	
	$obj->setCruisecontrolId( Validator::sanitizeInt($cruisecontrol_id) );
	$obj->setTrackerId( Validator::sanitizeInt($tracker_id) );
	try {
		$obj->save();
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'addCruisecontrolPublisher','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

$server->register(
	'deleteCruisecontrolPublisher',
	array(
		'sessionHash'	=> 'xsd:string',
		'cruisecontrol_id' => 'xsd:int',
		'tracker_id' => 'xsd:int',
	), 
	array('deleteCruisecontrolPublisher' => 'xsd:int'),
	$uri, 
	$uri.'#deleteCruisecontrolPublisher',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single CruisecontrolPublisher
 * @param	$cruisecontrol_id
 * @param	$tracker_id
 */
function deleteCruisecontrolPublisher($sessionHash, $cruisecontrol_id, $tracker_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = CruisecontrolPublisherPeer::retrieveByPK($cruisecontrol_id, $tracker_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteCruisecontrolPublisher','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteCruisecontrolPublisher','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}

function CruisecontrolPublisherToSOAP(&$obj) {
	$ret = array();
	$ret['cruisecontrol_id'] = $obj->getCruisecontrolId();
	$ret['tracker_id'] = $obj->getTrackerId();
	return $ret;
}
?>
