<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling DocmanFolder

require_once('plugins/docman/lib/DocmanFolder.php');
require_once('plugins/docman/lib/DocmanFolderPeer.php');
require_once('plugins/docman/lib/DocmanFileFolderPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'DocmanFolder',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'docman_folder_id' => array('name'=>'docman_folder_id', 'type' => 'xsd:int'),
		'project_id' => array('name'=>'project_id', 'type' => 'xsd:int'),
		'folder_name' => array('name'=>'folder_name', 'type' => 'xsd:string'),
		'is_public' => array('name'=>'is_public', 'type' => 'xsd:boolean'),
		'parent_folder_id' => array('name'=>'parent_folder_id', 'type' => 'xsd:int'),
		'doc_count' => array('name'=>'doc_count', 'type' => 'xsd:int'),
		'sort_order' => array('name'=>'sort_order', 'type' => 'xsd:int'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfDocmanFolder',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:DocmanFolder[]')),
	'tns:DocmanFolder'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getDocmanFolders',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
	), 
	array('getDocmanFolders' => 'tns:ArrayOfDocmanFolder'),
	$uri, 
	$uri.'#getDocmanFolders',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of DocmanFolder
 * @param	$project_id	Filter by project_id
 */
function getDocmanFolders($sessionHash, $project_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$crit = new Criteria();
	
	if (!is_null($project_id) && $project_id >= 0) {
		//Verifies that the incoming (home) default folder exists, if not, it creates it
		DocmanFolderPeer::verifyAndCreateIncoming($project_id);
		$crit->add(DocmanFolderPeer::PROJECT_ID, $project_id);
	}
	$crit->addAscendingOrderByColumn(DocmanFolderPeer::PARENT_FOLDER_ID);
	$crit->addAscendingOrderByColumn(DocmanFolderPeer::SORT_ORDER);
	$objs = DocmanFolderPeer::doSelect($crit);
	$res = array();
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$res[] = DocmanFolderToSOAP($obj);
		}
	}
	return $res;
}

$server->register(
	'getDocmanFolderArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_folder_id' => 'tns:ArrayOfInt',
	), 
	array('getDocmanFolder' => 'tns:ArrayOfDocmanFolder'),
	$uri, 
	$uri.'#getDocmanFolder',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of DocmanFolder
 * @param	$docman_folder_id	Retrieve the DocmanFolder array identified by the array of docman_folder_id
 */
function getDocmanFolderArray($sessionHash, $docman_folder_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$ret = array();

	$crit=new Criteria();
        $crit->add(DocmanFolderPeer::DOCMAN_FOLDER_ID,$docman_folder_idArr,Criteria::IN);
	$objs = DocmanFolderPeer::doSelect($crit);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = DocmanFolderToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getDocmanFolder',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_folder_id' => 'xsd:int',
	), 
	array('getDocmanFolder' => 'tns:DocmanFolder'),
	$uri, 
	$uri.'#getDocmanFolder',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single DocmanFolder
 * @param	$docman_folder_id	Retrieve the DocmanFolder identified by this docman_folder_id
 */
function getDocmanFolder($sessionHash, $docman_folder_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = DocmanFolderPeer::retrieveByPK($docman_folder_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->userCanRead()) {
		return new soap_fault('', 'getDocmanFolder','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return DocmanFolderToSOAP($obj);
}


$server->register(
	'deleteDocmanFolder',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_folder_id' => 'xsd:int',
	), 
	array('deleteDocmanFolder' => 'xsd:int'),
	$uri, 
	$uri.'#deleteDocmanFolder',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single DocmanFolder
 * @param	$docman_folder_id	Delete the DocmanFolder identified by this docman_folder_id
 */
function deleteDocmanFolder($sessionHash, $docman_folder_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = DocmanFolderPeer::retrieveByPK($docman_folder_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteDocmanFolder','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteDocmanFolder','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'updateDocmanFolder',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_folder_id' => 'xsd:int',
		'folder_name' => 'xsd:string',
		'is_public' => 'xsd:boolean',
		'parent_folder_id' => 'xsd:int',
		'sort_order' => 'xsd:int',
	), 
	array('updateDocmanFolder' => 'xsd:int'),
	$uri, 
	$uri.'#updateDocmanFolder',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update DocmanFolder
 * @param	$docman_folder_id	 DocmanFolder to update
 * @param	$folder_name
 * @param	$is_public
 * @param	$parent_folder_id
 * @param	$sort_order
 */
function updateDocmanFolder($sessionHash, $docman_folder_id, $folder_name, $is_public, $parent_folder_id, $sort_order) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = DocmanFolderPeer::retrieveByPK($docman_folder_id);
	if (is_null($obj)) {
		return new soap_fault('', 'updateDocmanFolder','Couldn\'t update DocmanFolder: invalid docman_folder_id '.$docman_folder_id);
	}
	
	if (!is_null($folder_name)) {
		$obj->setFolderName( Validator::sanitizeString($folder_name) );
	}
	if (!is_null($is_public)) {
		$obj->setIsPublic( Validator::sanitizeBool($is_public) );
	}
	if (!is_null($parent_folder_id)) {
		$obj->setParentFolderId( Validator::sanitizeInt($parent_folder_id) );
	}
	if (!is_null($sort_order)) {
		$obj->setSortOrder( Validator::sanitizeInt($sort_order) );
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateDocmanFolder','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}



$server->register(
	'addDocmanFolder',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
		'folder_name' => 'xsd:string',
		'is_public' => 'xsd:boolean',
		'parent_folder_id' => 'xsd:int',
		'sort_order' => 'xsd:int',
	), 
	array('addDocmanFolder' => 'xsd:int'),
	$uri, 
	$uri.'#addDocmanFolder',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a DocmanFolder
 * @param	$project_id
 * @param	$folder_name
 * @param	$is_public
 * @param	$parent_folder_id
 * @param	$sort_order
 */
function addDocmanFolder($sessionHash, $project_id, $folder_name, $is_public, $parent_folder_id, $sort_order) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	//Check if folder parent is "incoming", we must set parent folder as 0 in that case
	$c = new Criteria();
	$c->add(DocmanFolderPeer::FOLDER_NAME, 'incoming');
	$c->add(DocmanFolderPeer::PROJECT_ID, $project_id);
	$incoming_folder = DocmanFolderPeer::doSelectOne($c);
	if (is_object($incoming_folder)) {
		if ($incoming_folder->getDocmanFolderId() == $parent_folder_id) {
			$parent_folder_id = 0;
		}
	}

	$obj = new DocmanFolder();
	
	$obj->setProjectId( Validator::sanitizeInt($project_id) );
	$obj->setFolderName( Validator::sanitizeString($folder_name) );
	$obj->setIsPublic( Validator::sanitizeBool($is_public) );
	$obj->setParentFolderId( Validator::sanitizeInt($parent_folder_id) );
	$obj->setSortOrder( Validator::sanitizeInt($sort_order) );
	try {
		$obj->save();
		return $obj->getDocmanFolderId();
	} catch (Exception $e) {
		return new soap_fault('', 'addDocmanFolder','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}


/**
 * SOAP method to retrieve the list of folders a DocmanFile belongs to (since v6.1, a DocmanFile may belong
 * to several folders)
 */
$server->register(
	'getDocmanFileFolders',
	array(
		'sessionHash'	=> 'xsd:string',
		'docman_file_id' => 'xsd:int',
	),
	array('getDocmanFileFolders' => 'tns:ArrayOfDocmanFolder'),
	$uri, 
	$uri.'#getDocmanFileFolders',
	'rpc',
	'encoded'
);

function getDocmanFileFolders($sessionHash, $docman_file_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$doc = DocmanFilePeer::retrieveByPk($docman_file_id);
	if (is_null($doc)) {
		return new soap_fault('', 'getDocmanFileFolders', "Couldn't retrieve docman folder: invalid folder ID $docman_file_id");
	}

	if (!$doc->userCanRead()) {
		return new soap_fault('', 'getDocmanFileFolders','Permission Denied');
	}

	$c = new Criteria();
	$c->add(DocmanFileFolderPeer::DOCMAN_FILE_ID, $docman_file_id);
	$file_folders = DocmanFileFolderPeer::doSelectJoinDocmanFolder($c);
	$ret = array();

	foreach ($file_folders as $file_folder) {
		$folder = $file_folder->getDocmanFolder();
		if ($folder->userCanRead()) {
			$ret[] = DocmanFolderToSOAP($folder);
		}
	}

	return $ret;
}

function DocmanFolderToSOAP(&$obj) {
	$ret = array();
	$ret['docman_folder_id'] = $obj->getDocmanFolderId();
	$ret['project_id'] = $obj->getProjectId();
	$ret['folder_name'] = $obj->getFolderName();
	$ret['is_public'] = $obj->getIsPublic();
	$ret['parent_folder_id'] = $obj->getParentFolderId();
	$ret['doc_count'] = $obj->getDocCount();
	$ret['sort_order'] = $obj->getSortOrder();
	return $ret;
}
?>