<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling FrsPackage

require_once('plugins/frs/lib/FrsPackage.php');
require_once('plugins/frs/lib/FrsPackagePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'FrsPackage',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'frs_package_id' => array('name'=>'frs_package_id', 'type' => 'xsd:int'),
		'project_id' => array('name'=>'project_id', 'type' => 'xsd:int'),
		'package_name' => array('name'=>'package_name', 'type' => 'xsd:string'),
		'status_id' => array('name'=>'status_id', 'type' => 'xsd:int'),
		'is_public' => array('name'=>'is_public', 'type' => 'xsd:boolean'),
		'require_login' => array('name'=>'require_login', 'type' => 'xsd:boolean')
	)
);


$server->wsdl->addComplexType(
	'ArrayOfFrsPackage',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:FrsPackage[]')),
	'tns:FrsPackage'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getFrsPackages',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int'
	), 
	array('getFrsPackages' => 'tns:ArrayOfFrsPackage'),
	$uri, 
	$uri.'#getFrsPackages',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of FrsPackage
 * @param	$project_id	Filter by project_id
 */
function getFrsPackages($sessionHash, $project_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$c = new Criteria();
	
	if (!is_null($project_id)) {
		$c->add(FrsPackagePeer::PROJECT_ID, $project_id);
	}
	
	$objs = FrsPackagePeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$res[] = FrsPackageToSOAP($obj);
		}
	}
	
	return $res;
	
}

$server->register(
	'getFrsPackageArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'frs_package_id' => 'tns:ArrayOfInt'
	), 
	array('getFrsPackage' => 'tns:ArrayOfFrsPackage'),
	$uri, 
	$uri.'#getFrsPackage',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of FrsPackage
 * @param	$frs_package_id	Retrieve the FrsPackage array identified by the array of frs_package_id
 */
function getFrsPackageArray($sessionHash, $frs_package_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$ret = array();

	$crit=new Criteria();
        $crit->add(FrsPackagePeer::FRS_PACKAGE_ID,$frs_package_idArr,Criteria::IN);
	$objs = FrsPackagePeer::doSelect($crit);
	foreach ($objs as $obj) {
		if ($obj->userCanRead()) {
			$ret[] = FrsPackageToSOAP($obj);
		}
	}
	return $ret;
}

$server->register(
	'getFrsPackage',
	array(
		'sessionHash'	=> 'xsd:string',
		'frs_package_id' => 'xsd:int'
	), 
	array('getFrsPackage' => 'tns:FrsPackage'),
	$uri, 
	$uri.'#getFrsPackage',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single FrsPackage
 * @param	$frs_package_id	Retrieve the FrsPackage identified by this frs_package_id
 */
function getFrsPackage($sessionHash, $frs_package_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = FrsPackagePeer::retrieveByPK($frs_package_id);
	if (is_null($obj)) {
		return null;
	}
	if (!$obj->userCanRead()) {
		return new soap_fault('', 'getFrsPackage','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	return FrsPackageToSOAP($obj);
}


$server->register(
	'deleteFrsPackage',
	array(
		'sessionHash'	=> 'xsd:string',
		'frs_package_id' => 'xsd:int'
	), 
	array('deleteFrsPackage' => 'xsd:int'),
	$uri, 
	$uri.'#deleteFrsPackage',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single FrsPackage
 * @param	$frs_package_id	Delete the FrsPackage identified by this frs_package_id
 */
function deleteFrsPackage($sessionHash, $frs_package_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = FrsPackagePeer::retrieveByPK($frs_package_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteFrsPackage','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteFrsPackage','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'updateFrsPackage',
	array(
		'sessionHash'	=> 'xsd:string',
		'frs_package_id' => 'xsd:int',
		'package_name' => 'xsd:string',
		'status_id' => 'xsd:int',
		'is_public' => 'xsd:boolean',
		'require_login' => 'xsd:boolean'
	), 
	array('updateFrsPackage' => 'xsd:int'),
	$uri, 
	$uri.'#updateFrsPackage',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update FrsPackage
 * @param	$frs_package_id	 FrsPackage to update
 * @param	$package_name
 * @param	$status_id
 * @param	$is_public
 * @param	$require_login
 */
function updateFrsPackage($sessionHash, $frs_package_id, $package_name, $status_id, $is_public, $require_login) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = FrsPackagePeer::retrieveByPK($frs_package_id);
	if (is_null($obj)) {
		return new soap_fault('', 'updateFrsPackage','Couldn\'t update FrsPackage: invalid frs_package_id '.$frs_package_id);
	}
	
	if (!is_null($package_name)) {
		$obj->setPackageName( Validator::sanitizeString($package_name) );
	}
	if (!is_null($status_id)) {
		$obj->setStatusId( Validator::sanitizeInt($status_id) );
	}
	if (!is_null($is_public)) {
		$obj->setIsPublic( Validator::sanitizeBool($is_public) );
	}
	if (!is_null($require_login)) {
		$obj->setRequireLogin( Validator::sanitizeBool($require_login) );
	}
	
	try {
		$obj->save();
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'updateFrsPackage','Permission Denied - Cannot update This Record',$e->getMessage());
	}
}



$server->register(
	'addFrsPackage',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
		'package_name' => 'xsd:string',
		'status_id' => 'xsd:int',
		'is_public' => 'xsd:boolean',
		'require_login' => 'xsd:boolean',
	), 
	array('addFrsPackage' => 'xsd:int'),
	$uri, 
	$uri.'#getFrsPackages',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a FrsPackage
 * @param	$project_id
 * @param	$package_name
 * @param	$status_id
 * @param	$is_public
 * @param	$require_login
 */
function addFrsPackage($sessionHash, $project_id, $package_name, $status_id, $is_public, $require_login) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);
	
	$project = ProjectPeer::retrieveByPK($project_id);
	
	if (is_null($project) || !$project->getPermission()->check(FrsPlugin::SECTION,0,FrsPlugin::WRITE)) {
		return new soap_fault('', 'addFrsPackage','Permission Denied - Cannot add This Record', 'Permission Denied - Cannot add This Record');
	}

	$obj = new FrsPackage();
	
	$obj->setProjectId( Validator::sanitizeInt($project_id) );
	$obj->setPackageName( Validator::sanitizeString($package_name) );
	$obj->setStatusId( Validator::sanitizeInt($status_id) );
	$obj->setIsPublic( Validator::sanitizeBool($is_public) );
	$obj->setRequireLogin( Validator::sanitizeBool($require_login) );
	try {
		$obj->save();
		return $obj->getFrsPackageId();
	} catch (Exception $e) {
		return new soap_fault('', 'addFrsPackage','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function FrsPackageToSOAP(&$obj) {
	$ret = array();
	$ret['frs_package_id'] = $obj->getFrsPackageId();
	$ret['project_id'] = $obj->getProjectId();
	$ret['package_name'] = $obj->getPackageName();
	$ret['status_id'] = $obj->getStatusId();
	$ret['is_public'] = $obj->getIsPublic();
	$ret['require_login'] = $obj->getRequireLogin();
	return $ret;
}
?>
