<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling ProjectScm

require_once('plugins/scm/lib/ProjectScm.php');
require_once('plugins/scm/lib/ProjectScmPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'ProjectScm',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'project_id' => array('name'=>'project_id', 'type' => 'xsd:int'),
		'enable_anonread' => array('name'=>'enable_anonread', 'type' => 'xsd:boolean'),
		'scm_server_id' => array('name'=>'scm_server_id', 'type' => 'xsd:int'),
		'controlled' => array('name'=>'controlled', 'type' => 'xsd:boolean'),
		'hostname' => array('name'=>'hostname', 'type' => 'xsd:string'),
		'root_directory' => array('name'=>'root_directory', 'type' => 'xsd:string'),
		'external_browse_url' => array('name'=>'external_browse_url', 'type' => 'xsd:string'),
		'browse_text' => array('name'=>'browse_text', 'type' => 'xsd:string'),
		'access_text' => array('name'=>'access_text', 'type' => 'xsd:string'),
		'bytes_used' => array('name'=>'bytes_used', 'type' => 'xsd:int'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfProjectScm',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:ProjectScm[]')),
	'tns:ProjectScm'
);

$server->register(
	'getProjectScmArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'ArrayOfxsd:int',
	), 
	array('getProjectScm' => 'tns:ArrayOfProjectScm'),
	$uri, 
	$uri.'#getProjectScm',
	'rpc',
	'encoded'
);

$server->register(
	'getProjectScm',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
	), 
	array('getProjectScm' => 'tns:ProjectScm'),
	$uri, 
	$uri.'#getProjectScm',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single ProjectScm
 * @param	$project_id	Retrieve the ProjectScm identified by this project_id
 */
function getProjectScm($sessionHash, $project_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$PrjObj = ProjectPeer::retrieveByPK($project_id);
	if (is_null($PrjObj) || !$PrjObj->userCanRead()) {
		return new soap_fault('', 'getProjectScm','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$obj = ProjectScmPeer::retrieveByPK($PrjObj->getProjectId());
	if (is_null($obj)) {
		return null;
	}
	return ProjectScmToSOAP($obj);
}

/*
$server->register(
	'deleteProjectScm',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
	), 
	array('deleteProjectScm' => 'xsd:int'),
	$uri, 
	$uri.'#deleteProjectScm',
	'rpc',
	'encoded'
);
*/
/**
 * SOAP method to delete a single ProjectScm
 * @param	$project_id	Delete the ProjectScm identified by this project_id
 */
/*function deleteProjectScm($sessionHash, $project_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$PrjObj = ProjectPeer::retrieveByPK($project_id);
	if (is_null($PrjObj) || !$PrjObj->userCanRead()) {
		return new soap_fault('', 'deleteProjectScm','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$obj = ProjectScmPeer::retrieveByPK($PrjObj->getProjectId());

	if (is_null($obj)) {
		return new soap_fault('', 'deleteProjectScm','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteProjectScm','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}
*/

$server->register(
	'updateProjectScm',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
		'enable_anonread' => 'xsd:boolean',
		'scm_server_id' => 'xsd:int',
		'controlled' => 'xsd:boolean',
		'hostname' => 'xsd:string',
		'root_directory' => 'xsd:string',
		'external_browse_url' => 'xsd:string',
		'browse_text' => 'xsd:string',
		'access_text' => 'xsd:string',
	), 
	array('updateProjectScm' => 'xsd:int'),
	$uri, 
	$uri.'#updateProjectScm',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update ProjectScm
 * @param	$project_id	 ProjectScm to update
 * @param	$enable_anonread
 * @param	$scm_server_id
 * @param	$controlled
 * @param	$hostname
 * @param	$root_directory
 * @param	$external_browse_url
 * @param	$browse_text
 * @param	$access_text
 * @param	$bytes_used
 */
function updateProjectScm($sessionHash, $project_id, $enable_anonread, $scm_server_id, $controlled, $hostname, $root_directory, $external_browse_url, $browse_text, $access_text) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$PrjObj = ProjectPeer::retrieveByPK($project_id);
	if (is_null($PrjObj) || !$PrjObj->userCanUpdate()) {
		throw new soap_fault('', 'updateProjectScm', 'Couldn\'t update ProjectScm: Permission Denied.');
	}

	$obj = ProjectScmPeer::retrieveByPK($project_id);
	if (is_null($obj)) {
		throw new soap_fault('', 'updateProjectScm', 'Couldn\'t update ProjectScm: invalid project_id '.$project_id);
	}
	
	if (!is_null($enable_anonread)) {
		$obj->setEnableAnonread( Validator::sanitizeBool($enable_anonread) );
	}
	if (!is_null($scm_server_id)) {
		$obj->setScmServerId( Validator::sanitizeInt($scm_server_id) );
	}
	if (!is_null($controlled)) {
		$obj->setControlled( Validator::sanitizeBool($controlled) );
	}
	if (!is_null($hostname)) {
		$obj->setHostname( Validator::sanitizeString($hostname) );
	}
	if (!is_null($root_directory)) {
		$obj->setRootDirectory( Validator::sanitizeString($root_directory) );
	}
	if (!is_null($external_browse_url)) {
		$obj->setExternalBrowseUrl( Validator::sanitizeString($external_browse_url) );
	}
	if (!is_null($browse_text)) {
		$obj->setBrowseText( Validator::sanitizeString($browse_text) );
	}
	if (!is_null($access_text)) {
		$obj->setAccessText( Validator::sanitizeString($access_text) );
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateProjectScm','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}



$server->register(
	'addProjectScm',
	array(
		'sessionHash'	=> 'xsd:string',
		'project_id' => 'xsd:int',
		'enable_anonread' => 'xsd:boolean',
		'scm_server_id' => 'xsd:int',
		'controlled' => 'xsd:boolean',
		'hostname' => 'xsd:string',
		'root_directory' => 'xsd:string',
		'external_browse_url' => 'xsd:string',
		'browse_text' => 'xsd:string',
		'access_text' => 'xsd:string',
	), 
	array('addProjectScm' => 'xsd:int'),
	$uri, 
	$uri.'#addProjectScm',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a ProjectScm
 * @param	$project_id
 * @param	$enable_anonread
 * @param	$scm_server_id
 * @param	$controlled
 * @param	$hostname
 * @param	$root_directory
 * @param	$external_browse_url
 * @param	$browse_text
 * @param	$access_text
 * @param	$bytes_used
 */
function addProjectScm($sessionHash, $project_id, $enable_anonread, $scm_server_id, $controlled, $hostname, $root_directory, $external_browse_url, $browse_text, $access_text) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$PrjObj = ProjectPeer::retrieveByPK($project_id);
	if (is_null($PrjObj) || !$PrjObj->userCanUpdate()) {
		return new soap_fault('', 'addProjectScm','Permission Denied - Cannot add This Record');
	}

	$obj = new ProjectScm();
	
	$obj->setProjectId( Validator::sanitizeInt($project_id) );
	$obj->setEnableAnonread( Validator::sanitizeBool($enable_anonread) );
	$obj->setScmServerId( Validator::sanitizeInt($scm_server_id) );
	$obj->setControlled( Validator::sanitizeBool($controlled) );
	$obj->setHostname( Validator::sanitizeString($hostname) );
	$obj->setRootDirectory( Validator::sanitizeString($root_directory) );
	$obj->setExternalBrowseUrl( Validator::sanitizeString($external_browse_url) );
	$obj->setBrowseText( Validator::sanitizeString($browse_text) );
	$obj->setAccessText( Validator::sanitizeString($access_text) );
	$obj->setBytesUsed( 0 );
	try {
		$obj->save();
		return $obj->getPrimaryKey();
	} catch (Exception $e) {
		return new soap_fault('', 'addProjectScm','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function ProjectScmToSOAP(&$obj) {
	$ret = array();
	$ret['project_id'] = $obj->getProjectId();
	$ret['enable_anonread'] = $obj->getEnableAnonread();
	$ret['scm_server_id'] = $obj->getScmServerId();
	$ret['controlled'] = $obj->getControlled();
	$ret['hostname'] = $obj->getHostname();
	$ret['root_directory'] = $obj->getRootDirectory();
	$ret['external_browse_url'] = $obj->getExternalBrowseUrl();
	$ret['browse_text'] = $obj->getBrowseText();
	$ret['access_text'] = $obj->getAccessText();
	$ret['bytes_used'] = $obj->getBytesUsed();
	return $ret;
}
?>
