<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling TrackerExtraFieldData

require_once('plugins/tracker/lib/TrackerExtraFieldData.php');
require_once('plugins/tracker/lib/TrackerExtraFieldDataPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'TrackerExtraFieldData',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'tracker_extra_field_id' => array('name'=>'tracker_extra_field_id', 'type' => 'xsd:int'),
		'field_data' => array('name'=>'field_data', 'type' => 'xsd:string'),
		'tracker_extra_field_data_id' => array('name'=>'tracker_extra_field_data_id', 'type' => 'xsd:int'),
		'tracker_item_id' => array('name'=>'tracker_item_id', 'type' => 'xsd:int')
	)
);

	
$server->wsdl->addComplexType(
	'ArrayOfTrackerExtraFieldData',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:TrackerExtraFieldData[]')),
	'tns:TrackerExtraFieldData'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */


$server->register(
	'getTrackerExtraFieldDatas',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'tracker_extra_field_id' => 'xsd:int'
	), 
	array('getTrackerExtraFieldDatas' => 'tns:ArrayOfTrackerExtraFieldData'),
	$uri, 
	$uri.'#getTrackerExtraFieldDatas',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of TrackerExtraFieldData
 * @param	$tracker_item_id	Filter by tracker_item_id
 */

function getTrackerExtraFieldDatas($sessionHash, $tracker_item_id, $tracker_extra_field_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$c = new Criteria();
	if (!checkTrackerItemPermission($tracker_item_id)) {
		return new soap_fault('', 'getTrackerExtraFieldDatas','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$c->add(TrackerExtraFieldDataPeer::TRACKER_ITEM_ID, $tracker_item_id);

	if ($tracker_extra_field_id >= 0) {
		$c->add(TrackerExtraFieldDataPeer::TRACKER_EXTRA_FIELD_ID, $tracker_extra_field_id);
	}
	
	$objs = TrackerExtraFieldDataPeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = TrackerExtraFieldDataToSOAP($obj);
	}
	
	return $res;
	
}

/*
$server->register(
	'getTrackerExtraFieldDataArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'tracker_extra_field_data_id' => 'tns:ArrayOfInt',
	), 
	array('getTrackerExtraFieldData' => 'tns:ArrayOfTrackerExtraFieldData'),
	$uri, 
	$uri.'#getTrackerExtraFieldData',
	'rpc',
	'encoded'
);
*/

/**
 * SOAP method to retrieve an array of TrackerExtraFieldData
 * @param	$tracker_extra_field_data_id	Retrieve the TrackerExtraFieldData array identified by the array of tracker_extra_field_data_id
 */
function getTrackerExtraFieldDataArray($sessionHash, $tracker_item_id, $tracker_extra_field_data_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id)) {
		return new soap_fault('', 'getTrackerExtraFieldDataArray','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$ret = array();

	$crit=new Criteria();
	$crit->add(TrackerExtraFieldDataPeer::TRACKER_EXTRA_FIELD_DATA_ID,$tracker_extra_field_data_idArr,Criteria::IN);
	$crit->add(TrackerExtraFieldDataPeer::TRACKER_ITEM_ID, $tracker_item_id);
	$objs = TrackerExtraFieldDataPeer::doSelect($crit);
	foreach ($objs as $obj) {
		$ret[] = TrackerExtraFieldDataToSOAP($obj);
	}
	return $ret;
}

$server->register(
	'getTrackerExtraFieldData',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'tracker_extra_field_data_id' => 'xsd:int',
	), 
	array('getTrackerExtraFieldData' => 'tns:TrackerExtraFieldData'),
	$uri, 
	$uri.'#getTrackerExtraFieldData',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single TrackerExtraFieldData
 * @param	$tracker_extra_field_data_id	Retrieve the TrackerExtraFieldData identified by this tracker_extra_field_data_id
 */
function getTrackerExtraFieldData($sessionHash, $tracker_item_id, $tracker_extra_field_data_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id)) {
		return new soap_fault('', 'getTrackerExtraFieldData','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$obj = TrackerExtraFieldDataPeer::retrieveByPK($tracker_extra_field_data_id);
	if (is_null($obj) || $obj->getTrackerItemId() != $tracker_item_id) {
		return null;
	}
	return TrackerExtraFieldDataToSOAP($obj);
}

/*
$server->register(
	'deleteTrackerExtraFieldData',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'tracker_extra_field_data_id' => 'xsd:int',
	), 
	array('deleteTrackerExtraFieldData' => 'xsd:int'),
	$uri, 
	$uri.'#deleteTrackerExtraFieldData',
	'rpc',
	'encoded'
);
*/

/**
 * SOAP method to delete a single TrackerExtraFieldData
 * @param	$tracker_extra_field_data_id	Delete the TrackerExtraFieldData identified by this tracker_extra_field_data_id
 */

/*
function deleteTrackerExtraFieldData($sessionHash, $tracker_item_id, $tracker_extra_field_data_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerExtraFieldDataPeer::retrieveByPK($tracker_extra_field_data_id);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteTrackerExtraFieldData','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteTrackerExtraFieldData','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}

*/
$server->register(
	'updateTrackerExtraFieldData',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'tracker_extra_field_id' => 'xsd:int',
		'tracker_extra_field_data_id' => 'xsd:int',
		'field_data' => 'xsd:string'
	), 
	array('updateTrackerExtraFieldData' => 'xsd:int'),
	$uri, 
	$uri.'#updateTrackerExtraFieldData',
	'rpc',
	'encoded'
);



/**
 * SOAP method to update TrackerExtraFieldData
 * @param	$tracker_item_id
 * @param	$tracker_extra_field_id
 * @param	$tracker_extra_field_data_id	 TrackerExtraFieldData to update
 * @param	$field_data
 */
 

function updateTrackerExtraFieldData($sessionHash, $tracker_item_id, $tracker_extra_field_id, $tracker_extra_field_data_id, $field_data) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = TrackerExtraFieldDataPeer::retrieveByPK($tracker_extra_field_data_id);
	if (is_null($obj)) {
		return new soap_fault('', 'updateTrackerExtraFieldData','Couldn\'t update TrackerExtraFieldData: invalid tracker_extra_field_data_id '.$tracker_extra_field_data_id);
	}
	
	//Get old data
	$old_field_data = $obj->getFieldData();
	
	if (!is_null($tracker_item_id)) {
		$obj->setTrackerItemId( Validator::sanitizeInt($tracker_item_id) );
	}
	if (!is_null($field_data)) {
		$obj->setFieldData( Validator::sanitizeString($field_data) );
	}
	if (!is_null($tracker_extra_field_id)) {
		$obj->setTrackerExtraFieldId( Validator::sanitizeInt($tracker_extra_field_id) );
	}
	
	try {
		$obj->save();
		
		//Audit the field data change
		$Audit = new AuditTrail();
		//get the data
		$user_id = $obj->getTrackerItem()->getUser()->getUserId();
		$section = TrackerItem::SECTION;
		$ref_id = $obj->getTrackerItemId();
		$field_name = substr(strtolower(substr(TrackerExtraFieldDataPeer::FIELD_DATA,strlen(TrackerExtraFieldDataPeer::TABLE_NAME)+1)),0,20); // since were getting things like Tablename.field_name, we just want to store the field name
		// set the audit object and save it to DB
		$Audit->setOldValue($old_field_data);
		$Audit->setNewValue($obj->getFieldData());
		$Audit->setRefId($ref_id);
		$Audit->setSection($section);
		$Audit->setUserId($user_id);
		$Audit->setFieldName($field_name);
		$Audit->save();
		
	} catch (Exception $e) {
		return new soap_fault('', 'updateTrackerExtraFieldData','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}

$server->register(
	'addTrackerExtraFieldData',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'tracker_extra_field_id' => 'xsd:int',
		'field_data' => 'xsd:string'
	), 
	array('addTrackerExtraFieldData' => 'xsd:int'),
	$uri, 
	$uri.'#addTrackerExtraFieldData',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a TrackerExtraFieldData
 * @param	$tracker_item_id
 * @param	$tracker_extra_field_id
 * @param	$field_data
 */


function addTrackerExtraFieldData($sessionHash, $tracker_item_id, $tracker_extra_field_id, $field_data) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = new TrackerExtraFieldData();
	
	//$obj->setTrackerExtraFieldDataId( Validator::sanitizeInt($tracker_extra_field_data_id) );
	$obj->setTrackerItemId( Validator::sanitizeInt($tracker_item_id) );
	$obj->setFieldData( Validator::sanitizeString($field_data) );
	$obj->setTrackerExtraFieldId( Validator::sanitizeInt($tracker_extra_field_id) );
	try {
		$obj->save();
		return $obj->getTrackerExtraFieldDataId();
	} catch (Exception $e) {
		return new soap_fault('', 'addTrackerExtraFieldData','Permission Denied - Cannot add This Record',$e->getMessage());
	}
	
}


$server->register(
	'getTrackerItemByExtraFieldData',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
		'tracker_extra_field_id' => 'xsd:int',
		'tracker_extra_field_alias' => 'xsd:string',
		'tracker_extra_field_data' => 'xsd:string',
		
	), 
	array('getTrackerItemByExtraFieldData' => 'tns:ArrayOfTrackerItem'),
	$uri, 
	$uri.'#getTrackerItemByExtraFieldData',
	'rpc',
	'encoded'
);



/**
 * SOAP method to retrieve a list of TrackerItem
 * @param   $tracker_id  tracker_id (must provide alias)
 * @param   $tracker_extra_field_alias Extra Field to look for (must provide tracker_id)
 * @param   $tracker_extra_field_id ID of the Extra Field to look for
 * @param   $tracker_extra_field_data Value of the extra field to look for
 * 
 */

function getTrackerItemByExtraFieldData($sessionHash, $tracker_id, $tracker_extra_field_id, $tracker_extra_field_alias, $tracker_extra_field_data) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$c = new Criteria();
	$tracker_id = Validator::sanitizeInt($tracker_id);
	$tracker_extra_field_id = Validator::sanitizeInt($tracker_extra_field_id);
	$tracker_extra_field_alias = Validator::sanitizeString($tracker_extra_field_alias);
	$tracker_extra_field_data = Validator::sanitizeString($tracker_extra_field_data);


	$c->add(TrackerExtraFieldDataPeer::FIELD_DATA, $tracker_extra_field_data);
	$c->addJoin(TrackerExtraFieldDataPeer::TRACKER_EXTRA_FIELD_ID, TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID);

	if (is_null($tracker_extra_field_id) || $tracker_extra_field_id == 0) {
		// Select by tracker id and alias
		if ($tracker_extra_field_alias == '' || is_null($tracker_extra_field_alias) || $tracker_id == 0 || is_null($tracker_id)) {
			return new soap_fault('', 'getTrackerItemByExtraFieldData','Invalid Parameters Provided','Invalid Parameters Provided');
		}
		else {
			$c->add(TrackerExtraFieldPeer::ALIAS, $tracker_extra_field_alias);
			$c->add(TrackerExtraFieldPeer::TRACKER_ID, $tracker_id);
		}
	}
	else {
		// Select by tracker_extra_field_id
		$c->add(TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID, $tracker_extra_field_id);
	}

	$c->addJoin(TrackerExtraFieldDataPeer::TRACKER_ITEM_ID, TrackerItemPeer::TRACKER_ITEM_ID);


	$objs = TrackerExtraFieldDataPeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$obj = $obj->getTrackerItem();
		if ($obj->userCanRead()) {
			$res[] = TrackerItemToSOAP($obj);
		}
	}

	return $res;

}



$server->register(
	'getTrackerItemsNewerThanExtraField',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
		'tracker_extra_field_id' => 'xsd:int',
		'tracker_extra_field_alias' => 'xsd:string',	
	), 
	array('getTrackerItemsNewerThanExtraField' => 'tns:ArrayOfTrackerItem'),
	$uri, 
	$uri.'#getTrackerItemsNewerThanExtraField',
	'rpc',
	'encoded'
);



/**
 * SOAP method to retrieve a list of TrackerItems newer than the value of an extra_field_data
 * @param   $tracker_id  tracker_id (must provide alias)
 * @param   $tracker_extra_field_alias Extra Field to look for (must provide tracker_id)
 * @param   $tracker_extra_field_id ID of the Extra Field to look for
 * 
 */

function getTrackerItemsNewerThanExtraField($sessionHash, $tracker_id, $tracker_extra_field_id, $tracker_extra_field_alias) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$c = new Criteria();
	$tracker_id = Validator::sanitizeInt($tracker_id);
	$tracker_extra_field_id = Validator::sanitizeInt($tracker_extra_field_id);
	$tracker_extra_field_alias = Validator::sanitizeString($tracker_extra_field_alias);


	$c->addJoin(TrackerExtraFieldDataPeer::TRACKER_EXTRA_FIELD_ID, TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID);

	if (is_null($tracker_extra_field_id) || $tracker_extra_field_id == 0) {
		// Select by tracker id and alias
		if ($tracker_extra_field_alias == '' || is_null($tracker_extra_field_alias) || $tracker_id == 0 || is_null($tracker_id)) {
			return new soap_fault('', 'getTrackerItemsNewerThanExtraField','Invalid Parameters Provided','Invalid Parameters Provided');
		}
		else {
			$c->add(TrackerExtraFieldPeer::ALIAS, $tracker_extra_field_alias);
			$c->add(TrackerExtraFieldPeer::TRACKER_ID, $tracker_id);
		}
	}
	else {
		// Select by tracker_extra_field_id
		$c->add(TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID, $tracker_extra_field_id);
	}

	$c->addJoin(TrackerExtraFieldDataPeer::TRACKER_ITEM_ID, TrackerItemPeer::TRACKER_ITEM_ID);
	$c->add(TrackerItemPeer::LAST_MODIFIED_DATE, TrackerItemPeer::LAST_MODIFIED_DATE . " at time zone 'UTC' > to_timestamp(" . TrackerExtraFieldDataPeer::FIELD_DATA . ", 'YYYY-MM-DD HH24:MI:SS')", Criteria::CUSTOM );


	$objs = TrackerExtraFieldDataPeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$obj = $obj->getTrackerItem();
		if ($obj->userCanRead()) {
			$res[] = TrackerItemToSOAP($obj);
		}
	}

	return $res;

}


function TrackerExtraFieldDataToSOAP(&$obj) {
	$ret = array();
	$ret['tracker_extra_field_data_id'] = $obj->getTrackerExtraFieldDataId();
	$ret['tracker_item_id'] = $obj->getTrackerItemId();
	$ret['tracker_extra_field_id'] = $obj->getTrackerExtraFieldId();
	$ret['field_data'] = $obj->getFieldData();
	return $ret;
}
?>