<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling TrackerExtraField

require_once('plugins/tracker/lib/TrackerExtraField.php');
require_once('plugins/tracker/lib/TrackerExtraFieldPeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'TrackerExtraField',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'tracker_extra_field_id' => array('name'=>'tracker_extra_field_id', 'type' => 'xsd:int'),
		'tracker_id' => array('name'=>'tracker_id', 'type' => 'xsd:int'),
		'field_name' => array('name'=>'field_name', 'type' => 'xsd:string'),
		'field_type' => array('name'=>'field_type', 'type' => 'xsd:int'),
		'attribute1' => array('name'=>'attribute1', 'type' => 'xsd:int'),
		'attribute2' => array('name'=>'attribute2', 'type' => 'xsd:int'),
		'is_required' => array('name'=>'is_required', 'type' => 'xsd:boolean'),
		'alias' => array('name'=>'alias', 'type' => 'xsd:string'),
		'field_order' => array('name'=>'field_order', 'type' => 'xsd:int'),
		'use_in_quick_browse' => array('name'=>'use_in_quick_browse', 'type' => 'xsd:boolean'),
		'show_on_submit' => array('name'=>'show_on_submit', 'type' => 'xsd:boolean'),
		'use_auto_assign' => array('name'=>'use_auto_assign', 'type' => 'xsd:boolean'),
		'use_workflow' => array('name'=>'use_workflow', 'type' => 'xsd:boolean'),
		'use_commit_rule' => array('name'=>'use_commit_rule', 'type' => 'xsd:boolean'),
		)
);


$server->wsdl->addComplexType(
	'ArrayOfTrackerExtraField',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:TrackerExtraField[]')),
	'tns:TrackerExtraField'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getTrackerExtraFields',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
		'field_type' => 'xsd:int',
		'use_in_quick_browse' => 'xsd:boolean',
		'show_on_submit' => 'xsd:boolean',
		'use_auto_assign' => 'xsd:boolean',
		'use_workflow' => 'xsd:boolean',
		'use_commit_rule' => 'xsd:boolean',
		'alias' => 'xsd:string',
	), 
	array('getTrackerExtraFields' => 'tns:ArrayOfTrackerExtraField'),
	$uri, 
	$uri.'#getTrackerExtraFields',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of TrackerExtraField
 * @param	$tracker_id	Filter by tracker_id
 * @param	$field_type	Filter by field_type
 * @param	$use_in_quick_browse	Filter by use_in_quick_browse
 * @param	$show_on_submit	Filter by show_on_submit
 * @param	$use_auto_assign	Filter by use_auto_assign
 * @param	$use_workflow	Filter by use_workflow
 * @param   $use_commit_rule Filter by use_commit_rule
 */
function getTrackerExtraFields($sessionHash, $tracker_id, $field_type,  $use_in_quick_browse, $show_on_submit, $use_auto_assign, $use_workflow, $use_commit_rule, $alias) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$c = new Criteria();

	if (!checkTrackerPermission($tracker_id)) {
		return new soap_fault('', 'getTrackerExtraFields','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}
	$c->add(TrackerExtraFieldPeer::TRACKER_ID, $tracker_id);

	if ($field_type >= 0) {
		$c->add(TrackerExtraFieldPeer::FIELD_TYPE, $field_type);
	}
	if (!is_null($use_in_quick_browse)) {
		$c->add(TrackerExtraFieldPeer::USE_IN_QUICK_BROWSE, $use_in_quick_browse);
	}
	if (!is_null($show_on_submit)) {
		$c->add(TrackerExtraFieldPeer::SHOW_ON_SUBMIT, $show_on_submit);
	}
	if (!is_null($use_auto_assign)) {
		$c->add(TrackerExtraFieldPeer::USE_AUTO_ASSIGN, $use_auto_assign);
	}
	if (!is_null($use_workflow)) {
		$c->add(TrackerExtraFieldPeer::USE_WORKFLOW, $use_workflow);
	}
	if (!is_null($use_commit_rule)) {
		$c->add(TrackerExtraFieldPeer::USE_COMMIT_RULE, $use_commit_rule);
	}
	if (!is_null($alias)) {
		$c->add(TrackerExtraFieldPeer::ALIAS, $alias);
	}
	
	$objs = TrackerExtraFieldPeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = TrackerExtraFieldToSOAP($obj);
	}
	return $res;
}

$server->register(
	'getTrackerExtraFieldArray',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
		'tracker_extra_field_id' => 'tns:ArrayOfInt',
	), 
	array('getTrackerExtraField' => 'tns:ArrayOfTrackerExtraField'),
	$uri, 
	$uri.'#getTrackerExtraField',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve an array of TrackerExtraField
 * @param	$tracker_id	Filter by tracker_id
 * @param	$tracker_extra_field_id	Retrieve the TrackerExtraField array identified by the array of tracker_extra_field_id
 */
function getTrackerExtraFieldArray($sessionHash, $tracker_id, $tracker_extra_field_idArr) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerPermission($tracker_id)) {
		return new soap_fault('', 'getTrackerExtraFieldArray','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$ret = array();

	$crit=new Criteria();
	$crit->add(TrackerExtraFieldPeer::TRACKER_EXTRA_FIELD_ID,$tracker_extra_field_idArr,Criteria::IN);
	$crit->add(TrackerExtraFieldPeer::TRACKER_ID,$tracker_id);
	$objs = TrackerExtraFieldPeer::doSelect($crit);
	foreach ($objs as $obj) {
		$ret[] = TrackerExtraFieldToSOAP($obj);
	}
	return $ret;
}

$server->register(
	'getTrackerExtraField',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
		'tracker_extra_field_id' => 'xsd:int',
	), 
	array('getTrackerExtraField' => 'tns:TrackerExtraField'),
	$uri, 
	$uri.'#getTrackerExtraField',
	'rpc',
	'encoded'
);

/**
 * SOAP method to retrieve a single TrackerExtraField
 * @param	$tracker_id	Filter by tracker_id
 * @param	$tracker_extra_field_id	Retrieve the TrackerExtraField identified by this tracker_extra_field_id
 */
function getTrackerExtraField($sessionHash, $tracker_id, $tracker_extra_field_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerPermission($tracker_id)) {
		return new soap_fault('', 'getTrackerExtraField','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$obj = TrackerExtraFieldPeer::retrieveByPK($tracker_extra_field_id);
	if (is_null($obj) || $obj->getTrackerId() != $tracker_id) {
		return null;
	}
	return TrackerExtraFieldToSOAP($obj);
}


$server->register(
	'deleteTrackerExtraField',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
		'tracker_extra_field_id' => 'xsd:int',
	), 
	array('deleteTrackerExtraField' => 'xsd:int'),
	$uri, 
	$uri.'#deleteTrackerExtraField',
	'rpc',
	'encoded'
);

/**
 * SOAP method to delete a single TrackerExtraField
 * @param	$tracker_id	Filter by tracker_id
 * @param	$tracker_extra_field_id	Delete the TrackerExtraField identified by this tracker_extra_field_id
 */
function deleteTrackerExtraField($sessionHash, $tracker_id, $tracker_extra_field_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerPermission($tracker_id, 'delete')) {
		return new soap_fault('', 'deleteTrackerExtraField','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$obj = TrackerExtraFieldPeer::retrieveByPK($tracker_extra_field_id);
	if (is_null($obj) || $obj->getTrackerId() != $tracker_id) {
		return new soap_fault('', 'deleteTrackerExtraField','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
	} catch (Exception $e) {
		return new soap_fault('', 'deleteTrackerExtraField','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'updateTrackerExtraField',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
		'tracker_extra_field_id' => 'xsd:int',
		'field_name' => 'xsd:string',
		'attribute1' => 'xsd:int',
		'attribute2' => 'xsd:int',
		'is_required' => 'xsd:boolean',
		'alias' => 'xsd:string',
		'field_order' => 'xsd:int',
		'use_in_quick_browse' => 'xsd:boolean',
		'show_on_submit' => 'xsd:boolean',
		'use_auto_assign' => 'xsd:boolean',
		'use_workflow' => 'xsd:boolean',
		'use_commit_rule' => 'xsd:boolean',
	), 
	array('updateTrackerExtraField' => 'xsd:int'),
	$uri, 
	$uri.'#updateTrackerExtraField',
	'rpc',
	'encoded'
);

/**
 * SOAP method to update TrackerExtraField
 * @param	$tracker_id
 * @param	$tracker_extra_field_id	 TrackerExtraField to update
 * @param	$field_name
 * @param	$attribute1
 * @param	$attribute2
 * @param	$is_required
 * @param	$alias
 * @param	$field_order
 * @param	$use_in_quick_browse
 * @param	$show_on_submit
 * @param	$use_auto_assign
 * @param	$use_workflow
 * @param	$use_commit_rule
 */
function updateTrackerExtraField($sessionHash, $tracker_id, $tracker_extra_field_id, $field_name, $attribute1, $attribute2, $is_required, $alias, $field_order, $use_in_quick_browse, $show_on_submit, $use_auto_assign, $use_workflow, $use_commit_rule) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerPermission($tracker_id, 'update')) {
		return new soap_fault('', 'updateTrackerExtraField','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$obj = TrackerExtraFieldPeer::retrieveByPK($tracker_extra_field_id);
	if (is_null($obj) || $obj->getTrackerId() != $tracker_id) {
		return new soap_fault('', 'updateTrackerExtraField','Couldn\'t update TrackerExtraField: invalid tracker_extra_field_id '.$tracker_extra_field_id);
	}

	if (!is_null($field_name)) {
		$obj->setFieldName( Validator::sanitizeString($field_name) );
	}
	if (!is_null($attribute1)) {
		$obj->setAttribute1( Validator::sanitizeInt($attribute1) );
	}
	if (!is_null($attribute2)) {
		$obj->setAttribute2( Validator::sanitizeInt($attribute2) );
	}
	if (!is_null($is_required)) {
		$obj->setIsRequired( Validator::sanitizeString($is_required) );
	}
	if (!is_null($alias)) {
		$obj->setAlias( Validator::sanitizeString($alias) );
	}
	if (!is_null($field_order)) {
		$obj->setFieldOrder( Validator::sanitizeInt($field_order) );
	}
	if (!is_null($use_in_quick_browse)) {
		$obj->setUseInQuickBrowse( Validator::sanitizeString($use_in_quick_browse) );
	}
	if (!is_null($show_on_submit)) {
		$obj->setShowOnSubmit( Validator::sanitizeString($show_on_submit) );
	}
	if (!is_null($use_auto_assign)) {
		$obj->setUseAutoAssign( Validator::sanitizeString($use_auto_assign) );
	}
	if (!is_null($use_workflow)) {
		$obj->setUseWorkflow( Validator::sanitizeString($use_workflow) );
	}
	if (!is_null($use_commit_rule)) {
		$obj->setUseCommitRule( Validator::sanitizeString($use_commit_rule) );
	}
	
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'updateTrackerExtraField','Permission Denied - Cannot update This Record',$e->getMessage());
	}
	return 1;
}

$server->register(
	'addTrackerExtraField',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_id' => 'xsd:int',
		'field_name' => 'xsd:string',
		'field_type' => 'xsd:int',
		'attribute1' => 'xsd:int',
		'attribute2' => 'xsd:int',
		'is_required' => 'xsd:boolean',
		'alias' => 'xsd:string',
		'field_order' => 'xsd:int',
		'use_in_quick_browse' => 'xsd:boolean',
		'show_on_submit' => 'xsd:boolean',
		'use_auto_assign' => 'xsd:boolean',
		'use_workflow' => 'xsd:boolean',
		'use_commit_rule' => 'xsd:boolean',
	), 
	array('addTrackerExtraField' => 'xsd:int'),
	$uri, 
	$uri.'#addTrackerExtraField',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a TrackerExtraField
 * @param	$tracker_id
 * @param	$field_name
 * @param	$field_type
 * @param	$attribute1
 * @param	$attribute2
 * @param	$is_required
 * @param	$alias
 * @param	$field_order
 * @param	$use_in_quick_browse
 * @param	$show_on_submit
 * @param	$use_auto_assign
 * @param	$use_workflow
 * @param	$use_commit_rule
 */
function addTrackerExtraField($sessionHash, $tracker_id, $field_name, $field_type, $attribute1, $attribute2, $is_required, $alias, $field_order, $use_in_quick_browse, $show_on_submit, $use_auto_assign, $use_workflow, $use_commit_rule) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerPermission($tracker_id, 'update')) {
		return new soap_fault('', 'addTrackerExtraField','Permission Denied - Cannot Add This Record','Permission Denied - Cannot Add This Record');
	}

	$obj = new TrackerExtraField();
	
	$obj->setTrackerId( Validator::sanitizeInt($tracker_id) );
	$obj->setFieldName( Validator::sanitizeString($field_name) );
	$obj->setFieldType( Validator::sanitizeInt($field_type) );
	$obj->setAttribute1( Validator::sanitizeInt($attribute1) );
	$obj->setAttribute2( Validator::sanitizeInt($attribute2) );
	$obj->setIsRequired( Validator::sanitizeBool($is_required) );
	if ($alias) {
		$obj->setAlias( Validator::sanitizeString($alias) );
	}
	$obj->setFieldOrder( Validator::sanitizeInt($field_order) );
	$obj->setUseInQuickBrowse( Validator::sanitizeBool($use_in_quick_browse) );
	$obj->setShowOnSubmit( Validator::sanitizeBool($show_on_submit) );
	$obj->setUseAutoAssign( Validator::sanitizeBool($use_auto_assign) );
	$obj->setUseWorkflow( Validator::sanitizeBool($use_workflow) );
	$obj->setUseCommitRule( Validator::sanitizeBool($use_commit_rule) );
	try {
		$obj->save();
		return $obj->getTrackerExtraFieldId();
	} catch (Exception $e) {
		return new soap_fault('', 'addTrackerExtraField','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function checkTrackerExtraFieldPermission($tracker_extra_field_id, $type='') {
	$obj = TrackerExtraFieldPeer::retrieveByPK($tracker_extra_field_id);
	if (is_null($obj)) {
		return false;
	} else if ($type=='delete') {
		return checkTrackerPermission($obj->getTrackerId(), 'delete');
	} else if ($type=='create') {
		return checkTrackerPermission($obj->getTrackerId(), 'create');
	} else if ($type=='update') {
		return checkTrackerPermission($obj->getTrackerId(), 'update');
	} else {
		return checkTrackerPermission($obj->getTrackerId());
	}
}

function TrackerExtraFieldToSOAP(&$obj) {
	$ret = array();
	$ret['tracker_id'] = $obj->getTrackerId();
	$ret['tracker_extra_field_id'] = $obj->getTrackerExtraFieldId();
	$ret['field_name'] = $obj->getFieldName();
	$ret['field_type'] = $obj->getFieldType();
	$ret['attribute1'] = $obj->getAttribute1();
	$ret['attribute2'] = $obj->getAttribute2();
	$ret['is_required'] = $obj->getIsRequired();
	$ret['alias'] = $obj->getAlias();
	$ret['field_order'] = $obj->getFieldOrder();
	$ret['use_in_quick_browse'] = $obj->getUseInQuickBrowse();
	$ret['show_on_submit'] = $obj->getShowOnSubmit();
	$ret['use_auto_assign'] = $obj->getUseAutoAssign();
	$ret['use_workflow'] = $obj->getUseWorkflow();
	$ret['use_commit_rule'] = $obj->getUseCommitRule();
	return $ret;
}
?>
