<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling TrackerItemAssignee

require_once('plugins/tracker/lib/TrackerItemAssignee.php');
require_once('plugins/tracker/lib/TrackerItemAssigneePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'TrackerItemAssignee',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'tracker_item_id' => array('name'=>'tracker_item_id', 'type' => 'xsd:int'),
		'assignee' => array('name'=>'assignee', 'type' => 'xsd:int'),
		'percent_utilization' => array('name'=>'percent_utilization', 'type' => 'xsd:int'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfTrackerItemAssignee',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:TrackerItemAssignee[]')),
	'tns:TrackerItemAssignee'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getTrackerItemAssignees',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int'
	), 
	array('getTrackerItemAssignees' => 'tns:ArrayOfTrackerItemAssignee'),
	$uri, 
	$uri.'#getTrackerItemAssignees',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of TrackerItemAssignee
 * @param	$tracker_item_id	Filter by tracker_item_id
 */

function getTrackerItemAssignees($sessionHash, $tracker_item_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id)) {
		return new soap_fault('', 'getTrackerItemAssignees','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$c = new Criteria();
	$c->add(TrackerItemAssigneePeer::TRACKER_ITEM_ID, $tracker_item_id);
	$objs = TrackerItemAssigneePeer::doSelect($c);
	$res = array();
	foreach ($objs as $obj) {
		$res[] = TrackerItemAssigneeToSOAP($obj);
	}
	return $res;
}


$server->register(
	'deleteTrackerItemAssignee',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'assignee' => 'xsd:int',
	), 
	array('deleteTrackerItemAssignee' => 'xsd:int'),
	$uri, 
	$uri.'#deleteTrackerItemAssignee',
	'rpc',
	'encoded'
);


/**
 * SOAP method to update a TrackerItemAssignee
 * @param	$tracker_item_id
 * @param	$assignee
 */
 
function deleteTrackerItemAssignee($sessionHash, $tracker_item_id, $assignee) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id, 'update')) {
		return new soap_fault('', 'deleteTrackerItemAssignee','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$c = new Criteria();
	$c->add(TrackerItemAssigneePeer::TRACKER_ITEM_ID, $tracker_item_id);
	$c->add(TrackerItemAssigneePeer::ASSIGNEE, $assignee);
	$obj = TrackerItemAssigneePeer::doSelectOne($c);
	if (is_null($obj)) {
		return new soap_fault('', 'deleteTrackerItemAssignee','Permission Denied - Cannot Delete This Record','Permission Denied - Cannot View This Record');
	}
	try {
		$obj->delete();
		
    //Audit the field data change
    $Audit = new AuditTrail();
    //get the data
    $user_id = $obj->getTrackerItem()->getUser()->getUserId();
    $old_field_data = UserPeer::retrieveByPk($assignee)->getFullname();
    $section = TrackerItem::SECTION;
    $ref_id = $obj->getTrackerItem()->getTrackerItemId();
    $field_name = 'assignee';
    // set the audit object and save it to DB
    $Audit->setOldValue($old_field_data);
    $Audit->setNewValue('Removed');
    $Audit->setRefId($ref_id);
    $Audit->setSection($section);
    $Audit->setUserId($user_id);
    $Audit->setFieldName($field_name);
    $Audit->save();
		
	} catch (Exception $e) {
		return new soap_fault('', 'deleteTrackerItemAssignee','Permission Denied - Cannot Delete This Record',$e->getMessage());
	}
	return 1;
}


$server->register(
	'updateTrackerItemAssignee',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'assignee' => 'xsd:int',
		'percent_utilization' => 'xsd:int',
	), 
	array('updateTrackerItemAssignee' => 'xsd:int'),
	$uri, 
	$uri.'#updateTrackerItemAssignee',
	'rpc',
	'encoded'
);


/**
 * SOAP method to update a TrackerItemAssignee
 * @param	$tracker_item_id
 * @param	$assignee
 * @param	$percent_utilization
 */
 
function updateTrackerItemAssignee($sessionHash, $tracker_item_id, $assignee, $percent_utilization) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id, 'update')) {
		return new soap_fault('', 'updateTrackerItemAssignee','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$c = new Criteria();
	$c->add(TrackerItemAssigneePeer::TRACKER_ITEM_ID, $tracker_item_id);
	$c->add(TrackerItemAssigneePeer::ASSIGNEE, $assignee);
	$obj = TrackerItemAssigneePeer::doSelectOne($c);

	if (is_null($obj)) {
		return new soap_fault('', 'updateTrackerItemAssignee','Permission Denied - Cannot Update This Record','Permission Denied - Cannot View This Record');
	}

	$obj->setAssignee( Validator::sanitizeInt($assignee) );
	$obj->setPercentUtilization( Validator::sanitizeInt($percent_utilization) );
	try {
		$obj->save();
	} catch (Exception $e) {
		return new soap_fault('', 'addTrackerItemAssignee','Permission Denied - Cannot add This Record',$e->getMessage());
	}
	return 1;
}

$server->register(
	'addTrackerItemAssignee',
	array(
		'sessionHash'	=> 'xsd:string',
		'tracker_item_id' => 'xsd:int',
		'assignee' => 'xsd:int',
		'percent_utilization' => 'xsd:int',
	), 
	array('addTrackerItemAssignee' => 'xsd:int'),
	$uri, 
	$uri.'#addTrackerItemAssignee',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a TrackerItemAssignee
 * @param	$tracker_item_id
 * @param	$assignee
 * @param	$percent_utilization
 */
 
function addTrackerItemAssignee($sessionHash, $tracker_item_id, $assignee, $percent_utilization) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	if (!checkTrackerItemPermission($tracker_item_id, 'update')) {
		return new soap_fault('', 'addTrackerItemAssignee','Permission Denied - Cannot View This Record','Permission Denied - Cannot View This Record');
	}

	$obj = new TrackerItemAssignee();

	$obj->setTrackerItemId( Validator::sanitizeInt($tracker_item_id) );
	$obj->setAssignee( Validator::sanitizeInt($assignee) );
	$obj->setPercentUtilization( Validator::sanitizeInt($percent_utilization) );
	try {
		$obj->save();
		
    //Audit the field data change
    $Audit = new AuditTrail();
    //get the data
    $user_id = $obj->getTrackerItem()->getUser()->getUserId();
    
    $section = TrackerItem::SECTION;
    $ref_id = $obj->getTrackerItem()->getTrackerItemId();
    $field_name = 'assignee';
    $new_field_data = UserPeer::retrieveByPk($assignee)->getFullname();
    // set the audit object and save it to DB
    $Audit->setOldValue('None');
    $Audit->setNewValue($new_field_data);
    $Audit->setRefId($ref_id);
    $Audit->setSection($section);
    $Audit->setUserId($user_id);
    $Audit->setFieldName($field_name);
    $Audit->save();
		
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'addTrackerItemAssignee','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function TrackerItemAssigneeToSOAP(&$obj) {
	$ret = array();
	$ret['tracker_item_id'] = $obj->getTrackerItemId();
	$ret['assignee'] = $obj->getAssignee();
	$ret['percent_utilization'] = $obj->getPercentUtilization();
	return $ret;
}
?>