<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

// This code generates a SOAP API for handling TrackerMyQueue

require_once('plugins/tracker/lib/TrackerMyQueue.php');
require_once('plugins/tracker/lib/TrackerMyQueuePeer.php');
require_once('Validator.php');

/* ******************************************************************** */
/*                                 DATA TYPES                           */
/* ******************************************************************** */
$server->wsdl->addComplexType(
	'TrackerMyQueue',
	'complexType',
	'struct',
	'sequence',
	'',
	array(
		'user_id' => array('name'=>'user_id', 'type' => 'xsd:int'),
		'tracker_item_id' => array('name'=>'tracker_item_id', 'type' => 'xsd:int'),
		'sort_order' => array('name'=>'sort_order', 'type' => 'xsd:int'),
	)
);


$server->wsdl->addComplexType(
	'ArrayOfTrackerMyQueue',
	'complexType',
	'array',
	'',
	'SOAP-ENC:Array',
	array(),
	array(array('ref' => 'SOAP-ENC:arrayType', 'wsdl:arrayType' => 'tns:TrackerMyQueue[]')),
	'tns:TrackerMyQueue'
);

/* ******************************************************************** */
/*                                 METHODS                              */
/* ******************************************************************** */

$server->register(
	'getTrackerMyQueues',
	array(
		'sessionHash'	=> 'xsd:string',
		'user_id' => 'xsd:int',
	), 
	array('getTrackerMyQueues' => 'tns:ArrayOfTrackerMyQueue'),
	$uri, 
	$uri.'#getTrackerMyQueues',
	'rpc',
	'encoded'
);


/**
 * SOAP method to retrieve a list of TrackerMyQueue
 * @param	$user_id	Filter by user_id
 */
function getTrackerMyQueues($sessionHash, $user_id) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);
	$User = Session::getSession()->getUser();

	$crit = new Criteria();
	
	if (!is_null($user_id)) {
		$crit->add(TrackerMyQueuePeer::USER_ID, $user_id);
	}
	$crit->addAscendingOrderByColumn(TrackerMyQueuePeer::SORT_ORDER);
	$objs = TrackerMyQueuePeer::doSelect($crit);
	
	$res = array();
	foreach ($objs as $obj) {
		if ($User->isSiteAdmin() || $User->getUserId() == $obj->getUserId()) {
			$res[] = TrackerMyQueueToSOAP($obj);
		}
	}
	return $res;
}



$server->register(
	'addTrackerMyQueue',
	array(
		'sessionHash'	=> 'xsd:string',
		'user_id' => 'xsd:int',
		'tracker_item_id' => 'xsd:int',
		'sort_order' => 'xsd:int',
	), 
	array('addTrackerMyQueue' => 'xsd:int'),
	$uri, 
	$uri.'#addTrackerMyQueue',
	'rpc',
	'encoded'
);


/**
 * SOAP method to add a TrackerMyQueue
 * @param	$user_id
 * @param	$tracker_item_id
 * @param	$sort_order
 */
function addTrackerMyQueue($sessionHash, $user_id, $tracker_item_id, $sort_order) {
	$ip = $_SERVER['REMOTE_ADDR'];
	Session::getSession()->setup($ip, $sessionHash);

	$obj = new TrackerMyQueue();
	
	$obj->setUserId( Validator::sanitizeInt($user_id) );
	$obj->setTrackerItemId( Validator::sanitizeInt($tracker_item_id) );
	$obj->setSortOrder( Validator::sanitizeInt($sort_order) );
	try {
		$obj->save();
		return 1;
	} catch (Exception $e) {
		return new soap_fault('', 'addTrackerMyQueue','Permission Denied - Cannot add This Record',$e->getMessage());
	}
}

function TrackerMyQueueToSOAP(&$obj) {
	$ret = array();
	$ret['user_id'] = $obj->getUserId();
	$ret['tracker_item_id'] = $obj->getTrackerItemId();
	$ret['sort_order'] = $obj->getSortOrder();
	return $ret;
}
?>