<?php

/* * **********************************************************************
 * CSS and Javascript Combinator 0.5
 * Copyright 2006 by Niels Leenheer
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

require_once ('initialize.php');
require_once ('GFConf.php');

$base     = dirname(__FILE__);
$realbase = realpath($base);
$baseUrl  = '//'.GFConf::get('system.hostName');
$cache    = GFConf::get('system.cacheResources', true);
$cachedir = GFConf::get('system.cacheDir').'/resources';

$type     = '';
$elements = array();

// Determine the directory and type we should use
if (array_key_exists('type', $_GET)) {
	$type = $_GET['type'];
	switch ($_GET['type']) {
		case 'css':
            $contentType = 'text/css';
			break;
        case 'js':
		case 'javascript':
            $contentType = 'application/x-javascript';
			break;
		default:
			error_log("Resource combiner received a request with type = $type, invalid");
			header("HTTP/1.0 503 Not Implemented");
			exit;
	};
} else {
	error_log("Resource combiner received a request without a type, invalid");
	header("HTTP/1.0 503 Not Implemented");
	exit;
}

if (array_key_exists('files', $_GET)) {
	//error_log("Combine using file list ".$_GET['files']);
	$elements = explode(',', $_GET['files']);
}

if (!is_array($elements) || count($elements) < 1) {
	error_log("Resource combiner received a request for $type, but no elements");
	exit;
}

// Determine last modification date of the files
$lastmodified = 0;
while (list(, $element) = each($elements)) {
	$path = trim($base.$element);

	if ($path > $realbase) {
		if (($type == 'javascript' && substr($path, -3) != '.js') ||
			($type == 'css' && substr($path, -4) != '.css')) {
			error_log("Resource $path is not allowed for type $type");
			header("HTTP/1.0 403 Forbidden");
			exit;
		} else if (!file_exists($path)) {
			error_log("Resource $path does not exist");
			header("HTTP/1.0 404 Not Found");
			exit;
		} else {
			$lastmodified = max($lastmodified, filemtime($path));
		}
	}
}

// Send Etag hash, last modified and content type headers first thing
$hash = $lastmodified.'-'.md5($_GET['files']);
header("Etag: \"".$hash."\"");

$lastModDate = gmdate('D, d M Y H:i:s ', $lastmodified) . 'GMT';
header("Last-Modified: {$lastModDate}");

header("Content-Type: {$contentType}");

if ( (eTagModified($hash, $_SERVER) == false) &&
     (modifiedSince($lastModDate, $_SERVER) == false) ) {
	// Return visit and no modifications, so do not send anything
	header("HTTP/1.0 304 Not Modified");
	header('Content-Length: 0');
} else {
	// First time visit or files were modified
	if ($cache) {
		// Determine supported compression method
		$ae      = array_key_exists('HTTP_ACCEPT_ENCODING', $_SERVER)?$_SERVER['HTTP_ACCEPT_ENCODING']:'';
		$gzip    = strstr($ae, 'gzip');
		$deflate = strstr($ae, 'deflate');

		// Determine used compression method
		$encoding = $gzip?'gzip':($deflate?'deflate':'none');

		// Check for buggy versions of Internet Explorer
		$ua = array_key_exists('HTTP_USER_AGENT', $_SERVER)?$_SERVER['HTTP_USER_AGENT']:'';
		if (!strstr($ua, 'Opera') &&
			preg_match('/^Mozilla\/4\.0 \(compatible; MSIE ([0-9]\.[0-9])/i', $ua, $matches)) {
			$version = floatval($matches[1]);

			if ($version < 6) {
				$encoding = 'none';
			}

			if ($version == 6 && !strstr($ua, 'EV1')) {
				$encoding = 'none';
			}
		}

		// Try the cache first to see if the combined files were already generated
		$cachefile = 'cache-'.$hash.'.'.$type.($encoding != 'none'?'.'.$encoding:'');

		if (file_exists($cachedir.'/'.$cachefile)) {
			$fp = fopen($cachedir.'/'.$cachefile, 'rb');
			if ($fp) {
				//error_log("Serving cached content from $cachedir/$cachefile FTW");

				if ($encoding != 'none') {
					header("Content-Encoding: ".$encoding);
				}

				header("Content-Length: ".filesize($cachedir.'/'.$cachefile));

				fpassthru($fp);
				fclose($fp);
				exit;
			} else {
				error_log("Could not read cached content from $cachedir/$cachefile");
			}
		}
	}

	// Get contents of the files
	$contents = '';
	// Resource combining makes the relative path of each individual CSS file go away.
	// This is the regex used to re-write URLs embedded in CSS,
	// so that relative paths are made into absolute ones.
	// Original pattern: '/url\((.+)\)/i'
	// Old pattern: '/url\([\'"]?[\/]?(.+)[\'"]?\)/i'
	$url_pattern   = '/url\(([^\/])(.+)\)/i';
	$single_quotes = '/url\(\'(.+)\'\)/i';
	$double_quotes = '/url\(\"(.+)\"\)/i';

	reset($elements);
	//error_log("combiner, elements are: ".print_r($elements, true));
	while (list(, $element) = each($elements)) {
		$path = trim($base.$element);
		// Skip any stray spaces or double-commas in the list
		if ($path > $realbase) {
			if (file_exists($path)) {
				$fileContents = file_get_contents($path);

				if ($type == 'css') {
					$subDir = dirname($element);
					//str_replace($realbase, '', dirname($path));

					if (substr($subDir, 0, 1) != '/' &&
						substr($baseUrl, -1, 1) != '/') {
						$subDir = "/{$subDir}";
					}
					// This is the regex for the absolute URL to be used for each url from pattern above
					$replaceWith = 'url('.$baseUrl.$subDir.'/$1$2)';

					//error_log("Creating css content URLs for $element (in $subDir), using base URL $baseUrl");
					$fileContents = preg_replace($single_quotes, "url($1)", $fileContents);
					$fileContents = preg_replace($double_quotes, "url($1)", $fileContents);
					$fileContents = preg_replace($url_pattern, $replaceWith, $fileContents);
				}

				$contents .= "\n\n".$fileContents;
			} else {
				error_log("Resource Combiner, file $path was requested but does not exist, skipped.");
			}
		}
	}

	if (isset($encoding) && $encoding != 'none') {
		// Send compressed contents
		$contents = gzencode($contents, 9, $gzip?FORCE_GZIP:FORCE_DEFLATE);
		header("Content-Encoding: ".$encoding);
	}

	header('Content-Length: '.strlen($contents));
	echo $contents;

	// Store cache
	if ($cache) {
		if (!file_exists($cachedir) || !is_dir($cachedir)) {
			if (!mkdir($cachedir, 0770)) {
				error_log("Could not create $cachedir with permissions 770");
				exit;
			}
		}

		$fp = fopen($cachedir.'/'.$cachefile, 'wb');
		if ($fp) {
			fwrite($fp, $contents);
			fclose($fp);
		} else {
			error_log("Could not write cached content to $cachefile");
		}
	}
}

function eTagModified($etag, $serverVars) {
    if (isset($_SERVER['HTTP_IF_MODIFIED_SINCE'])) {
        $actual = rtrim(ltrim($serverVars['HTTP_IF_NONE_MATCH'], "'\""), "'\"");
        return ($actual != $etag);
    }
    return true;
}

function modifiedSince($modDate, $serverVars) {
    if (isset($_SERVER['HTTP_IF_MODIFIED_SINCE'])) {
        $if_modified_since = $_SERVER['HTTP_IF_MODIFIED_SINCE'];
        return ($modDate > $if_modified_since);
    }
    return true;
}