<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @author
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//$_SERVER['REMOTE_USER']='ruben@foo.com';

set_error_handler("errorHandler");
register_shutdown_function("shutdownHandler");


/**
 * Auxiliary function used to parse values in php.ini like '8M'
 */
function parse_bytes($val) {
	$last = strtolower($val{strlen($val) - 1});
	$val = intval($val);
	switch ($last) {
		case 'g':
			$val *= 1024;
		case 'm':
			$val *= 1024;
		case 'k':
			$val *= 1024;
	}

	return $val;
}

/**
 * GForge's main web script
 */
try {
	$libPath = dirname(dirname(__FILE__)).'/lib';

	set_include_path($libPath.
		PATH_SEPARATOR.
		$libPath.'/external'.
		PATH_SEPARATOR.
		get_include_path());

	// Initialize propel and database connection
	require_once('initialize.php');

	require_once('GFConf.php');
	require_once('GFUrl.php');
	require_once('session/Session.php');
	require_once('core/Language.php');
	require_once('plugin/PluginFactory.php');
	require_once('core/ActivityLog.php');
	require_once('exceptions/PermissionDeniedException.php');
	require_once('exceptions/NotLoggedInException.php');
	require_once('exceptions/InvalidProjectException.php');
	require_once('exceptions/NotFoundException.php');
	require_once('exceptions/InvalidDataException.php');

	//Disable clickjacking by only allowing frames in the same origin
	if (!GFConf::get('system.enableiFrames', false)) {
		header('X-Frame-Options: SAMEORIGIN');
	}

	// Initialize plugins
	// Set up session
	$session = Session::getSession();

	// user may already be logged in
	$ip = $_SERVER['REMOTE_ADDR'];
	try {
		$session->setup($ip);
	} catch (PropelException $pe) {
		header($_SERVER['SERVER_PROTOCOL'] . ' 500 Internal Server Error', true, 500);
		echo '<strong>Could Not Connect To Database</strong> <!--' . $pe->getMessage();			echo '<br /><pre>';
		echo $pe->getTraceAsString();
		echo '</pre>-->';
		die();
	} catch (SessionException $ex) {
		//throw new NotLoggedInException();  // Pass the exception to the exception handler
	}

	if ($session->isLoggedIn()) {
		//sso begin
		if (GFConf::get('system.useSSO') && GFConf::get('system.SSOServerVarAlwaysPresent')) {
			//the SSO VAR was not passed, so log out
			if (!isset($_SERVER[GFConf::get('system.SSOServerVar')])) {
				throw new SessionException("Logging out of GForge since you are logged out of Siteminder");
			}
			//the remote address may need to be split and take only the first part
			$tmpsplitchar = GFConf::get('system.SSOServerVarSplitChar');
			if (!empty($tmpsplitchar)) {
				$tmpserverid = explode(GFConf::get('system.SSOServerVarSplitChar'), $_SERVER[GFConf::get('system.SSOServerVar')]);
				$_SERVER[GFConf::get('system.SSOServerVar')] = $tmpserverid[0];
			}
			//make sure the passed SSO VAR == the session user
			if ($_SERVER[GFConf::get('system.SSOServerVar')] != $session->getUser()->getExternalId() && strtolower($_SERVER[GFConf::get('system.SSOServerVar')]) != $session->getUser()->getUnixName()) {
				// 2012-11-01 RMT [#9285]
				// Fix an SSO-only problem with multiple users
				// logging in through SSO from the same browser window

				// We used to throw SessionException here, but the right
				// answer is to invalidate the old session and start over
				// with the supplied SSO parameters
                $session->terminate();
				throw new NotLoggedInException();  // Pass the exception to the exception handler
			}
		}
		//sso end

        // TFA active + logged user has no cell, redirect to edit cell form
        /*
		if (GFConf::get('system.useTFA')
            && $_REQUEST['action'] != 'Logout'
            && $_REQUEST['action'] != 'UserEditCellTFA'
            && $_REQUEST['action'] != 'MessageWallUserNotice') {
            $user = $session->getUser();
            $cell = $user->getCell();
            if (empty($cell)) {// user needs to update cell
                header("Location: /gf/account/?action=UserEditCellTFA");
                exit;
            }
        }
        */

	} else {
		// Set up language and theme
		if (isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
			// User is not logged in, so, we try to set the browser language
			// if it doesn't exists, we just set de defaultlanguage from conf file
			$langcodes_arr = explode(',', $_SERVER['HTTP_ACCEPT_LANGUAGE']);
			$langcode = substr($langcodes_arr[0], 0, 2);

			// retrieve the language object from the DB
			$c = new Criteria(LanguagePeer::DATABASE_NAME);
			$c->add(LanguagePeer::LANGUAGE_CODE, $langcode);
			$browserlang = LanguagePeer::doSelectOne($c);
			if (is_object($browserlang)) {
				$session->setLanguage($browserlang);
			}
		}
		//sso begin
		if (GFConf::get('system.useSSO') && isset($_SERVER[GFConf::get('system.SSOServerVar')])) {
			$allowed_actions = array();
			$allowed_actions['Login'] = 1;
			$allowed_actions['LoginAction'] = 1;
			$allowed_actions['Logout'] = 1;
			$allowed_actions['UserAdd'] = 1;
			$allowed_actions['UserAddAction'] = 1;
			$allowed_actions['ChangeSSO'] = 1;
			$allowed_actions['ChangeSSOAction'] = 1;
			if (isset($_REQUEST['action']) && isset($allowed_actions[$_REQUEST['action']])) {

			} else {
				throw new NotLoggedInException();
			}
		}
		//sso end
		// Check for manadatory login for all users except some specificpages
		if (GFConf::get('system.forceAllLogin')) {
			$allowed = false;

			if ($_SERVER['REQUEST_URI'] == '/gf/' || $_SERVER['REQUEST_URI'] == '/gf.php/') {
				// Home page
				$allowed = true;
			} elseif (strpos($_SERVER['REQUEST_URI'], '/gf/account/') === 0 || strpos($_SERVER['REQUEST_URI'], '/gf.php/account/') === 0) {
				// Login page
				$allowed = true;
			} elseif (strpos($_SERVER['REQUEST_URI'], '/gf/?action=secureimage') === 0 || strpos($_SERVER['REQUEST_URI'], '/gf.php/?action=secureimage') === 0) {
				// CAPTCHAs (images and audio)
				$allowed = true;
			} elseif ($_SERVER['REQUEST_URI'] == '/gf/graphs/?action=Activity' || $_SERVER['REQUEST_URI'] == '/gf.php/graphs/?action=Activity') {
				// Activity graph
				$allowed = true;
			} elseif (strpos($_SERVER['REQUEST_URI'], '/gf/build/') === 0 || strpos($_SERVER['REQUEST_URI'], '//gf/build/') === 0
					|| strpos($_SERVER['REQUEST_URI'], '/gf.php/build/') === 0 || strpos($_SERVER['REQUEST_URI'], '//gf.php/build/') === 0) {
				// Cruise Control
				$allowed = true;
			} elseif (strpos($_SERVER['REQUEST_URI'], '/gf/xmlcompatibility/') === 0 || strpos($_SERVER['REQUEST_URI'], '//gf/xmlcompatibility/') === 0
					|| strpos($_SERVER['REQUEST_URI'], '/gf.php/xmlcompatibility/') === 0 || strpos($_SERVER['REQUEST_URI'], '//gf.php/xmlcompatibility/') === 0) {
				// SOAP interface
				$allowed = true;
      } else if (strpos($_SERVER['REQUEST_URI'], '/gf/?action=Isup') === 0 || strpos($_SERVER['REQUEST_URI'], '/gf.php/?action=Isup') === 0) {
        // Server status check
        $allowed = true;
      }

			if (!$allowed) {
				throw new NotLoggedInException();
			}
		}
		//session_start();
	}


	$language = $session->getLanguage();
	// This loads the theme class
	$theme = $session->getTheme();

	// gz handler -- if you need to disable it, call Session::disableGZHandler()
	if (strpos($_SERVER['PHP_SELF'], '/gf.php/download') !== 0 &&
			strpos($_SERVER['PHP_SELF'], '/gf.php/xmlcompatibility/soap') !== 0 &&
			strpos($_SERVER['PHP_SELF'], '/gf.php/xmlcompatibility/msproject') !== 0) {
		ob_start("ob_gzhandler");
	} else {

	}
  //We need to call it again in case the error handler is triggered. If the handler is triggered,
  //We need to clear the buffer and rewrite it to show the exception page, which causes errors
  //when combined with ob_gzhandler. If the handler is not called, then the buffer is automatically
  //flushed into the previous handler above (if it is created).
  ob_start(NULL);

	// Get the handler script and the corresponding action from the URL
	$url = GFUrl::getUrl();
	$handler = $url->getHandler();
	$action = $url->getAction();
	if (!file_exists($handler)) {
		throw new Exception($language->getText('Exception.NoHandler', array($handler)));
	}

	// Validate the size of $_POST. If it is too big, we'll get nasty results since
	// PHP erases the contents of $_POST and $_GET
	// NOTICE: we don't throw the exception when the handler is /upload, since we need to return the JSON response to the widget
	$max_content_length = parse_bytes(ini_get('post_max_size'));
	if (isset($_SERVER['CONTENT_LENGTH']) && $_SERVER['CONTENT_LENGTH'] > $max_content_length && !(strpos($handler, 'wwwlib/upload/') > 0)) {
		throw new Exception($language->getText('Exception.PostTooBig', array($max_content_length)));
	}

	// Make sure the site plugins are initialized
	$plugfact = PluginFactory::getFactory();
	$plugfact->initializePlugins();

	// $action will be handled by the handler
	require_once($handler);
} catch (PropelException $pe) {
	header($_SERVER['SERVER_PROTOCOL'] . ' 500 Internal Server Error', true, 500);
	error_log("PropelException in gf.php: {$pe->getMessage()}");
	error_log($pe->getTraceAsString());

	echo '<strong>Unhandled PropelException</strong> ' . $pe->getMessage();
	echo '<br /><pre>';
	echo $pe->getTraceAsString();
	echo '</pre>';
	die();
} catch (NotLoggedInException $nlie) {
	header("Location: /gf/account/?action=Login&redirect=" . urlencode($_SERVER['REQUEST_URI']));
	exit;
} catch (PermissionDeniedException $pde) {
	if (!Session::getSession()->isLoggedIn()) {
		header("Location: /gf/account/?action=Login&redirect=" . urlencode($_SERVER['REQUEST_URI']));
		exit;
	} else {
		$params = array();
		$params['title'] = 'Permission Denied';
		require_once('html/SiteTemplate.php');
		$template = new SiteTemplate();
		$template->header($params);
		echo '<h1>Permission Denied</h1>';
		$template->footer();
	}
} catch (InvalidDataException $e) {
	error_log("InvalidDataException in gf.php: {$e->getMessage()}");
	error_log($e->getTraceAsString());

	$params = array();
	$params['title'] = 'Invalid data';
	require_once('html/SiteTemplate.php');
	$template = new SiteTemplate();
	$template->header($params);
	echo '<h1>Invalid data</h1>';

	$errors = $e->getErrors();
	if (is_array($errors)) {
		$feedback = implode('<br /> *', $errors);
	} else {
		$feedback = $errors;
	}
	echo '<!--' . $feedback . '-->';

	$template->footer();
} catch (InvalidProjectException $e) {
  header('HTTP/1.0 404 Not Found');
  include('www/404.php');
  exit;
} catch (NotFoundException $e) {
  header('HTTP/1.0 404 Not Found');
  include('www/404.php');
  exit;
} catch (InvalidLicenseException $e) {
	error_log("InvalidLicenseException in gf.php: {$e->getMessage()}");
	error_log($e->getTraceAsString());

	$params = array();
	$params['title'] = 'Exception';
	require_once('html/SiteTemplate.php');
	$template = new SiteTemplate();
	$template->header($params);
	echo $e->getMessage();
	$template->footer();
	exit;
} catch (Exception $e) {
	error_log("Exception in gf.php: {$e->getMessage()}");
	error_log($e->getTraceAsString());

	$params = array();
	$params['title'] = 'Exception';
	require_once('html/SiteTemplate.php');
	$template = new SiteTemplate();
	$template->header($params);
	echo $e->getMessage();
	$template->footer();
	exit;
}

function errorHandler($error_level, $error_message, $error_file, $error_line, $error_context) {
  switch ($error_level) {
  case E_ERROR:
  case E_CORE_ERROR:
  case E_COMPILE_ERROR:
  case E_PARSE:
  case E_USER_ERROR:
    $error = "[ERROR] $error_level:$error_message FILE:$error_file at line $error_line";
    error_log($error);
    renderErrorPage($error_message, $error_level, $error_file, $error_line, $error_context);
    
    break;
  }
  return true;
}

function shutdownHandler() {
  $lasterror = error_get_last();
  switch ($lasterror['type'])
  {
  case E_ERROR:
  case E_CORE_ERROR:
  case E_COMPILE_ERROR:
  case E_USER_ERROR:
  case E_RECOVERABLE_ERROR:
  case E_CORE_WARNING:
  case E_COMPILE_WARNING:
  case E_PARSE:
    $error = "[SHUTDOWN] " . $lasterror['type'] . ":" . $lasterror['message'] . "FILE:" . $lasterror['file'] . " at line " . $lasterror['line'];
    error_log($error);
    renderErrorPage($lasterror['message'], $lasterror['type'], $lasterror['file'], $lasterror['line']);
  }
}

function renderErrorPage($error, $type, $file, $line, $context = array()) {
  //Clear any current buffer, we don't know where the error is going to occur so we must start over
  ob_end_clean();
  $params = array();
  $params['title'] = 'Exception';
	require_once('html/SiteTemplate.php');
  //If the header was already rendered (the error happened in the body of the page), then we need
  // to reset this global so that it will re-render properly
  $GLOBALS['headershown'] = false; 
  $template = new SiteTemplate();
  $template->header($params);
?>
<!-- 
<?php 
echo "Error level $type in $file at line $line";
?> -->
<h1>Error</h1><br/>
<h3><?php echo $error ?></h3>
<?php
  $template->footer();
  exit;
}