<?php
require_once 'tfa/TwoFactorAuthenticator.php';
require_once 'core/UserPeer.php';

$session = Session::getSession();

if (empty($_SESSION['register_tfa'])) {// only new users with TFA token
    GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL(''));
} else { // verify the user is in the db
    $c = new Criteria();
    $c->add(UserPeer::CELL, $_SESSION['register_tfa']['cell']);
    $User = UserPeer::DoSelectOne($c);
    if (empty($User) || $User->getStatus() != User::STATUS_TFA_PENDING) {
        // session exists but user not in db or not in TFA validation
        GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL(''));
    }
}
$Lang = $session->getLanguage();

//--- if user POSTed form evaluate token
$errorMessage = '';
if ($_SERVER['REQUEST_METHOD'] == 'POST') {// user posted token
    $token = Validator::sanitizeString($_POST['token']);
    $tfa = new TwoFactorAuthenticator();
    $tokenError = $tfa->verifyToken($token, 'register');
    if (empty($tokenError)) {
        $User->saveAfterTFAConfirmation();
        // If the administrator doesn't have to approve this account, send
        // the account confirmation email
        if (GFConf::get('system.approveUsers') == false) {
            $User->sendRegistrationEmail();
        }
        GFUrl::getUrl()->redirect(GFUrl::getUrl()
          ->currentBaseURL('PostUserRegistration'));
    } else {
        error_log($tokenError);
        $errorMessage = $Lang->getText('Validation.TFATokenInvalid');
    }
}
//--- end token evaluation

require_once('html/SiteTemplate.php');
$theme = new SiteTemplate();

$params = array(
    'selectedtoptab' => 'my',
    'selectedbottomtab' => 'core',
    'title' => $Lang->getText('User.TFA.PostRegisterTitle'),
    'feedback' => $errorMessage
    );

$theme->header($params);
?>
<p><strong><?php echo $Lang->getText('User.TFA.ActivateCellTitle'); ?></strong></p>
<p><?php echo $Lang->getText('User.TFA.ActivateCellDescription'); ?></p>
<form action="<?php echo GFUrl::getUrl()->currentBaseURL('PostUserRegistrationTFA') ?>" method="POST" autocomplete="off">
    <table border="0">
    <tr>
        <td>
        <strong><?php echo $Lang->getText('User.TFA.TokenName'); ?></strong>
        <br />
        <input type="text" name="token" maxlength="12" value=""  autocomplete="off" />
        </td>
    </tr>
    <tr>
        <td>
        <?php echo HTMLWidget::button("submit","" .   $Lang->getText('UserAddForm.submit') . " ","submit","","")?>
        </td>
    </tr>
    </table>
</form>

<?php
$theme->footer($params);
?>
