<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

//
// This code creates a new User and saves it to the 
// database
//
require_once('Validator.php');
require_once('SystemUtils.php');
require_once('exceptions/InvalidDataException.php');
require_once('lib/core/User.php');
require_once('session/Session.php');
require_once('securimage/securimage.php');

$User = new User();
$session = Session::getSession();

if ($session->isLoggedIn()) {
	// if user is already logged in, go to the logout page
	header('Location: /gf/account/?action=Logout');
}

if(GFConf::get('system.usesLdap') && !GFConf::get('ldap.dbLoginPrefered')){
	throw new PermissionDeniedException();
}

$errors = array();

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

if (!isset($_POST['password']) || !isset($_POST['password_confirm'])) {
	$errors['password'] = Language::getSessText('User.EmptyPassword');
}
// note that we make this validation _outside_ the User class on purpose
if ($_POST['password'] != $_POST['password_confirm']) {
	$errors['password'] = Language::getSessText('User.PasswordsDontMatch');
}
if (GFConf::get('system.userRegClickLicense')) {
	if (isset($_REQUEST['iagree']) && Validator::sanitizeBool($_REQUEST['iagree'])) {
	} else {
		$errors['license']=Language::getSessText('License.DidntCheck');
	}
}

//if captcha is enabled, checks captcha code entered
if (GFConf::get('system.useCaptcha') == true) { 
	//Get form captcha
	$captcha_code = isset($_POST['captcha_code']) ? $_POST['captcha_code'] : 0;
	$securimage = new Securimage();
	//Verify captcha
	if ($securimage->check($captcha_code) == false) {
		$errors['captcha']=Language::getSessText('Captcha.DontMatch');
	}
}

if ($_POST['hash']) {
  $hash = Validator::sanitizeString($_POST['hash']);
  $invite = getInviteByHash($hash);
  if (!$invite) {
    $errors['invite'] = Language::getSessText("Project.Invite.Accept.InvalidHash");
  }
}

$c = new Criteria();
$c->add(UserPeer::UNIX_NAME, Validator::sanitizeString($_POST['unix_name']));
$u = UserPeer::doSelectOne($c);

// TFA active and new user still didn't confirmed cell
/*
if (is_object($u) && $u->getStatus() == User::STATUS_TFA_PENDING) {
    GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('PostUserRegistrationTFA'));
}
*/

if (is_object($u) && $u->getStatus() == User::STATUS_PENDING) {
  $url = GFUrl::getUrl()->currentUrl(array(
    'action' => 'ResendConfirmation',
    'user' => $u->getUnixName()
  ));
  $e = Language::getSessText("Validation.UserExistsPending");
  $e .= "<br />&nbsp;&nbsp;&nbsp;<a href='" . $url . "'>" . Language::getSessText("Validation.UserExistsPendingLink") . "</a><br />";
  $errors['exists'] = $e;
}

$cell = isset($_POST['cell']) ? Validator::sanitizeString($_POST['cell']) : '';

if (empty($cell)) {
	/*
    if (GFConf::get('system.useTFA')) {// TFA active, cell is required
        $errors['cell'] = Language::getSessText("Validation.CellEmpty");
    }
	*/
} else if (!isCellUnique($cell)) {// validate cell is unique
    $errors['cell'] = Language::getSessText("Validation.CellExists");
}

if (count($errors) == 0) {
	$User->setUnixName( isset($_POST['unix_name']) ? Validator::sanitizeString($_POST['unix_name']) : '' );
  if ($invite) {
    $User->setEmail($invite['email']);
  } else {
	  $User->setEmail( isset($_POST['email']) ? Validator::sanitizeString($_POST['email']) : '' );
  }
	$User->setPassword( $_POST['password'] );		// Password must not be sanitized
	$User->setTimezone( isset($_POST['timezone']) ? Validator::sanitizeString($_POST['timezone']) : '' );
	$User->setDateFormat( isset($_POST['date_format']) ? Validator::sanitizeString($_POST['date_format']) : '' );
	$User->setAddress( isset($_POST['address']) ? Validator::sanitizeString($_POST['address']) : '' );
	$User->setAddress2( isset($_POST['address2']) ? Validator::sanitizeString($_POST['address2']) : '' );
    $User->setPhone( isset($_POST['phone']) ? Validator::sanitizeString($_POST['phone']) : '' );
    $User->setCell($cell);
	$User->setFax( isset($_POST['fax']) ? Validator::sanitizeString($_POST['fax']) : '' );
	$User->setTitle( isset($_POST['title']) ? Validator::sanitizeString($_POST['title']) : '' );
	$User->setFirstname( isset($_POST['firstname']) ? Validator::sanitizeString($_POST['firstname']) : '' );
	$User->setLastname( isset($_POST['lastname']) ? Validator::sanitizeString($_POST['lastname']) : '' );
	$User->setCcode( isset($_POST['ccode']) ? Validator::sanitizeString($_POST['ccode']) : '' );
	$User->setLanguageId( isset($_POST['language_id']) ? Validator::sanitizeInt($_POST['language_id']) : '' );
	$User->setThemeId( isset($_POST['theme_id']) ? Validator::sanitizeInt($_POST['theme_id']) : '' );
	$User->setConfirmHash(substr(md5(Session::getSession()->getSessionHash() . $_POST['password'] . time()),0,16));

	try {
		$User->save();
    if ($invite) {
      $user_project = new UserProject();
      $user_project->setUser($User);
      $user_project->setProjectId($invite['project_id']);
      $upr = new UserProjectRole();
      $upr->setRoleId($invite['role_id']);
      $user_project->addUserProjectRole($upr);
      $user_project->save(null, true);
      deleteByHash($hash);
    }

	if (Validator::sanitizeInt($_POST['selected_avatar']) == 1) {
		$User->setAvatarFromField( Validator::sanitizeInt($_POST['avatar_id']), $_FILES);
		$User->setUseCustomAvatar(true);
		$User->save();
	} else {
		$User->setAvatarId( Validator::sanitizeInt($_POST['avatar_id']) );
		$User->setUseCustomAvatar(false);
		$User->save();
	}

	// Two Factor Auth active, user requires to confirm cell token
	/*
	if (GFConf::get('system.useTFA')) {
	    GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('PostUserRegistrationTFA'));
	}
	*/

		// If the administrator doesn't have to approve this account, send the account confirmation
		// email
		if (GFConf::get('system.approveUsers') == false) {
			$User->sendRegistrationEmail();
		}
	GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('PostUserRegistration'));
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());		// attach object errors to validation errors
		$feedback = '* '.implode('<br />* ', $errors);
		include('UserAdd.php');
	}
} else {
	$feedback = '* '.implode('<br />* ', $errors);
  $keep_feedback_tags = true;
	include('UserAdd.php');	
}

function getInviteByHash($hash) {
  $c = new Criteria();
  $c->add(SettingPeer::SECTION, 'project');
  $c->add(SettingPeer::SETTING_VALUE, $hash);
  $s = SettingPeer::doSelectOne($c);
  if (is_object($s)) {
    $arr = explode(':', $s->getSettingName());
    return array('email' => $arr[1], 'role_id' => $arr[2],
      'project_id' => $s->getProjectId());
  } else {
    return null;
  }
}

function deleteByHash($hash) {
  $c = new Criteria();
  $c->add(SettingPeer::SECTION, 'project');
  $c->add(SettingPeer::SETTING_VALUE, $hash);
  $s = SettingPeer::doSelectOne($c);
  if (is_object($s)) {
    $s->delete();
  }
}

function isCellUnique($cell)
{
    if (empty($cell)) { // cell is empty
        return false;
    }
    $c = new Criteria();
    $c->add(UserPeer::CELL, $cell);
    $u = UserPeer::doSelectOne($c);
    return (empty($u)) ? true : false;
}

?>
