<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a User and saves it to the 
// database
//
require_once('core/User.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$User = $session->getUser();

$errors = array();
$language = Language::getSessLang();

//
// TODO Validate the values passed by $_POST. Store the errors in $error like
// $errors["field_name"] = error_description
//


if (count($errors) == 0) {
	$prevCellNumber = $User->getCell();
	$User->setTimezone( Validator::sanitizeString($_POST['timezone']) );
	$User->setDateFormat( Validator::sanitizeString($_POST['date_format']) );
	$User->setAddress( Validator::sanitizeString($_POST['address']) );
	$User->setAddress2( Validator::sanitizeString($_POST['address2']) );
	$User->setPhone( Validator::sanitizeString($_POST['phone']) );
	$User->setCell( Validator::sanitizeString($_POST['cell']) );
	$User->setFax( Validator::sanitizeString($_POST['fax']) );
	$User->setTitle( Validator::sanitizeString($_POST['title']) );
	$User->setFirstname( Validator::sanitizeString($_POST['firstname']) );
	$User->setLastname( Validator::sanitizeString($_POST['lastname']) );
	$User->setCcode( Validator::sanitizeString($_POST['ccode']) );
	$User->setLanguageId( Validator::sanitizeInt($_POST['language_id']) );
	$User->setThemeId( Validator::sanitizeInt($_POST['theme_id']) );
	if (Validator::sanitizeInt($_POST['selected_avatar']) == 1) {
		//set a new avatar only if file was selected for upload
		if (!empty($_FILES['avatar']) && $_FILES['avatar']['error'] != UPLOAD_ERR_NO_FILE) {
			$User->setAvatarFromField( Validator::sanitizeInt($_POST['avatar_id']), $_FILES);
		}
	} else {
		$User->deleteCustomAvatar();
		$User->setAvatarId( Validator::sanitizeInt($_POST['avatar_id']) );
		$User->setUseCustomAvatar(false);
	}

	try {
	    $User->save();
	    $url = GFUrl::getUrl();
	    // If the language has changed, get the right Language object to get the proper feedback strings
	    if ($language->getLanguageId() != $User->getLanguageId()) {
	    	$language = LanguagePeer::retrieveByPk($User->getLanguageId());
	    }
		/*
	    if (GFConf::get('system.useTFA')) {
	    	if ($prevCellNumber != $User->getCell()) {// cell changed, send TFA
	    		$tfa = new TwoFactorAuthenticator();
	    		$tfa->sendToken($User->getCell(), 'edit');
	    		$smsUrl = $url->baseURL('UserEditCellTFA', '', 'account', array('editprofile' => 1));
	    		GFUrl::redirect($smsUrl, array('feedback' => $language->getText('User.EditCellTFAFeedback')));
	    	} else {
	    		GFUrl::redirect($url->currentBaseURL('UserBrowse'), array('feedback' => $language->getText('User.EditFeedback')));
	    	}
	    } else {
	    	GFUrl::redirect($url->currentBaseURL('UserBrowse'), array('feedback' => $language->getText('User.EditFeedback')));
	    }
		*/
		GFUrl::redirect($url->currentBaseURL('UserBrowse'), array('feedback' => $language->getText('User.EditFeedback')));
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors()); // attach object errors to validation errors
		$feedback = '* '.implode('<br />* ', $errors);
		include('UserEdit.php');
	}
} else {
	$feedback = implode('<br />', $errors);
	include('UserEdit.php');
}

?>
