<?php
require_once('tfa/TwoFactorAuthenticator.php');

$session = Session::getSession();
$user = $session->getUser();
if (empty($user)) {// not logged in?
    GFUrl::redirect(GFUrl::getUrl()->baseURL('logout', '', 'account'));
}
$Lang = $session->getLanguage();
$showTokenForm = isset($_POST['evalToken']);
$formUrl = GFUrl::getUrl()->currentBaseURL('UserEditCellTFA');
if (isset($_REQUEST['editprofile'])) {
    $showTokenForm = true;
    $tfaSessionPrefix = 'edit';
    $editProfileUrl = GFUrl::getUrl()->baseURL('UserBrowse', '', 'account');
} else {
    $tfaSessionPrefix = 'catchup';
    $editProfileUrl = ''; // unused because user is not editing his profile
}
$feedback = isset($_REQUEST['feedback']) ?
    Validator::sanitizeString($_REQUEST['feedback']) :
    $Lang->getText('Validation.TFACellRequired');
if ($_SERVER['REQUEST_METHOD'] == 'POST') {// form submit
    $tfa = new TwoFactorAuthenticator();
    if ($showTokenForm) {// validate token
        $token = Validator::sanitizeString($_POST['token']);
        try {
            $cell = $tfa->getSessionCell($tfaSessionPrefix);
            $feedback = $tfa->verifyToken($token, $tfaSessionPrefix);
            if (empty($feedback)) {// cell confirmed, save
                try {
                    $user->setCell($cell);
                    $user->save();
                    if (isset($_POST['editprofile'])) {
                        GFUrl::redirect($editProfileUrl, array('feedback' => $language->getText('User.EditCellOKFeedback')));
                    } else {
                        GFUrl::redirect($url->baseURL());
                    }
                } catch (Exception $e) {
                    // save() throws error if cell not unique (race condition)
                    $feedback = $Lang->getText('Validation.CellExists');
                    $showTokenForm = false;
                }
            }
        } catch (Exception $e) {
            $feedback = $e->getMessage();
        }
    } else { // validate cell
        $cell= isset($_POST['cell']) ? htmlspecialchars($_POST['cell']) : '';
        if (!$tfa->isValidCell($cell)) {
            $feedback = $Lang->getText('User.BadCellNumber');
        } else {// cell format valid, is it unique?
            $c = new Criteria();
            $c->add(UserPeer::CELL, $cell);
            $u = UserPeer::doSelectOne($c);
            if (!empty($u)) {// cell already in db
                $feedback = $Lang->getText('Validation.CellExists');
            } else {
                try {
                    $tfa->sendToken($cell, 'catchup');
                    $showTokenForm = true;
                    $feedback = $Lang->getText('Validation.TFATokenRequired');
                } catch (Exception $e) {
                    $feedback = $e->getMessage();
                }
            }
        }
    }
}

require_once('html/SiteTemplate.php');
$theme = new SiteTemplate();

$params = array(
    'selectedtoptab'    => 'my',
    'selectedbottomtab' => 'core',
    'title'             => $Lang->getText('User.TFA.AddCellTitle'),
    'feedback'          => $feedback
);

$theme->header($params);
?>
<p><strong><?php echo $Lang->getText('User.TFA.AddCellTitle'); ?></strong></p>
<p><?php echo $Lang->getText('User.TFA.AddCellDescription'); ?></p>
<form action="<?php echo $formUrl ?>" method="POST" autocomplete="off">
    <input type="hidden" name="posted" value="1"  />
    <table border="0">
        <tr>
            <td>
            <?php if ($showTokenForm) : ?>
                <strong><?php echo $Lang->getText('User.TFA.TokenName'); ?></strong>
                <br />
                <input type="hidden" name="evalToken" value="1" />
                <?php if (!empty($editProfileUrl)) : ?>
                    <input type="hidden" name="editprofile" value="1" />
                <?php endif; ?>
                <input type="text" name="token" maxlength="12" value=""  autocomplete="off" />
            <?php else : ?>
                <strong><?php echo $Lang->getText('User.CellName'); ?></strong>
                <br />
                <input type="text" name="cell" value="<?php echo $cell; ?>"  />
            <?php endif; ?>
            </td>
        </tr>
        <tr>
            <td>
                <?php echo HTMLWidget::button("submit","" .   $Lang->getText('UserAddForm.submit') . " ","submit","","")?>
                <?php
                    $backUrl = (!empty($editProfileUrl)) ? $editProfileUrl : $formUrl;
                    $backUrl = "document.location='{$backUrl}'";
                    if ($showTokenForm) {// show back only on token form
                        echo HTMLWidget::button("", $Lang->getText('Common.Back') . " ","btn_back","", $backUrl);
                    }
                ?>
            </td>
        </tr>
    </table>
</form>

<?php
$theme->footer($params);
?>
