<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code edits a User and saves it to the 
// database
//
require_once('core/User.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
require_once('session/LoginFactory.php');


if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$password_expired = isset($_REQUEST['password_expired']) ? validator::sanitizeBool($_REQUEST['password_expired']) : false;
$username = isset($_REQUEST['username']) ? Validator::sanitizeString($_REQUEST['username']) : '';

if (!$session->isLoggedIn() && !($password_expired == true)) {
	throw new NotLoggedInException();
}

if ($password_expired) {
	//If in password expired context, get user from DB
	$c = new Criteria();
	$c->add(UserPeer::UNIX_NAME, $username);
	$User = UserPeer::doSelectOne($c);
	if (!is_object($User)) {
		throw new PermissionDeniedException();
	}
} else {
	//If in session user password edit context, get user from session
	$User = $session->getUser();
}

$errors = array();
$language = Language::getSessLang();

if (GFConf::get('system.useSSO') && !GFConf::get('system.SSORequireStoredPassword')) {
        //some sso systems would not have an initial stored password
} elseif($User->getPasswordMd5() != md5($_POST['old_password'])) {
        $errors[] = $language->getText('MyUserPasswordEdit.ErrorOldPasswordIncorrect');
}

//new passwords must match
if ($_POST['new_password'] != $_POST['new_password_conf']) {
	$errors[] = $language->getText('MyUserPasswordEdit.PasswordsMustMatch');
} else {
	$passresult = Validator::validatePassword($_POST['new_password']);
	if (!$passresult[0]) {
		$errors = array_merge($errors,$passresult[1]);
	}
}

if (count($errors) == 0) {
	$User->setPassword( $_POST['new_password'] );		// Password must not be sanitized
	try {
		$User->setPasswordChangeDate(time());
		$User->save();
	} catch(Exception $e) {
		throw new PermissionDeniedException();
	}
	$url = GFUrl::getUrl();
	//If in password expired context, create user session, then redirect
	if (isset($_REQUEST['redirect']) && isset($_REQUEST['password_expired']) && $_REQUEST['password_expired'] == '1') {	
		$login = LoginFactory::getLogin();
		$ip = $_SERVER["REMOTE_ADDR"];
		$userSession = $login->doLogin($username, $_POST['new_password']);
		
		$session = Session::getSession();
		$session->setup($ip);
		$session->setUserSession($userSession, $ip);

		$params = array('username' => $_REQUEST['username'],
						'password' => $_REQUEST['password']);
		ActionManager::notify('USER_LOGIN', $params);

		if(isset($_REQUEST['redirect']) && !empty($_REQUEST['redirect']) && $_REQUEST['redirect']) { 
			header("Location: ".StringUtils::stripHtmlTags($_REQUEST['redirect']));
		} else {
			header("Location: /gf/my/");
		}		
		
	} else  {
		//If in password edit context, redirect
		GFUrl::redirect($url->currentBaseURL('index'), array('feedback' => $language->getText('MyUserPasswordEdit.EditFeedback')));
	}
} else {
	$feedback = implode('<br />', $errors);		
	include('UserPasswordEdit.php');
}

?>
