<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code lists Project
//
require_once('core/ProjectPeer.php');
require_once('core/Project.php');
require_once('html/HTMLWidget.php');
require_once('html/TroveRenderer.php');

$session = Session::getSession();
$Lang = $session->getLanguage();

require_once('html/templates/SiteAdminTemplate.php');
$theme = new SiteAdminTemplate();

//$Project = GFUrl::getUrl()->getProject();
//$Project = $ProjectPeer::retrieveByPk($_GET['id']);

$params = array(
	//	'project' => $Project,
	//	'SiteTemplate' => $SiteTemplate,
		'action' => isset($action) ? $action : '',
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('Project.Browse')
		);
$theme->header($params);


if (array_key_exists('project_name_search',$_REQUEST)) {
	$project_name_search = Validator::sanitizeString($_REQUEST['project_name_search']);
} else {
	$project_name_search = null;
}

$columns = array(
	//Modified per #10965
	// ProjectPeer::PROJECT_ID => $Lang->getText('Project.ProjectIdName'),
	' ',
	ProjectPeer::UNIX_NAME => $Lang->getText('Project.UnixNameName'),
	ProjectPeer::PROJECT_NAME => $Lang->getText('Project.ProjectNameName'),
	ProjectPeer::IS_PUBLIC => $Lang->getText('Project.IsPublicName'),
	ProjectPeer::STATUS => $Lang->getText('Project.StatusName'),
	ProjectPeer::IS_TEMPLATE => $Lang->getText('Project.IsTemplate'),
	$Lang->getText('ProjectRating.RatingTitle'),
	$Lang->getText('Common.Action')
);
//Legacy: Search by project name first letter:
$project_name_search = isset($_REQUEST['project_name_search']) ? Validator::sanitizeString($_REQUEST['project_name_search']) : false;

//#10965: New search fields START
//Keywords (text input, free form)
$text_search = isset($_REQUEST['text_search']) ? Validator::sanitizeString($_REQUEST['text_search']) : '';

// Trove categorization
$category_search = isset($_REQUEST['category']) && Validator::sanitizeInt($_REQUEST['category']) > 0 ? Validator::sanitizeInt($_REQUEST['category']) : -1;
ob_start();
TroveRenderer::buildTroveOptions(0, FALSE);
$trove_options = ob_get_clean();

//Status (drop down of Active, Pending, Deleted)
$status_search = isset($_REQUEST['status']) ? (int)$_REQUEST['status'] : -1;
$statusOptions = array(
	'-1'                     => '',
	Project::STATUS_ACTIVE   => Language::getSessText('Project.Active'),
	Project::STATUS_PENDING  => Language::getSessText('Project.Pending'),
	Project::STATUS_HOLDING  => Language::getSessText('Project.Holding')
);
//Is Project Template (checkbox)
$is_template_search = isset($_REQUEST['is_template']);
//Show projects with no members (checkbox)
$no_members_search = isset($_REQUEST['no_members']);
//Show projects with no activity in the past (drop-down).
$activity_search = isset($_REQUEST['no_activity']);
$time_since_search = isset($_REQUEST['no_activity_since']) ? Validator::sanitizeString($_REQUEST['no_activity_since']) : '0';
//An array with the options for the activity dropdown.
//NOTE: In case you want to add more values, be sure the array key is understand by strtotime
$noActivityOptions = array(
    '2 weeks'  => '2 '.$Lang->getText('Common.Weeks'),
    '1 month'  => '1 '.$Lang->getText('Common.Month'),
    '2 months' => '2 '.$Lang->getText('Common.Months'),
    '3 months' => '3 '.$Lang->getText('Common.Months'),
    '6 months' => '6 '.$Lang->getText('Common.Months'),
    '1 year'   => '1 '.$Lang->getText('Common.Year')
);
//#10965: New search fields END

//Prepare the criteria
$crit = new Criteria();
//Legacy: Search by first letter
if ($project_name_search) {
	$crtPNS = $crit->getNewCriterion(ProjectPeer::PROJECT_NAME, $project_name_search.'%', Criteria::ILIKE);
	$crit->add($crtPNS);
}
//Text search
if (strlen($text_search) > 0) {
	$crtTS1 = $crit->getNewCriterion(ProjectPeer::PROJECT_NAME, '%'.$text_search.'%', Criteria::ILIKE);
	$crtTS2 = $crit->getNewCriterion(ProjectPeer::UNIX_NAME, '%'.$text_search.'%', Criteria::ILIKE);
	$crtTS3 = $crit->getNewCriterion(ProjectPeer::DESCRIPTION, '%'.$text_search.'%', Criteria::ILIKE);
	$crtTS1->addOr($crtTS2);
	$crtTS1->addOr($crtTS3);
	$crit->add($crtTS1);
}

//Status search
if ($status_search > -1 && $status_search != 2) {
	$crtStatus = $crit->getNewCriterion(ProjectPeer::STATUS, (int)$status_search);
	$crit->add($crtStatus);
} elseif ($status_search == 2) {
	//TODO: Add search for removed projects. (another query?)
}
//Template search
if (!$is_template_search) {
	$crtTpl = $crit->getNewCriterion(ProjectPeer::IS_TEMPLATE, false);
	$crit->add($crtTpl);
}
//Looks like custom criteria cannot be chained, so manually it is:
$custom_query = array();
//No members search
if ($no_members_search) {
	$custom_query[] = ProjectPeer::PROJECT_ID . ' NOT IN (SELECT DISTINCT project_id FROM user_project)';
}

//Activity search
if ($activity_search) {
	$date = "'".date('Y-m-d', strtotime('now -'.$noActivityOptions[$time_since_search]))."'";
	$custom_query[] = ProjectPeer::PROJECT_ID . ' NOT IN (SELECT DISTINCT project_id FROM activity_log WHERE activity_date::date > '.$date.')';
}

// Category search
if ($category_search > -1) {
    $custom_query[] = ProjectPeer::PROJECT_ID . ' IN (SELECT DISTINCT ref_id FROM trove_link WHERE section=\'project\' AND trove_category_id='.$category_search.')';
}

//Glue the pieces together (if any) and add them to the query
if (count($custom_query) > 0) {
	$customCrit = $crit->getNewCriterion(
		ProjectPeer::PROJECT_ID,
		implode(' AND ', $custom_query),
		Criteria::CUSTOM
	);
	$crit->add($customCrit);
}

//Echo filters form
include('forms/ProjectBrowseSearch-tpl.php');
//Get data from the DB
$total = ProjectPeer::doCount($crit);
$paginator = new Paginator('prjadmin', -1, $total, ProjectPeer::PROJECT_ID, $columns);
$paginator->addPageClauses($crit);
//$crit->addAscendingOrderByColumn(ProjectPeer::PROJECT_ID);
$arr = ProjectPeer::doSelect($crit);

$theader = $paginator->getSortableTableHeaders($theme);

//echo $paginator->showLimitOptions();

/*$theader = array();

$theader[] = $Lang->getText('Project.ProjectIdName');
$theader[] = $Lang->getText('Project.ProjectNameName');
$theader[] = $Lang->getText('Project.UnixNameName');
$theader[] = $Lang->getText('Project.IsPublicName');
$theader[] = $Lang->getText('Project.StatusName');
$theader[] = $Lang->getText('Project.IsTemplate');
$theader[] = $Lang->getText('Edit');
$theader[] = $Lang->getText('Delete');*/

$buttons_array = array();
$buttons_array[] = '<a href="'. GFUrl::getUrl()->baseURL('ProjectAdd','','register') .'">'. $Lang->getText('Project.AddNew') .'</a>';
$buttons_array[] = '<a href="gotoBatchDelete()">'. $Lang->getText('SiteAdmin.Project.DeleteSelected') .'</a>';
$buttons_array[] = '<a href="'. GFUrl::getUrl()->currentBaseUrl('').'">'. $Lang->getText('Common.Back') .'</a>';
echo $theme->tableBottom($buttons_array, array(true, true, true), "", false, true, true, array(true, false, true));

echo $theme->tableTop($theader);

if (count($arr) < 1) {
	echo $theme->tableFullRow('<span class="nonefound">'.$Lang->getText('Project.NoneFound').'</span>');
} else {
	foreach ($arr as $item) {
		$trow = array();
		//Modified per #10965
		//$trow[]=$item->getProjectId();
		$trow[] = '<input type="checkbox" name="project[]" value="'.$item->getProjectId().'" onchange="getSelectedProjects()">';
		$trow[]=$item->getUnixName();
		$trow[]='<a href="/gf/project/'.$item->getUnixName().'/">'.$item->getProjectName().'</a>';
		if ($item->getIsPublic()) {
			$trow[] = Language::getSessText('Project.Public');;
		} else {
			$trow[] = Language::getSessText('Project.Private');
		}
		switch ($item->getStatus()) {
			case Project::STATUS_ACTIVE : $status = Language::getSessText('SiteAdmin.UserEdit.Active');break;
			case Project::STATUS_PENDING : $status = Language::getSessText('SiteAdmin.UserEdit.Pending');break;
			case Project::STATUS_HOLDING : $status = Language::getSessText('SiteAdmin.ProjectEdit.Holding');break;
		}	
		$trow[]=$status;
		if ($item->getIsTemplate()) {
			$trow[] = Language::getSessText('Common.Yes').' - '.'<a href="'.GFUrl::getUrl()->currentBaseURL('ExportTplAction').'&amp;id='.$item->getID() .'">'.Language::getSessText('ProjectTemplate.LegacyExportText').'</a>';
			$deletelink = '';
		} else {
			$trow[] = Language::getSessText('Common.No');
			$deletelink = '<a href="'. GFUrl::getUrl()->currentBaseUrl('ProjectDelete').'&amp;id='.$item->getID() .'">'. $Lang->getText('Delete') .'</a>';
		}
		$prs = ActionManager::notify('PROJECT_LISTING_COLS', array('project' => $item));
		foreach ($prs as $pr) {
			$cols = $pr->getResult();
			foreach ($cols as $content) {
				$trow[] = $content;
			}
		}
		if ($item->getStatus()==Project::STATUS_PENDING) {
			$trow[]='&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' . $deletelink;
		} else {
			$trow[]='<a href="'. GFUrl::getUrl()->currentBaseUrl('ProjectEdit').'&amp;id='.$item->getID() .'">'. $Lang->getText('Edit') .'</a>&nbsp;' . $deletelink;
		}
		echo $theme->tableRow($trow, array(), false);
	}
}
//echo $theme->listPaginatorRow($paginator);

echo $theme->tableBottom($buttons_array, array(true, true, true), "", false, true, true, array(true, false, true));
if(isset($paginator) && is_object($paginator)) {
	echo '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
}

//Simple js to call the batch delete action
?>
<script>
function getSelectedProjects() {
	var inputs = document.getElementsByTagName("input"); //or document.forms[0].elements;
	var checked = []; //will contain all checked checkboxes
	for (var i = 0; i < inputs.length; i++) {
	  if (inputs[i].type == "checkbox") {
	    if (inputs[i].checked && inputs[i].value != "on") {
	      checked.push(inputs[i].value);
	    }
	  }
	}
	return checked;
	//console.log(checked);
}
function gotoBatchDelete() {
	var ids = getSelectedProjects();
	if (ids.length == 0) {
		alert("<?php echo $Lang->getText('Report.ErrorNoProjectsSelected') ?>");
		return;
	}
	query = ids.join('-');
	location.href = "<?php echo GFUrl::getUrl()->currentBaseUrl('ProjectDeleteBatch').'&ids=' ?>" + query;
}
function toggleDropdown(checkbox) {
	var is_disabled = !checkbox.checked;
	document.getElementById('no_activity_dropdown').disabled=is_disabled;
    $('select[name="no_activity_since"]').trigger("chosen:updated");
}

$(function(){
    var option_any_text = '<?php echo Language::getSessText('Options.Any') ?>',
        category_select = $('select[name="category"]');
    // Chosen select init
    $('select[name="status"]').attr('data-placeholder', option_any_text).chosen({
        disable_search_threshold: 10,
        allow_single_deselect: true
    });
    $('option[value="<?php echo $category_search ?>"]', category_select).attr('selected', 'selected');
    category_select.attr('data-placeholder', option_any_text).chosen({
        disable_search_threshold: 10,
        allow_single_deselect: true
    });
    $('select[name="no_activity_since"]').chosen({disable_search_threshold: 10});
});
</script>
<?php
$theme->footer($params);

?>
