<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

//
// This code creates a new Project and saves it to the 
// database
//
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('core/Project.php');
require_once('exceptions/NotLoggedInException.php');
require_once('lib/GFExportImport.php');

$session = Session::getSession();
$Lang = $session->getLanguage();
if (!$session->isLoggedIn()) {
	throw new NotLoggedInException();
}

if ($_SERVER['REQUEST_METHOD'] != 'POST' || !$session->isSiteAdmin()) {
	throw new PermissionDeniedException();
}

if (!isset($_FILES['template']['tmp_name']) || $_FILES['template']['tmp_name'] == '') {
	$feedback = '* '.$Lang->getText('ProjectTemplate.NoFileUploaded');
	include('ProjectImport.php');
	exit();
}
if (!isset($_REQUEST['confirm']) || !Validator::sanitizeBool($_REQUEST['confirm'])) {
	$feedback = $Lang->getText('ProjectTemplate.DidntConfirm');
	include('ProjectImport.php');
	exit();
}

if(!isset($_REQUEST['importProjectName'])) {
	include('ProjectImportTpl.php');
	exit();
}

$_REQUEST['importProjectName'] = Validator::sanitizeString($_REQUEST['importProjectName']);
$_FILES['template']['name'] = Validator::sanitizeString($_FILES['template']['name']);

// Save Template to a temporary file
$temp_file = '/tmp/template_user-' . $session->getUser()->getUserId() . '-' . basename($_FILES['template']['name']);
$temp_dir = '/tmp/template_user_dir-' . $session->getUser()->getUserId() . '-' . basename($_FILES['template']['name']);

if (!move_uploaded_file($_FILES['template']['tmp_name'], $temp_file)) {
	$feedback = '* '.$Lang->getText('ProjectTemplate.UploadError');
	include('ProjectImportTpl.php');
	exit();
}

if (isset($_REQUEST['importMode'])) {
	$importMode = Validator::sanitizeString($_REQUEST['importMode']);
}
else {
	$importMode='standard';
}


// Unzip, untar the temp file
$output = array();
if(file_exists($temp_dir)) {
	SystemUtils::execute("rm -rf \"$temp_dir\"", $output);
}
mkdir($temp_dir);
SystemUtils::execute("tar -xjf \"$temp_file\" -C \"$temp_dir\"", $output);

//Inside temp_dir there should be a project name, then a date_string dir. We won't check this, its checked downstream, 
//We just go into the first two subdirs.
$directories = scandir($temp_dir);
foreach($directories as $dir) {
	if($dir != '.' && $dir != '..' && is_dir(SystemUtils::buildFilePath($temp_dir, $dir))) {
		$temp_dir = SystemUtils::buildFilePath($temp_dir, $dir);
		break;
	}
}

$directories = scandir($temp_dir);
foreach($directories as $dir) {
	if($dir != '.' && $dir != '..' && is_dir(SystemUtils::buildFilePath($temp_dir, $dir))) {
		$temp_dir = SystemUtils::buildFilePath($temp_dir, $dir);
		break;
	}
}

try {
	$ei = new GFExportImport();
	$ei->import($temp_dir, false, false, "PROJECT_TPLIMPORT", $_REQUEST['importProjectName'], 'lib/systemSpecDataTpl.xml');

	/*
	switch($importMode) {
		case 'overwrite':
			$ei->import($temp_dir, true, false, "PROJECT_TPLIMPORT");
			break;
		case 'merge';
			$ei->import($temp_dir, false, true, "PROJECT_TPLIMPORT");
			break;
		default:
			$ei->import($temp_dir, false, false, "PROJECT_TPLIMPORT");
			break;
	}
	*/

	// Delete the temporary file
	@unlink($temp_file);
	SystemUtils::execute("rm -rf \"$temp_dir\"", $output);

	$feedback = '* '.$Lang->getText('ProjectTemplate.SuccessfulImport');

	include('ProjectImportTpl.php');
} catch (Exception $ex) {
	$feedback = $ex->getMessage();
	include('ProjectImportTpl.php');
	exit();
}

GFDaemon::notify('DAEMON_PROJECT_APPROVE', $_REQUEST['importProjectName']);
?>
