<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version $Id$
 * @author Daniel A. P�ez	daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

/**
 * performAction() - Updates the indicated user status
 *
 * @param				string  $newStatus - the new user status
 * @param				string  $statusString - the status string to display
 * @param				string  $user_id - the user id to act upon
 * @return 			string operation message
 */


require_once('core/User.php');
require_once('core/UserProjectPeer.php');
require_once('Validator.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');
if (!Session::getSession()->isSiteAdmin()) {
	throw new PermissionDeniedException();
}

function performAction($newStatus, $statusString, $user_id) {
	$user = UserPeer::retrieveByPk($user_id);
	if (!is_object($user)) {
		return Language::getSessText('LostPass.NotExists');
	}
	
	if($newStatus==User::STATUS_DELETED) {
		$user->delete();
		$user->save();
		$user->setSiteMonitoring(false);
	} elseif ($newStatus==User::STATUS_ACTIVE) {
		$user->activate();
		$user->setSiteMonitoring(true);
	} else {
		$user->setStatus($newStatus);
		$user->save();
	}
	return "<h2>" .Language::getSessText('SiteAdmin.UserList.UserUpdated',array($statusString))."</h2>";
}

if (array_key_exists('user_id',$_REQUEST)) {
	$user_id = Validator::sanitizeInt($_REQUEST['user_id']);
} else {
	$user_id = null;
}

$User = UserPeer::retrieveByPk($user_id);

if (!is_object($User) ) {
	throw new PermissionDeniedException();
}

if (array_key_exists('_action',$_REQUEST)) {
	$action = Validator::sanitizeString($_REQUEST['_action']);
} else {
	$action = null;
}

if (array_key_exists('return_to',$_REQUEST)) {
	$return_to = $_REQUEST['return_to'];
} else {
	$return_to = 'UserEdit';
}

if (array_key_exists('sure',$_REQUEST)) {
	$sure = Validator::sanitizeBool($_REQUEST['sure']);
} else {
	$sure = null;
}

// quick status changers in user browse
if ($action) {
	switch ($action) {
		case 'delete' : {
			if ($sure) {
				$feedback = performAction(User::STATUS_DELETED, "DELETED", $user_id);
			} else {
				$feedback = Language::getSessText('SiteAdmin.UserEdit.NoChanges');
			}
			break;
		}
		case 'activate' : {
			$feedback = performAction(User::STATUS_ACTIVE, "ACTIVE", $user_id);
			break;
		}
		case 'suspend' : {
			$feedback = performAction(User::STATUS_SUSPENDED, "SUSPENDED", $user_id);
			break;
		}
	}
} else {
	// normal update procedure
	$errors = array();
	$Lang = Language::getSessLang();
	
	if (!Validator::validateEmail($_POST['email'])) {
		$errors['email'] = $Lang->getText('User.InvalidEmail');
	}

	if (count($errors) == 0) {
		$User->setEmail( Validator::sanitizeString($_POST['email']) );
    if (GFConf::varExists('system.enableUdfEdit') 
      && (GFConf::get('system.enableUdfEdit') == 'users' || GFConf::get('system.enableUdfEdit') == 'on')) {
      $User->setUserText1(Validator::sanitizeString($_POST['udf1']));
      $User->setUserText2(Validator::sanitizeString($_POST['udf2']));
      $User->setUserText3(Validator::sanitizeString($_POST['udf3']));
      $User->setUserText4(Validator::sanitizeString($_POST['udf4']));
      $User->setUserText5(Validator::sanitizeString($_POST['udf5']));
      $User->setUserText6(Validator::sanitizeString($_POST['udf6']));
    }
    if (!is_null($_POST['unlock'])) {
      $User->setLoginAttempts(-1);
      $User->sendUnlockEmail();
      $feedback = Language::getSessText('SiteAdmin.UserUnlocked')."<br />";
    }

		$User->save();
		$feedback .= 'Changes Saved';
	} else {
		$feedback = implode('<br />', $errors);
		include('UserEdit.php');
		exit;
	}	
}

GFUrl::redirect(GFUrl::getUrl()->baseURL($return_to,'','admin',array('feedback'=>$feedback,'user_id'=>$user_id)));


?>
