<?php

header('Access-Control-Allow-Origin: *', true);

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('core/UserPreference.php');
require_once('plugins/forum/lib/ForumPlugin.php');
require_once('plugins/forum/lib/ForumThreadPeer.php');
require_once('plugins/forum/lib/ForumPeer.php');
require_once('lib/ChatManager.php');

$session = Session::getSession();

$forum = null;
$thread = null;

if ($session == null || $session->getUser() == null) {
    header("HTTP/1.0 401 Authentication Required", true, '401');
    return;
}

$name = null;
if (array_key_exists('name', $_GET)) {
    $name = Validator::sanitizeString($_GET['name']);
}

// Look at the thread id, use if set, 404 if invalid
//error_log('ChatAdd enter');
if (isset($_GET['thread_id'])) {

    $forum = ChatManager::getForumByThreadId($_GET['thread_id']);
    // Go get the discussion thread
    $thread = ForumThreadPeer::retrieveByPk($_GET['thread_id']);

// Then section and refid to find/create a forum
} elseif (isset($_GET['section']) && isset($_GET['refid'])) {
    $forum = ChatManager::getForumByRefIdSection($_GET['section'], $_GET['refid'], $name);

// Then check forum id, 404 if invalid forum
} elseif (isset($_GET['forum_id'])) {

    $forum = ChatManager::getForumById($_GET['forum_id']);
}

// Did we get a forum?
if (is_null($forum)) {
    error_log("ChatAdd, cannot find requested forum, returning 404");
    header("HTTP/1.0 404 Not Found", true, '404');
    return;

// Is the forum connected to a valid project,
// and does that project have forum plugin active?
} elseif (is_null($forum->getProject()) ||
        !$forum->getProject()->getPluginByName(ForumPlugin::SECTION)) {
    error_log("ChatAdd, forum project does not exist, or forum plugin is disabled for project, returning 404");
    header("HTTP/1.0 404 Not Found", true, '404');
    return;

// Does the forum's related item still exist?
} elseif (!ForumPeer::relatedItemExists($forum)) {
    error_log("ChatAdd, related item for {$forum->getSection()} {$forum->getRefId()} does not exist, returning 404");
    header("HTTP/1.0 404 Not Found", true, '404');
    return;

// Can this user read the forum?
} elseif (!$forum->userCanRead()) {
    error_log("ChatAdd, user cannot read forum {$forum->getForumName()}");
    header("HTTP/1.0 403 Not Allowed", true, '403');
    return;
}
//error_log("ChatAdd, using forum {$forum->getForumName()}");
// Go get the discussion thread, if we don't already have it
if ($thread == null) {
    $thread = ForumThreadPeer::getNamedThread($forum, $session->getUser());
}
//error_log("ChatAdd, using thread {$thread->getThreadName()}");
// Did we find the thread?
if (is_null($thread)) {
    error_log("ChatAdd, cannot find requested thread for section/ref $section/$refid, returning 404");
    header("HTTP/1.0 404 Not Found", true, '404');
    return;
}

//Add user to chat windows
echo ChatManager::getChatWindow($forum, $thread);


