
<?php

header('Access-Control-Allow-Origin: *', true);

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('core/UserPreferencePeer.php');
require_once('core/UserPeer.php');
require_once('core/AuditTrailPeer.php');
require_once('core/ActivityLogPeer.php');
require_once('GFUrl.php');
require_once('plugins/forum/lib/ForumThreadPeer.php');
require_once('plugins/forum/lib/ForumPeer.php');
require_once('plugins/forum/lib/ForumMessagePeer.php');
require_once('lib/core/WallPostPeer.php');
require_once('lib/LinkRenderer.php');
require_once('lib/ChatManager.php');

$CHAT_ROOM_PREFIX = 'chat-room-';

$session = Session::getSession();
session_write_close();

if ($session == null || $session->getUser() == null)
{
	header("HTTP/1.0 401 Authentication Required", true, '401');
	return;
}

$Lang = $session->getLanguage();

// Create a new chat message
// Shortcut the stuff below, since we
// no longer return message structure
// as the response for submitting a msg.
if (count($_POST) > 0)
{
	SubmitMessage($session, $CHAT_ROOM_PREFIX);
	return;
}

$prefs = GetUserChatRooms($session->getUser()->getUserId(), $CHAT_ROOM_PREFIX);

$poll_threads = MapRequestToPollList(
	PrefsToPollList($prefs, $CHAT_ROOM_PREFIX),
	$_GET, $CHAT_ROOM_PREFIX);

// If this is a regular poll, we will try up to x times
// to get new messages for any of the requested threads.
$max_poll_count = 25;
$repeat_count = 0;
$keep_polling = true;
$start_timestamp = time();
$deleted_messages = array();

$wallPrefName = 'toaster-since';
$last_wall_msg = UserPreferencePeer::getUserPreferenceValue($session->getUser()->getUserId(), $wallPrefName, 0);
$wall_msgs = array();

while ($keep_polling)
{
	$repeat_count++;

	// Check once per big loop for wall postings
	$wall_posts = WallPostPeer::GetNotificationsSince($session->getUser()->getUserId(), $last_wall_msg);
	if (count($wall_posts) > 0) {
		//error_log("User {$session->getUser()->getUnixName()} received ".count($wall_posts).", adding to response data");
		foreach ($wall_posts as $wall_post) {
			$wall_msgs[] = WallPostToJson($wall_post);
			if ($wall_post->getPrimaryKey() > $last_wall_msg) {
				$last_wall_msg = $wall_post->getPrimaryKey();
				//error_log("Updated last wall message preference for {$session->getUser()->getUnixName()} to {$wall_post->getPrimaryKey()}");
			}
		}
		UserPreferencePeer::setUserPref($session->getUser()->getUserId(), $wallPrefName, $last_wall_msg);
		$keep_polling = false;
	}

	// If we get a new user preference, that's an event
	$prefs = GetUserChatRooms($session->getUser()->getUserId(), $CHAT_ROOM_PREFIX);
	$new_threads = PrefsToPollList($prefs, $CHAT_ROOM_PREFIX);
	foreach ($new_threads as $thread_id => $thread_info)
	{

		//If we had a new thread show up, quit at the end of this loop
		if (!array_key_exists("$thread_id", $poll_threads))
		{
			//error_log("User {$session->getUser()->getUnixName()} joined thread {$thread_id}, adding to list");
			$poll_threads["$thread_id"] = $thread_info;
			$keep_polling = false;
		}
	}

	// Do the actual message gathering
	foreach ($poll_threads as $thread_info)
	{
		$thread_id = $thread_info['thread_id'];
		$room_key = $CHAT_ROOM_PREFIX . $thread_id;

		// If this is the first time through for this thread,
		// retrieve the DB object and check permissions
		if (!array_key_exists('thread', $thread_info))
		{
			$thread = ForumThreadPeer::retrieveByPk($thread_id);
			// Not a valid ID? ignore from now on
			if (is_null($thread))
			{
				error_log("User {$session->getUser()->getUnixName()} requested thread {$thread_id}, does not exist");
				UserPreferencePeer::deleteDiscussionPreferences($thread_id);
				unset($poll_threads["$thread_id"]);
				$keep_polling = false;
				continue;
			} else
			{
				// Check access for reading requested thread
				$forum = $thread->getForum();
				if (!$forum->userCanRead())
				{
					error_log("User {$session->getUser()->getUnixName()} requested thread {$thread_id}, no read access");
				} else
				{
					$thread_info['thread'] = $thread;
					$poll_threads["$thread_id"] = $thread_info;
				}
			}
		} else
		{
			$thread = $thread_info['thread'];
		}

		if (array_key_exists('include_activity', $thread_info) &&
			$thread_info['include_activity'] == 'Y')
		{
			if (!array_key_exists('project_id', $thread_info))
			{
				//error_log("Setting up project activity for $thread_id");
				$proj = ForumPeer::getRelatedProject($thread->getForum());
				if ($proj != null)
				{
					//error_log("Project Activity for $thread_id, related project id is {$proj->getProjectId()}");
					$thread_info['project_id'] = $proj->getProjectId();
				} else
				{
					error_log("User {$session->getUser()->getUnixName()} requested project activity for thread {$thread_id}, could not find related project");
					$thread_info['include_activity'] = 'N';
				}
			}
		}

		// Only check prune list
		// once per thread per request
		if (!array_key_exists('users', $thread_info))
		{
			// Manage user presence
			UserPreferencePeer::setUserPref($session->getUser()->getUserId(), $room_key, 'Y');
			CheckPruneUserList($room_key, $CHAT_ROOM_PREFIX);
		}


		// GATHER USER LIST FOR THIS THREAD
		$users = GetUserList($room_key, $session->getUser());
		// If the user list changes, that's an event
		if (array_key_exists('users', $thread_info) &&
			count($thread_info['users']) != count($users))
		{
			//error_log("user list change triggering poll exit");
			$keep_polling = false;
		}
		$thread_info['users'] = $users;


		// GATHER NEW MESSAGES FOR THIS THREAD
		$messages = ForumMessagePeer::getMessagesByThreadSince($thread, $thread_info['msg_since']);
		if (count($messages) > 0)
		{
			//error_log("new messages triggering poll exit");
			$keep_polling = false;
		}
		if ($thread_info['msg_since'] == -1)
		{
			//error_log("initial poll triggering poll exit");
			$keep_polling = false;
		}
		$thread_info['messages'] = $messages;

		// RELATED PROJECT ACTIVITY FOR THIS THREAD, IF ENABLED
		if (array_key_exists('include_activity', $thread_info) &&
			$thread_info['include_activity'] == 'Y' &&
			array_key_exists('project_id', $thread_info)) {
			$project_id = $thread_info['project_id'];
			$since_id = $thread_info['activity_since_id'];
			//error_log("Checking project activity for $project_id");
			$activity_temp = ActivityLogPeer::getProjectActivitySince($project_id, $since_id);
			$activity = array();
			foreach ($activity_temp as $at) {
				if ($at->getSection() != 'forumthread') {
					// #9945
					// Use a key in the array, to prevent duplicate messages
					$activity_key = "activity-{$at->getPrimaryKey()}";
					$activity[$activity_key] = $at;
				}
			}
			if (count($activity) > 0) {
				//error_log("new activity triggering poll exit");
				$keep_polling = false;

				echo "\n"; //<-- send this to the client
				if (connection_status() != 0)
				{
					error_log("Client went away in chat poll");
					return;
				}
			}
			$thread_info['activity'] = $activity;
		}


		// GATHER RELATED THREADS FOR THIS THREAD
		$related_threads = GetRelatedThreads($thread, $poll_threads);
		// If the related thread list changes, that's an event
		if (array_key_exists('related_threads', $thread_info) &&
			count($thread_info['related_threads']) != count($related_threads))
		{
			$keep_polling = false;
		}
		$thread_info['related_threads'] = $related_threads;


		// Update the master array
		$poll_threads[$thread_id] = $thread_info;

		// If a tab is just being set up on the client, return after first poll
		if ($thread_info['msg_since'] == -1)
		{
			//error_log("since triggering poll exit");
			$keep_polling = false;
		}
	}  // foreach thread
	// GATHER DELETED MESSAGES FOR THIS THREAD - ACROSS ANY OPEN CHATS
	$newly_deleted = ForumMessagePeer::getDeletedMessagesSince($start_timestamp);
	if (count($newly_deleted) != count($deleted_messages))
	{
		//error_log("ChatMessage, found a deleted message!");
		$keep_polling = false;
	}
	$deleted_messages = $newly_deleted;

	if ($repeat_count > $max_poll_count)
	{
		//error_log("max poll count triggering poll exit");
		$keep_polling = false;
	}

	if ($keep_polling)
	{
		time_sleep_until(microtime(true) + 1);  // sleep for 1 second
		echo "\n"; //<-- send this to the client
		if (connection_status() != 0)
		{
			//error_log("Client went away in chat poll");
			return;
		}
	}
}

// Build the initial/current list of threads this user is chatting on
$thread_data_list = PrefsToThreadList($prefs, $CHAT_ROOM_PREFIX);

// Now go through and add messages to each thread
foreach ($poll_threads as $thread_info)
{
	if (!is_array($thread_info))
		continue;

	$thread_id = $thread_info['thread_id'];
	$thread = $thread_info['thread'];
	$room_key = $CHAT_ROOM_PREFIX . $thread_id;

	if ($thread == null)
		continue;

	if (!array_key_exists($thread_id, $thread_data_list))
	{
		$temp_arr = ThreadToThreadData($thread);
		if ($temp_arr == null)
			continue;

		$thread_data_list[$thread_id] = $temp_arr;
	}

	$thread_data = $thread_data_list[$thread_id];

	$messages = array();
	$first_msg_id = 0;
	$last_msg_id = 0;
	foreach ($thread_info['messages'] as $message)
	{
		$messages[] = array(
			'sort' => strtotime($message->getPostDate()),
			'message_id' => $message->getForumMessageId(),
			'message_type' => 'message',
			'date' => $message->getPostDate('Y/m/d H:i:s'),
			'user' => UserToJson($message->getUser()),
			'subject' => $message->getSubject(),
			'body' => StringUtils::toValidUTF8(html_entity_decode($message->getBody()))
		);
		// We only take first and last from actual thread postings, not from activity (below).
		if ($first_msg_id == 0)
			$first_msg_id = $message->getForumMessageId();
		$last_msg_id = $message->getForumMessageId();
	}

	if (array_key_exists('activity', $thread_info))
	{
		$activity = $thread_info['activity'];
		if (count($activity) > 0)
		{
			//error_log("Persisting activity records to message list for thread $thread_id");
			foreach ($activity as $al)
			{
				$dt = strtotime($al->getActivityDate());
				$body = GetActivityDescription($al);
				//error_log("Adding activity record to message list for thread $thread_id: $body");
				$messages[] = array(
					'sort' => $dt,
					'message_id' => $al->getPrimaryKey(),
					'message_type' => 'project',
					'date' => date('Y/m/d H:i:s', $dt),
					'user' => UserToJson($al->getUser()),
					'subject' => '',
					'body' => $body
				);
			}
		}
		// We're combining two separate data sources with no common key.
		// So we have to sort the combined list by date/time stamp.
		usort($messages, 'SortByCustomKey');
	}

	$thread_data['first'] = $first_msg_id;
	$thread_data['last'] = $last_msg_id;
	$thread_data['users'] = UsersToJson($thread_info['users']);
	$thread_data['messages'] = $messages;
	$thread_data['related'] = $thread_info['related_threads'];

	$thread_data_list["$thread_id"] = $thread_data;
}

//error_log("Message Loop, preparing response that includes ".count($wall_msgs)." wall messages");

$result = array(
	'threads' => array_values($thread_data_list),
	'wall' => array_values($wall_msgs),
	'deleted' => $deleted_messages,
	'error' => ''
);
echo json_encode($result);

function GetUserList($room_key, $sessionUser)
{

	$prefs = UserPreferencePeer::getUserPrefsByPrefix($room_key);

	$ret = array();
	foreach ($prefs as $pref)
	{
		if ($pref->getPreferenceValue() == 'Y')
		{
			$ret[] = $pref->getUser();
		}
	}

	return $ret;
}

function GetRelatedThreads($thread)
{

	$ret = array();

	$threads = array_merge($thread->getForum()->getForumThreads(), ForumThreadPeer::GetMentionedItemDiscussions($thread));
	foreach ($threads as $candidate)
	{
		if ($candidate->getForumThreadId() != $thread->getForumThreadId())
		{
			$newItem = ThreadToThreadData($candidate);
			if ($newItem != null)
			{
				// Use a key to ensure unique results, no duplicate threads!
				$ret["{$candidate->getForumThreadId()}"] = $newItem;
			}
		}
	}
	return array_values($ret);
}

function CheckPruneUserList($room, $prefix)
{
	$before = date('c', strtotime('-75 seconds'));
	$user_prefs = UserPreferencePeer::updateUserPrefsBefore($room, $before, 'N');

	foreach ($user_prefs as $user_pref)
	{
		$str_prefname = $user_pref->getPreferenceName();
		if (strpos($str_prefname, $prefix, 0) === 0)
		{
			//Persist the event to the DB (create a message)
			$arr_pref = explode('_', $str_prefname);
			$thread_id = isset($arr_pref[1]) ? $arr_pref[1] : 0;
			$thread = ForumThreadPeer::retrieveByPk($thread_id);
			if (is_object($thread))
			{
				ForumPeer::logChatEvent($thread, ForumPeer::CHAT_EVENT_USER_LEAVE, $user_pref->getUser());
			}
		}
	}
}

function UserToJson($user)
{
	if ($user->getUserId() == User::UNDEFINED_USER_ID)
	{
		$unix_name = '';
	} else
	{
		$unix_name = $user->getUnixName();
	}
	return array(
		'user_id' => $user->getUserId(),
		'unix_name' => $unix_name,
		'full_name' => $user->getFullName(),
		'email' => $user->getEmail(),
		'avatar' => $user->getAvatarUrl()
	);
}

function UsersToJson($users)
{
	$ret = array();
	foreach ($users as $user)
	{
		$ret[] = UserToJson($user);
	}
	return $ret;
}

function PrefsToThreadList($prefs, $prefix)
{
	$ret = array();

	foreach ($prefs as $pref)
	{
		$thread_id = str_ireplace($prefix, '', $pref->getPreferenceName());
		$thread = ForumThreadPeer::retrieveByPk($thread_id);
		if ($thread != null)
		{
			if (ForumPeer::relatedItemExists($thread))
			{
				$ret[$thread_id] = ThreadToThreadData($thread);
			} else
			{
				error_log("Deleting pref record for chat for non-existent item {$pref->getPreferenceName()}");
				$pref->delete();
			}
		}
	}
	return $ret;
}

function PrefsToPollList($prefs, $prefix)
{
	$ret = array();
	foreach ($prefs as $pref)
	{
		$thread_id = str_ireplace($prefix, '', $pref->getPreferenceName());
		$ret["$thread_id"] = array(
			'thread_id' => $thread_id,
			'pref_key' => $pref->getPreferenceName(),
			'include_activity' => UserPreferencePeer::getUserPreferenceValue($pref->getUserId(), "chat-{$thread_id}-proj", 'N'),
			'since' => -1
		);
	}
	return $ret;
}

function ThreadToThreadData($thread)
{
	try
	{
		$arr_title = ChatManager::getItemTitle($thread);
		if (is_array($arr_title) && count($arr_title) > 0)
		{
			$title = $arr_title['title'];
			$title_full = $arr_title['title_full'];
     
			return array(
				'thread_id' => $thread->getForumThreadId(),
				'tab_url' => "/gf/chat/?action=ChatTab&thread_id={$thread->getForumThreadId()}",
				'base_url' => "/gf/chat/",
				'title' => "$title",
				'title_full' => "$title_full",
			);
		}
	} catch (Exception $e)
	{
		error_log("ThreadToThreadData: {$e->getMessage()}");
	}
	return null;
}


function MapRequestToPollList($threads, $params, $prefix)
{
	foreach ($params as $key => $value)
	{
		if (is_numeric($key))
		{
			$ids = explode(',', $value);

			$msg_since = 0;
			$activity_since = 0;
			switch (count($ids))
			{
				case 2:
					$activity_since = $ids[1];
				case 1:
					$msg_since = $ids[0];
					break;
			}
			if ($msg_since > 0)
			{
				if (!array_key_exists("$key", $threads))
				{
					$thread_info = array(
						'thread_id' => $key,
						'pref_key' => $prefix . $key
					);
				} else
				{
					$thread_info = $threads["$key"];
				}
				$thread_info['msg_since'] = $msg_since;
				$thread_info['activity_since'] = $activity_since;
				$threads["$key"] = $thread_info;
			}
		}
	}

	return $threads;
}

function SubmitMessage($session, $prefix)
{
	//Define an html purifier config that supports custom properties and extra tags used in chat
	$config = HTMLPurifier_Config::createDefault();
	$config->set('HTML.Doctype', 'XHTML 1.0 Transitional');
	$config->set('Cache.SerializerPath', GFConf::get('system.varDir') . '/caches/Serializer');
	$config->set('HTML.Allowed', GFConf::get('system.basicHTMLWhiteList') . ',span[class|data-sound-path]');
	$config->set('AutoFormat.Linkify', true);
	$def = $config->getHTMLDefinition(true);
	$def->addAttribute('span', 'data-sound-path', 'Text');

	$stringUtils = new StringUtils(StringUtils::BASIC_HTML, $config);

	$thread_id = Validator::sanitizeInt($_REQUEST['thread_id']);
	$room_key = $prefix . $thread_id;
	$thread = ForumThreadPeer::retrieveByPk($thread_id);
	if (is_null($thread))
	{
		header("HTTP/1.0 404 Not Found", true, '404');
		return;
	}
	$forum = $thread->getForum();
	if (!$forum->userCanPost() ||
		$forum->getModerationLevel() > Forum::NO_MODERATION)
	{
		header("HTTP/1.0 403 Not Allowed", true, '403');
		return;
	}

	$body = '';
	if (array_key_exists('body', $_POST))
	{
		$body = $_POST['body'];
		//error_log("A body was submitted: $body");
	}
	// If the msg is multiple lines long,
	// then we wrap in pre tags and present it as code (not markup).
	if (substr_count($body, "\n") > 1 ||
		substr_count($body, "<br") > 1)
	{
		//error_log("Body has multiple lines, encoding and wrapping in pre tag");
		$body = preg_replace('/<br[\w]*[\/]?>/i', "\n", $body);
		$body = '<pre>' . htmlspecialchars(htmlspecialchars($body)) . '</pre>';
	} else
	{
		//error_log("Body does not have multiple lines, scrubbing HTML and checking for links to render");
		// If not multiple lines, then be smart about what was submitted (links, GForge items, etc.)
		$body = $stringUtils->cleanHtml($body);
		$body = LinkRenderer::renderHtmlLinks($body);
	}
	$body = trim($body);
	//error_log("Final message to save: $body");

	$message = new ForumMessage();
	$message->setBody($body);
	$message->setCreatedBy($session->getUser()->getUserId());
	$message->setUser($session->getUser());
	$message->setForumThread($thread);
	$message->setPostDate(time());
	$message->setSubject('Chat message');
	$message->setPostDate(time());

	$thread->addForumMessage($message);
	$thread->save();

	$params = array('body' => $_POST['body'], 'message_id' => $message->getForumMessageId());
	ActionManager::notify('CHAT_MESSAGE_SAVE', $params);

	// Set up info for user wall mentions, if any
	$to_users = StringUtils::getMentionedUsers($message->getBody());

	if (count($to_users) > 0)
	{
		$user = $session->getUser();
		$userUrl = GFUrl::getUrl()->userURL($user);
		$userlink = "<a href=\"{$userUrl}\" target=\"_new\" title=\"User profile for {$user->getUnixName()}\">{$user->getUnixName()}</a>";
		$link = "<a href=\"javascript:;\" class=\"discuss\" data-section=\"{$forum->getSection()}\" data-thread-id=\"{$thread_id}\" data-ref-id=\"{$thread_id}\" data-forum-name=\"{$thread->getForum()->getForumName()}\">{$thread->getForum()->getForumName()}/{$thread->getThreadName()}</a>";
		$wall_comment = "$userlink mentioned you in $link: <br /><br />{$message->getBody()}";
		ForumPeer::sendUserWallMentions($to_users, $session->getUser()->getUserId(), $wall_comment, $room_key, false);
	}
}

function SortByCustomKey($arr_a, $arr_b)
{
	$a = $arr_a['sort'];
	$b = $arr_b['sort'];

	if ($a == $b)
		return 0;
	if ($a > $b)
		return 1;
	return -1;
}

function GetActivityDescription($al)
{
	$ret = $al->getDescription();
	$uname = ($al->getUser()) ? $al->getUser()->getUnixName() . ' ' : 'nobody';
	if (!strncmp($ret, $uname, strlen($uname)))
	{
		$ret = str_replace($uname, 'I ', $ret);
	}
	$url = $al->getUrl();
	if ($url != null && strlen($url) > 0)
	{
		if (stripos($url, '://') < 1)
		{
			$url = GFUrl::getUrl()->getFullURL($url);
		}
		$ret .= " (<a href=\"$url\" target=\"_new\" title=\"More details\">link</a>)";
	}
	return $ret;
}

function GetUserChatRooms($user_id, $prefix)
{
	$pref_temps = UserPreferencePeer::getUserPrefsByUserAndPrefix($user_id, $prefix);
	$ret = array();

	foreach ($pref_temps as $pref)
	{
		if ($pref->getPreferenceValue() == 'Y')
		{
			$ret[] = $pref;
		}
	}
	return $ret;
}

function WallPostToJson(WallPost $notice, $browse_url = '') {

	$maxLength = 6000;

	$comment = $notice->getComment();

	if (strlen($comment) > $maxLength) {
		$comment = substr($comment, 0, $maxLength) . "..."; //."&nbsp;&nbsp;<a href='$browse_url' title='Read more' target='_new'>read more</a>";
	}

	return array(
		'id' => $notice->getWallPostId(),
		'message_id' => '',
		'message_type' => 'wall',
		'body' => $comment,
		'user' => UserToJson($notice->getUser()),
		'date' => $notice->getPostDate('Y/m/d H:i:s')
	);
}

