<?php

header('Access-Control-Allow-Origin: *', true);

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('core/UserPreference.php');
require_once('plugins/forum/lib/ForumThreadPeer.php');
require_once('plugins/forum/lib/ForumPeer.php');
require_once('html/HTMLWidget.php');
require_once('html/SiteTemplate.php');
require_once('GFUrl.php');
require_once('lib/ChatManager.php');

$session = Session::getSession();

if ($session == null || $session->getUser() == null) {
    header("HTTP/1.0 403 Not Allowed", true, '403');
    return;
}

$user = $session->getUser();
$Lang = Session::getSession()->getLanguage();

$CHAT_ROOM_PREFIX = 'chat-room-';

$prefs = UserPreferencePeer::getUserPrefsByUserAndPrefix($user->getUserId(), $CHAT_ROOM_PREFIX);

$items = array();
foreach ($prefs as $pref) {
	$threadId = str_replace($CHAT_ROOM_PREFIX, '', $pref->getPreferenceName());
	$thread = ForumThreadPeer::retrieveByPk($threadId);
	if ($thread) {
		$forum = $thread->getForum();
		$project = ForumPeer::getRelatedProject($forum);
		$url = GFUrl::getUrl();

		// /gf/project/vsplugin/forum/?_forum_action=ForumMessageBrowse&thread_id=5377&action=ForumBrowse&forum_id=8609
		$forum_url = $url->projectURL($project,
				'ForumBrowse',
				Forum::SECTION, '',
				array(
					'_forum_action' => 'ForumMessageBrowse',
					'thread_id' => $thread->getPrimaryKey(),
					'forum_id' => $forum->getPrimaryKey()
				));

		$project_url = $url->projectURL($project);
		$title_arr = ChatManager::getItemTitle($thread);

		if (is_array($title_arr) && count($title_arr) >  0) {
			$items[] = array(
				'section' => $forum->getSection(),
				'ref_id' => $forum->getRefId(),
				'thread_id' => $thread->getForumThreadId(),
				'forum_name' => $forum->getForumName(),
				'forum_url' => $forum_url,
				'title' => $title_arr['title'],
				'title_full' => $title_arr['title_full'],
				'project' => $project,
				'project_name' => $project->getProjectName(),
				'project_url' => $project_url,
				'last' => getLastDateTime(strtotime($forum->getMostRecentDate()), $Lang),
				'sort' => strtotime($forum->getMostRecentDate())
			);
		}
	}
}

usort($items, 'sortBySortKey');

$params = array(
	'header' => langText($Lang, 'Discuss.Recent.For', "Recent Chats for {$user->getUnixName()}", array($user->getUnixName())),
	'discuss.header.join' => langText($Lang, 'Discuss.Recent.Header.Join', "Join"),
	'discuss.header.topic' => langText($Lang, 'Discuss.Recent.Header.Topic', "Topic"),
	'discuss.header.project' => langText($Lang, 'Discuss.Recent.Header.Project', "Project"),
	'discuss.header.last' => langText($Lang, 'Discuss.Recent.Header.Last', "Last Post"),
	'items' => $items
);

include 'forms/RecentChatsTab-tpl.php';

function sortBySortKey($arr_a, $arr_b) {
	$a = $arr_a['sort'];
	$b = $arr_b['sort'];

	if ($a == $b) return 0;
	// Sort descending = reverse of normal sort comparison
	if ($a < $b) return 1;
	return -1;
}

function getLastDateTime($time, $Lang) {

	$now = time();
	$since = $now - $time;
	$today = strtotime(date('Y-m-d 00:00:00', $now));

	if ($since < 60) {
		return langText($Lang, 'Discuss.Recent.Header.JustNow', 'Just Now');
	} else if ($time >= $today) {
		return date('h:i A', $time);
	} else {
		return date('D, d M y H:i A', $time);
	}

}

function langText($Lang, $key, $default, $values = array()) {
	$ret = $Lang->getText($key, $values);
	if (strlen($ret) == 0) return $default;
	return $ret;
}