<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version $Id$
 * @author Celina Vizcaya	<cvizcaya@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

/**
 * A collection of utilities for chat manage
 */

class ChatManager {
    
    /**
    * Searchs Forum by section and ref_id
    *
    * @param string $section, String $refid
    * @return $forum
    */
    static function getForumByRefIdSection($section, $refid, $name = null) {

        $section = Validator::sanitizeString($section);
        $refid = Validator::sanitizeString($refid);
        $forum = ForumPeer::getOrCreateForum($section, $refid, $name);
        return $forum;
    }

 
    /**
    * Searchs Forum by thread_id 
    *
    * @param int $thread_id
    * @return $forum
    */    
    static function getForumByThreadId($thread_id) {

        $thread_id = Validator::sanitizeInt($thread_id);
        $thread = ForumThreadPeer::retrieveByPk($thread_id);
        if (is_null($thread)) {
            error_log("ChatAdd, cannot find requested thread $thread_id, returning 404");
            header("HTTP/1.0 404 Not Found", true, '404');
            return;
        }
        return $thread->getForum();
    }

    /**
    * Searchs Forum by id 
    *
    * @param int $forum_id
    * @return $forum
    */        
    static function getForumById($forum_id) {
        $forum_id = Validator::sanitizeInt($forum_id);

        $forum = ForumPeer::retrieveByPk($forum_id);

        return $forum;
    }
     
    /**
    * Returns necesary data to join the chat
    *
    * @param $forum, $thread
    * @return json
    */      
    static function getChatWindow($forum, $thread) {

        
        $session = Session::getSession();
        if ($forum->getModerationLevel() != Forum::NO_MODERATION) {
            $return = json_encode(array(
                'success' => FALSE,
                'message' => "Discussion {$forum->getForumName()} is moderated, so Live Discussion is not allowed.",
                'thread_id' => 0
            ));
        } else {
            // Add back-end record for this user to join the chat
            if (ChatManager::AddChatUser($session->getUser(), $thread)) {
                ForumPeer::logChatEvent($thread, ForumPeer::CHAT_EVENT_USER_ENTER, $session->getUser());
            }

            $return = json_encode(array(
                'success' => TRUE,
                'thread_id' => $thread->getPrimaryKey()
            ));
        }
        return $return;
    }

    /**
    * Searchs user preferences and adds them to the chat
    *
    * @param $sessionUser, $thread
    * @return boolean
    */    
    static function AddChatUser($sessionUser, $thread) {

        $chat_room_prefix = 'chat-room-';
        $room = $chat_room_prefix . $thread->getForumThreadId();

        $last_value = UserPreferencePeer::getUserPreferenceValue($sessionUser->getUserId(), $room, '');

        if ($last_value == 'Y')
            return FALSE;

        return (UserPreferencePeer::setUserPref($sessionUser->getUserId(), $room, 'Y') == 1);
    }


	/**
	 * Creates display-ready strings for short and full title of
	 * the GForge object to which the specific forum/thread is bound.
	 * @param mixed $forumOrThread A Forum or Thread object.
	 * @return array The title and full_title keys contain the constructed names.
	 */
	static function getItemTitle($forumOrThread) {
		$url = '';
		$title = '';
		try{
			if ($forumOrThread instanceof Forum) {
				$url = $forumOrThread->getForumUrl();
				if ($forumOrThread->getSection() == Project::SECTION) {
					$title = $forumOrThread->getForumName();
					if ($title != $forumOrThread->getProject()->getProjectName()) {
						$title = $title . ' in '. $forumOrThread->getProject()->getProjectName();
					}
				} else {
					$arr_title = ForumPeer::getRelatedItemTitle($forumOrThread);
					$title = $arr_title['title'];
				}
			} else {
				$url = $forumOrThread->getForumThreadUrl();
				if ($forumOrThread->getForum()->getSection() == Project::SECTION) {
					$title = $forumOrThread->getThreadName();
					if ($title != $forumOrThread->getForum()->getProject()->getProjectName()) {
						$title = $title . ' in '. $forumOrThread->getForum()->getProject()->getProjectName();
					}
				} else {
					$arr_title = ForumPeer::getRelatedItemTitle($forumOrThread->getForum());
					$title = $arr_title['title'];
				}
			}
			$arr_title = array();
			$arr_title['url'] = $url;
			$arr_title['title'] = $title;
			$arr_title['title_full'] = $title;
			$maxlen = 25;
			if (strlen($arr_title['title']) > $maxlen) {
				$arr_title['title'] = StringUtils::truncateString(trim($arr_title['title']), $maxlen);
			}
		} catch (Exception $ex) {
			error_log($ex->getMessage());
			error_log($ex->getTraceAsString());
		}
		return $arr_title;
	}
}
?>
