<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code shows a form for editing News
//
require_once('lib/core/WallPost.php');
require_once('lib/core/WallPostPeer.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('lib/html/MyTemplate.php');

$session = Session::getSession();
$Lang = $session->getLanguage();
$user = $session->getUser();

//Creat new wall post object
$wallpost = new WallPost();

$errors = array();

if (!is_object($user)) {
  $errors[] = $Lang->getText("WallPost.NotLoggedIn");
} else {

  $theme = new MyTemplate();

  $comment = isset($_REQUEST['comment']) ? Validator::sanitizeString($_REQUEST['comment']) : "";
  $section = isset($_REQUEST['section']) ? Validator::sanitizeString($_REQUEST['section']) : "";
  $ref_id = isset($_REQUEST['ref_id']) ? Validator::sanitizeInt($_REQUEST['ref_id']) : 0;
  $user_to_unixname = isset($_REQUEST['unix_name']) ? trim(Validator::sanitizeString($_REQUEST['unix_name'])) : "";

  //Retrieve and validate target user
  $c = new Criteria();
  $c->add(UserPeer::UNIX_NAME, $user_to_unixname);
  $user_to = UserPeer::doSelectOne($c);
  if (!is_object($user_to)) {
    $errors[] = "User: " . $user_to_unixname . " does not exist.";
  }
}


//Set data
if (sizeof($errors) == 0) {
	$wallpost->setComment($comment);
	$wallpost->setPostedBy($session->getUser()->getUserId());
	$wallpost->setSection("user");
	$wallpost->setPostDate(time());
	$wallpost->setRefId($user_to->getUserId());

	if (!$wallpost->userCanCreate()) {
		throw new PermissionDeniedException();
	}

	//Try saving the wall post
	if (sizeof($errors) == 0)  {
		try {
			$wallpost->save();
		} catch (InvalidDataException $ex) {
			$errors = array_merge($errors, $ex->getErrors());		
		}
	}
}

//if errors found, the wall post wasn't saved, so show the page again and display errors
if (sizeof($errors) > 0)  {
	$feedback = implode('<br />',$errors);
	require_once("MessageWallPostBrowse.php");
} else {
	//Wall post was saved successfully, continue  processing attachments, links and target users
	
	//Associates the attached file, if any
	if ($_FILES['addfile']['name'] != "") {
		$attach = new Filesystem();
		$attach->setRefId($wallpost->getWallPostId());
		$attach->setSection("messagewall");
		$attach->setPostedBy($session->getUser()->getUserId());
		$attach->setFileType($_FILES['addfile']['type']);
		$attach->setFileName($_FILES['addfile']['name']);
		$attach->save();
		$attach->setFileDataFromPath($_FILES['addfile']['tmp_name']);
	}	
	
	//Associates a link, if any
	if ($section != "" && $ref_id > 0) {
		$association = new Association();
		$association->setSection("usermessagewall");
		$association->setRefId($wallpost->getWallPostId());
		$association->setToSection($section);
		$association->setToRefId($ref_id);
		$association->save();
	}
	
	GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL(''), array('feedback' => $language->getText('News.AddFeedback')));
}
?>
