<?php

/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code lists News
//
require_once('lib/core/WallPost.php');
require_once('lib/core/WallPostPeer.php');
require_once('lib/core/WallPostComment.php');
require_once('lib/core/WallPostCommentPeer.php');
require_once('lib/core/UserPreference.php');
require_once('lib/core/UserPreferencePeer.php');

if (!function_exists('time_sleep_until')) {

	function time_sleep_until($future) {
		if ($future < time()) {
			trigger_error("Time in past", E_USER_WARNING);
			return false;
		}
		usleep(($future - microtime(1)) * 1000000);
		return true;
	}

}

$session = Session::getSession();
$user = $session->getUser();
$Lang = $session->getLanguage();
session_write_close();

if (!($session->isLoggedIn())) {
	throw new PermissionDeniedException();
}

$history = (isset($_GET['history'])) ? true : false;
$since = (isset($_GET['since'])) ? intval($_GET['since']) : 0;

if (!$history) {
	$prefName = 'toaster-since';
    $since = UserPreferencePeer::getUserPreferenceValue($user->getUserId(), $prefName, 0);

	$max_poll_count = 28;
	$repeat_count = 0;
	$keep_polling = true;
	while ($keep_polling) {
		$repeat_count++;
		//error_log("MessageWallUserNotice, user last saw message id {$since->getPreferenceValue()}");
		$notices = WallPostPeer::GetNotificationsSince($user->getUserId(), $since);
	
		if (count($notices) > 0) {
			$keep_polling = false;
		}
	
		if ($repeat_count > $max_poll_count) {
			$keep_polling = false;
		}
	
		if ($keep_polling) {
			time_sleep_until(microtime(true) + 1);  // sleep for 1 second
			echo "\n"; //<-- send this to the client
			if (connection_status() != 0) {
				return;
			}
		}
	}
	
	$newCount = count($notices);
	$messages = array();
	$browse_url = "/gf/my/messagewall/?action=MessageWallPostBrowse&filter=2";
	$last_id = 0;
	
	if ($newCount > 0) {
		$last_id = $notices[$newCount - 1]->getWallPostId();
	
		if ($newCount > 3) {
			$messages[] = array(
				'id' => 0,
				'comment' => "You have $newCount new <a href='$browse_url' title='Browse your Message Wall' target='_new'>Wall Postings</a>",
				'user' => null
			);
		} else {
			foreach ($notices as $notice) {
				$messages[] = WallPostToJson($notice, $browse_url);
			}
		}
	}
	
	$response = array(
		'since' => $last_id,
		'messages' => $messages
	);
	
	if ($last_id > 0) {
		try {
            UserPreferencePeer::setUserPref($user->getUserId(), $prefName, $last_id);
		} catch (Exception $ex) {
			error_log("Could not update user preference value for last-seen message wall ID: {$ex->getMessage()}");
		}
	}
} else {
	$notices = WallPostPeer::GetHistoryNotifications($user->getUserId(), $since, 4);
	
	$messages = array();
	foreach ($notices as $notice) {
		$messages[] = WallPostToJson($notice, $browse_url);
	}
	
	$last_id = $since;
	if (sizeof($notices) > 0) {
		$last_id = $notices[0]->getWallPostId();
	}
	
	$response = array(
		'since' => $last_id,
		'messages' => $messages
	);
}

echo json_encode($response);

function WallPostToJson(WallPost $notice, $browse_url = '') {

	$maxLength = 600;

	$comment = $notice->getComment();

	if (strlen($comment) > $maxLength) {
		$comment = substr($comment, 0, $maxLength) . "..."; //."&nbsp;&nbsp;<a href='$browse_url' title='Read more' target='_new'>read more</a>";
	}

	return array(
		'id' => $notice->getWallPostId(),
		'comment' => $comment,
		'user' => UserToJson($notice->getUser()),
		'post_date' => $notice->getPostDate()
	);
}

function UserToJson($user) {
	if ($user == null)
		return null;
	return array(
		'user_id' => $user->getUserId(),
		'unix_name' => $user->getUnixName(),
		'email' => $user->getEmail(),
		'avatar' => $user->getAvatarId()
	);
}
