<?php
/**
 * Copyright (c) 2013 GForge, LLC
 *
 * @version 1 rickbutton  
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('core/ProjectPeer.php');
require_once('core/ProjectJoinRequestPeer.php');
require_once('session/Session.php');
require_once('core/Plugin.php');
require_once('core/RolePeer.php');
require_once('html/HTMLWidget.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$session = Session::getSession();
$project = GFUrl::getUrl()->getProject();
$project_id = $project->getProjectId();
$Lang = $session->getLanguage();

if (!$project->userIsAdmin()) {
  require_once('exceptions/PermissionDeniedException.php');
  throw new PermissionDeniedException();
}

$emails_field  = $_POST['emails'];
$emails_field = str_replace("\r", "", $emails_field);
$role_id = $_POST['role'];

$emails = explode("\n", $emails_field);

if ($role_id == 100) {
  $role = null;
} else {
  $crit = new Criteria();
  $crit->add(RolePeer::ROLE_ID, $role_id);
  $role = RolePeer::doSelectOne($crit);
}

$errors = array();

if (count($emails) == 0) {
  $errors['emails'] = $Lang->getText("ProjectAdmin.Invite.AtLeastOne");
}

if (count($emails) == 1 && trim($emails[0]) == false) {
  $errors['emails'] = $Lang->getText("ProjectAdmin.Invite.AtLeastOne");
}

$users_map = array();

if (count($errors) == 0) {
  //send invites
  foreach($emails as $email) {
    $email = trim($email);
    $crit = new Criteria();
    $crit->add(UserPeer::EMAIL, $email);
    $user = UserPeer::doSelectOne($crit);
    $users_map[$email] = $user;
    $error = checkInvalid($project, $user, $role, $email);
    if ($error != false) {
      $errors[$email] = "$email: " . $error;
    }
     
  } 
} else {
  handleErrors($errors, $emails_field, $role_id);
}

if (count($errors) == 0) {
  foreach ($emails as $email) {
    $user = $users_map[$email];
    $success = false;
    if (is_object($user)) {
      $success = inviteExistingUser($project, $user, $role);
    } else {
      $success = inviteNewUser($project, $email, $role);
    }
    if (!$success) {
      handleErrors(array(), $emails_field, $role_id);
    }
  }
} else {
  handleErrors($errors, $emails_field, $role_id);
}

GFUrl::getUrl()->redirect(
  GFUrl::getUrl()->currentBaseURL('', 
                                  array('feedback' => $Lang->getText("ProjectAdmin.Invite.Success"))
));

function checkInvalid($project, $user, $role, $email) {
  $Lang = Language::getSessLang();
  if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
    return $Lang->getText("ProjectAdmin.Invite.InvalidEmail");
  }
  if (!is_object($user)) {
    $hash = getInviteSetting($project, $email, $role);
    if ($hash != null) {
      return $Lang->getText("ProjectAdmin.Invite.AlreadySent");
    }
    return false;
  } else {
    if ($user->getStatus() != User::STATUS_ACTIVE) {
      return $Lang->getText("ProjectAdmin.Invite.NotActive");
    }
    $c = new Criteria();
    $c->add(UserProjectPeer::USER_ID, $user->getUserId());
    $c->add(UserProjectPeer::PROJECT_ID, $project->getProjectId());
    $user_project = UserProjectPeer::doSelectOne($c);
    if (is_object($user_project)) {
      return $Lang->getText("ProjectAdmin.Invite.AlreadyInProject");
    }
    return false;
  }
}

function getInviteByHash($hash) {
  $c = new Criteria();
  $c->add(SettingPeer::SECTION, 'project');
  $c->add(SettingPeer::SETTING_VALUE, $hash);
  $s = SettingPeer::doSelectOne($c);
  if (is_object($s)) {
    $arr = explode(':', $s->getSettingValue());
    return array('email' => $arr[1], 'role_id' => $arr[2]);
  } else {
    return null;
  }
}

function getInviteSetting($project, $email, $role) {
  $c = new Criteria();
  $c->add(SettingPeer::SECTION, 'project');
  $c->add(SettingPeer::REF_ID, $project->getProjectId());
  $c->add(SettingPeer::SETTING_NAME, "invite:$email:{$role->getRoleId()}");
  $s = SettingPeer::doSelectOne($c);
  if (is_object($s)) {
    return $s->getSettingValue();
  } else {
    return null;
  }
}

function setInviteSetting($project, $email, $role) {
  $hash = sha1(uniqid(mt_rand(), true));
  SettingPeer::SaveSetting('project',
    $project->getProjectId(),
    "invite:$email:{$role->getRoleId()}",
    $hash,
    $project->getProjectId());
  return $hash;
}

function inviteNewUser($project, $email, $role) {
  $hash = getInviteSetting($project, $email, $role);
  if ($hash != null) {
    return false; 
  }
  $hash = setInviteSetting($project, $email, $role);
  //send email
  sendInviteEmail($email, $hash, $project, $role);
  return true;
}

function inviteExistingUser($project, $user, $role) {
  if ($user->getStatus() == User::STATUS_ACTIVE) {
    //check if there is a join request first
    $crit = new Criteria();
    $crit->add(ProjectJoinRequestPeer::PROJECT_ID, $project->getProjectId());
    $crit->add(ProjectJoinRequestPeer::USER_ID, $user->getUserId());
    $join_request = ProjectJoinRequestPeer::doSelectOne($crit);

    //if there's a join request placed
    $send_accepted_message = false;
    if ( is_object($join_request) ) {
      $send_accepted_message = true;
      $join_request->delete();				
    }

    //check if user already a part of project
    $c = new Criteria();
    $c->add(UserProjectPeer::USER_ID, $user->getUserId());
    $c->add(UserProjectPeer::PROJECT_ID, $project->getProjectId());
    $user_project = UserProjectPeer::doSelectOne($c);
    if (is_object($user_project)) {
      return false;
    } else {
      $user_project = new UserProject();
      $user_project->setUser($user);
      $user_project->setProject($project);
      $upr = new UserProjectRole();
      $upr->setRoleId($role->getRoleId());
      $user_project->addUserProjectRole($upr);

      if ($send_accepted_message == true) {
        $join_request->sendAcceptMessage();
      }
      $user_project->save();

      if(GFConf::get('system.useDaemon')){
        GFDaemon::notify('DAEMON_UP_SAVE', $project->getUnixName());
      }

      return true;
    }
  } else {
    return false;
  }
}

function handleErrors($errors, $emails_field, $role_field) {
  $Lang = Language::getSessLang();
  $feedback = $Lang->getText("ProjectAdmin.Invite.Errors") . " <br /><br />";
  $feedback .= implode('<br />', $errors);
  $feedback .= "<br /><br /> " . $Lang->getText("ProjectAdmin.Invite.NoneSent");
  $emails = $emails_field;
  $role_id = $role_field;
  include("ProjectInvite.php");
  exit;
}

function sendInviteEmail($email, $hash, $project, $role) {
  $serverName = GFConf::get('system.name');
  $projectName = $project->getProjectName();
  $roleName = $role->getRoleName();
  $url = GFUrl::getUrl()->getFullURL("/gf/account/?action=UserAdd&hash=$hash");
  $body = <<<EOF
Hello,

You have been invited to the $serverName project $projectName as a $roleName. To accept this invitation, go to the following web address:

$url
EOF;
  $subject = "Invite to Project $projectName";
  SystemUtils::sendMail($subject, $body, $email);
}

