<?php
require_once ('lib/ProjectNotification.php');
require_once ('core/Notification.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new InvalidDataException('Only POST method allowed');
}

$session = Session::getSession();
if (!$session->isLoggedIn()) {
	throw new NotLoggedInException;
}


$Lang = $session->getLanguage();
$user = $session->getUser();
/** @var Project $project */
$project = GFUrl::getUrl()->getProject();
if (!$project->userIsAdmin()) {
	throw new PermissionDeniedException("Only project admins allowed");
}
$errors = array();

$message = '';
if (isset($_POST['message'])) {
	$message = Validator::sanitizeString($_POST['message'], false);
}

$subject = '';
if (isset($_POST['subject'])) {
	$subject = Validator::sanitizeString($_POST['subject'], false);

}

$receivers = Validator::sanitizeInt($_POST['receivers']);

if (empty($message)) {
	$errors[] = $Lang->getText('ProjectAdmin.MassEmail.EmptyMessage');
}

if (empty($subject)) {
	$errors[] = $Lang->getText('ProjectAdmin.MassEmail.EmptySubject');
}

/** @var User[] $members */
$members = $project->getMembers();
$count = count($members);

//Build an array of valid unix names, extra validation but just in case
$validUsers = array();
for ($i = 0; $i < $count; $i++) {
	$validUsers[$i] = $members[$i]->getUnixName();
}
$destUsers = array();
$uids = array();


//Process receivers
if ($receivers == ProjectNotification::SEND_ALL) {
	foreach($members as $user) {
		$uids[] = $user->getUserId();
	}
} else if ($receivers == ProjectNotification::SEND_INDIVIDUAL) {
	if (!isset($_POST['usernames'])) {
		$errors[] = $Lang->getText('ProjectAdmin.MassEmail.EmptyReceivers');
	} else {
		$destUsers = explode(',', Validator::sanitizeString(trim($_POST['usernames'], ','), false));
		$invalidUsers = array();
		foreach ($destUsers as $key => $destUser) {
			$destUser = trim($destUser);
			if (empty($destUser))  {
				continue;
			}
			$i = array_search($destUser, $validUsers);
			if ( $i !== false) {
				$uids[] = $members[$i]->getUserId();
			} else {
				$invalidUsers[] = $destUser;
			}
		}
		if (count($invalidUsers)) {
			$errors[] = $Lang->getText('ProjectAdmin.MassEmail.InvalidReceivers', array(implode(',', $invalidUsers)));
		}

	}
} else if ($receivers == ProjectNotification::SEND_GROUPS_ROLES) {
	if (!isset($_POST['role_id']) && (!isset($_POST['group_id']))) {
		$errors[] = $Lang->getText('ProjectAdmin.MassEmail.NoGroupRoleSelected');
	}

	$roles = array();
	$groups = array();
	if (isset($_POST['group_id'])) {
		foreach ($_POST['group_id'] as $id) {
			$groups[] = Validator::sanitizeInt($id);
		}
	}

	if (isset($_POST['role_id'])) {
		foreach ($_POST['role_id'] as $id) {
			$roles[] = Validator::sanitizeInt($id);
		}
	}

	//Get users for each group
	$c = new Criteria();
	$c->add(UserPeer::USER_ID, $groups, Criteria::IN);
	/** @var User[] $users */
	$users = UserPeer::doSelect($c);

	foreach ($users as $user) {
		$groupMembers = $user->getUsergroupUsers();
		foreach ($groupMembers as $groupMember) {
			$uids[] = $groupMember->getUserId();
		}
	}

	//Get users for each role
	$c = new Criteria();
	$c->add(UserProjectRolePeer::ROLE_ID, $roles, Criteria::IN);
	$objs = UserProjectRolePeer::doSelectJoinUserProject($c);

	foreach ($objs as $obj) {
		/** @var User $user */
		$user = $obj->getUserProject()->getUser();
		if ($user->getIsGroup()) {
			$users = $user->getUsergroupUsers();
			foreach ($users as $user) {
				$uids [] = $user->getUserId();
			}
		} else {
			$uids[] = $user->getUserId();
		}
	}

	$uids = array_unique($uids);


} else {
	//And invalid data exception since there is no way to set a different value unless bypassing the form
	throw new InvalidDataException('Invalid data for receivers type');
}



if (count($errors)) {
	$feedback = '* '. implode('<br />*', $errors);
	include('EmailMembers.php');
} else {

	//Save the Notification first in notification table

	$notification =  new Notification();
	$notification->setRefId($project->getId());
	$notification->setSection(Project::SECTION);
	$notification->setBody($message);
	$notification->setSubject($subject);
	$notification->setUser($session->getUser());
	$notification->save();

	//Attach files to Notification
	$uploadHandler = new UploadHandler();
	$files = $uploadHandler->getTmpUploadedFiles();
	foreach ($files as $file) {
		$notification->attachFile($file, Notification::SECTION);
	}
	$uploadHandler->purgeTmpDir();

	//Finally, queue  for delivery
	$projectNotification = new ProjectNotification($uids, $project);
	$projectNotification->setNotificationMessage($message);
	$projectNotification->setNotificationSubject($subject);
	$projectNotification->setMessageId($notification->getPrimaryKey());
	$projectNotification->setNotificationId($notification->getPrimaryKey());
	NotificationManager::getManager()->queue($projectNotification);


	$feedback = $Lang->getText('ProjectAdmin.MassEmail.SuccessfullyAdded');
	GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('EmailMembers', array('feedback' => $feedback)));
}
