<?php

/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Ruben Gutierrez ruben@gforgegroup.com
 * @date 2006-10-10
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('core/ProjectPeer.php');
require_once('session/Session.php');
require_once('plugins/tracker/lib/TrackerExtraField.php');
require_once('plugins/tracker/lib/TrackerQueryPeer.php');
require_once('plugins/tracker/lib/TrackerItemSavedQuery.php');
require_once('html/HTMLWidget.php');

require('lib/external/jpgraph/jpgraph.php');
require('lib/external/jpgraph/jpgraph_bar.php');
require('lib/external/jpgraph/jpgraph_line.php');
require('lib/external/jpgraph/jpgraph_date.php');

    const FILTER_RELEASE = 'R';
    const FILTER_EXTRA_FIELD = 'E';
    const FILTER_QUERY = 'Q';
    const FILTER_SPRINT = 'S';

$session = Session::getSession();
$project = GFUrl::getUrl()->getProject();
if ($project == null || !$project->userCanRead())
{
    require_once('exceptions/PermissionDeniedException.php');
    throw new PermissionDeniedException();
}
$lang = $session->getLanguage();

// Get parameters, figure out which subquery to use
$subquery = '0';

// Priority 1: Release ID
$release_id = Validator::sanitizeInt($_REQUEST['release_id']);
if ($release_id > 0)
{
    $type = FILTER_RELEASE;
    $subquery = subqueryRelease($release_id);

    require_once('plugins/frs/lib/FrsReleasePeer.php');
    $release = FrsReleasePeer::retrieveByPk($release_id);
    if ($release)
    {
        $title = $project->getProjectName() . ' - ' . $release->getReleaseName();
    }
} else
{
    // Priority 2: Tracker Query
    $tracker_query_id = Validator::sanitizeInt($_REQUEST['query_id']);

    if ($tracker_query_id > 0)
    {
        $type = FILTER_QUERY;
        $subquery = subqueryQuery($tracker_query_id);

        $tsq = TrackerQueryPeer::retrieveByPk($tracker_query_id);
        if ($tsq)
        {
            $title = $tsq->getTracker()->getTrackerName() . ' - ' . $tsq->getQueryName();
        }
    } else
    {

        // Priority 3: Tracker Sprint
        $tracker_sprint_id = Validator::sanitizeInt($_REQUEST['sprint_id']);

        if ($tracker_sprint_id > 0)
        {
            $type = FILTER_SPRINT;
            $subquery = subquerySprint($tracker_sprint_id);

            $sprint = TrackerSprintPeer::retrieveByPk($tracker_sprint_id);
            if ($sprint)
            {
                $title = $sprint->getSprintTitleText();
            }
        } else
        {
            // Priority 4: Extra field & Value
            $ef_id = Validator::sanitizeInt($_REQUEST['ef_id']);
            $ef_value = Validator::sanitizeString($_REQUEST['ef_value']);

            if ($ef_id > 0 && $ef_value != '')
            {
                $type = FILTER_QUERY;
                $subquery = subqueryExtraField($ef_id, $ef_value);
            }
            // If nothing is set at this point, we'll default to 0 as the subquery and no data will be displayed.
        }
    }
}

$full_chart = (array_key_exists('full', $_REQUEST) ?
        (Validator::sanitizeInt($_REQUEST['full']) == 1) : 0);

// Using any tracker items assigned to this release, get the following:
// 1. Date range: this week and the previous two full weeks
$current_monday = strtotime('last monday', strtotime('next sunday'));
$fmt = date('Y-m-d', $current_monday);

$start_date = date('Y-m-d', strtotime('-2 weeks', $current_monday));
$end_date = date('Y-m-d', time() + 86400);
$period = 'day';


// 2. # of TI's open BEFORE start date
$open_before = bugs_open_before($start_date, $subquery);

// 3. # of TI's opened and closed each day between start and end date
$daily_activity = period_activity($start_date, $end_date, $period, $subquery);
$created_deleted = period_created_deleted($start_date, $end_date, $period, $subquery);

// Then calculate:
// 1. Net TI's open for each day (previous day's net + opened - closed)
// 2. Average velocity (net closed) per day - may be negative
// 3. Project and plot the next full week burn-down based on average velocity
$bins = dateRangeArray($start_date, $end_date, $period);
$resultSet = array();
$addedSet = array();
$removedSet = array();
$runningSet = array();

$running_total = $open_before;
$total_closed = 0;

$minY2 = -1;
$maxY2 = 1;

foreach ($bins as $bin)
{
    $added = 0;
    $removed = 0;
    $net = 0;
    if (array_key_exists($bin, $daily_activity))
    {
        $added += $daily_activity[$bin][1];
        $removed += $daily_activity[$bin][0];
    }
    if (array_key_exists($bin, $created_deleted))
    {
        $added += $created_deleted[$bin]['_created_'];
        $removed += $created_deleted[$bin]['_deleted_'];
    }

    $net = $added - $removed;
    $total_closed += $removed;
    $resultSet[$bin] = array(
        'pre' => $running_total,
        'new' => $added,
        'closed' => $removed,
        'net' => $net,
        'post' => $running_total + $net
    );
    $running_total += $net;
    if ($maxY2 < $running_total)
    {
        $maxY2 = $running_total;
    }
    if ($maxY2 < $added)
    {
        $maxY2 = $added;
    }
    if ($minY2 > (-$removed))
    {
        $minY2 = (-$removed);
    }

    $addedSet[] = $added;
    $removedSet[] = -$removed;
    $runningSet[] = $running_total;
}

if (isset($_GET['debug']))
{
    echo "<table>
			<tr>
				<th>date</th>
				<th>pre</th>
				<th>new</th>
				<th>closed</th>
				<th>net</th>
				<th>post</th>
			</tr>";
    foreach ($resultSet as $d => $row)
    {
        echo "
			<tr>
				<td>$d</td>
				<td>{$row['pre']}</td>
				<td>{$row['new']}</td>
				<td>{$row['closed']}</td>
				<td>{$row['net']}</td>
				<td>{$row['post']}</td>
			</tr>";
    }
    echo "</table>";
    return;
}

$per_day = 0;
if (count($bins) > 0)
{
    $per_day = $total_closed / count($bins);
}

// Add forecasted numbers for the rest of this week through end of day next Friday
$future_end_date = date('Y-m-d', strtotime('next friday', strtotime('next sunday')));
$future = dateRangeArray($end_date, $future_end_date, $period);
foreach ($future as $d)
{
    $running_total -= $per_day;
    if ($running_total < 1)
    {
        $runningSet[] = 0;
        break;
    }
    $runningSet[] = round($running_total);
}

// Figure out when the count will reach zero
$projected_completion = 'N/A';

// Only project completion if the last actual day's count is > 0
// Otherwise, it's already done or not started yet
$lastActualTotal = $runningSet[count($bins) - 1];
if ($lastActualTotal > 0)
{
    if ($per_day < 0)
    {
        // Should not be possible anymore
        $projected_completion = 'None';
    } else
    {
        $daysRemaining = round($lastActualTotal / ($per_day + 1)) + 1;
        if ($daysRemaining > 30)
        {
            $projected_completion = 'Quite a while';
        } else
        {
            $projected_completion = date('M d, Y', strtotime("+{$daysRemaining} days", strtotime($end_date)));
        }

        // Sometimes, straight division doesn't give the same answer as the running total approach.
        // To prevent the chart and the label from saying different completion dates,
        // we override the calculated date with the running totals (chart data), if there's a zero there.
        for ($i = count($bins); $i < count($runningSet); $i++)
        {
            if ($runningSet[$i] == 0)
            {
                $offset = $i - count($bins);
                $projected_completion = date('M d, Y', strtotime("+{$offset} days", strtotime($end_date)));
                break;
            }
        }
    }
}

$addedPlot = new BarPlot($addedSet);
$addedPlot->SetFillColor('green');
$addedPlot->SetColor('green');
$addedPlot->SetWidth(8);

$removedPlot = new BarPlot($removedSet);
$removedPlot->SetFillColor('red');
$removedPlot->SetColor('red');
$removedPlot->SetWidth(8);

// Create the linear plot
$lineplot = new LinePlot($runningSet);
$lineplot->SetStyle('solid');
$lineplot->SetColor('darkgreen');
$lineplot->SetLineWeight(1);
$lineplot->SetBarCenter(TRUE);
$lineplot->mark->Show();
$lineplot->mark->SetType(MARK_SQUARE);
$lineplot->mark->SetFillColor('darkgreen');
$lineplot->mark->SetColor('darkgreen');
$lineplot->mark->SetSize(1);

$todayIndex = count($bins) - (round((strtotime($end_date) - time()) / 86400));
$todayBand = new PlotBand(VERTICAL, BAND_DIAGCROSS, $todayIndex, $todayIndex + 1, '#c0c0c0');
$todayBand->ShowFrame(TRUE);
$todayBand->SetOrder(DEPTH_FRONT);

$futureBand = new PlotBand(VERTICAL, BAND_RDIAG, $todayIndex + 1, 'max', '#c0c0c0');
$futureBand->ShowFrame(FALSE);
$futureBand->SetOrder(DEPTH_FRONT);

// Create the grouped bar plot
//$gbplot = new GroupBarPlot (array( $addedPlot , $removedPlot ));
$gbplot = new AccBarPlot(array($addedPlot, $removedPlot));

// Width and height of the graph
$width = 800;
$height = ($full_chart ? 400 : 200);

// Create a graph instance
$graph = new Graph($width, $height);
$graph->SetBox(true, '#808080', 1);

// Specify what scale we want to use,
// text = text labels for the X-axis
// int = integer scale for the Y-axis
$graph->SetScale('textint', $minY2, $maxY2);
if ($maxY2 < 20)
{
    $graph->yscale->ticks->Set(5);
} else
{
    $graph->yscale->ticks->Set(10);
}
$graph->yscale->SetAutoTicks();

if ($full_chart)
{
    $graph->img->SetMargin(36, 12, 12, 60);
    $graph->xaxis->SetFont(FF_DEFAULT, FS_NORMAL, 8);
    $graph->xaxis->SetLabelAngle(70);
    $xLabels = dateRangeArray($start_date, $future_end_date, $period, 'M-d');
    $graph->xaxis->SetTickLabels($xLabels);
    $graph->xaxis->SetPos('min');
} else
{
    $graph->img->SetMargin(2, 2, 2, 2);
    $graph->xaxis->HideLabels();
    $graph->xaxis->HideTicks();
    $graph->yaxis->HideLabels();
    $graph->yaxis->HideTicks();
}

if (strlen($title) > 0)
{
    $graph->title->SetFont(FF_DEFAULT, FS_NORMAL, 12);
    $graph->title->Set($title);
}

if (strlen($projected_completion) > 0)
{
    $graph->subtitle->SetFont(FF_DEFAULT, FS_NORMAL, 10);
    $graph->subtitle->Set("Est. Completion: {$projected_completion}");
}
$graph->ygrid->SetFill(FALSE);
$graph->yaxis->scale->SetGrace(2);

$graph->Add($gbplot);
$graph->Add($lineplot);
$graph->Add($todayBand);
$graph->Add($futureBand);

$sat = 5;
while ($sat < count($runningSet))
{
    $graph->Add(weekendBand($sat));
    $sat += 7;
}

// Display the graph
if (ob_get_level())
    ob_end_clean();
$graph->Stroke();
return;

function weekendBand($start_index)
{
    $ret = new PlotBand(VERTICAL, BAND_SOLID, $start_index, $start_index + 2, '#f2f2f2');
    $ret->ShowFrame(FALSE);
    $ret->SetOrder(DEPTH_BACK);
    return $ret;
}

function bugs_open_before($before_date, $subquery = '0')
{
    $sql = "select count(atr.ref_id) as already_open
			from (
				select ref_id, max(audit_trail_id) as audit_trail_id
				from audit_trail
				where section = 'trackeritem'
				and ref_id in ($subquery)
				and field_name IN ('_created_', 'status_id')
				and change_date < '$before_date'
				group by ref_id
			) sl
			inner join audit_trail atr
			on sl.audit_trail_id = atr.audit_trail_id
			WHERE atr.field_name = '_created_' OR atr.new_value = '1'
			";

    $ret = 0;
    $rs = runQuery($sql);

    if ($rs != null && $rs->next())
    {
        $ret = $rs->getInt('already_open');
    }

    return $ret;
}

function period_activity($start_date, $end_date, $period = 'day', $subquery = '0')
{

    $ret = array();

    $sql = "
		select date_trunc('$period', change_date) as bin, new_value, count(audit_trail_id) as item_count
		  from audit_trail
		where section = 'trackeritem'
		  and ref_id in ($subquery)
		  and field_name = 'status_id'
		  and change_date between '$start_date' and '$end_date'
		group by
		  date_trunc('$period', change_date),
		  new_value
		order by
		  date_trunc('$period', change_date), new_value
	";

    $rs = runQuery($sql);

    if ($rs != null)
    {
        while ($rs->next())
        {
            $bin = date('Y-m-d', strtotime($rs->getString('bin')));
            $new_value = $rs->getInt('new_value');
            $item_count = $rs->getInt('item_count');

            if (!array_key_exists($bin, $ret))
            {
                $ret[$bin] = array(0, 0);
            }
            $ret[$bin][$new_value] = $item_count;
        }
    }

    return $ret;
}

function period_created_deleted($start_date, $end_date, $period = 'day', $subquery = '0')
{

    $ret = array();

    $sql = "
		select date_trunc('$period', change_date) as bin, field_name, count(audit_trail_id) as item_count
		  from audit_trail
		where section = 'trackeritem'
		  and ref_id in ($subquery)
		  and field_name IN ('_created_', '_deleted_')
		  and change_date between '$start_date' and '$end_date'
		group by
		  date_trunc('$period', change_date), field_name
		order by
		  date_trunc('$period', change_date), field_name
	";

    $rs = runQuery($sql);

    if ($rs != null)
    {
        while ($rs->next())
        {
            $bin = date('Y-m-d', strtotime($rs->getString('bin')));
            $field_name = $rs->getString('field_name');
            $item_count = $rs->getInt('item_count');

            if (!array_key_exists($bin, $ret))
            {
                $ret[$bin] = array(
                    '_created_' => 0,
                    '_deleted_' => 0
                );
            }

            $ret[$bin][$field_name] = $item_count;
        }
    }

    return $ret;
}

function runQuery($sql)
{
    // Get connection
    $con = Propel::getConnection(UserPeer::DATABASE_NAME);
    $stmt = $con->createStatement();
    $rs = $stmt->executeQuery($sql);

    return $rs;
}

function dateRangeArray($start, $end, $period = 'day', $format = 'Y-m-d')
{

    $ret = array();

    $cur = strtotime($start);
    $max = strtotime($end);

    while ($cur < $max)
    {
        // Add this new day to the aDays array
        $ret[] = date($format, $cur);

        // Add a day to the current date
        $cur = strtotime("+1 $period", $cur);
    }
    return $ret;
}

function getSubquery($type, $filter1, $filter2 = null)
{
    switch ($type)
    {
        case FILTER_EXTRA_FIELD:
            return subqueryExtraField($filter1, $filter2);
            break;
        case FILTER_QUERY:
            return subqueryQuery($filter1);
            break;
        default:  // includes FILTER_RELEASE
            return subqueryRelease($filter1);
            break;
    }
    return "0";
}

function subqueryRelease($release_id)
{
    return "SELECT
            distinct tefd.tracker_item_id
          FROM
            public.tracker_extra_field tef,
            public.tracker_extra_field_data tefd
          WHERE
            tefd.tracker_extra_field_id = tef.tracker_extra_field_id AND
            tef.field_type = 8 AND
            tefd.field_data = '$release_id'";
}

function subqueryExtraField($ef_id, $ef_value)
{
    return "SELECT
            distinct tefd.tracker_item_id
          FROM
            public.tracker_extra_field_data tefd
          WHERE
            tefd.tracker_extra_field_id = $ef_id AND
            tefd.field_data = '$ef_value'";
}

function subquerySprint($sprint_id)
{
    return "SELECT
            distinct tracker_item_id
          FROM
            tracker_item
          WHERE
            sprint_id = {$sprint_id}";
}

function subqueryQuery($query_id)
{
    $tsq = TrackerQueryPeer::retrieveByPk($query_id);
    if ($tsq == null)
    {
        return "0";
    }

    // TODO - Should we also check if the user is allowed to view/query the Tracker itself?
    $session = Session::getSession();
    if ($session->getUser()->isSiteAdmin() ||
        $tsq->getIsShared() ||
        $tsq->getUserId() == $session->getUser()->getUserId())
    {
        $tq = new TrackerItemSavedQuery($tsq->getTracker(), $tsq, $session->getUser());
    } else
    {
        throw new PermissionDeniedException();
    }

    $where = $tq->getWhereClause();

    return "SELECT distinct ti.tracker_item_id FROM tracker_item ti $where";
}