<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code shows a form for editing News
//
require_once('lib/core/WallPost.php');
require_once('lib/core/WallPostPeer.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('lib/html/ProjectTemplate.php');

$session = Session::getSession();
$Lang = $session->getLanguage();
$user = $session->getUser();
$project = GFUrl::getUrl()->getProject();

//Create new wall post object
$wallpost = new WallPost();

$errors = array();

if (!is_object($user)) {
  $errors[] = $Lang->getText("WallPost.NotLoggedIn");
} else {

  $comment = isset($_REQUEST['comment']) ? Validator::sanitizeString($_REQUEST['comment']) : "";
  $section = isset($_REQUEST['section']) ? Validator::sanitizeString($_REQUEST['section']) : "";
  $ref_id = isset($_REQUEST['ref_id']) ? Validator::sanitizeInt($_REQUEST['ref_id']) : 0;
  $share_users = isset($_REQUEST['share_users']) ? trim(Validator::sanitizeString($_REQUEST['share_users'])) : "";


  //Retrieve and validate target users, if any
  $arr_share_user_ids = array();
  if ($share_users != "") {
    $arr_share_users = explode(',', $share_users);

    foreach ($arr_share_users as $share_user) {
      $c = new Criteria();
      $c->add(UserPeer::UNIX_NAME, $share_user);
      $user_to = UserPeer::doSelectOne($c);

      $user_to_id = 0;
      if (is_object($user_to)) {
        if ($user_to->isProjectMember($project) || $project->isPublic()) {
          $arr_share_user_ids[] = $user_to->getUserId();
        } else {
          $errors[] = "User: " . $share_user . " is not a project member.";
        }
      } else {
        $errors[] = "User: " . $share_user . " does not exist.";
      }
    }
  }

  //Set data
  $wallpost->setComment($comment);
  $wallpost->setPostedBy($session->getUser()->getUserId());
  $wallpost->setSection("project");
  $wallpost->setPostDate(time());
  $wallpost->setRefId($project->getProjectId());

  if (!$wallpost->userCanCreate()) {
    throw new PermissionDeniedException();
  }

  //If no erros so far, try saving the wall post
  if (sizeof($errors) == 0)  {
    try {
      $wallpost->save();
    } catch (InvalidDataException $ex) {
      $errors = array_merge($errors, $ex->getErrors());		
    }
  }
}
//if errors found, the wall post wasn't saved, so show the page again and display errors
if (sizeof($errors) > 0)  {
	$feedback = implode('<br />',$errors);
	require_once("MessageWallPostBrowse.php");
} else {
	//Wall post was saved successfully, continue  processing attachments, links and target users
	
	//Associates the attached file, if any
	if ($_FILES['addfile']['name'] != "") {
		$attach = new Filesystem();
		$attach->setRefId($wallpost->getWallPostId());
		$attach->setSection("messagewall");
		$attach->setPostedBy($session->getUser()->getUserId());
		$attach->setFileType($_FILES['addfile']['type']);
		$attach->setFileName($_FILES['addfile']['name']);
		$attach->save();
		$attach->setFileDataFromPath($_FILES['addfile']['tmp_name']);
	}	
	
	//Associates a link, if any
	if ($section != "" && $ref_id > 0) {
		$association = new Association();
		$association->setSection(WallPost::SECTION);
		$association->setRefId($wallpost->getWallPostId());
		$association->setToSection($section);
		$association->setToRefId($ref_id);
		$association->save();
	}
	
	//Associates target users, if any
	foreach ($arr_share_user_ids as $user_to_id) {
		if ($user_to_id > 0) {
			$association = new Association();
			$association->setSection(WallPost::SECTION);
			$association->setRefId($wallpost->getWallPostId());
			$association->setToSection(User::SECTION);
			$association->setToRefId($user_to_id);
			$association->save();
		}
	}

	GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL(''), array('feedback' => $language->getText('News.AddFeedback')));
}
?>
