<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('lib/core/WallPost.php');
require_once('lib/core/WallPostPeer.php');
require_once('lib/core/WallPostComment.php');
require_once('lib/core/WallPostCommentPeer.php');
require_once('lib/html/ProjectTemplate.php');

$session = Session::getSession();
$user = $session->getUser();
$project = GFUrl::getUrl()->getProject();
$Lang = $session->getLanguage();
$template = new ProjectTemplate();

$params = array(				// Parameters for the template
			'title' => $Lang->getText('Tabs.ProjectMessageWall'),
			'project' => $project,
			'selectedbottomtab'=>'messagewall',
			'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		);

$wallPost = new WallPost;
$wallPost->setSection("project");
$bolCanCreate = $wallPost->userCanCreate();

$filter = isset($_REQUEST['filter']) ? intval($_REQUEST['filter']) : "0";
$year = isset($_REQUEST['year']) ? intval($_REQUEST['year']) : "0";
$month = isset($_REQUEST['month']) ? intval($_REQUEST['month']) : "0";

//Variables from the add-post form - used when returning from WallPostAddAction with errors
$comment = isset($_REQUEST['comment']) ? Validator::sanitizeString($_REQUEST['comment']) : "";
$section = isset($_REQUEST['section']) ? Validator::sanitizeString($_REQUEST['section']) : "";
$ref_id = isset($_REQUEST['ref_id']) ? Validator::sanitizeInt($_REQUEST['ref_id']) : "";
$share_users = isset($_REQUEST['share_users']) ? Validator::sanitizeString($_REQUEST['share_users']) : "";
$wall_post_id = isset($_REQUEST['id']) ? Validator::sanitizeInt($_REQUEST['id']) : "";
$con = Propel::getConnection(WallPostPeer::DATABASE_NAME);

//Retrieves the wall posts
$crit = new Criteria();
$crit->add(WallPostPeer::SECTION, "project");
$crit->add(WallPostPeer::REF_ID, $project->getProjectId());
if ($wall_post_id != "") {
	$crit->add(WallPostPeer::WALL_POST_ID, $wall_post_id);
}


//Refines search with year/month, if selected
if ($year != 0 && $month != 0) {
	$start_date = $year."-".$month."-01";
	$last_day_month = date('t',strtotime($month."/1/".$year));
	$end_date = $year."-".$month."-".$last_day_month;
	
	$crit->add(WallPostPeer::POST_DATE, $start_date, Criteria::GREATER_EQUAL);
	$crit->addAnd(WallPostPeer::POST_DATE, $end_date, Criteria::LESS_EQUAL);
}

//Filter posts by current user posts
if ($filter == "1") {
	$crit->add(WallPostPeer::POSTED_BY, $user->getUserId());
}
//Filter posts by current user as target of the posts
if ($filter == "2") {
	$crit->addJoin(WallPostPeer::WALL_POST_ID, AssociationPeer::REF_ID, Criteria::LEFT_JOIN);
	$crit->add(AssociationPeer::SECTION, "messagewall");
	$crit->add(AssociationPeer::TO_REF_ID, $user->getUserId());
}

$crit->addDescendingOrderByColumn(WallPostPeer::WALL_POST_ID);

$total = WallPostPeer::doCount($crit);

$paginator = new Paginator('messagewall', 5, $total, WallPostPeer::POST_DATE, array());
$paginator->addPageClauses($crit);
$arr = WallPostPeer::doSelect($crit);

//Get archives data (month/year)
$str_sql = "SELECT date_part('month', post_date) as smonth, date_part('year', post_date) as syear 
						FROM wall_post
						WHERE section='project' AND ref_id = " . $project->getProjectId() . "
						GROUP BY syear, smonth
						ORDER BY syear, smonth DESC";

$stmtArchives = $con->createStatement();
$rs_archives = $stmtArchives->executeQuery($str_sql);

$arr_archives = array();
$i=0;
foreach ($rs_archives as $row) {
	$arr_archives[$i]['month'] =  $row['smonth'];
	$arr_archives[$i]['year'] =  $row['syear'];
	$arr_archives[$i]['desc'] =  date("F", mktime(0, 0, 0, $row['smonth'])) . " " . $row['syear'];
	$i++;
}

//Get Frequent Posters
$str_sql = "SELECT count(wall_post_id) as total, posted_by FROM wall_post GROUP BY posted_by ORDER BY count(wall_post_id) DESC LIMIT 5";
							
$stmtArchives = $con->createStatement();
$rs_frequent_posters = $stmtArchives->executeQuery($str_sql);

$template->header($params);

//Display posts
require_once("forms/MessageWallPostBrowse-tpl.php");

$str_paginator = '';
if(isset($paginator) && is_object($paginator)) {
	$str_paginator = '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
}

echo $paginator->getPaginator();

$template->footer();
?>
