<?php
// This script outputs HTML and Javascript for burndown chart and configuration widget,
// initially displayed in the project home page using the %%BURNDOWN%% token.

/**
 * Expected parameters:
 *
 * $bd_index int The numeric position of this burndown widget relative to any others on the page.  Examples include 1, 2, etc.
 * $bd_query string The query string parameter (param=value&param2=value) to be appended to the burndown chart image URL.
 * $project Project The project model object, used for getting other required data (e.g., Tracker).
 *
 */
function burndownWidget($project, $bd_index, $bd_query)
{
    $query_pieces = array();
    parse_str($bd_query, $query_pieces);

// Figure out which type of query (if any) we're using
// See similar rules in Burndown.php
// Priority 1: Release ID
    if (array_key_exists('release_id', $query_pieces))
    {
        $release_id = $query_pieces['release_id'];

// Priority 2: Tracker Query
    } elseif (array_key_exists('query_id', $query_pieces))
    {
        $query_id = $query_pieces['query_id'];

        // If query_id is zero, this is a new instance of burndown that has not had a query selected yet.
        // We can proceed with rendering the dropdown list but don't need to render the image tag yet.
        if ($query_id > 0) {
            // For this type, we need to check access to the tracker this query uses
            // If the current user cannot read the tracker, we need to hide the widget.
            $tsq = TrackerQueryPeer::retrieveByPk($query_id);
            if (!$tsq || !$tsq->getTracker()->userCanRead()) {
                // Nope!
                return '';
            }
        }


// Priority 3: Extra field & Value
    } elseif (array_key_exists('ef_id', $query_pieces))
    {
        $ef_id = $query_pieces['ef_id'];
        $ef_value = $query_pieces['ef_value'];
    }

    $tracker_id = (array_key_exists('tracker_id', $query_pieces)) ? $query_pieces['tracker_id'] : 0;

    $project = GFUrl::getUrl()->getProject();
    $tracker_list = array();
    if ($project)
    {
        foreach ($project->getTrackers() as $t)
        {
            $tracker_list[$t->getTrackerId()] = $t->getTrackerName();
        }
    }

    $query_pieces['full'] = 1;

    $chart_url = GFUrl::getURL()->projectURL($project, 'Burndown', 'graphs', '', $query_pieces);

    ob_start();
?>
    <!-- Chart Area -->
    <img style="width: 80%;" src="<?php echo $chart_url ?>" alt="Burndown chart" />
    <br />
    <?php if ($project->userIsAdmin()) { ?>
    <!-- Display / Config Area -->
    <form method="POST" action="<?php echo GFUrl::getURL()->projectURL($project, 'SetBurndown', 'admin'); ?>">
        <input type="hidden" name="bd_index" value="<?php echo $bd_index ?>" />
        <select name="query_id" class="tracker-list">
            <?php
            foreach ($project->getTrackers() as $tracker)
            {
                $queries = array_filter($tracker->getTrackerQuerys(), function($q) {
                    return $q->getIsShared();
                });

                if (count($queries) > 0) {
                    echo "<optgroup label=\"{$tracker->getTrackerName()}\">";
                    foreach ($queries as $query) {
                        if ($query->getIsShared()) {
                            $selected = ($query_id == $query->getTrackerQueryId()) ? 'selected' : '';
                            echo "<option $selected value=\"{$query->getTrackerQueryId()}\">{$query->getQueryName()}</option>\n";
                        }
                    }
                    echo "</optgroup>";
                }
            }
            ?>
        </select>
        <input type="submit" name="submit" value="Save" />
    </form>
    <?php } // if ($project->userIsAdmin()) ?>
    <br />
<?php
    $content = ob_get_clean();

    return $content;
}