<?php

require_once('lib/core/TagEntry.php');
require_once('lib/core/TagEntryPeer.php');
require_once('plugins/tracker/lib/TrackerItem.php');
require_once('plugins/docman/lib/DocmanFile.php');
require_once('plugins/forum/lib/ForumMessage.php');
require_once('plugins/forum/lib/ForumThreadPeer.php');
require_once('lib/html/ProjectTemplate.php');
require_once('Validator.php');

$Lang = Session::getSession()->getLanguage();
$project = GFUrl::getUrl()->getProject();
$theme = new ProjectTemplate();

$params = array(
		'project' => $project,
		'action' => isset($action) ? $action : '',
		'feedback' => isset($_REQUEST['feedback']) ? $_REQUEST['feedback'] : (isset($feedback) ? $feedback : ''),
		'title' => $Lang->getText('Tag.RelatedTags')
		);
$theme->header($params);


if(isset($_GET['section']) && !empty($_GET['section']) &&
	isset($_GET['ref_id']) && !empty($_GET['ref_id']) &&
	isset($_GET['tag_name']) && !empty($_GET['tag_name'])){
	
	$section = Validator::sanitizeString($_GET['section']);
	$ref_id = Validator::sanitizeInt($_GET['ref_id']);
	$tag_name = Validator::sanitizeString($_GET['tag_name']);
	
	if(!TagEntryPeer::userCanRead($section, $ref_id)){
		throw new PermissionDeniedException();
	}
	
	echo '<p><strong>' . $Lang->getText('Tag.RelatedItems') . ' ' . $tag_name . '</strong></p>';
	//Get items
	$c = new Criteria();
	$c->add(TagEntryPeer::SECTION, $_GET['section']);
	$c->add(TagEntryPeer::TAG_NAME, $_GET['tag_name']);
	$tags = TagEntryPeer::doSelect($c);

	if(is_null($tags) || empty($tags)){
		throw new InvalidDataException(array('Tag not found'));
	}
	
	$item_ids = array();
	foreach($tags as $tag){
		$item_ids[] = $tag->getRefId();
	}	
	$theader= array();
	$c = new Criteria();
	switch($section){
		case TrackerItem::SECTION:
			$tracker_id = Validator::sanitizeInt($_GET['subsection_id']);
			if(!$tracker_id){
				throw new InvalidDataException(array('Missing parameters'));
			}

			$theader[] = $Lang->getText('TrackerItem.TrackerItemIdName');
			$theader[] = $Lang->getText('TrackerItem.SummaryName');
			
			//Get tracker items tagged inside this tracker
			$c = new Criteria();
			$c->add(TrackerItemPeer::TRACKER_ITEM_ID, $item_ids, Criteria::IN);
			$c->add(TrackerItemPeer::TRACKER_ID, $tracker_id);
			$items = TrackerItemPeer::doSelect($c);
			break;

		case DocmanFile::SECTION:
			$theader[] = $Lang->getText('DocmanFile.DocmanFileIdName');
			$theader[] = $Lang->getText('Docman.DescriptionName');

			//Get Docman files tagged inside this project
			$c = new Criteria();
			$c->add(DocmanFilePeer::DOCMAN_FILE_ID, $item_ids, Criteria::IN);
			$c->add(DocmanFolderPeer::PROJECT_ID, $project->getProjectId());
			$c->addJoin(DocmanFolderPeer::DOCMAN_FOLDER_ID, DocmanFilePeer::DOCMAN_FOLDER_ID);
			$items = DocmanFilePeer::doSelect($c);
			break;

		case ForumMessage::SECTION:
			$forum_id = Validator::sanitizeInt($_GET['subsection_id']);
			if(!$forum_id){
				throw new InvalidDataException(array('Missing parameters'));
			}

			//Get tagged messages inside this forum
			$theader[] = $Lang->getText('Forum.ThreadMessage');
			$theader[] = $Lang->getText('Forum.Author');
			$theader[] = $Lang->getText('ForumMessage.PostDateName');
			$c = new Criteria();
			$c->add(ForumMessagePeer::FORUM_MESSAGE_ID, $item_ids, Criteria::IN);
			$c->add(ForumThreadPeer::FORUM_ID, $forum_id);
			$c->addJoin(ForumMessagePeer::FORUM_THREAD_ID, ForumThreadPeer::FORUM_THREAD_ID);
			$items = ForumMessagePeer::doSelect($c);
			break;

		case FrsRelease::SECTION:
			$package_id = Validator::sanitizeInt($_GET['subsection_id']);
			if(!$package_id){
				throw new InvalidDataException(array('Missing parameters'));
			}
			
			//Get tagged messages inside this Package
			$theader[] = $Lang->getText('FrsRelease.ReleaseNameName');
			$theader[] = $Lang->getText('FrsRelease.ReleaseDateName');

			$c = new Criteria();
			$c->add(FrsReleasePeer::FRS_RELEASE_ID, $item_ids, Criteria::IN);
			$c->add(FrsReleasePeer::FRS_PACKAGE_ID, $package_id);
			$items = FrsReleasePeer::doSelect($c);
			break;

		case WikiPage::SECTION:
			$theader[] = $Lang->getText('Wiki.PageName');

			$c = new Criteria();
			$c->add(WikiPagePeer::WIKI_PAGE_ID, $item_ids, Criteria::IN);
			$c->add(WikiPagePeer::REF_ID, $project->getProjectId());
			$items = WikiPagePeer::doSelect($c);
			break;

		case ScmCommit::SECTION:
			$theader[] = $Lang->getText('ScmCommit.Id');
			$theader[] = $Lang->getText('Scm.CommitDetails');

			$c = new Criteria();
			$c->add(ScmCommitPeer::SCM_COMMIT_ID, $item_ids, Criteria::IN);
			$c->add(ScmCommitPeer::PROJECT_ID, $project->getProjectId());
			$items = ScmCommitPeer::doSelect($c);
			break;

		default:
			throw new InvalidDataException(array('Invalid section'));
	}
	echo $theme->tableTop($theader);


	//This normally shouldn't happen, just in case
	if(count($items) < 1){
		echo $theme->TableFullRow($Lang->getText('Tag.RelatedItemsNoneFound'));
	} else {
		foreach($items as $item){
			$trow = array();
			switch($section){

				//Tracker item
				case TrackerItem::SECTION:
					$trow[] = $item->getTrackerItemId();
					$trow[] = '<a href="'. GFUrl::getUrl()->projectUrl($project,'TrackerItemEdit','tracker','',array('tracker_item_id'=>$item->getTrackerItemId())) . '">'.$item->getSummary()."</a>";
					break;

				//Docman File
				case DocmanFile::SECTION:
					$trow[] = $item->getDocmanFileId();
					$trow[] = '<a href="' . GFUrl::getUrl()->projectUrl($project, 'DocmanFileEdit', 'docman','',array('id'=>$item->getDocmanFileId())) . '">' . $item->getDescription() . '</a>';
					break;

				//ForumMessage
				case ForumMessage::SECTION:
					$trow[] = '<a href="'. GFUrl::getUrl()->projectUrl($project,'ForumBrowse', 'forum', '',array('thread_id'=>$item->getForumThreadId(), 'message_id'=>$item->getForumMessageId(), 'forum_id'=>$item->getForumThread()->getForumId())).'&_forum_action=ForumMessageBrowse">' . $item->getSubject() .'</a>';
					$user = UserPeer::retrieveByPk($item->getCreatedBy());
					$trow[] = '<a href="' . GFUrl::getUrl()->baseUrl('','','/user/'.$user->getUnixName(),array()) .'">' . $user->getFirstname(). ' ' . $user->getLastName() . '</a>';
					$trow[] = $item->getPostDate();
					break;

				//Frs Release		
				case FrsRelease::SECTION:
					$trow[] = '<a href="' . GFUrl::getUrl()->projectUrl($project,'FrsReleaseView', 'frs', '', array('frs_package_id'=>$item->getFrsPackageId(), 'release_id'=>$item->getFrsReleaseId())).'">' . $item->getReleaseName() ."</a>";
					$trow[] = $item->getReleaseDate();
					break;

				//Wiki 
				case WikiPage::SECTION:
					$pagename = $item->getPagename();
					$trow[] = '<a href="'. GFUrl::getUrl()->projectUrl($project, '', 'wiki','',array('pagename'=>$pagename)) .'">'. $pagename .'</a>';
					break;

				//Commits
				case ScmCommit::SECTION:
					$trow[] = $item->getScmCommitId();
					$trow[] = '<a href="'.$item->getScmCommitUrl() . '">'.$item->getMessageLog()."</>";
					break;
					
				
			}
			echo $theme->TableRow($trow);
		}
	}

	echo $theme->TableBottom();
	
	
	echo $theme->footer($params);
				

} else {
	throw new InvalidDataException(array('Missing parameters'));
}
	
		
		
?>
