<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

//
// This code creates a new Project and saves it to the 
// database
//
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('core/Project.php');
require_once('html/TroveRenderer.php');
require_once('exceptions/NotLoggedInException.php');

$session = Session::getSession();
if (!$session->isLoggedIn()) {
	throw new NotLoggedInException();
}

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$Lang = $session->getLanguage();

$Project = new Project();

// this was checked before, but let�s double check
if (Session::getSession()->isSiteAdmin() || (GFConf::get('system.projectRegRestricted')==false) ) {

	try {
		if (GFConf::get('system.projectRegClickLicense')) {
			if (isset($_REQUEST['iagree']) && Validator::sanitizeBool($_REQUEST['iagree'])) {
			} else {
				throw new InvalidDataException(array(Language::getSessText('License.DidntCheck')));
			}
		}
		$redirect = (isset($_POST['redirect']) ? htmlspecialchars($_POST['redirect']) : '');	
		$Project->setProjectName( Validator::sanitizeString(trim($_POST['project_name'])) );
		$Project->setUnixName( Validator::sanitizeString(trim($_POST['unix_name'])) );
		$Project->setHomepageUrl( Validator::sanitizeString($_POST['homepage_url']) );
		//$is_public = isset($_POST['is_public']) ? Validator::sanitizeBool($_POST['is_public']) : false;
		$is_public = true;
		$Project->setIsPublic( $is_public );
		$Project->setDescription( Validator::sanitizeString(trim($_POST['description'])) );
		$Project->setRegisterPurpose( Validator::sanitizeString($_POST['register_purpose']) );
		
		// This will be moved to Trove
		//$Project->setRegisterLicenseOther( Validator::sanitizeString($_POST['register_license_other']) );
		$Project->setRegisterLicenseOther('');
		
		if (!isset($_POST['template_project_id'])) {
			$template_project_id = 0;
		} else {
			$template_project_id = Validator::sanitizeInt($_POST['template_project_id']);
		}

		$Project->setTemplateProjectId($template_project_id);
		$Project->setStatus(Project::STATUS_PENDING);
		$Project->save();
                
                //in case the user already has a preference, we update it with the new template election, if not, we generate the new preference
                $gf_proj_templates_trove = GFConf::get('system.troveGfProjTemplates', false);
                $troveCategory = TroveCategoryPeer::retrieveByPk($gf_proj_templates_trove);
                $userPreference = BaseUserPreferencePeer::retrieveByPK(Session::getSession()->getUser()->getId(), $troveCategory->getAlias());
                if ($userPreference && is_object($userPreference)) {
                    $userPreference->setPreferenceValue($template_project_id);
                    $userPreference->save();
                } else {
                    $preference = new UserPreference();
                    $preference->setPreferenceName($troveCategory->getAlias());
                    $preference->setUser(Session::getSession()->getUser());
                    $preference->setPreferenceValue($template_project_id);
                    $preference->save(); 
                }            
		// now save the trove information
		$trove_renderer = new TroveRenderer();
		$trove_renderer->setSection(Project::SECTION);
		$trove_renderer->setRefId($Project->getPrimaryKey());
		$trove_renderer->savePOSTData();

		$auto_approve = GFConf::get('system.autoApproveProjects', false, true);
		// You might not believe it, but simply comparing this value to true
		// ($auto_approve == true) was always evaluating to TRUE.
		// Super WTF.
	    if ($auto_approve === true) {
			$Project = ProjectPeer::retrieveByUnixName($Project->getUnixName(), false);
			$Project->approve(true);
			$str_feedback = $Lang->getText('Project.PostRegisterSiteAdminText', array(GFConf::get('system.name')));
			GFUrl::getUrl()->redirect(GFUrl::getUrl()->projectURL($Project, '', '', '', array('feedback' => $str_feedback)));
	    } else if ($redirect != '') {
			if (Session::getSession()->isSiteAdmin()) {
				//Refresh object
				$Project = ProjectPeer::retrieveByUnixName($Project->getUnixName(), false);
				$Project->approve();
				$str_feedback = $Lang->getText('Project.PostRegisterSiteAdminText', array(GFConf::get('system.name')));
			} else {
				$str_feedback = $Lang->getText('Project.PostRegisterText', array(GFConf::get('system.name')));
			}
			
			GFUrl::getUrl()->redirect($redirect, array('feedback' => $str_feedback));
		} else {
			GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('PostRegister'));
		}
		
	} catch (InvalidDataException $ex) {
		$errors = $ex->getErrors();
		$feedback = '* '.implode('<br />* ', $errors);
		include_once('ProjectAdd.php');
	}
} else {
	GFUrl::redirect(GFUrl::getUrl()->currentBaseURL('ProjectAdd'),array('feedback'=>Language::getSessText('Common.PermissionDenied')));
}

?>
