<?php
/**
 * Copyright (c) 2005-2011 GForge, LLC
 *
 * @version $Id$
 * @author Daniel A. P�rez	daniel@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('lib/core/TagEntryPeer.php');

// this file contains common search functions for site and project

	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return resultset
	 */
	function getSearchResults($params) {
		$section = $params['section'];
		if (strlen($params['words'])>0) {
			$searcher = new GFSearch($params);
			if ($section=='project') {
				return searchProjects($searcher);
			} elseif ($section=='user') {
				if (isset($params['sub_section']) && $params['sub_section'] == 'addUserSearch') {
					if (isset($params['formname'])) {
						return addUserSearch($searcher, $params['formname']);
					} else {
						return addUserSearch($searcher);
					}
				} else {
					return searchUsers($searcher);
				}
			}
		} else {
			return "Empty Search";
		}
	}

	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return string
	 */
	function searchProjects(GFSearch $search, $paginate=true, $userProjectsOnly=false) {
		$paginator = null;

		if (GFConf::get('system.dbtype') == 'postgres') {		
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();
			$searchAlsoInTrove = isset($_REQUEST['search_trove']) && $_REQUEST['search_trove'] == "1";


			$columns = array(
				'project_name'=> Language::getSessText('SiteAdmin.ProjectList.ProjectName'),
				'unix_name'=> Language::getSessText('Project.UnixNameName'),
				'description' => Language::getSessText('Project.DescriptionName')
			);
			if ($searchAlsoInTrove) {
				$columns['category_name'] = Language::getSessText('TroveEdit.MainCategory');
				$columns['sub_category'] = Language::getSessText('TroveEdit.Subcategory'); 
				$columns['t_description'] = Language::getSessText('TroveAdd.Description');

			}

			// Get the extra columns from plugins
			$prs = ActionManager::notify('PROJECT_LISTING_HEADER_COLS');
			foreach ($prs as $pr) {
				$cols = $pr->getResult();
				foreach ($cols as $key => $title) {
					$columns[$key] = $title;
				}
			}



			$restrict_clause = ' project.status = 1 ';
			if (!Session::getSession()->isLoggedIn()) {
				$restrict_clause .= ' AND project.is_public ';
			} else {
				$user = Session::getSession()->getUser();
				$project_ids = $user->getAllProjectIds();

				$extraRestriction = '';

				if ($userProjectsOnly) {
					if (count($project_ids)) {
						$str = implode(',', $project_ids);
						$extraRestriction = "  AND project.project_id IN ($str) ";
					} else {
						$extraRestriction = ' AND 1 = 2';
					}
				} else {
					if (count($project_ids)) {
						$str = implode(',', $project_ids);
						$extraRestriction = "  AND (project.project_id IN ($str) OR project.is_public)";
					} else {
						$extraRestriction .= ' AND project.is_public';
					}
				}

				$restrict_clause .= $extraRestriction;

			}

			if ($paginate) {
				if ($words == '*') {
					$sql = "SELECT COUNT(*) as total
					FROM project WHERE $restrict_clause";
				} else {
					if ($searchAlsoInTrove) {
						$sql =
							" SELECT COUNT(*) as total FROM
	                            (SELECT headline(project.project_name, q1) as project_name,  headline(trove_category.category_name,q1) as trove_name,
	                                headline(project.description,q1) as description,
	                                headline(project.unix_name, q1) as unix_name,headline(trove_category.description, q1) as trove_description
 							 FROM to_tsquery('$words') as q1, project NATURAL JOIN projects_idx INNER JOIN trove_link ON project.project_id=trove_link.ref_id
 							 	INNER JOIN trove_category ON trove_link.trove_category_id = trove_category.trove_category_id INNER JOIN trove_category_idx
	                                ON trove_category.trove_category_id = trove_category_idx.trove_category_id
	                             WHERE trove_link.section='project' AND
									trove_category.parent_trove_category_id IN (SELECT trove_category_id FROM trove_category WHERE show_in_search)
									AND ((trove_category_idx.vectors @@ q1) OR (projects_idx.vectors @@ q1)) AND $restrict_clause) tmp";
					} else {
					$sql = "SELECT COUNT(*) as total FROM
							(SELECT headline(project.project_name, q) as project_name,
							headline(project.description, q) as description, project.project_id, project.is_public
							FROM project, projects_idx, to_tsquery('$words') AS q
							WHERE project.project_id=projects_idx.project_id
							AND ((projects_idx.vectors @@ q)) AND $restrict_clause) tmp";
					}
				}


				try {
					$stmt = $con->createStatement();
					$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
				} catch (SQLException $e) {
					return '<span class="feedback">'.Language::getSessText('Search.InvalidSearch').'</span>';
				}



				if ($rs->next()) {
					$total = $rs->getInt('total');
				} else {
					$total = 0;
				}
			}


			if ($words == '*') {
				//$_REQUEST['prjsearchsort_order'] = 'asc';

				$sql = "SELECT project_name,unix_name,description,project_id as pid, average_rating
				FROM project WHERE
				$restrict_clause GROUP BY pid, project_name, unix_name, description, average_rating
				";
				if ($paginate) {
					$paginator = new Paginator('prjsearch', -1, $total, 'project_id', $columns);
					$sql .= $paginator->getSQL();
				}
			} else {
				if ($searchAlsoInTrove) {
					$sql = "SELECT headline(project.project_name, q1) as project_name, headline(project.description,q1) as description,
 								headline(project.unix_name, q1) as unix_name, headline(trove_category.category_name, q1) as trove_name,
 								headline(trove_category.description, q1) as trove_description, project.project_id as pid,
 								(SELECT category_name FROM get_top_parent_trove(trove_category.trove_category_id)) as category_name
 							 FROM to_tsquery('$words') as q1, project INNER JOIN projects_idx ON project.project_id = projects_idx.project_id INNER JOIN trove_link ON project.project_id=trove_link.ref_id
 							 	INNER JOIN trove_category ON trove_link.trove_category_id = trove_category.trove_category_id INNER JOIN trove_category_idx
 							 	ON trove_category.trove_category_id = trove_category_idx.trove_category_id
 							 WHERE trove_link.section='project' AND
                        ((trove_category_idx.vectors @@ q1) OR (projects_idx.vectors @@ q1)) AND $restrict_clause
								GROUP BY project_name, project.description, unix_name, trove_name, trove_description, pid, q1.q1, average_rating, category_name, trove_category.trove_category_id";
				} else {
					$sql = "SELECT headline(project.project_name, q) as project_name,
						headline(project.description, q) as description, headline(project.unix_name, q) as unix_name,
						project.project_id as pid, project.is_public as is_public
						FROM project, projects_idx, to_tsquery('$words') AS q
						WHERE project.project_id=projects_idx.project_id
						AND ((projects_idx.vectors @@ q)) AND $restrict_clause GROUP BY project_name, description, pid, is_public, unix_name,q.q, average_rating";
				}
				//$_REQUEST['prjsearchsort_order'] = 'desc';
				if ($paginate) {
					$paginator = new Paginator('prjsearch', -1, $total, 'pid', $columns);
					$sql .= $paginator->getSQL();
				}
			}

			$template = $search->getTemplate();

			$ret = '<br />';
			if ($paginate) {
				$ret .= $template->tableTop($paginator->getSortableTableHeaders($template));
			} else {
				$ret .= $template->tableTop(array_values($columns));
			}

			if($paginate && isset($paginator) && is_object($paginator)) {
                $ret .= '<div class="paginator">' . $paginator->getPaginator() . '</div>';
            }

			try {
				//echo $sql;die();
				$stmt = $con->createStatement();
				$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			} catch (SQLException $e) {
				return '<span class="feedback">'.Language::getSessText('Search.InvalidSearch').'</span>';

				//echo $e->getMessage();
				//throw new InvalidDataException(array($e->getMessage()));

			}
			
			if ($rs->getRecordCount()>0) {
				while ($rs->next()) {
					$trow = array();
					$project_id = $rs->getInt('pid');
					$project = ProjectPeer::retrieveByPk($project_id);
					if($project->userCanRead()){
						$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project) . '">' . $rs->getString('project_name').'</a>';
						$trow[] = $rs->getString('unix_name');
						$trow[] = $rs->getString('description');

						if ($searchAlsoInTrove) {
							$trow[] = $rs->getString('category_name');
							$trow[] = $rs->getString('trove_name');
							$trow[] = $rs->getString('trove_description');

						}

						// Get the extra columns from plugins
						$prs = ActionManager::notify('PROJECT_LISTING_COLS', array('project' => $project));
						foreach ($prs as $pr) {
							$cols = $pr->getResult();
							foreach ($cols as $content) {
								$trow[] = $content;
							}
						}

						//Get the trove categories content
						/*
						foreach ($troves_sql as $alias => $query) {
							$trow[] = $rs->getString($alias);
						}*/
						$ret .= $template->tableRow($trow);
					}
				}
			} else {
				// zero results for search
				$ret .= $template->tableFullRow(Language::getSessText('Search.NoResults'));
			}
			
			$ret .= $template->tableBottom();
			if($paginate && isset($paginator) && is_object($paginator)) {
				$ret .= '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
			}

			return $ret;
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}
	}

	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return string
	 */
	function searchUsers(GFSearch $search) {
		if (GFConf::get('system.dbtype') == 'postgres') {		
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();
			$columns = array(
				Language::getSessText('User.UnixNameName'),
				Language::getSessText('SiteAdmin.Search.UserName'));
				
			if ($words == '*') {
				$sql = "SELECT COUNT(*) 
				FROM \"user\"
				WHERE status=". User::STATUS_ACTIVE;
			} else {
				$sql = "SELECT COUNT(*)
				FROM \"user\", users_idx, to_tsquery('$words') AS q 
				WHERE (\"user\".user_id=users_idx.user_id 
				AND (users_idx.vectors @@ q))
				AND \"user\".status=". User::STATUS_ACTIVE;
			}
			
			try {
				$stmt = $con->createStatement();
				$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			} catch (SQLException $e) {
				throw new InvalidDataException(array($e->getMessage()));
			}	

			$rs->next();
			$total = $rs->getInt(1);
			
			if ($words == '*') {
				$paginator = new Paginator('usrsearch', -1, $total, 'user_id', $columns);	
				$sql = "SELECT unix_name,user_id,firstname,lastname 
				FROM \"user\"
				WHERE status=". User::STATUS_ACTIVE . ' ' . $paginator->getSQL();
			} else {
				$_REQUEST['usrsearchsort_order'] = 'desc';
				$paginator = new Paginator('usrsearch', -1, $total, 'rank(users_idx.vectors, q)', $columns);					
				$sql = "SELECT headline(\"user\".unix_name, q) as unix_name, \"user\".user_id,
				headline(\"user\".firstname, q) as firstname, headline(\"user\".lastname, q) as lastname
				FROM \"user\", users_idx, to_tsquery('$words') AS q 
				WHERE (\"user\".user_id=users_idx.user_id 
				AND (users_idx.vectors @@ q))
				AND \"user\".status=". User::STATUS_ACTIVE . ' ' . $paginator->getSQL();
			}
			
			try {
				$stmt = $con->createStatement();
				$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			} catch (SQLException $e) {
				throw new InvalidDataException(array($e->getMessage()));
			}	
			
			$template = $search->getTemplate();
			$ret = '<br />';
			$ret .= $template->tableTop($paginator->getSortableTableHeaders($template));
			
			if ($rs->getRecordCount()>0) {
				$project = ProjectPeer::retrieveByPk($search->getProjectId());
				$i = 0;
				while ($rs->next()) {
					$user_id = $rs->getInt('user_id');
					$user = UserPeer::retrieveByPk($user_id);
					$trow = array();
					$trow[] = '<a href="'.GFUrl::getUrl()->userURL($user) . '">' . $rs->getString('unix_name').'</a>';
					$trow[] = $rs->getString('firstname').' '.$rs->getString('lastname');
					$ret .= $template->tableRow($trow);
				}
			} else {
				// zero results for search
				$ret .= $template->tableFullRow(Language::getSessText('Search.NoResults'));
			}
			
			$ret .= $template->tableBottom();
			if(isset($paginator) && is_object($paginator)) {
				$ret .= '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
			}
			return $ret;
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}		
	}	


	function searchTags($search_params){

		if(array_key_exists('project', $search_params)){
			$global_search = false;
		} else {
			$global_search = true;
		}

		$template = $search_params['template'];
		$Lang = Session::getSession()->getLanguage();
		
		$ret = '<h3>' . $Lang->getText('Tag.SearchResultTitle') . '</h3>';
		
		$search_params['words'] = strtolower($search_params['words']);
		$results = ActionManager::notify('DO_TAG_SEARCH', $search_params);
		
		$ret .= '<div class="tabber">';
		foreach($results as $result){
			$result = $result->getResult();
			if(!is_array($result)){
				continue;
			}

			$title = $result['title'];
			$content = $result['content'];

			$ret .= '<div class="tabbertab" title="'.$title.'">';
			$ret .= $content;
			$ret .= '</div>';
		}
		$ret .= '</div>';

		return $ret;

	}

	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return string
	 */
	function addUserSearch(GFSearch $search, $formname='useradd') {
		if (GFConf::get('system.dbtype') == 'postgres') {		
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();
			
			if (isset($search->params['groups']) && !$search->params['groups']) {
				// Don't search in groups
				$groups_sql = 'AND "user".is_group=false';
			} else {
				$groups_sql = '';
			}
			
			if ($words == '*') {
				$sql = "SELECT unix_name,user_id,firstname,lastname 
				FROM \"user\"
				WHERE status=". User::STATUS_ACTIVE ."
				$groups_sql
				ORDER BY user_id LIMIT 250";
			} else {

				$sql = "SELECT headline(\"user\".unix_name, q) as unix_name, \"user\".user_id,
				headline(\"user\".firstname, q) as firstname, headline(\"user\".lastname, q) as lastname
				FROM \"user\", users_idx, to_tsquery('$words') AS q 
				WHERE (\"user\".user_id=users_idx.user_id 
				$groups_sql
				AND (users_idx.vectors @@ q) AND \"user\".status = 1)
				AND \"user\".status=". User::STATUS_ACTIVE ."
				ORDER BY rank(users_idx.vectors, q) DESC";
			}
			
			try {
				$stmt = $con->createStatement();
				$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			} catch (SQLException $e) {
				throw new InvalidDataException(array($e->getMessage()));
			}	

			if ($rs->getRecordCount()>0) {
				$ret = '';
				$template = $search->getTemplate();
				$project = ProjectPeer::retrieveByPk($search->getProjectId());
				$ret = $template->tableTop(array(Language::getSessText('User.UnixNameName'),Language::getSessText('SiteAdmin.Search.UserName')));
				$i = 0;
				while ($rs->next()) {
					$user_id = $rs->getInt('user_id');
					$user = UserPeer::retrieveByPk($user_id);
					$trow = array();
					if (isset($_REQUEST['js_accumulate_users']) && !empty($_REQUEST['js_accumulate_users'])) {
						$trow[] = '<a href="#" onclick="if(window.opener.document.'.$formname.'.share_users.value.length > 0) { window.opener.document.'.$formname.'.share_users.value = window.opener.document.'.$formname.'.share_users.value + \','.$user->getUnixName().'\'; } else { window.opener.document.'.$formname.'.share_users.value = \''.$user->getUnixName().'\'; } window.close();">' . $rs->getString('unix_name').'</a>';
					} else {
						$trow[] = '<a href="#" onclick="window.opener.document.'.$formname.'.useradd.unix_name.value = \''.$user->getUnixName().'\'; window.close();">' . $rs->getString('unix_name').'</a>';
					}
					$trow[] = $rs->getString('firstname').' '.$rs->getString('lastname');
					$ret .= $template->tableRow($trow);
				}
				$ret .= $template->tableBottom();
				return $ret;
			} else {
				// cero results for search
				return Language::getSessText('Search.NoResults');
			}
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}		
	}

/**
 * @param PluginResult[] $params
 */
function getSortedArrayFromPR($Prs) {
	$return = array();
	foreach ($Prs as $pr) {
		$element = $pr->getResult();
		$return[]= $element[0];
	}

	usort($return, 'buildOptionsCmp');

	return $return;
}

function buildOptionsCmp($a, $b) {
	if ($a['display'] == $b['display']) {
		return 0;
	}
	return  ($a['display'] < $b['display']) ? -1 : 1;
}

function setSearchParams($search_params) {
	$ret = array();

	if (isset($search_params['search_by'])) {
		$ret['search_by'] = $search_params['search_by'];
	} else {
		$ret['search_by'] = 'keywords';
		}

	if (isset($search_params['search'])) {
		$ret['search'] = $search_params['search'];
	} else {
		$ret['search'] = array();
	}

	if (isset($search_params['search_attach'])) {
		$ret['search_attach'] = $search_params['search_attach'];
	} else {
		$ret['search_attach'] = false;
	}

	if (isset($search_params['search_scope'])) {
		$ret['search_scope'] = $search_params['search_scope'];
	} else {
		$ret['search_scope'] = 'all';
	}

	if (isset($search_params['startdate'])) {
		$ret['startdate'] = $search_params['startdate'];
	} else {
		$ret['stardate'] = '';
	}

	if (isset($search_params['enddate'])) {
		$ret['enddate'] = $search_params['enddate'];
	} else {
		$ret['enddate'] = '';
	}

	return serialize($ret);

}
